EXPECTED = {'EUTRA-InterNodeDefinitions': {'extensibility-implied': False,
                                'imports': {'EUTRA-RRC-Definitions': ['AntennaInfoCommon',
                                                                      'AntennaInfoDedicated-v10i0',
                                                                      'ARFCN-ValueEUTRA',
                                                                      'ARFCN-ValueEUTRA-v9e0',
                                                                      'ARFCN-ValueEUTRA-r9',
                                                                      'CellIdentity',
                                                                      'C-RNTI',
                                                                      'DL-DCCH-Message',
                                                                      'DRB-Identity',
                                                                      'DRB-ToReleaseList',
                                                                      'FreqBandIndicator-r11',
                                                                      'InDeviceCoexIndication-r11',
                                                                      'LWA-Config-r13',
                                                                      'MasterInformationBlock',
                                                                      'maxBands',
                                                                      'maxFreq',
                                                                      'maxDRB',
                                                                      'maxSCell-r10',
                                                                      'maxSCell-r13',
                                                                      'maxServCell-r10',
                                                                      'maxServCell-r13',
                                                                      'MBMSInterestIndication-r11',
                                                                      'MeasConfig',
                                                                      'MeasGapConfig',
                                                                      'MeasGapConfigPerCC-List-r14',
                                                                      'MeasResultForRSSI-r13',
                                                                      'MeasResultListWLAN-r13',
                                                                      'OtherConfig-r9',
                                                                      'PhysCellId',
                                                                      'P-Max',
                                                                      'PowerCoordinationInfo-r12',
                                                                      'SidelinkUEInformation-r12',
                                                                      'SL-CommConfig-r12',
                                                                      'SL-DiscConfig-r12',
                                                                      'RadioResourceConfigDedicated',
                                                                      'RCLWI-Configuration-r13',
                                                                      'RSRP-Range',
                                                                      'RSRQ-Range',
                                                                      'RSRQ-Range-v1250',
                                                                      'RS-SINR-Range-r13',
                                                                      'SCellToAddModList-r10',
                                                                      'SCellToAddModListExt-r13',
                                                                      'SCG-ConfigPartSCG-r12',
                                                                      'SecurityAlgorithmConfig',
                                                                      'SCellIndex-r10',
                                                                      'SCellIndex-r13',
                                                                      'SCellToReleaseList-r10',
                                                                      'SCellToReleaseListExt-r13',
                                                                      'ServCellIndex-r10',
                                                                      'ServCellIndex-r13',
                                                                      'ShortMAC-I',
                                                                      'MeasResultSSTD-r13',
                                                                      'SL-V2X-ConfigDedicated-r14',
                                                                      'SystemInformationBlockType1',
                                                                      'SystemInformationBlockType1-v890-IEs',
                                                                      'SystemInformationBlockType2',
                                                                      'UEAssistanceInformation-r11',
                                                                      'UECapabilityInformation',
                                                                      'UE-CapabilityRAT-ContainerList',
                                                                      'UE-RadioPagingInfo-r12',
                                                                      'WLANConnectionStatusReport-r13',
                                                                      'WLAN-OffloadConfig-r12']},
                                'object-classes': {},
                                'object-sets': {},
                                'tags': 'AUTOMATIC',
                                'types': {'AS-Config': {'members': [{'name': 'sourceMeasConfig',
                                                                     'type': 'MeasConfig'},
                                                                    {'name': 'sourceRadioResourceConfig',
                                                                     'type': 'RadioResourceConfigDedicated'},
                                                                    {'name': 'sourceSecurityAlgorithmConfig',
                                                                     'type': 'SecurityAlgorithmConfig'},
                                                                    {'name': 'sourceUE-Identity',
                                                                     'type': 'C-RNTI'},
                                                                    {'name': 'sourceMasterInformationBlock',
                                                                     'type': 'MasterInformationBlock'},
                                                                    {'name': 'sourceSystemInformationBlockType1',
                                                                     'type': 'SystemInformationBlockType1',
                                                                     'with-components': [None,
                                                                                         ('nonCriticalExtension',
                                                                                          'ABSENT')]},
                                                                    {'name': 'sourceSystemInformationBlockType2',
                                                                     'type': 'SystemInformationBlockType2'},
                                                                    {'name': 'antennaInfoCommon',
                                                                     'type': 'AntennaInfoCommon'},
                                                                    {'name': 'sourceDl-CarrierFreq',
                                                                     'type': 'ARFCN-ValueEUTRA'},
                                                                    None,
                                                                    [{'name': 'sourceSystemInformationBlockType1Ext',
                                                                      'optional': True,
                                                                      'type': 'OCTET '
                                                                              'STRING'},
                                                                     {'name': 'sourceOtherConfig-r9',
                                                                      'type': 'OtherConfig-r9'}],
                                                                    [{'name': 'sourceSCellConfigList-r10',
                                                                      'optional': True,
                                                                      'type': 'SCellToAddModList-r10'}],
                                                                    [{'name': 'sourceConfigSCG-r12',
                                                                      'optional': True,
                                                                      'type': 'SCG-Config-r12'}]],
                                                        'type': 'SEQUENCE'},
                                          'AS-Config-v10j0': {'members': [{'name': 'antennaInfoDedicatedPCell-v10i0',
                                                                           'optional': True,
                                                                           'type': 'AntennaInfoDedicated-v10i0'}],
                                                              'type': 'SEQUENCE'},
                                          'AS-Config-v1250': {'members': [{'name': 'sourceWlan-OffloadConfig-r12',
                                                                           'optional': True,
                                                                           'type': 'WLAN-OffloadConfig-r12'},
                                                                          {'name': 'sourceSL-CommConfig-r12',
                                                                           'optional': True,
                                                                           'type': 'SL-CommConfig-r12'},
                                                                          {'name': 'sourceSL-DiscConfig-r12',
                                                                           'optional': True,
                                                                           'type': 'SL-DiscConfig-r12'}],
                                                              'type': 'SEQUENCE'},
                                          'AS-Config-v1320': {'members': [{'name': 'sourceSCellConfigList-r13',
                                                                           'optional': True,
                                                                           'type': 'SCellToAddModListExt-r13'},
                                                                          {'name': 'sourceRCLWI-Configuration-r13',
                                                                           'optional': True,
                                                                           'type': 'RCLWI-Configuration-r13'}],
                                                              'type': 'SEQUENCE'},
                                          'AS-Config-v1430': {'members': [{'name': 'sourceSL-V2X-CommConfig-r14',
                                                                           'optional': True,
                                                                           'type': 'SL-V2X-ConfigDedicated-r14'},
                                                                          {'name': 'sourceLWA-Config-r14',
                                                                           'optional': True,
                                                                           'type': 'LWA-Config-r13'},
                                                                          {'name': 'sourceWLAN-MeasResult-r14',
                                                                           'optional': True,
                                                                           'type': 'MeasResultListWLAN-r13'}],
                                                              'type': 'SEQUENCE'},
                                          'AS-Config-v9e0': {'members': [{'name': 'sourceDl-CarrierFreq-v9e0',
                                                                          'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                             'type': 'SEQUENCE'},
                                          'AS-Context': {'members': [{'name': 'reestablishmentInfo',
                                                                      'optional': True,
                                                                      'type': 'ReestablishmentInfo'}],
                                                         'type': 'SEQUENCE'},
                                          'AS-Context-v1130': {'members': [{'name': 'idc-Indication-r11',
                                                                            'optional': True,
                                                                            'type': 'OCTET '
                                                                                    'STRING'},
                                                                           {'name': 'mbmsInterestIndication-r11',
                                                                            'optional': True,
                                                                            'type': 'OCTET '
                                                                                    'STRING'},
                                                                           {'name': 'powerPrefIndication-r11',
                                                                            'optional': True,
                                                                            'type': 'OCTET '
                                                                                    'STRING'},
                                                                           None,
                                                                           [{'name': 'sidelinkUEInformation-r12',
                                                                             'optional': True,
                                                                             'type': 'OCTET '
                                                                                     'STRING'}]],
                                                               'type': 'SEQUENCE'},
                                          'AS-Context-v1320': {'members': [{'name': 'wlanConnectionStatusReport-r13',
                                                                            'optional': True,
                                                                            'type': 'OCTET '
                                                                                    'STRING'}],
                                                               'type': 'SEQUENCE'},
                                          'AdditionalReestabInfo': {'members': [{'name': 'cellIdentity',
                                                                                 'type': 'CellIdentity'},
                                                                                {'name': 'key-eNodeB-Star',
                                                                                 'type': 'Key-eNodeB-Star'},
                                                                                {'name': 'shortMAC-I',
                                                                                 'type': 'ShortMAC-I'}],
                                                                    'type': 'SEQUENCE'},
                                          'AdditionalReestabInfoList': {'element': {'type': 'AdditionalReestabInfo'},
                                                                        'size': [(1,
                                                                                  'maxReestabInfo')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                          'CandidateCellInfo-r10': {'members': [{'name': 'physCellId-r10',
                                                                                 'type': 'PhysCellId'},
                                                                                {'name': 'dl-CarrierFreq-r10',
                                                                                 'type': 'ARFCN-ValueEUTRA'},
                                                                                {'name': 'rsrpResult-r10',
                                                                                 'optional': True,
                                                                                 'type': 'RSRP-Range'},
                                                                                {'name': 'rsrqResult-r10',
                                                                                 'optional': True,
                                                                                 'type': 'RSRQ-Range'},
                                                                                None,
                                                                                [{'name': 'dl-CarrierFreq-v1090',
                                                                                  'optional': True,
                                                                                  'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                                                [{'name': 'rsrqResult-v1250',
                                                                                  'optional': True,
                                                                                  'type': 'RSRQ-Range-v1250'}],
                                                                                [{'name': 'rs-sinr-Result-r13',
                                                                                  'optional': True,
                                                                                  'type': 'RS-SINR-Range-r13'}]],
                                                                    'type': 'SEQUENCE'},
                                          'CandidateCellInfoList-r10': {'element': {'type': 'CandidateCellInfo-r10'},
                                                                        'size': [(1,
                                                                                  'maxFreq')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                          'Cell-ToAddMod-r12': {'members': [{'name': 'sCellIndex-r12',
                                                                             'type': 'SCellIndex-r10'},
                                                                            {'members': [{'name': 'physCellId-r12',
                                                                                          'type': 'PhysCellId'},
                                                                                         {'name': 'dl-CarrierFreq-r12',
                                                                                          'type': 'ARFCN-ValueEUTRA-r9'}],
                                                                             'name': 'cellIdentification-r12',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'rsrpResult-r12',
                                                                                          'type': 'RSRP-Range'},
                                                                                         {'name': 'rsrqResult-r12',
                                                                                          'type': 'RSRQ-Range'}],
                                                                             'name': 'measResultCellToAdd-r12',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            None,
                                                                            [{'name': 'sCellIndex-r13',
                                                                              'optional': True,
                                                                              'type': 'SCellIndex-r13'},
                                                                             {'members': [{'name': 'rs-sinr-Result-r13',
                                                                                           'type': 'RS-SINR-Range-r13'}],
                                                                              'name': 'measResultCellToAdd-v1310',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'}]],
                                                                'type': 'SEQUENCE'},
                                          'DRB-InfoListSCG-r12': {'element': {'type': 'DRB-InfoSCG-r12'},
                                                                  'size': [(1,
                                                                            'maxDRB')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                          'DRB-InfoSCG-r12': {'members': [{'name': 'eps-BearerIdentity-r12',
                                                                           'optional': True,
                                                                           'restricted-to': [(0,
                                                                                              15)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'drb-Identity-r12',
                                                                           'type': 'DRB-Identity'},
                                                                          {'name': 'drb-Type-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('split',
                                                                                       0),
                                                                                      ('scg',
                                                                                       1)]},
                                                                          None],
                                                              'type': 'SEQUENCE'},
                                          'HandoverCommand': {'members': [{'members': [{'members': [{'name': 'handoverCommand-r8',
                                                                                                     'type': 'HandoverCommand-r8-IEs'},
                                                                                                    {'name': 'spare7',
                                                                                                     'type': 'NULL'},
                                                                                                    {'name': 'spare6',
                                                                                                     'type': 'NULL'},
                                                                                                    {'name': 'spare5',
                                                                                                     'type': 'NULL'},
                                                                                                    {'name': 'spare4',
                                                                                                     'type': 'NULL'},
                                                                                                    {'name': 'spare3',
                                                                                                     'type': 'NULL'},
                                                                                                    {'name': 'spare2',
                                                                                                     'type': 'NULL'},
                                                                                                    {'name': 'spare1',
                                                                                                     'type': 'NULL'}],
                                                                                        'name': 'c1',
                                                                                        'type': 'CHOICE'},
                                                                                       {'members': [],
                                                                                        'name': 'criticalExtensionsFuture',
                                                                                        'type': 'SEQUENCE'}],
                                                                           'name': 'criticalExtensions',
                                                                           'type': 'CHOICE'}],
                                                              'type': 'SEQUENCE'},
                                          'HandoverCommand-r8-IEs': {'members': [{'name': 'handoverCommandMessage',
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 {'members': [],
                                                                                  'name': 'nonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'SEQUENCE'}],
                                                                     'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation': {'members': [{'members': [{'members': [{'name': 'handoverPreparationInformation-r8',
                                                                                                                    'type': 'HandoverPreparationInformation-r8-IEs'},
                                                                                                                   {'name': 'spare7',
                                                                                                                    'type': 'NULL'},
                                                                                                                   {'name': 'spare6',
                                                                                                                    'type': 'NULL'},
                                                                                                                   {'name': 'spare5',
                                                                                                                    'type': 'NULL'},
                                                                                                                   {'name': 'spare4',
                                                                                                                    'type': 'NULL'},
                                                                                                                   {'name': 'spare3',
                                                                                                                    'type': 'NULL'},
                                                                                                                   {'name': 'spare2',
                                                                                                                    'type': 'NULL'},
                                                                                                                   {'name': 'spare1',
                                                                                                                    'type': 'NULL'}],
                                                                                                       'name': 'c1',
                                                                                                       'type': 'CHOICE'},
                                                                                                      {'members': [],
                                                                                                       'name': 'criticalExtensionsFuture',
                                                                                                       'type': 'SEQUENCE'}],
                                                                                          'name': 'criticalExtensions',
                                                                                          'type': 'CHOICE'}],
                                                                             'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-r8-IEs': {'members': [{'name': 'ue-RadioAccessCapabilityInfo',
                                                                                                 'type': 'UE-CapabilityRAT-ContainerList'},
                                                                                                {'name': 'as-Config',
                                                                                                 'optional': True,
                                                                                                 'type': 'AS-Config'},
                                                                                                {'name': 'rrm-Config',
                                                                                                 'optional': True,
                                                                                                 'type': 'RRM-Config'},
                                                                                                {'name': 'as-Context',
                                                                                                 'optional': True,
                                                                                                 'type': 'AS-Context'},
                                                                                                {'name': 'nonCriticalExtension',
                                                                                                 'optional': True,
                                                                                                 'type': 'HandoverPreparationInformation-v920-IEs'}],
                                                                                    'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-v10j0-IEs': {'members': [{'name': 'as-Config-v10j0',
                                                                                                    'optional': True,
                                                                                                    'type': 'AS-Config-v10j0'},
                                                                                                   {'members': [],
                                                                                                    'name': 'nonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'SEQUENCE'}],
                                                                                       'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-v1130-IEs': {'members': [{'name': 'as-Context-v1130',
                                                                                                    'optional': True,
                                                                                                    'type': 'AS-Context-v1130'},
                                                                                                   {'name': 'nonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'HandoverPreparationInformation-v1250-IEs'}],
                                                                                       'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-v1250-IEs': {'members': [{'name': 'ue-SupportedEARFCN-r12',
                                                                                                    'optional': True,
                                                                                                    'type': 'ARFCN-ValueEUTRA-r9'},
                                                                                                   {'name': 'as-Config-v1250',
                                                                                                    'optional': True,
                                                                                                    'type': 'AS-Config-v1250'},
                                                                                                   {'name': 'nonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'HandoverPreparationInformation-v1320-IEs'}],
                                                                                       'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-v1320-IEs': {'members': [{'name': 'as-Config-v1320',
                                                                                                    'optional': True,
                                                                                                    'type': 'AS-Config-v1320'},
                                                                                                   {'name': 'as-Context-v1320',
                                                                                                    'optional': True,
                                                                                                    'type': 'AS-Context-v1320'},
                                                                                                   {'name': 'nonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'HandoverPreparationInformation-v1430-IEs'}],
                                                                                       'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-v1430-IEs': {'members': [{'name': 'as-Config-v1430',
                                                                                                    'optional': True,
                                                                                                    'type': 'AS-Config-v1430'},
                                                                                                   {'name': 'makeBeforeBreakReq-r14',
                                                                                                    'optional': True,
                                                                                                    'type': 'ENUMERATED',
                                                                                                    'values': [('true',
                                                                                                                0)]},
                                                                                                   {'members': [],
                                                                                                    'name': 'nonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'SEQUENCE'}],
                                                                                       'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-v920-IEs': {'members': [{'name': 'ue-ConfigRelease-r9',
                                                                                                   'optional': True,
                                                                                                   'type': 'ENUMERATED',
                                                                                                   'values': [('rel9',
                                                                                                               0),
                                                                                                              ('rel10',
                                                                                                               1),
                                                                                                              ('rel11',
                                                                                                               2),
                                                                                                              ('rel12',
                                                                                                               3),
                                                                                                              ('v10j0',
                                                                                                               4),
                                                                                                              ('v11e0',
                                                                                                               5),
                                                                                                              ('v1280',
                                                                                                               6),
                                                                                                              ('rel13',
                                                                                                               7),
                                                                                                              None,
                                                                                                              ('rel14',
                                                                                                               8)]},
                                                                                                  {'name': 'nonCriticalExtension',
                                                                                                   'optional': True,
                                                                                                   'type': 'HandoverPreparationInformation-v9d0-IEs'}],
                                                                                      'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-v9d0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                                   'optional': True,
                                                                                                   'type': 'OCTET '
                                                                                                           'STRING'},
                                                                                                  {'name': 'nonCriticalExtension',
                                                                                                   'optional': True,
                                                                                                   'type': 'HandoverPreparationInformation-v9e0-IEs'}],
                                                                                      'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-v9e0-IEs': {'members': [{'name': 'as-Config-v9e0',
                                                                                                   'optional': True,
                                                                                                   'type': 'AS-Config-v9e0'},
                                                                                                  {'name': 'nonCriticalExtension',
                                                                                                   'optional': True,
                                                                                                   'type': 'HandoverPreparationInformation-v1130-IEs'}],
                                                                                      'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-v9j0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                                   'optional': True,
                                                                                                   'type': 'OCTET '
                                                                                                           'STRING'},
                                                                                                  {'name': 'nonCriticalExtension',
                                                                                                   'optional': True,
                                                                                                   'type': 'HandoverPreparationInformation-v10j0-IEs'}],
                                                                                      'type': 'SEQUENCE'},
                                          'Key-eNodeB-Star': {'size': [256],
                                                              'type': 'BIT '
                                                                      'STRING'},
                                          'MeasResultListRSSI-SCG-r13': {'element': {'type': 'MeasResultRSSI-SCG-r13'},
                                                                         'size': [(1,
                                                                                   'maxServCell-r13')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                          'MeasResultRSSI-SCG-r13': {'members': [{'name': 'servCellId-r13',
                                                                                  'type': 'ServCellIndex-r13'},
                                                                                 {'name': 'measResultForRSSI-r13',
                                                                                  'type': 'MeasResultForRSSI-r13'}],
                                                                     'type': 'SEQUENCE'},
                                          'MeasResultServCellListSCG-Ext-r13': {'element': {'type': 'MeasResultServCellSCG-r12'},
                                                                                'size': [(1,
                                                                                          'maxServCell-r13')],
                                                                                'type': 'SEQUENCE '
                                                                                        'OF'},
                                          'MeasResultServCellListSCG-r12': {'element': {'type': 'MeasResultServCellSCG-r12'},
                                                                            'size': [(1,
                                                                                      'maxServCell-r10')],
                                                                            'type': 'SEQUENCE '
                                                                                    'OF'},
                                          'MeasResultServCellSCG-r12': {'members': [{'name': 'servCellId-r12',
                                                                                     'type': 'ServCellIndex-r10'},
                                                                                    {'members': [{'name': 'rsrpResultSCell-r12',
                                                                                                  'type': 'RSRP-Range'},
                                                                                                 {'name': 'rsrqResultSCell-r12',
                                                                                                  'type': 'RSRQ-Range'}],
                                                                                     'name': 'measResultSCell-r12',
                                                                                     'type': 'SEQUENCE'},
                                                                                    None,
                                                                                    [{'name': 'servCellId-r13',
                                                                                      'optional': True,
                                                                                      'type': 'ServCellIndex-r13'},
                                                                                     {'members': [{'name': 'rs-sinr-ResultSCell-r13',
                                                                                                   'type': 'RS-SINR-Range-r13'}],
                                                                                      'name': 'measResultSCell-v1310',
                                                                                      'optional': True,
                                                                                      'type': 'SEQUENCE'}]],
                                                                        'type': 'SEQUENCE'},
                                          'RRM-Config': {'members': [{'name': 'ue-InactiveTime',
                                                                      'optional': True,
                                                                      'type': 'ENUMERATED',
                                                                      'values': [('s1',
                                                                                  0),
                                                                                 ('s2',
                                                                                  1),
                                                                                 ('s3',
                                                                                  2),
                                                                                 ('s5',
                                                                                  3),
                                                                                 ('s7',
                                                                                  4),
                                                                                 ('s10',
                                                                                  5),
                                                                                 ('s15',
                                                                                  6),
                                                                                 ('s20',
                                                                                  7),
                                                                                 ('s25',
                                                                                  8),
                                                                                 ('s30',
                                                                                  9),
                                                                                 ('s40',
                                                                                  10),
                                                                                 ('s50',
                                                                                  11),
                                                                                 ('min1',
                                                                                  12),
                                                                                 ('min1s20c',
                                                                                  13),
                                                                                 ('min1s40',
                                                                                  14),
                                                                                 ('min2',
                                                                                  15),
                                                                                 ('min2s30',
                                                                                  16),
                                                                                 ('min3',
                                                                                  17),
                                                                                 ('min3s30',
                                                                                  18),
                                                                                 ('min4',
                                                                                  19),
                                                                                 ('min5',
                                                                                  20),
                                                                                 ('min6',
                                                                                  21),
                                                                                 ('min7',
                                                                                  22),
                                                                                 ('min8',
                                                                                  23),
                                                                                 ('min9',
                                                                                  24),
                                                                                 ('min10',
                                                                                  25),
                                                                                 ('min12',
                                                                                  26),
                                                                                 ('min14',
                                                                                  27),
                                                                                 ('min17',
                                                                                  28),
                                                                                 ('min20',
                                                                                  29),
                                                                                 ('min24',
                                                                                  30),
                                                                                 ('min28',
                                                                                  31),
                                                                                 ('min33',
                                                                                  32),
                                                                                 ('min38',
                                                                                  33),
                                                                                 ('min44',
                                                                                  34),
                                                                                 ('min50',
                                                                                  35),
                                                                                 ('hr1',
                                                                                  36),
                                                                                 ('hr1min30',
                                                                                  37),
                                                                                 ('hr2',
                                                                                  38),
                                                                                 ('hr2min30',
                                                                                  39),
                                                                                 ('hr3',
                                                                                  40),
                                                                                 ('hr3min30',
                                                                                  41),
                                                                                 ('hr4',
                                                                                  42),
                                                                                 ('hr5',
                                                                                  43),
                                                                                 ('hr6',
                                                                                  44),
                                                                                 ('hr8',
                                                                                  45),
                                                                                 ('hr10',
                                                                                  46),
                                                                                 ('hr13',
                                                                                  47),
                                                                                 ('hr16',
                                                                                  48),
                                                                                 ('hr20',
                                                                                  49),
                                                                                 ('day1',
                                                                                  50),
                                                                                 ('day1hr12',
                                                                                  51),
                                                                                 ('day2',
                                                                                  52),
                                                                                 ('day2hr12',
                                                                                  53),
                                                                                 ('day3',
                                                                                  54),
                                                                                 ('day4',
                                                                                  55),
                                                                                 ('day5',
                                                                                  56),
                                                                                 ('day7',
                                                                                  57),
                                                                                 ('day10',
                                                                                  58),
                                                                                 ('day14',
                                                                                  59),
                                                                                 ('day19',
                                                                                  60),
                                                                                 ('day24',
                                                                                  61),
                                                                                 ('day30',
                                                                                  62),
                                                                                 ('dayMoreThan30',
                                                                                  63)]},
                                                                     None,
                                                                     [{'name': 'candidateCellInfoList-r10',
                                                                       'optional': True,
                                                                       'type': 'CandidateCellInfoList-r10'}]],
                                                         'type': 'SEQUENCE'},
                                          'ReestablishmentInfo': {'members': [{'name': 'sourcePhysCellId',
                                                                               'type': 'PhysCellId'},
                                                                              {'name': 'targetCellShortMAC-I',
                                                                               'type': 'ShortMAC-I'},
                                                                              {'name': 'additionalReestabInfoList',
                                                                               'optional': True,
                                                                               'type': 'AdditionalReestabInfoList'},
                                                                              None],
                                                                  'type': 'SEQUENCE'},
                                          'SCG-Config-r12': {'members': [{'members': [{'members': [{'name': 'scg-Config-r12',
                                                                                                    'type': 'SCG-Config-r12-IEs'},
                                                                                                   {'name': 'spare7',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare6',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare5',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare4',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare3',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare2',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare1',
                                                                                                    'type': 'NULL'}],
                                                                                       'name': 'c1',
                                                                                       'type': 'CHOICE'},
                                                                                      {'members': [],
                                                                                       'name': 'criticalExtensionsFuture',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'name': 'criticalExtensions',
                                                                          'type': 'CHOICE'}],
                                                             'type': 'SEQUENCE'},
                                          'SCG-Config-r12-IEs': {'members': [{'name': 'scg-RadioConfig-r12',
                                                                              'optional': True,
                                                                              'type': 'SCG-ConfigPartSCG-r12'},
                                                                             {'members': [],
                                                                              'name': 'nonCriticalExtension',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'}],
                                                                 'type': 'SEQUENCE'},
                                          'SCG-ConfigInfo-r12': {'members': [{'members': [{'members': [{'name': 'scg-ConfigInfo-r12',
                                                                                                        'type': 'SCG-ConfigInfo-r12-IEs'},
                                                                                                       {'name': 'spare7',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare6',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare5',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare4',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare3',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare2',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare1',
                                                                                                        'type': 'NULL'}],
                                                                                           'name': 'c1',
                                                                                           'type': 'CHOICE'},
                                                                                          {'members': [],
                                                                                           'name': 'criticalExtensionsFuture',
                                                                                           'type': 'SEQUENCE'}],
                                                                              'name': 'criticalExtensions',
                                                                              'type': 'CHOICE'}],
                                                                 'type': 'SEQUENCE'},
                                          'SCG-ConfigInfo-r12-IEs': {'members': [{'name': 'radioResourceConfigDedMCG-r12',
                                                                                  'optional': True,
                                                                                  'type': 'RadioResourceConfigDedicated'},
                                                                                 {'name': 'sCellToAddModListMCG-r12',
                                                                                  'optional': True,
                                                                                  'type': 'SCellToAddModList-r10'},
                                                                                 {'name': 'measGapConfig-r12',
                                                                                  'optional': True,
                                                                                  'type': 'MeasGapConfig'},
                                                                                 {'name': 'powerCoordinationInfo-r12',
                                                                                  'optional': True,
                                                                                  'type': 'PowerCoordinationInfo-r12'},
                                                                                 {'name': 'scg-RadioConfig-r12',
                                                                                  'optional': True,
                                                                                  'type': 'SCG-ConfigPartSCG-r12'},
                                                                                 {'name': 'eutra-CapabilityInfo-r12',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 {'name': 'scg-ConfigRestrictInfo-r12',
                                                                                  'optional': True,
                                                                                  'type': 'SCG-ConfigRestrictInfo-r12'},
                                                                                 {'name': 'mbmsInterestIndication-r12',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 {'name': 'measResultServCellListSCG-r12',
                                                                                  'optional': True,
                                                                                  'type': 'MeasResultServCellListSCG-r12'},
                                                                                 {'name': 'drb-ToAddModListSCG-r12',
                                                                                  'optional': True,
                                                                                  'type': 'DRB-InfoListSCG-r12'},
                                                                                 {'name': 'drb-ToReleaseListSCG-r12',
                                                                                  'optional': True,
                                                                                  'type': 'DRB-ToReleaseList'},
                                                                                 {'name': 'sCellToAddModListSCG-r12',
                                                                                  'optional': True,
                                                                                  'type': 'SCellToAddModListSCG-r12'},
                                                                                 {'name': 'sCellToReleaseListSCG-r12',
                                                                                  'optional': True,
                                                                                  'type': 'SCellToReleaseList-r10'},
                                                                                 {'name': 'p-Max-r12',
                                                                                  'optional': True,
                                                                                  'type': 'P-Max'},
                                                                                 {'name': 'nonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'SCG-ConfigInfo-v1310-IEs'}],
                                                                     'type': 'SEQUENCE'},
                                          'SCG-ConfigInfo-v1310-IEs': {'members': [{'name': 'measResultSSTD-r13',
                                                                                    'optional': True,
                                                                                    'type': 'MeasResultSSTD-r13'},
                                                                                   {'name': 'sCellToAddModListMCG-Ext-r13',
                                                                                    'optional': True,
                                                                                    'type': 'SCellToAddModListExt-r13'},
                                                                                   {'name': 'measResultServCellListSCG-Ext-r13',
                                                                                    'optional': True,
                                                                                    'type': 'MeasResultServCellListSCG-Ext-r13'},
                                                                                   {'name': 'sCellToAddModListSCG-Ext-r13',
                                                                                    'optional': True,
                                                                                    'type': 'SCellToAddModListSCG-Ext-r13'},
                                                                                   {'name': 'sCellToReleaseListSCG-Ext-r13',
                                                                                    'optional': True,
                                                                                    'type': 'SCellToReleaseListExt-r13'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SCG-ConfigInfo-v1330-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                          'SCG-ConfigInfo-v1330-IEs': {'members': [{'name': 'measResultListRSSI-SCG-r13',
                                                                                    'optional': True,
                                                                                    'type': 'MeasResultListRSSI-SCG-r13'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SCG-ConfigInfo-v1430-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                          'SCG-ConfigInfo-v1430-IEs': {'members': [{'name': 'makeBeforeBreakSCG-Req-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]},
                                                                                   {'name': 'measGapConfigPerCC-List',
                                                                                    'optional': True,
                                                                                    'type': 'MeasGapConfigPerCC-List-r14'},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                          'SCG-ConfigRestrictInfo-r12': {'members': [{'name': 'maxSCH-TB-BitsDL-r12',
                                                                                      'restricted-to': [(1,
                                                                                                         100)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'maxSCH-TB-BitsUL-r12',
                                                                                      'restricted-to': [(1,
                                                                                                         100)],
                                                                                      'type': 'INTEGER'}],
                                                                         'type': 'SEQUENCE'},
                                          'SCellToAddModListSCG-Ext-r13': {'element': {'type': 'Cell-ToAddMod-r12'},
                                                                           'size': [(1,
                                                                                     'maxSCell-r13')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                          'SCellToAddModListSCG-r12': {'element': {'type': 'Cell-ToAddMod-r12'},
                                                                       'size': [(1,
                                                                                 'maxSCell-r10')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                          'UEPagingCoverageInformation': {'members': [{'members': [{'members': [{'name': 'uePagingCoverageInformation-r13',
                                                                                                                 'type': 'UEPagingCoverageInformation-r13-IEs'},
                                                                                                                {'name': 'spare7',
                                                                                                                 'type': 'NULL'},
                                                                                                                {'name': 'spare6',
                                                                                                                 'type': 'NULL'},
                                                                                                                {'name': 'spare5',
                                                                                                                 'type': 'NULL'},
                                                                                                                {'name': 'spare4',
                                                                                                                 'type': 'NULL'},
                                                                                                                {'name': 'spare3',
                                                                                                                 'type': 'NULL'},
                                                                                                                {'name': 'spare2',
                                                                                                                 'type': 'NULL'},
                                                                                                                {'name': 'spare1',
                                                                                                                 'type': 'NULL'}],
                                                                                                    'name': 'c1',
                                                                                                    'type': 'CHOICE'},
                                                                                                   {'members': [],
                                                                                                    'name': 'criticalExtensionsFuture',
                                                                                                    'type': 'SEQUENCE'}],
                                                                                       'name': 'criticalExtensions',
                                                                                       'type': 'CHOICE'}],
                                                                          'type': 'SEQUENCE'},
                                          'UEPagingCoverageInformation-r13-IEs': {'members': [{'name': 'mpdcch-NumRepetition-r13',
                                                                                               'optional': True,
                                                                                               'restricted-to': [(1,
                                                                                                                  256)],
                                                                                               'type': 'INTEGER'},
                                                                                              {'members': [],
                                                                                               'name': 'nonCriticalExtension',
                                                                                               'optional': True,
                                                                                               'type': 'SEQUENCE'}],
                                                                                  'type': 'SEQUENCE'},
                                          'UERadioAccessCapabilityInformation': {'members': [{'members': [{'members': [{'name': 'ueRadioAccessCapabilityInformation-r8',
                                                                                                                        'type': 'UERadioAccessCapabilityInformation-r8-IEs'},
                                                                                                                       {'name': 'spare7',
                                                                                                                        'type': 'NULL'},
                                                                                                                       {'name': 'spare6',
                                                                                                                        'type': 'NULL'},
                                                                                                                       {'name': 'spare5',
                                                                                                                        'type': 'NULL'},
                                                                                                                       {'name': 'spare4',
                                                                                                                        'type': 'NULL'},
                                                                                                                       {'name': 'spare3',
                                                                                                                        'type': 'NULL'},
                                                                                                                       {'name': 'spare2',
                                                                                                                        'type': 'NULL'},
                                                                                                                       {'name': 'spare1',
                                                                                                                        'type': 'NULL'}],
                                                                                                           'name': 'c1',
                                                                                                           'type': 'CHOICE'},
                                                                                                          {'members': [],
                                                                                                           'name': 'criticalExtensionsFuture',
                                                                                                           'type': 'SEQUENCE'}],
                                                                                              'name': 'criticalExtensions',
                                                                                              'type': 'CHOICE'}],
                                                                                 'type': 'SEQUENCE'},
                                          'UERadioAccessCapabilityInformation-r8-IEs': {'members': [{'name': 'ue-RadioAccessCapabilityInfo',
                                                                                                     'type': 'OCTET '
                                                                                                             'STRING'},
                                                                                                    {'members': [],
                                                                                                     'name': 'nonCriticalExtension',
                                                                                                     'optional': True,
                                                                                                     'type': 'SEQUENCE'}],
                                                                                        'type': 'SEQUENCE'},
                                          'UERadioPagingInformation': {'members': [{'members': [{'members': [{'name': 'ueRadioPagingInformation-r12',
                                                                                                              'type': 'UERadioPagingInformation-r12-IEs'},
                                                                                                             {'name': 'spare7',
                                                                                                              'type': 'NULL'},
                                                                                                             {'name': 'spare6',
                                                                                                              'type': 'NULL'},
                                                                                                             {'name': 'spare5',
                                                                                                              'type': 'NULL'},
                                                                                                             {'name': 'spare4',
                                                                                                              'type': 'NULL'},
                                                                                                             {'name': 'spare3',
                                                                                                              'type': 'NULL'},
                                                                                                             {'name': 'spare2',
                                                                                                              'type': 'NULL'},
                                                                                                             {'name': 'spare1',
                                                                                                              'type': 'NULL'}],
                                                                                                 'name': 'c1',
                                                                                                 'type': 'CHOICE'},
                                                                                                {'members': [],
                                                                                                 'name': 'criticalExtensionsFuture',
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'name': 'criticalExtensions',
                                                                                    'type': 'CHOICE'}],
                                                                       'type': 'SEQUENCE'},
                                          'UERadioPagingInformation-r12-IEs': {'members': [{'name': 'ue-RadioPagingInfo-r12',
                                                                                            'type': 'OCTET '
                                                                                                    'STRING'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'UERadioPagingInformation-v1310-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                          'UERadioPagingInformation-v1310-IEs': {'members': [{'element': {'type': 'FreqBandIndicator-r11'},
                                                                                              'name': 'supportedBandListEUTRAForPaging-r13',
                                                                                              'optional': True,
                                                                                              'size': [(1,
                                                                                                        'maxBands')],
                                                                                              'type': 'SEQUENCE '
                                                                                                      'OF'},
                                                                                             {'members': [],
                                                                                              'name': 'nonCriticalExtension',
                                                                                              'optional': True,
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'type': 'SEQUENCE'}},
                                'values': {'maxReestabInfo': {'type': 'INTEGER',
                                                              'value': 32}}},
 'EUTRA-RRC-Definitions': {'extensibility-implied': False,
                           'imports': {},
                           'object-classes': {},
                           'object-sets': {},
                           'tags': 'AUTOMATIC',
                           'types': {'AC-BarringConfig': {'members': [{'name': 'ac-BarringFactor',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('p00',
                                                                                   0),
                                                                                  ('p05',
                                                                                   1),
                                                                                  ('p10',
                                                                                   2),
                                                                                  ('p15',
                                                                                   3),
                                                                                  ('p20',
                                                                                   4),
                                                                                  ('p25',
                                                                                   5),
                                                                                  ('p30',
                                                                                   6),
                                                                                  ('p40',
                                                                                   7),
                                                                                  ('p50',
                                                                                   8),
                                                                                  ('p60',
                                                                                   9),
                                                                                  ('p70',
                                                                                   10),
                                                                                  ('p75',
                                                                                   11),
                                                                                  ('p80',
                                                                                   12),
                                                                                  ('p85',
                                                                                   13),
                                                                                  ('p90',
                                                                                   14),
                                                                                  ('p95',
                                                                                   15)]},
                                                                      {'name': 'ac-BarringTime',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('s4',
                                                                                   0),
                                                                                  ('s8',
                                                                                   1),
                                                                                  ('s16',
                                                                                   2),
                                                                                  ('s32',
                                                                                   3),
                                                                                  ('s64',
                                                                                   4),
                                                                                  ('s128',
                                                                                   5),
                                                                                  ('s256',
                                                                                   6),
                                                                                  ('s512',
                                                                                   7)]},
                                                                      {'name': 'ac-BarringForSpecialAC',
                                                                       'size': [5],
                                                                       'type': 'BIT '
                                                                               'STRING'}],
                                                          'type': 'SEQUENCE'},
                                     'AC-BarringConfig1XRTT-r9': {'members': [{'name': 'ac-Barring0to9-r9',
                                                                               'restricted-to': [(0,
                                                                                                  63)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'ac-Barring10-r9',
                                                                               'restricted-to': [(0,
                                                                                                  7)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'ac-Barring11-r9',
                                                                               'restricted-to': [(0,
                                                                                                  7)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'ac-Barring12-r9',
                                                                               'restricted-to': [(0,
                                                                                                  7)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'ac-Barring13-r9',
                                                                               'restricted-to': [(0,
                                                                                                  7)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'ac-Barring14-r9',
                                                                               'restricted-to': [(0,
                                                                                                  7)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'ac-Barring15-r9',
                                                                               'restricted-to': [(0,
                                                                                                  7)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'ac-BarringMsg-r9',
                                                                               'restricted-to': [(0,
                                                                                                  7)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'ac-BarringReg-r9',
                                                                               'restricted-to': [(0,
                                                                                                  7)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'ac-BarringEmg-r9',
                                                                               'restricted-to': [(0,
                                                                                                  7)],
                                                                               'type': 'INTEGER'}],
                                                                  'type': 'SEQUENCE'},
                                     'AC-BarringPerPLMN-List-r12': {'element': {'type': 'AC-BarringPerPLMN-r12'},
                                                                    'size': [(1,
                                                                              'maxPLMN-r11')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'AC-BarringPerPLMN-r12': {'members': [{'name': 'plmn-IdentityIndex-r12',
                                                                            'restricted-to': [(1,
                                                                                               'maxPLMN-r11')],
                                                                            'type': 'INTEGER'},
                                                                           {'members': [{'name': 'ac-BarringForEmergency-r12',
                                                                                         'type': 'BOOLEAN'},
                                                                                        {'name': 'ac-BarringForMO-Signalling-r12',
                                                                                         'optional': True,
                                                                                         'type': 'AC-BarringConfig'},
                                                                                        {'name': 'ac-BarringForMO-Data-r12',
                                                                                         'optional': True,
                                                                                         'type': 'AC-BarringConfig'}],
                                                                            'name': 'ac-BarringInfo-r12',
                                                                            'optional': True,
                                                                            'type': 'SEQUENCE'},
                                                                           {'name': 'ac-BarringSkipForMMTELVoice-r12',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('true',
                                                                                        0)]},
                                                                           {'name': 'ac-BarringSkipForMMTELVideo-r12',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('true',
                                                                                        0)]},
                                                                           {'name': 'ac-BarringSkipForSMS-r12',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('true',
                                                                                        0)]},
                                                                           {'name': 'ac-BarringForCSFB-r12',
                                                                            'optional': True,
                                                                            'type': 'AC-BarringConfig'},
                                                                           {'name': 'ssac-BarringForMMTEL-Voice-r12',
                                                                            'optional': True,
                                                                            'type': 'AC-BarringConfig'},
                                                                           {'name': 'ssac-BarringForMMTEL-Video-r12',
                                                                            'optional': True,
                                                                            'type': 'AC-BarringConfig'}],
                                                               'type': 'SEQUENCE'},
                                     'ACDC-BarringForCommon-r13': {'members': [{'name': 'acdc-HPLMNonly-r13',
                                                                                'type': 'BOOLEAN'},
                                                                               {'name': 'barringPerACDC-CategoryList-r13',
                                                                                'type': 'BarringPerACDC-CategoryList-r13'}],
                                                                   'type': 'SEQUENCE'},
                                     'ACDC-BarringPerPLMN-List-r13': {'element': {'type': 'ACDC-BarringPerPLMN-r13'},
                                                                      'size': [(1,
                                                                                'maxPLMN-r11')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'ACDC-BarringPerPLMN-r13': {'members': [{'name': 'plmn-IdentityIndex-r13',
                                                                              'restricted-to': [(1,
                                                                                                 'maxPLMN-r11')],
                                                                              'type': 'INTEGER'},
                                                                             {'name': 'acdc-OnlyForHPLMN-r13',
                                                                              'type': 'BOOLEAN'},
                                                                             {'name': 'barringPerACDC-CategoryList-r13',
                                                                              'type': 'BarringPerACDC-CategoryList-r13'}],
                                                                 'type': 'SEQUENCE'},
                                     'ARFCN-ValueCDMA2000': {'restricted-to': [(0,
                                                                                2047)],
                                                             'type': 'INTEGER'},
                                     'ARFCN-ValueEUTRA': {'restricted-to': [(0,
                                                                             'maxEARFCN')],
                                                          'type': 'INTEGER'},
                                     'ARFCN-ValueEUTRA-r9': {'restricted-to': [(0,
                                                                                'maxEARFCN2')],
                                                             'type': 'INTEGER'},
                                     'ARFCN-ValueEUTRA-v9e0': {'restricted-to': [('maxEARFCN-Plus1',
                                                                                  'maxEARFCN2')],
                                                               'type': 'INTEGER'},
                                     'ARFCN-ValueGERAN': {'restricted-to': [(0,
                                                                             1023)],
                                                          'type': 'INTEGER'},
                                     'ARFCN-ValueUTRA': {'restricted-to': [(0,
                                                                            16383)],
                                                         'type': 'INTEGER'},
                                     'AbsoluteTimeInfo-r10': {'size': [48],
                                                              'type': 'BIT '
                                                                      'STRING'},
                                     'AccessStratumRelease': {'type': 'ENUMERATED',
                                                              'values': [('rel8',
                                                                          0),
                                                                         ('rel9',
                                                                          1),
                                                                         ('rel10',
                                                                          2),
                                                                         ('rel11',
                                                                          3),
                                                                         ('rel12',
                                                                          4),
                                                                         ('rel13',
                                                                          5),
                                                                         ('rel14',
                                                                          6),
                                                                         ('spare1',
                                                                          7),
                                                                         None]},
                                     'AdditionalSI-Info-r9': {'members': [{'name': 'csg-MemberStatus-r9',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('member',
                                                                                       0)]},
                                                                          {'name': 'csg-Identity-r9',
                                                                           'optional': True,
                                                                           'type': 'CSG-Identity'}],
                                                              'type': 'SEQUENCE'},
                                     'AdditionalSpectrumEmission': {'restricted-to': [(1,
                                                                                       32)],
                                                                    'type': 'INTEGER'},
                                     'AdditionalSpectrumEmission-v10l0': {'restricted-to': [(33,
                                                                                             288)],
                                                                          'type': 'INTEGER'},
                                     'AffectedCarrierFreq-r11': {'members': [{'name': 'carrierFreq-r11',
                                                                              'type': 'MeasObjectId'},
                                                                             {'name': 'interferenceDirection-r11',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('eutra',
                                                                                          0),
                                                                                         ('other',
                                                                                          1),
                                                                                         ('both',
                                                                                          2),
                                                                                         ('spare',
                                                                                          3)]}],
                                                                 'type': 'SEQUENCE'},
                                     'AffectedCarrierFreq-v1310': {'members': [{'name': 'carrierFreq-v1310',
                                                                                'optional': True,
                                                                                'type': 'MeasObjectId-v1310'}],
                                                                   'type': 'SEQUENCE'},
                                     'AffectedCarrierFreqComb-r11': {'element': {'type': 'MeasObjectId'},
                                                                     'size': [(2,
                                                                               'maxServCell-r10')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'AffectedCarrierFreqComb-r13': {'element': {'type': 'MeasObjectId-r13'},
                                                                     'size': [(2,
                                                                               'maxServCell-r13')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'AffectedCarrierFreqCombList-r11': {'element': {'type': 'AffectedCarrierFreqComb-r11'},
                                                                         'size': [(1,
                                                                                   'maxCombIDC-r11')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'AffectedCarrierFreqCombList-r13': {'element': {'type': 'AffectedCarrierFreqComb-r13'},
                                                                         'size': [(1,
                                                                                   'maxCombIDC-r11')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'AffectedCarrierFreqList-r11': {'element': {'type': 'AffectedCarrierFreq-r11'},
                                                                     'size': [(1,
                                                                               'maxFreqIDC-r11')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'AffectedCarrierFreqList-v1310': {'element': {'type': 'AffectedCarrierFreq-v1310'},
                                                                       'size': [(1,
                                                                                 'maxFreqIDC-r11')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'AllowedMeasBandwidth': {'type': 'ENUMERATED',
                                                              'values': [('mbw6',
                                                                          0),
                                                                         ('mbw15',
                                                                          1),
                                                                         ('mbw25',
                                                                          2),
                                                                         ('mbw50',
                                                                          3),
                                                                         ('mbw75',
                                                                          4),
                                                                         ('mbw100',
                                                                          5)]},
                                     'Alpha-r12': {'type': 'ENUMERATED',
                                                   'values': [('al0', 0),
                                                              ('al04', 1),
                                                              ('al05', 2),
                                                              ('al06', 3),
                                                              ('al07', 4),
                                                              ('al08', 5),
                                                              ('al09', 6),
                                                              ('al1', 7)]},
                                     'AltTTT-CellsToAddMod-r12': {'members': [{'name': 'cellIndex-r12',
                                                                               'restricted-to': [(1,
                                                                                                  'maxCellMeas')],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'physCellIdRange-r12',
                                                                               'type': 'PhysCellIdRange'}],
                                                                  'type': 'SEQUENCE'},
                                     'AltTTT-CellsToAddModList-r12': {'element': {'type': 'AltTTT-CellsToAddMod-r12'},
                                                                      'size': [(1,
                                                                                'maxCellMeas')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'AntennaInfoCommon': {'members': [{'name': 'antennaPortsCount',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('an1',
                                                                                    0),
                                                                                   ('an2',
                                                                                    1),
                                                                                   ('an4',
                                                                                    2),
                                                                                   ('spare1',
                                                                                    3)]}],
                                                           'type': 'SEQUENCE'},
                                     'AntennaInfoDedicated': {'members': [{'name': 'transmissionMode',
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('tm1',
                                                                                       0),
                                                                                      ('tm2',
                                                                                       1),
                                                                                      ('tm3',
                                                                                       2),
                                                                                      ('tm4',
                                                                                       3),
                                                                                      ('tm5',
                                                                                       4),
                                                                                      ('tm6',
                                                                                       5),
                                                                                      ('tm7',
                                                                                       6),
                                                                                      ('tm8-v920',
                                                                                       7)]},
                                                                          {'members': [{'name': 'n2TxAntenna-tm3',
                                                                                        'size': [2],
                                                                                        'type': 'BIT '
                                                                                                'STRING'},
                                                                                       {'name': 'n4TxAntenna-tm3',
                                                                                        'size': [4],
                                                                                        'type': 'BIT '
                                                                                                'STRING'},
                                                                                       {'name': 'n2TxAntenna-tm4',
                                                                                        'size': [6],
                                                                                        'type': 'BIT '
                                                                                                'STRING'},
                                                                                       {'name': 'n4TxAntenna-tm4',
                                                                                        'size': [64],
                                                                                        'type': 'BIT '
                                                                                                'STRING'},
                                                                                       {'name': 'n2TxAntenna-tm5',
                                                                                        'size': [4],
                                                                                        'type': 'BIT '
                                                                                                'STRING'},
                                                                                       {'name': 'n4TxAntenna-tm5',
                                                                                        'size': [16],
                                                                                        'type': 'BIT '
                                                                                                'STRING'},
                                                                                       {'name': 'n2TxAntenna-tm6',
                                                                                        'size': [4],
                                                                                        'type': 'BIT '
                                                                                                'STRING'},
                                                                                       {'name': 'n4TxAntenna-tm6',
                                                                                        'size': [16],
                                                                                        'type': 'BIT '
                                                                                                'STRING'}],
                                                                           'name': 'codebookSubsetRestriction',
                                                                           'optional': True,
                                                                           'type': 'CHOICE'},
                                                                          {'members': [{'name': 'release',
                                                                                        'type': 'NULL'},
                                                                                       {'name': 'setup',
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('closedLoop',
                                                                                                    0),
                                                                                                   ('openLoop',
                                                                                                    1)]}],
                                                                           'name': 'ue-TransmitAntennaSelection',
                                                                           'type': 'CHOICE'}],
                                                              'type': 'SEQUENCE'},
                                     'AntennaInfoDedicated-r10': {'members': [{'name': 'transmissionMode-r10',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('tm1',
                                                                                           0),
                                                                                          ('tm2',
                                                                                           1),
                                                                                          ('tm3',
                                                                                           2),
                                                                                          ('tm4',
                                                                                           3),
                                                                                          ('tm5',
                                                                                           4),
                                                                                          ('tm6',
                                                                                           5),
                                                                                          ('tm7',
                                                                                           6),
                                                                                          ('tm8-v920',
                                                                                           7),
                                                                                          ('tm9-v1020',
                                                                                           8),
                                                                                          ('tm10-v1130',
                                                                                           9),
                                                                                          ('spare6',
                                                                                           10),
                                                                                          ('spare5',
                                                                                           11),
                                                                                          ('spare4',
                                                                                           12),
                                                                                          ('spare3',
                                                                                           13),
                                                                                          ('spare2',
                                                                                           14),
                                                                                          ('spare1',
                                                                                           15)]},
                                                                              {'name': 'codebookSubsetRestriction-r10',
                                                                               'optional': True,
                                                                               'type': 'BIT '
                                                                                       'STRING'},
                                                                              {'members': [{'name': 'release',
                                                                                            'type': 'NULL'},
                                                                                           {'name': 'setup',
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('closedLoop',
                                                                                                        0),
                                                                                                       ('openLoop',
                                                                                                        1)]}],
                                                                               'name': 'ue-TransmitAntennaSelection',
                                                                               'type': 'CHOICE'}],
                                                                  'type': 'SEQUENCE'},
                                     'AntennaInfoDedicated-v10i0': {'members': [{'name': 'maxLayersMIMO-r10',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('twoLayers',
                                                                                             0),
                                                                                            ('fourLayers',
                                                                                             1),
                                                                                            ('eightLayers',
                                                                                             2)]}],
                                                                    'type': 'SEQUENCE'},
                                     'AntennaInfoDedicated-v1250': {'members': [{'name': 'alternativeCodebookEnabledFor4TX-r12',
                                                                                 'type': 'BOOLEAN'}],
                                                                    'type': 'SEQUENCE'},
                                     'AntennaInfoDedicated-v1430': {'members': [{'name': 'ce-UE-TxAntennaSelection-config-r14',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('on',
                                                                                             0)]}],
                                                                    'type': 'SEQUENCE'},
                                     'AntennaInfoDedicated-v920': {'members': [{'members': [{'name': 'n2TxAntenna-tm8-r9',
                                                                                             'size': [6],
                                                                                             'type': 'BIT '
                                                                                                     'STRING'},
                                                                                            {'name': 'n4TxAntenna-tm8-r9',
                                                                                             'size': [32],
                                                                                             'type': 'BIT '
                                                                                                     'STRING'}],
                                                                                'name': 'codebookSubsetRestriction-v920',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'}],
                                                                   'type': 'SEQUENCE'},
                                     'AntennaInfoUL-r10': {'members': [{'name': 'transmissionModeUL-r10',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('tm1',
                                                                                    0),
                                                                                   ('tm2',
                                                                                    1),
                                                                                   ('spare6',
                                                                                    2),
                                                                                   ('spare5',
                                                                                    3),
                                                                                   ('spare4',
                                                                                    4),
                                                                                   ('spare3',
                                                                                    5),
                                                                                   ('spare2',
                                                                                    6),
                                                                                   ('spare1',
                                                                                    7)]},
                                                                       {'name': 'fourAntennaPortActivated-r10',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('setup',
                                                                                    0)]}],
                                                           'type': 'SEQUENCE'},
                                     'AreaConfiguration-r10': {'members': [{'name': 'cellGlobalIdList-r10',
                                                                            'type': 'CellGlobalIdList-r10'},
                                                                           {'name': 'trackingAreaCodeList-r10',
                                                                            'type': 'TrackingAreaCodeList-r10'}],
                                                               'type': 'CHOICE'},
                                     'AreaConfiguration-v1130': {'members': [{'name': 'trackingAreaCodeList-v1130',
                                                                              'type': 'TrackingAreaCodeList-v1130'}],
                                                                 'type': 'SEQUENCE'},
                                     'BCCH-BCH-Message': {'members': [{'name': 'message',
                                                                       'type': 'BCCH-BCH-MessageType'}],
                                                          'type': 'SEQUENCE'},
                                     'BCCH-BCH-Message-MBMS': {'members': [{'name': 'message',
                                                                            'type': 'BCCH-BCH-MessageType-MBMS-r14'}],
                                                               'type': 'SEQUENCE'},
                                     'BCCH-BCH-MessageType': {'type': 'MasterInformationBlock'},
                                     'BCCH-BCH-MessageType-MBMS-r14': {'type': 'MasterInformationBlock-MBMS-r14'},
                                     'BCCH-Config': {'members': [{'name': 'modificationPeriodCoeff',
                                                                  'type': 'ENUMERATED',
                                                                  'values': [('n2',
                                                                              0),
                                                                             ('n4',
                                                                              1),
                                                                             ('n8',
                                                                              2),
                                                                             ('n16',
                                                                              3)]}],
                                                     'type': 'SEQUENCE'},
                                     'BCCH-Config-v1310': {'members': [{'name': 'modificationPeriodCoeff-v1310',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('n64',
                                                                                    0)]}],
                                                           'type': 'SEQUENCE'},
                                     'BCCH-DL-SCH-Message': {'members': [{'name': 'message',
                                                                          'type': 'BCCH-DL-SCH-MessageType'}],
                                                             'type': 'SEQUENCE'},
                                     'BCCH-DL-SCH-Message-BR': {'members': [{'name': 'message',
                                                                             'type': 'BCCH-DL-SCH-MessageType-BR-r13'}],
                                                                'type': 'SEQUENCE'},
                                     'BCCH-DL-SCH-Message-MBMS': {'members': [{'name': 'message',
                                                                               'type': 'BCCH-DL-SCH-MessageType-MBMS-r14'}],
                                                                  'type': 'SEQUENCE'},
                                     'BCCH-DL-SCH-MessageType': {'members': [{'members': [{'name': 'systemInformation',
                                                                                           'type': 'SystemInformation'},
                                                                                          {'name': 'systemInformationBlockType1',
                                                                                           'type': 'SystemInformationBlockType1'}],
                                                                              'name': 'c1',
                                                                              'type': 'CHOICE'},
                                                                             {'members': [],
                                                                              'name': 'messageClassExtension',
                                                                              'type': 'SEQUENCE'}],
                                                                 'type': 'CHOICE'},
                                     'BCCH-DL-SCH-MessageType-BR-r13': {'members': [{'members': [{'name': 'systemInformation-BR-r13',
                                                                                                  'type': 'SystemInformation-BR-r13'},
                                                                                                 {'name': 'systemInformationBlockType1-BR-r13',
                                                                                                  'type': 'SystemInformationBlockType1-BR-r13'}],
                                                                                     'name': 'c1',
                                                                                     'type': 'CHOICE'},
                                                                                    {'members': [],
                                                                                     'name': 'messageClassExtension',
                                                                                     'type': 'SEQUENCE'}],
                                                                        'type': 'CHOICE'},
                                     'BCCH-DL-SCH-MessageType-MBMS-r14': {'members': [{'members': [{'name': 'systemInformation-MBMS-r14',
                                                                                                    'type': 'SystemInformation-MBMS-r14'},
                                                                                                   {'name': 'systemInformationBlockType1-MBMS-r14',
                                                                                                    'type': 'SystemInformationBlockType1-MBMS-r14'}],
                                                                                       'name': 'c1',
                                                                                       'type': 'CHOICE'},
                                                                                      {'members': [],
                                                                                       'name': 'messageClassExtension',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'type': 'CHOICE'},
                                     'BLER-Range-r12': {'restricted-to': [(0,
                                                                           31)],
                                                        'type': 'INTEGER'},
                                     'BLER-Result-r12': {'members': [{'name': 'bler-r12',
                                                                      'type': 'BLER-Range-r12'},
                                                                     {'members': [{'name': 'n-r12',
                                                                                   'size': [3],
                                                                                   'type': 'BIT '
                                                                                           'STRING'},
                                                                                  {'name': 'm-r12',
                                                                                   'size': [8],
                                                                                   'type': 'BIT '
                                                                                           'STRING'}],
                                                                      'name': 'blocksReceived-r12',
                                                                      'type': 'SEQUENCE'}],
                                                         'type': 'SEQUENCE'},
                                     'BW-Preference-r14': {'members': [{'name': 'dl-Preference-r14',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('mhz1dot4',
                                                                                    0),
                                                                                   ('mhz5',
                                                                                    1),
                                                                                   ('mhz20',
                                                                                    2)]},
                                                                       {'name': 'ul-Preference-r14',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('mhz1dot4',
                                                                                    0),
                                                                                   ('mhz5',
                                                                                    1)]}],
                                                           'type': 'SEQUENCE'},
                                     'BandClassInfoCDMA2000': {'members': [{'name': 'bandClass',
                                                                            'type': 'BandclassCDMA2000'},
                                                                           {'name': 'cellReselectionPriority',
                                                                            'optional': True,
                                                                            'type': 'CellReselectionPriority'},
                                                                           {'name': 'threshX-High',
                                                                            'restricted-to': [(0,
                                                                                               63)],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'threshX-Low',
                                                                            'restricted-to': [(0,
                                                                                               63)],
                                                                            'type': 'INTEGER'},
                                                                           None],
                                                               'type': 'SEQUENCE'},
                                     'BandClassListCDMA2000': {'element': {'type': 'BandClassInfoCDMA2000'},
                                                               'size': [(1,
                                                                         'maxCDMA-BandClass')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'BandClassPriority1XRTT': {'members': [{'name': 'bandClass',
                                                                             'type': 'BandclassCDMA2000'},
                                                                            {'name': 'cellReselectionPriority',
                                                                             'type': 'CellReselectionPriority'}],
                                                                'type': 'SEQUENCE'},
                                     'BandClassPriorityHRPD': {'members': [{'name': 'bandClass',
                                                                            'type': 'BandclassCDMA2000'},
                                                                           {'name': 'cellReselectionPriority',
                                                                            'type': 'CellReselectionPriority'}],
                                                               'type': 'SEQUENCE'},
                                     'BandClassPriorityList1XRTT': {'element': {'type': 'BandClassPriority1XRTT'},
                                                                    'size': [(1,
                                                                              'maxCDMA-BandClass')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'BandClassPriorityListHRPD': {'element': {'type': 'BandClassPriorityHRPD'},
                                                                   'size': [(1,
                                                                             'maxCDMA-BandClass')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'BandCombination-r14': {'element': {'type': 'BandIndication-r14'},
                                                             'size': [(1,
                                                                       'maxSimultaneousBands-r10')],
                                                             'type': 'SEQUENCE '
                                                                     'OF'},
                                     'BandCombinationList-r14': {'element': {'type': 'BandCombination-r14'},
                                                                 'size': [(1,
                                                                           'maxBandComb-r13')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'BandCombinationListEUTRA-r10': {'element': {'type': 'BandInfoEUTRA'},
                                                                      'size': [(1,
                                                                                'maxBandComb-r10')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'BandCombinationParameters-r10': {'element': {'type': 'BandParameters-r10'},
                                                                       'size': [(1,
                                                                                 'maxSimultaneousBands-r10')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'BandCombinationParameters-r11': {'members': [{'element': {'type': 'BandParameters-r11'},
                                                                                    'name': 'bandParameterList-r11',
                                                                                    'size': [(1,
                                                                                              'maxSimultaneousBands-r10')],
                                                                                    'type': 'SEQUENCE '
                                                                                            'OF'},
                                                                                   {'name': 'supportedBandwidthCombinationSet-r11',
                                                                                    'optional': True,
                                                                                    'type': 'SupportedBandwidthCombinationSet-r10'},
                                                                                   {'name': 'multipleTimingAdvance-r11',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'simultaneousRx-Tx-r11',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'bandInfoEUTRA-r11',
                                                                                    'type': 'BandInfoEUTRA'},
                                                                                   None],
                                                                       'type': 'SEQUENCE'},
                                     'BandCombinationParameters-r13': {'members': [{'name': 'differentFallbackSupported-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]},
                                                                                   {'element': {'type': 'BandParameters-r13'},
                                                                                    'name': 'bandParameterList-r13',
                                                                                    'size': [(1,
                                                                                              'maxSimultaneousBands-r10')],
                                                                                    'type': 'SEQUENCE '
                                                                                            'OF'},
                                                                                   {'name': 'supportedBandwidthCombinationSet-r13',
                                                                                    'optional': True,
                                                                                    'type': 'SupportedBandwidthCombinationSet-r10'},
                                                                                   {'name': 'multipleTimingAdvance-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'simultaneousRx-Tx-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'bandInfoEUTRA-r13',
                                                                                    'type': 'BandInfoEUTRA'},
                                                                                   {'members': [{'name': 'asynchronous-r13',
                                                                                                 'optional': True,
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('supported',
                                                                                                             0)]},
                                                                                                {'members': [{'name': 'threeEntries-r13',
                                                                                                              'size': [3],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'},
                                                                                                             {'name': 'fourEntries-r13',
                                                                                                              'size': [7],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'},
                                                                                                             {'name': 'fiveEntries-r13',
                                                                                                              'size': [15],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'}],
                                                                                                 'name': 'supportedCellGrouping-r13',
                                                                                                 'optional': True,
                                                                                                 'type': 'CHOICE'}],
                                                                                    'name': 'dc-Support-r13',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'},
                                                                                   {'name': 'supportedNAICS-2CRS-AP-r13',
                                                                                    'optional': True,
                                                                                    'size': [(1,
                                                                                              'maxNAICS-Entries-r12')],
                                                                                    'type': 'BIT '
                                                                                            'STRING'},
                                                                                   {'name': 'commSupportedBandsPerBC-r13',
                                                                                    'optional': True,
                                                                                    'size': [(1,
                                                                                              'maxBands')],
                                                                                    'type': 'BIT '
                                                                                            'STRING'}],
                                                                       'type': 'SEQUENCE'},
                                     'BandCombinationParameters-v1090': {'element': {'type': 'BandParameters-v1090'},
                                                                         'size': [(1,
                                                                                   'maxSimultaneousBands-r10')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'BandCombinationParameters-v10i0': {'members': [{'element': {'type': 'BandParameters-v10i0'},
                                                                                      'name': 'bandParameterList-v10i0',
                                                                                      'optional': True,
                                                                                      'size': [(1,
                                                                                                'maxSimultaneousBands-r10')],
                                                                                      'type': 'SEQUENCE '
                                                                                              'OF'}],
                                                                         'type': 'SEQUENCE'},
                                     'BandCombinationParameters-v1130': {'members': [{'name': 'multipleTimingAdvance-r11',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('supported',
                                                                                                  0)]},
                                                                                     {'name': 'simultaneousRx-Tx-r11',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('supported',
                                                                                                  0)]},
                                                                                     {'element': {'type': 'BandParameters-v1130'},
                                                                                      'name': 'bandParameterList-r11',
                                                                                      'optional': True,
                                                                                      'size': [(1,
                                                                                                'maxSimultaneousBands-r10')],
                                                                                      'type': 'SEQUENCE '
                                                                                              'OF'},
                                                                                     None],
                                                                         'type': 'SEQUENCE'},
                                     'BandCombinationParameters-v1250': {'members': [{'members': [{'name': 'asynchronous-r12',
                                                                                                   'optional': True,
                                                                                                   'type': 'ENUMERATED',
                                                                                                   'values': [('supported',
                                                                                                               0)]},
                                                                                                  {'members': [{'name': 'threeEntries-r12',
                                                                                                                'size': [3],
                                                                                                                'type': 'BIT '
                                                                                                                        'STRING'},
                                                                                                               {'name': 'fourEntries-r12',
                                                                                                                'size': [7],
                                                                                                                'type': 'BIT '
                                                                                                                        'STRING'},
                                                                                                               {'name': 'fiveEntries-r12',
                                                                                                                'size': [15],
                                                                                                                'type': 'BIT '
                                                                                                                        'STRING'}],
                                                                                                   'name': 'supportedCellGrouping-r12',
                                                                                                   'optional': True,
                                                                                                   'type': 'CHOICE'}],
                                                                                      'name': 'dc-Support-r12',
                                                                                      'optional': True,
                                                                                      'type': 'SEQUENCE'},
                                                                                     {'name': 'supportedNAICS-2CRS-AP-r12',
                                                                                      'optional': True,
                                                                                      'size': [(1,
                                                                                                'maxNAICS-Entries-r12')],
                                                                                      'type': 'BIT '
                                                                                              'STRING'},
                                                                                     {'name': 'commSupportedBandsPerBC-r12',
                                                                                      'optional': True,
                                                                                      'size': [(1,
                                                                                                'maxBands')],
                                                                                      'type': 'BIT '
                                                                                              'STRING'},
                                                                                     None],
                                                                         'type': 'SEQUENCE'},
                                     'BandCombinationParameters-v1270': {'members': [{'element': {'type': 'BandParameters-v1270'},
                                                                                      'name': 'bandParameterList-v1270',
                                                                                      'optional': True,
                                                                                      'size': [(1,
                                                                                                'maxSimultaneousBands-r10')],
                                                                                      'type': 'SEQUENCE '
                                                                                              'OF'}],
                                                                         'type': 'SEQUENCE'},
                                     'BandCombinationParameters-v1320': {'members': [{'element': {'type': 'BandParameters-v1320'},
                                                                                      'name': 'bandParameterList-v1320',
                                                                                      'optional': True,
                                                                                      'size': [(1,
                                                                                                'maxSimultaneousBands-r10')],
                                                                                      'type': 'SEQUENCE '
                                                                                              'OF'},
                                                                                     {'name': 'additionalRx-Tx-PerformanceReq-r13',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('supported',
                                                                                                  0)]}],
                                                                         'type': 'SEQUENCE'},
                                     'BandCombinationParameters-v1430': {'members': [{'element': {'type': 'BandParameters-v1430'},
                                                                                      'name': 'bandParameterList-v1430',
                                                                                      'optional': True,
                                                                                      'size': [(1,
                                                                                                'maxSimultaneousBands-r10')],
                                                                                      'type': 'SEQUENCE '
                                                                                              'OF'},
                                                                                     {'name': 'v2x-SupportedTxBandCombListPerBC-r14',
                                                                                      'optional': True,
                                                                                      'size': [(1,
                                                                                                'maxBandComb-r13')],
                                                                                      'type': 'BIT '
                                                                                              'STRING'},
                                                                                     {'name': 'v2x-SupportedRxBandCombListPerBC-r14',
                                                                                      'optional': True,
                                                                                      'size': [(1,
                                                                                                'maxBandComb-r13')],
                                                                                      'type': 'BIT '
                                                                                              'STRING'}],
                                                                         'type': 'SEQUENCE'},
                                     'BandCombinationParametersExt-r10': {'members': [{'name': 'supportedBandwidthCombinationSet-r10',
                                                                                       'optional': True,
                                                                                       'type': 'SupportedBandwidthCombinationSet-r10'}],
                                                                          'type': 'SEQUENCE'},
                                     'BandIndication-r14': {'members': [{'name': 'bandEUTRA-r14',
                                                                         'type': 'FreqBandIndicator-r11'},
                                                                        {'name': 'ca-BandwidthClassDL-r14',
                                                                         'type': 'CA-BandwidthClass-r10'},
                                                                        {'name': 'ca-BandwidthClassUL-r14',
                                                                         'optional': True,
                                                                         'type': 'CA-BandwidthClass-r10'}],
                                                            'type': 'SEQUENCE'},
                                     'BandIndicatorGERAN': {'type': 'ENUMERATED',
                                                            'values': [('dcs1800',
                                                                        0),
                                                                       ('pcs1900',
                                                                        1)]},
                                     'BandInfoEUTRA': {'members': [{'name': 'interFreqBandList',
                                                                    'type': 'InterFreqBandList'},
                                                                   {'name': 'interRAT-BandList',
                                                                    'optional': True,
                                                                    'type': 'InterRAT-BandList'}],
                                                       'type': 'SEQUENCE'},
                                     'BandListEUTRA': {'element': {'type': 'BandInfoEUTRA'},
                                                       'size': [(1,
                                                                 'maxBands')],
                                                       'type': 'SEQUENCE OF'},
                                     'BandParameters-r10': {'members': [{'name': 'bandEUTRA-r10',
                                                                         'type': 'FreqBandIndicator'},
                                                                        {'name': 'bandParametersUL-r10',
                                                                         'optional': True,
                                                                         'type': 'BandParametersUL-r10'},
                                                                        {'name': 'bandParametersDL-r10',
                                                                         'optional': True,
                                                                         'type': 'BandParametersDL-r10'}],
                                                            'type': 'SEQUENCE'},
                                     'BandParameters-r11': {'members': [{'name': 'bandEUTRA-r11',
                                                                         'type': 'FreqBandIndicator-r11'},
                                                                        {'name': 'bandParametersUL-r11',
                                                                         'optional': True,
                                                                         'type': 'BandParametersUL-r10'},
                                                                        {'name': 'bandParametersDL-r11',
                                                                         'optional': True,
                                                                         'type': 'BandParametersDL-r10'},
                                                                        {'name': 'supportedCSI-Proc-r11',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('n1',
                                                                                     0),
                                                                                    ('n3',
                                                                                     1),
                                                                                    ('n4',
                                                                                     2)]}],
                                                            'type': 'SEQUENCE'},
                                     'BandParameters-r13': {'members': [{'name': 'bandEUTRA-r13',
                                                                         'type': 'FreqBandIndicator-r11'},
                                                                        {'name': 'bandParametersUL-r13',
                                                                         'optional': True,
                                                                         'type': 'BandParametersUL-r13'},
                                                                        {'name': 'bandParametersDL-r13',
                                                                         'optional': True,
                                                                         'type': 'BandParametersDL-r13'},
                                                                        {'name': 'supportedCSI-Proc-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('n1',
                                                                                     0),
                                                                                    ('n3',
                                                                                     1),
                                                                                    ('n4',
                                                                                     2)]}],
                                                            'type': 'SEQUENCE'},
                                     'BandParameters-v1090': {'members': [{'name': 'bandEUTRA-v1090',
                                                                           'optional': True,
                                                                           'type': 'FreqBandIndicator-v9e0'},
                                                                          None],
                                                              'type': 'SEQUENCE'},
                                     'BandParameters-v10i0': {'members': [{'element': {'type': 'CA-MIMO-ParametersDL-v10i0'},
                                                                           'name': 'bandParametersDL-v10i0',
                                                                           'size': [(1,
                                                                                     'maxBandwidthClass-r10')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'}],
                                                              'type': 'SEQUENCE'},
                                     'BandParameters-v1130': {'members': [{'name': 'supportedCSI-Proc-r11',
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('n1',
                                                                                       0),
                                                                                      ('n3',
                                                                                       1),
                                                                                      ('n4',
                                                                                       2)]}],
                                                              'type': 'SEQUENCE'},
                                     'BandParameters-v1270': {'members': [{'element': {'type': 'CA-MIMO-ParametersDL-v1270'},
                                                                           'name': 'bandParametersDL-v1270',
                                                                           'size': [(1,
                                                                                     'maxBandwidthClass-r10')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'}],
                                                              'type': 'SEQUENCE'},
                                     'BandParameters-v1320': {'members': [{'name': 'bandParametersDL-v1320',
                                                                           'type': 'MIMO-CA-ParametersPerBoBC-r13'}],
                                                              'type': 'SEQUENCE'},
                                     'BandParameters-v1430': {'members': [{'name': 'bandParametersDL-v1430',
                                                                           'optional': True,
                                                                           'type': 'MIMO-CA-ParametersPerBoBC-v1430'},
                                                                          {'name': 'ul-256QAM-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'element': {'type': 'UL-256QAM-perCC-Info-r14'},
                                                                           'name': 'ul-256QAM-perCC-InfoList-r14',
                                                                           'optional': True,
                                                                           'size': [(2,
                                                                                     'maxServCell-r13')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                                                          {'element': {'type': 'RetuningTimeInfo-r14'},
                                                                           'name': 'retuningTimeInfoBandList-r14',
                                                                           'optional': True,
                                                                           'size': [(1,
                                                                                     'maxSimultaneousBands-r10')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'}],
                                                              'type': 'SEQUENCE'},
                                     'BandParametersDL-r10': {'element': {'type': 'CA-MIMO-ParametersDL-r10'},
                                                              'size': [(1,
                                                                        'maxBandwidthClass-r10')],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'BandParametersDL-r13': {'type': 'CA-MIMO-ParametersDL-r13'},
                                     'BandParametersRxSL-r14': {'members': [{'name': 'v2x-BandwidthClassRxSL-r14',
                                                                             'type': 'V2X-BandwidthClassSL-r14'},
                                                                            {'name': 'v2x-HighReception-r14',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]}],
                                                                'type': 'SEQUENCE'},
                                     'BandParametersTxSL-r14': {'members': [{'name': 'v2x-BandwidthClassTxSL-r14',
                                                                             'type': 'V2X-BandwidthClassSL-r14'},
                                                                            {'name': 'v2x-eNB-Scheduled-r14',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]},
                                                                            {'name': 'v2x-HighPower-r14',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]}],
                                                                'type': 'SEQUENCE'},
                                     'BandParametersUL-r10': {'element': {'type': 'CA-MIMO-ParametersUL-r10'},
                                                              'size': [(1,
                                                                        'maxBandwidthClass-r10')],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'BandParametersUL-r13': {'type': 'CA-MIMO-ParametersUL-r10'},
                                     'BandclassCDMA2000': {'type': 'ENUMERATED',
                                                           'values': [('bc0',
                                                                       0),
                                                                      ('bc1',
                                                                       1),
                                                                      ('bc2',
                                                                       2),
                                                                      ('bc3',
                                                                       3),
                                                                      ('bc4',
                                                                       4),
                                                                      ('bc5',
                                                                       5),
                                                                      ('bc6',
                                                                       6),
                                                                      ('bc7',
                                                                       7),
                                                                      ('bc8',
                                                                       8),
                                                                      ('bc9',
                                                                       9),
                                                                      ('bc10',
                                                                       10),
                                                                      ('bc11',
                                                                       11),
                                                                      ('bc12',
                                                                       12),
                                                                      ('bc13',
                                                                       13),
                                                                      ('bc14',
                                                                       14),
                                                                      ('bc15',
                                                                       15),
                                                                      ('bc16',
                                                                       16),
                                                                      ('bc17',
                                                                       17),
                                                                      ('bc18-v9a0',
                                                                       18),
                                                                      ('bc19-v9a0',
                                                                       19),
                                                                      ('bc20-v9a0',
                                                                       20),
                                                                      ('bc21-v9a0',
                                                                       21),
                                                                      ('spare10',
                                                                       22),
                                                                      ('spare9',
                                                                       23),
                                                                      ('spare8',
                                                                       24),
                                                                      ('spare7',
                                                                       25),
                                                                      ('spare6',
                                                                       26),
                                                                      ('spare5',
                                                                       27),
                                                                      ('spare4',
                                                                       28),
                                                                      ('spare3',
                                                                       29),
                                                                      ('spare2',
                                                                       30),
                                                                      ('spare1',
                                                                       31),
                                                                      None]},
                                     'BarringPerACDC-Category-r13': {'members': [{'name': 'acdc-Category-r13',
                                                                                  'restricted-to': [(1,
                                                                                                     'maxACDC-Cat-r13')],
                                                                                  'type': 'INTEGER'},
                                                                                 {'members': [{'name': 'ac-BarringFactor-r13',
                                                                                               'type': 'ENUMERATED',
                                                                                               'values': [('p00',
                                                                                                           0),
                                                                                                          ('p05',
                                                                                                           1),
                                                                                                          ('p10',
                                                                                                           2),
                                                                                                          ('p15',
                                                                                                           3),
                                                                                                          ('p20',
                                                                                                           4),
                                                                                                          ('p25',
                                                                                                           5),
                                                                                                          ('p30',
                                                                                                           6),
                                                                                                          ('p40',
                                                                                                           7),
                                                                                                          ('p50',
                                                                                                           8),
                                                                                                          ('p60',
                                                                                                           9),
                                                                                                          ('p70',
                                                                                                           10),
                                                                                                          ('p75',
                                                                                                           11),
                                                                                                          ('p80',
                                                                                                           12),
                                                                                                          ('p85',
                                                                                                           13),
                                                                                                          ('p90',
                                                                                                           14),
                                                                                                          ('p95',
                                                                                                           15)]},
                                                                                              {'name': 'ac-BarringTime-r13',
                                                                                               'type': 'ENUMERATED',
                                                                                               'values': [('s4',
                                                                                                           0),
                                                                                                          ('s8',
                                                                                                           1),
                                                                                                          ('s16',
                                                                                                           2),
                                                                                                          ('s32',
                                                                                                           3),
                                                                                                          ('s64',
                                                                                                           4),
                                                                                                          ('s128',
                                                                                                           5),
                                                                                                          ('s256',
                                                                                                           6),
                                                                                                          ('s512',
                                                                                                           7)]}],
                                                                                  'name': 'acdc-BarringConfig-r13',
                                                                                  'optional': True,
                                                                                  'type': 'SEQUENCE'}],
                                                                     'type': 'SEQUENCE'},
                                     'BarringPerACDC-CategoryList-r13': {'element': {'type': 'BarringPerACDC-Category-r13'},
                                                                         'size': [(1,
                                                                                   'maxACDC-Cat-r13')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'BlackCellsToAddMod': {'members': [{'name': 'cellIndex',
                                                                         'restricted-to': [(1,
                                                                                            'maxCellMeas')],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'physCellIdRange',
                                                                         'type': 'PhysCellIdRange'}],
                                                            'type': 'SEQUENCE'},
                                     'BlackCellsToAddModList': {'element': {'type': 'BlackCellsToAddMod'},
                                                                'size': [(1,
                                                                          'maxCellMeas')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'C-RNTI': {'size': [16],
                                                'type': 'BIT STRING'},
                                     'CA-BandwidthClass-r10': {'type': 'ENUMERATED',
                                                               'values': [('a',
                                                                           0),
                                                                          ('b',
                                                                           1),
                                                                          ('c',
                                                                           2),
                                                                          ('d',
                                                                           3),
                                                                          ('e',
                                                                           4),
                                                                          ('f',
                                                                           5),
                                                                          None]},
                                     'CA-MIMO-ParametersDL-r10': {'members': [{'name': 'ca-BandwidthClassDL-r10',
                                                                               'type': 'CA-BandwidthClass-r10'},
                                                                              {'name': 'supportedMIMO-CapabilityDL-r10',
                                                                               'optional': True,
                                                                               'type': 'MIMO-CapabilityDL-r10'}],
                                                                  'type': 'SEQUENCE'},
                                     'CA-MIMO-ParametersDL-r13': {'members': [{'name': 'ca-BandwidthClassDL-r13',
                                                                               'type': 'CA-BandwidthClass-r10'},
                                                                              {'name': 'supportedMIMO-CapabilityDL-r13',
                                                                               'optional': True,
                                                                               'type': 'MIMO-CapabilityDL-r10'},
                                                                              {'name': 'fourLayerTM3-TM4-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'element': {'type': 'IntraBandContiguousCC-Info-r12'},
                                                                               'name': 'intraBandContiguousCC-InfoList-r13',
                                                                               'size': [(1,
                                                                                         'maxServCell-r13')],
                                                                               'type': 'SEQUENCE '
                                                                                       'OF'}],
                                                                  'type': 'SEQUENCE'},
                                     'CA-MIMO-ParametersDL-v10i0': {'members': [{'name': 'fourLayerTM3-TM4-r10',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('supported',
                                                                                             0)]}],
                                                                    'type': 'SEQUENCE'},
                                     'CA-MIMO-ParametersDL-v1270': {'members': [{'element': {'type': 'IntraBandContiguousCC-Info-r12'},
                                                                                 'name': 'intraBandContiguousCC-InfoList-r12',
                                                                                 'size': [(1,
                                                                                           'maxServCell-r10')],
                                                                                 'type': 'SEQUENCE '
                                                                                         'OF'}],
                                                                    'type': 'SEQUENCE'},
                                     'CA-MIMO-ParametersUL-r10': {'members': [{'name': 'ca-BandwidthClassUL-r10',
                                                                               'type': 'CA-BandwidthClass-r10'},
                                                                              {'name': 'supportedMIMO-CapabilityUL-r10',
                                                                               'optional': True,
                                                                               'type': 'MIMO-CapabilityUL-r10'}],
                                                                  'type': 'SEQUENCE'},
                                     'CDMA2000-Type': {'type': 'ENUMERATED',
                                                       'values': [('type1XRTT',
                                                                   0),
                                                                  ('typeHRPD',
                                                                   1)]},
                                     'CE-Parameters-r13': {'members': [{'name': 'ce-ModeA-r13',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('supported',
                                                                                    0)]},
                                                                       {'name': 'ce-ModeB-r13',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('supported',
                                                                                    0)]}],
                                                           'type': 'SEQUENCE'},
                                     'CE-Parameters-v1320': {'members': [{'name': 'intraFreqA3-CE-ModeA-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'intraFreqA3-CE-ModeB-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'intraFreqHO-CE-ModeA-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'intraFreqHO-CE-ModeB-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'CE-Parameters-v1350': {'members': [{'name': 'unicastFrequencyHopping-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'CE-Parameters-v1370': {'members': [{'name': 'tm9-CE-ModeA-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'tm9-CE-ModeB-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'CE-Parameters-v1430': {'members': [{'name': 'ce-SwitchWithoutHO-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'CIOT-EPS-OptimisationInfo-r13': {'element': {'type': 'CIOT-OptimisationPLMN-r13'},
                                                                       'size': [(1,
                                                                                 'maxPLMN-r11')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'CIOT-OptimisationPLMN-r13': {'members': [{'name': 'up-CIoT-EPS-Optimisation-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('true',
                                                                                            0)]},
                                                                               {'name': 'cp-CIoT-EPS-Optimisation-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('true',
                                                                                            0)]},
                                                                               {'name': 'attachWithoutPDN-Connectivity-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('true',
                                                                                            0)]}],
                                                                   'type': 'SEQUENCE'},
                                     'CQI-ReportAperiodic-r10': {'members': [{'name': 'release',
                                                                              'type': 'NULL'},
                                                                             {'members': [{'name': 'cqi-ReportModeAperiodic-r10',
                                                                                           'type': 'CQI-ReportModeAperiodic'},
                                                                                          {'members': [{'name': 'trigger1-r10',
                                                                                                        'size': [8],
                                                                                                        'type': 'BIT '
                                                                                                                'STRING'},
                                                                                                       {'name': 'trigger2-r10',
                                                                                                        'size': [8],
                                                                                                        'type': 'BIT '
                                                                                                                'STRING'}],
                                                                                           'name': 'aperiodicCSI-Trigger-r10',
                                                                                           'optional': True,
                                                                                           'type': 'SEQUENCE'}],
                                                                              'name': 'setup',
                                                                              'type': 'SEQUENCE'}],
                                                                 'type': 'CHOICE'},
                                     'CQI-ReportAperiodic-v1250': {'members': [{'name': 'release',
                                                                                'type': 'NULL'},
                                                                               {'members': [{'members': [{'name': 'trigger-SubframeSetIndicator-r12',
                                                                                                          'type': 'ENUMERATED',
                                                                                                          'values': [('s1',
                                                                                                                      0),
                                                                                                                     ('s2',
                                                                                                                      1)]},
                                                                                                         {'name': 'trigger1-SubframeSetIndicator-r12',
                                                                                                          'size': [8],
                                                                                                          'type': 'BIT '
                                                                                                                  'STRING'},
                                                                                                         {'name': 'trigger2-SubframeSetIndicator-r12',
                                                                                                          'size': [8],
                                                                                                          'type': 'BIT '
                                                                                                                  'STRING'}],
                                                                                             'name': 'aperiodicCSI-Trigger-v1250',
                                                                                             'type': 'SEQUENCE'}],
                                                                                'name': 'setup',
                                                                                'type': 'SEQUENCE'}],
                                                                   'type': 'CHOICE'},
                                     'CQI-ReportAperiodic-v1310': {'members': [{'name': 'release',
                                                                                'type': 'NULL'},
                                                                               {'members': [{'members': [{'name': 'trigger1-r13',
                                                                                                          'size': [32],
                                                                                                          'type': 'BIT '
                                                                                                                  'STRING'},
                                                                                                         {'name': 'trigger2-r13',
                                                                                                          'size': [32],
                                                                                                          'type': 'BIT '
                                                                                                                  'STRING'},
                                                                                                         {'name': 'trigger3-r13',
                                                                                                          'size': [32],
                                                                                                          'type': 'BIT '
                                                                                                                  'STRING'},
                                                                                                         {'name': 'trigger4-r13',
                                                                                                          'size': [32],
                                                                                                          'type': 'BIT '
                                                                                                                  'STRING'},
                                                                                                         {'name': 'trigger5-r13',
                                                                                                          'size': [32],
                                                                                                          'type': 'BIT '
                                                                                                                  'STRING'},
                                                                                                         {'name': 'trigger6-r13',
                                                                                                          'size': [32],
                                                                                                          'type': 'BIT '
                                                                                                                  'STRING'}],
                                                                                             'name': 'aperiodicCSI-Trigger-v1310',
                                                                                             'optional': True,
                                                                                             'type': 'SEQUENCE'},
                                                                                            {'members': [{'name': 'release',
                                                                                                          'type': 'NULL'},
                                                                                                         {'members': [{'name': 'trigger1-SubframeSetIndicator-r13',
                                                                                                                       'size': [32],
                                                                                                                       'type': 'BIT '
                                                                                                                               'STRING'},
                                                                                                                      {'name': 'trigger2-SubframeSetIndicator-r13',
                                                                                                                       'size': [32],
                                                                                                                       'type': 'BIT '
                                                                                                                               'STRING'},
                                                                                                                      {'name': 'trigger3-SubframeSetIndicator-r13',
                                                                                                                       'size': [32],
                                                                                                                       'type': 'BIT '
                                                                                                                               'STRING'},
                                                                                                                      {'name': 'trigger4-SubframeSetIndicator-r13',
                                                                                                                       'size': [32],
                                                                                                                       'type': 'BIT '
                                                                                                                               'STRING'},
                                                                                                                      {'name': 'trigger5-SubframeSetIndicator-r13',
                                                                                                                       'size': [32],
                                                                                                                       'type': 'BIT '
                                                                                                                               'STRING'},
                                                                                                                      {'name': 'trigger6-SubframeSetIndicator-r13',
                                                                                                                       'size': [32],
                                                                                                                       'type': 'BIT '
                                                                                                                               'STRING'}],
                                                                                                          'name': 'setup',
                                                                                                          'type': 'SEQUENCE'}],
                                                                                             'name': 'aperiodicCSI-Trigger2-r13',
                                                                                             'optional': True,
                                                                                             'type': 'CHOICE'}],
                                                                                'name': 'setup',
                                                                                'type': 'SEQUENCE'}],
                                                                   'type': 'CHOICE'},
                                     'CQI-ReportAperiodicHybrid-r14': {'members': [{'members': [{'members': [{'name': 'trigger1-Indicator-r14',
                                                                                                              'size': [8],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'}],
                                                                                                 'name': 'oneBit-r14',
                                                                                                 'type': 'SEQUENCE'},
                                                                                                {'members': [{'name': 'trigger01-Indicator-r14',
                                                                                                              'size': [8],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'},
                                                                                                             {'name': 'trigger10-Indicator-r14',
                                                                                                              'size': [8],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'},
                                                                                                             {'name': 'trigger11-Indicator-r14',
                                                                                                              'size': [8],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'}],
                                                                                                 'name': 'twoBit-r14',
                                                                                                 'type': 'SEQUENCE'},
                                                                                                {'members': [{'name': 'trigger001-Indicator-r14',
                                                                                                              'size': [32],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'},
                                                                                                             {'name': 'trigger010-Indicator-r14',
                                                                                                              'size': [32],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'},
                                                                                                             {'name': 'trigger011-Indicator-r14',
                                                                                                              'size': [32],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'},
                                                                                                             {'name': 'trigger100-Indicator-r14',
                                                                                                              'size': [32],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'},
                                                                                                             {'name': 'trigger101-Indicator-r14',
                                                                                                              'size': [32],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'},
                                                                                                             {'name': 'trigger110-Indicator-r14',
                                                                                                              'size': [32],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'},
                                                                                                             {'name': 'trigger111-Indicator-r14',
                                                                                                              'size': [32],
                                                                                                              'type': 'BIT '
                                                                                                                      'STRING'}],
                                                                                                 'name': 'threeBit-r14',
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'name': 'triggers-r14',
                                                                                    'optional': True,
                                                                                    'type': 'CHOICE'}],
                                                                       'type': 'SEQUENCE'},
                                     'CQI-ReportAperiodicProc-r11': {'members': [{'name': 'cqi-ReportModeAperiodic-r11',
                                                                                  'type': 'CQI-ReportModeAperiodic'},
                                                                                 {'name': 'trigger01-r11',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'trigger10-r11',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'trigger11-r11',
                                                                                  'type': 'BOOLEAN'}],
                                                                     'type': 'SEQUENCE'},
                                     'CQI-ReportAperiodicProc-v1310': {'members': [{'name': 'trigger001-r13',
                                                                                    'type': 'BOOLEAN'},
                                                                                   {'name': 'trigger010-r13',
                                                                                    'type': 'BOOLEAN'},
                                                                                   {'name': 'trigger011-r13',
                                                                                    'type': 'BOOLEAN'},
                                                                                   {'name': 'trigger100-r13',
                                                                                    'type': 'BOOLEAN'},
                                                                                   {'name': 'trigger101-r13',
                                                                                    'type': 'BOOLEAN'},
                                                                                   {'name': 'trigger110-r13',
                                                                                    'type': 'BOOLEAN'},
                                                                                   {'name': 'trigger111-r13',
                                                                                    'type': 'BOOLEAN'}],
                                                                       'type': 'SEQUENCE'},
                                     'CQI-ReportBoth-r11': {'members': [{'name': 'csi-IM-ConfigToReleaseList-r11',
                                                                         'optional': True,
                                                                         'type': 'CSI-IM-ConfigToReleaseList-r11'},
                                                                        {'name': 'csi-IM-ConfigToAddModList-r11',
                                                                         'optional': True,
                                                                         'type': 'CSI-IM-ConfigToAddModList-r11'},
                                                                        {'name': 'csi-ProcessToReleaseList-r11',
                                                                         'optional': True,
                                                                         'type': 'CSI-ProcessToReleaseList-r11'},
                                                                        {'name': 'csi-ProcessToAddModList-r11',
                                                                         'optional': True,
                                                                         'type': 'CSI-ProcessToAddModList-r11'}],
                                                            'type': 'SEQUENCE'},
                                     'CQI-ReportBoth-v1250': {'members': [{'name': 'csi-IM-ConfigToReleaseListExt-r12',
                                                                           'optional': True,
                                                                           'type': 'CSI-IM-ConfigId-v1250'},
                                                                          {'name': 'csi-IM-ConfigToAddModListExt-r12',
                                                                           'optional': True,
                                                                           'type': 'CSI-IM-ConfigExt-r12'}],
                                                              'type': 'SEQUENCE'},
                                     'CQI-ReportBoth-v1310': {'members': [{'name': 'csi-IM-ConfigToReleaseListExt-r13',
                                                                           'optional': True,
                                                                           'type': 'CSI-IM-ConfigToReleaseListExt-r13'},
                                                                          {'name': 'csi-IM-ConfigToAddModListExt-r13',
                                                                           'optional': True,
                                                                           'type': 'CSI-IM-ConfigToAddModListExt-r13'}],
                                                              'type': 'SEQUENCE'},
                                     'CQI-ReportBothProc-r11': {'members': [{'name': 'ri-Ref-CSI-ProcessId-r11',
                                                                             'optional': True,
                                                                             'type': 'CSI-ProcessId-r11'},
                                                                            {'name': 'pmi-RI-Report-r11',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('setup',
                                                                                         0)]}],
                                                                'type': 'SEQUENCE'},
                                     'CQI-ReportConfig': {'members': [{'name': 'cqi-ReportModeAperiodic',
                                                                       'optional': True,
                                                                       'type': 'CQI-ReportModeAperiodic'},
                                                                      {'name': 'nomPDSCH-RS-EPRE-Offset',
                                                                       'restricted-to': [(-1,
                                                                                          6)],
                                                                       'type': 'INTEGER'},
                                                                      {'name': 'cqi-ReportPeriodic',
                                                                       'optional': True,
                                                                       'type': 'CQI-ReportPeriodic'}],
                                                          'type': 'SEQUENCE'},
                                     'CQI-ReportConfig-r10': {'members': [{'name': 'cqi-ReportAperiodic-r10',
                                                                           'optional': True,
                                                                           'type': 'CQI-ReportAperiodic-r10'},
                                                                          {'name': 'nomPDSCH-RS-EPRE-Offset',
                                                                           'restricted-to': [(-1,
                                                                                              6)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'cqi-ReportPeriodic-r10',
                                                                           'optional': True,
                                                                           'type': 'CQI-ReportPeriodic-r10'},
                                                                          {'name': 'pmi-RI-Report-r9',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('setup',
                                                                                       0)]},
                                                                          {'members': [{'name': 'release',
                                                                                        'type': 'NULL'},
                                                                                       {'members': [{'name': 'csi-MeasSubframeSet1-r10',
                                                                                                     'type': 'MeasSubframePattern-r10'},
                                                                                                    {'name': 'csi-MeasSubframeSet2-r10',
                                                                                                     'type': 'MeasSubframePattern-r10'}],
                                                                                        'name': 'setup',
                                                                                        'type': 'SEQUENCE'}],
                                                                           'name': 'csi-SubframePatternConfig-r10',
                                                                           'optional': True,
                                                                           'type': 'CHOICE'}],
                                                              'type': 'SEQUENCE'},
                                     'CQI-ReportConfig-v1130': {'members': [{'name': 'cqi-ReportPeriodic-v1130',
                                                                             'type': 'CQI-ReportPeriodic-v1130'},
                                                                            {'name': 'cqi-ReportBoth-r11',
                                                                             'type': 'CQI-ReportBoth-r11'}],
                                                                'type': 'SEQUENCE'},
                                     'CQI-ReportConfig-v1250': {'members': [{'members': [{'name': 'release',
                                                                                          'type': 'NULL'},
                                                                                         {'members': [{'name': 'csi-MeasSubframeSets-r12',
                                                                                                       'size': [10],
                                                                                                       'type': 'BIT '
                                                                                                               'STRING'}],
                                                                                          'name': 'setup',
                                                                                          'type': 'SEQUENCE'}],
                                                                             'name': 'csi-SubframePatternConfig-r12',
                                                                             'optional': True,
                                                                             'type': 'CHOICE'},
                                                                            {'name': 'cqi-ReportBoth-v1250',
                                                                             'optional': True,
                                                                             'type': 'CQI-ReportBoth-v1250'},
                                                                            {'name': 'cqi-ReportAperiodic-v1250',
                                                                             'optional': True,
                                                                             'type': 'CQI-ReportAperiodic-v1250'},
                                                                            {'name': 'altCQI-Table-r12',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('allSubframes',
                                                                                         0),
                                                                                        ('csi-SubframeSet1',
                                                                                         1),
                                                                                        ('csi-SubframeSet2',
                                                                                         2),
                                                                                        ('spare1',
                                                                                         3)]}],
                                                                'type': 'SEQUENCE'},
                                     'CQI-ReportConfig-v1310': {'members': [{'name': 'cqi-ReportBoth-v1310',
                                                                             'optional': True,
                                                                             'type': 'CQI-ReportBoth-v1310'},
                                                                            {'name': 'cqi-ReportAperiodic-v1310',
                                                                             'optional': True,
                                                                             'type': 'CQI-ReportAperiodic-v1310'},
                                                                            {'name': 'cqi-ReportPeriodic-v1310',
                                                                             'optional': True,
                                                                             'type': 'CQI-ReportPeriodic-v1310'}],
                                                                'type': 'SEQUENCE'},
                                     'CQI-ReportConfig-v1320': {'members': [{'name': 'cqi-ReportPeriodic-v1320',
                                                                             'optional': True,
                                                                             'type': 'CQI-ReportPeriodic-v1320'}],
                                                                'type': 'SEQUENCE'},
                                     'CQI-ReportConfig-v1430': {'members': [{'name': 'cqi-ReportAperiodicHybrid-r14',
                                                                             'optional': True,
                                                                             'type': 'CQI-ReportAperiodicHybrid-r14'}],
                                                                'type': 'SEQUENCE'},
                                     'CQI-ReportConfig-v920': {'members': [{'name': 'cqi-Mask-r9',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('setup',
                                                                                        0)]},
                                                                           {'name': 'pmi-RI-Report-r9',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('setup',
                                                                                        0)]}],
                                                               'type': 'SEQUENCE'},
                                     'CQI-ReportConfigSCell-r10': {'members': [{'name': 'cqi-ReportModeAperiodic-r10',
                                                                                'optional': True,
                                                                                'type': 'CQI-ReportModeAperiodic'},
                                                                               {'name': 'nomPDSCH-RS-EPRE-Offset-r10',
                                                                                'restricted-to': [(-1,
                                                                                                   6)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'cqi-ReportPeriodicSCell-r10',
                                                                                'optional': True,
                                                                                'type': 'CQI-ReportPeriodic-r10'},
                                                                               {'name': 'pmi-RI-Report-r10',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('setup',
                                                                                            0)]}],
                                                                   'type': 'SEQUENCE'},
                                     'CQI-ReportModeAperiodic': {'type': 'ENUMERATED',
                                                                 'values': [('rm12',
                                                                             0),
                                                                            ('rm20',
                                                                             1),
                                                                            ('rm22',
                                                                             2),
                                                                            ('rm30',
                                                                             3),
                                                                            ('rm31',
                                                                             4),
                                                                            ('rm32-v1250',
                                                                             5),
                                                                            ('rm10-v1310',
                                                                             6),
                                                                            ('rm11-v1310',
                                                                             7)]},
                                     'CQI-ReportPeriodic': {'members': [{'name': 'release',
                                                                         'type': 'NULL'},
                                                                        {'members': [{'name': 'cqi-PUCCH-ResourceIndex',
                                                                                      'restricted-to': [(0,
                                                                                                         1185)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'cqi-pmi-ConfigIndex',
                                                                                      'restricted-to': [(0,
                                                                                                         1023)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'members': [{'name': 'widebandCQI',
                                                                                                   'type': 'NULL'},
                                                                                                  {'members': [{'name': 'k',
                                                                                                                'restricted-to': [(1,
                                                                                                                                   4)],
                                                                                                                'type': 'INTEGER'}],
                                                                                                   'name': 'subbandCQI',
                                                                                                   'type': 'SEQUENCE'}],
                                                                                      'name': 'cqi-FormatIndicatorPeriodic',
                                                                                      'type': 'CHOICE'},
                                                                                     {'name': 'ri-ConfigIndex',
                                                                                      'optional': True,
                                                                                      'restricted-to': [(0,
                                                                                                         1023)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'simultaneousAckNackAndCQI',
                                                                                      'type': 'BOOLEAN'}],
                                                                         'name': 'setup',
                                                                         'type': 'SEQUENCE'}],
                                                            'type': 'CHOICE'},
                                     'CQI-ReportPeriodic-r10': {'members': [{'name': 'release',
                                                                             'type': 'NULL'},
                                                                            {'members': [{'name': 'cqi-PUCCH-ResourceIndex-r10',
                                                                                          'restricted-to': [(0,
                                                                                                             1184)],
                                                                                          'type': 'INTEGER'},
                                                                                         {'name': 'cqi-PUCCH-ResourceIndexP1-r10',
                                                                                          'optional': True,
                                                                                          'restricted-to': [(0,
                                                                                                             1184)],
                                                                                          'type': 'INTEGER'},
                                                                                         {'name': 'cqi-pmi-ConfigIndex',
                                                                                          'restricted-to': [(0,
                                                                                                             1023)],
                                                                                          'type': 'INTEGER'},
                                                                                         {'members': [{'members': [{'name': 'csi-ReportMode-r10',
                                                                                                                    'optional': True,
                                                                                                                    'type': 'ENUMERATED',
                                                                                                                    'values': [('submode1',
                                                                                                                                0),
                                                                                                                               ('submode2',
                                                                                                                                1)]}],
                                                                                                       'name': 'widebandCQI-r10',
                                                                                                       'type': 'SEQUENCE'},
                                                                                                      {'members': [{'name': 'k',
                                                                                                                    'restricted-to': [(1,
                                                                                                                                       4)],
                                                                                                                    'type': 'INTEGER'},
                                                                                                                   {'name': 'periodicityFactor-r10',
                                                                                                                    'type': 'ENUMERATED',
                                                                                                                    'values': [('n2',
                                                                                                                                0),
                                                                                                                               ('n4',
                                                                                                                                1)]}],
                                                                                                       'name': 'subbandCQI-r10',
                                                                                                       'type': 'SEQUENCE'}],
                                                                                          'name': 'cqi-FormatIndicatorPeriodic-r10',
                                                                                          'type': 'CHOICE'},
                                                                                         {'name': 'ri-ConfigIndex',
                                                                                          'optional': True,
                                                                                          'restricted-to': [(0,
                                                                                                             1023)],
                                                                                          'type': 'INTEGER'},
                                                                                         {'name': 'simultaneousAckNackAndCQI',
                                                                                          'type': 'BOOLEAN'},
                                                                                         {'name': 'cqi-Mask-r9',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('setup',
                                                                                                      0)]},
                                                                                         {'members': [{'name': 'release',
                                                                                                       'type': 'NULL'},
                                                                                                      {'members': [{'name': 'cqi-pmi-ConfigIndex2-r10',
                                                                                                                    'restricted-to': [(0,
                                                                                                                                       1023)],
                                                                                                                    'type': 'INTEGER'},
                                                                                                                   {'name': 'ri-ConfigIndex2-r10',
                                                                                                                    'optional': True,
                                                                                                                    'restricted-to': [(0,
                                                                                                                                       1023)],
                                                                                                                    'type': 'INTEGER'}],
                                                                                                       'name': 'setup',
                                                                                                       'type': 'SEQUENCE'}],
                                                                                          'name': 'csi-ConfigIndex-r10',
                                                                                          'optional': True,
                                                                                          'type': 'CHOICE'}],
                                                                             'name': 'setup',
                                                                             'type': 'SEQUENCE'}],
                                                                'type': 'CHOICE'},
                                     'CQI-ReportPeriodic-v1130': {'members': [{'name': 'simultaneousAckNackAndCQI-Format3-r11',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('setup',
                                                                                           0)]},
                                                                              {'name': 'cqi-ReportPeriodicProcExtToReleaseList-r11',
                                                                               'optional': True,
                                                                               'type': 'CQI-ReportPeriodicProcExtToReleaseList-r11'},
                                                                              {'name': 'cqi-ReportPeriodicProcExtToAddModList-r11',
                                                                               'optional': True,
                                                                               'type': 'CQI-ReportPeriodicProcExtToAddModList-r11'}],
                                                                  'type': 'SEQUENCE'},
                                     'CQI-ReportPeriodic-v1310': {'members': [{'name': 'cri-ReportConfig-r13',
                                                                               'optional': True,
                                                                               'type': 'CRI-ReportConfig-r13'},
                                                                              {'name': 'simultaneousAckNackAndCQI-Format4-Format5-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('setup',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'CQI-ReportPeriodic-v1320': {'members': [{'name': 'periodicityFactorWB-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n2',
                                                                                           0),
                                                                                          ('n4',
                                                                                           1)]}],
                                                                  'type': 'SEQUENCE'},
                                     'CQI-ReportPeriodicProcExt-r11': {'members': [{'name': 'cqi-ReportPeriodicProcExtId-r11',
                                                                                    'type': 'CQI-ReportPeriodicProcExtId-r11'},
                                                                                   {'name': 'cqi-pmi-ConfigIndex-r11',
                                                                                    'restricted-to': [(0,
                                                                                                       1023)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'members': [{'members': [{'name': 'csi-ReportMode-r11',
                                                                                                              'optional': True,
                                                                                                              'type': 'ENUMERATED',
                                                                                                              'values': [('submode1',
                                                                                                                          0),
                                                                                                                         ('submode2',
                                                                                                                          1)]}],
                                                                                                 'name': 'widebandCQI-r11',
                                                                                                 'type': 'SEQUENCE'},
                                                                                                {'members': [{'name': 'k',
                                                                                                              'restricted-to': [(1,
                                                                                                                                 4)],
                                                                                                              'type': 'INTEGER'},
                                                                                                             {'name': 'periodicityFactor-r11',
                                                                                                              'type': 'ENUMERATED',
                                                                                                              'values': [('n2',
                                                                                                                          0),
                                                                                                                         ('n4',
                                                                                                                          1)]}],
                                                                                                 'name': 'subbandCQI-r11',
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'name': 'cqi-FormatIndicatorPeriodic-r11',
                                                                                    'type': 'CHOICE'},
                                                                                   {'name': 'ri-ConfigIndex-r11',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(0,
                                                                                                       1023)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'members': [{'name': 'release',
                                                                                                 'type': 'NULL'},
                                                                                                {'members': [{'name': 'cqi-pmi-ConfigIndex2-r11',
                                                                                                              'restricted-to': [(0,
                                                                                                                                 1023)],
                                                                                                              'type': 'INTEGER'},
                                                                                                             {'name': 'ri-ConfigIndex2-r11',
                                                                                                              'optional': True,
                                                                                                              'restricted-to': [(0,
                                                                                                                                 1023)],
                                                                                                              'type': 'INTEGER'}],
                                                                                                 'name': 'setup',
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'name': 'csi-ConfigIndex-r11',
                                                                                    'optional': True,
                                                                                    'type': 'CHOICE'},
                                                                                   None,
                                                                                   [{'name': 'cri-ReportConfig-r13',
                                                                                     'optional': True,
                                                                                     'type': 'CRI-ReportConfig-r13'}],
                                                                                   [{'name': 'periodicityFactorWB-r13',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('n2',
                                                                                                 0),
                                                                                                ('n4',
                                                                                                 1)]}]],
                                                                       'type': 'SEQUENCE'},
                                     'CQI-ReportPeriodicProcExtId-r11': {'restricted-to': [(1,
                                                                                            'maxCQI-ProcExt-r11')],
                                                                         'type': 'INTEGER'},
                                     'CQI-ReportPeriodicProcExtToAddModList-r11': {'element': {'type': 'CQI-ReportPeriodicProcExt-r11'},
                                                                                   'size': [(1,
                                                                                             'maxCQI-ProcExt-r11')],
                                                                                   'type': 'SEQUENCE '
                                                                                           'OF'},
                                     'CQI-ReportPeriodicProcExtToReleaseList-r11': {'element': {'type': 'CQI-ReportPeriodicProcExtId-r11'},
                                                                                    'size': [(1,
                                                                                              'maxCQI-ProcExt-r11')],
                                                                                    'type': 'SEQUENCE '
                                                                                            'OF'},
                                     'CRI-ConfigIndex-r13': {'restricted-to': [(0,
                                                                                1023)],
                                                             'type': 'INTEGER'},
                                     'CRI-ReportConfig-r13': {'members': [{'name': 'release',
                                                                           'type': 'NULL'},
                                                                          {'members': [{'name': 'cri-ConfigIndex-r13',
                                                                                        'type': 'CRI-ConfigIndex-r13'},
                                                                                       {'name': 'cri-ConfigIndex2-r13',
                                                                                        'optional': True,
                                                                                        'type': 'CRI-ConfigIndex-r13'}],
                                                                           'name': 'setup',
                                                                           'type': 'SEQUENCE'}],
                                                              'type': 'CHOICE'},
                                     'CRS-AssistanceInfo-r11': {'members': [{'name': 'physCellId-r11',
                                                                             'type': 'PhysCellId'},
                                                                            {'name': 'antennaPortsCount-r11',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('an1',
                                                                                         0),
                                                                                        ('an2',
                                                                                         1),
                                                                                        ('an4',
                                                                                         2),
                                                                                        ('spare1',
                                                                                         3)]},
                                                                            {'name': 'mbsfn-SubframeConfigList-r11',
                                                                             'type': 'MBSFN-SubframeConfigList'},
                                                                            None,
                                                                            [{'name': 'mbsfn-SubframeConfigList-v1430',
                                                                              'optional': True,
                                                                              'type': 'MBSFN-SubframeConfigList-v1430'}]],
                                                                'type': 'SEQUENCE'},
                                     'CRS-AssistanceInfo-r13': {'members': [{'name': 'physCellId-r13',
                                                                             'type': 'PhysCellId'},
                                                                            {'name': 'antennaPortsCount-r13',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('an1',
                                                                                         0),
                                                                                        ('an2',
                                                                                         1),
                                                                                        ('an4',
                                                                                         2),
                                                                                        ('spare1',
                                                                                         3)]},
                                                                            {'name': 'mbsfn-SubframeConfigList-r13',
                                                                             'optional': True,
                                                                             'type': 'MBSFN-SubframeConfigList'},
                                                                            None,
                                                                            [{'name': 'mbsfn-SubframeConfigList-v1430',
                                                                              'optional': True,
                                                                              'type': 'MBSFN-SubframeConfigList-v1430'}]],
                                                                'type': 'SEQUENCE'},
                                     'CRS-AssistanceInfoList-r11': {'element': {'type': 'CRS-AssistanceInfo-r11'},
                                                                    'size': [(1,
                                                                              'maxCellReport')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'CRS-AssistanceInfoList-r13': {'element': {'type': 'CRS-AssistanceInfo-r13'},
                                                                    'size': [(1,
                                                                              'maxCellReport')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'CSFB-RegistrationParam1XRTT': {'members': [{'name': 'sid',
                                                                                  'size': [15],
                                                                                  'type': 'BIT '
                                                                                          'STRING'},
                                                                                 {'name': 'nid',
                                                                                  'size': [16],
                                                                                  'type': 'BIT '
                                                                                          'STRING'},
                                                                                 {'name': 'multipleSID',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'multipleNID',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'homeReg',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'foreignSIDReg',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'foreignNIDReg',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'parameterReg',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'powerUpReg',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'registrationPeriod',
                                                                                  'size': [7],
                                                                                  'type': 'BIT '
                                                                                          'STRING'},
                                                                                 {'name': 'registrationZone',
                                                                                  'size': [12],
                                                                                  'type': 'BIT '
                                                                                          'STRING'},
                                                                                 {'name': 'totalZone',
                                                                                  'size': [3],
                                                                                  'type': 'BIT '
                                                                                          'STRING'},
                                                                                 {'name': 'zoneTimer',
                                                                                  'size': [3],
                                                                                  'type': 'BIT '
                                                                                          'STRING'}],
                                                                     'type': 'SEQUENCE'},
                                     'CSFB-RegistrationParam1XRTT-v920': {'members': [{'name': 'powerDownReg-r9',
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('true',
                                                                                                   0)]}],
                                                                          'type': 'SEQUENCE'},
                                     'CSFBParametersRequestCDMA2000': {'members': [{'members': [{'name': 'csfbParametersRequestCDMA2000-r8',
                                                                                                 'type': 'CSFBParametersRequestCDMA2000-r8-IEs'},
                                                                                                {'members': [],
                                                                                                 'name': 'criticalExtensionsFuture',
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'name': 'criticalExtensions',
                                                                                    'type': 'CHOICE'}],
                                                                       'type': 'SEQUENCE'},
                                     'CSFBParametersRequestCDMA2000-r8-IEs': {'members': [{'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'CSFBParametersRequestCDMA2000-v8a0-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'CSFBParametersRequestCDMA2000-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'OCTET '
                                                                                                     'STRING'},
                                                                                            {'members': [],
                                                                                             'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'SEQUENCE'}],
                                                                                'type': 'SEQUENCE'},
                                     'CSFBParametersResponseCDMA2000': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                     'type': 'RRC-TransactionIdentifier'},
                                                                                    {'members': [{'name': 'csfbParametersResponseCDMA2000-r8',
                                                                                                  'type': 'CSFBParametersResponseCDMA2000-r8-IEs'},
                                                                                                 {'members': [],
                                                                                                  'name': 'criticalExtensionsFuture',
                                                                                                  'type': 'SEQUENCE'}],
                                                                                     'name': 'criticalExtensions',
                                                                                     'type': 'CHOICE'}],
                                                                        'type': 'SEQUENCE'},
                                     'CSFBParametersResponseCDMA2000-r8-IEs': {'members': [{'name': 'rand',
                                                                                            'type': 'RAND-CDMA2000'},
                                                                                           {'name': 'mobilityParameters',
                                                                                            'type': 'MobilityParametersCDMA2000'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'CSFBParametersResponseCDMA2000-v8a0-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'CSFBParametersResponseCDMA2000-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                              'optional': True,
                                                                                              'type': 'OCTET '
                                                                                                      'STRING'},
                                                                                             {'members': [],
                                                                                              'name': 'nonCriticalExtension',
                                                                                              'optional': True,
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'type': 'SEQUENCE'},
                                     'CSG-AllowedReportingCells-r9': {'members': [{'name': 'physCellIdRangeUTRA-FDDList-r9',
                                                                                   'optional': True,
                                                                                   'type': 'PhysCellIdRangeUTRA-FDDList-r9'}],
                                                                      'type': 'SEQUENCE'},
                                     'CSG-Identity': {'size': [27],
                                                      'type': 'BIT STRING'},
                                     'CSG-ProximityIndicationParameters-r9': {'members': [{'name': 'intraFreqProximityIndication-r9',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('supported',
                                                                                                       0)]},
                                                                                          {'name': 'interFreqProximityIndication-r9',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('supported',
                                                                                                       0)]},
                                                                                          {'name': 'utran-ProximityIndication-r9',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('supported',
                                                                                                       0)]}],
                                                                              'type': 'SEQUENCE'},
                                     'CSI-IM-Config-r11': {'members': [{'name': 'csi-IM-ConfigId-r11',
                                                                        'type': 'CSI-IM-ConfigId-r11'},
                                                                       {'name': 'resourceConfig-r11',
                                                                        'restricted-to': [(0,
                                                                                           31)],
                                                                        'type': 'INTEGER'},
                                                                       {'name': 'subframeConfig-r11',
                                                                        'restricted-to': [(0,
                                                                                           154)],
                                                                        'type': 'INTEGER'},
                                                                       None,
                                                                       [{'name': 'interferenceMeasRestriction-r13',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'}]],
                                                           'type': 'SEQUENCE'},
                                     'CSI-IM-ConfigExt-r12': {'members': [{'name': 'csi-IM-ConfigId-v1250',
                                                                           'type': 'CSI-IM-ConfigId-v1250'},
                                                                          {'name': 'resourceConfig-r12',
                                                                           'restricted-to': [(0,
                                                                                              31)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'subframeConfig-r12',
                                                                           'restricted-to': [(0,
                                                                                              154)],
                                                                           'type': 'INTEGER'},
                                                                          None,
                                                                          [{'name': 'interferenceMeasRestriction-r13',
                                                                            'optional': True,
                                                                            'type': 'BOOLEAN'},
                                                                           {'name': 'csi-IM-ConfigId-v1310',
                                                                            'optional': True,
                                                                            'type': 'CSI-IM-ConfigId-v1310'}]],
                                                              'type': 'SEQUENCE'},
                                     'CSI-IM-ConfigId-r11': {'restricted-to': [(1,
                                                                                'maxCSI-IM-r11')],
                                                             'type': 'INTEGER'},
                                     'CSI-IM-ConfigId-r12': {'restricted-to': [(1,
                                                                                'maxCSI-IM-r12')],
                                                             'type': 'INTEGER'},
                                     'CSI-IM-ConfigId-r13': {'restricted-to': [(1,
                                                                                'maxCSI-IM-r13')],
                                                             'type': 'INTEGER'},
                                     'CSI-IM-ConfigId-v1250': {'restricted-to': ['maxCSI-IM-r12'],
                                                               'type': 'INTEGER'},
                                     'CSI-IM-ConfigId-v1310': {'restricted-to': [('minCSI-IM-r13',
                                                                                  'maxCSI-IM-r13')],
                                                               'type': 'INTEGER'},
                                     'CSI-IM-ConfigToAddModList-r11': {'element': {'type': 'CSI-IM-Config-r11'},
                                                                       'size': [(1,
                                                                                 'maxCSI-IM-r11')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'CSI-IM-ConfigToAddModListExt-r13': {'element': {'type': 'CSI-IM-ConfigExt-r12'},
                                                                          'size': [(1,
                                                                                    'maxCSI-IM-v1310')],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                     'CSI-IM-ConfigToReleaseList-r11': {'element': {'type': 'CSI-IM-ConfigId-r11'},
                                                                        'size': [(1,
                                                                                  'maxCSI-IM-r11')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'CSI-IM-ConfigToReleaseListExt-r13': {'element': {'type': 'CSI-IM-ConfigId-v1310'},
                                                                           'size': [(1,
                                                                                     'maxCSI-IM-v1310')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'CSI-Process-r11': {'members': [{'name': 'csi-ProcessId-r11',
                                                                      'type': 'CSI-ProcessId-r11'},
                                                                     {'name': 'csi-RS-ConfigNZPId-r11',
                                                                      'type': 'CSI-RS-ConfigNZPId-r11'},
                                                                     {'name': 'csi-IM-ConfigId-r11',
                                                                      'type': 'CSI-IM-ConfigId-r11'},
                                                                     {'name': 'p-C-AndCBSRList-r11',
                                                                      'type': 'P-C-AndCBSR-Pair-r13a'},
                                                                     {'name': 'cqi-ReportBothProc-r11',
                                                                      'optional': True,
                                                                      'type': 'CQI-ReportBothProc-r11'},
                                                                     {'name': 'cqi-ReportPeriodicProcId-r11',
                                                                      'optional': True,
                                                                      'restricted-to': [(0,
                                                                                         'maxCQI-ProcExt-r11')],
                                                                      'type': 'INTEGER'},
                                                                     {'name': 'cqi-ReportAperiodicProc-r11',
                                                                      'optional': True,
                                                                      'type': 'CQI-ReportAperiodicProc-r11'},
                                                                     None,
                                                                     [{'name': 'alternativeCodebookEnabledFor4TXProc-r12',
                                                                       'optional': True,
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('true',
                                                                                   0)]},
                                                                      {'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'element': {'type': 'CSI-IM-ConfigId-r12'},
                                                                                    'name': 'setup',
                                                                                    'size': [(1,
                                                                                              2)],
                                                                                    'type': 'SEQUENCE '
                                                                                            'OF'}],
                                                                       'name': 'csi-IM-ConfigIdList-r12',
                                                                       'optional': True,
                                                                       'type': 'CHOICE'},
                                                                      {'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'name': 'setup',
                                                                                    'type': 'CQI-ReportAperiodicProc-r11'}],
                                                                       'name': 'cqi-ReportAperiodicProc2-r12',
                                                                       'optional': True,
                                                                       'type': 'CHOICE'}],
                                                                     [{'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'name': 'setup',
                                                                                    'type': 'CQI-ReportAperiodicProc-v1310'}],
                                                                       'name': 'cqi-ReportAperiodicProc-v1310',
                                                                       'optional': True,
                                                                       'type': 'CHOICE'},
                                                                      {'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'name': 'setup',
                                                                                    'type': 'CQI-ReportAperiodicProc-v1310'}],
                                                                       'name': 'cqi-ReportAperiodicProc2-v1310',
                                                                       'optional': True,
                                                                       'type': 'CHOICE'},
                                                                      {'name': 'eMIMO-Type-r13',
                                                                       'optional': True,
                                                                       'type': 'CSI-RS-ConfigEMIMO-r13'}],
                                                                     [{'name': 'eMIMO-Type-v1430',
                                                                       'optional': True,
                                                                       'type': 'CSI-RS-ConfigEMIMO-v1430'},
                                                                      {'name': 'eMIMO-Hybrid-r14',
                                                                       'optional': True,
                                                                       'type': 'CSI-RS-ConfigEMIMO-Hybrid-r14'},
                                                                      {'name': 'advancedCodebookEnabled-r14',
                                                                       'optional': True,
                                                                       'type': 'BOOLEAN'}]],
                                                         'type': 'SEQUENCE'},
                                     'CSI-ProcessId-r11': {'restricted-to': [(1,
                                                                              'maxCSI-Proc-r11')],
                                                           'type': 'INTEGER'},
                                     'CSI-ProcessToAddModList-r11': {'element': {'type': 'CSI-Process-r11'},
                                                                     'size': [(1,
                                                                               'maxCSI-Proc-r11')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'CSI-ProcessToReleaseList-r11': {'element': {'type': 'CSI-ProcessId-r11'},
                                                                      'size': [(1,
                                                                                'maxCSI-Proc-r11')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'CSI-RS-Config-NZP-v1430': {'members': [{'name': 'transmissionComb-r14',
                                                                              'optional': True,
                                                                              'type': 'NZP-TransmissionComb-r14'},
                                                                             {'name': 'frequencyDensity-r14',
                                                                              'optional': True,
                                                                              'type': 'NZP-FrequencyDensity-r14'}],
                                                                 'type': 'SEQUENCE'},
                                     'CSI-RS-Config-r10': {'members': [{'members': [{'name': 'release',
                                                                                     'type': 'NULL'},
                                                                                    {'members': [{'name': 'antennaPortsCount-r10',
                                                                                                  'type': 'ENUMERATED',
                                                                                                  'values': [('an1',
                                                                                                              0),
                                                                                                             ('an2',
                                                                                                              1),
                                                                                                             ('an4',
                                                                                                              2),
                                                                                                             ('an8',
                                                                                                              3)]},
                                                                                                 {'name': 'resourceConfig-r10',
                                                                                                  'restricted-to': [(0,
                                                                                                                     31)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'name': 'subframeConfig-r10',
                                                                                                  'restricted-to': [(0,
                                                                                                                     154)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'name': 'p-C-r10',
                                                                                                  'restricted-to': [(-8,
                                                                                                                     15)],
                                                                                                  'type': 'INTEGER'}],
                                                                                     'name': 'setup',
                                                                                     'type': 'SEQUENCE'}],
                                                                        'name': 'csi-RS-r10',
                                                                        'optional': True,
                                                                        'type': 'CHOICE'},
                                                                       {'name': 'zeroTxPowerCSI-RS-r10',
                                                                        'optional': True,
                                                                        'type': 'ZeroTxPowerCSI-RS-Conf-r12'}],
                                                           'type': 'SEQUENCE'},
                                     'CSI-RS-Config-v1250': {'members': [{'name': 'zeroTxPowerCSI-RS2-r12',
                                                                          'optional': True,
                                                                          'type': 'ZeroTxPowerCSI-RS-Conf-r12'},
                                                                         {'members': [{'name': 'release',
                                                                                       'type': 'NULL'},
                                                                                      {'members': [{'element': {'type': 'ZeroTxPowerCSI-RS-r12'},
                                                                                                    'name': 'zeroTxPowerCSI-RS-List-r12',
                                                                                                    'size': [(1,
                                                                                                              'maxDS-ZTP-CSI-RS-r12')],
                                                                                                    'type': 'SEQUENCE '
                                                                                                            'OF'}],
                                                                                       'name': 'setup',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'name': 'ds-ZeroTxPowerCSI-RS-r12',
                                                                          'optional': True,
                                                                          'type': 'CHOICE'}],
                                                             'type': 'SEQUENCE'},
                                     'CSI-RS-Config-v1310': {'members': [{'name': 'eMIMO-Type-r13',
                                                                          'optional': True,
                                                                          'type': 'CSI-RS-ConfigEMIMO-r13'}],
                                                             'type': 'SEQUENCE'},
                                     'CSI-RS-Config-v1430': {'members': [{'name': 'eMIMO-Type-v1430',
                                                                          'optional': True,
                                                                          'type': 'CSI-RS-ConfigEMIMO-v1430'},
                                                                         {'name': 'eMIMO-Hybrid-r14',
                                                                          'optional': True,
                                                                          'type': 'CSI-RS-ConfigEMIMO-Hybrid-r14'},
                                                                         {'name': 'advancedCodebookEnabled-r14',
                                                                          'optional': True,
                                                                          'type': 'BOOLEAN'}],
                                                             'type': 'SEQUENCE'},
                                     'CSI-RS-ConfigBeamformed-r13': {'members': [{'element': {'type': 'CSI-RS-ConfigNZPId-r13'},
                                                                                  'name': 'csi-RS-ConfigNZPIdListExt-r13',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            7)],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'element': {'type': 'CSI-IM-ConfigId-r13'},
                                                                                  'name': 'csi-IM-ConfigIdList-r13',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            8)],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'element': {'type': 'P-C-AndCBSR-Pair-r13'},
                                                                                  'name': 'p-C-AndCBSR-PerResourceConfigList-r13',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            8)],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'element': {'type': 'BOOLEAN'},
                                                                                  'name': 'ace-For4Tx-PerResourceConfigList-r13',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            7)],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'name': 'alternativeCodebookEnabledBeamformed-r13',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('true',
                                                                                              0)]},
                                                                                 {'name': 'channelMeasRestriction-r13',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('on',
                                                                                              0)]}],
                                                                     'type': 'SEQUENCE'},
                                     'CSI-RS-ConfigBeamformed-r14': {'members': [{'element': {'type': 'CSI-RS-ConfigNZPId-r13'},
                                                                                  'name': 'csi-RS-ConfigNZPIdListExt-r14',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            7)],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'element': {'type': 'CSI-IM-ConfigId-r13'},
                                                                                  'name': 'csi-IM-ConfigIdList-r14',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            8)],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'element': {'type': 'P-C-AndCBSR-Pair-r13'},
                                                                                  'name': 'p-C-AndCBSR-PerResourceConfigList-r14',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            8)],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'element': {'type': 'BOOLEAN'},
                                                                                  'name': 'ace-For4Tx-PerResourceConfigList-r14',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            7)],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'name': 'alternativeCodebookEnabledBeamformed-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('true',
                                                                                              0)]},
                                                                                 {'name': 'channelMeasRestriction-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('on',
                                                                                              0)]},
                                                                                 {'element': {'type': 'CSI-RS-ConfigNZP-r11'},
                                                                                  'name': 'csi-RS-ConfigNZP-ApList-r14',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            8)],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'name': 'nzp-ResourceConfigOriginal-v1430',
                                                                                  'optional': True,
                                                                                  'type': 'CSI-RS-Config-NZP-v1430'},
                                                                                 {'name': 'csi-RS-NZP-Activation-r14',
                                                                                  'optional': True,
                                                                                  'type': 'CSI-RS-ConfigNZP-Activation-r14'}],
                                                                     'type': 'SEQUENCE'},
                                     'CSI-RS-ConfigBeamformed-v1430': {'members': [{'element': {'type': 'CSI-RS-ConfigNZP-r11'},
                                                                                    'name': 'csi-RS-ConfigNZP-ApList-r14',
                                                                                    'optional': True,
                                                                                    'size': [(1,
                                                                                              8)],
                                                                                    'type': 'SEQUENCE '
                                                                                            'OF'},
                                                                                   {'name': 'nzp-ResourceConfigOriginal-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'CSI-RS-Config-NZP-v1430'},
                                                                                   {'name': 'csi-RS-NZP-Activation-r14',
                                                                                    'optional': True,
                                                                                    'type': 'CSI-RS-ConfigNZP-Activation-r14'}],
                                                                       'type': 'SEQUENCE'},
                                     'CSI-RS-ConfigEMIMO-Hybrid-r14': {'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'members': [{'name': 'periodicityOffsetIndex-r14',
                                                                                                 'optional': True,
                                                                                                 'restricted-to': [(0,
                                                                                                                    1023)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'eMIMO-Type2-r14',
                                                                                                 'optional': True,
                                                                                                 'type': 'CSI-RS-ConfigEMIMO2-r14'}],
                                                                                    'name': 'setup',
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'CHOICE'},
                                     'CSI-RS-ConfigEMIMO-r13': {'members': [{'name': 'release',
                                                                             'type': 'NULL'},
                                                                            {'members': [{'name': 'nonPrecoded-r13',
                                                                                          'type': 'CSI-RS-ConfigNonPrecoded-r13'},
                                                                                         {'name': 'beamformed-r13',
                                                                                          'type': 'CSI-RS-ConfigBeamformed-r13'}],
                                                                             'name': 'setup',
                                                                             'type': 'CHOICE'}],
                                                                'type': 'CHOICE'},
                                     'CSI-RS-ConfigEMIMO-v1430': {'members': [{'name': 'release',
                                                                               'type': 'NULL'},
                                                                              {'members': [{'name': 'nonPrecoded-v1430',
                                                                                            'type': 'CSI-RS-ConfigNonPrecoded-v1430'},
                                                                                           {'name': 'beamformed-v1430',
                                                                                            'type': 'CSI-RS-ConfigBeamformed-v1430'}],
                                                                               'name': 'setup',
                                                                               'type': 'CHOICE'}],
                                                                  'type': 'CHOICE'},
                                     'CSI-RS-ConfigEMIMO2-r14': {'members': [{'name': 'release',
                                                                              'type': 'NULL'},
                                                                             {'name': 'setup',
                                                                              'type': 'CSI-RS-ConfigBeamformed-r14'}],
                                                                 'type': 'CHOICE'},
                                     'CSI-RS-ConfigNZP-Activation-r14': {'members': [{'name': 'csi-RS-NZP-mode-r14',
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('semiPersistent',
                                                                                                  0),
                                                                                                 ('aperiodic',
                                                                                                  1)]},
                                                                                     {'name': 'activatedResources-r14',
                                                                                      'restricted-to': [(0,
                                                                                                         4)],
                                                                                      'type': 'INTEGER'}],
                                                                         'type': 'SEQUENCE'},
                                     'CSI-RS-ConfigNZP-EMIMO-r13': {'members': [{'name': 'release',
                                                                                 'type': 'NULL'},
                                                                                {'members': [{'element': {'type': 'NZP-ResourceConfig-r13'},
                                                                                              'name': 'nzp-resourceConfigList-r13',
                                                                                              'size': [(1,
                                                                                                        2)],
                                                                                              'type': 'SEQUENCE '
                                                                                                      'OF'},
                                                                                             {'name': 'cdmType-r13',
                                                                                              'optional': True,
                                                                                              'type': 'ENUMERATED',
                                                                                              'values': [('cdm2',
                                                                                                          0),
                                                                                                         ('cdm4',
                                                                                                          1)]}],
                                                                                 'name': 'setup',
                                                                                 'type': 'SEQUENCE'}],
                                                                    'type': 'CHOICE'},
                                     'CSI-RS-ConfigNZP-EMIMO-v1430': {'members': [{'element': {'type': 'NZP-ResourceConfig-r13'},
                                                                                   'name': 'nzp-resourceConfigListExt-r14',
                                                                                   'size': [(0,
                                                                                             4)],
                                                                                   'type': 'SEQUENCE '
                                                                                           'OF'},
                                                                                  {'name': 'cdmType-v1430',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('cdm8',
                                                                                               0)]}],
                                                                      'type': 'SEQUENCE'},
                                     'CSI-RS-ConfigNZP-r11': {'members': [{'name': 'csi-RS-ConfigNZPId-r11',
                                                                           'type': 'CSI-RS-ConfigNZPId-r11'},
                                                                          {'name': 'antennaPortsCount-r11',
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('an1',
                                                                                       0),
                                                                                      ('an2',
                                                                                       1),
                                                                                      ('an4',
                                                                                       2),
                                                                                      ('an8',
                                                                                       3)]},
                                                                          {'name': 'resourceConfig-r11',
                                                                           'restricted-to': [(0,
                                                                                              31)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'subframeConfig-r11',
                                                                           'restricted-to': [(0,
                                                                                              154)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'scramblingIdentity-r11',
                                                                           'restricted-to': [(0,
                                                                                              503)],
                                                                           'type': 'INTEGER'},
                                                                          {'members': [{'name': 'qcl-ScramblingIdentity-r11',
                                                                                        'restricted-to': [(0,
                                                                                                           503)],
                                                                                        'type': 'INTEGER'},
                                                                                       {'name': 'crs-PortsCount-r11',
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('n1',
                                                                                                    0),
                                                                                                   ('n2',
                                                                                                    1),
                                                                                                   ('n4',
                                                                                                    2),
                                                                                                   ('spare1',
                                                                                                    3)]},
                                                                                       {'members': [{'name': 'release',
                                                                                                     'type': 'NULL'},
                                                                                                    {'members': [{'name': 'subframeConfigList',
                                                                                                                  'type': 'MBSFN-SubframeConfigList'}],
                                                                                                     'name': 'setup',
                                                                                                     'type': 'SEQUENCE'}],
                                                                                        'name': 'mbsfn-SubframeConfigList-r11',
                                                                                        'optional': True,
                                                                                        'type': 'CHOICE'}],
                                                                           'name': 'qcl-CRS-Info-r11',
                                                                           'optional': True,
                                                                           'type': 'SEQUENCE'},
                                                                          None,
                                                                          [{'name': 'csi-RS-ConfigNZPId-v1310',
                                                                            'optional': True,
                                                                            'type': 'CSI-RS-ConfigNZPId-v1310'}],
                                                                          [{'name': 'transmissionComb-r14',
                                                                            'optional': True,
                                                                            'type': 'NZP-TransmissionComb-r14'},
                                                                           {'name': 'frequencyDensity-r14',
                                                                            'optional': True,
                                                                            'type': 'NZP-FrequencyDensity-r14'}],
                                                                          [{'members': [{'name': 'release',
                                                                                         'type': 'NULL'},
                                                                                        {'members': [{'name': 'subframeConfigList-v1430',
                                                                                                      'type': 'MBSFN-SubframeConfigList-v1430'}],
                                                                                         'name': 'setup',
                                                                                         'type': 'SEQUENCE'}],
                                                                            'name': 'mbsfn-SubframeConfigList-v1430',
                                                                            'optional': True,
                                                                            'type': 'CHOICE'}]],
                                                              'type': 'SEQUENCE'},
                                     'CSI-RS-ConfigNZPId-r11': {'restricted-to': [(1,
                                                                                   'maxCSI-RS-NZP-r11')],
                                                                'type': 'INTEGER'},
                                     'CSI-RS-ConfigNZPId-r13': {'restricted-to': [(1,
                                                                                   'maxCSI-RS-NZP-r13')],
                                                                'type': 'INTEGER'},
                                     'CSI-RS-ConfigNZPId-v1310': {'restricted-to': [('minCSI-RS-NZP-r13',
                                                                                     'maxCSI-RS-NZP-r13')],
                                                                  'type': 'INTEGER'},
                                     'CSI-RS-ConfigNZPToAddModList-r11': {'element': {'type': 'CSI-RS-ConfigNZP-r11'},
                                                                          'size': [(1,
                                                                                    'maxCSI-RS-NZP-r11')],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                     'CSI-RS-ConfigNZPToAddModListExt-r13': {'element': {'type': 'CSI-RS-ConfigNZP-r11'},
                                                                             'size': [(1,
                                                                                       'maxCSI-RS-NZP-v1310')],
                                                                             'type': 'SEQUENCE '
                                                                                     'OF'},
                                     'CSI-RS-ConfigNZPToReleaseList-r11': {'element': {'type': 'CSI-RS-ConfigNZPId-r11'},
                                                                           'size': [(1,
                                                                                     'maxCSI-RS-NZP-r11')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'CSI-RS-ConfigNZPToReleaseListExt-r13': {'element': {'type': 'CSI-RS-ConfigNZPId-v1310'},
                                                                              'size': [(1,
                                                                                        'maxCSI-RS-NZP-v1310')],
                                                                              'type': 'SEQUENCE '
                                                                                      'OF'},
                                     'CSI-RS-ConfigNonPrecoded-r13': {'members': [{'name': 'p-C-AndCBSRList-r13',
                                                                                   'optional': True,
                                                                                   'type': 'P-C-AndCBSR-Pair-r13'},
                                                                                  {'name': 'codebookConfigN1-r13',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('n1',
                                                                                               0),
                                                                                              ('n2',
                                                                                               1),
                                                                                              ('n3',
                                                                                               2),
                                                                                              ('n4',
                                                                                               3),
                                                                                              ('n8',
                                                                                               4)]},
                                                                                  {'name': 'codebookConfigN2-r13',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('n1',
                                                                                               0),
                                                                                              ('n2',
                                                                                               1),
                                                                                              ('n3',
                                                                                               2),
                                                                                              ('n4',
                                                                                               3),
                                                                                              ('n8',
                                                                                               4)]},
                                                                                  {'name': 'codebookOverSamplingRateConfig-O1-r13',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('n4',
                                                                                               0),
                                                                                              ('n8',
                                                                                               1)]},
                                                                                  {'name': 'codebookOverSamplingRateConfig-O2-r13',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('n4',
                                                                                               0),
                                                                                              ('n8',
                                                                                               1)]},
                                                                                  {'name': 'codebookConfig-r13',
                                                                                   'restricted-to': [(1,
                                                                                                      4)],
                                                                                   'type': 'INTEGER'},
                                                                                  {'element': {'type': 'CSI-IM-ConfigId-r13'},
                                                                                   'name': 'csi-IM-ConfigIdList-r13',
                                                                                   'optional': True,
                                                                                   'size': [(1,
                                                                                             2)],
                                                                                   'type': 'SEQUENCE '
                                                                                           'OF'},
                                                                                  {'name': 'csi-RS-ConfigNZP-EMIMO-r13',
                                                                                   'optional': True,
                                                                                   'type': 'CSI-RS-ConfigNZP-EMIMO-r13'}],
                                                                      'type': 'SEQUENCE'},
                                     'CSI-RS-ConfigNonPrecoded-v1430': {'members': [{'name': 'csi-RS-ConfigNZP-EMIMO-v1430',
                                                                                     'optional': True,
                                                                                     'type': 'CSI-RS-ConfigNZP-EMIMO-v1430'},
                                                                                    {'name': 'codebookConfigN1-v1430',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('n5',
                                                                                                 0),
                                                                                                ('n6',
                                                                                                 1),
                                                                                                ('n7',
                                                                                                 2),
                                                                                                ('n10',
                                                                                                 3),
                                                                                                ('n12',
                                                                                                 4),
                                                                                                ('n14',
                                                                                                 5),
                                                                                                ('n16',
                                                                                                 6)]},
                                                                                    {'name': 'codebookConfigN2-r1430',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('n5',
                                                                                                 0),
                                                                                                ('n6',
                                                                                                 1),
                                                                                                ('n7',
                                                                                                 2)]},
                                                                                    {'name': 'nzp-ResourceConfigTM9-Original-v1430',
                                                                                     'type': 'CSI-RS-Config-NZP-v1430'}],
                                                                        'type': 'SEQUENCE'},
                                     'CSI-RS-ConfigZP-ApList-r14': {'members': [{'name': 'release',
                                                                                 'type': 'NULL'},
                                                                                {'element': {'type': 'CSI-RS-ConfigZP-r11'},
                                                                                 'name': 'setup',
                                                                                 'size': [(1,
                                                                                           'maxCSI-RS-ZP-r11')],
                                                                                 'type': 'SEQUENCE '
                                                                                         'OF'}],
                                                                    'type': 'CHOICE'},
                                     'CSI-RS-ConfigZP-r11': {'members': [{'name': 'csi-RS-ConfigZPId-r11',
                                                                          'type': 'CSI-RS-ConfigZPId-r11'},
                                                                         {'name': 'resourceConfigList-r11',
                                                                          'size': [16],
                                                                          'type': 'BIT '
                                                                                  'STRING'},
                                                                         {'name': 'subframeConfig-r11',
                                                                          'restricted-to': [(0,
                                                                                             154)],
                                                                          'type': 'INTEGER'},
                                                                         None],
                                                             'type': 'SEQUENCE'},
                                     'CSI-RS-ConfigZPId-r11': {'restricted-to': [(1,
                                                                                  'maxCSI-RS-ZP-r11')],
                                                               'type': 'INTEGER'},
                                     'CSI-RS-ConfigZPToAddModList-r11': {'element': {'type': 'CSI-RS-ConfigZP-r11'},
                                                                         'size': [(1,
                                                                                   'maxCSI-RS-ZP-r11')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'CSI-RS-ConfigZPToReleaseList-r11': {'element': {'type': 'CSI-RS-ConfigZPId-r11'},
                                                                          'size': [(1,
                                                                                    'maxCSI-RS-ZP-r11')],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                     'CSI-RSRP-Range-r12': {'restricted-to': [(0,
                                                                               97)],
                                                            'type': 'INTEGER'},
                                     'CarrierBandwidthEUTRA': {'members': [{'name': 'dl-Bandwidth',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('n6',
                                                                                        0),
                                                                                       ('n15',
                                                                                        1),
                                                                                       ('n25',
                                                                                        2),
                                                                                       ('n50',
                                                                                        3),
                                                                                       ('n75',
                                                                                        4),
                                                                                       ('n100',
                                                                                        5),
                                                                                       ('spare10',
                                                                                        6),
                                                                                       ('spare9',
                                                                                        7),
                                                                                       ('spare8',
                                                                                        8),
                                                                                       ('spare7',
                                                                                        9),
                                                                                       ('spare6',
                                                                                        10),
                                                                                       ('spare5',
                                                                                        11),
                                                                                       ('spare4',
                                                                                        12),
                                                                                       ('spare3',
                                                                                        13),
                                                                                       ('spare2',
                                                                                        14),
                                                                                       ('spare1',
                                                                                        15)]},
                                                                           {'name': 'ul-Bandwidth',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('n6',
                                                                                        0),
                                                                                       ('n15',
                                                                                        1),
                                                                                       ('n25',
                                                                                        2),
                                                                                       ('n50',
                                                                                        3),
                                                                                       ('n75',
                                                                                        4),
                                                                                       ('n100',
                                                                                        5),
                                                                                       ('spare10',
                                                                                        6),
                                                                                       ('spare9',
                                                                                        7),
                                                                                       ('spare8',
                                                                                        8),
                                                                                       ('spare7',
                                                                                        9),
                                                                                       ('spare6',
                                                                                        10),
                                                                                       ('spare5',
                                                                                        11),
                                                                                       ('spare4',
                                                                                        12),
                                                                                       ('spare3',
                                                                                        13),
                                                                                       ('spare2',
                                                                                        14),
                                                                                       ('spare1',
                                                                                        15)]}],
                                                               'type': 'SEQUENCE'},
                                     'CarrierFreqCDMA2000': {'members': [{'name': 'bandClass',
                                                                          'type': 'BandclassCDMA2000'},
                                                                         {'name': 'arfcn',
                                                                          'type': 'ARFCN-ValueCDMA2000'}],
                                                             'type': 'SEQUENCE'},
                                     'CarrierFreqEUTRA': {'members': [{'name': 'dl-CarrierFreq',
                                                                       'type': 'ARFCN-ValueEUTRA'},
                                                                      {'name': 'ul-CarrierFreq',
                                                                       'optional': True,
                                                                       'type': 'ARFCN-ValueEUTRA'}],
                                                          'type': 'SEQUENCE'},
                                     'CarrierFreqEUTRA-v9e0': {'members': [{'name': 'dl-CarrierFreq-v9e0',
                                                                            'type': 'ARFCN-ValueEUTRA-r9'},
                                                                           {'name': 'ul-CarrierFreq-v9e0',
                                                                            'optional': True,
                                                                            'type': 'ARFCN-ValueEUTRA-r9'}],
                                                               'type': 'SEQUENCE'},
                                     'CarrierFreqGERAN': {'members': [{'name': 'arfcn',
                                                                       'type': 'ARFCN-ValueGERAN'},
                                                                      {'name': 'bandIndicator',
                                                                       'type': 'BandIndicatorGERAN'}],
                                                          'type': 'SEQUENCE'},
                                     'CarrierFreqInfoUTRA-FDD-v8h0': {'members': [{'element': {'type': 'FreqBandIndicator-UTRA-FDD'},
                                                                                   'name': 'multiBandInfoList',
                                                                                   'optional': True,
                                                                                   'size': [(1,
                                                                                             'maxMultiBands')],
                                                                                   'type': 'SEQUENCE '
                                                                                           'OF'}],
                                                                      'type': 'SEQUENCE'},
                                     'CarrierFreqInfoUTRA-v1250': {'members': [{'name': 'reducedMeasPerformance-r12',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('true',
                                                                                            0)]}],
                                                                   'type': 'SEQUENCE'},
                                     'CarrierFreqListMBMS-r11': {'element': {'type': 'ARFCN-ValueEUTRA-r9'},
                                                                 'size': [(1,
                                                                           'maxFreqMBMS-r11')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'CarrierFreqListUTRA-FDD': {'element': {'type': 'CarrierFreqUTRA-FDD'},
                                                                 'size': [(1,
                                                                           'maxUTRA-FDD-Carrier')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'CarrierFreqListUTRA-FDD-Ext-r12': {'element': {'type': 'CarrierFreqUTRA-FDD-Ext-r12'},
                                                                         'size': [(1,
                                                                                   'maxUTRA-FDD-Carrier')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'CarrierFreqListUTRA-TDD': {'element': {'type': 'CarrierFreqUTRA-TDD'},
                                                                 'size': [(1,
                                                                           'maxUTRA-TDD-Carrier')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'CarrierFreqListUTRA-TDD-Ext-r12': {'element': {'type': 'CarrierFreqUTRA-TDD-r12'},
                                                                         'size': [(1,
                                                                                   'maxUTRA-TDD-Carrier')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'CarrierFreqListUTRA-TDD-r10': {'element': {'type': 'ARFCN-ValueUTRA'},
                                                                     'size': [(1,
                                                                               'maxFreqUTRA-TDD-r10')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'CarrierFreqUTRA-FDD': {'members': [{'name': 'carrierFreq',
                                                                          'type': 'ARFCN-ValueUTRA'},
                                                                         {'name': 'cellReselectionPriority',
                                                                          'optional': True,
                                                                          'type': 'CellReselectionPriority'},
                                                                         {'name': 'threshX-High',
                                                                          'type': 'ReselectionThreshold'},
                                                                         {'name': 'threshX-Low',
                                                                          'type': 'ReselectionThreshold'},
                                                                         {'name': 'q-RxLevMin',
                                                                          'restricted-to': [(-60,
                                                                                             -13)],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'p-MaxUTRA',
                                                                          'restricted-to': [(-50,
                                                                                             33)],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'q-QualMin',
                                                                          'restricted-to': [(-24,
                                                                                             0)],
                                                                          'type': 'INTEGER'},
                                                                         None,
                                                                         [{'members': [{'name': 'threshX-HighQ-r9',
                                                                                        'type': 'ReselectionThresholdQ-r9'},
                                                                                       {'name': 'threshX-LowQ-r9',
                                                                                        'type': 'ReselectionThresholdQ-r9'}],
                                                                           'name': 'threshX-Q-r9',
                                                                           'optional': True,
                                                                           'type': 'SEQUENCE'}]],
                                                             'type': 'SEQUENCE'},
                                     'CarrierFreqUTRA-FDD-Ext-r12': {'members': [{'name': 'carrierFreq-r12',
                                                                                  'type': 'ARFCN-ValueUTRA'},
                                                                                 {'name': 'cellReselectionPriority-r12',
                                                                                  'optional': True,
                                                                                  'type': 'CellReselectionPriority'},
                                                                                 {'name': 'threshX-High-r12',
                                                                                  'type': 'ReselectionThreshold'},
                                                                                 {'name': 'threshX-Low-r12',
                                                                                  'type': 'ReselectionThreshold'},
                                                                                 {'name': 'q-RxLevMin-r12',
                                                                                  'restricted-to': [(-60,
                                                                                                     -13)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'name': 'p-MaxUTRA-r12',
                                                                                  'restricted-to': [(-50,
                                                                                                     33)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'name': 'q-QualMin-r12',
                                                                                  'restricted-to': [(-24,
                                                                                                     0)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'members': [{'name': 'threshX-HighQ-r12',
                                                                                               'type': 'ReselectionThresholdQ-r9'},
                                                                                              {'name': 'threshX-LowQ-r12',
                                                                                               'type': 'ReselectionThresholdQ-r9'}],
                                                                                  'name': 'threshX-Q-r12',
                                                                                  'optional': True,
                                                                                  'type': 'SEQUENCE'},
                                                                                 {'element': {'type': 'FreqBandIndicator-UTRA-FDD'},
                                                                                  'name': 'multiBandInfoList-r12',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            'maxMultiBands')],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'name': 'reducedMeasPerformance-r12',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('true',
                                                                                              0)]},
                                                                                 None],
                                                                     'type': 'SEQUENCE'},
                                     'CarrierFreqUTRA-TDD': {'members': [{'name': 'carrierFreq',
                                                                          'type': 'ARFCN-ValueUTRA'},
                                                                         {'name': 'cellReselectionPriority',
                                                                          'optional': True,
                                                                          'type': 'CellReselectionPriority'},
                                                                         {'name': 'threshX-High',
                                                                          'type': 'ReselectionThreshold'},
                                                                         {'name': 'threshX-Low',
                                                                          'type': 'ReselectionThreshold'},
                                                                         {'name': 'q-RxLevMin',
                                                                          'restricted-to': [(-60,
                                                                                             -13)],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'p-MaxUTRA',
                                                                          'restricted-to': [(-50,
                                                                                             33)],
                                                                          'type': 'INTEGER'},
                                                                         None],
                                                             'type': 'SEQUENCE'},
                                     'CarrierFreqUTRA-TDD-r12': {'members': [{'name': 'carrierFreq-r12',
                                                                              'type': 'ARFCN-ValueUTRA'},
                                                                             {'name': 'cellReselectionPriority-r12',
                                                                              'optional': True,
                                                                              'type': 'CellReselectionPriority'},
                                                                             {'name': 'threshX-High-r12',
                                                                              'type': 'ReselectionThreshold'},
                                                                             {'name': 'threshX-Low-r12',
                                                                              'type': 'ReselectionThreshold'},
                                                                             {'name': 'q-RxLevMin-r12',
                                                                              'restricted-to': [(-60,
                                                                                                 -13)],
                                                                              'type': 'INTEGER'},
                                                                             {'name': 'p-MaxUTRA-r12',
                                                                              'restricted-to': [(-50,
                                                                                                 33)],
                                                                              'type': 'INTEGER'},
                                                                             {'name': 'reducedMeasPerformance-r12',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('true',
                                                                                          0)]},
                                                                             None],
                                                                 'type': 'SEQUENCE'},
                                     'CarrierFreqsGERAN': {'members': [{'name': 'startingARFCN',
                                                                        'type': 'ARFCN-ValueGERAN'},
                                                                       {'name': 'bandIndicator',
                                                                        'type': 'BandIndicatorGERAN'},
                                                                       {'members': [{'name': 'explicitListOfARFCNs',
                                                                                     'type': 'ExplicitListOfARFCNs'},
                                                                                    {'members': [{'name': 'arfcn-Spacing',
                                                                                                  'restricted-to': [(1,
                                                                                                                     8)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'name': 'numberOfFollowingARFCNs',
                                                                                                  'restricted-to': [(0,
                                                                                                                     31)],
                                                                                                  'type': 'INTEGER'}],
                                                                                     'name': 'equallySpacedARFCNs',
                                                                                     'type': 'SEQUENCE'},
                                                                                    {'name': 'variableBitMapOfARFCNs',
                                                                                     'size': [(1,
                                                                                               16)],
                                                                                     'type': 'OCTET '
                                                                                             'STRING'}],
                                                                        'name': 'followingARFCNs',
                                                                        'type': 'CHOICE'}],
                                                           'type': 'SEQUENCE'},
                                     'CarrierFreqsInfoGERAN': {'members': [{'name': 'carrierFreqs',
                                                                            'type': 'CarrierFreqsGERAN'},
                                                                           {'members': [{'name': 'cellReselectionPriority',
                                                                                         'optional': True,
                                                                                         'type': 'CellReselectionPriority'},
                                                                                        {'name': 'ncc-Permitted',
                                                                                         'size': [8],
                                                                                         'type': 'BIT '
                                                                                                 'STRING'},
                                                                                        {'name': 'q-RxLevMin',
                                                                                         'restricted-to': [(0,
                                                                                                            45)],
                                                                                         'type': 'INTEGER'},
                                                                                        {'name': 'p-MaxGERAN',
                                                                                         'optional': True,
                                                                                         'restricted-to': [(0,
                                                                                                            39)],
                                                                                         'type': 'INTEGER'},
                                                                                        {'name': 'threshX-High',
                                                                                         'type': 'ReselectionThreshold'},
                                                                                        {'name': 'threshX-Low',
                                                                                         'type': 'ReselectionThreshold'}],
                                                                            'name': 'commonInfo',
                                                                            'type': 'SEQUENCE'},
                                                                           None],
                                                               'type': 'SEQUENCE'},
                                     'CarrierFreqsInfoListGERAN': {'element': {'type': 'CarrierFreqsInfoGERAN'},
                                                                   'size': [(1,
                                                                             'maxGNFG')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'CellAccessRelatedInfo-r14': {'members': [{'name': 'plmn-IdentityList-r14',
                                                                                'type': 'PLMN-IdentityList'},
                                                                               {'name': 'trackingAreaCode-r14',
                                                                                'type': 'TrackingAreaCode'},
                                                                               {'name': 'cellIdentity-r14',
                                                                                'type': 'CellIdentity'}],
                                                                   'type': 'SEQUENCE'},
                                     'CellChangeOrder': {'members': [{'name': 't304',
                                                                      'type': 'ENUMERATED',
                                                                      'values': [('ms100',
                                                                                  0),
                                                                                 ('ms200',
                                                                                  1),
                                                                                 ('ms500',
                                                                                  2),
                                                                                 ('ms1000',
                                                                                  3),
                                                                                 ('ms2000',
                                                                                  4),
                                                                                 ('ms4000',
                                                                                  5),
                                                                                 ('ms8000',
                                                                                  6),
                                                                                 ('ms10000-v1310',
                                                                                  7)]},
                                                                     {'members': [{'members': [{'name': 'physCellId',
                                                                                                'type': 'PhysCellIdGERAN'},
                                                                                               {'name': 'carrierFreq',
                                                                                                'type': 'CarrierFreqGERAN'},
                                                                                               {'name': 'networkControlOrder',
                                                                                                'optional': True,
                                                                                                'size': [2],
                                                                                                'type': 'BIT '
                                                                                                        'STRING'},
                                                                                               {'name': 'systemInformation',
                                                                                                'optional': True,
                                                                                                'type': 'SI-OrPSI-GERAN'}],
                                                                                   'name': 'geran',
                                                                                   'type': 'SEQUENCE'},
                                                                                  None],
                                                                      'name': 'targetRAT-Type',
                                                                      'type': 'CHOICE'}],
                                                         'type': 'SEQUENCE'},
                                     'CellGlobalIdCDMA2000': {'members': [{'name': 'cellGlobalId1XRTT',
                                                                           'size': [47],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'cellGlobalIdHRPD',
                                                                           'size': [128],
                                                                           'type': 'BIT '
                                                                                   'STRING'}],
                                                              'type': 'CHOICE'},
                                     'CellGlobalIdEUTRA': {'members': [{'name': 'plmn-Identity',
                                                                        'type': 'PLMN-Identity'},
                                                                       {'name': 'cellIdentity',
                                                                        'type': 'CellIdentity'}],
                                                           'type': 'SEQUENCE'},
                                     'CellGlobalIdGERAN': {'members': [{'name': 'plmn-Identity',
                                                                        'type': 'PLMN-Identity'},
                                                                       {'name': 'locationAreaCode',
                                                                        'size': [16],
                                                                        'type': 'BIT '
                                                                                'STRING'},
                                                                       {'name': 'cellIdentity',
                                                                        'size': [16],
                                                                        'type': 'BIT '
                                                                                'STRING'}],
                                                           'type': 'SEQUENCE'},
                                     'CellGlobalIdList-r10': {'element': {'type': 'CellGlobalIdEUTRA'},
                                                              'size': [(1, 32)],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'CellGlobalIdUTRA': {'members': [{'name': 'plmn-Identity',
                                                                       'type': 'PLMN-Identity'},
                                                                      {'name': 'cellIdentity',
                                                                       'size': [28],
                                                                       'type': 'BIT '
                                                                               'STRING'}],
                                                          'type': 'SEQUENCE'},
                                     'CellIdentity': {'size': [28],
                                                      'type': 'BIT STRING'},
                                     'CellIndex': {'restricted-to': [(1,
                                                                      'maxCellMeas')],
                                                   'type': 'INTEGER'},
                                     'CellIndexList': {'element': {'type': 'CellIndex'},
                                                       'size': [(1,
                                                                 'maxCellMeas')],
                                                       'type': 'SEQUENCE OF'},
                                     'CellInfoGERAN-r9': {'members': [{'name': 'physCellId-r9',
                                                                       'type': 'PhysCellIdGERAN'},
                                                                      {'name': 'carrierFreq-r9',
                                                                       'type': 'CarrierFreqGERAN'},
                                                                      {'name': 'systemInformation-r9',
                                                                       'type': 'SystemInfoListGERAN'}],
                                                          'type': 'SEQUENCE'},
                                     'CellInfoListGERAN-r9': {'element': {'type': 'CellInfoGERAN-r9'},
                                                              'size': [(1,
                                                                        'maxCellInfoGERAN-r9')],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'CellInfoListUTRA-FDD-r9': {'element': {'type': 'CellInfoUTRA-FDD-r9'},
                                                                 'size': [(1,
                                                                           'maxCellInfoUTRA-r9')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'CellInfoListUTRA-TDD-r10': {'element': {'type': 'CellInfoUTRA-TDD-r10'},
                                                                  'size': [(1,
                                                                            'maxCellInfoUTRA-r9')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'CellInfoListUTRA-TDD-r9': {'element': {'type': 'CellInfoUTRA-TDD-r9'},
                                                                 'size': [(1,
                                                                           'maxCellInfoUTRA-r9')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'CellInfoUTRA-FDD-r9': {'members': [{'name': 'physCellId-r9',
                                                                          'type': 'PhysCellIdUTRA-FDD'},
                                                                         {'name': 'utra-BCCH-Container-r9',
                                                                          'type': 'OCTET '
                                                                                  'STRING'}],
                                                             'type': 'SEQUENCE'},
                                     'CellInfoUTRA-TDD-r10': {'members': [{'name': 'physCellId-r10',
                                                                           'type': 'PhysCellIdUTRA-TDD'},
                                                                          {'name': 'carrierFreq-r10',
                                                                           'type': 'ARFCN-ValueUTRA'},
                                                                          {'name': 'utra-BCCH-Container-r10',
                                                                           'type': 'OCTET '
                                                                                   'STRING'}],
                                                              'type': 'SEQUENCE'},
                                     'CellInfoUTRA-TDD-r9': {'members': [{'name': 'physCellId-r9',
                                                                          'type': 'PhysCellIdUTRA-TDD'},
                                                                         {'name': 'utra-BCCH-Container-r9',
                                                                          'type': 'OCTET '
                                                                                  'STRING'}],
                                                             'type': 'SEQUENCE'},
                                     'CellReselectionParametersCDMA2000': {'members': [{'name': 'bandClassList',
                                                                                        'type': 'BandClassListCDMA2000'},
                                                                                       {'name': 'neighCellList',
                                                                                        'type': 'NeighCellListCDMA2000'},
                                                                                       {'name': 't-ReselectionCDMA2000',
                                                                                        'type': 'T-Reselection'},
                                                                                       {'name': 't-ReselectionCDMA2000-SF',
                                                                                        'optional': True,
                                                                                        'type': 'SpeedStateScaleFactors'}],
                                                                           'type': 'SEQUENCE'},
                                     'CellReselectionParametersCDMA2000-r11': {'members': [{'name': 'bandClassList',
                                                                                            'type': 'BandClassListCDMA2000'},
                                                                                           {'element': {'type': 'NeighCellCDMA2000-r11'},
                                                                                            'name': 'neighCellList-r11',
                                                                                            'size': [(1,
                                                                                                      16)],
                                                                                            'type': 'SEQUENCE '
                                                                                                    'OF'},
                                                                                           {'name': 't-ReselectionCDMA2000',
                                                                                            'type': 'T-Reselection'},
                                                                                           {'name': 't-ReselectionCDMA2000-SF',
                                                                                            'optional': True,
                                                                                            'type': 'SpeedStateScaleFactors'}],
                                                                               'type': 'SEQUENCE'},
                                     'CellReselectionParametersCDMA2000-v920': {'members': [{'name': 'neighCellList-v920',
                                                                                             'type': 'NeighCellListCDMA2000-v920'}],
                                                                                'type': 'SEQUENCE'},
                                     'CellReselectionPriority': {'restricted-to': [(0,
                                                                                    7)],
                                                                 'type': 'INTEGER'},
                                     'CellReselectionServingFreqInfo-v1310': {'members': [{'name': 'cellReselectionSubPriority-r13',
                                                                                           'type': 'CellReselectionSubPriority-r13'}],
                                                                              'type': 'SEQUENCE'},
                                     'CellReselectionSubPriority-r13': {'type': 'ENUMERATED',
                                                                        'values': [('oDot2',
                                                                                    0),
                                                                                   ('oDot4',
                                                                                    1),
                                                                                   ('oDot6',
                                                                                    2),
                                                                                   ('oDot8',
                                                                                    3)]},
                                     'CellSelectionInfo-v1130': {'members': [{'name': 'q-QualMinWB-r11',
                                                                              'type': 'Q-QualMin-r9'}],
                                                                 'type': 'SEQUENCE'},
                                     'CellSelectionInfo-v1250': {'members': [{'name': 'q-QualMinRSRQ-OnAllSymbols-r12',
                                                                              'type': 'Q-QualMin-r9'}],
                                                                 'type': 'SEQUENCE'},
                                     'CellSelectionInfo-v920': {'members': [{'name': 'q-QualMin-r9',
                                                                             'type': 'Q-QualMin-r9'},
                                                                            {'name': 'q-QualMinOffset-r9',
                                                                             'optional': True,
                                                                             'restricted-to': [(1,
                                                                                                8)],
                                                                             'type': 'INTEGER'}],
                                                                'type': 'SEQUENCE'},
                                     'CellSelectionInfoCE-r13': {'members': [{'name': 'q-RxLevMinCE-r13',
                                                                              'type': 'Q-RxLevMin'},
                                                                             {'name': 'q-QualMinRSRQ-CE-r13',
                                                                              'optional': True,
                                                                              'type': 'Q-QualMin-r9'}],
                                                                 'type': 'SEQUENCE'},
                                     'CellSelectionInfoCE1-r13': {'members': [{'name': 'q-RxLevMinCE1-r13',
                                                                               'type': 'Q-RxLevMin'},
                                                                              {'name': 'q-QualMinRSRQ-CE1-r13',
                                                                               'optional': True,
                                                                               'type': 'Q-QualMin-r9'}],
                                                                  'type': 'SEQUENCE'},
                                     'CellSelectionInfoCE1-v1360': {'members': [{'name': 'delta-RxLevMinCE1-v1360',
                                                                                 'restricted-to': [(-8,
                                                                                                    -1)],
                                                                                 'type': 'INTEGER'}],
                                                                    'type': 'SEQUENCE'},
                                     'CellSelectionInfoNFreq-r13': {'members': [{'name': 'q-RxLevMin-r13',
                                                                                 'type': 'Q-RxLevMin'},
                                                                                {'name': 'q-RxLevMinOffset',
                                                                                 'optional': True,
                                                                                 'restricted-to': [(1,
                                                                                                    8)],
                                                                                 'type': 'INTEGER'},
                                                                                {'name': 'q-Hyst-r13',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('dB0',
                                                                                             0),
                                                                                            ('dB1',
                                                                                             1),
                                                                                            ('dB2',
                                                                                             2),
                                                                                            ('dB3',
                                                                                             3),
                                                                                            ('dB4',
                                                                                             4),
                                                                                            ('dB5',
                                                                                             5),
                                                                                            ('dB6',
                                                                                             6),
                                                                                            ('dB8',
                                                                                             7),
                                                                                            ('dB10',
                                                                                             8),
                                                                                            ('dB12',
                                                                                             9),
                                                                                            ('dB14',
                                                                                             10),
                                                                                            ('dB16',
                                                                                             11),
                                                                                            ('dB18',
                                                                                             12),
                                                                                            ('dB20',
                                                                                             13),
                                                                                            ('dB22',
                                                                                             14),
                                                                                            ('dB24',
                                                                                             15)]},
                                                                                {'name': 'q-RxLevMinReselection-r13',
                                                                                 'type': 'Q-RxLevMin'},
                                                                                {'name': 't-ReselectionEUTRA-r13',
                                                                                 'type': 'T-Reselection'}],
                                                                    'type': 'SEQUENCE'},
                                     'CellsToAddMod': {'members': [{'name': 'cellIndex',
                                                                    'restricted-to': [(1,
                                                                                       'maxCellMeas')],
                                                                    'type': 'INTEGER'},
                                                                   {'name': 'physCellId',
                                                                    'type': 'PhysCellId'},
                                                                   {'name': 'cellIndividualOffset',
                                                                    'type': 'Q-OffsetRange'}],
                                                       'type': 'SEQUENCE'},
                                     'CellsToAddModCDMA2000': {'members': [{'name': 'cellIndex',
                                                                            'restricted-to': [(1,
                                                                                               'maxCellMeas')],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'physCellId',
                                                                            'type': 'PhysCellIdCDMA2000'}],
                                                               'type': 'SEQUENCE'},
                                     'CellsToAddModList': {'element': {'type': 'CellsToAddMod'},
                                                           'size': [(1,
                                                                     'maxCellMeas')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'CellsToAddModListCDMA2000': {'element': {'type': 'CellsToAddModCDMA2000'},
                                                                   'size': [(1,
                                                                             'maxCellMeas')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'CellsToAddModListUTRA-FDD': {'element': {'type': 'CellsToAddModUTRA-FDD'},
                                                                   'size': [(1,
                                                                             'maxCellMeas')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'CellsToAddModListUTRA-TDD': {'element': {'type': 'CellsToAddModUTRA-TDD'},
                                                                   'size': [(1,
                                                                             'maxCellMeas')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'CellsToAddModUTRA-FDD': {'members': [{'name': 'cellIndex',
                                                                            'restricted-to': [(1,
                                                                                               'maxCellMeas')],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'physCellId',
                                                                            'type': 'PhysCellIdUTRA-FDD'}],
                                                               'type': 'SEQUENCE'},
                                     'CellsToAddModUTRA-TDD': {'members': [{'name': 'cellIndex',
                                                                            'restricted-to': [(1,
                                                                                               'maxCellMeas')],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'physCellId',
                                                                            'type': 'PhysCellIdUTRA-TDD'}],
                                                               'type': 'SEQUENCE'},
                                     'CipheringAlgorithm-r12': {'type': 'ENUMERATED',
                                                                'values': [('eea0',
                                                                            0),
                                                                           ('eea1',
                                                                            1),
                                                                           ('eea2',
                                                                            2),
                                                                           ('eea3-v1130',
                                                                            3),
                                                                           ('spare4',
                                                                            4),
                                                                           ('spare3',
                                                                            5),
                                                                           ('spare2',
                                                                            6),
                                                                           ('spare1',
                                                                            7),
                                                                           None]},
                                     'CommonSF-AllocPatternList-r14': {'element': {'type': 'MBSFN-SubframeConfig-v1430'},
                                                                       'size': [(1,
                                                                                 'maxMBSFN-Allocations')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'CommonSF-AllocPatternList-r9': {'element': {'type': 'MBSFN-SubframeConfig'},
                                                                      'size': [(1,
                                                                                'maxMBSFN-Allocations')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'ConnEstFailReport-r11': {'members': [{'name': 'failedCellId-r11',
                                                                            'type': 'CellGlobalIdEUTRA'},
                                                                           {'name': 'locationInfo-r11',
                                                                            'optional': True,
                                                                            'type': 'LocationInfo-r10'},
                                                                           {'members': [{'name': 'rsrpResult-r11',
                                                                                         'type': 'RSRP-Range'},
                                                                                        {'name': 'rsrqResult-r11',
                                                                                         'optional': True,
                                                                                         'type': 'RSRQ-Range'}],
                                                                            'name': 'measResultFailedCell-r11',
                                                                            'type': 'SEQUENCE'},
                                                                           {'members': [{'name': 'measResultListEUTRA-r11',
                                                                                         'optional': True,
                                                                                         'type': 'MeasResultList2EUTRA-r9'},
                                                                                        {'name': 'measResultListUTRA-r11',
                                                                                         'optional': True,
                                                                                         'type': 'MeasResultList2UTRA-r9'},
                                                                                        {'name': 'measResultListGERAN-r11',
                                                                                         'optional': True,
                                                                                         'type': 'MeasResultListGERAN'},
                                                                                        {'name': 'measResultsCDMA2000-r11',
                                                                                         'optional': True,
                                                                                         'type': 'MeasResultList2CDMA2000-r9'}],
                                                                            'name': 'measResultNeighCells-r11',
                                                                            'optional': True,
                                                                            'type': 'SEQUENCE'},
                                                                           {'name': 'numberOfPreamblesSent-r11',
                                                                            'type': 'NumberOfPreamblesSent-r11'},
                                                                           {'name': 'contentionDetected-r11',
                                                                            'type': 'BOOLEAN'},
                                                                           {'name': 'maxTxPowerReached-r11',
                                                                            'type': 'BOOLEAN'},
                                                                           {'name': 'timeSinceFailure-r11',
                                                                            'type': 'TimeSinceFailure-r11'},
                                                                           {'name': 'measResultListEUTRA-v1130',
                                                                            'optional': True,
                                                                            'type': 'MeasResultList2EUTRA-v9e0'},
                                                                           None,
                                                                           [{'name': 'measResultFailedCell-v1250',
                                                                             'optional': True,
                                                                             'type': 'RSRQ-Range-v1250'},
                                                                            {'name': 'failedCellRSRQ-Type-r12',
                                                                             'optional': True,
                                                                             'type': 'RSRQ-Type-r12'},
                                                                            {'name': 'measResultListEUTRA-v1250',
                                                                             'optional': True,
                                                                             'type': 'MeasResultList2EUTRA-v1250'}],
                                                                           [{'name': 'measResultFailedCell-v1360',
                                                                             'optional': True,
                                                                             'type': 'RSRP-Range-v1360'}]],
                                                               'type': 'SEQUENCE'},
                                     'CounterCheck': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                   'type': 'RRC-TransactionIdentifier'},
                                                                  {'members': [{'members': [{'name': 'counterCheck-r8',
                                                                                             'type': 'CounterCheck-r8-IEs'},
                                                                                            {'name': 'spare3',
                                                                                             'type': 'NULL'},
                                                                                            {'name': 'spare2',
                                                                                             'type': 'NULL'},
                                                                                            {'name': 'spare1',
                                                                                             'type': 'NULL'}],
                                                                                'name': 'c1',
                                                                                'type': 'CHOICE'},
                                                                               {'members': [],
                                                                                'name': 'criticalExtensionsFuture',
                                                                                'type': 'SEQUENCE'}],
                                                                   'name': 'criticalExtensions',
                                                                   'type': 'CHOICE'}],
                                                      'type': 'SEQUENCE'},
                                     'CounterCheck-r8-IEs': {'members': [{'name': 'drb-CountMSB-InfoList',
                                                                          'type': 'DRB-CountMSB-InfoList'},
                                                                         {'name': 'nonCriticalExtension',
                                                                          'optional': True,
                                                                          'type': 'CounterCheck-v8a0-IEs'}],
                                                             'type': 'SEQUENCE'},
                                     'CounterCheck-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                            'optional': True,
                                                                            'type': 'OCTET '
                                                                                    'STRING'},
                                                                           {'members': [],
                                                                            'name': 'nonCriticalExtension',
                                                                            'optional': True,
                                                                            'type': 'SEQUENCE'}],
                                                               'type': 'SEQUENCE'},
                                     'CounterCheckResponse': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                           'type': 'RRC-TransactionIdentifier'},
                                                                          {'members': [{'name': 'counterCheckResponse-r8',
                                                                                        'type': 'CounterCheckResponse-r8-IEs'},
                                                                                       {'members': [],
                                                                                        'name': 'criticalExtensionsFuture',
                                                                                        'type': 'SEQUENCE'}],
                                                                           'name': 'criticalExtensions',
                                                                           'type': 'CHOICE'}],
                                                              'type': 'SEQUENCE'},
                                     'CounterCheckResponse-r8-IEs': {'members': [{'name': 'drb-CountInfoList',
                                                                                  'type': 'DRB-CountInfoList'},
                                                                                 {'name': 'nonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'CounterCheckResponse-v8a0-IEs'}],
                                                                     'type': 'SEQUENCE'},
                                     'CounterCheckResponse-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                     'CountingRequestInfo-r10': {'members': [{'name': 'tmgi-r10',
                                                                              'type': 'TMGI-r9'},
                                                                             None],
                                                                 'type': 'SEQUENCE'},
                                     'CountingRequestList-r10': {'element': {'type': 'CountingRequestInfo-r10'},
                                                                 'size': [(1,
                                                                           'maxServiceCount')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'CountingResponseInfo-r10': {'members': [{'name': 'countingResponseService-r10',
                                                                               'restricted-to': [(0,
                                                                                                  'maxServiceCount-1')],
                                                                               'type': 'INTEGER'},
                                                                              None],
                                                                  'type': 'SEQUENCE'},
                                     'CountingResponseList-r10': {'element': {'type': 'CountingResponseInfo-r10'},
                                                                  'size': [(1,
                                                                            'maxServiceCount')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'CrossCarrierSchedulingConfig-r10': {'members': [{'members': [{'members': [{'name': 'cif-Presence-r10',
                                                                                                                 'type': 'BOOLEAN'}],
                                                                                                    'name': 'own-r10',
                                                                                                    'type': 'SEQUENCE'},
                                                                                                   {'members': [{'name': 'schedulingCellId-r10',
                                                                                                                 'type': 'ServCellIndex-r10'},
                                                                                                                {'name': 'pdsch-Start-r10',
                                                                                                                 'restricted-to': [(1,
                                                                                                                                    4)],
                                                                                                                 'type': 'INTEGER'}],
                                                                                                    'name': 'other-r10',
                                                                                                    'type': 'SEQUENCE'}],
                                                                                       'name': 'schedulingCellInfo-r10',
                                                                                       'type': 'CHOICE'}],
                                                                          'type': 'SEQUENCE'},
                                     'CrossCarrierSchedulingConfig-r13': {'members': [{'members': [{'members': [{'name': 'cif-Presence-r13',
                                                                                                                 'type': 'BOOLEAN'}],
                                                                                                    'name': 'own-r13',
                                                                                                    'type': 'SEQUENCE'},
                                                                                                   {'members': [{'name': 'schedulingCellId-r13',
                                                                                                                 'type': 'ServCellIndex-r13'},
                                                                                                                {'name': 'pdsch-Start-r13',
                                                                                                                 'restricted-to': [(1,
                                                                                                                                    4)],
                                                                                                                 'type': 'INTEGER'},
                                                                                                                {'name': 'cif-InSchedulingCell-r13',
                                                                                                                 'restricted-to': [(1,
                                                                                                                                    7)],
                                                                                                                 'type': 'INTEGER'}],
                                                                                                    'name': 'other-r13',
                                                                                                    'type': 'SEQUENCE'}],
                                                                                       'name': 'schedulingCellInfo-r13',
                                                                                       'type': 'CHOICE'}],
                                                                          'type': 'SEQUENCE'},
                                     'CrossCarrierSchedulingConfigLAA-UL-r14': {'members': [{'name': 'schedulingCellId-r14',
                                                                                             'type': 'ServCellIndex-r13'},
                                                                                            {'name': 'cif-InSchedulingCell-r14',
                                                                                             'restricted-to': [(1,
                                                                                                                7)],
                                                                                             'type': 'INTEGER'}],
                                                                                'type': 'SEQUENCE'},
                                     'DC-Parameters-r12': {'members': [{'name': 'drb-TypeSplit-r12',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('supported',
                                                                                    0)]},
                                                                       {'name': 'drb-TypeSCG-r12',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('supported',
                                                                                    0)]}],
                                                           'type': 'SEQUENCE'},
                                     'DC-Parameters-v1310': {'members': [{'name': 'pdcp-TransferSplitUL-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'ue-SSTD-Meas-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'DL-AM-RLC': {'members': [{'name': 't-Reordering',
                                                                'type': 'T-Reordering'},
                                                               {'name': 't-StatusProhibit',
                                                                'type': 'T-StatusProhibit'}],
                                                   'type': 'SEQUENCE'},
                                     'DL-CCCH-Message': {'members': [{'name': 'message',
                                                                      'type': 'DL-CCCH-MessageType'}],
                                                         'type': 'SEQUENCE'},
                                     'DL-CCCH-MessageType': {'members': [{'members': [{'name': 'rrcConnectionReestablishment',
                                                                                       'type': 'RRCConnectionReestablishment'},
                                                                                      {'name': 'rrcConnectionReestablishmentReject',
                                                                                       'type': 'RRCConnectionReestablishmentReject'},
                                                                                      {'name': 'rrcConnectionReject',
                                                                                       'type': 'RRCConnectionReject'},
                                                                                      {'name': 'rrcConnectionSetup',
                                                                                       'type': 'RRCConnectionSetup'}],
                                                                          'name': 'c1',
                                                                          'type': 'CHOICE'},
                                                                         {'members': [],
                                                                          'name': 'messageClassExtension',
                                                                          'type': 'SEQUENCE'}],
                                                             'type': 'CHOICE'},
                                     'DL-DCCH-Message': {'members': [{'name': 'message',
                                                                      'type': 'DL-DCCH-MessageType'}],
                                                         'type': 'SEQUENCE'},
                                     'DL-DCCH-MessageType': {'members': [{'members': [{'name': 'csfbParametersResponseCDMA2000',
                                                                                       'type': 'CSFBParametersResponseCDMA2000'},
                                                                                      {'name': 'dlInformationTransfer',
                                                                                       'type': 'DLInformationTransfer'},
                                                                                      {'name': 'handoverFromEUTRAPreparationRequest',
                                                                                       'type': 'HandoverFromEUTRAPreparationRequest'},
                                                                                      {'name': 'mobilityFromEUTRACommand',
                                                                                       'type': 'MobilityFromEUTRACommand'},
                                                                                      {'name': 'rrcConnectionReconfiguration',
                                                                                       'type': 'RRCConnectionReconfiguration'},
                                                                                      {'name': 'rrcConnectionRelease',
                                                                                       'type': 'RRCConnectionRelease'},
                                                                                      {'name': 'securityModeCommand',
                                                                                       'type': 'SecurityModeCommand'},
                                                                                      {'name': 'ueCapabilityEnquiry',
                                                                                       'type': 'UECapabilityEnquiry'},
                                                                                      {'name': 'counterCheck',
                                                                                       'type': 'CounterCheck'},
                                                                                      {'name': 'ueInformationRequest-r9',
                                                                                       'type': 'UEInformationRequest-r9'},
                                                                                      {'name': 'loggedMeasurementConfiguration-r10',
                                                                                       'type': 'LoggedMeasurementConfiguration-r10'},
                                                                                      {'name': 'rnReconfiguration-r10',
                                                                                       'type': 'RNReconfiguration-r10'},
                                                                                      {'name': 'rrcConnectionResume-r13',
                                                                                       'type': 'RRCConnectionResume-r13'},
                                                                                      {'name': 'spare3',
                                                                                       'type': 'NULL'},
                                                                                      {'name': 'spare2',
                                                                                       'type': 'NULL'},
                                                                                      {'name': 'spare1',
                                                                                       'type': 'NULL'}],
                                                                          'name': 'c1',
                                                                          'type': 'CHOICE'},
                                                                         {'members': [],
                                                                          'name': 'messageClassExtension',
                                                                          'type': 'SEQUENCE'}],
                                                             'type': 'CHOICE'},
                                     'DL-UM-RLC': {'members': [{'name': 'sn-FieldLength',
                                                                'type': 'SN-FieldLength'},
                                                               {'name': 't-Reordering',
                                                                'type': 'T-Reordering'}],
                                                   'type': 'SEQUENCE'},
                                     'DLInformationTransfer': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                            'type': 'RRC-TransactionIdentifier'},
                                                                           {'members': [{'members': [{'name': 'dlInformationTransfer-r8',
                                                                                                      'type': 'DLInformationTransfer-r8-IEs'},
                                                                                                     {'name': 'spare3',
                                                                                                      'type': 'NULL'},
                                                                                                     {'name': 'spare2',
                                                                                                      'type': 'NULL'},
                                                                                                     {'name': 'spare1',
                                                                                                      'type': 'NULL'}],
                                                                                         'name': 'c1',
                                                                                         'type': 'CHOICE'},
                                                                                        {'members': [],
                                                                                         'name': 'criticalExtensionsFuture',
                                                                                         'type': 'SEQUENCE'}],
                                                                            'name': 'criticalExtensions',
                                                                            'type': 'CHOICE'}],
                                                               'type': 'SEQUENCE'},
                                     'DLInformationTransfer-r8-IEs': {'members': [{'members': [{'name': 'dedicatedInfoNAS',
                                                                                                'type': 'DedicatedInfoNAS'},
                                                                                               {'name': 'dedicatedInfoCDMA2000-1XRTT',
                                                                                                'type': 'DedicatedInfoCDMA2000'},
                                                                                               {'name': 'dedicatedInfoCDMA2000-HRPD',
                                                                                                'type': 'DedicatedInfoCDMA2000'}],
                                                                                   'name': 'dedicatedInfoType',
                                                                                   'type': 'CHOICE'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'DLInformationTransfer-v8a0-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'DLInformationTransfer-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'OCTET '
                                                                                             'STRING'},
                                                                                    {'members': [],
                                                                                     'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'}],
                                                                        'type': 'SEQUENCE'},
                                     'DMRS-Config-r11': {'members': [{'name': 'release',
                                                                      'type': 'NULL'},
                                                                     {'members': [{'name': 'scramblingIdentity-r11',
                                                                                   'restricted-to': [(0,
                                                                                                      503)],
                                                                                   'type': 'INTEGER'},
                                                                                  {'name': 'scramblingIdentity2-r11',
                                                                                   'restricted-to': [(0,
                                                                                                      503)],
                                                                                   'type': 'INTEGER'}],
                                                                      'name': 'setup',
                                                                      'type': 'SEQUENCE'}],
                                                         'type': 'CHOICE'},
                                     'DMRS-Config-v1310': {'members': [{'name': 'dmrs-tableAlt-r13',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('true',
                                                                                    0)]}],
                                                           'type': 'SEQUENCE'},
                                     'DRB-CountInfo': {'members': [{'name': 'drb-Identity',
                                                                    'type': 'DRB-Identity'},
                                                                   {'name': 'count-Uplink',
                                                                    'restricted-to': [(0,
                                                                                       4294967295)],
                                                                    'type': 'INTEGER'},
                                                                   {'name': 'count-Downlink',
                                                                    'restricted-to': [(0,
                                                                                       4294967295)],
                                                                    'type': 'INTEGER'}],
                                                       'type': 'SEQUENCE'},
                                     'DRB-CountInfoList': {'element': {'type': 'DRB-CountInfo'},
                                                           'size': [(0,
                                                                     'maxDRB')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'DRB-CountMSB-Info': {'members': [{'name': 'drb-Identity',
                                                                        'type': 'DRB-Identity'},
                                                                       {'name': 'countMSB-Uplink',
                                                                        'restricted-to': [(0,
                                                                                           33554431)],
                                                                        'type': 'INTEGER'},
                                                                       {'name': 'countMSB-Downlink',
                                                                        'restricted-to': [(0,
                                                                                           33554431)],
                                                                        'type': 'INTEGER'}],
                                                           'type': 'SEQUENCE'},
                                     'DRB-CountMSB-InfoList': {'element': {'type': 'DRB-CountMSB-Info'},
                                                               'size': [(1,
                                                                         'maxDRB')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'DRB-Identity': {'restricted-to': [(1,
                                                                         32)],
                                                      'type': 'INTEGER'},
                                     'DRB-ToAddMod': {'members': [{'name': 'eps-BearerIdentity',
                                                                   'optional': True,
                                                                   'restricted-to': [(0,
                                                                                      15)],
                                                                   'type': 'INTEGER'},
                                                                  {'name': 'drb-Identity',
                                                                   'type': 'DRB-Identity'},
                                                                  {'name': 'pdcp-Config',
                                                                   'optional': True,
                                                                   'type': 'PDCP-Config'},
                                                                  {'name': 'rlc-Config',
                                                                   'optional': True,
                                                                   'type': 'RLC-Config'},
                                                                  {'name': 'logicalChannelIdentity',
                                                                   'optional': True,
                                                                   'restricted-to': [(3,
                                                                                      10)],
                                                                   'type': 'INTEGER'},
                                                                  {'name': 'logicalChannelConfig',
                                                                   'optional': True,
                                                                   'type': 'LogicalChannelConfig'},
                                                                  None,
                                                                  [{'name': 'drb-TypeChange-r12',
                                                                    'optional': True,
                                                                    'type': 'ENUMERATED',
                                                                    'values': [('toMCG',
                                                                                0)]},
                                                                   {'name': 'rlc-Config-v1250',
                                                                    'optional': True,
                                                                    'type': 'RLC-Config-v1250'}],
                                                                  [{'name': 'rlc-Config-v1310',
                                                                    'optional': True,
                                                                    'type': 'RLC-Config-v1310'},
                                                                   {'name': 'drb-TypeLWA-r13',
                                                                    'optional': True,
                                                                    'type': 'BOOLEAN'},
                                                                   {'name': 'drb-TypeLWIP-r13',
                                                                    'optional': True,
                                                                    'type': 'ENUMERATED',
                                                                    'values': [('lwip',
                                                                                0),
                                                                               ('lwip-DL-only',
                                                                                1),
                                                                               ('lwip-UL-only',
                                                                                2),
                                                                               ('eutran',
                                                                                3)]}],
                                                                  [{'name': 'rlc-Config-v1430',
                                                                    'optional': True,
                                                                    'type': 'RLC-Config-v1430'},
                                                                   {'name': 'lwip-UL-Aggregation-r14',
                                                                    'optional': True,
                                                                    'type': 'BOOLEAN'},
                                                                   {'name': 'lwip-DL-Aggregation-r14',
                                                                    'optional': True,
                                                                    'type': 'BOOLEAN'},
                                                                   {'name': 'lwa-WLAN-AC-r14',
                                                                    'optional': True,
                                                                    'type': 'ENUMERATED',
                                                                    'values': [('ac-bk',
                                                                                0),
                                                                               ('ac-be',
                                                                                1),
                                                                               ('ac-vi',
                                                                                2),
                                                                               ('ac-vo',
                                                                                3)]}]],
                                                      'type': 'SEQUENCE'},
                                     'DRB-ToAddModList': {'element': {'type': 'DRB-ToAddMod'},
                                                          'size': [(1,
                                                                    'maxDRB')],
                                                          'type': 'SEQUENCE '
                                                                  'OF'},
                                     'DRB-ToAddModListSCG-r12': {'element': {'type': 'DRB-ToAddModSCG-r12'},
                                                                 'size': [(1,
                                                                           'maxDRB')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'DRB-ToAddModSCG-r12': {'members': [{'name': 'drb-Identity-r12',
                                                                          'type': 'DRB-Identity'},
                                                                         {'members': [{'name': 'split-r12',
                                                                                       'type': 'NULL'},
                                                                                      {'members': [{'name': 'eps-BearerIdentity-r12',
                                                                                                    'optional': True,
                                                                                                    'restricted-to': [(0,
                                                                                                                       15)],
                                                                                                    'type': 'INTEGER'},
                                                                                                   {'name': 'pdcp-Config-r12',
                                                                                                    'optional': True,
                                                                                                    'type': 'PDCP-Config'}],
                                                                                       'name': 'scg-r12',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'name': 'drb-Type-r12',
                                                                          'optional': True,
                                                                          'type': 'CHOICE'},
                                                                         {'name': 'rlc-ConfigSCG-r12',
                                                                          'optional': True,
                                                                          'type': 'RLC-Config'},
                                                                         {'name': 'rlc-Config-v1250',
                                                                          'optional': True,
                                                                          'type': 'RLC-Config-v1250'},
                                                                         {'name': 'logicalChannelIdentitySCG-r12',
                                                                          'optional': True,
                                                                          'restricted-to': [(3,
                                                                                             10)],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'logicalChannelConfigSCG-r12',
                                                                          'optional': True,
                                                                          'type': 'LogicalChannelConfig'},
                                                                         None,
                                                                         [{'name': 'rlc-Config-v1430',
                                                                           'optional': True,
                                                                           'type': 'RLC-Config-v1430'}]],
                                                             'type': 'SEQUENCE'},
                                     'DRB-ToReleaseList': {'element': {'type': 'DRB-Identity'},
                                                           'size': [(1,
                                                                     'maxDRB')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'DRX-Config': {'members': [{'name': 'release',
                                                                 'type': 'NULL'},
                                                                {'members': [{'name': 'onDurationTimer',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('psf1',
                                                                                          0),
                                                                                         ('psf2',
                                                                                          1),
                                                                                         ('psf3',
                                                                                          2),
                                                                                         ('psf4',
                                                                                          3),
                                                                                         ('psf5',
                                                                                          4),
                                                                                         ('psf6',
                                                                                          5),
                                                                                         ('psf8',
                                                                                          6),
                                                                                         ('psf10',
                                                                                          7),
                                                                                         ('psf20',
                                                                                          8),
                                                                                         ('psf30',
                                                                                          9),
                                                                                         ('psf40',
                                                                                          10),
                                                                                         ('psf50',
                                                                                          11),
                                                                                         ('psf60',
                                                                                          12),
                                                                                         ('psf80',
                                                                                          13),
                                                                                         ('psf100',
                                                                                          14),
                                                                                         ('psf200',
                                                                                          15)]},
                                                                             {'name': 'drx-InactivityTimer',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('psf1',
                                                                                          0),
                                                                                         ('psf2',
                                                                                          1),
                                                                                         ('psf3',
                                                                                          2),
                                                                                         ('psf4',
                                                                                          3),
                                                                                         ('psf5',
                                                                                          4),
                                                                                         ('psf6',
                                                                                          5),
                                                                                         ('psf8',
                                                                                          6),
                                                                                         ('psf10',
                                                                                          7),
                                                                                         ('psf20',
                                                                                          8),
                                                                                         ('psf30',
                                                                                          9),
                                                                                         ('psf40',
                                                                                          10),
                                                                                         ('psf50',
                                                                                          11),
                                                                                         ('psf60',
                                                                                          12),
                                                                                         ('psf80',
                                                                                          13),
                                                                                         ('psf100',
                                                                                          14),
                                                                                         ('psf200',
                                                                                          15),
                                                                                         ('psf300',
                                                                                          16),
                                                                                         ('psf500',
                                                                                          17),
                                                                                         ('psf750',
                                                                                          18),
                                                                                         ('psf1280',
                                                                                          19),
                                                                                         ('psf1920',
                                                                                          20),
                                                                                         ('psf2560',
                                                                                          21),
                                                                                         ('psf0-v1020',
                                                                                          22),
                                                                                         ('spare9',
                                                                                          23),
                                                                                         ('spare8',
                                                                                          24),
                                                                                         ('spare7',
                                                                                          25),
                                                                                         ('spare6',
                                                                                          26),
                                                                                         ('spare5',
                                                                                          27),
                                                                                         ('spare4',
                                                                                          28),
                                                                                         ('spare3',
                                                                                          29),
                                                                                         ('spare2',
                                                                                          30),
                                                                                         ('spare1',
                                                                                          31)]},
                                                                             {'name': 'drx-RetransmissionTimer',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('psf1',
                                                                                          0),
                                                                                         ('psf2',
                                                                                          1),
                                                                                         ('psf4',
                                                                                          2),
                                                                                         ('psf6',
                                                                                          3),
                                                                                         ('psf8',
                                                                                          4),
                                                                                         ('psf16',
                                                                                          5),
                                                                                         ('psf24',
                                                                                          6),
                                                                                         ('psf33',
                                                                                          7)]},
                                                                             {'members': [{'name': 'sf10',
                                                                                           'restricted-to': [(0,
                                                                                                              9)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf20',
                                                                                           'restricted-to': [(0,
                                                                                                              19)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf32',
                                                                                           'restricted-to': [(0,
                                                                                                              31)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf40',
                                                                                           'restricted-to': [(0,
                                                                                                              39)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf64',
                                                                                           'restricted-to': [(0,
                                                                                                              63)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf80',
                                                                                           'restricted-to': [(0,
                                                                                                              79)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf128',
                                                                                           'restricted-to': [(0,
                                                                                                              127)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf160',
                                                                                           'restricted-to': [(0,
                                                                                                              159)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf256',
                                                                                           'restricted-to': [(0,
                                                                                                              255)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf320',
                                                                                           'restricted-to': [(0,
                                                                                                              319)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf512',
                                                                                           'restricted-to': [(0,
                                                                                                              511)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf640',
                                                                                           'restricted-to': [(0,
                                                                                                              639)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf1024',
                                                                                           'restricted-to': [(0,
                                                                                                              1023)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf1280',
                                                                                           'restricted-to': [(0,
                                                                                                              1279)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf2048',
                                                                                           'restricted-to': [(0,
                                                                                                              2047)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sf2560',
                                                                                           'restricted-to': [(0,
                                                                                                              2559)],
                                                                                           'type': 'INTEGER'}],
                                                                              'name': 'longDRX-CycleStartOffset',
                                                                              'type': 'CHOICE'},
                                                                             {'members': [{'name': 'shortDRX-Cycle',
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('sf2',
                                                                                                       0),
                                                                                                      ('sf5',
                                                                                                       1),
                                                                                                      ('sf8',
                                                                                                       2),
                                                                                                      ('sf10',
                                                                                                       3),
                                                                                                      ('sf16',
                                                                                                       4),
                                                                                                      ('sf20',
                                                                                                       5),
                                                                                                      ('sf32',
                                                                                                       6),
                                                                                                      ('sf40',
                                                                                                       7),
                                                                                                      ('sf64',
                                                                                                       8),
                                                                                                      ('sf80',
                                                                                                       9),
                                                                                                      ('sf128',
                                                                                                       10),
                                                                                                      ('sf160',
                                                                                                       11),
                                                                                                      ('sf256',
                                                                                                       12),
                                                                                                      ('sf320',
                                                                                                       13),
                                                                                                      ('sf512',
                                                                                                       14),
                                                                                                      ('sf640',
                                                                                                       15)]},
                                                                                          {'name': 'drxShortCycleTimer',
                                                                                           'restricted-to': [(1,
                                                                                                              16)],
                                                                                           'type': 'INTEGER'}],
                                                                              'name': 'shortDRX',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'}],
                                                                 'name': 'setup',
                                                                 'type': 'SEQUENCE'}],
                                                    'type': 'CHOICE'},
                                     'DRX-Config-r13': {'members': [{'name': 'onDurationTimer-v1310',
                                                                     'optional': True,
                                                                     'type': 'ENUMERATED',
                                                                     'values': [('psf300',
                                                                                 0),
                                                                                ('psf400',
                                                                                 1),
                                                                                ('psf500',
                                                                                 2),
                                                                                ('psf600',
                                                                                 3),
                                                                                ('psf800',
                                                                                 4),
                                                                                ('psf1000',
                                                                                 5),
                                                                                ('psf1200',
                                                                                 6),
                                                                                ('psf1600',
                                                                                 7)]},
                                                                    {'name': 'drx-RetransmissionTimer-v1310',
                                                                     'optional': True,
                                                                     'type': 'ENUMERATED',
                                                                     'values': [('psf40',
                                                                                 0),
                                                                                ('psf64',
                                                                                 1),
                                                                                ('psf80',
                                                                                 2),
                                                                                ('psf96',
                                                                                 3),
                                                                                ('psf112',
                                                                                 4),
                                                                                ('psf128',
                                                                                 5),
                                                                                ('psf160',
                                                                                 6),
                                                                                ('psf320',
                                                                                 7)]},
                                                                    {'name': 'drx-ULRetransmissionTimer-r13',
                                                                     'optional': True,
                                                                     'type': 'ENUMERATED',
                                                                     'values': [('psf0',
                                                                                 0),
                                                                                ('psf1',
                                                                                 1),
                                                                                ('psf2',
                                                                                 2),
                                                                                ('psf4',
                                                                                 3),
                                                                                ('psf6',
                                                                                 4),
                                                                                ('psf8',
                                                                                 5),
                                                                                ('psf16',
                                                                                 6),
                                                                                ('psf24',
                                                                                 7),
                                                                                ('psf33',
                                                                                 8),
                                                                                ('psf40',
                                                                                 9),
                                                                                ('psf64',
                                                                                 10),
                                                                                ('psf80',
                                                                                 11),
                                                                                ('psf96',
                                                                                 12),
                                                                                ('psf112',
                                                                                 13),
                                                                                ('psf128',
                                                                                 14),
                                                                                ('psf160',
                                                                                 15),
                                                                                ('psf320',
                                                                                 16)]}],
                                                        'type': 'SEQUENCE'},
                                     'DRX-Config-v1130': {'members': [{'name': 'drx-RetransmissionTimer-v1130',
                                                                       'optional': True,
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('psf0-v1130',
                                                                                   0)]},
                                                                      {'members': [{'name': 'sf60-v1130',
                                                                                    'restricted-to': [(0,
                                                                                                       59)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'sf70-v1130',
                                                                                    'restricted-to': [(0,
                                                                                                       69)],
                                                                                    'type': 'INTEGER'}],
                                                                       'name': 'longDRX-CycleStartOffset-v1130',
                                                                       'optional': True,
                                                                       'type': 'CHOICE'},
                                                                      {'name': 'shortDRX-Cycle-v1130',
                                                                       'optional': True,
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('sf4-v1130',
                                                                                   0)]}],
                                                          'type': 'SEQUENCE'},
                                     'DRX-Config-v1310': {'members': [{'members': [{'name': 'sf60-v1310',
                                                                                    'restricted-to': [(0,
                                                                                                       59)],
                                                                                    'type': 'INTEGER'}],
                                                                       'name': 'longDRX-CycleStartOffset-v1310',
                                                                       'optional': True,
                                                                       'type': 'SEQUENCE'}],
                                                          'type': 'SEQUENCE'},
                                     'DataBLER-MCH-Result-r12': {'members': [{'name': 'mch-Index-r12',
                                                                              'restricted-to': [(1,
                                                                                                 'maxPMCH-PerMBSFN')],
                                                                              'type': 'INTEGER'},
                                                                             {'name': 'dataBLER-Result-r12',
                                                                              'type': 'BLER-Result-r12'}],
                                                                 'type': 'SEQUENCE'},
                                     'DataBLER-MCH-ResultList-r12': {'element': {'type': 'DataBLER-MCH-Result-r12'},
                                                                     'size': [(1,
                                                                               'maxPMCH-PerMBSFN')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'DataInactivityTimer-r14': {'type': 'ENUMERATED',
                                                                 'values': [('s1',
                                                                             0),
                                                                            ('s2',
                                                                             1),
                                                                            ('s3',
                                                                             2),
                                                                            ('s5',
                                                                             3),
                                                                            ('s7',
                                                                             4),
                                                                            ('s10',
                                                                             5),
                                                                            ('s15',
                                                                             6),
                                                                            ('s20',
                                                                             7),
                                                                            ('s40',
                                                                             8),
                                                                            ('s50',
                                                                             9),
                                                                            ('s60',
                                                                             10),
                                                                            ('s80',
                                                                             11),
                                                                            ('s100',
                                                                             12),
                                                                            ('s120',
                                                                             13),
                                                                            ('s150',
                                                                             14),
                                                                            ('s180',
                                                                             15)]},
                                     'DedicatedInfoCDMA2000': {'type': 'OCTET '
                                                                       'STRING'},
                                     'DedicatedInfoNAS': {'type': 'OCTET '
                                                                  'STRING'},
                                     'DelayBudgetReport-r14': {'members': [{'name': 'type1',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('msMinus1280',
                                                                                        0),
                                                                                       ('msMinus640',
                                                                                        1),
                                                                                       ('msMinus320',
                                                                                        2),
                                                                                       ('msMinus160',
                                                                                        3),
                                                                                       ('msMinus80',
                                                                                        4),
                                                                                       ('msMinus60',
                                                                                        5),
                                                                                       ('msMinus40',
                                                                                        6),
                                                                                       ('msMinus20',
                                                                                        7),
                                                                                       ('ms0',
                                                                                        8),
                                                                                       ('ms20',
                                                                                        9),
                                                                                       ('ms40',
                                                                                        10),
                                                                                       ('ms60',
                                                                                        11),
                                                                                       ('ms80',
                                                                                        12),
                                                                                       ('ms160',
                                                                                        13),
                                                                                       ('ms320',
                                                                                        14),
                                                                                       ('ms640',
                                                                                        15),
                                                                                       ('ms1280',
                                                                                        16)]},
                                                                           {'name': 'type2',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('msMinus192',
                                                                                        0),
                                                                                       ('msMinus168',
                                                                                        1),
                                                                                       ('msMinus144',
                                                                                        2),
                                                                                       ('msMinus120',
                                                                                        3),
                                                                                       ('msMinus96',
                                                                                        4),
                                                                                       ('msMinus72',
                                                                                        5),
                                                                                       ('msMinus48',
                                                                                        6),
                                                                                       ('msMinus24',
                                                                                        7),
                                                                                       ('ms0',
                                                                                        8),
                                                                                       ('ms24',
                                                                                        9),
                                                                                       ('ms48',
                                                                                        10),
                                                                                       ('ms72',
                                                                                        11),
                                                                                       ('ms96',
                                                                                        12),
                                                                                       ('ms120',
                                                                                        13),
                                                                                       ('ms144',
                                                                                        14),
                                                                                       ('ms168',
                                                                                        15),
                                                                                       ('ms192',
                                                                                        16)]}],
                                                               'type': 'CHOICE'},
                                     'DeltaFList-PUCCH': {'members': [{'name': 'deltaF-PUCCH-Format1',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('deltaF-2',
                                                                                   0),
                                                                                  ('deltaF0',
                                                                                   1),
                                                                                  ('deltaF2',
                                                                                   2)]},
                                                                      {'name': 'deltaF-PUCCH-Format1b',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('deltaF1',
                                                                                   0),
                                                                                  ('deltaF3',
                                                                                   1),
                                                                                  ('deltaF5',
                                                                                   2)]},
                                                                      {'name': 'deltaF-PUCCH-Format2',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('deltaF-2',
                                                                                   0),
                                                                                  ('deltaF0',
                                                                                   1),
                                                                                  ('deltaF1',
                                                                                   2),
                                                                                  ('deltaF2',
                                                                                   3)]},
                                                                      {'name': 'deltaF-PUCCH-Format2a',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('deltaF-2',
                                                                                   0),
                                                                                  ('deltaF0',
                                                                                   1),
                                                                                  ('deltaF2',
                                                                                   2)]},
                                                                      {'name': 'deltaF-PUCCH-Format2b',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('deltaF-2',
                                                                                   0),
                                                                                  ('deltaF0',
                                                                                   1),
                                                                                  ('deltaF2',
                                                                                   2)]}],
                                                          'type': 'SEQUENCE'},
                                     'DeltaTxD-OffsetListPUCCH-r10': {'members': [{'name': 'deltaTxD-OffsetPUCCH-Format1-r10',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('dB0',
                                                                                               0),
                                                                                              ('dB-2',
                                                                                               1)]},
                                                                                  {'name': 'deltaTxD-OffsetPUCCH-Format1a1b-r10',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('dB0',
                                                                                               0),
                                                                                              ('dB-2',
                                                                                               1)]},
                                                                                  {'name': 'deltaTxD-OffsetPUCCH-Format22a2b-r10',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('dB0',
                                                                                               0),
                                                                                              ('dB-2',
                                                                                               1)]},
                                                                                  {'name': 'deltaTxD-OffsetPUCCH-Format3-r10',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('dB0',
                                                                                               0),
                                                                                              ('dB-2',
                                                                                               1)]},
                                                                                  None],
                                                                      'type': 'SEQUENCE'},
                                     'DeltaTxD-OffsetListPUCCH-v1130': {'members': [{'name': 'deltaTxD-OffsetPUCCH-Format1bCS-r11',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('dB0',
                                                                                                 0),
                                                                                                ('dB-1',
                                                                                                 1)]}],
                                                                        'type': 'SEQUENCE'},
                                     'E-CSFB-r9': {'members': [{'name': 'messageContCDMA2000-1XRTT-r9',
                                                                'optional': True,
                                                                'type': 'OCTET '
                                                                        'STRING'},
                                                               {'name': 'mobilityCDMA2000-HRPD-r9',
                                                                'optional': True,
                                                                'type': 'ENUMERATED',
                                                                'values': [('handover',
                                                                            0),
                                                                           ('redirection',
                                                                            1)]},
                                                               {'name': 'messageContCDMA2000-HRPD-r9',
                                                                'optional': True,
                                                                'type': 'OCTET '
                                                                        'STRING'},
                                                               {'name': 'redirectCarrierCDMA2000-HRPD-r9',
                                                                'optional': True,
                                                                'type': 'CarrierFreqCDMA2000'}],
                                                   'type': 'SEQUENCE'},
                                     'EAB-Config-r11': {'members': [{'name': 'eab-Category-r11',
                                                                     'type': 'ENUMERATED',
                                                                     'values': [('a',
                                                                                 0),
                                                                                ('b',
                                                                                 1),
                                                                                ('c',
                                                                                 2)]},
                                                                    {'name': 'eab-BarringBitmap-r11',
                                                                     'size': [10],
                                                                     'type': 'BIT '
                                                                             'STRING'}],
                                                        'type': 'SEQUENCE'},
                                     'EAB-ConfigPLMN-r11': {'members': [{'name': 'eab-Config-r11',
                                                                         'optional': True,
                                                                         'type': 'EAB-Config-r11'}],
                                                            'type': 'SEQUENCE'},
                                     'EIMTA-MainConfig-r12': {'members': [{'name': 'release',
                                                                           'type': 'NULL'},
                                                                          {'members': [{'name': 'eimta-RNTI-r12',
                                                                                        'type': 'C-RNTI'},
                                                                                       {'name': 'eimta-CommandPeriodicity-r12',
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('sf10',
                                                                                                    0),
                                                                                                   ('sf20',
                                                                                                    1),
                                                                                                   ('sf40',
                                                                                                    2),
                                                                                                   ('sf80',
                                                                                                    3)]},
                                                                                       {'name': 'eimta-CommandSubframeSet-r12',
                                                                                        'size': [10],
                                                                                        'type': 'BIT '
                                                                                                'STRING'}],
                                                                           'name': 'setup',
                                                                           'type': 'SEQUENCE'}],
                                                              'type': 'CHOICE'},
                                     'EIMTA-MainConfigServCell-r12': {'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'members': [{'name': 'eimta-UL-DL-ConfigIndex-r12',
                                                                                                'restricted-to': [(1,
                                                                                                                   5)],
                                                                                                'type': 'INTEGER'},
                                                                                               {'name': 'eimta-HARQ-ReferenceConfig-r12',
                                                                                                'type': 'ENUMERATED',
                                                                                                'values': [('sa2',
                                                                                                            0),
                                                                                                           ('sa4',
                                                                                                            1),
                                                                                                           ('sa5',
                                                                                                            2)]},
                                                                                               {'members': [{'name': 'release',
                                                                                                             'type': 'NULL'},
                                                                                                            {'members': [{'name': 'subframeConfigList-r12',
                                                                                                                          'type': 'MBSFN-SubframeConfigList'}],
                                                                                                             'name': 'setup',
                                                                                                             'type': 'SEQUENCE'}],
                                                                                                'name': 'mbsfn-SubframeConfigList-v1250',
                                                                                                'type': 'CHOICE'}],
                                                                                   'name': 'setup',
                                                                                   'type': 'SEQUENCE'}],
                                                                      'type': 'CHOICE'},
                                     'EPDCCH-Config-r11': {'members': [{'members': [{'name': 'release',
                                                                                     'type': 'NULL'},
                                                                                    {'members': [{'members': [{'name': 'release',
                                                                                                               'type': 'NULL'},
                                                                                                              {'members': [{'name': 'subframePattern-r11',
                                                                                                                            'type': 'MeasSubframePattern-r10'}],
                                                                                                               'name': 'setup',
                                                                                                               'type': 'SEQUENCE'}],
                                                                                                  'name': 'subframePatternConfig-r11',
                                                                                                  'optional': True,
                                                                                                  'type': 'CHOICE'},
                                                                                                 {'name': 'startSymbol-r11',
                                                                                                  'optional': True,
                                                                                                  'restricted-to': [(1,
                                                                                                                     4)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'name': 'setConfigToReleaseList-r11',
                                                                                                  'optional': True,
                                                                                                  'type': 'EPDCCH-SetConfigToReleaseList-r11'},
                                                                                                 {'name': 'setConfigToAddModList-r11',
                                                                                                  'optional': True,
                                                                                                  'type': 'EPDCCH-SetConfigToAddModList-r11'}],
                                                                                     'name': 'setup',
                                                                                     'type': 'SEQUENCE'}],
                                                                        'name': 'config-r11',
                                                                        'type': 'CHOICE'}],
                                                           'type': 'SEQUENCE'},
                                     'EPDCCH-SetConfig-r11': {'members': [{'name': 'setConfigId-r11',
                                                                           'type': 'EPDCCH-SetConfigId-r11'},
                                                                          {'name': 'transmissionType-r11',
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('localised',
                                                                                       0),
                                                                                      ('distributed',
                                                                                       1)]},
                                                                          {'members': [{'name': 'numberPRB-Pairs-r11',
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('n2',
                                                                                                    0),
                                                                                                   ('n4',
                                                                                                    1),
                                                                                                   ('n8',
                                                                                                    2)]},
                                                                                       {'name': 'resourceBlockAssignment-r11',
                                                                                        'size': [(4,
                                                                                                  38)],
                                                                                        'type': 'BIT '
                                                                                                'STRING'}],
                                                                           'name': 'resourceBlockAssignment-r11',
                                                                           'type': 'SEQUENCE'},
                                                                          {'name': 'dmrs-ScramblingSequenceInt-r11',
                                                                           'restricted-to': [(0,
                                                                                              503)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'pucch-ResourceStartOffset-r11',
                                                                           'restricted-to': [(0,
                                                                                              2047)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 're-MappingQCL-ConfigId-r11',
                                                                           'optional': True,
                                                                           'type': 'PDSCH-RE-MappingQCL-ConfigId-r11'},
                                                                          None,
                                                                          [{'members': [{'name': 'release',
                                                                                         'type': 'NULL'},
                                                                                        {'name': 'setup',
                                                                                         'type': 'CSI-RS-ConfigZPId-r11'}],
                                                                            'name': 'csi-RS-ConfigZPId2-r12',
                                                                            'optional': True,
                                                                            'type': 'CHOICE'}],
                                                                          [{'members': [{'name': 'release',
                                                                                         'type': 'NULL'},
                                                                                        {'name': 'setup',
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('n6',
                                                                                                     0)]}],
                                                                            'name': 'numberPRB-Pairs-v1310',
                                                                            'optional': True,
                                                                            'type': 'CHOICE'},
                                                                           {'members': [{'name': 'release',
                                                                                         'type': 'NULL'},
                                                                                        {'members': [{'name': 'csi-NumRepetitionCE-r13',
                                                                                                      'type': 'ENUMERATED',
                                                                                                      'values': [('sf1',
                                                                                                                  0),
                                                                                                                 ('sf2',
                                                                                                                  1),
                                                                                                                 ('sf4',
                                                                                                                  2),
                                                                                                                 ('sf8',
                                                                                                                  3),
                                                                                                                 ('sf16',
                                                                                                                  4),
                                                                                                                 ('sf32',
                                                                                                                  5)]},
                                                                                                     {'name': 'mpdcch-pdsch-HoppingConfig-r13',
                                                                                                      'type': 'ENUMERATED',
                                                                                                      'values': [('on',
                                                                                                                  0),
                                                                                                                 ('off',
                                                                                                                  1)]},
                                                                                                     {'members': [{'name': 'fdd-r13',
                                                                                                                   'type': 'ENUMERATED',
                                                                                                                   'values': [('v1',
                                                                                                                               0),
                                                                                                                              ('v1dot5',
                                                                                                                               1),
                                                                                                                              ('v2',
                                                                                                                               2),
                                                                                                                              ('v2dot5',
                                                                                                                               3),
                                                                                                                              ('v4',
                                                                                                                               4),
                                                                                                                              ('v5',
                                                                                                                               5),
                                                                                                                              ('v8',
                                                                                                                               6),
                                                                                                                              ('v10',
                                                                                                                               7)]},
                                                                                                                  {'name': 'tdd-r13',
                                                                                                                   'type': 'ENUMERATED',
                                                                                                                   'values': [('v1',
                                                                                                                               0),
                                                                                                                              ('v2',
                                                                                                                               1),
                                                                                                                              ('v4',
                                                                                                                               2),
                                                                                                                              ('v5',
                                                                                                                               3),
                                                                                                                              ('v8',
                                                                                                                               4),
                                                                                                                              ('v10',
                                                                                                                               5),
                                                                                                                              ('v20',
                                                                                                                               6),
                                                                                                                              ('spare1',
                                                                                                                               7)]}],
                                                                                                      'name': 'mpdcch-StartSF-UESS-r13',
                                                                                                      'type': 'CHOICE'},
                                                                                                     {'name': 'mpdcch-NumRepetition-r13',
                                                                                                      'type': 'ENUMERATED',
                                                                                                      'values': [('r1',
                                                                                                                  0),
                                                                                                                 ('r2',
                                                                                                                  1),
                                                                                                                 ('r4',
                                                                                                                  2),
                                                                                                                 ('r8',
                                                                                                                  3),
                                                                                                                 ('r16',
                                                                                                                  4),
                                                                                                                 ('r32',
                                                                                                                  5),
                                                                                                                 ('r64',
                                                                                                                  6),
                                                                                                                 ('r128',
                                                                                                                  7),
                                                                                                                 ('r256',
                                                                                                                  8)]},
                                                                                                     {'name': 'mpdcch-Narrowband-r13',
                                                                                                      'restricted-to': [(1,
                                                                                                                         'maxAvailNarrowBands-r13')],
                                                                                                      'type': 'INTEGER'}],
                                                                                         'name': 'setup',
                                                                                         'type': 'SEQUENCE'}],
                                                                            'name': 'mpdcch-config-r13',
                                                                            'optional': True,
                                                                            'type': 'CHOICE'}]],
                                                              'type': 'SEQUENCE'},
                                     'EPDCCH-SetConfigId-r11': {'restricted-to': [(0,
                                                                                   1)],
                                                                'type': 'INTEGER'},
                                     'EPDCCH-SetConfigToAddModList-r11': {'element': {'type': 'EPDCCH-SetConfig-r11'},
                                                                          'size': [(1,
                                                                                    'maxEPDCCH-Set-r11')],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                     'EPDCCH-SetConfigToReleaseList-r11': {'element': {'type': 'EPDCCH-SetConfigId-r11'},
                                                                           'size': [(1,
                                                                                     'maxEPDCCH-Set-r11')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'Enable256QAM-r14': {'members': [{'name': 'release',
                                                                       'type': 'NULL'},
                                                                      {'members': [{'members': [{'name': 'subframeSet1-DCI-Format0-r14',
                                                                                                 'type': 'BOOLEAN'},
                                                                                                {'name': 'subframeSet1-DCI-Format4-r14',
                                                                                                 'type': 'BOOLEAN'},
                                                                                                {'name': 'subframeSet2-DCI-Format0-r14',
                                                                                                 'type': 'BOOLEAN'},
                                                                                                {'name': 'subframeSet2-DCI-Format4-r14',
                                                                                                 'type': 'BOOLEAN'}],
                                                                                    'name': 'tpc-SubframeSet-Configured-r14',
                                                                                    'type': 'SEQUENCE'},
                                                                                   {'members': [{'name': 'dci-Format0-r14',
                                                                                                 'type': 'BOOLEAN'},
                                                                                                {'name': 'dci-Format4-r14',
                                                                                                 'type': 'BOOLEAN'}],
                                                                                    'name': 'tpc-SubframeSet-NotConfigured-r14',
                                                                                    'type': 'SEQUENCE'}],
                                                                       'name': 'setup',
                                                                       'type': 'CHOICE'}],
                                                          'type': 'CHOICE'},
                                     'EstablishmentCause': {'type': 'ENUMERATED',
                                                            'values': [('emergency',
                                                                        0),
                                                                       ('highPriorityAccess',
                                                                        1),
                                                                       ('mt-Access',
                                                                        2),
                                                                       ('mo-Signalling',
                                                                        3),
                                                                       ('mo-Data',
                                                                        4),
                                                                       ('delayTolerantAccess-v1020',
                                                                        5),
                                                                       ('mo-VoiceCall-v1280',
                                                                        6),
                                                                       ('spare1',
                                                                        7)]},
                                     'ExplicitListOfARFCNs': {'element': {'type': 'ARFCN-ValueGERAN'},
                                                              'size': [(0, 31)],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'FailureReportSCG-r12': {'members': [{'name': 'failureType-r12',
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('t313-Expiry',
                                                                                       0),
                                                                                      ('randomAccessProblem',
                                                                                       1),
                                                                                      ('rlc-MaxNumRetx',
                                                                                       2),
                                                                                      ('scg-ChangeFailure',
                                                                                       3)]},
                                                                          {'name': 'measResultServFreqList-r12',
                                                                           'optional': True,
                                                                           'type': 'MeasResultServFreqList-r10'},
                                                                          {'name': 'measResultNeighCells-r12',
                                                                           'optional': True,
                                                                           'type': 'MeasResultList2EUTRA-r9'},
                                                                          None,
                                                                          [{'name': 'failureType-v1290',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('maxUL-TimingDiff-v1290',
                                                                                        0)]}],
                                                                          [{'name': 'measResultServFreqListExt-r13',
                                                                            'optional': True,
                                                                            'type': 'MeasResultServFreqListExt-r13'}]],
                                                              'type': 'SEQUENCE'},
                                     'FailureReportSCG-v12d0': {'members': [{'name': 'measResultNeighCells-v12d0',
                                                                             'optional': True,
                                                                             'type': 'MeasResultList2EUTRA-v9e0'}],
                                                                'type': 'SEQUENCE'},
                                     'FeMBMS-Unicast-Parameters-r14': {'members': [{'name': 'unicast-fembmsMixedSCell-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'emptyUnicastRegion-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]}],
                                                                       'type': 'SEQUENCE'},
                                     'FilterCoefficient': {'type': 'ENUMERATED',
                                                           'values': [('fc0',
                                                                       0),
                                                                      ('fc1',
                                                                       1),
                                                                      ('fc2',
                                                                       2),
                                                                      ('fc3',
                                                                       3),
                                                                      ('fc4',
                                                                       4),
                                                                      ('fc5',
                                                                       5),
                                                                      ('fc6',
                                                                       6),
                                                                      ('fc7',
                                                                       7),
                                                                      ('fc8',
                                                                       8),
                                                                      ('fc9',
                                                                       9),
                                                                      ('fc11',
                                                                       10),
                                                                      ('fc13',
                                                                       11),
                                                                      ('fc15',
                                                                       12),
                                                                      ('fc17',
                                                                       13),
                                                                      ('fc19',
                                                                       14),
                                                                      ('spare1',
                                                                       15),
                                                                      None]},
                                     'Format4-resource-r13': {'members': [{'name': 'startingPRB-format4-r13',
                                                                           'restricted-to': [(0,
                                                                                              109)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'numberOfPRB-format4-r13',
                                                                           'restricted-to': [(0,
                                                                                              7)],
                                                                           'type': 'INTEGER'}],
                                                              'type': 'SEQUENCE'},
                                     'Format5-resource-r13': {'members': [{'name': 'startingPRB-format5-r13',
                                                                           'restricted-to': [(0,
                                                                                              109)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'cdm-index-format5-r13',
                                                                           'restricted-to': [(0,
                                                                                              1)],
                                                                           'type': 'INTEGER'}],
                                                              'type': 'SEQUENCE'},
                                     'FreqBandIndicator': {'restricted-to': [(1,
                                                                              'maxFBI')],
                                                           'type': 'INTEGER'},
                                     'FreqBandIndicator-UTRA-FDD': {'restricted-to': [(1,
                                                                                       86)],
                                                                    'type': 'INTEGER'},
                                     'FreqBandIndicator-r11': {'restricted-to': [(1,
                                                                                  'maxFBI2')],
                                                               'type': 'INTEGER'},
                                     'FreqBandIndicator-v9e0': {'restricted-to': [('maxFBI-Plus1',
                                                                                   'maxFBI2')],
                                                                'type': 'INTEGER'},
                                     'FreqBandIndicatorListEUTRA-r12': {'element': {'type': 'FreqBandIndicator-r11'},
                                                                        'size': [(1,
                                                                                  'maxBands')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'FreqHoppingParameters-r13': {'members': [{'name': 'dummy',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('nb2',
                                                                                            0),
                                                                                           ('nb4',
                                                                                            1)]},
                                                                               {'members': [{'name': 'interval-FDD-r13',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('int1',
                                                                                                         0),
                                                                                                        ('int2',
                                                                                                         1),
                                                                                                        ('int4',
                                                                                                         2),
                                                                                                        ('int8',
                                                                                                         3)]},
                                                                                            {'name': 'interval-TDD-r13',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('int1',
                                                                                                         0),
                                                                                                        ('int5',
                                                                                                         1),
                                                                                                        ('int10',
                                                                                                         2),
                                                                                                        ('int20',
                                                                                                         3)]}],
                                                                                'name': 'dummy2',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'},
                                                                               {'members': [{'name': 'interval-FDD-r13',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('int2',
                                                                                                         0),
                                                                                                        ('int4',
                                                                                                         1),
                                                                                                        ('int8',
                                                                                                         2),
                                                                                                        ('int16',
                                                                                                         3)]},
                                                                                            {'name': 'interval-TDD-r13',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('int5',
                                                                                                         0),
                                                                                                        ('int10',
                                                                                                         1),
                                                                                                        ('int20',
                                                                                                         2),
                                                                                                        ('int40',
                                                                                                         3)]}],
                                                                                'name': 'dummy3',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'},
                                                                               {'members': [{'name': 'interval-FDD-r13',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('int1',
                                                                                                         0),
                                                                                                        ('int2',
                                                                                                         1),
                                                                                                        ('int4',
                                                                                                         2),
                                                                                                        ('int8',
                                                                                                         3)]},
                                                                                            {'name': 'interval-TDD-r13',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('int1',
                                                                                                         0),
                                                                                                        ('int5',
                                                                                                         1),
                                                                                                        ('int10',
                                                                                                         2),
                                                                                                        ('int20',
                                                                                                         3)]}],
                                                                                'name': 'interval-ULHoppingConfigCommonModeA-r13',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'},
                                                                               {'members': [{'name': 'interval-FDD-r13',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('int2',
                                                                                                         0),
                                                                                                        ('int4',
                                                                                                         1),
                                                                                                        ('int8',
                                                                                                         2),
                                                                                                        ('int16',
                                                                                                         3)]},
                                                                                            {'name': 'interval-TDD-r13',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('int5',
                                                                                                         0),
                                                                                                        ('int10',
                                                                                                         1),
                                                                                                        ('int20',
                                                                                                         2),
                                                                                                        ('int40',
                                                                                                         3)]}],
                                                                                'name': 'interval-ULHoppingConfigCommonModeB-r13',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'},
                                                                               {'name': 'dummy4',
                                                                                'optional': True,
                                                                                'restricted-to': [(1,
                                                                                                   'maxAvailNarrowBands-r13')],
                                                                                'type': 'INTEGER'}],
                                                                   'type': 'SEQUENCE'},
                                     'FreqPriorityEUTRA': {'members': [{'name': 'carrierFreq',
                                                                        'type': 'ARFCN-ValueEUTRA'},
                                                                       {'name': 'cellReselectionPriority',
                                                                        'type': 'CellReselectionPriority'}],
                                                           'type': 'SEQUENCE'},
                                     'FreqPriorityEUTRA-r12': {'members': [{'name': 'carrierFreq-r12',
                                                                            'type': 'ARFCN-ValueEUTRA-r9'},
                                                                           {'name': 'cellReselectionPriority-r12',
                                                                            'type': 'CellReselectionPriority'}],
                                                               'type': 'SEQUENCE'},
                                     'FreqPriorityEUTRA-v1310': {'members': [{'name': 'cellReselectionSubPriority-r13',
                                                                              'optional': True,
                                                                              'type': 'CellReselectionSubPriority-r13'}],
                                                                 'type': 'SEQUENCE'},
                                     'FreqPriorityEUTRA-v9e0': {'members': [{'name': 'carrierFreq-v9e0',
                                                                             'optional': True,
                                                                             'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                                'type': 'SEQUENCE'},
                                     'FreqPriorityListEUTRA': {'element': {'type': 'FreqPriorityEUTRA'},
                                                               'size': [(1,
                                                                         'maxFreq')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'FreqPriorityListEUTRA-v1310': {'element': {'type': 'FreqPriorityEUTRA-v1310'},
                                                                     'size': [(1,
                                                                               'maxFreq')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'FreqPriorityListExtEUTRA-r12': {'element': {'type': 'FreqPriorityEUTRA-r12'},
                                                                      'size': [(1,
                                                                                'maxFreq')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'FreqPriorityListExtEUTRA-v1310': {'element': {'type': 'FreqPriorityEUTRA-v1310'},
                                                                        'size': [(1,
                                                                                  'maxFreq')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'FreqPriorityListUTRA-FDD': {'element': {'type': 'FreqPriorityUTRA-FDD'},
                                                                  'size': [(1,
                                                                            'maxUTRA-FDD-Carrier')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'FreqPriorityListUTRA-TDD': {'element': {'type': 'FreqPriorityUTRA-TDD'},
                                                                  'size': [(1,
                                                                            'maxUTRA-TDD-Carrier')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'FreqPriorityUTRA-FDD': {'members': [{'name': 'carrierFreq',
                                                                           'type': 'ARFCN-ValueUTRA'},
                                                                          {'name': 'cellReselectionPriority',
                                                                           'type': 'CellReselectionPriority'}],
                                                              'type': 'SEQUENCE'},
                                     'FreqPriorityUTRA-TDD': {'members': [{'name': 'carrierFreq',
                                                                           'type': 'ARFCN-ValueUTRA'},
                                                                          {'name': 'cellReselectionPriority',
                                                                           'type': 'CellReselectionPriority'}],
                                                              'type': 'SEQUENCE'},
                                     'FreqsPriorityGERAN': {'members': [{'name': 'carrierFreqs',
                                                                         'type': 'CarrierFreqsGERAN'},
                                                                        {'name': 'cellReselectionPriority',
                                                                         'type': 'CellReselectionPriority'}],
                                                            'type': 'SEQUENCE'},
                                     'FreqsPriorityListGERAN': {'element': {'type': 'FreqsPriorityGERAN'},
                                                                'size': [(1,
                                                                          'maxGNFG')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'Handover': {'members': [{'name': 'targetRAT-Type',
                                                               'type': 'ENUMERATED',
                                                               'values': [('utra',
                                                                           0),
                                                                          ('geran',
                                                                           1),
                                                                          ('cdma2000-1XRTT',
                                                                           2),
                                                                          ('cdma2000-HRPD',
                                                                           3),
                                                                          ('spare4',
                                                                           4),
                                                                          ('spare3',
                                                                           5),
                                                                          ('spare2',
                                                                           6),
                                                                          ('spare1',
                                                                           7),
                                                                          None]},
                                                              {'name': 'targetRAT-MessageContainer',
                                                               'type': 'OCTET '
                                                                       'STRING'},
                                                              {'name': 'nas-SecurityParamFromEUTRA',
                                                               'optional': True,
                                                               'size': [1],
                                                               'type': 'OCTET '
                                                                       'STRING'},
                                                              {'name': 'systemInformation',
                                                               'optional': True,
                                                               'type': 'SI-OrPSI-GERAN'}],
                                                  'type': 'SEQUENCE'},
                                     'HandoverFromEUTRAPreparationRequest': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                          'type': 'RRC-TransactionIdentifier'},
                                                                                         {'members': [{'members': [{'name': 'handoverFromEUTRAPreparationRequest-r8',
                                                                                                                    'type': 'HandoverFromEUTRAPreparationRequest-r8-IEs'},
                                                                                                                   {'name': 'spare3',
                                                                                                                    'type': 'NULL'},
                                                                                                                   {'name': 'spare2',
                                                                                                                    'type': 'NULL'},
                                                                                                                   {'name': 'spare1',
                                                                                                                    'type': 'NULL'}],
                                                                                                       'name': 'c1',
                                                                                                       'type': 'CHOICE'},
                                                                                                      {'members': [],
                                                                                                       'name': 'criticalExtensionsFuture',
                                                                                                       'type': 'SEQUENCE'}],
                                                                                          'name': 'criticalExtensions',
                                                                                          'type': 'CHOICE'}],
                                                                             'type': 'SEQUENCE'},
                                     'HandoverFromEUTRAPreparationRequest-r8-IEs': {'members': [{'name': 'cdma2000-Type',
                                                                                                 'type': 'CDMA2000-Type'},
                                                                                                {'name': 'rand',
                                                                                                 'optional': True,
                                                                                                 'type': 'RAND-CDMA2000'},
                                                                                                {'name': 'mobilityParameters',
                                                                                                 'optional': True,
                                                                                                 'type': 'MobilityParametersCDMA2000'},
                                                                                                {'name': 'nonCriticalExtension',
                                                                                                 'optional': True,
                                                                                                 'type': 'HandoverFromEUTRAPreparationRequest-v890-IEs'}],
                                                                                    'type': 'SEQUENCE'},
                                     'HandoverFromEUTRAPreparationRequest-v1020-IEs': {'members': [{'name': 'dualRxTxRedirectIndicator-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'ENUMERATED',
                                                                                                    'values': [('true',
                                                                                                                0)]},
                                                                                                   {'name': 'redirectCarrierCDMA2000-1XRTT-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'CarrierFreqCDMA2000'},
                                                                                                   {'members': [],
                                                                                                    'name': 'nonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'SEQUENCE'}],
                                                                                       'type': 'SEQUENCE'},
                                     'HandoverFromEUTRAPreparationRequest-v890-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                                   'optional': True,
                                                                                                   'type': 'OCTET '
                                                                                                           'STRING'},
                                                                                                  {'name': 'nonCriticalExtension',
                                                                                                   'optional': True,
                                                                                                   'type': 'HandoverFromEUTRAPreparationRequest-v920-IEs'}],
                                                                                      'type': 'SEQUENCE'},
                                     'HandoverFromEUTRAPreparationRequest-v920-IEs': {'members': [{'name': 'concurrPrepCDMA2000-HRPD-r9',
                                                                                                   'optional': True,
                                                                                                   'type': 'BOOLEAN'},
                                                                                                  {'name': 'nonCriticalExtension',
                                                                                                   'optional': True,
                                                                                                   'type': 'HandoverFromEUTRAPreparationRequest-v1020-IEs'}],
                                                                                      'type': 'SEQUENCE'},
                                     'HighSpeedConfig-r14': {'members': [{'name': 'highSpeedEnhancedMeasFlag-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('true',
                                                                                      0)]},
                                                                         {'name': 'highSpeedEnhancedDemodulationFlag-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('true',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'HighSpeedConfigSCell-r14': {'members': [{'name': 'highSpeedEnhancedDemodulationFlag-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('true',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'HighSpeedEnhParameters-r14': {'members': [{'name': 'measurementEnhancements-r14',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('supported',
                                                                                             0)]},
                                                                                {'name': 'demodulationEnhancements-r14',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('supported',
                                                                                             0)]},
                                                                                {'name': 'prach-Enhancements-r14',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('supported',
                                                                                             0)]}],
                                                                    'type': 'SEQUENCE'},
                                     'Hysteresis': {'restricted-to': [(0, 30)],
                                                    'type': 'INTEGER'},
                                     'IDC-Config-r11': {'members': [{'name': 'idc-Indication-r11',
                                                                     'optional': True,
                                                                     'type': 'ENUMERATED',
                                                                     'values': [('setup',
                                                                                 0)]},
                                                                    {'members': [{'name': 'autonomousDenialSubframes-r11',
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('n2',
                                                                                              0),
                                                                                             ('n5',
                                                                                              1),
                                                                                             ('n10',
                                                                                              2),
                                                                                             ('n15',
                                                                                              3),
                                                                                             ('n20',
                                                                                              4),
                                                                                             ('n30',
                                                                                              5),
                                                                                             ('spare2',
                                                                                              6),
                                                                                             ('spare1',
                                                                                              7)]},
                                                                                 {'name': 'autonomousDenialValidity-r11',
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('sf200',
                                                                                              0),
                                                                                             ('sf500',
                                                                                              1),
                                                                                             ('sf1000',
                                                                                              2),
                                                                                             ('sf2000',
                                                                                              3),
                                                                                             ('spare4',
                                                                                              4),
                                                                                             ('spare3',
                                                                                              5),
                                                                                             ('spare2',
                                                                                              6),
                                                                                             ('spare1',
                                                                                              7)]}],
                                                                     'name': 'autonomousDenialParameters-r11',
                                                                     'optional': True,
                                                                     'type': 'SEQUENCE'},
                                                                    None,
                                                                    [{'name': 'idc-Indication-UL-CA-r11',
                                                                      'optional': True,
                                                                      'type': 'ENUMERATED',
                                                                      'values': [('setup',
                                                                                  0)]}],
                                                                    [{'name': 'idc-HardwareSharingIndication-r13',
                                                                      'optional': True,
                                                                      'type': 'ENUMERATED',
                                                                      'values': [('setup',
                                                                                  0)]}]],
                                                        'type': 'SEQUENCE'},
                                     'IDC-SubframePattern-r11': {'members': [{'name': 'subframePatternFDD-r11',
                                                                              'size': [4],
                                                                              'type': 'BIT '
                                                                                      'STRING'},
                                                                             {'members': [{'name': 'subframeConfig0-r11',
                                                                                           'size': [70],
                                                                                           'type': 'BIT '
                                                                                                   'STRING'},
                                                                                          {'name': 'subframeConfig1-5-r11',
                                                                                           'size': [10],
                                                                                           'type': 'BIT '
                                                                                                   'STRING'},
                                                                                          {'name': 'subframeConfig6-r11',
                                                                                           'size': [60],
                                                                                           'type': 'BIT '
                                                                                                   'STRING'}],
                                                                              'name': 'subframePatternTDD-r11',
                                                                              'type': 'CHOICE'},
                                                                             None],
                                                                 'type': 'CHOICE'},
                                     'IDC-SubframePatternList-r11': {'element': {'type': 'IDC-SubframePattern-r11'},
                                                                     'size': [(1,
                                                                               'maxSubframePatternIDC-r11')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'IKE-Identity-r13': {'members': [{'name': 'idI-r13',
                                                                       'type': 'OCTET '
                                                                               'STRING'}],
                                                          'type': 'SEQUENCE'},
                                     'IMSI': {'element': {'type': 'IMSI-Digit'},
                                              'size': [(6, 21)],
                                              'type': 'SEQUENCE OF'},
                                     'IMSI-Digit': {'restricted-to': [(0, 9)],
                                                    'type': 'INTEGER'},
                                     'IP-Address-r13': {'members': [{'name': 'ipv4-r13',
                                                                     'size': [32],
                                                                     'type': 'BIT '
                                                                             'STRING'},
                                                                    {'name': 'ipv6-r13',
                                                                     'size': [128],
                                                                     'type': 'BIT '
                                                                             'STRING'}],
                                                        'type': 'CHOICE'},
                                     'IRAT-ParametersCDMA2000-1XRTT': {'members': [{'name': 'supportedBandList1XRTT',
                                                                                    'type': 'SupportedBandList1XRTT'},
                                                                                   {'name': 'tx-Config1XRTT',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('single',
                                                                                                0),
                                                                                               ('dual',
                                                                                                1)]},
                                                                                   {'name': 'rx-Config1XRTT',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('single',
                                                                                                0),
                                                                                               ('dual',
                                                                                                1)]}],
                                                                       'type': 'SEQUENCE'},
                                     'IRAT-ParametersCDMA2000-1XRTT-v1020': {'members': [{'name': 'e-CSFB-dual-1XRTT-r10',
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('supported',
                                                                                                      0)]}],
                                                                             'type': 'SEQUENCE'},
                                     'IRAT-ParametersCDMA2000-1XRTT-v920': {'members': [{'name': 'e-CSFB-1XRTT-r9',
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('supported',
                                                                                                     0)]},
                                                                                        {'name': 'e-CSFB-ConcPS-Mob1XRTT-r9',
                                                                                         'optional': True,
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('supported',
                                                                                                     0)]}],
                                                                            'type': 'SEQUENCE'},
                                     'IRAT-ParametersCDMA2000-HRPD': {'members': [{'name': 'supportedBandListHRPD',
                                                                                   'type': 'SupportedBandListHRPD'},
                                                                                  {'name': 'tx-ConfigHRPD',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('single',
                                                                                               0),
                                                                                              ('dual',
                                                                                               1)]},
                                                                                  {'name': 'rx-ConfigHRPD',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('single',
                                                                                               0),
                                                                                              ('dual',
                                                                                               1)]}],
                                                                      'type': 'SEQUENCE'},
                                     'IRAT-ParametersCDMA2000-v1130': {'members': [{'name': 'cdma2000-NW-Sharing-r11',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]}],
                                                                       'type': 'SEQUENCE'},
                                     'IRAT-ParametersGERAN': {'members': [{'name': 'supportedBandListGERAN',
                                                                           'type': 'SupportedBandListGERAN'},
                                                                          {'name': 'interRAT-PS-HO-ToGERAN',
                                                                           'type': 'BOOLEAN'}],
                                                              'type': 'SEQUENCE'},
                                     'IRAT-ParametersGERAN-v920': {'members': [{'name': 'dtm-r9',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('supported',
                                                                                            0)]},
                                                                               {'name': 'e-RedirectionGERAN-r9',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('supported',
                                                                                            0)]}],
                                                                   'type': 'SEQUENCE'},
                                     'IRAT-ParametersUTRA-FDD': {'members': [{'name': 'supportedBandListUTRA-FDD',
                                                                              'type': 'SupportedBandListUTRA-FDD'}],
                                                                 'type': 'SEQUENCE'},
                                     'IRAT-ParametersUTRA-TDD-v1020': {'members': [{'name': 'e-RedirectionUTRA-TDD-r10',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]}],
                                                                       'type': 'SEQUENCE'},
                                     'IRAT-ParametersUTRA-TDD128': {'members': [{'name': 'supportedBandListUTRA-TDD128',
                                                                                 'type': 'SupportedBandListUTRA-TDD128'}],
                                                                    'type': 'SEQUENCE'},
                                     'IRAT-ParametersUTRA-TDD384': {'members': [{'name': 'supportedBandListUTRA-TDD384',
                                                                                 'type': 'SupportedBandListUTRA-TDD384'}],
                                                                    'type': 'SEQUENCE'},
                                     'IRAT-ParametersUTRA-TDD768': {'members': [{'name': 'supportedBandListUTRA-TDD768',
                                                                                 'type': 'SupportedBandListUTRA-TDD768'}],
                                                                    'type': 'SEQUENCE'},
                                     'IRAT-ParametersUTRA-v920': {'members': [{'name': 'e-RedirectionUTRA-r9',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'IRAT-ParametersUTRA-v9c0': {'members': [{'name': 'voiceOverPS-HS-UTRA-FDD-r9',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'voiceOverPS-HS-UTRA-TDD128-r9',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'srvcc-FromUTRA-FDD-ToUTRA-FDD-r9',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'srvcc-FromUTRA-FDD-ToGERAN-r9',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'srvcc-FromUTRA-TDD128-ToUTRA-TDD128-r9',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'srvcc-FromUTRA-TDD128-ToGERAN-r9',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'IRAT-ParametersUTRA-v9h0': {'members': [{'name': 'mfbi-UTRA-r9',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'IRAT-ParametersWLAN-r13': {'members': [{'element': {'type': 'WLAN-BandIndicator-r13'},
                                                                              'name': 'supportedBandListWLAN-r13',
                                                                              'optional': True,
                                                                              'size': [(1,
                                                                                        'maxWLAN-Bands-r13')],
                                                                              'type': 'SEQUENCE '
                                                                                      'OF'}],
                                                                 'type': 'SEQUENCE'},
                                     'IdleModeMobilityControlInfo': {'members': [{'name': 'freqPriorityListEUTRA',
                                                                                  'optional': True,
                                                                                  'type': 'FreqPriorityListEUTRA'},
                                                                                 {'name': 'freqPriorityListGERAN',
                                                                                  'optional': True,
                                                                                  'type': 'FreqsPriorityListGERAN'},
                                                                                 {'name': 'freqPriorityListUTRA-FDD',
                                                                                  'optional': True,
                                                                                  'type': 'FreqPriorityListUTRA-FDD'},
                                                                                 {'name': 'freqPriorityListUTRA-TDD',
                                                                                  'optional': True,
                                                                                  'type': 'FreqPriorityListUTRA-TDD'},
                                                                                 {'name': 'bandClassPriorityListHRPD',
                                                                                  'optional': True,
                                                                                  'type': 'BandClassPriorityListHRPD'},
                                                                                 {'name': 'bandClassPriorityList1XRTT',
                                                                                  'optional': True,
                                                                                  'type': 'BandClassPriorityList1XRTT'},
                                                                                 {'name': 't320',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('min5',
                                                                                              0),
                                                                                             ('min10',
                                                                                              1),
                                                                                             ('min20',
                                                                                              2),
                                                                                             ('min30',
                                                                                              3),
                                                                                             ('min60',
                                                                                              4),
                                                                                             ('min120',
                                                                                              5),
                                                                                             ('min180',
                                                                                              6),
                                                                                             ('spare1',
                                                                                              7)]},
                                                                                 None,
                                                                                 [{'name': 'freqPriorityListExtEUTRA-r12',
                                                                                   'optional': True,
                                                                                   'type': 'FreqPriorityListExtEUTRA-r12'}],
                                                                                 [{'name': 'freqPriorityListEUTRA-v1310',
                                                                                   'optional': True,
                                                                                   'type': 'FreqPriorityListEUTRA-v1310'},
                                                                                  {'name': 'freqPriorityListExtEUTRA-v1310',
                                                                                   'optional': True,
                                                                                   'type': 'FreqPriorityListExtEUTRA-v1310'}]],
                                                                     'type': 'SEQUENCE'},
                                     'IdleModeMobilityControlInfo-v9e0': {'members': [{'element': {'type': 'FreqPriorityEUTRA-v9e0'},
                                                                                       'name': 'freqPriorityListEUTRA-v9e0',
                                                                                       'size': [(1,
                                                                                                 'maxFreq')],
                                                                                       'type': 'SEQUENCE '
                                                                                               'OF'}],
                                                                          'type': 'SEQUENCE'},
                                     'InDeviceCoexIndication-r11': {'members': [{'members': [{'members': [{'name': 'inDeviceCoexIndication-r11',
                                                                                                           'type': 'InDeviceCoexIndication-r11-IEs'},
                                                                                                          {'name': 'spare3',
                                                                                                           'type': 'NULL'},
                                                                                                          {'name': 'spare2',
                                                                                                           'type': 'NULL'},
                                                                                                          {'name': 'spare1',
                                                                                                           'type': 'NULL'}],
                                                                                              'name': 'c1',
                                                                                              'type': 'CHOICE'},
                                                                                             {'members': [],
                                                                                              'name': 'criticalExtensionsFuture',
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'name': 'criticalExtensions',
                                                                                 'type': 'CHOICE'}],
                                                                    'type': 'SEQUENCE'},
                                     'InDeviceCoexIndication-r11-IEs': {'members': [{'name': 'affectedCarrierFreqList-r11',
                                                                                     'optional': True,
                                                                                     'type': 'AffectedCarrierFreqList-r11'},
                                                                                    {'name': 'tdm-AssistanceInfo-r11',
                                                                                     'optional': True,
                                                                                     'type': 'TDM-AssistanceInfo-r11'},
                                                                                    {'name': 'lateNonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'OCTET '
                                                                                             'STRING'},
                                                                                    {'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'InDeviceCoexIndication-v11d0-IEs'}],
                                                                        'type': 'SEQUENCE'},
                                     'InDeviceCoexIndication-v11d0-IEs': {'members': [{'members': [{'name': 'affectedCarrierFreqCombList-r11',
                                                                                                    'optional': True,
                                                                                                    'type': 'AffectedCarrierFreqCombList-r11'},
                                                                                                   {'name': 'victimSystemType-r11',
                                                                                                    'type': 'VictimSystemType-r11'}],
                                                                                       'name': 'ul-CA-AssistanceInfo-r11',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'},
                                                                                      {'name': 'nonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'InDeviceCoexIndication-v1310-IEs'}],
                                                                          'type': 'SEQUENCE'},
                                     'InDeviceCoexIndication-v1310-IEs': {'members': [{'name': 'affectedCarrierFreqList-v1310',
                                                                                       'optional': True,
                                                                                       'type': 'AffectedCarrierFreqList-v1310'},
                                                                                      {'name': 'affectedCarrierFreqCombList-r13',
                                                                                       'optional': True,
                                                                                       'type': 'AffectedCarrierFreqCombList-r13'},
                                                                                      {'name': 'nonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'InDeviceCoexIndication-v1360-IEs'}],
                                                                          'type': 'SEQUENCE'},
                                     'InDeviceCoexIndication-v1360-IEs': {'members': [{'name': 'hardwareSharingProblem-r13',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('true',
                                                                                                   0)]},
                                                                                      {'members': [],
                                                                                       'name': 'nonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'}],
                                                                          'type': 'SEQUENCE'},
                                     'InitialUE-Identity': {'members': [{'name': 's-TMSI',
                                                                         'type': 'S-TMSI'},
                                                                        {'name': 'randomValue',
                                                                         'size': [40],
                                                                         'type': 'BIT '
                                                                                 'STRING'}],
                                                            'type': 'CHOICE'},
                                     'InterFreqBandInfo': {'members': [{'name': 'interFreqNeedForGaps',
                                                                        'type': 'BOOLEAN'}],
                                                           'type': 'SEQUENCE'},
                                     'InterFreqBandList': {'element': {'type': 'InterFreqBandInfo'},
                                                           'size': [(1,
                                                                     'maxBands')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'InterFreqBlackCellList': {'element': {'type': 'PhysCellIdRange'},
                                                                'size': [(1,
                                                                          'maxCellBlack')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'InterFreqCarrierFreqInfo': {'members': [{'name': 'dl-CarrierFreq',
                                                                               'type': 'ARFCN-ValueEUTRA'},
                                                                              {'name': 'q-RxLevMin',
                                                                               'type': 'Q-RxLevMin'},
                                                                              {'name': 'p-Max',
                                                                               'optional': True,
                                                                               'type': 'P-Max'},
                                                                              {'name': 't-ReselectionEUTRA',
                                                                               'type': 'T-Reselection'},
                                                                              {'name': 't-ReselectionEUTRA-SF',
                                                                               'optional': True,
                                                                               'type': 'SpeedStateScaleFactors'},
                                                                              {'name': 'threshX-High',
                                                                               'type': 'ReselectionThreshold'},
                                                                              {'name': 'threshX-Low',
                                                                               'type': 'ReselectionThreshold'},
                                                                              {'name': 'allowedMeasBandwidth',
                                                                               'type': 'AllowedMeasBandwidth'},
                                                                              {'name': 'presenceAntennaPort1',
                                                                               'type': 'PresenceAntennaPort1'},
                                                                              {'name': 'cellReselectionPriority',
                                                                               'optional': True,
                                                                               'type': 'CellReselectionPriority'},
                                                                              {'name': 'neighCellConfig',
                                                                               'type': 'NeighCellConfig'},
                                                                              {'default': 'dB0',
                                                                               'name': 'q-OffsetFreq',
                                                                               'type': 'Q-OffsetRange'},
                                                                              {'name': 'interFreqNeighCellList',
                                                                               'optional': True,
                                                                               'type': 'InterFreqNeighCellList'},
                                                                              {'name': 'interFreqBlackCellList',
                                                                               'optional': True,
                                                                               'type': 'InterFreqBlackCellList'},
                                                                              None,
                                                                              [{'name': 'q-QualMin-r9',
                                                                                'optional': True,
                                                                                'type': 'Q-QualMin-r9'},
                                                                               {'members': [{'name': 'threshX-HighQ-r9',
                                                                                             'type': 'ReselectionThresholdQ-r9'},
                                                                                            {'name': 'threshX-LowQ-r9',
                                                                                             'type': 'ReselectionThresholdQ-r9'}],
                                                                                'name': 'threshX-Q-r9',
                                                                                'optional': True,
                                                                                'type': 'SEQUENCE'}],
                                                                              [{'name': 'q-QualMinWB-r11',
                                                                                'optional': True,
                                                                                'type': 'Q-QualMin-r9'}]],
                                                                  'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqInfo-r12': {'members': [{'name': 'dl-CarrierFreq-r12',
                                                                                   'type': 'ARFCN-ValueEUTRA-r9'},
                                                                                  {'name': 'q-RxLevMin-r12',
                                                                                   'type': 'Q-RxLevMin'},
                                                                                  {'name': 'p-Max-r12',
                                                                                   'optional': True,
                                                                                   'type': 'P-Max'},
                                                                                  {'name': 't-ReselectionEUTRA-r12',
                                                                                   'type': 'T-Reselection'},
                                                                                  {'name': 't-ReselectionEUTRA-SF-r12',
                                                                                   'optional': True,
                                                                                   'type': 'SpeedStateScaleFactors'},
                                                                                  {'name': 'threshX-High-r12',
                                                                                   'type': 'ReselectionThreshold'},
                                                                                  {'name': 'threshX-Low-r12',
                                                                                   'type': 'ReselectionThreshold'},
                                                                                  {'name': 'allowedMeasBandwidth-r12',
                                                                                   'type': 'AllowedMeasBandwidth'},
                                                                                  {'name': 'presenceAntennaPort1-r12',
                                                                                   'type': 'PresenceAntennaPort1'},
                                                                                  {'name': 'cellReselectionPriority-r12',
                                                                                   'optional': True,
                                                                                   'type': 'CellReselectionPriority'},
                                                                                  {'name': 'neighCellConfig-r12',
                                                                                   'type': 'NeighCellConfig'},
                                                                                  {'default': 'dB0',
                                                                                   'name': 'q-OffsetFreq-r12',
                                                                                   'type': 'Q-OffsetRange'},
                                                                                  {'name': 'interFreqNeighCellList-r12',
                                                                                   'optional': True,
                                                                                   'type': 'InterFreqNeighCellList'},
                                                                                  {'name': 'interFreqBlackCellList-r12',
                                                                                   'optional': True,
                                                                                   'type': 'InterFreqBlackCellList'},
                                                                                  {'name': 'q-QualMin-r12',
                                                                                   'optional': True,
                                                                                   'type': 'Q-QualMin-r9'},
                                                                                  {'members': [{'name': 'threshX-HighQ-r12',
                                                                                                'type': 'ReselectionThresholdQ-r9'},
                                                                                               {'name': 'threshX-LowQ-r12',
                                                                                                'type': 'ReselectionThresholdQ-r9'}],
                                                                                   'name': 'threshX-Q-r12',
                                                                                   'optional': True,
                                                                                   'type': 'SEQUENCE'},
                                                                                  {'name': 'q-QualMinWB-r12',
                                                                                   'optional': True,
                                                                                   'type': 'Q-QualMin-r9'},
                                                                                  {'name': 'multiBandInfoList-r12',
                                                                                   'optional': True,
                                                                                   'type': 'MultiBandInfoList-r11'},
                                                                                  {'name': 'reducedMeasPerformance-r12',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]},
                                                                                  {'name': 'q-QualMinRSRQ-OnAllSymbols-r12',
                                                                                   'optional': True,
                                                                                   'type': 'Q-QualMin-r9'},
                                                                                  None],
                                                                      'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqInfo-v10j0': {'members': [{'name': 'freqBandInfo-r10',
                                                                                     'optional': True,
                                                                                     'type': 'NS-PmaxList-r10'},
                                                                                    {'name': 'multiBandInfoList-v10j0',
                                                                                     'optional': True,
                                                                                     'type': 'MultiBandInfoList-v10j0'}],
                                                                        'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqInfo-v10l0': {'members': [{'name': 'freqBandInfo-v10l0',
                                                                                     'optional': True,
                                                                                     'type': 'NS-PmaxList-v10l0'},
                                                                                    {'name': 'multiBandInfoList-v10l0',
                                                                                     'optional': True,
                                                                                     'type': 'MultiBandInfoList-v10l0'}],
                                                                        'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqInfo-v1250': {'members': [{'name': 'reducedMeasPerformance-r12',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('true',
                                                                                                 0)]},
                                                                                    {'name': 'q-QualMinRSRQ-OnAllSymbols-r12',
                                                                                     'optional': True,
                                                                                     'type': 'Q-QualMin-r9'}],
                                                                        'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqInfo-v1310': {'members': [{'name': 'cellReselectionSubPriority-r13',
                                                                                     'optional': True,
                                                                                     'type': 'CellReselectionSubPriority-r13'},
                                                                                    {'name': 'redistributionInterFreqInfo-r13',
                                                                                     'optional': True,
                                                                                     'type': 'RedistributionInterFreqInfo-r13'},
                                                                                    {'name': 'cellSelectionInfoCE-r13',
                                                                                     'optional': True,
                                                                                     'type': 'CellSelectionInfoCE-r13'},
                                                                                    {'name': 't-ReselectionEUTRA-CE-r13',
                                                                                     'optional': True,
                                                                                     'type': 'T-ReselectionEUTRA-CE-r13'}],
                                                                        'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqInfo-v1350': {'members': [{'name': 'cellSelectionInfoCE1-r13',
                                                                                     'optional': True,
                                                                                     'type': 'CellSelectionInfoCE1-r13'}],
                                                                        'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqInfo-v1360': {'members': [{'name': 'cellSelectionInfoCE1-v1360',
                                                                                     'optional': True,
                                                                                     'type': 'CellSelectionInfoCE1-v1360'}],
                                                                        'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqInfo-v8h0': {'members': [{'name': 'multiBandInfoList',
                                                                                    'optional': True,
                                                                                    'type': 'MultiBandInfoList'}],
                                                                       'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqInfo-v9e0': {'members': [{'name': 'dl-CarrierFreq-v9e0',
                                                                                    'optional': True,
                                                                                    'type': 'ARFCN-ValueEUTRA-v9e0'},
                                                                                   {'name': 'multiBandInfoList-v9e0',
                                                                                    'optional': True,
                                                                                    'type': 'MultiBandInfoList-v9e0'}],
                                                                       'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqList': {'element': {'type': 'InterFreqCarrierFreqInfo'},
                                                                  'size': [(1,
                                                                            'maxFreq')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'InterFreqCarrierFreqList-v1250': {'element': {'type': 'InterFreqCarrierFreqInfo-v1250'},
                                                                        'size': [(1,
                                                                                  'maxFreq')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'InterFreqCarrierFreqList-v1310': {'element': {'type': 'InterFreqCarrierFreqInfo-v1310'},
                                                                        'size': [(1,
                                                                                  'maxFreq')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'InterFreqCarrierFreqList-v1350': {'element': {'type': 'InterFreqCarrierFreqInfo-v1350'},
                                                                        'size': [(1,
                                                                                  'maxFreq')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'InterFreqCarrierFreqListExt-r12': {'element': {'type': 'InterFreqCarrierFreqInfo-r12'},
                                                                         'size': [(1,
                                                                                   'maxFreq')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'InterFreqCarrierFreqListExt-v1280': {'element': {'type': 'InterFreqCarrierFreqInfo-v10j0'},
                                                                           'size': [(1,
                                                                                     'maxFreq')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'InterFreqCarrierFreqListExt-v1310': {'element': {'type': 'InterFreqCarrierFreqInfo-v1310'},
                                                                           'size': [(1,
                                                                                     'maxFreq')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'InterFreqCarrierFreqListExt-v1350': {'element': {'type': 'InterFreqCarrierFreqInfo-v1350'},
                                                                           'size': [(1,
                                                                                     'maxFreq')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'InterFreqCarrierFreqListExt-v1360': {'element': {'type': 'InterFreqCarrierFreqInfo-v1360'},
                                                                           'size': [(1,
                                                                                     'maxFreq')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'InterFreqNeighCellInfo': {'members': [{'name': 'physCellId',
                                                                             'type': 'PhysCellId'},
                                                                            {'name': 'q-OffsetCell',
                                                                             'type': 'Q-OffsetRange'}],
                                                                'type': 'SEQUENCE'},
                                     'InterFreqNeighCellList': {'element': {'type': 'InterFreqNeighCellInfo'},
                                                                'size': [(1,
                                                                          'maxCellInter')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'InterFreqRSTDMeasurementIndication-r10': {'members': [{'members': [{'members': [{'name': 'interFreqRSTDMeasurementIndication-r10',
                                                                                                                       'type': 'InterFreqRSTDMeasurementIndication-r10-IEs'},
                                                                                                                      {'name': 'spare3',
                                                                                                                       'type': 'NULL'},
                                                                                                                      {'name': 'spare2',
                                                                                                                       'type': 'NULL'},
                                                                                                                      {'name': 'spare1',
                                                                                                                       'type': 'NULL'}],
                                                                                                          'name': 'c1',
                                                                                                          'type': 'CHOICE'},
                                                                                                         {'members': [],
                                                                                                          'name': 'criticalExtensionsFuture',
                                                                                                          'type': 'SEQUENCE'}],
                                                                                             'name': 'criticalExtensions',
                                                                                             'type': 'CHOICE'}],
                                                                                'type': 'SEQUENCE'},
                                     'InterFreqRSTDMeasurementIndication-r10-IEs': {'members': [{'members': [{'members': [{'name': 'rstd-InterFreqInfoList-r10',
                                                                                                                           'type': 'RSTD-InterFreqInfoList-r10'}],
                                                                                                              'name': 'start',
                                                                                                              'type': 'SEQUENCE'},
                                                                                                             {'name': 'stop',
                                                                                                              'type': 'NULL'}],
                                                                                                 'name': 'rstd-InterFreqIndication-r10',
                                                                                                 'type': 'CHOICE'},
                                                                                                {'name': 'lateNonCriticalExtension',
                                                                                                 'optional': True,
                                                                                                 'type': 'OCTET '
                                                                                                         'STRING'},
                                                                                                {'members': [],
                                                                                                 'name': 'nonCriticalExtension',
                                                                                                 'optional': True,
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'type': 'SEQUENCE'},
                                     'InterRAT-BandInfo': {'members': [{'name': 'interRAT-NeedForGaps',
                                                                        'type': 'BOOLEAN'}],
                                                           'type': 'SEQUENCE'},
                                     'InterRAT-BandList': {'element': {'type': 'InterRAT-BandInfo'},
                                                           'size': [(1,
                                                                     'maxBands')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'IntraBandContiguousCC-Info-r12': {'members': [{'name': 'fourLayerTM3-TM4-perCC-r12',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('supported',
                                                                                                 0)]},
                                                                                    {'name': 'supportedMIMO-CapabilityDL-r12',
                                                                                     'optional': True,
                                                                                     'type': 'MIMO-CapabilityDL-r10'},
                                                                                    {'name': 'supportedCSI-Proc-r12',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('n1',
                                                                                                 0),
                                                                                                ('n3',
                                                                                                 1),
                                                                                                ('n4',
                                                                                                 2)]}],
                                                                        'type': 'SEQUENCE'},
                                     'IntraFreqBlackCellList': {'element': {'type': 'PhysCellIdRange'},
                                                                'size': [(1,
                                                                          'maxCellBlack')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'IntraFreqNeighCellInfo': {'members': [{'name': 'physCellId',
                                                                             'type': 'PhysCellId'},
                                                                            {'name': 'q-OffsetCell',
                                                                             'type': 'Q-OffsetRange'},
                                                                            None],
                                                                'type': 'SEQUENCE'},
                                     'IntraFreqNeighCellList': {'element': {'type': 'IntraFreqNeighCellInfo'},
                                                                'size': [(1,
                                                                          'maxCellIntra')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'LAA-Parameters-r13': {'members': [{'name': 'crossCarrierSchedulingLAA-DL-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]},
                                                                        {'name': 'csi-RS-DRS-RRM-MeasurementsLAA-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]},
                                                                        {'name': 'downlinkLAA-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]},
                                                                        {'name': 'endingDwPTS-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]},
                                                                        {'name': 'secondSlotStartingPosition-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]},
                                                                        {'name': 'tm9-LAA-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]},
                                                                        {'name': 'tm10-LAA-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]}],
                                                            'type': 'SEQUENCE'},
                                     'LAA-Parameters-v1430': {'members': [{'name': 'crossCarrierSchedulingLAA-UL-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'uplinkLAA-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'twoStepSchedulingTimingInfo-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('nPlus1',
                                                                                       0),
                                                                                      ('nPlus2',
                                                                                       1),
                                                                                      ('nPlus3',
                                                                                       2)]},
                                                                          {'name': 'uss-BlindDecodingAdjustment-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'uss-BlindDecodingReduction-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'outOfSequenceGrantHandling-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'LAA-SCellConfiguration-r13': {'members': [{'name': 'subframeStartPosition-r13',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('s0',
                                                                                             0),
                                                                                            ('s07',
                                                                                             1)]},
                                                                                {'name': 'laa-SCellSubframeConfig-r13',
                                                                                 'size': [8],
                                                                                 'type': 'BIT '
                                                                                         'STRING'}],
                                                                    'type': 'SEQUENCE'},
                                     'LAA-SCellConfiguration-v1430': {'members': [{'members': [{'name': 'release',
                                                                                                'type': 'NULL'},
                                                                                               {'members': [{'name': 'crossCarrierSchedulingConfigLAA-UL-r14',
                                                                                                             'type': 'CrossCarrierSchedulingConfigLAA-UL-r14'}],
                                                                                                'name': 'setup',
                                                                                                'type': 'SEQUENCE'}],
                                                                                   'name': 'crossCarrierSchedulingConfig-UL-r14',
                                                                                   'optional': True,
                                                                                   'type': 'CHOICE'},
                                                                                  {'name': 'lbt-Config-r14',
                                                                                   'optional': True,
                                                                                   'type': 'LBT-Config-r14'},
                                                                                  {'name': 'pdcch-ConfigLAA-r14',
                                                                                   'optional': True,
                                                                                   'type': 'PDCCH-ConfigLAA-r14'},
                                                                                  {'name': 'absenceOfAnyOtherTechnology-r14',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]},
                                                                                  {'name': 'soundingRS-UL-ConfigDedicatedAperiodic-v1430',
                                                                                   'optional': True,
                                                                                   'type': 'SoundingRS-UL-ConfigDedicatedAperiodic-v1430'}],
                                                                      'type': 'SEQUENCE'},
                                     'LBT-Config-r14': {'members': [{'name': 'maxEnergyDetectionThreshold-r14',
                                                                     'restricted-to': [(-85,
                                                                                        -52)],
                                                                     'type': 'INTEGER'},
                                                                    {'name': 'energyDetectionThresholdOffset-r14',
                                                                     'restricted-to': [(-13,
                                                                                        20)],
                                                                     'type': 'INTEGER'}],
                                                        'type': 'CHOICE'},
                                     'LWA-Config-r13': {'members': [{'name': 'lwa-MobilityConfig-r13',
                                                                     'optional': True,
                                                                     'type': 'WLAN-MobilityConfig-r13'},
                                                                    {'name': 'lwa-WT-Counter-r13',
                                                                     'optional': True,
                                                                     'restricted-to': [(0,
                                                                                        65535)],
                                                                     'type': 'INTEGER'},
                                                                    None,
                                                                    [{'name': 'wt-MAC-Address-r14',
                                                                      'optional': True,
                                                                      'size': [6],
                                                                      'type': 'OCTET '
                                                                              'STRING'}]],
                                                        'type': 'SEQUENCE'},
                                     'LWA-Configuration-r13': {'members': [{'name': 'release',
                                                                            'type': 'NULL'},
                                                                           {'members': [{'name': 'lwa-Config-r13',
                                                                                         'type': 'LWA-Config-r13'}],
                                                                            'name': 'setup',
                                                                            'type': 'SEQUENCE'}],
                                                               'type': 'CHOICE'},
                                     'LWA-Parameters-r13': {'members': [{'name': 'lwa-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]},
                                                                        {'name': 'lwa-SplitBearer-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]},
                                                                        {'name': 'wlan-MAC-Address-r13',
                                                                         'optional': True,
                                                                         'size': [6],
                                                                         'type': 'OCTET '
                                                                                 'STRING'},
                                                                        {'name': 'lwa-BufferSize-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]}],
                                                            'type': 'SEQUENCE'},
                                     'LWA-Parameters-v1430': {'members': [{'name': 'lwa-HO-WithoutWT-Change-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'lwa-UL-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'wlan-PeriodicMeas-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'wlan-ReportAnyWLAN-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'wlan-SupportedDataRate-r14',
                                                                           'optional': True,
                                                                           'restricted-to': [(1,
                                                                                              2048)],
                                                                           'type': 'INTEGER'}],
                                                              'type': 'SEQUENCE'},
                                     'LWA-Parameters-v1440': {'members': [{'name': 'lwa-RLC-UM-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'LWIP-Config-r13': {'members': [{'name': 'lwip-MobilityConfig-r13',
                                                                      'optional': True,
                                                                      'type': 'WLAN-MobilityConfig-r13'},
                                                                     {'name': 'tunnelConfigLWIP-r13',
                                                                      'optional': True,
                                                                      'type': 'TunnelConfigLWIP-r13'},
                                                                     None],
                                                         'type': 'SEQUENCE'},
                                     'LWIP-Configuration-r13': {'members': [{'name': 'release',
                                                                             'type': 'NULL'},
                                                                            {'members': [{'name': 'lwip-Config-r13',
                                                                                          'type': 'LWIP-Config-r13'}],
                                                                             'name': 'setup',
                                                                             'type': 'SEQUENCE'}],
                                                                'type': 'CHOICE'},
                                     'LWIP-Parameters-r13': {'members': [{'name': 'lwip-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'LWIP-Parameters-v1430': {'members': [{'name': 'lwip-Aggregation-DL-r14',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]},
                                                                           {'name': 'lwip-Aggregation-UL-r14',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]}],
                                                               'type': 'SEQUENCE'},
                                     'LocationInfo-r10': {'members': [{'members': [{'name': 'ellipsoid-Point-r10',
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'ellipsoidPointWithAltitude-r10',
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   None,
                                                                                   {'name': 'ellipsoidPointWithUncertaintyCircle-r11',
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'ellipsoidPointWithUncertaintyEllipse-r11',
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'ellipsoidPointWithAltitudeAndUncertaintyEllipsoid-r11',
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'ellipsoidArc-r11',
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'polygon-r11',
                                                                                    'type': 'OCTET '
                                                                                            'STRING'}],
                                                                       'name': 'locationCoordinates-r10',
                                                                       'type': 'CHOICE'},
                                                                      {'name': 'horizontalVelocity-r10',
                                                                       'optional': True,
                                                                       'type': 'OCTET '
                                                                               'STRING'},
                                                                      {'name': 'gnss-TOD-msec-r10',
                                                                       'optional': True,
                                                                       'type': 'OCTET '
                                                                               'STRING'},
                                                                      None],
                                                          'type': 'SEQUENCE'},
                                     'LogMeasInfo-r10': {'members': [{'name': 'locationInfo-r10',
                                                                      'optional': True,
                                                                      'type': 'LocationInfo-r10'},
                                                                     {'name': 'relativeTimeStamp-r10',
                                                                      'restricted-to': [(0,
                                                                                         7200)],
                                                                      'type': 'INTEGER'},
                                                                     {'name': 'servCellIdentity-r10',
                                                                      'type': 'CellGlobalIdEUTRA'},
                                                                     {'members': [{'name': 'rsrpResult-r10',
                                                                                   'type': 'RSRP-Range'},
                                                                                  {'name': 'rsrqResult-r10',
                                                                                   'type': 'RSRQ-Range'}],
                                                                      'name': 'measResultServCell-r10',
                                                                      'type': 'SEQUENCE'},
                                                                     {'members': [{'name': 'measResultListEUTRA-r10',
                                                                                   'optional': True,
                                                                                   'type': 'MeasResultList2EUTRA-r9'},
                                                                                  {'name': 'measResultListUTRA-r10',
                                                                                   'optional': True,
                                                                                   'type': 'MeasResultList2UTRA-r9'},
                                                                                  {'name': 'measResultListGERAN-r10',
                                                                                   'optional': True,
                                                                                   'type': 'MeasResultList2GERAN-r10'},
                                                                                  {'name': 'measResultListCDMA2000-r10',
                                                                                   'optional': True,
                                                                                   'type': 'MeasResultList2CDMA2000-r9'}],
                                                                      'name': 'measResultNeighCells-r10',
                                                                      'optional': True,
                                                                      'type': 'SEQUENCE'},
                                                                     None,
                                                                     [{'name': 'measResultListEUTRA-v1090',
                                                                       'optional': True,
                                                                       'type': 'MeasResultList2EUTRA-v9e0'}],
                                                                     [{'name': 'measResultListMBSFN-r12',
                                                                       'optional': True,
                                                                       'type': 'MeasResultListMBSFN-r12'},
                                                                      {'name': 'measResultServCell-v1250',
                                                                       'optional': True,
                                                                       'type': 'RSRQ-Range-v1250'},
                                                                      {'name': 'servCellRSRQ-Type-r12',
                                                                       'optional': True,
                                                                       'type': 'RSRQ-Type-r12'},
                                                                      {'name': 'measResultListEUTRA-v1250',
                                                                       'optional': True,
                                                                       'type': 'MeasResultList2EUTRA-v1250'}],
                                                                     [{'name': 'inDeviceCoexDetected-r13',
                                                                       'optional': True,
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('true',
                                                                                   0)]}],
                                                                     [{'name': 'measResultServCell-v1360',
                                                                       'optional': True,
                                                                       'type': 'RSRP-Range-v1360'}]],
                                                         'type': 'SEQUENCE'},
                                     'LogMeasInfoList-r10': {'element': {'type': 'LogMeasInfo-r10'},
                                                             'size': [(1,
                                                                       'maxLogMeasReport-r10')],
                                                             'type': 'SEQUENCE '
                                                                     'OF'},
                                     'LogMeasReport-r10': {'members': [{'name': 'absoluteTimeStamp-r10',
                                                                        'type': 'AbsoluteTimeInfo-r10'},
                                                                       {'name': 'traceReference-r10',
                                                                        'type': 'TraceReference-r10'},
                                                                       {'name': 'traceRecordingSessionRef-r10',
                                                                        'size': [2],
                                                                        'type': 'OCTET '
                                                                                'STRING'},
                                                                       {'name': 'tce-Id-r10',
                                                                        'size': [1],
                                                                        'type': 'OCTET '
                                                                                'STRING'},
                                                                       {'name': 'logMeasInfoList-r10',
                                                                        'type': 'LogMeasInfoList-r10'},
                                                                       {'name': 'logMeasAvailable-r10',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('true',
                                                                                    0)]},
                                                                       None],
                                                           'type': 'SEQUENCE'},
                                     'LoggedMeasurementConfiguration-r10': {'members': [{'members': [{'members': [{'name': 'loggedMeasurementConfiguration-r10',
                                                                                                                   'type': 'LoggedMeasurementConfiguration-r10-IEs'},
                                                                                                                  {'name': 'spare3',
                                                                                                                   'type': 'NULL'},
                                                                                                                  {'name': 'spare2',
                                                                                                                   'type': 'NULL'},
                                                                                                                  {'name': 'spare1',
                                                                                                                   'type': 'NULL'}],
                                                                                                      'name': 'c1',
                                                                                                      'type': 'CHOICE'},
                                                                                                     {'members': [],
                                                                                                      'name': 'criticalExtensionsFuture',
                                                                                                      'type': 'SEQUENCE'}],
                                                                                         'name': 'criticalExtensions',
                                                                                         'type': 'CHOICE'}],
                                                                            'type': 'SEQUENCE'},
                                     'LoggedMeasurementConfiguration-r10-IEs': {'members': [{'name': 'traceReference-r10',
                                                                                             'type': 'TraceReference-r10'},
                                                                                            {'name': 'traceRecordingSessionRef-r10',
                                                                                             'size': [2],
                                                                                             'type': 'OCTET '
                                                                                                     'STRING'},
                                                                                            {'name': 'tce-Id-r10',
                                                                                             'size': [1],
                                                                                             'type': 'OCTET '
                                                                                                     'STRING'},
                                                                                            {'name': 'absoluteTimeInfo-r10',
                                                                                             'type': 'AbsoluteTimeInfo-r10'},
                                                                                            {'name': 'areaConfiguration-r10',
                                                                                             'optional': True,
                                                                                             'type': 'AreaConfiguration-r10'},
                                                                                            {'name': 'loggingDuration-r10',
                                                                                             'type': 'LoggingDuration-r10'},
                                                                                            {'name': 'loggingInterval-r10',
                                                                                             'type': 'LoggingInterval-r10'},
                                                                                            {'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'LoggedMeasurementConfiguration-v1080-IEs'}],
                                                                                'type': 'SEQUENCE'},
                                     'LoggedMeasurementConfiguration-v1080-IEs': {'members': [{'name': 'lateNonCriticalExtension-r10',
                                                                                               'optional': True,
                                                                                               'type': 'OCTET '
                                                                                                       'STRING'},
                                                                                              {'name': 'nonCriticalExtension',
                                                                                               'optional': True,
                                                                                               'type': 'LoggedMeasurementConfiguration-v1130-IEs'}],
                                                                                  'type': 'SEQUENCE'},
                                     'LoggedMeasurementConfiguration-v1130-IEs': {'members': [{'name': 'plmn-IdentityList-r11',
                                                                                               'optional': True,
                                                                                               'type': 'PLMN-IdentityList3-r11'},
                                                                                              {'name': 'areaConfiguration-v1130',
                                                                                               'optional': True,
                                                                                               'type': 'AreaConfiguration-v1130'},
                                                                                              {'name': 'nonCriticalExtension',
                                                                                               'optional': True,
                                                                                               'type': 'LoggedMeasurementConfiguration-v1250-IEs'}],
                                                                                  'type': 'SEQUENCE'},
                                     'LoggedMeasurementConfiguration-v1250-IEs': {'members': [{'name': 'targetMBSFN-AreaList-r12',
                                                                                               'optional': True,
                                                                                               'type': 'TargetMBSFN-AreaList-r12'},
                                                                                              {'members': [],
                                                                                               'name': 'nonCriticalExtension',
                                                                                               'optional': True,
                                                                                               'type': 'SEQUENCE'}],
                                                                                  'type': 'SEQUENCE'},
                                     'LoggingDuration-r10': {'type': 'ENUMERATED',
                                                             'values': [('min10',
                                                                         0),
                                                                        ('min20',
                                                                         1),
                                                                        ('min40',
                                                                         2),
                                                                        ('min60',
                                                                         3),
                                                                        ('min90',
                                                                         4),
                                                                        ('min120',
                                                                         5),
                                                                        ('spare2',
                                                                         6),
                                                                        ('spare1',
                                                                         7)]},
                                     'LoggingInterval-r10': {'type': 'ENUMERATED',
                                                             'values': [('ms1280',
                                                                         0),
                                                                        ('ms2560',
                                                                         1),
                                                                        ('ms5120',
                                                                         2),
                                                                        ('ms10240',
                                                                         3),
                                                                        ('ms20480',
                                                                         4),
                                                                        ('ms30720',
                                                                         5),
                                                                        ('ms40960',
                                                                         6),
                                                                        ('ms61440',
                                                                         7)]},
                                     'LogicalChGroupInfoList-r13': {'element': {'type': 'SL-PriorityList-r13'},
                                                                    'size': [(1,
                                                                              'maxLCG-r13')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'LogicalChannelConfig': {'members': [{'members': [{'name': 'priority',
                                                                                        'restricted-to': [(1,
                                                                                                           16)],
                                                                                        'type': 'INTEGER'},
                                                                                       {'name': 'prioritisedBitRate',
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('kBps0',
                                                                                                    0),
                                                                                                   ('kBps8',
                                                                                                    1),
                                                                                                   ('kBps16',
                                                                                                    2),
                                                                                                   ('kBps32',
                                                                                                    3),
                                                                                                   ('kBps64',
                                                                                                    4),
                                                                                                   ('kBps128',
                                                                                                    5),
                                                                                                   ('kBps256',
                                                                                                    6),
                                                                                                   ('infinity',
                                                                                                    7),
                                                                                                   ('kBps512-v1020',
                                                                                                    8),
                                                                                                   ('kBps1024-v1020',
                                                                                                    9),
                                                                                                   ('kBps2048-v1020',
                                                                                                    10),
                                                                                                   ('spare5',
                                                                                                    11),
                                                                                                   ('spare4',
                                                                                                    12),
                                                                                                   ('spare3',
                                                                                                    13),
                                                                                                   ('spare2',
                                                                                                    14),
                                                                                                   ('spare1',
                                                                                                    15)]},
                                                                                       {'name': 'bucketSizeDuration',
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('ms50',
                                                                                                    0),
                                                                                                   ('ms100',
                                                                                                    1),
                                                                                                   ('ms150',
                                                                                                    2),
                                                                                                   ('ms300',
                                                                                                    3),
                                                                                                   ('ms500',
                                                                                                    4),
                                                                                                   ('ms1000',
                                                                                                    5),
                                                                                                   ('spare2',
                                                                                                    6),
                                                                                                   ('spare1',
                                                                                                    7)]},
                                                                                       {'name': 'logicalChannelGroup',
                                                                                        'optional': True,
                                                                                        'restricted-to': [(0,
                                                                                                           3)],
                                                                                        'type': 'INTEGER'}],
                                                                           'name': 'ul-SpecificParameters',
                                                                           'optional': True,
                                                                           'type': 'SEQUENCE'},
                                                                          None,
                                                                          [{'name': 'logicalChannelSR-Mask-r9',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('setup',
                                                                                        0)]}],
                                                                          [{'name': 'logicalChannelSR-Prohibit-r12',
                                                                            'optional': True,
                                                                            'type': 'BOOLEAN'}],
                                                                          [{'name': 'laa-UL-Allowed-r14',
                                                                            'optional': True,
                                                                            'type': 'BOOLEAN'},
                                                                           {'name': 'bitRateQueryProhibitTimer-r14',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('s0',
                                                                                        0),
                                                                                       ('s0dot4',
                                                                                        1),
                                                                                       ('s0dot8',
                                                                                        2),
                                                                                       ('s1dot6',
                                                                                        3),
                                                                                       ('s3',
                                                                                        4),
                                                                                       ('s6',
                                                                                        5),
                                                                                       ('s12',
                                                                                        6),
                                                                                       ('s30',
                                                                                        7)]}]],
                                                              'type': 'SEQUENCE'},
                                     'MAC-MainConfig': {'members': [{'members': [{'name': 'maxHARQ-Tx',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('n1',
                                                                                              0),
                                                                                             ('n2',
                                                                                              1),
                                                                                             ('n3',
                                                                                              2),
                                                                                             ('n4',
                                                                                              3),
                                                                                             ('n5',
                                                                                              4),
                                                                                             ('n6',
                                                                                              5),
                                                                                             ('n7',
                                                                                              6),
                                                                                             ('n8',
                                                                                              7),
                                                                                             ('n10',
                                                                                              8),
                                                                                             ('n12',
                                                                                              9),
                                                                                             ('n16',
                                                                                              10),
                                                                                             ('n20',
                                                                                              11),
                                                                                             ('n24',
                                                                                              12),
                                                                                             ('n28',
                                                                                              13),
                                                                                             ('spare2',
                                                                                              14),
                                                                                             ('spare1',
                                                                                              15)]},
                                                                                 {'name': 'periodicBSR-Timer',
                                                                                  'optional': True,
                                                                                  'type': 'PeriodicBSR-Timer-r12'},
                                                                                 {'name': 'retxBSR-Timer',
                                                                                  'type': 'RetxBSR-Timer-r12'},
                                                                                 {'name': 'ttiBundling',
                                                                                  'type': 'BOOLEAN'}],
                                                                     'name': 'ul-SCH-Config',
                                                                     'optional': True,
                                                                     'type': 'SEQUENCE'},
                                                                    {'name': 'drx-Config',
                                                                     'optional': True,
                                                                     'type': 'DRX-Config'},
                                                                    {'name': 'timeAlignmentTimerDedicated',
                                                                     'type': 'TimeAlignmentTimer'},
                                                                    {'members': [{'name': 'release',
                                                                                  'type': 'NULL'},
                                                                                 {'members': [{'name': 'periodicPHR-Timer',
                                                                                               'type': 'ENUMERATED',
                                                                                               'values': [('sf10',
                                                                                                           0),
                                                                                                          ('sf20',
                                                                                                           1),
                                                                                                          ('sf50',
                                                                                                           2),
                                                                                                          ('sf100',
                                                                                                           3),
                                                                                                          ('sf200',
                                                                                                           4),
                                                                                                          ('sf500',
                                                                                                           5),
                                                                                                          ('sf1000',
                                                                                                           6),
                                                                                                          ('infinity',
                                                                                                           7)]},
                                                                                              {'name': 'prohibitPHR-Timer',
                                                                                               'type': 'ENUMERATED',
                                                                                               'values': [('sf0',
                                                                                                           0),
                                                                                                          ('sf10',
                                                                                                           1),
                                                                                                          ('sf20',
                                                                                                           2),
                                                                                                          ('sf50',
                                                                                                           3),
                                                                                                          ('sf100',
                                                                                                           4),
                                                                                                          ('sf200',
                                                                                                           5),
                                                                                                          ('sf500',
                                                                                                           6),
                                                                                                          ('sf1000',
                                                                                                           7)]},
                                                                                              {'name': 'dl-PathlossChange',
                                                                                               'type': 'ENUMERATED',
                                                                                               'values': [('dB1',
                                                                                                           0),
                                                                                                          ('dB3',
                                                                                                           1),
                                                                                                          ('dB6',
                                                                                                           2),
                                                                                                          ('infinity',
                                                                                                           3)]}],
                                                                                  'name': 'setup',
                                                                                  'type': 'SEQUENCE'}],
                                                                     'name': 'phr-Config',
                                                                     'optional': True,
                                                                     'type': 'CHOICE'},
                                                                    None,
                                                                    [{'name': 'sr-ProhibitTimer-r9',
                                                                      'optional': True,
                                                                      'restricted-to': [(0,
                                                                                         7)],
                                                                      'type': 'INTEGER'}],
                                                                    [{'members': [{'name': 'sCellDeactivationTimer-r10',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('rf2',
                                                                                               0),
                                                                                              ('rf4',
                                                                                               1),
                                                                                              ('rf8',
                                                                                               2),
                                                                                              ('rf16',
                                                                                               3),
                                                                                              ('rf32',
                                                                                               4),
                                                                                              ('rf64',
                                                                                               5),
                                                                                              ('rf128',
                                                                                               6),
                                                                                              ('spare',
                                                                                               7)]},
                                                                                  {'name': 'extendedBSR-Sizes-r10',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('setup',
                                                                                               0)]},
                                                                                  {'name': 'extendedPHR-r10',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('setup',
                                                                                               0)]}],
                                                                      'name': 'mac-MainConfig-v1020',
                                                                      'optional': True,
                                                                      'type': 'SEQUENCE'}],
                                                                    [{'name': 'stag-ToReleaseList-r11',
                                                                      'optional': True,
                                                                      'type': 'STAG-ToReleaseList-r11'},
                                                                     {'name': 'stag-ToAddModList-r11',
                                                                      'optional': True,
                                                                      'type': 'STAG-ToAddModList-r11'},
                                                                     {'name': 'drx-Config-v1130',
                                                                      'optional': True,
                                                                      'type': 'DRX-Config-v1130'}],
                                                                    [{'name': 'e-HARQ-Pattern-r12',
                                                                      'optional': True,
                                                                      'type': 'BOOLEAN'},
                                                                     {'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'members': [{'name': 'phr-ModeOtherCG-r12',
                                                                                                'type': 'ENUMERATED',
                                                                                                'values': [('real',
                                                                                                            0),
                                                                                                           ('virtual',
                                                                                                            1)]}],
                                                                                   'name': 'setup',
                                                                                   'type': 'SEQUENCE'}],
                                                                      'name': 'dualConnectivityPHR',
                                                                      'optional': True,
                                                                      'type': 'CHOICE'},
                                                                     {'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'members': [{'name': 'logicalChannelSR-ProhibitTimer-r12',
                                                                                                'type': 'ENUMERATED',
                                                                                                'values': [('sf20',
                                                                                                            0),
                                                                                                           ('sf40',
                                                                                                            1),
                                                                                                           ('sf64',
                                                                                                            2),
                                                                                                           ('sf128',
                                                                                                            3),
                                                                                                           ('sf512',
                                                                                                            4),
                                                                                                           ('sf1024',
                                                                                                            5),
                                                                                                           ('sf2560',
                                                                                                            6),
                                                                                                           ('spare1',
                                                                                                            7)]}],
                                                                                   'name': 'setup',
                                                                                   'type': 'SEQUENCE'}],
                                                                      'name': 'logicalChannelSR-Config-r12',
                                                                      'optional': True,
                                                                      'type': 'CHOICE'}],
                                                                    [{'name': 'drx-Config-v1310',
                                                                      'optional': True,
                                                                      'type': 'DRX-Config-v1310'},
                                                                     {'name': 'extendedPHR2-r13',
                                                                      'optional': True,
                                                                      'type': 'BOOLEAN'},
                                                                     {'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'members': [{'name': 'sf5120',
                                                                                                'restricted-to': [(0,
                                                                                                                   1)],
                                                                                                'type': 'INTEGER'},
                                                                                               {'name': 'sf10240',
                                                                                                'restricted-to': [(0,
                                                                                                                   3)],
                                                                                                'type': 'INTEGER'}],
                                                                                   'name': 'setup',
                                                                                   'type': 'CHOICE'}],
                                                                      'name': 'eDRX-Config-CycleStartOffset-r13',
                                                                      'optional': True,
                                                                      'type': 'CHOICE'}],
                                                                    [{'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'name': 'setup',
                                                                                   'type': 'DRX-Config-r13'}],
                                                                      'name': 'drx-Config-r13',
                                                                      'optional': True,
                                                                      'type': 'CHOICE'}],
                                                                    [{'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'members': [{'name': 'skipUplinkTxSPS-r14',
                                                                                                'optional': True,
                                                                                                'type': 'ENUMERATED',
                                                                                                'values': [('true',
                                                                                                            0)]},
                                                                                               {'name': 'skipUplinkTxDynamic-r14',
                                                                                                'optional': True,
                                                                                                'type': 'ENUMERATED',
                                                                                                'values': [('true',
                                                                                                            0)]}],
                                                                                   'name': 'setup',
                                                                                   'type': 'SEQUENCE'}],
                                                                      'name': 'skipUplinkTx-r14',
                                                                      'optional': True,
                                                                      'type': 'CHOICE'},
                                                                     {'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'members': [{'name': 'dataInactivityTimer-r14',
                                                                                                'type': 'DataInactivityTimer-r14'}],
                                                                                   'name': 'setup',
                                                                                   'type': 'SEQUENCE'}],
                                                                      'name': 'dataInactivityTimerConfig-r14',
                                                                      'optional': True,
                                                                      'type': 'CHOICE'}],
                                                                    [{'name': 'rai-Activation-r14',
                                                                      'optional': True,
                                                                      'type': 'ENUMERATED',
                                                                      'values': [('true',
                                                                                  0)]}]],
                                                        'type': 'SEQUENCE'},
                                     'MAC-MainConfigSCell-r11': {'members': [{'name': 'stag-Id-r11',
                                                                              'optional': True,
                                                                              'type': 'STAG-Id-r11'},
                                                                             None],
                                                                 'type': 'SEQUENCE'},
                                     'MAC-MainConfigSL-r12': {'members': [{'name': 'periodic-BSR-TimerSL',
                                                                           'optional': True,
                                                                           'type': 'PeriodicBSR-Timer-r12'},
                                                                          {'name': 'retx-BSR-TimerSL',
                                                                           'type': 'RetxBSR-Timer-r12'}],
                                                              'type': 'SEQUENCE'},
                                     'MAC-Parameters-r12': {'members': [{'name': 'logicalChannelSR-ProhibitTimer-r12',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]},
                                                                        {'name': 'longDRX-Command-r12',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]}],
                                                            'type': 'SEQUENCE'},
                                     'MAC-Parameters-v1310': {'members': [{'name': 'extendedMAC-LengthField-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'extendedLongDRX-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'MAC-Parameters-v1430': {'members': [{'name': 'shortSPS-IntervalFDD-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'shortSPS-IntervalTDD-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'skipUplinkDynamic-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'skipUplinkSPS-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'multipleUplinkSPS-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'dataInactMon-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'MAC-Parameters-v1440': {'members': [{'name': 'rai-Support-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'MBMS-CarrierType-r14': {'members': [{'name': 'carrierType-r14',
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('mbms',
                                                                                       0),
                                                                                      ('fembmsMixed',
                                                                                       1),
                                                                                      ('fembmsDedicated',
                                                                                       2)]},
                                                                          {'name': 'frameOffset-r14',
                                                                           'optional': True,
                                                                           'restricted-to': [(0,
                                                                                              3)],
                                                                           'type': 'INTEGER'}],
                                                              'type': 'SEQUENCE'},
                                     'MBMS-InterFreqCarrierTypeList-r14': {'element': {'type': 'MBMS-CarrierType-r14'},
                                                                           'size': [(1,
                                                                                     'maxFreq')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'MBMS-NotificationConfig-r9': {'members': [{'name': 'notificationRepetitionCoeff-r9',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('n2',
                                                                                             0),
                                                                                            ('n4',
                                                                                             1)]},
                                                                                {'name': 'notificationOffset-r9',
                                                                                 'restricted-to': [(0,
                                                                                                    10)],
                                                                                 'type': 'INTEGER'},
                                                                                {'name': 'notificationSF-Index-r9',
                                                                                 'restricted-to': [(1,
                                                                                                    6)],
                                                                                 'type': 'INTEGER'}],
                                                                    'type': 'SEQUENCE'},
                                     'MBMS-NotificationConfig-v1430': {'members': [{'name': 'notificationSF-Index-v1430',
                                                                                    'restricted-to': [(7,
                                                                                                       10)],
                                                                                    'type': 'INTEGER'}],
                                                                       'type': 'SEQUENCE'},
                                     'MBMS-Parameters-r11': {'members': [{'name': 'mbms-SCell-r11',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'mbms-NonServingCell-r11',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'MBMS-Parameters-v1250': {'members': [{'name': 'mbms-AsyncDC-r12',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]}],
                                                               'type': 'SEQUENCE'},
                                     'MBMS-Parameters-v1430': {'members': [{'name': 'fembmsDedicatedCell-r14',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]},
                                                                           {'name': 'fembmsMixedCell-r14',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]},
                                                                           {'name': 'subcarrierSpacingMBMS-khz7dot5-r14',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]},
                                                                           {'name': 'subcarrierSpacingMBMS-khz1dot25-r14',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]}],
                                                               'type': 'SEQUENCE'},
                                     'MBMS-SAI-InterFreq-r11': {'members': [{'name': 'dl-CarrierFreq-r11',
                                                                             'type': 'ARFCN-ValueEUTRA-r9'},
                                                                            {'name': 'mbms-SAI-List-r11',
                                                                             'type': 'MBMS-SAI-List-r11'}],
                                                                'type': 'SEQUENCE'},
                                     'MBMS-SAI-InterFreq-v1140': {'members': [{'name': 'multiBandInfoList-r11',
                                                                               'optional': True,
                                                                               'type': 'MultiBandInfoList-r11'}],
                                                                  'type': 'SEQUENCE'},
                                     'MBMS-SAI-InterFreqList-r11': {'element': {'type': 'MBMS-SAI-InterFreq-r11'},
                                                                    'size': [(1,
                                                                              'maxFreq')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'MBMS-SAI-InterFreqList-v1140': {'element': {'type': 'MBMS-SAI-InterFreq-v1140'},
                                                                      'size': [(1,
                                                                                'maxFreq')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'MBMS-SAI-List-r11': {'element': {'type': 'MBMS-SAI-r11'},
                                                           'size': [(1,
                                                                     'maxSAI-MBMS-r11')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'MBMS-SAI-r11': {'restricted-to': [(0,
                                                                         65535)],
                                                      'type': 'INTEGER'},
                                     'MBMS-ServiceInfo-r13': {'members': [{'name': 'tmgi-r13',
                                                                           'type': 'TMGI-r9'}],
                                                              'type': 'SEQUENCE'},
                                     'MBMS-ServiceList-r13': {'element': {'type': 'MBMS-ServiceInfo-r13'},
                                                              'size': [(0,
                                                                        'maxMBMS-ServiceListPerUE-r13')],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'MBMS-SessionInfo-r9': {'members': [{'name': 'tmgi-r9',
                                                                          'type': 'TMGI-r9'},
                                                                         {'name': 'sessionId-r9',
                                                                          'optional': True,
                                                                          'size': [1],
                                                                          'type': 'OCTET '
                                                                                  'STRING'},
                                                                         {'name': 'logicalChannelIdentity-r9',
                                                                          'restricted-to': [(0,
                                                                                             'maxSessionPerPMCH-1')],
                                                                          'type': 'INTEGER'},
                                                                         None],
                                                             'type': 'SEQUENCE'},
                                     'MBMS-SessionInfoList-r9': {'element': {'type': 'MBMS-SessionInfo-r9'},
                                                                 'size': [(0,
                                                                           'maxSessionPerPMCH')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'MBMSCountingRequest-r10': {'members': [{'name': 'countingRequestList-r10',
                                                                              'type': 'CountingRequestList-r10'},
                                                                             {'name': 'lateNonCriticalExtension',
                                                                              'optional': True,
                                                                              'type': 'OCTET '
                                                                                      'STRING'},
                                                                             {'members': [],
                                                                              'name': 'nonCriticalExtension',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'}],
                                                                 'type': 'SEQUENCE'},
                                     'MBMSCountingResponse-r10': {'members': [{'members': [{'members': [{'name': 'countingResponse-r10',
                                                                                                         'type': 'MBMSCountingResponse-r10-IEs'},
                                                                                                        {'name': 'spare3',
                                                                                                         'type': 'NULL'},
                                                                                                        {'name': 'spare2',
                                                                                                         'type': 'NULL'},
                                                                                                        {'name': 'spare1',
                                                                                                         'type': 'NULL'}],
                                                                                            'name': 'c1',
                                                                                            'type': 'CHOICE'},
                                                                                           {'members': [],
                                                                                            'name': 'criticalExtensionsFuture',
                                                                                            'type': 'SEQUENCE'}],
                                                                               'name': 'criticalExtensions',
                                                                               'type': 'CHOICE'}],
                                                                  'type': 'SEQUENCE'},
                                     'MBMSCountingResponse-r10-IEs': {'members': [{'name': 'mbsfn-AreaIndex-r10',
                                                                                   'optional': True,
                                                                                   'restricted-to': [(0,
                                                                                                      'maxMBSFN-Area-1')],
                                                                                   'type': 'INTEGER'},
                                                                                  {'name': 'countingResponseList-r10',
                                                                                   'optional': True,
                                                                                   'type': 'CountingResponseList-r10'},
                                                                                  {'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'members': [],
                                                                                   'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'SEQUENCE'}],
                                                                      'type': 'SEQUENCE'},
                                     'MBMSInterestIndication-r11': {'members': [{'members': [{'members': [{'name': 'interestIndication-r11',
                                                                                                           'type': 'MBMSInterestIndication-r11-IEs'},
                                                                                                          {'name': 'spare3',
                                                                                                           'type': 'NULL'},
                                                                                                          {'name': 'spare2',
                                                                                                           'type': 'NULL'},
                                                                                                          {'name': 'spare1',
                                                                                                           'type': 'NULL'}],
                                                                                              'name': 'c1',
                                                                                              'type': 'CHOICE'},
                                                                                             {'members': [],
                                                                                              'name': 'criticalExtensionsFuture',
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'name': 'criticalExtensions',
                                                                                 'type': 'CHOICE'}],
                                                                    'type': 'SEQUENCE'},
                                     'MBMSInterestIndication-r11-IEs': {'members': [{'name': 'mbms-FreqList-r11',
                                                                                     'optional': True,
                                                                                     'type': 'CarrierFreqListMBMS-r11'},
                                                                                    {'name': 'mbms-Priority-r11',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('true',
                                                                                                 0)]},
                                                                                    {'name': 'lateNonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'OCTET '
                                                                                             'STRING'},
                                                                                    {'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'MBMSInterestIndication-v1310-IEs'}],
                                                                        'type': 'SEQUENCE'},
                                     'MBMSInterestIndication-v1310-IEs': {'members': [{'name': 'mbms-Services-r13',
                                                                                       'optional': True,
                                                                                       'type': 'MBMS-ServiceList-r13'},
                                                                                      {'members': [],
                                                                                       'name': 'nonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'}],
                                                                          'type': 'SEQUENCE'},
                                     'MBMSSessionInfo-r13': {'members': [{'name': 'tmgi-r13',
                                                                          'type': 'TMGI-r9'},
                                                                         {'name': 'sessionId-r13',
                                                                          'optional': True,
                                                                          'size': [1],
                                                                          'type': 'OCTET '
                                                                                  'STRING'}],
                                                             'type': 'SEQUENCE'},
                                     'MBSFN-AreaId-r12': {'restricted-to': [(0,
                                                                             255)],
                                                          'type': 'INTEGER'},
                                     'MBSFN-AreaInfo-r9': {'members': [{'name': 'mbsfn-AreaId-r9',
                                                                        'type': 'MBSFN-AreaId-r12'},
                                                                       {'name': 'non-MBSFNregionLength',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('s1',
                                                                                    0),
                                                                                   ('s2',
                                                                                    1)]},
                                                                       {'name': 'notificationIndicator-r9',
                                                                        'restricted-to': [(0,
                                                                                           7)],
                                                                        'type': 'INTEGER'},
                                                                       {'members': [{'name': 'mcch-RepetitionPeriod-r9',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('rf32',
                                                                                                 0),
                                                                                                ('rf64',
                                                                                                 1),
                                                                                                ('rf128',
                                                                                                 2),
                                                                                                ('rf256',
                                                                                                 3)]},
                                                                                    {'name': 'mcch-Offset-r9',
                                                                                     'restricted-to': [(0,
                                                                                                        10)],
                                                                                     'type': 'INTEGER'},
                                                                                    {'name': 'mcch-ModificationPeriod-r9',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('rf512',
                                                                                                 0),
                                                                                                ('rf1024',
                                                                                                 1)]},
                                                                                    {'name': 'sf-AllocInfo-r9',
                                                                                     'size': [6],
                                                                                     'type': 'BIT '
                                                                                             'STRING'},
                                                                                    {'name': 'signallingMCS-r9',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('n2',
                                                                                                 0),
                                                                                                ('n7',
                                                                                                 1),
                                                                                                ('n13',
                                                                                                 2),
                                                                                                ('n19',
                                                                                                 3)]}],
                                                                        'name': 'mcch-Config-r9',
                                                                        'type': 'SEQUENCE'},
                                                                       None,
                                                                       [{'members': [{'name': 'mcch-RepetitionPeriod-v1430',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('rf1',
                                                                                                  0),
                                                                                                 ('rf2',
                                                                                                  1),
                                                                                                 ('rf4',
                                                                                                  2),
                                                                                                 ('rf8',
                                                                                                  3),
                                                                                                 ('rf16',
                                                                                                  4)]},
                                                                                     {'name': 'mcch-ModificationPeriod-v1430',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('rf1',
                                                                                                  0),
                                                                                                 ('rf2',
                                                                                                  1),
                                                                                                 ('rf4',
                                                                                                  2),
                                                                                                 ('rf8',
                                                                                                  3),
                                                                                                 ('rf16',
                                                                                                  4),
                                                                                                 ('rf32',
                                                                                                  5),
                                                                                                 ('rf64',
                                                                                                  6),
                                                                                                 ('rf128',
                                                                                                  7),
                                                                                                 ('rf256',
                                                                                                  8),
                                                                                                 ('spare7',
                                                                                                  9)]}],
                                                                         'name': 'mcch-Config-r14',
                                                                         'optional': True,
                                                                         'type': 'SEQUENCE'},
                                                                        {'name': 'subcarrierSpacingMBMS-r14',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('khz-7dot5',
                                                                                     0),
                                                                                    ('khz-1dot25',
                                                                                     1)]}]],
                                                           'type': 'SEQUENCE'},
                                     'MBSFN-AreaInfoList-r9': {'element': {'type': 'MBSFN-AreaInfo-r9'},
                                                               'size': [(1,
                                                                         'maxMBSFN-Area')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'MBSFN-RSRQ-Range-r12': {'restricted-to': [(0,
                                                                                 31)],
                                                              'type': 'INTEGER'},
                                     'MBSFN-SubframeConfig': {'members': [{'name': 'radioframeAllocationPeriod',
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('n1',
                                                                                       0),
                                                                                      ('n2',
                                                                                       1),
                                                                                      ('n4',
                                                                                       2),
                                                                                      ('n8',
                                                                                       3),
                                                                                      ('n16',
                                                                                       4),
                                                                                      ('n32',
                                                                                       5)]},
                                                                          {'name': 'radioframeAllocationOffset',
                                                                           'restricted-to': [(0,
                                                                                              7)],
                                                                           'type': 'INTEGER'},
                                                                          {'members': [{'name': 'oneFrame',
                                                                                        'size': [6],
                                                                                        'type': 'BIT '
                                                                                                'STRING'},
                                                                                       {'name': 'fourFrames',
                                                                                        'size': [24],
                                                                                        'type': 'BIT '
                                                                                                'STRING'}],
                                                                           'name': 'subframeAllocation',
                                                                           'type': 'CHOICE'}],
                                                              'type': 'SEQUENCE'},
                                     'MBSFN-SubframeConfig-v1430': {'members': [{'members': [{'name': 'oneFrame-v1430',
                                                                                              'size': [2],
                                                                                              'type': 'BIT '
                                                                                                      'STRING'},
                                                                                             {'name': 'fourFrames-v1430',
                                                                                              'size': [8],
                                                                                              'type': 'BIT '
                                                                                                      'STRING'}],
                                                                                 'name': 'subframeAllocation-v1430',
                                                                                 'type': 'CHOICE'}],
                                                                    'type': 'SEQUENCE'},
                                     'MBSFN-SubframeConfigList': {'element': {'type': 'MBSFN-SubframeConfig'},
                                                                  'size': [(1,
                                                                            'maxMBSFN-Allocations')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'MBSFN-SubframeConfigList-v1430': {'element': {'type': 'MBSFN-SubframeConfig-v1430'},
                                                                        'size': [(1,
                                                                                  'maxMBSFN-Allocations')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'MBSFNAreaConfiguration-r9': {'members': [{'name': 'commonSF-Alloc-r9',
                                                                                'type': 'CommonSF-AllocPatternList-r9'},
                                                                               {'name': 'commonSF-AllocPeriod-r9',
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('rf4',
                                                                                            0),
                                                                                           ('rf8',
                                                                                            1),
                                                                                           ('rf16',
                                                                                            2),
                                                                                           ('rf32',
                                                                                            3),
                                                                                           ('rf64',
                                                                                            4),
                                                                                           ('rf128',
                                                                                            5),
                                                                                           ('rf256',
                                                                                            6)]},
                                                                               {'name': 'pmch-InfoList-r9',
                                                                                'type': 'PMCH-InfoList-r9'},
                                                                               {'name': 'nonCriticalExtension',
                                                                                'optional': True,
                                                                                'type': 'MBSFNAreaConfiguration-v930-IEs'}],
                                                                   'type': 'SEQUENCE'},
                                     'MBSFNAreaConfiguration-v1250-IEs': {'members': [{'name': 'pmch-InfoListExt-r12',
                                                                                       'optional': True,
                                                                                       'type': 'PMCH-InfoListExt-r12'},
                                                                                      {'name': 'nonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'MBSFNAreaConfiguration-v1430-IEs'}],
                                                                          'type': 'SEQUENCE'},
                                     'MBSFNAreaConfiguration-v1430-IEs': {'members': [{'name': 'commonSF-Alloc-r14',
                                                                                       'type': 'CommonSF-AllocPatternList-r14'},
                                                                                      {'members': [],
                                                                                       'name': 'nonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'}],
                                                                          'type': 'SEQUENCE'},
                                     'MBSFNAreaConfiguration-v930-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'OCTET '
                                                                                              'STRING'},
                                                                                     {'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'MBSFNAreaConfiguration-v1250-IEs'}],
                                                                         'type': 'SEQUENCE'},
                                     'MCC': {'element': {'type': 'MCC-MNC-Digit'},
                                             'size': [3],
                                             'type': 'SEQUENCE OF'},
                                     'MCC-MNC-Digit': {'restricted-to': [(0,
                                                                          9)],
                                                       'type': 'INTEGER'},
                                     'MCCH-Message': {'members': [{'name': 'message',
                                                                   'type': 'MCCH-MessageType'}],
                                                      'type': 'SEQUENCE'},
                                     'MCCH-MessageType': {'members': [{'members': [{'name': 'mbsfnAreaConfiguration-r9',
                                                                                    'type': 'MBSFNAreaConfiguration-r9'}],
                                                                       'name': 'c1',
                                                                       'type': 'CHOICE'},
                                                                      {'members': [{'members': [{'name': 'mbmsCountingRequest-r10',
                                                                                                 'type': 'MBMSCountingRequest-r10'}],
                                                                                    'name': 'c2',
                                                                                    'type': 'CHOICE'},
                                                                                   {'members': [],
                                                                                    'name': 'messageClassExtension',
                                                                                    'type': 'SEQUENCE'}],
                                                                       'name': 'later',
                                                                       'type': 'CHOICE'}],
                                                          'type': 'CHOICE'},
                                     'MIMO-BeamformedCapabilities-r13': {'members': [{'name': 'k-Max-r13',
                                                                                      'restricted-to': [(1,
                                                                                                         8)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'n-MaxList-r13',
                                                                                      'optional': True,
                                                                                      'size': [(1,
                                                                                                7)],
                                                                                      'type': 'BIT '
                                                                                              'STRING'}],
                                                                         'type': 'SEQUENCE'},
                                     'MIMO-BeamformedCapabilityList-r13': {'element': {'type': 'MIMO-BeamformedCapabilities-r13'},
                                                                           'size': [(1,
                                                                                     'maxCSI-Proc-r11')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'MIMO-CA-ParametersPerBoBC-r13': {'members': [{'name': 'parametersTM9-r13',
                                                                                    'optional': True,
                                                                                    'type': 'MIMO-CA-ParametersPerBoBCPerTM-r13'},
                                                                                   {'name': 'parametersTM10-r13',
                                                                                    'optional': True,
                                                                                    'type': 'MIMO-CA-ParametersPerBoBCPerTM-r13'}],
                                                                       'type': 'SEQUENCE'},
                                     'MIMO-CA-ParametersPerBoBC-v1430': {'members': [{'name': 'parametersTM9-v1430',
                                                                                      'optional': True,
                                                                                      'type': 'MIMO-CA-ParametersPerBoBCPerTM-v1430'},
                                                                                     {'name': 'parametersTM10-v1430',
                                                                                      'optional': True,
                                                                                      'type': 'MIMO-CA-ParametersPerBoBCPerTM-v1430'}],
                                                                         'type': 'SEQUENCE'},
                                     'MIMO-CA-ParametersPerBoBCPerTM-r13': {'members': [{'name': 'nonPrecoded-r13',
                                                                                         'optional': True,
                                                                                         'type': 'MIMO-NonPrecodedCapabilities-r13'},
                                                                                        {'name': 'beamformed-r13',
                                                                                         'optional': True,
                                                                                         'type': 'MIMO-BeamformedCapabilityList-r13'},
                                                                                        {'name': 'dmrs-Enhancements-r13',
                                                                                         'optional': True,
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('different',
                                                                                                     0)]}],
                                                                            'type': 'SEQUENCE'},
                                     'MIMO-CA-ParametersPerBoBCPerTM-v1430': {'members': [{'name': 'csi-ReportingNP-r14',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('different',
                                                                                                       0)]},
                                                                                          {'name': 'csi-ReportingAdvanced-r14',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('different',
                                                                                                       0)]}],
                                                                              'type': 'SEQUENCE'},
                                     'MIMO-CapabilityDL-r10': {'type': 'ENUMERATED',
                                                               'values': [('twoLayers',
                                                                           0),
                                                                          ('fourLayers',
                                                                           1),
                                                                          ('eightLayers',
                                                                           2)]},
                                     'MIMO-CapabilityUL-r10': {'type': 'ENUMERATED',
                                                               'values': [('twoLayers',
                                                                           0),
                                                                          ('fourLayers',
                                                                           1)]},
                                     'MIMO-NonPrecodedCapabilities-r13': {'members': [{'name': 'config1-r13',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('supported',
                                                                                                   0)]},
                                                                                      {'name': 'config2-r13',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('supported',
                                                                                                   0)]},
                                                                                      {'name': 'config3-r13',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('supported',
                                                                                                   0)]},
                                                                                      {'name': 'config4-r13',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('supported',
                                                                                                   0)]}],
                                                                          'type': 'SEQUENCE'},
                                     'MIMO-UE-BeamformedCapabilities-r13': {'members': [{'name': 'altCodebook-r13',
                                                                                         'optional': True,
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('supported',
                                                                                                     0)]},
                                                                                        {'name': 'mimo-BeamformedCapabilities-r13',
                                                                                         'type': 'MIMO-BeamformedCapabilityList-r13'}],
                                                                            'type': 'SEQUENCE'},
                                     'MIMO-UE-Parameters-r13': {'members': [{'name': 'parametersTM9-r13',
                                                                             'optional': True,
                                                                             'type': 'MIMO-UE-ParametersPerTM-r13'},
                                                                            {'name': 'parametersTM10-r13',
                                                                             'optional': True,
                                                                             'type': 'MIMO-UE-ParametersPerTM-r13'},
                                                                            {'name': 'srs-EnhancementsTDD-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]},
                                                                            {'name': 'srs-Enhancements-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]},
                                                                            {'name': 'interferenceMeasRestriction-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]}],
                                                                'type': 'SEQUENCE'},
                                     'MIMO-UE-Parameters-v1430': {'members': [{'name': 'parametersTM9-v1430',
                                                                               'optional': True,
                                                                               'type': 'MIMO-UE-ParametersPerTM-v1430'},
                                                                              {'name': 'parametersTM10-v1430',
                                                                               'optional': True,
                                                                               'type': 'MIMO-UE-ParametersPerTM-v1430'}],
                                                                  'type': 'SEQUENCE'},
                                     'MIMO-UE-ParametersPerTM-r13': {'members': [{'name': 'nonPrecoded-r13',
                                                                                  'optional': True,
                                                                                  'type': 'MIMO-NonPrecodedCapabilities-r13'},
                                                                                 {'name': 'beamformed-r13',
                                                                                  'optional': True,
                                                                                  'type': 'MIMO-UE-BeamformedCapabilities-r13'},
                                                                                 {'name': 'channelMeasRestriction-r13',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('supported',
                                                                                              0)]},
                                                                                 {'name': 'dmrs-Enhancements-r13',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('supported',
                                                                                              0)]},
                                                                                 {'name': 'csi-RS-EnhancementsTDD-r13',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('supported',
                                                                                              0)]}],
                                                                     'type': 'SEQUENCE'},
                                     'MIMO-UE-ParametersPerTM-v1430': {'members': [{'members': [{'name': 'nMaxProc-r14',
                                                                                                 'restricted-to': [(5,
                                                                                                                    32)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'nMaxResource-r14',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('ffs1',
                                                                                                             0),
                                                                                                            ('ffs2',
                                                                                                             1),
                                                                                                            ('ffs3',
                                                                                                             2),
                                                                                                            ('ffs4',
                                                                                                             3)]}],
                                                                                    'name': 'nzp-CSI-RS-AperiodicInfo-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'},
                                                                                   {'members': [{'name': 'nMaxResource-r14',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('ffs1',
                                                                                                             0),
                                                                                                            ('ffs2',
                                                                                                             1),
                                                                                                            ('ffs3',
                                                                                                             2),
                                                                                                            ('ffs4',
                                                                                                             3)]}],
                                                                                    'name': 'nzp-CSI-RS-PeriodicInfo-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'},
                                                                                   {'name': 'zp-CSI-RS-AperiodicInfo-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'ul-dmrs-Enhancements-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'densityReductionNP-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'densityReductionBF-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'hybridCSI-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'semiOL-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'csi-ReportingNP-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]},
                                                                                   {'name': 'csi-ReportingAdvanced-r14',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('supported',
                                                                                                0)]}],
                                                                       'type': 'SEQUENCE'},
                                     'MMEC': {'size': [8],
                                              'type': 'BIT STRING'},
                                     'MMTEL-Parameters-r14': {'members': [{'name': 'delayBudgetReporting-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'pusch-Enhancements-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'recommendedBitRate-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'recommendedBitRateQuery-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'MNC': {'element': {'type': 'MCC-MNC-Digit'},
                                             'size': [(2, 3)],
                                             'type': 'SEQUENCE OF'},
                                     'MasterInformationBlock': {'members': [{'name': 'dl-Bandwidth',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('n6',
                                                                                         0),
                                                                                        ('n15',
                                                                                         1),
                                                                                        ('n25',
                                                                                         2),
                                                                                        ('n50',
                                                                                         3),
                                                                                        ('n75',
                                                                                         4),
                                                                                        ('n100',
                                                                                         5)]},
                                                                            {'name': 'phich-Config',
                                                                             'type': 'PHICH-Config'},
                                                                            {'name': 'systemFrameNumber',
                                                                             'size': [8],
                                                                             'type': 'BIT '
                                                                                     'STRING'},
                                                                            {'name': 'schedulingInfoSIB1-BR-r13',
                                                                             'restricted-to': [(0,
                                                                                                31)],
                                                                             'type': 'INTEGER'},
                                                                            {'name': 'spare',
                                                                             'size': [5],
                                                                             'type': 'BIT '
                                                                                     'STRING'}],
                                                                'type': 'SEQUENCE'},
                                     'MasterInformationBlock-MBMS-r14': {'members': [{'name': 'dl-Bandwidth-MBMS-r14',
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('n6',
                                                                                                  0),
                                                                                                 ('n15',
                                                                                                  1),
                                                                                                 ('n25',
                                                                                                  2),
                                                                                                 ('n50',
                                                                                                  3),
                                                                                                 ('n75',
                                                                                                  4),
                                                                                                 ('n100',
                                                                                                  5)]},
                                                                                     {'name': 'systemFrameNumber-r14',
                                                                                      'size': [6],
                                                                                      'type': 'BIT '
                                                                                              'STRING'},
                                                                                     {'name': 'additionalNonMBSFNSubframes-r14',
                                                                                      'restricted-to': [(0,
                                                                                                         3)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'spare',
                                                                                      'size': [13],
                                                                                      'type': 'BIT '
                                                                                              'STRING'}],
                                                                         'type': 'SEQUENCE'},
                                     'MeasCSI-RS-Config-r12': {'members': [{'name': 'measCSI-RS-Id-r12',
                                                                            'type': 'MeasCSI-RS-Id-r12'},
                                                                           {'name': 'physCellId-r12',
                                                                            'restricted-to': [(0,
                                                                                               503)],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'scramblingIdentity-r12',
                                                                            'restricted-to': [(0,
                                                                                               503)],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'resourceConfig-r12',
                                                                            'restricted-to': [(0,
                                                                                               31)],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'subframeOffset-r12',
                                                                            'restricted-to': [(0,
                                                                                               4)],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'csi-RS-IndividualOffset-r12',
                                                                            'type': 'Q-OffsetRange'},
                                                                           None],
                                                               'type': 'SEQUENCE'},
                                     'MeasCSI-RS-Id-r12': {'restricted-to': [(1,
                                                                              'maxCSI-RS-Meas-r12')],
                                                           'type': 'INTEGER'},
                                     'MeasCSI-RS-ToAddModList-r12': {'element': {'type': 'MeasCSI-RS-Config-r12'},
                                                                     'size': [(1,
                                                                               'maxCSI-RS-Meas-r12')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'MeasCSI-RS-ToRemoveList-r12': {'element': {'type': 'MeasCSI-RS-Id-r12'},
                                                                     'size': [(1,
                                                                               'maxCSI-RS-Meas-r12')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'MeasConfig': {'members': [{'name': 'measObjectToRemoveList',
                                                                 'optional': True,
                                                                 'type': 'MeasObjectToRemoveList'},
                                                                {'name': 'measObjectToAddModList',
                                                                 'optional': True,
                                                                 'type': 'MeasObjectToAddModList'},
                                                                {'name': 'reportConfigToRemoveList',
                                                                 'optional': True,
                                                                 'type': 'ReportConfigToRemoveList'},
                                                                {'name': 'reportConfigToAddModList',
                                                                 'optional': True,
                                                                 'type': 'ReportConfigToAddModList'},
                                                                {'name': 'measIdToRemoveList',
                                                                 'optional': True,
                                                                 'type': 'MeasIdToRemoveList'},
                                                                {'name': 'measIdToAddModList',
                                                                 'optional': True,
                                                                 'type': 'MeasIdToAddModList'},
                                                                {'name': 'quantityConfig',
                                                                 'optional': True,
                                                                 'type': 'QuantityConfig'},
                                                                {'name': 'measGapConfig',
                                                                 'optional': True,
                                                                 'type': 'MeasGapConfig'},
                                                                {'name': 's-Measure',
                                                                 'optional': True,
                                                                 'type': 'RSRP-Range'},
                                                                {'name': 'preRegistrationInfoHRPD',
                                                                 'optional': True,
                                                                 'type': 'PreRegistrationInfoHRPD'},
                                                                {'members': [{'name': 'release',
                                                                              'type': 'NULL'},
                                                                             {'members': [{'name': 'mobilityStateParameters',
                                                                                           'type': 'MobilityStateParameters'},
                                                                                          {'name': 'timeToTrigger-SF',
                                                                                           'type': 'SpeedStateScaleFactors'}],
                                                                              'name': 'setup',
                                                                              'type': 'SEQUENCE'}],
                                                                 'name': 'speedStatePars',
                                                                 'optional': True,
                                                                 'type': 'CHOICE'},
                                                                None,
                                                                [{'name': 'measObjectToAddModList-v9e0',
                                                                  'optional': True,
                                                                  'type': 'MeasObjectToAddModList-v9e0'}],
                                                                [{'name': 'allowInterruptions-r11',
                                                                  'optional': True,
                                                                  'type': 'BOOLEAN'}],
                                                                [{'members': [{'name': 'release',
                                                                               'type': 'NULL'},
                                                                              {'name': 'setup',
                                                                               'type': 'MeasScaleFactor-r12'}],
                                                                  'name': 'measScaleFactor-r12',
                                                                  'optional': True,
                                                                  'type': 'CHOICE'},
                                                                 {'name': 'measIdToRemoveListExt-r12',
                                                                  'optional': True,
                                                                  'type': 'MeasIdToRemoveListExt-r12'},
                                                                 {'name': 'measIdToAddModListExt-r12',
                                                                  'optional': True,
                                                                  'type': 'MeasIdToAddModListExt-r12'},
                                                                 {'name': 'measRSRQ-OnAllSymbols-r12',
                                                                  'optional': True,
                                                                  'type': 'BOOLEAN'}],
                                                                [{'name': 'measObjectToRemoveListExt-r13',
                                                                  'optional': True,
                                                                  'type': 'MeasObjectToRemoveListExt-r13'},
                                                                 {'name': 'measObjectToAddModListExt-r13',
                                                                  'optional': True,
                                                                  'type': 'MeasObjectToAddModListExt-r13'},
                                                                 {'name': 'measIdToAddModList-v1310',
                                                                  'optional': True,
                                                                  'type': 'MeasIdToAddModList-v1310'},
                                                                 {'name': 'measIdToAddModListExt-v1310',
                                                                  'optional': True,
                                                                  'type': 'MeasIdToAddModListExt-v1310'}],
                                                                [{'name': 'measGapConfigPerCC-List-r14',
                                                                  'optional': True,
                                                                  'type': 'MeasGapConfigPerCC-List-r14'},
                                                                 {'name': 'measGapSharingConfig-r14',
                                                                  'optional': True,
                                                                  'type': 'MeasGapSharingConfig-r14'}]],
                                                    'type': 'SEQUENCE'},
                                     'MeasCycleSCell-r10': {'type': 'ENUMERATED',
                                                            'values': [('sf160',
                                                                        0),
                                                                       ('sf256',
                                                                        1),
                                                                       ('sf320',
                                                                        2),
                                                                       ('sf512',
                                                                        3),
                                                                       ('sf640',
                                                                        4),
                                                                       ('sf1024',
                                                                        5),
                                                                       ('sf1280',
                                                                        6),
                                                                       ('spare1',
                                                                        7)]},
                                     'MeasDS-Config-r12': {'members': [{'name': 'release',
                                                                        'type': 'NULL'},
                                                                       {'members': [{'members': [{'name': 'ms40-r12',
                                                                                                  'restricted-to': [(0,
                                                                                                                     39)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'name': 'ms80-r12',
                                                                                                  'restricted-to': [(0,
                                                                                                                     79)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'name': 'ms160-r12',
                                                                                                  'restricted-to': [(0,
                                                                                                                     159)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 None],
                                                                                     'name': 'dmtc-PeriodOffset-r12',
                                                                                     'type': 'CHOICE'},
                                                                                    {'members': [{'name': 'durationFDD-r12',
                                                                                                  'restricted-to': [(1,
                                                                                                                     'maxDS-Duration-r12')],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'name': 'durationTDD-r12',
                                                                                                  'restricted-to': [(2,
                                                                                                                     'maxDS-Duration-r12')],
                                                                                                  'type': 'INTEGER'}],
                                                                                     'name': 'ds-OccasionDuration-r12',
                                                                                     'type': 'CHOICE'},
                                                                                    {'name': 'measCSI-RS-ToRemoveList-r12',
                                                                                     'optional': True,
                                                                                     'type': 'MeasCSI-RS-ToRemoveList-r12'},
                                                                                    {'name': 'measCSI-RS-ToAddModList-r12',
                                                                                     'optional': True,
                                                                                     'type': 'MeasCSI-RS-ToAddModList-r12'},
                                                                                    None],
                                                                        'name': 'setup',
                                                                        'type': 'SEQUENCE'}],
                                                           'type': 'CHOICE'},
                                     'MeasGapConfig': {'members': [{'name': 'release',
                                                                    'type': 'NULL'},
                                                                   {'members': [{'members': [{'name': 'gp0',
                                                                                              'restricted-to': [(0,
                                                                                                                 39)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'gp1',
                                                                                              'restricted-to': [(0,
                                                                                                                 79)],
                                                                                              'type': 'INTEGER'},
                                                                                             None,
                                                                                             {'name': 'gp2-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 39)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'gp3-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 79)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'gp-ncsg0-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 39)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'gp-ncsg1-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 79)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'gp-ncsg2-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 39)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'gp-ncsg3-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 79)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'gp-nonUniform1-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 1279)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'gp-nonUniform2-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 2559)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'gp-nonUniform3-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 5119)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'gp-nonUniform4-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 10239)],
                                                                                              'type': 'INTEGER'}],
                                                                                 'name': 'gapOffset',
                                                                                 'type': 'CHOICE'}],
                                                                    'name': 'setup',
                                                                    'type': 'SEQUENCE'}],
                                                       'type': 'CHOICE'},
                                     'MeasGapConfigPerCC-List-r14': {'members': [{'name': 'release',
                                                                                  'type': 'NULL'},
                                                                                 {'members': [{'name': 'measGapConfigToRemoveList-r14',
                                                                                               'optional': True,
                                                                                               'type': 'MeasGapConfigToRemoveList-r14'},
                                                                                              {'name': 'measGapConfigToAddModList-r14',
                                                                                               'optional': True,
                                                                                               'type': 'MeasGapConfigToAddModList-r14'}],
                                                                                  'name': 'setup',
                                                                                  'type': 'SEQUENCE'}],
                                                                     'type': 'CHOICE'},
                                     'MeasGapConfigPerCC-r14': {'members': [{'name': 'servCellId-r14',
                                                                             'type': 'ServCellIndex-r13'},
                                                                            {'name': 'measGapConfigCC-r14',
                                                                             'type': 'MeasGapConfig'}],
                                                                'type': 'SEQUENCE'},
                                     'MeasGapConfigToAddModList-r14': {'element': {'type': 'MeasGapConfigPerCC-r14'},
                                                                       'size': [(1,
                                                                                 'maxServCell-r13')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'MeasGapConfigToRemoveList-r14': {'element': {'type': 'ServCellIndex-r13'},
                                                                       'size': [(1,
                                                                                 'maxServCell-r13')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'MeasGapSharingConfig-r14': {'members': [{'name': 'release',
                                                                               'type': 'NULL'},
                                                                              {'members': [{'name': 'measGapSharingScheme-r14',
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('scheme00',
                                                                                                        0),
                                                                                                       ('scheme01',
                                                                                                        1),
                                                                                                       ('scheme10',
                                                                                                        2),
                                                                                                       ('scheme11',
                                                                                                        3)]}],
                                                                               'name': 'setup',
                                                                               'type': 'SEQUENCE'}],
                                                                  'type': 'CHOICE'},
                                     'MeasId': {'restricted-to': [(1,
                                                                   'maxMeasId')],
                                                'type': 'INTEGER'},
                                     'MeasId-v1250': {'restricted-to': [('maxMeasId-Plus1',
                                                                         'maxMeasId-r12')],
                                                      'type': 'INTEGER'},
                                     'MeasIdToAddMod': {'members': [{'name': 'measId',
                                                                     'type': 'MeasId'},
                                                                    {'name': 'measObjectId',
                                                                     'type': 'MeasObjectId'},
                                                                    {'name': 'reportConfigId',
                                                                     'type': 'ReportConfigId'}],
                                                        'type': 'SEQUENCE'},
                                     'MeasIdToAddMod-v1310': {'members': [{'name': 'measObjectId-v1310',
                                                                           'optional': True,
                                                                           'type': 'MeasObjectId-v1310'}],
                                                              'type': 'SEQUENCE'},
                                     'MeasIdToAddModExt-r12': {'members': [{'name': 'measId-v1250',
                                                                            'type': 'MeasId-v1250'},
                                                                           {'name': 'measObjectId-r12',
                                                                            'type': 'MeasObjectId'},
                                                                           {'name': 'reportConfigId-r12',
                                                                            'type': 'ReportConfigId'}],
                                                               'type': 'SEQUENCE'},
                                     'MeasIdToAddModList': {'element': {'type': 'MeasIdToAddMod'},
                                                            'size': [(1,
                                                                      'maxMeasId')],
                                                            'type': 'SEQUENCE '
                                                                    'OF'},
                                     'MeasIdToAddModList-v1310': {'element': {'type': 'MeasIdToAddMod-v1310'},
                                                                  'size': [(1,
                                                                            'maxMeasId')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'MeasIdToAddModListExt-r12': {'element': {'type': 'MeasIdToAddModExt-r12'},
                                                                   'size': [(1,
                                                                             'maxMeasId')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'MeasIdToAddModListExt-v1310': {'element': {'type': 'MeasIdToAddMod-v1310'},
                                                                     'size': [(1,
                                                                               'maxMeasId')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'MeasIdToRemoveList': {'element': {'type': 'MeasId'},
                                                            'size': [(1,
                                                                      'maxMeasId')],
                                                            'type': 'SEQUENCE '
                                                                    'OF'},
                                     'MeasIdToRemoveListExt-r12': {'element': {'type': 'MeasId-v1250'},
                                                                   'size': [(1,
                                                                             'maxMeasId')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'MeasObjectCDMA2000': {'members': [{'name': 'cdma2000-Type',
                                                                         'type': 'CDMA2000-Type'},
                                                                        {'name': 'carrierFreq',
                                                                         'type': 'CarrierFreqCDMA2000'},
                                                                        {'name': 'searchWindowSize',
                                                                         'optional': True,
                                                                         'restricted-to': [(0,
                                                                                            15)],
                                                                         'type': 'INTEGER'},
                                                                        {'default': 0,
                                                                         'name': 'offsetFreq',
                                                                         'type': 'Q-OffsetRangeInterRAT'},
                                                                        {'name': 'cellsToRemoveList',
                                                                         'optional': True,
                                                                         'type': 'CellIndexList'},
                                                                        {'name': 'cellsToAddModList',
                                                                         'optional': True,
                                                                         'type': 'CellsToAddModListCDMA2000'},
                                                                        {'name': 'cellForWhichToReportCGI',
                                                                         'optional': True,
                                                                         'type': 'PhysCellIdCDMA2000'},
                                                                        None],
                                                            'type': 'SEQUENCE'},
                                     'MeasObjectEUTRA': {'members': [{'name': 'carrierFreq',
                                                                      'type': 'ARFCN-ValueEUTRA'},
                                                                     {'name': 'allowedMeasBandwidth',
                                                                      'type': 'AllowedMeasBandwidth'},
                                                                     {'name': 'presenceAntennaPort1',
                                                                      'type': 'PresenceAntennaPort1'},
                                                                     {'name': 'neighCellConfig',
                                                                      'type': 'NeighCellConfig'},
                                                                     {'default': 'dB0',
                                                                      'name': 'offsetFreq',
                                                                      'type': 'Q-OffsetRange'},
                                                                     {'name': 'cellsToRemoveList',
                                                                      'optional': True,
                                                                      'type': 'CellIndexList'},
                                                                     {'name': 'cellsToAddModList',
                                                                      'optional': True,
                                                                      'type': 'CellsToAddModList'},
                                                                     {'name': 'blackCellsToRemoveList',
                                                                      'optional': True,
                                                                      'type': 'CellIndexList'},
                                                                     {'name': 'blackCellsToAddModList',
                                                                      'optional': True,
                                                                      'type': 'BlackCellsToAddModList'},
                                                                     {'name': 'cellForWhichToReportCGI',
                                                                      'optional': True,
                                                                      'type': 'PhysCellId'},
                                                                     None,
                                                                     [{'name': 'measCycleSCell-r10',
                                                                       'optional': True,
                                                                       'type': 'MeasCycleSCell-r10'},
                                                                      {'name': 'measSubframePatternConfigNeigh-r10',
                                                                       'optional': True,
                                                                       'type': 'MeasSubframePatternConfigNeigh-r10'}],
                                                                     [{'name': 'widebandRSRQ-Meas-r11',
                                                                       'optional': True,
                                                                       'type': 'BOOLEAN'}],
                                                                     [{'name': 'altTTT-CellsToRemoveList-r12',
                                                                       'optional': True,
                                                                       'type': 'CellIndexList'},
                                                                      {'name': 'altTTT-CellsToAddModList-r12',
                                                                       'optional': True,
                                                                       'type': 'AltTTT-CellsToAddModList-r12'},
                                                                      {'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'name': 'setup',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('ms0',
                                                                                                0),
                                                                                               ('ms50',
                                                                                                1),
                                                                                               ('ms100',
                                                                                                2),
                                                                                               ('ms200',
                                                                                                3),
                                                                                               ('ms300',
                                                                                                4),
                                                                                               ('ms400',
                                                                                                5),
                                                                                               ('ms500',
                                                                                                6),
                                                                                               ('ms1000',
                                                                                                7)]}],
                                                                       'name': 't312-r12',
                                                                       'optional': True,
                                                                       'type': 'CHOICE'},
                                                                      {'name': 'reducedMeasPerformance-r12',
                                                                       'optional': True,
                                                                       'type': 'BOOLEAN'},
                                                                      {'name': 'measDS-Config-r12',
                                                                       'optional': True,
                                                                       'type': 'MeasDS-Config-r12'}],
                                                                     [{'name': 'whiteCellsToRemoveList-r13',
                                                                       'optional': True,
                                                                       'type': 'CellIndexList'},
                                                                      {'name': 'whiteCellsToAddModList-r13',
                                                                       'optional': True,
                                                                       'type': 'WhiteCellsToAddModList-r13'},
                                                                      {'name': 'rmtc-Config-r13',
                                                                       'optional': True,
                                                                       'type': 'RMTC-Config-r13'},
                                                                      {'name': 'carrierFreq-r13',
                                                                       'optional': True,
                                                                       'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                                     [{'name': 'tx-ResourcePoolToRemoveList-r14',
                                                                       'optional': True,
                                                                       'type': 'Tx-ResourcePoolMeasList-r14'},
                                                                      {'name': 'tx-ResourcePoolToAddList-r14',
                                                                       'optional': True,
                                                                       'type': 'Tx-ResourcePoolMeasList-r14'},
                                                                      {'name': 'fembms-MixedCarrier-r14',
                                                                       'optional': True,
                                                                       'type': 'BOOLEAN'}]],
                                                         'type': 'SEQUENCE'},
                                     'MeasObjectEUTRA-v9e0': {'members': [{'name': 'carrierFreq-v9e0',
                                                                           'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                              'type': 'SEQUENCE'},
                                     'MeasObjectGERAN': {'members': [{'name': 'carrierFreqs',
                                                                      'type': 'CarrierFreqsGERAN'},
                                                                     {'default': 0,
                                                                      'name': 'offsetFreq',
                                                                      'type': 'Q-OffsetRangeInterRAT'},
                                                                     {'default': '0b11111111',
                                                                      'name': 'ncc-Permitted',
                                                                      'size': [8],
                                                                      'type': 'BIT '
                                                                              'STRING'},
                                                                     {'name': 'cellForWhichToReportCGI',
                                                                      'optional': True,
                                                                      'type': 'PhysCellIdGERAN'},
                                                                     None],
                                                         'type': 'SEQUENCE'},
                                     'MeasObjectId': {'restricted-to': [(1,
                                                                         'maxObjectId')],
                                                      'type': 'INTEGER'},
                                     'MeasObjectId-r13': {'restricted-to': [(1,
                                                                             'maxObjectId-r13')],
                                                          'type': 'INTEGER'},
                                     'MeasObjectId-v1310': {'restricted-to': [('maxObjectId-Plus1-r13',
                                                                               'maxObjectId-r13')],
                                                            'type': 'INTEGER'},
                                     'MeasObjectToAddMod': {'members': [{'name': 'measObjectId',
                                                                         'type': 'MeasObjectId'},
                                                                        {'members': [{'name': 'measObjectEUTRA',
                                                                                      'type': 'MeasObjectEUTRA'},
                                                                                     {'name': 'measObjectUTRA',
                                                                                      'type': 'MeasObjectUTRA'},
                                                                                     {'name': 'measObjectGERAN',
                                                                                      'type': 'MeasObjectGERAN'},
                                                                                     {'name': 'measObjectCDMA2000',
                                                                                      'type': 'MeasObjectCDMA2000'},
                                                                                     None,
                                                                                     {'name': 'measObjectWLAN-r13',
                                                                                      'type': 'MeasObjectWLAN-r13'}],
                                                                         'name': 'measObject',
                                                                         'type': 'CHOICE'}],
                                                            'type': 'SEQUENCE'},
                                     'MeasObjectToAddMod-v9e0': {'members': [{'name': 'measObjectEUTRA-v9e0',
                                                                              'optional': True,
                                                                              'type': 'MeasObjectEUTRA-v9e0'}],
                                                                 'type': 'SEQUENCE'},
                                     'MeasObjectToAddModExt-r13': {'members': [{'name': 'measObjectId-r13',
                                                                                'type': 'MeasObjectId-v1310'},
                                                                               {'members': [{'name': 'measObjectEUTRA-r13',
                                                                                             'type': 'MeasObjectEUTRA'},
                                                                                            {'name': 'measObjectUTRA-r13',
                                                                                             'type': 'MeasObjectUTRA'},
                                                                                            {'name': 'measObjectGERAN-r13',
                                                                                             'type': 'MeasObjectGERAN'},
                                                                                            {'name': 'measObjectCDMA2000-r13',
                                                                                             'type': 'MeasObjectCDMA2000'},
                                                                                            None,
                                                                                            {'name': 'measObjectWLAN-v1320',
                                                                                             'type': 'MeasObjectWLAN-r13'}],
                                                                                'name': 'measObject-r13',
                                                                                'type': 'CHOICE'}],
                                                                   'type': 'SEQUENCE'},
                                     'MeasObjectToAddModList': {'element': {'type': 'MeasObjectToAddMod'},
                                                                'size': [(1,
                                                                          'maxObjectId')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'MeasObjectToAddModList-v9e0': {'element': {'type': 'MeasObjectToAddMod-v9e0'},
                                                                     'size': [(1,
                                                                               'maxObjectId')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'MeasObjectToAddModListExt-r13': {'element': {'type': 'MeasObjectToAddModExt-r13'},
                                                                       'size': [(1,
                                                                                 'maxObjectId')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'MeasObjectToRemoveList': {'element': {'type': 'MeasObjectId'},
                                                                'size': [(1,
                                                                          'maxObjectId')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'MeasObjectToRemoveListExt-r13': {'element': {'type': 'MeasObjectId-v1310'},
                                                                       'size': [(1,
                                                                                 'maxObjectId')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'MeasObjectUTRA': {'members': [{'name': 'carrierFreq',
                                                                     'type': 'ARFCN-ValueUTRA'},
                                                                    {'default': 0,
                                                                     'name': 'offsetFreq',
                                                                     'type': 'Q-OffsetRangeInterRAT'},
                                                                    {'name': 'cellsToRemoveList',
                                                                     'optional': True,
                                                                     'type': 'CellIndexList'},
                                                                    {'members': [{'name': 'cellsToAddModListUTRA-FDD',
                                                                                  'type': 'CellsToAddModListUTRA-FDD'},
                                                                                 {'name': 'cellsToAddModListUTRA-TDD',
                                                                                  'type': 'CellsToAddModListUTRA-TDD'}],
                                                                     'name': 'cellsToAddModList',
                                                                     'optional': True,
                                                                     'type': 'CHOICE'},
                                                                    {'members': [{'name': 'utra-FDD',
                                                                                  'type': 'PhysCellIdUTRA-FDD'},
                                                                                 {'name': 'utra-TDD',
                                                                                  'type': 'PhysCellIdUTRA-TDD'}],
                                                                     'name': 'cellForWhichToReportCGI',
                                                                     'optional': True,
                                                                     'type': 'CHOICE'},
                                                                    None,
                                                                    [{'name': 'csg-allowedReportingCells-v930',
                                                                      'optional': True,
                                                                      'type': 'CSG-AllowedReportingCells-r9'}],
                                                                    [{'name': 'reducedMeasPerformance-r12',
                                                                      'optional': True,
                                                                      'type': 'BOOLEAN'}]],
                                                        'type': 'SEQUENCE'},
                                     'MeasObjectWLAN-r13': {'members': [{'members': [{'element': {'type': 'WLAN-BandIndicator-r13'},
                                                                                      'name': 'bandIndicatorListWLAN-r13',
                                                                                      'size': [(1,
                                                                                                'maxWLAN-Bands-r13')],
                                                                                      'type': 'SEQUENCE '
                                                                                              'OF'},
                                                                                     {'element': {'type': 'WLAN-CarrierInfo-r13'},
                                                                                      'name': 'carrierInfoListWLAN-r13',
                                                                                      'size': [(1,
                                                                                                'maxWLAN-CarrierInfo-r13')],
                                                                                      'type': 'SEQUENCE '
                                                                                              'OF'}],
                                                                         'name': 'carrierFreq-r13',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'},
                                                                        {'name': 'wlan-ToAddModList-r13',
                                                                         'optional': True,
                                                                         'type': 'WLAN-Id-List-r13'},
                                                                        {'name': 'wlan-ToRemoveList-r13',
                                                                         'optional': True,
                                                                         'type': 'WLAN-Id-List-r13'},
                                                                        None],
                                                            'type': 'SEQUENCE'},
                                     'MeasParameters': {'members': [{'name': 'bandListEUTRA',
                                                                     'type': 'BandListEUTRA'}],
                                                        'type': 'SEQUENCE'},
                                     'MeasParameters-v1020': {'members': [{'name': 'bandCombinationListEUTRA-r10',
                                                                           'type': 'BandCombinationListEUTRA-r10'}],
                                                              'type': 'SEQUENCE'},
                                     'MeasParameters-v1130': {'members': [{'name': 'rsrqMeasWideband-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'MeasParameters-v11a0': {'members': [{'name': 'benefitsFromInterruption-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('true',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'MeasParameters-v1250': {'members': [{'name': 'timerT312-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'alternativeTimeToTrigger-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'incMonEUTRA-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'incMonUTRA-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'extendedMaxMeasId-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'extendedRSRQ-LowerRange-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'rsrq-OnAllSymbols-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'crs-DiscoverySignalsMeas-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'csi-RS-DiscoverySignalsMeas-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'MeasParameters-v1310': {'members': [{'name': 'rs-SINR-Meas-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'whiteCellList-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'extendedMaxObjectId-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'ul-PDCP-Delay-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'extendedFreqPriorities-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'multiBandInfoReport-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'rssi-AndChannelOccupancyReporting-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'MeasParameters-v1430': {'members': [{'name': 'ceMeasurements-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'ncsg-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'shortMeasurementGap-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'perServingCellMeasurementGap-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'nonUniformGap-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'MeasRSSI-ReportConfig-r13': {'members': [{'name': 'channelOccupancyThreshold-r13',
                                                                                'optional': True,
                                                                                'type': 'RSSI-Range-r13'}],
                                                                   'type': 'SEQUENCE'},
                                     'MeasResult2CDMA2000-r9': {'members': [{'name': 'carrierFreq-r9',
                                                                             'type': 'CarrierFreqCDMA2000'},
                                                                            {'name': 'measResultList-r9',
                                                                             'type': 'MeasResultsCDMA2000'}],
                                                                'type': 'SEQUENCE'},
                                     'MeasResult2EUTRA-r9': {'members': [{'name': 'carrierFreq-r9',
                                                                          'type': 'ARFCN-ValueEUTRA'},
                                                                         {'name': 'measResultList-r9',
                                                                          'type': 'MeasResultListEUTRA'}],
                                                             'type': 'SEQUENCE'},
                                     'MeasResult2EUTRA-v1250': {'members': [{'name': 'rsrq-Type-r12',
                                                                             'optional': True,
                                                                             'type': 'RSRQ-Type-r12'}],
                                                                'type': 'SEQUENCE'},
                                     'MeasResult2EUTRA-v9e0': {'members': [{'name': 'carrierFreq-v9e0',
                                                                            'optional': True,
                                                                            'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                               'type': 'SEQUENCE'},
                                     'MeasResult2UTRA-r9': {'members': [{'name': 'carrierFreq-r9',
                                                                         'type': 'ARFCN-ValueUTRA'},
                                                                        {'name': 'measResultList-r9',
                                                                         'type': 'MeasResultListUTRA'}],
                                                            'type': 'SEQUENCE'},
                                     'MeasResultCBR-r14': {'members': [{'name': 'poolIdentity-r14',
                                                                        'type': 'SL-V2X-TxPoolReportIdentity-r14'},
                                                                       {'name': 'cbr-PSSCH-r14',
                                                                        'type': 'SL-CBR-r14'},
                                                                       {'name': 'cbr-PSCCH-r14',
                                                                        'optional': True,
                                                                        'type': 'SL-CBR-r14'}],
                                                           'type': 'SEQUENCE'},
                                     'MeasResultCDMA2000': {'members': [{'name': 'physCellId',
                                                                         'type': 'PhysCellIdCDMA2000'},
                                                                        {'name': 'cgi-Info',
                                                                         'optional': True,
                                                                         'type': 'CellGlobalIdCDMA2000'},
                                                                        {'members': [{'name': 'pilotPnPhase',
                                                                                      'optional': True,
                                                                                      'restricted-to': [(0,
                                                                                                         32767)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'pilotStrength',
                                                                                      'restricted-to': [(0,
                                                                                                         63)],
                                                                                      'type': 'INTEGER'},
                                                                                     None],
                                                                         'name': 'measResult',
                                                                         'type': 'SEQUENCE'}],
                                                            'type': 'SEQUENCE'},
                                     'MeasResultCSI-RS-List-r12': {'element': {'type': 'MeasResultCSI-RS-r12'},
                                                                   'size': [(1,
                                                                             'maxCellReport')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'MeasResultCSI-RS-r12': {'members': [{'name': 'measCSI-RS-Id-r12',
                                                                           'type': 'MeasCSI-RS-Id-r12'},
                                                                          {'name': 'csi-RSRP-Result-r12',
                                                                           'type': 'CSI-RSRP-Range-r12'},
                                                                          None],
                                                              'type': 'SEQUENCE'},
                                     'MeasResultEUTRA': {'members': [{'name': 'physCellId',
                                                                      'type': 'PhysCellId'},
                                                                     {'members': [{'name': 'cellGlobalId',
                                                                                   'type': 'CellGlobalIdEUTRA'},
                                                                                  {'name': 'trackingAreaCode',
                                                                                   'type': 'TrackingAreaCode'},
                                                                                  {'name': 'plmn-IdentityList',
                                                                                   'optional': True,
                                                                                   'type': 'PLMN-IdentityList2'}],
                                                                      'name': 'cgi-Info',
                                                                      'optional': True,
                                                                      'type': 'SEQUENCE'},
                                                                     {'members': [{'name': 'rsrpResult',
                                                                                   'optional': True,
                                                                                   'type': 'RSRP-Range'},
                                                                                  {'name': 'rsrqResult',
                                                                                   'optional': True,
                                                                                   'type': 'RSRQ-Range'},
                                                                                  None,
                                                                                  [{'name': 'additionalSI-Info-r9',
                                                                                    'optional': True,
                                                                                    'type': 'AdditionalSI-Info-r9'}],
                                                                                  [{'name': 'primaryPLMN-Suitable-r12',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]},
                                                                                   {'name': 'measResult-v1250',
                                                                                    'optional': True,
                                                                                    'type': 'RSRQ-Range-v1250'}],
                                                                                  [{'name': 'rs-sinr-Result-r13',
                                                                                    'optional': True,
                                                                                    'type': 'RS-SINR-Range-r13'},
                                                                                   {'members': [{'name': 'freqBandIndicator-r13',
                                                                                                 'optional': True,
                                                                                                 'type': 'FreqBandIndicator-r11'},
                                                                                                {'name': 'multiBandInfoList-r13',
                                                                                                 'optional': True,
                                                                                                 'type': 'MultiBandInfoList-r11'},
                                                                                                {'name': 'freqBandIndicatorPriority-r13',
                                                                                                 'optional': True,
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('true',
                                                                                                             0)]}],
                                                                                    'name': 'cgi-Info-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                                  [{'name': 'measResult-v1360',
                                                                                    'optional': True,
                                                                                    'type': 'RSRP-Range-v1360'}]],
                                                                      'name': 'measResult',
                                                                      'type': 'SEQUENCE'}],
                                                         'type': 'SEQUENCE'},
                                     'MeasResultForECID-r9': {'members': [{'name': 'ue-RxTxTimeDiffResult-r9',
                                                                           'restricted-to': [(0,
                                                                                              4095)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'currentSFN-r9',
                                                                           'size': [10],
                                                                           'type': 'BIT '
                                                                                   'STRING'}],
                                                              'type': 'SEQUENCE'},
                                     'MeasResultForRSSI-r13': {'members': [{'name': 'rssi-Result-r13',
                                                                            'type': 'RSSI-Range-r13'},
                                                                           {'name': 'channelOccupancy-r13',
                                                                            'restricted-to': [(0,
                                                                                               100)],
                                                                            'type': 'INTEGER'},
                                                                           None],
                                                               'type': 'SEQUENCE'},
                                     'MeasResultGERAN': {'members': [{'name': 'carrierFreq',
                                                                      'type': 'CarrierFreqGERAN'},
                                                                     {'name': 'physCellId',
                                                                      'type': 'PhysCellIdGERAN'},
                                                                     {'members': [{'name': 'cellGlobalId',
                                                                                   'type': 'CellGlobalIdGERAN'},
                                                                                  {'name': 'routingAreaCode',
                                                                                   'optional': True,
                                                                                   'size': [8],
                                                                                   'type': 'BIT '
                                                                                           'STRING'}],
                                                                      'name': 'cgi-Info',
                                                                      'optional': True,
                                                                      'type': 'SEQUENCE'},
                                                                     {'members': [{'name': 'rssi',
                                                                                   'restricted-to': [(0,
                                                                                                      63)],
                                                                                   'type': 'INTEGER'},
                                                                                  None],
                                                                      'name': 'measResult',
                                                                      'type': 'SEQUENCE'}],
                                                         'type': 'SEQUENCE'},
                                     'MeasResultList2CDMA2000-r9': {'element': {'type': 'MeasResult2CDMA2000-r9'},
                                                                    'size': [(1,
                                                                              'maxFreq')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'MeasResultList2EUTRA-r9': {'element': {'type': 'MeasResult2EUTRA-r9'},
                                                                 'size': [(1,
                                                                           'maxFreq')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'MeasResultList2EUTRA-v1250': {'element': {'type': 'MeasResult2EUTRA-v1250'},
                                                                    'size': [(1,
                                                                              'maxFreq')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'MeasResultList2EUTRA-v9e0': {'element': {'type': 'MeasResult2EUTRA-v9e0'},
                                                                   'size': [(1,
                                                                             'maxFreq')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'MeasResultList2GERAN-r10': {'element': {'type': 'MeasResultListGERAN'},
                                                                  'size': [(1,
                                                                            'maxCellListGERAN')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'MeasResultList2UTRA-r9': {'element': {'type': 'MeasResult2UTRA-r9'},
                                                                'size': [(1,
                                                                          'maxFreq')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'MeasResultListCBR-r14': {'element': {'type': 'MeasResultCBR-r14'},
                                                               'size': [(1,
                                                                         'maxCBR-Report-r14')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'MeasResultListCDMA2000': {'element': {'type': 'MeasResultCDMA2000'},
                                                                'size': [(1,
                                                                          'maxCellReport')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'MeasResultListEUTRA': {'element': {'type': 'MeasResultEUTRA'},
                                                             'size': [(1,
                                                                       'maxCellReport')],
                                                             'type': 'SEQUENCE '
                                                                     'OF'},
                                     'MeasResultListGERAN': {'element': {'type': 'MeasResultGERAN'},
                                                             'size': [(1,
                                                                       'maxCellReport')],
                                                             'type': 'SEQUENCE '
                                                                     'OF'},
                                     'MeasResultListMBSFN-r12': {'element': {'type': 'MeasResultMBSFN-r12'},
                                                                 'size': [(1,
                                                                           'maxMBSFN-Area')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'MeasResultListUTRA': {'element': {'type': 'MeasResultUTRA'},
                                                            'size': [(1,
                                                                      'maxCellReport')],
                                                            'type': 'SEQUENCE '
                                                                    'OF'},
                                     'MeasResultListWLAN-r13': {'element': {'type': 'MeasResultWLAN-r13'},
                                                                'size': [(1,
                                                                          'maxCellReport')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'MeasResultListWLAN-r14': {'element': {'type': 'MeasResultWLAN-r13'},
                                                                'size': [(1,
                                                                          'maxWLAN-Id-Report-r14')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'MeasResultMBSFN-r12': {'members': [{'members': [{'name': 'mbsfn-AreaId-r12',
                                                                                       'type': 'MBSFN-AreaId-r12'},
                                                                                      {'name': 'carrierFreq-r12',
                                                                                       'type': 'ARFCN-ValueEUTRA-r9'}],
                                                                          'name': 'mbsfn-Area-r12',
                                                                          'type': 'SEQUENCE'},
                                                                         {'name': 'rsrpResultMBSFN-r12',
                                                                          'type': 'RSRP-Range'},
                                                                         {'name': 'rsrqResultMBSFN-r12',
                                                                          'type': 'MBSFN-RSRQ-Range-r12'},
                                                                         {'name': 'signallingBLER-Result-r12',
                                                                          'optional': True,
                                                                          'type': 'BLER-Result-r12'},
                                                                         {'name': 'dataBLER-MCH-ResultList-r12',
                                                                          'optional': True,
                                                                          'type': 'DataBLER-MCH-ResultList-r12'},
                                                                         None],
                                                             'type': 'SEQUENCE'},
                                     'MeasResultSSTD-r13': {'members': [{'name': 'sfn-OffsetResult-r13',
                                                                         'restricted-to': [(0,
                                                                                            1023)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'frameBoundaryOffsetResult-r13',
                                                                         'restricted-to': [(-5,
                                                                                            4)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'subframeBoundaryOffsetResult-r13',
                                                                         'restricted-to': [(0,
                                                                                            127)],
                                                                         'type': 'INTEGER'}],
                                                            'type': 'SEQUENCE'},
                                     'MeasResultServFreq-r10': {'members': [{'name': 'servFreqId-r10',
                                                                             'type': 'ServCellIndex-r10'},
                                                                            {'members': [{'name': 'rsrpResultSCell-r10',
                                                                                          'type': 'RSRP-Range'},
                                                                                         {'name': 'rsrqResultSCell-r10',
                                                                                          'type': 'RSRQ-Range'}],
                                                                             'name': 'measResultSCell-r10',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'physCellId-r10',
                                                                                          'type': 'PhysCellId'},
                                                                                         {'name': 'rsrpResultNCell-r10',
                                                                                          'type': 'RSRP-Range'},
                                                                                         {'name': 'rsrqResultNCell-r10',
                                                                                          'type': 'RSRQ-Range'}],
                                                                             'name': 'measResultBestNeighCell-r10',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            None,
                                                                            [{'name': 'measResultSCell-v1250',
                                                                              'optional': True,
                                                                              'type': 'RSRQ-Range-v1250'},
                                                                             {'name': 'measResultBestNeighCell-v1250',
                                                                              'optional': True,
                                                                              'type': 'RSRQ-Range-v1250'}],
                                                                            [{'members': [{'name': 'rs-sinr-Result-r13',
                                                                                           'type': 'RS-SINR-Range-r13'}],
                                                                              'name': 'measResultSCell-v1310',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'},
                                                                             {'members': [{'name': 'rs-sinr-Result-r13',
                                                                                           'type': 'RS-SINR-Range-r13'}],
                                                                              'name': 'measResultBestNeighCell-v1310',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'}]],
                                                                'type': 'SEQUENCE'},
                                     'MeasResultServFreq-r13': {'members': [{'name': 'servFreqId-r13',
                                                                             'type': 'ServCellIndex-r13'},
                                                                            {'members': [{'name': 'rsrpResultSCell-r13',
                                                                                          'type': 'RSRP-Range'},
                                                                                         {'name': 'rsrqResultSCell-r13',
                                                                                          'type': 'RSRQ-Range-r13'},
                                                                                         {'name': 'rs-sinr-Result-r13',
                                                                                          'optional': True,
                                                                                          'type': 'RS-SINR-Range-r13'}],
                                                                             'name': 'measResultSCell-r13',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'physCellId-r13',
                                                                                          'type': 'PhysCellId'},
                                                                                         {'name': 'rsrpResultNCell-r13',
                                                                                          'type': 'RSRP-Range'},
                                                                                         {'name': 'rsrqResultNCell-r13',
                                                                                          'type': 'RSRQ-Range-r13'},
                                                                                         {'name': 'rs-sinr-Result-r13',
                                                                                          'optional': True,
                                                                                          'type': 'RS-SINR-Range-r13'}],
                                                                             'name': 'measResultBestNeighCell-r13',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            None,
                                                                            [{'members': [{'name': 'rsrpResultNCell-v1360',
                                                                                           'type': 'RSRP-Range-v1360'}],
                                                                              'name': 'measResultBestNeighCell-v1360',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'}]],
                                                                'type': 'SEQUENCE'},
                                     'MeasResultServFreqList-r10': {'element': {'type': 'MeasResultServFreq-r10'},
                                                                    'size': [(1,
                                                                              'maxServCell-r10')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'MeasResultServFreqListExt-r13': {'element': {'type': 'MeasResultServFreq-r13'},
                                                                       'size': [(1,
                                                                                 'maxServCell-r13')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'MeasResultUTRA': {'members': [{'members': [{'name': 'fdd',
                                                                                  'type': 'PhysCellIdUTRA-FDD'},
                                                                                 {'name': 'tdd',
                                                                                  'type': 'PhysCellIdUTRA-TDD'}],
                                                                     'name': 'physCellId',
                                                                     'type': 'CHOICE'},
                                                                    {'members': [{'name': 'cellGlobalId',
                                                                                  'type': 'CellGlobalIdUTRA'},
                                                                                 {'name': 'locationAreaCode',
                                                                                  'optional': True,
                                                                                  'size': [16],
                                                                                  'type': 'BIT '
                                                                                          'STRING'},
                                                                                 {'name': 'routingAreaCode',
                                                                                  'optional': True,
                                                                                  'size': [8],
                                                                                  'type': 'BIT '
                                                                                          'STRING'},
                                                                                 {'name': 'plmn-IdentityList',
                                                                                  'optional': True,
                                                                                  'type': 'PLMN-IdentityList2'}],
                                                                     'name': 'cgi-Info',
                                                                     'optional': True,
                                                                     'type': 'SEQUENCE'},
                                                                    {'members': [{'name': 'utra-RSCP',
                                                                                  'optional': True,
                                                                                  'restricted-to': [(-5,
                                                                                                     91)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'name': 'utra-EcN0',
                                                                                  'optional': True,
                                                                                  'restricted-to': [(0,
                                                                                                     49)],
                                                                                  'type': 'INTEGER'},
                                                                                 None,
                                                                                 [{'name': 'additionalSI-Info-r9',
                                                                                   'optional': True,
                                                                                   'type': 'AdditionalSI-Info-r9'}],
                                                                                 [{'name': 'primaryPLMN-Suitable-r12',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]}]],
                                                                     'name': 'measResult',
                                                                     'type': 'SEQUENCE'}],
                                                        'type': 'SEQUENCE'},
                                     'MeasResultWLAN-r13': {'members': [{'name': 'wlan-Identifiers-r13',
                                                                         'type': 'WLAN-Identifiers-r12'},
                                                                        {'name': 'carrierInfoWLAN-r13',
                                                                         'optional': True,
                                                                         'type': 'WLAN-CarrierInfo-r13'},
                                                                        {'name': 'bandWLAN-r13',
                                                                         'optional': True,
                                                                         'type': 'WLAN-BandIndicator-r13'},
                                                                        {'name': 'rssiWLAN-r13',
                                                                         'type': 'WLAN-RSSI-Range-r13'},
                                                                        {'name': 'availableAdmissionCapacityWLAN-r13',
                                                                         'optional': True,
                                                                         'restricted-to': [(0,
                                                                                            31250)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'backhaulDL-BandwidthWLAN-r13',
                                                                         'optional': True,
                                                                         'type': 'WLAN-backhaulRate-r12'},
                                                                        {'name': 'backhaulUL-BandwidthWLAN-r13',
                                                                         'optional': True,
                                                                         'type': 'WLAN-backhaulRate-r12'},
                                                                        {'name': 'channelUtilizationWLAN-r13',
                                                                         'optional': True,
                                                                         'restricted-to': [(0,
                                                                                            255)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'stationCountWLAN-r13',
                                                                         'optional': True,
                                                                         'restricted-to': [(0,
                                                                                            65535)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'connectedWLAN-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('true',
                                                                                     0)]},
                                                                        None],
                                                            'type': 'SEQUENCE'},
                                     'MeasResults': {'members': [{'name': 'measId',
                                                                  'type': 'MeasId'},
                                                                 {'members': [{'name': 'rsrpResult',
                                                                               'type': 'RSRP-Range'},
                                                                              {'name': 'rsrqResult',
                                                                               'type': 'RSRQ-Range'}],
                                                                  'name': 'measResultPCell',
                                                                  'type': 'SEQUENCE'},
                                                                 {'members': [{'name': 'measResultListEUTRA',
                                                                               'type': 'MeasResultListEUTRA'},
                                                                              {'name': 'measResultListUTRA',
                                                                               'type': 'MeasResultListUTRA'},
                                                                              {'name': 'measResultListGERAN',
                                                                               'type': 'MeasResultListGERAN'},
                                                                              {'name': 'measResultsCDMA2000',
                                                                               'type': 'MeasResultsCDMA2000'},
                                                                              None],
                                                                  'name': 'measResultNeighCells',
                                                                  'optional': True,
                                                                  'type': 'CHOICE'},
                                                                 None,
                                                                 [{'name': 'measResultForECID-r9',
                                                                   'optional': True,
                                                                   'type': 'MeasResultForECID-r9'}],
                                                                 [{'name': 'locationInfo-r10',
                                                                   'optional': True,
                                                                   'type': 'LocationInfo-r10'},
                                                                  {'name': 'measResultServFreqList-r10',
                                                                   'optional': True,
                                                                   'type': 'MeasResultServFreqList-r10'}],
                                                                 [{'name': 'measId-v1250',
                                                                   'optional': True,
                                                                   'type': 'MeasId-v1250'},
                                                                  {'name': 'measResultPCell-v1250',
                                                                   'optional': True,
                                                                   'type': 'RSRQ-Range-v1250'},
                                                                  {'name': 'measResultCSI-RS-List-r12',
                                                                   'optional': True,
                                                                   'type': 'MeasResultCSI-RS-List-r12'}],
                                                                 [{'name': 'measResultForRSSI-r13',
                                                                   'optional': True,
                                                                   'type': 'MeasResultForRSSI-r13'},
                                                                  {'name': 'measResultServFreqListExt-r13',
                                                                   'optional': True,
                                                                   'type': 'MeasResultServFreqListExt-r13'},
                                                                  {'name': 'measResultSSTD-r13',
                                                                   'optional': True,
                                                                   'type': 'MeasResultSSTD-r13'},
                                                                  {'members': [{'name': 'rs-sinr-Result-r13',
                                                                                'type': 'RS-SINR-Range-r13'}],
                                                                   'name': 'measResultPCell-v1310',
                                                                   'optional': True,
                                                                   'type': 'SEQUENCE'},
                                                                  {'name': 'ul-PDCP-DelayResultList-r13',
                                                                   'optional': True,
                                                                   'type': 'UL-PDCP-DelayResultList-r13'},
                                                                  {'name': 'measResultListWLAN-r13',
                                                                   'optional': True,
                                                                   'type': 'MeasResultListWLAN-r13'}],
                                                                 [{'name': 'measResultPCell-v1360',
                                                                   'optional': True,
                                                                   'type': 'RSRP-Range-v1360'}],
                                                                 [{'name': 'measResultListCBR-r14',
                                                                   'optional': True,
                                                                   'type': 'MeasResultListCBR-r14'},
                                                                  {'name': 'measResultListWLAN-r14',
                                                                   'optional': True,
                                                                   'type': 'MeasResultListWLAN-r14'}]],
                                                     'type': 'SEQUENCE'},
                                     'MeasResultsCDMA2000': {'members': [{'name': 'preRegistrationStatusHRPD',
                                                                          'type': 'BOOLEAN'},
                                                                         {'name': 'measResultListCDMA2000',
                                                                          'type': 'MeasResultListCDMA2000'}],
                                                             'type': 'SEQUENCE'},
                                     'MeasScaleFactor-r12': {'type': 'ENUMERATED',
                                                             'values': [('sf-EUTRA-cf1',
                                                                         0),
                                                                        ('sf-EUTRA-cf2',
                                                                         1)]},
                                     'MeasSubframeCellList-r10': {'element': {'type': 'PhysCellIdRange'},
                                                                  'size': [(1,
                                                                            'maxCellMeas')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'MeasSubframePattern-r10': {'members': [{'name': 'subframePatternFDD-r10',
                                                                              'size': [40],
                                                                              'type': 'BIT '
                                                                                      'STRING'},
                                                                             {'members': [{'name': 'subframeConfig1-5-r10',
                                                                                           'size': [20],
                                                                                           'type': 'BIT '
                                                                                                   'STRING'},
                                                                                          {'name': 'subframeConfig0-r10',
                                                                                           'size': [70],
                                                                                           'type': 'BIT '
                                                                                                   'STRING'},
                                                                                          {'name': 'subframeConfig6-r10',
                                                                                           'size': [60],
                                                                                           'type': 'BIT '
                                                                                                   'STRING'},
                                                                                          None],
                                                                              'name': 'subframePatternTDD-r10',
                                                                              'type': 'CHOICE'},
                                                                             None],
                                                                 'type': 'CHOICE'},
                                     'MeasSubframePatternConfigNeigh-r10': {'members': [{'name': 'release',
                                                                                         'type': 'NULL'},
                                                                                        {'members': [{'name': 'measSubframePatternNeigh-r10',
                                                                                                      'type': 'MeasSubframePattern-r10'},
                                                                                                     {'name': 'measSubframeCellList-r10',
                                                                                                      'optional': True,
                                                                                                      'type': 'MeasSubframeCellList-r10'}],
                                                                                         'name': 'setup',
                                                                                         'type': 'SEQUENCE'}],
                                                                            'type': 'CHOICE'},
                                     'MeasSubframePatternPCell-r10': {'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'name': 'setup',
                                                                                   'type': 'MeasSubframePattern-r10'}],
                                                                      'type': 'CHOICE'},
                                     'MeasurementReport': {'members': [{'members': [{'members': [{'name': 'measurementReport-r8',
                                                                                                  'type': 'MeasurementReport-r8-IEs'},
                                                                                                 {'name': 'spare7',
                                                                                                  'type': 'NULL'},
                                                                                                 {'name': 'spare6',
                                                                                                  'type': 'NULL'},
                                                                                                 {'name': 'spare5',
                                                                                                  'type': 'NULL'},
                                                                                                 {'name': 'spare4',
                                                                                                  'type': 'NULL'},
                                                                                                 {'name': 'spare3',
                                                                                                  'type': 'NULL'},
                                                                                                 {'name': 'spare2',
                                                                                                  'type': 'NULL'},
                                                                                                 {'name': 'spare1',
                                                                                                  'type': 'NULL'}],
                                                                                     'name': 'c1',
                                                                                     'type': 'CHOICE'},
                                                                                    {'members': [],
                                                                                     'name': 'criticalExtensionsFuture',
                                                                                     'type': 'SEQUENCE'}],
                                                                        'name': 'criticalExtensions',
                                                                        'type': 'CHOICE'}],
                                                           'type': 'SEQUENCE'},
                                     'MeasurementReport-r8-IEs': {'members': [{'name': 'measResults',
                                                                               'type': 'MeasResults'},
                                                                              {'name': 'nonCriticalExtension',
                                                                               'optional': True,
                                                                               'type': 'MeasurementReport-v8a0-IEs'}],
                                                                  'type': 'SEQUENCE'},
                                     'MeasurementReport-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'OCTET '
                                                                                         'STRING'},
                                                                                {'members': [],
                                                                                 'name': 'nonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'SEQUENCE'}],
                                                                    'type': 'SEQUENCE'},
                                     'MobilityControlInfo': {'members': [{'name': 'targetPhysCellId',
                                                                          'type': 'PhysCellId'},
                                                                         {'name': 'carrierFreq',
                                                                          'optional': True,
                                                                          'type': 'CarrierFreqEUTRA'},
                                                                         {'name': 'carrierBandwidth',
                                                                          'optional': True,
                                                                          'type': 'CarrierBandwidthEUTRA'},
                                                                         {'name': 'additionalSpectrumEmission',
                                                                          'optional': True,
                                                                          'type': 'AdditionalSpectrumEmission'},
                                                                         {'name': 't304',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('ms50',
                                                                                      0),
                                                                                     ('ms100',
                                                                                      1),
                                                                                     ('ms150',
                                                                                      2),
                                                                                     ('ms200',
                                                                                      3),
                                                                                     ('ms500',
                                                                                      4),
                                                                                     ('ms1000',
                                                                                      5),
                                                                                     ('ms2000',
                                                                                      6),
                                                                                     ('ms10000-v1310',
                                                                                      7)]},
                                                                         {'name': 'newUE-Identity',
                                                                          'type': 'C-RNTI'},
                                                                         {'name': 'radioResourceConfigCommon',
                                                                          'type': 'RadioResourceConfigCommon'},
                                                                         {'name': 'rach-ConfigDedicated',
                                                                          'optional': True,
                                                                          'type': 'RACH-ConfigDedicated'},
                                                                         None,
                                                                         [{'name': 'carrierFreq-v9e0',
                                                                           'optional': True,
                                                                           'type': 'CarrierFreqEUTRA-v9e0'}],
                                                                         [{'name': 'drb-ContinueROHC-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('true',
                                                                                       0)]}],
                                                                         [{'name': 'mobilityControlInfoV2X-r14',
                                                                           'optional': True,
                                                                           'type': 'MobilityControlInfoV2X-r14'},
                                                                          {'name': 'handoverWithoutWT-Change-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('keepLWA-Config',
                                                                                       0),
                                                                                      ('sendEndMarker',
                                                                                       1)]},
                                                                          {'name': 'makeBeforeBreak-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('true',
                                                                                       0)]},
                                                                          {'name': 'rach-Skip-r14',
                                                                           'optional': True,
                                                                           'type': 'RACH-Skip-r14'},
                                                                          {'name': 'sameSFN-Indication-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('true',
                                                                                       0)]}]],
                                                             'type': 'SEQUENCE'},
                                     'MobilityControlInfo-v10l0': {'members': [{'name': 'additionalSpectrumEmission-v10l0',
                                                                                'optional': True,
                                                                                'type': 'AdditionalSpectrumEmission-v10l0'}],
                                                                   'type': 'SEQUENCE'},
                                     'MobilityControlInfoSCG-r12': {'members': [{'name': 't307-r12',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('ms50',
                                                                                             0),
                                                                                            ('ms100',
                                                                                             1),
                                                                                            ('ms150',
                                                                                             2),
                                                                                            ('ms200',
                                                                                             3),
                                                                                            ('ms500',
                                                                                             4),
                                                                                            ('ms1000',
                                                                                             5),
                                                                                            ('ms2000',
                                                                                             6),
                                                                                            ('spare1',
                                                                                             7)]},
                                                                                {'name': 'ue-IdentitySCG-r12',
                                                                                 'optional': True,
                                                                                 'type': 'C-RNTI'},
                                                                                {'name': 'rach-ConfigDedicated-r12',
                                                                                 'optional': True,
                                                                                 'type': 'RACH-ConfigDedicated'},
                                                                                {'name': 'cipheringAlgorithmSCG-r12',
                                                                                 'optional': True,
                                                                                 'type': 'CipheringAlgorithm-r12'},
                                                                                None,
                                                                                [{'name': 'makeBeforeBreakSCG-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('true',
                                                                                              0)]},
                                                                                 {'name': 'rach-SkipSCG-r14',
                                                                                  'optional': True,
                                                                                  'type': 'RACH-Skip-r14'}]],
                                                                    'type': 'SEQUENCE'},
                                     'MobilityControlInfoV2X-r14': {'members': [{'name': 'v2x-CommTxPoolExceptional-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-CommResourcePoolV2X-r14'},
                                                                                {'name': 'v2x-CommRxPool-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-CommRxPoolListV2X-r14'},
                                                                                {'name': 'v2x-CommSyncConfig-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-SyncConfigListV2X-r14'},
                                                                                {'name': 'cbr-MobilityTxConfigList-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-CBR-CommonTxConfigList-r14'}],
                                                                    'type': 'SEQUENCE'},
                                     'MobilityFromEUTRACommand': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                               'type': 'RRC-TransactionIdentifier'},
                                                                              {'members': [{'members': [{'name': 'mobilityFromEUTRACommand-r8',
                                                                                                         'type': 'MobilityFromEUTRACommand-r8-IEs'},
                                                                                                        {'name': 'mobilityFromEUTRACommand-r9',
                                                                                                         'type': 'MobilityFromEUTRACommand-r9-IEs'},
                                                                                                        {'name': 'spare2',
                                                                                                         'type': 'NULL'},
                                                                                                        {'name': 'spare1',
                                                                                                         'type': 'NULL'}],
                                                                                            'name': 'c1',
                                                                                            'type': 'CHOICE'},
                                                                                           {'members': [],
                                                                                            'name': 'criticalExtensionsFuture',
                                                                                            'type': 'SEQUENCE'}],
                                                                               'name': 'criticalExtensions',
                                                                               'type': 'CHOICE'}],
                                                                  'type': 'SEQUENCE'},
                                     'MobilityFromEUTRACommand-r8-IEs': {'members': [{'name': 'cs-FallbackIndicator',
                                                                                      'type': 'BOOLEAN'},
                                                                                     {'members': [{'name': 'handover',
                                                                                                   'type': 'Handover'},
                                                                                                  {'name': 'cellChangeOrder',
                                                                                                   'type': 'CellChangeOrder'}],
                                                                                      'name': 'purpose',
                                                                                      'type': 'CHOICE'},
                                                                                     {'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'MobilityFromEUTRACommand-v8a0-IEs'}],
                                                                         'type': 'SEQUENCE'},
                                     'MobilityFromEUTRACommand-r9-IEs': {'members': [{'name': 'cs-FallbackIndicator',
                                                                                      'type': 'BOOLEAN'},
                                                                                     {'members': [{'name': 'handover',
                                                                                                   'type': 'Handover'},
                                                                                                  {'name': 'cellChangeOrder',
                                                                                                   'type': 'CellChangeOrder'},
                                                                                                  {'name': 'e-CSFB-r9',
                                                                                                   'type': 'E-CSFB-r9'},
                                                                                                  None],
                                                                                      'name': 'purpose',
                                                                                      'type': 'CHOICE'},
                                                                                     {'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'MobilityFromEUTRACommand-v930-IEs'}],
                                                                         'type': 'SEQUENCE'},
                                     'MobilityFromEUTRACommand-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'OCTET '
                                                                                                'STRING'},
                                                                                       {'name': 'nonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'MobilityFromEUTRACommand-v8d0-IEs'}],
                                                                           'type': 'SEQUENCE'},
                                     'MobilityFromEUTRACommand-v8d0-IEs': {'members': [{'name': 'bandIndicator',
                                                                                        'optional': True,
                                                                                        'type': 'BandIndicatorGERAN'},
                                                                                       {'members': [],
                                                                                        'name': 'nonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'}],
                                                                           'type': 'SEQUENCE'},
                                     'MobilityFromEUTRACommand-v930-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'OCTET '
                                                                                                'STRING'},
                                                                                       {'name': 'nonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'MobilityFromEUTRACommand-v960-IEs'}],
                                                                           'type': 'SEQUENCE'},
                                     'MobilityFromEUTRACommand-v960-IEs': {'members': [{'name': 'bandIndicator',
                                                                                        'optional': True,
                                                                                        'type': 'BandIndicatorGERAN'},
                                                                                       {'members': [],
                                                                                        'name': 'nonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'}],
                                                                           'type': 'SEQUENCE'},
                                     'MobilityHistoryReport-r12': {'type': 'VisitedCellInfoList-r12'},
                                     'MobilityParameters-r14': {'members': [{'name': 'makeBeforeBreak-r14',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]},
                                                                            {'name': 'rach-Less-r14',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]}],
                                                                'type': 'SEQUENCE'},
                                     'MobilityParametersCDMA2000': {'type': 'OCTET '
                                                                            'STRING'},
                                     'MobilityStateParameters': {'members': [{'name': 't-Evaluation',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('s30',
                                                                                          0),
                                                                                         ('s60',
                                                                                          1),
                                                                                         ('s120',
                                                                                          2),
                                                                                         ('s180',
                                                                                          3),
                                                                                         ('s240',
                                                                                          4),
                                                                                         ('spare3',
                                                                                          5),
                                                                                         ('spare2',
                                                                                          6),
                                                                                         ('spare1',
                                                                                          7)]},
                                                                             {'name': 't-HystNormal',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('s30',
                                                                                          0),
                                                                                         ('s60',
                                                                                          1),
                                                                                         ('s120',
                                                                                          2),
                                                                                         ('s180',
                                                                                          3),
                                                                                         ('s240',
                                                                                          4),
                                                                                         ('spare3',
                                                                                          5),
                                                                                         ('spare2',
                                                                                          6),
                                                                                         ('spare1',
                                                                                          7)]},
                                                                             {'name': 'n-CellChangeMedium',
                                                                              'restricted-to': [(1,
                                                                                                 16)],
                                                                              'type': 'INTEGER'},
                                                                             {'name': 'n-CellChangeHigh',
                                                                              'restricted-to': [(1,
                                                                                                 16)],
                                                                              'type': 'INTEGER'}],
                                                                 'type': 'SEQUENCE'},
                                     'MultiBandInfo-v9e0': {'members': [{'name': 'freqBandIndicator-v9e0',
                                                                         'optional': True,
                                                                         'type': 'FreqBandIndicator-v9e0'}],
                                                            'type': 'SEQUENCE'},
                                     'MultiBandInfoList': {'element': {'type': 'FreqBandIndicator'},
                                                           'size': [(1,
                                                                     'maxMultiBands')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'MultiBandInfoList-r11': {'element': {'type': 'FreqBandIndicator-r11'},
                                                               'size': [(1,
                                                                         'maxMultiBands')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'MultiBandInfoList-v10j0': {'element': {'type': 'NS-PmaxList-r10'},
                                                                 'size': [(1,
                                                                           'maxMultiBands')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'MultiBandInfoList-v10l0': {'element': {'type': 'NS-PmaxList-v10l0'},
                                                                 'size': [(1,
                                                                           'maxMultiBands')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'MultiBandInfoList-v9e0': {'element': {'type': 'MultiBandInfo-v9e0'},
                                                                'size': [(1,
                                                                          'maxMultiBands')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'N1PUCCH-AN-CS-r10': {'element': {'restricted-to': [(0,
                                                                                          2047)],
                                                                       'type': 'INTEGER'},
                                                           'size': [(1, 4)],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'N1PUCCH-AN-InfoList-r13': {'element': {'restricted-to': [(0,
                                                                                                2047)],
                                                                             'type': 'INTEGER'},
                                                                 'size': [(1,
                                                                           'maxCE-Level-r13')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'N1PUCCH-AN-PersistentList': {'element': {'restricted-to': [(0,
                                                                                                  2047)],
                                                                               'type': 'INTEGER'},
                                                                   'size': [(1,
                                                                             4)],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'NAICS-AssistanceInfo-r12': {'members': [{'name': 'release',
                                                                               'type': 'NULL'},
                                                                              {'members': [{'name': 'neighCellsToReleaseList-r12',
                                                                                            'optional': True,
                                                                                            'type': 'NeighCellsToReleaseList-r12'},
                                                                                           {'name': 'neighCellsToAddModList-r12',
                                                                                            'optional': True,
                                                                                            'type': 'NeighCellsToAddModList-r12'},
                                                                                           {'name': 'servCellp-a-r12',
                                                                                            'optional': True,
                                                                                            'type': 'P-a'}],
                                                                               'name': 'setup',
                                                                               'type': 'SEQUENCE'}],
                                                                  'type': 'CHOICE'},
                                     'NAICS-Capability-Entry-r12': {'members': [{'name': 'numberOfNAICS-CapableCC-r12',
                                                                                 'restricted-to': [(1,
                                                                                                    5)],
                                                                                 'type': 'INTEGER'},
                                                                                {'name': 'numberOfAggregatedPRB-r12',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('n50',
                                                                                             0),
                                                                                            ('n75',
                                                                                             1),
                                                                                            ('n100',
                                                                                             2),
                                                                                            ('n125',
                                                                                             3),
                                                                                            ('n150',
                                                                                             4),
                                                                                            ('n175',
                                                                                             5),
                                                                                            ('n200',
                                                                                             6),
                                                                                            ('n225',
                                                                                             7),
                                                                                            ('n250',
                                                                                             8),
                                                                                            ('n275',
                                                                                             9),
                                                                                            ('n300',
                                                                                             10),
                                                                                            ('n350',
                                                                                             11),
                                                                                            ('n400',
                                                                                             12),
                                                                                            ('n450',
                                                                                             13),
                                                                                            ('n500',
                                                                                             14),
                                                                                            ('spare',
                                                                                             15)]},
                                                                                None],
                                                                    'type': 'SEQUENCE'},
                                     'NAICS-Capability-List-r12': {'element': {'type': 'NAICS-Capability-Entry-r12'},
                                                                   'size': [(1,
                                                                             'maxNAICS-Entries-r12')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'NS-PmaxList-r10': {'element': {'type': 'NS-PmaxValue-r10'},
                                                         'size': [(1,
                                                                   'maxNS-Pmax-r10')],
                                                         'type': 'SEQUENCE OF'},
                                     'NS-PmaxList-v10l0': {'element': {'type': 'NS-PmaxValue-v10l0'},
                                                           'size': [(1,
                                                                     'maxNS-Pmax-r10')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'NS-PmaxValue-r10': {'members': [{'name': 'additionalPmax-r10',
                                                                       'optional': True,
                                                                       'type': 'P-Max'},
                                                                      {'name': 'additionalSpectrumEmission',
                                                                       'type': 'AdditionalSpectrumEmission'}],
                                                          'type': 'SEQUENCE'},
                                     'NS-PmaxValue-v10l0': {'members': [{'name': 'additionalSpectrumEmission-v10l0',
                                                                         'optional': True,
                                                                         'type': 'AdditionalSpectrumEmission-v10l0'}],
                                                            'type': 'SEQUENCE'},
                                     'NZP-FrequencyDensity-r14': {'type': 'ENUMERATED',
                                                                  'values': [('d1',
                                                                              0),
                                                                             ('d2',
                                                                              1),
                                                                             ('d3',
                                                                              2)]},
                                     'NZP-ResourceConfig-r13': {'members': [{'name': 'resourceConfig-r13',
                                                                             'type': 'ResourceConfig-r13'},
                                                                            None,
                                                                            [{'name': 'transmissionComb-r14',
                                                                              'optional': True,
                                                                              'type': 'NZP-TransmissionComb-r14'},
                                                                             {'name': 'frequencyDensity-r14',
                                                                              'optional': True,
                                                                              'type': 'NZP-FrequencyDensity-r14'}]],
                                                                'type': 'SEQUENCE'},
                                     'NZP-TransmissionComb-r14': {'restricted-to': [(0,
                                                                                     2)],
                                                                  'type': 'INTEGER'},
                                     'NeighCellCDMA2000': {'members': [{'name': 'bandClass',
                                                                        'type': 'BandclassCDMA2000'},
                                                                       {'name': 'neighCellsPerFreqList',
                                                                        'type': 'NeighCellsPerBandclassListCDMA2000'}],
                                                           'type': 'SEQUENCE'},
                                     'NeighCellCDMA2000-r11': {'members': [{'name': 'bandClass',
                                                                            'type': 'BandclassCDMA2000'},
                                                                           {'element': {'type': 'NeighCellsPerBandclassCDMA2000-r11'},
                                                                            'name': 'neighFreqInfoList-r11',
                                                                            'size': [(1,
                                                                                      16)],
                                                                            'type': 'SEQUENCE '
                                                                                    'OF'}],
                                                               'type': 'SEQUENCE'},
                                     'NeighCellCDMA2000-v920': {'members': [{'name': 'neighCellsPerFreqList-v920',
                                                                             'type': 'NeighCellsPerBandclassListCDMA2000-v920'}],
                                                                'type': 'SEQUENCE'},
                                     'NeighCellConfig': {'size': [2],
                                                         'type': 'BIT STRING'},
                                     'NeighCellListCDMA2000': {'element': {'type': 'NeighCellCDMA2000'},
                                                               'size': [(1,
                                                                         16)],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'NeighCellListCDMA2000-v920': {'element': {'type': 'NeighCellCDMA2000-v920'},
                                                                    'size': [(1,
                                                                              16)],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'NeighCellSI-AcquisitionParameters-r9': {'members': [{'name': 'intraFreqSI-AcquisitionForHO-r9',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('supported',
                                                                                                       0)]},
                                                                                          {'name': 'interFreqSI-AcquisitionForHO-r9',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('supported',
                                                                                                       0)]},
                                                                                          {'name': 'utran-SI-AcquisitionForHO-r9',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('supported',
                                                                                                       0)]}],
                                                                              'type': 'SEQUENCE'},
                                     'NeighCellsCRS-Info-r11': {'members': [{'name': 'release',
                                                                             'type': 'NULL'},
                                                                            {'name': 'setup',
                                                                             'type': 'CRS-AssistanceInfoList-r11'}],
                                                                'type': 'CHOICE'},
                                     'NeighCellsCRS-Info-r13': {'members': [{'name': 'release',
                                                                             'type': 'NULL'},
                                                                            {'name': 'setup',
                                                                             'type': 'CRS-AssistanceInfoList-r13'}],
                                                                'type': 'CHOICE'},
                                     'NeighCellsInfo-r12': {'members': [{'name': 'physCellId-r12',
                                                                         'type': 'PhysCellId'},
                                                                        {'name': 'p-b-r12',
                                                                         'restricted-to': [(0,
                                                                                            3)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'crs-PortsCount-r12',
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('n1',
                                                                                     0),
                                                                                    ('n2',
                                                                                     1),
                                                                                    ('n4',
                                                                                     2),
                                                                                    ('spare',
                                                                                     3)]},
                                                                        {'name': 'mbsfn-SubframeConfig-r12',
                                                                         'optional': True,
                                                                         'type': 'MBSFN-SubframeConfigList'},
                                                                        {'element': {'type': 'P-a'},
                                                                         'name': 'p-aList-r12',
                                                                         'size': [(1,
                                                                                   'maxP-a-PerNeighCell-r12')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                                                        {'name': 'transmissionModeList-r12',
                                                                         'size': [8],
                                                                         'type': 'BIT '
                                                                                 'STRING'},
                                                                        {'name': 'resAllocGranularity-r12',
                                                                         'restricted-to': [(1,
                                                                                            4)],
                                                                         'type': 'INTEGER'},
                                                                        None],
                                                            'type': 'SEQUENCE'},
                                     'NeighCellsPerBandclassCDMA2000': {'members': [{'name': 'arfcn',
                                                                                     'type': 'ARFCN-ValueCDMA2000'},
                                                                                    {'name': 'physCellIdList',
                                                                                     'type': 'PhysCellIdListCDMA2000'}],
                                                                        'type': 'SEQUENCE'},
                                     'NeighCellsPerBandclassCDMA2000-r11': {'members': [{'name': 'arfcn',
                                                                                         'type': 'ARFCN-ValueCDMA2000'},
                                                                                        {'element': {'type': 'PhysCellIdCDMA2000'},
                                                                                         'name': 'physCellIdList-r11',
                                                                                         'size': [(1,
                                                                                                   40)],
                                                                                         'type': 'SEQUENCE '
                                                                                                 'OF'}],
                                                                            'type': 'SEQUENCE'},
                                     'NeighCellsPerBandclassCDMA2000-v920': {'members': [{'name': 'physCellIdList-v920',
                                                                                          'type': 'PhysCellIdListCDMA2000-v920'}],
                                                                             'type': 'SEQUENCE'},
                                     'NeighCellsPerBandclassListCDMA2000': {'element': {'type': 'NeighCellsPerBandclassCDMA2000'},
                                                                            'size': [(1,
                                                                                      16)],
                                                                            'type': 'SEQUENCE '
                                                                                    'OF'},
                                     'NeighCellsPerBandclassListCDMA2000-v920': {'element': {'type': 'NeighCellsPerBandclassCDMA2000-v920'},
                                                                                 'size': [(1,
                                                                                           16)],
                                                                                 'type': 'SEQUENCE '
                                                                                         'OF'},
                                     'NeighCellsToAddModList-r12': {'element': {'type': 'NeighCellsInfo-r12'},
                                                                    'size': [(1,
                                                                              'maxNeighCell-r12')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'NeighCellsToReleaseList-r12': {'element': {'type': 'PhysCellId'},
                                                                     'size': [(1,
                                                                               'maxNeighCell-r12')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'NextHopChainingCount': {'restricted-to': [(0,
                                                                                 7)],
                                                              'type': 'INTEGER'},
                                     'NonContiguousUL-RA-WithinCC-List-r10': {'element': {'type': 'NonContiguousUL-RA-WithinCC-r10'},
                                                                              'size': [(1,
                                                                                        'maxBands')],
                                                                              'type': 'SEQUENCE '
                                                                                      'OF'},
                                     'NonContiguousUL-RA-WithinCC-r10': {'members': [{'name': 'nonContiguousUL-RA-WithinCC-Info-r10',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('supported',
                                                                                                  0)]}],
                                                                         'type': 'SEQUENCE'},
                                     'NonMBSFN-SubframeConfig-r14': {'members': [{'name': 'radioFrameAllocationPeriod-r14',
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('rf4',
                                                                                              0),
                                                                                             ('rf8',
                                                                                              1),
                                                                                             ('rf16',
                                                                                              2),
                                                                                             ('rf32',
                                                                                              3),
                                                                                             ('rf64',
                                                                                              4),
                                                                                             ('rf128',
                                                                                              5),
                                                                                             ('rf512',
                                                                                              6)]},
                                                                                 {'name': 'radioFrameAllocationOffset-r14',
                                                                                  'restricted-to': [(0,
                                                                                                     7)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'name': 'subframeAllocation-r14',
                                                                                  'size': [9],
                                                                                  'type': 'BIT '
                                                                                          'STRING'}],
                                                                     'type': 'SEQUENCE'},
                                     'NumberOfPreamblesSent-r11': {'restricted-to': [(1,
                                                                                      200)],
                                                                   'type': 'INTEGER'},
                                     'OTDOA-PositioningCapabilities-r10': {'members': [{'name': 'otdoa-UE-Assisted-r10',
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('supported',
                                                                                                    0)]},
                                                                                       {'name': 'interFreqRSTD-Measurement-r10',
                                                                                        'optional': True,
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('supported',
                                                                                                    0)]}],
                                                                           'type': 'SEQUENCE'},
                                     'ObtainLocationConfig-r11': {'members': [{'name': 'obtainLocation-r11',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('setup',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'Other-Parameters-r11': {'members': [{'name': 'inDeviceCoexInd-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'powerPrefInd-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'ue-Rx-TxTimeDiffMeasurements-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'Other-Parameters-v11d0': {'members': [{'name': 'inDeviceCoexInd-UL-CA-r11',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]}],
                                                                'type': 'SEQUENCE'},
                                     'Other-Parameters-v1360': {'members': [{'name': 'inDeviceCoexInd-HardwareSharingInd-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]}],
                                                                'type': 'SEQUENCE'},
                                     'Other-Parameters-v1430': {'members': [{'name': 'bwPrefInd-r14',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]},
                                                                            {'name': 'rlm-ReportSupport-r14',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]}],
                                                                'type': 'SEQUENCE'},
                                     'OtherConfig-r9': {'members': [{'name': 'reportProximityConfig-r9',
                                                                     'optional': True,
                                                                     'type': 'ReportProximityConfig-r9'},
                                                                    None,
                                                                    [{'name': 'idc-Config-r11',
                                                                      'optional': True,
                                                                      'type': 'IDC-Config-r11'},
                                                                     {'name': 'powerPrefIndicationConfig-r11',
                                                                      'optional': True,
                                                                      'type': 'PowerPrefIndicationConfig-r11'},
                                                                     {'name': 'obtainLocationConfig-r11',
                                                                      'optional': True,
                                                                      'type': 'ObtainLocationConfig-r11'}],
                                                                    [{'name': 'bw-PreferenceIndicationTimer-r14',
                                                                      'optional': True,
                                                                      'type': 'ENUMERATED',
                                                                      'values': [('s0',
                                                                                  0),
                                                                                 ('s0dot5',
                                                                                  1),
                                                                                 ('s1',
                                                                                  2),
                                                                                 ('s2',
                                                                                  3),
                                                                                 ('s5',
                                                                                  4),
                                                                                 ('s10',
                                                                                  5),
                                                                                 ('s20',
                                                                                  6),
                                                                                 ('s30',
                                                                                  7),
                                                                                 ('s60',
                                                                                  8),
                                                                                 ('s90',
                                                                                  9),
                                                                                 ('s120',
                                                                                  10),
                                                                                 ('s300',
                                                                                  11),
                                                                                 ('s600',
                                                                                  12),
                                                                                 ('spare3',
                                                                                  13),
                                                                                 ('spare2',
                                                                                  14),
                                                                                 ('spare1',
                                                                                  15)]},
                                                                     {'name': 'sps-AssistanceInfoReport-r14',
                                                                      'optional': True,
                                                                      'type': 'BOOLEAN'},
                                                                     {'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'members': [{'name': 'delayBudgetReportingProhibitTimer-r14',
                                                                                                'type': 'ENUMERATED',
                                                                                                'values': [('s0',
                                                                                                            0),
                                                                                                           ('s0dot4',
                                                                                                            1),
                                                                                                           ('s0dot8',
                                                                                                            2),
                                                                                                           ('s1dot6',
                                                                                                            3),
                                                                                                           ('s3',
                                                                                                            4),
                                                                                                           ('s6',
                                                                                                            5),
                                                                                                           ('s12',
                                                                                                            6),
                                                                                                           ('s30',
                                                                                                            7)]}],
                                                                                   'name': 'setup',
                                                                                   'type': 'SEQUENCE'}],
                                                                      'name': 'delayBudgetReportingConfig-r14',
                                                                      'optional': True,
                                                                      'type': 'CHOICE'},
                                                                     {'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'members': [{'name': 'rlmReportTimer-r14',
                                                                                                'type': 'ENUMERATED',
                                                                                                'values': [('s0',
                                                                                                            0),
                                                                                                           ('s0dot5',
                                                                                                            1),
                                                                                                           ('s1',
                                                                                                            2),
                                                                                                           ('s2',
                                                                                                            3),
                                                                                                           ('s5',
                                                                                                            4),
                                                                                                           ('s10',
                                                                                                            5),
                                                                                                           ('s20',
                                                                                                            6),
                                                                                                           ('s30',
                                                                                                            7),
                                                                                                           ('s60',
                                                                                                            8),
                                                                                                           ('s90',
                                                                                                            9),
                                                                                                           ('s120',
                                                                                                            10),
                                                                                                           ('s300',
                                                                                                            11),
                                                                                                           ('s600',
                                                                                                            12),
                                                                                                           ('spare3',
                                                                                                            13),
                                                                                                           ('spare2',
                                                                                                            14),
                                                                                                           ('spare1',
                                                                                                            15)]},
                                                                                               {'name': 'rlmReportRep-MPDCCH-r14',
                                                                                                'optional': True,
                                                                                                'type': 'ENUMERATED',
                                                                                                'values': [('setup',
                                                                                                            0)]}],
                                                                                   'name': 'setup',
                                                                                   'type': 'SEQUENCE'}],
                                                                      'name': 'rlm-ReportConfig-r14',
                                                                      'optional': True,
                                                                      'type': 'CHOICE'}]],
                                                        'type': 'SEQUENCE'},
                                     'P-C-AndCBSR-Pair-r13': {'element': {'type': 'P-C-AndCBSR-r13'},
                                                              'size': [(1, 2)],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'P-C-AndCBSR-Pair-r13a': {'element': {'type': 'P-C-AndCBSR-r11'},
                                                               'size': [(1, 2)],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'P-C-AndCBSR-r11': {'members': [{'name': 'p-C-r11',
                                                                      'restricted-to': [(-8,
                                                                                         15)],
                                                                      'type': 'INTEGER'},
                                                                     {'name': 'codebookSubsetRestriction-r11',
                                                                      'type': 'BIT '
                                                                              'STRING'}],
                                                         'type': 'SEQUENCE'},
                                     'P-C-AndCBSR-r13': {'members': [{'name': 'p-C-r13',
                                                                      'restricted-to': [(-8,
                                                                                         15)],
                                                                      'type': 'INTEGER'},
                                                                     {'members': [{'members': [{'name': 'codebookSubsetRestriction1-r13',
                                                                                                'type': 'BIT '
                                                                                                        'STRING'},
                                                                                               {'name': 'codebookSubsetRestriction2-r13',
                                                                                                'type': 'BIT '
                                                                                                        'STRING'}],
                                                                                   'name': 'nonPrecoded-r13',
                                                                                   'type': 'SEQUENCE'},
                                                                                  {'members': [{'name': 'codebookSubsetRestriction3-r13',
                                                                                                'type': 'BIT '
                                                                                                        'STRING'}],
                                                                                   'name': 'beamformedK1a-r13',
                                                                                   'type': 'SEQUENCE'},
                                                                                  {'members': [{'name': 'codebookSubsetRestriction-r13',
                                                                                                'type': 'BIT '
                                                                                                        'STRING'}],
                                                                                   'name': 'beamformedKN-r13',
                                                                                   'type': 'SEQUENCE'}],
                                                                      'name': 'cbsr-Selection-r13',
                                                                      'type': 'CHOICE'},
                                                                     None],
                                                         'type': 'SEQUENCE'},
                                     'P-Max': {'restricted-to': [(-30, 33)],
                                               'type': 'INTEGER'},
                                     'P-a': {'type': 'ENUMERATED',
                                             'values': [('dB-6', 0),
                                                        ('dB-4dot77', 1),
                                                        ('dB-3', 2),
                                                        ('dB-1dot77', 3),
                                                        ('dB0', 4),
                                                        ('dB1', 5),
                                                        ('dB2', 6),
                                                        ('dB3', 7)]},
                                     'P0-SL-r12': {'restricted-to': [(-126,
                                                                      31)],
                                                   'type': 'INTEGER'},
                                     'PCCH-Config': {'members': [{'name': 'defaultPagingCycle',
                                                                  'type': 'ENUMERATED',
                                                                  'values': [('rf32',
                                                                              0),
                                                                             ('rf64',
                                                                              1),
                                                                             ('rf128',
                                                                              2),
                                                                             ('rf256',
                                                                              3)]},
                                                                 {'name': 'nB',
                                                                  'type': 'ENUMERATED',
                                                                  'values': [('fourT',
                                                                              0),
                                                                             ('twoT',
                                                                              1),
                                                                             ('oneT',
                                                                              2),
                                                                             ('halfT',
                                                                              3),
                                                                             ('quarterT',
                                                                              4),
                                                                             ('oneEighthT',
                                                                              5),
                                                                             ('oneSixteenthT',
                                                                              6),
                                                                             ('oneThirtySecondT',
                                                                              7)]}],
                                                     'type': 'SEQUENCE'},
                                     'PCCH-Config-v1310': {'members': [{'name': 'paging-narrowBands-r13',
                                                                        'restricted-to': [(1,
                                                                                           'maxAvailNarrowBands-r13')],
                                                                        'type': 'INTEGER'},
                                                                       {'name': 'mpdcch-NumRepetition-Paging-r13',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('r1',
                                                                                    0),
                                                                                   ('r2',
                                                                                    1),
                                                                                   ('r4',
                                                                                    2),
                                                                                   ('r8',
                                                                                    3),
                                                                                   ('r16',
                                                                                    4),
                                                                                   ('r32',
                                                                                    5),
                                                                                   ('r64',
                                                                                    6),
                                                                                   ('r128',
                                                                                    7),
                                                                                   ('r256',
                                                                                    8)]},
                                                                       {'name': 'nB-v1310',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('one64thT',
                                                                                    0),
                                                                                   ('one128thT',
                                                                                    1),
                                                                                   ('one256thT',
                                                                                    2)]}],
                                                           'type': 'SEQUENCE'},
                                     'PCCH-Message': {'members': [{'name': 'message',
                                                                   'type': 'PCCH-MessageType'}],
                                                      'type': 'SEQUENCE'},
                                     'PCCH-MessageType': {'members': [{'members': [{'name': 'paging',
                                                                                    'type': 'Paging'}],
                                                                       'name': 'c1',
                                                                       'type': 'CHOICE'},
                                                                      {'members': [],
                                                                       'name': 'messageClassExtension',
                                                                       'type': 'SEQUENCE'}],
                                                          'type': 'CHOICE'},
                                     'PCI-ARFCN-r13': {'members': [{'name': 'physCellId-r13',
                                                                    'type': 'PhysCellId'},
                                                                   {'name': 'carrierFreq-r13',
                                                                    'optional': True,
                                                                    'type': 'ARFCN-ValueEUTRA-r9'}],
                                                       'type': 'SEQUENCE'},
                                     'PDCCH-CandidateReductionValue-r13': {'type': 'ENUMERATED',
                                                                           'values': [('n0',
                                                                                       0),
                                                                                      ('n33',
                                                                                       1),
                                                                                      ('n66',
                                                                                       2),
                                                                                      ('n100',
                                                                                       3)]},
                                     'PDCCH-CandidateReductionValue-r14': {'type': 'ENUMERATED',
                                                                           'values': [('n0',
                                                                                       0),
                                                                                      ('n50',
                                                                                       1),
                                                                                      ('n100',
                                                                                       2),
                                                                                      ('n150',
                                                                                       3)]},
                                     'PDCCH-CandidateReductions-r13': {'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'members': [{'name': 'pdcch-candidateReductionAL1-r13',
                                                                                                 'type': 'PDCCH-CandidateReductionValue-r13'},
                                                                                                {'name': 'pdcch-candidateReductionAL2-r13',
                                                                                                 'type': 'PDCCH-CandidateReductionValue-r13'},
                                                                                                {'name': 'pdcch-candidateReductionAL3-r13',
                                                                                                 'type': 'PDCCH-CandidateReductionValue-r13'},
                                                                                                {'name': 'pdcch-candidateReductionAL4-r13',
                                                                                                 'type': 'PDCCH-CandidateReductionValue-r13'},
                                                                                                {'name': 'pdcch-candidateReductionAL5-r13',
                                                                                                 'type': 'PDCCH-CandidateReductionValue-r13'}],
                                                                                    'name': 'setup',
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'CHOICE'},
                                     'PDCCH-CandidateReductionsLAA-UL-r14': {'members': [{'name': 'release',
                                                                                          'type': 'NULL'},
                                                                                         {'members': [{'name': 'pdcch-candidateReductionAL1-r14',
                                                                                                       'type': 'PDCCH-CandidateReductionValue-r13'},
                                                                                                      {'name': 'pdcch-candidateReductionAL2-r14',
                                                                                                       'type': 'PDCCH-CandidateReductionValue-r13'},
                                                                                                      {'name': 'pdcch-candidateReductionAL3-r14',
                                                                                                       'type': 'PDCCH-CandidateReductionValue-r14'},
                                                                                                      {'name': 'pdcch-candidateReductionAL4-r14',
                                                                                                       'type': 'PDCCH-CandidateReductionValue-r14'},
                                                                                                      {'name': 'pdcch-candidateReductionAL5-r14',
                                                                                                       'type': 'PDCCH-CandidateReductionValue-r14'}],
                                                                                          'name': 'setup',
                                                                                          'type': 'SEQUENCE'}],
                                                                             'type': 'CHOICE'},
                                     'PDCCH-ConfigLAA-r14': {'members': [{'name': 'maxNumberOfSchedSubframes-Format0B-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('sf2',
                                                                                      0),
                                                                                     ('sf3',
                                                                                      1),
                                                                                     ('sf4',
                                                                                      2)]},
                                                                         {'name': 'maxNumberOfSchedSubframes-Format4B-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('sf2',
                                                                                      0),
                                                                                     ('sf3',
                                                                                      1),
                                                                                     ('sf4',
                                                                                      2)]},
                                                                         {'name': 'skipMonitoringDCI-Format0A-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('true',
                                                                                      0)]},
                                                                         {'name': 'skipMonitoringDCI-Format4A-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('true',
                                                                                      0)]},
                                                                         {'name': 'pdcch-CandidateReductions-Format0A-r14',
                                                                          'optional': True,
                                                                          'type': 'PDCCH-CandidateReductions-r13'},
                                                                         {'name': 'pdcch-CandidateReductions-Format4A-r14',
                                                                          'optional': True,
                                                                          'type': 'PDCCH-CandidateReductionsLAA-UL-r14'},
                                                                         {'name': 'pdcch-CandidateReductions-Format0B-r14',
                                                                          'optional': True,
                                                                          'type': 'PDCCH-CandidateReductionsLAA-UL-r14'},
                                                                         {'name': 'pdcch-CandidateReductions-Format4B-r14',
                                                                          'optional': True,
                                                                          'type': 'PDCCH-CandidateReductionsLAA-UL-r14'}],
                                                             'type': 'SEQUENCE'},
                                     'PDCCH-ConfigSCell-r13': {'members': [{'name': 'skipMonitoringDCI-format0-1A-r13',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('true',
                                                                                        0)]}],
                                                               'type': 'SEQUENCE'},
                                     'PDCP-Config': {'members': [{'name': 'discardTimer',
                                                                  'optional': True,
                                                                  'type': 'ENUMERATED',
                                                                  'values': [('ms50',
                                                                              0),
                                                                             ('ms100',
                                                                              1),
                                                                             ('ms150',
                                                                              2),
                                                                             ('ms300',
                                                                              3),
                                                                             ('ms500',
                                                                              4),
                                                                             ('ms750',
                                                                              5),
                                                                             ('ms1500',
                                                                              6),
                                                                             ('infinity',
                                                                              7)]},
                                                                 {'members': [{'name': 'statusReportRequired',
                                                                               'type': 'BOOLEAN'}],
                                                                  'name': 'rlc-AM',
                                                                  'optional': True,
                                                                  'type': 'SEQUENCE'},
                                                                 {'members': [{'name': 'pdcp-SN-Size',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('len7bits',
                                                                                           0),
                                                                                          ('len12bits',
                                                                                           1)]}],
                                                                  'name': 'rlc-UM',
                                                                  'optional': True,
                                                                  'type': 'SEQUENCE'},
                                                                 {'members': [{'name': 'notUsed',
                                                                               'type': 'NULL'},
                                                                              {'members': [{'default': 15,
                                                                                            'name': 'maxCID',
                                                                                            'restricted-to': [(1,
                                                                                                               16383)],
                                                                                            'type': 'INTEGER'},
                                                                                           {'members': [{'name': 'profile0x0001',
                                                                                                         'type': 'BOOLEAN'},
                                                                                                        {'name': 'profile0x0002',
                                                                                                         'type': 'BOOLEAN'},
                                                                                                        {'name': 'profile0x0003',
                                                                                                         'type': 'BOOLEAN'},
                                                                                                        {'name': 'profile0x0004',
                                                                                                         'type': 'BOOLEAN'},
                                                                                                        {'name': 'profile0x0006',
                                                                                                         'type': 'BOOLEAN'},
                                                                                                        {'name': 'profile0x0101',
                                                                                                         'type': 'BOOLEAN'},
                                                                                                        {'name': 'profile0x0102',
                                                                                                         'type': 'BOOLEAN'},
                                                                                                        {'name': 'profile0x0103',
                                                                                                         'type': 'BOOLEAN'},
                                                                                                        {'name': 'profile0x0104',
                                                                                                         'type': 'BOOLEAN'}],
                                                                                            'name': 'profiles',
                                                                                            'type': 'SEQUENCE'},
                                                                                           None],
                                                                               'name': 'rohc',
                                                                               'type': 'SEQUENCE'}],
                                                                  'name': 'headerCompression',
                                                                  'type': 'CHOICE'},
                                                                 None,
                                                                 [{'name': 'rn-IntegrityProtection-r10',
                                                                   'optional': True,
                                                                   'type': 'ENUMERATED',
                                                                   'values': [('enabled',
                                                                               0)]}],
                                                                 [{'name': 'pdcp-SN-Size-v1130',
                                                                   'optional': True,
                                                                   'type': 'ENUMERATED',
                                                                   'values': [('len15bits',
                                                                               0)]}],
                                                                 [{'name': 'ul-DataSplitDRB-ViaSCG-r12',
                                                                   'optional': True,
                                                                   'type': 'BOOLEAN'},
                                                                  {'name': 't-Reordering-r12',
                                                                   'optional': True,
                                                                   'type': 'ENUMERATED',
                                                                   'values': [('ms0',
                                                                               0),
                                                                              ('ms20',
                                                                               1),
                                                                              ('ms40',
                                                                               2),
                                                                              ('ms60',
                                                                               3),
                                                                              ('ms80',
                                                                               4),
                                                                              ('ms100',
                                                                               5),
                                                                              ('ms120',
                                                                               6),
                                                                              ('ms140',
                                                                               7),
                                                                              ('ms160',
                                                                               8),
                                                                              ('ms180',
                                                                               9),
                                                                              ('ms200',
                                                                               10),
                                                                              ('ms220',
                                                                               11),
                                                                              ('ms240',
                                                                               12),
                                                                              ('ms260',
                                                                               13),
                                                                              ('ms280',
                                                                               14),
                                                                              ('ms300',
                                                                               15),
                                                                              ('ms500',
                                                                               16),
                                                                              ('ms750',
                                                                               17),
                                                                              ('spare14',
                                                                               18),
                                                                              ('spare13',
                                                                               19),
                                                                              ('spare12',
                                                                               20),
                                                                              ('spare11',
                                                                               21),
                                                                              ('spare10',
                                                                               22),
                                                                              ('spare9',
                                                                               23),
                                                                              ('spare8',
                                                                               24),
                                                                              ('spare7',
                                                                               25),
                                                                              ('spare6',
                                                                               26),
                                                                              ('spare5',
                                                                               27),
                                                                              ('spare4',
                                                                               28),
                                                                              ('spare3',
                                                                               29),
                                                                              ('spare2',
                                                                               30),
                                                                              ('spare1',
                                                                               31)]}],
                                                                 [{'members': [{'name': 'release',
                                                                                'type': 'NULL'},
                                                                               {'name': 'setup',
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('b0',
                                                                                            0),
                                                                                           ('b100',
                                                                                            1),
                                                                                           ('b200',
                                                                                            2),
                                                                                           ('b400',
                                                                                            3),
                                                                                           ('b800',
                                                                                            4),
                                                                                           ('b1600',
                                                                                            5),
                                                                                           ('b3200',
                                                                                            6),
                                                                                           ('b6400',
                                                                                            7),
                                                                                           ('b12800',
                                                                                            8),
                                                                                           ('b25600',
                                                                                            9),
                                                                                           ('b51200',
                                                                                            10),
                                                                                           ('b102400',
                                                                                            11),
                                                                                           ('b204800',
                                                                                            12),
                                                                                           ('b409600',
                                                                                            13),
                                                                                           ('b819200',
                                                                                            14),
                                                                                           ('spare1',
                                                                                            15)]}],
                                                                   'name': 'ul-DataSplitThreshold-r13',
                                                                   'optional': True,
                                                                   'type': 'CHOICE'},
                                                                  {'name': 'pdcp-SN-Size-v1310',
                                                                   'optional': True,
                                                                   'type': 'ENUMERATED',
                                                                   'values': [('len18bits',
                                                                               0)]},
                                                                  {'members': [{'name': 'release',
                                                                                'type': 'NULL'},
                                                                               {'members': [{'name': 'statusPDU-TypeForPolling-r13',
                                                                                             'optional': True,
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('type1',
                                                                                                         0),
                                                                                                        ('type2',
                                                                                                         1)]},
                                                                                            {'name': 'statusPDU-Periodicity-Type1-r13',
                                                                                             'optional': True,
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('ms5',
                                                                                                         0),
                                                                                                        ('ms10',
                                                                                                         1),
                                                                                                        ('ms20',
                                                                                                         2),
                                                                                                        ('ms30',
                                                                                                         3),
                                                                                                        ('ms40',
                                                                                                         4),
                                                                                                        ('ms50',
                                                                                                         5),
                                                                                                        ('ms60',
                                                                                                         6),
                                                                                                        ('ms70',
                                                                                                         7),
                                                                                                        ('ms80',
                                                                                                         8),
                                                                                                        ('ms90',
                                                                                                         9),
                                                                                                        ('ms100',
                                                                                                         10),
                                                                                                        ('ms150',
                                                                                                         11),
                                                                                                        ('ms200',
                                                                                                         12),
                                                                                                        ('ms300',
                                                                                                         13),
                                                                                                        ('ms500',
                                                                                                         14),
                                                                                                        ('ms1000',
                                                                                                         15),
                                                                                                        ('ms2000',
                                                                                                         16),
                                                                                                        ('ms5000',
                                                                                                         17),
                                                                                                        ('ms10000',
                                                                                                         18),
                                                                                                        ('ms20000',
                                                                                                         19),
                                                                                                        ('ms50000',
                                                                                                         20)]},
                                                                                            {'name': 'statusPDU-Periodicity-Type2-r13',
                                                                                             'optional': True,
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('ms5',
                                                                                                         0),
                                                                                                        ('ms10',
                                                                                                         1),
                                                                                                        ('ms20',
                                                                                                         2),
                                                                                                        ('ms30',
                                                                                                         3),
                                                                                                        ('ms40',
                                                                                                         4),
                                                                                                        ('ms50',
                                                                                                         5),
                                                                                                        ('ms60',
                                                                                                         6),
                                                                                                        ('ms70',
                                                                                                         7),
                                                                                                        ('ms80',
                                                                                                         8),
                                                                                                        ('ms90',
                                                                                                         9),
                                                                                                        ('ms100',
                                                                                                         10),
                                                                                                        ('ms150',
                                                                                                         11),
                                                                                                        ('ms200',
                                                                                                         12),
                                                                                                        ('ms300',
                                                                                                         13),
                                                                                                        ('ms500',
                                                                                                         14),
                                                                                                        ('ms1000',
                                                                                                         15),
                                                                                                        ('ms2000',
                                                                                                         16),
                                                                                                        ('ms5000',
                                                                                                         17),
                                                                                                        ('ms10000',
                                                                                                         18),
                                                                                                        ('ms20000',
                                                                                                         19),
                                                                                                        ('ms50000',
                                                                                                         20)]},
                                                                                            {'name': 'statusPDU-Periodicity-Offset-r13',
                                                                                             'optional': True,
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('ms1',
                                                                                                         0),
                                                                                                        ('ms2',
                                                                                                         1),
                                                                                                        ('ms5',
                                                                                                         2),
                                                                                                        ('ms10',
                                                                                                         3),
                                                                                                        ('ms25',
                                                                                                         4),
                                                                                                        ('ms50',
                                                                                                         5),
                                                                                                        ('ms100',
                                                                                                         6),
                                                                                                        ('ms250',
                                                                                                         7),
                                                                                                        ('ms500',
                                                                                                         8),
                                                                                                        ('ms2500',
                                                                                                         9),
                                                                                                        ('ms5000',
                                                                                                         10),
                                                                                                        ('ms25000',
                                                                                                         11)]}],
                                                                                'name': 'setup',
                                                                                'type': 'SEQUENCE'}],
                                                                   'name': 'statusFeedback-r13',
                                                                   'optional': True,
                                                                   'type': 'CHOICE'}],
                                                                 [{'members': [{'name': 'release',
                                                                                'type': 'NULL'},
                                                                               {'members': [{'name': 'ul-LWA-DRB-ViaWLAN-r14',
                                                                                             'type': 'BOOLEAN'},
                                                                                            {'name': 'ul-LWA-DataSplitThreshold-r14',
                                                                                             'optional': True,
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('b0',
                                                                                                         0),
                                                                                                        ('b100',
                                                                                                         1),
                                                                                                        ('b200',
                                                                                                         2),
                                                                                                        ('b400',
                                                                                                         3),
                                                                                                        ('b800',
                                                                                                         4),
                                                                                                        ('b1600',
                                                                                                         5),
                                                                                                        ('b3200',
                                                                                                         6),
                                                                                                        ('b6400',
                                                                                                         7),
                                                                                                        ('b12800',
                                                                                                         8),
                                                                                                        ('b25600',
                                                                                                         9),
                                                                                                        ('b51200',
                                                                                                         10),
                                                                                                        ('b102400',
                                                                                                         11),
                                                                                                        ('b204800',
                                                                                                         12),
                                                                                                        ('b409600',
                                                                                                         13),
                                                                                                        ('b819200',
                                                                                                         14)]}],
                                                                                'name': 'setup',
                                                                                'type': 'SEQUENCE'}],
                                                                   'name': 'ul-LWA-Config-r14',
                                                                   'optional': True,
                                                                   'type': 'CHOICE'},
                                                                  {'members': [{'name': 'notUsed-r14',
                                                                                'type': 'NULL'},
                                                                               {'members': [{'default': 15,
                                                                                             'name': 'maxCID-r14',
                                                                                             'restricted-to': [(1,
                                                                                                                16383)],
                                                                                             'type': 'INTEGER'},
                                                                                            {'members': [{'name': 'profile0x0006-r14',
                                                                                                          'type': 'BOOLEAN'}],
                                                                                             'name': 'profiles-r14',
                                                                                             'type': 'SEQUENCE'},
                                                                                            None],
                                                                                'name': 'rohc-r14',
                                                                                'type': 'SEQUENCE'}],
                                                                   'name': 'uplinkOnlyHeaderCompression-r14',
                                                                   'optional': True,
                                                                   'type': 'CHOICE'}]],
                                                     'type': 'SEQUENCE'},
                                     'PDCP-Parameters': {'members': [{'members': [{'name': 'profile0x0001',
                                                                                   'type': 'BOOLEAN'},
                                                                                  {'name': 'profile0x0002',
                                                                                   'type': 'BOOLEAN'},
                                                                                  {'name': 'profile0x0003',
                                                                                   'type': 'BOOLEAN'},
                                                                                  {'name': 'profile0x0004',
                                                                                   'type': 'BOOLEAN'},
                                                                                  {'name': 'profile0x0006',
                                                                                   'type': 'BOOLEAN'},
                                                                                  {'name': 'profile0x0101',
                                                                                   'type': 'BOOLEAN'},
                                                                                  {'name': 'profile0x0102',
                                                                                   'type': 'BOOLEAN'},
                                                                                  {'name': 'profile0x0103',
                                                                                   'type': 'BOOLEAN'},
                                                                                  {'name': 'profile0x0104',
                                                                                   'type': 'BOOLEAN'}],
                                                                      'name': 'supportedROHC-Profiles',
                                                                      'type': 'SEQUENCE'},
                                                                     {'default': 'cs16',
                                                                      'name': 'maxNumberROHC-ContextSessions',
                                                                      'type': 'ENUMERATED',
                                                                      'values': [('cs2',
                                                                                  0),
                                                                                 ('cs4',
                                                                                  1),
                                                                                 ('cs8',
                                                                                  2),
                                                                                 ('cs12',
                                                                                  3),
                                                                                 ('cs16',
                                                                                  4),
                                                                                 ('cs24',
                                                                                  5),
                                                                                 ('cs32',
                                                                                  6),
                                                                                 ('cs48',
                                                                                  7),
                                                                                 ('cs64',
                                                                                  8),
                                                                                 ('cs128',
                                                                                  9),
                                                                                 ('cs256',
                                                                                  10),
                                                                                 ('cs512',
                                                                                  11),
                                                                                 ('cs1024',
                                                                                  12),
                                                                                 ('cs16384',
                                                                                  13),
                                                                                 ('spare2',
                                                                                  14),
                                                                                 ('spare1',
                                                                                  15)]},
                                                                     None],
                                                         'type': 'SEQUENCE'},
                                     'PDCP-Parameters-v1130': {'members': [{'name': 'pdcp-SN-Extension-r11',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]},
                                                                           {'name': 'supportRohcContextContinue-r11',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]}],
                                                               'type': 'SEQUENCE'},
                                     'PDCP-Parameters-v1310': {'members': [{'name': 'pdcp-SN-Extension-18bits-r13',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]}],
                                                               'type': 'SEQUENCE'},
                                     'PDCP-Parameters-v1430': {'members': [{'members': [{'name': 'profile0x0006-r14',
                                                                                         'type': 'BOOLEAN'}],
                                                                            'name': 'supportedUplinkOnlyROHC-Profiles-r14',
                                                                            'type': 'SEQUENCE'},
                                                                           {'default': 'cs16',
                                                                            'name': 'maxNumberROHC-ContextSessions-r14',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('cs2',
                                                                                        0),
                                                                                       ('cs4',
                                                                                        1),
                                                                                       ('cs8',
                                                                                        2),
                                                                                       ('cs12',
                                                                                        3),
                                                                                       ('cs16',
                                                                                        4),
                                                                                       ('cs24',
                                                                                        5),
                                                                                       ('cs32',
                                                                                        6),
                                                                                       ('cs48',
                                                                                        7),
                                                                                       ('cs64',
                                                                                        8),
                                                                                       ('cs128',
                                                                                        9),
                                                                                       ('cs256',
                                                                                        10),
                                                                                       ('cs512',
                                                                                        11),
                                                                                       ('cs1024',
                                                                                        12),
                                                                                       ('cs16384',
                                                                                        13),
                                                                                       ('spare2',
                                                                                        14),
                                                                                       ('spare1',
                                                                                        15)]}],
                                                               'type': 'SEQUENCE'},
                                     'PDSCH-ConfigCommon': {'members': [{'name': 'referenceSignalPower',
                                                                         'restricted-to': [(-60,
                                                                                            50)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'p-b',
                                                                         'restricted-to': [(0,
                                                                                            3)],
                                                                         'type': 'INTEGER'}],
                                                            'type': 'SEQUENCE'},
                                     'PDSCH-ConfigCommon-v1310': {'members': [{'name': 'pdsch-maxNumRepetitionCEmodeA-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('r16',
                                                                                           0),
                                                                                          ('r32',
                                                                                           1)]},
                                                                              {'name': 'pdsch-maxNumRepetitionCEmodeB-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('r192',
                                                                                           0),
                                                                                          ('r256',
                                                                                           1),
                                                                                          ('r384',
                                                                                           2),
                                                                                          ('r512',
                                                                                           3),
                                                                                          ('r768',
                                                                                           4),
                                                                                          ('r1024',
                                                                                           5),
                                                                                          ('r1536',
                                                                                           6),
                                                                                          ('r2048',
                                                                                           7)]}],
                                                                  'type': 'SEQUENCE'},
                                     'PDSCH-ConfigDedicated': {'members': [{'name': 'p-a',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('dB-6',
                                                                                        0),
                                                                                       ('dB-4dot77',
                                                                                        1),
                                                                                       ('dB-3',
                                                                                        2),
                                                                                       ('dB-1dot77',
                                                                                        3),
                                                                                       ('dB0',
                                                                                        4),
                                                                                       ('dB1',
                                                                                        5),
                                                                                       ('dB2',
                                                                                        6),
                                                                                       ('dB3',
                                                                                        7)]}],
                                                               'type': 'SEQUENCE'},
                                     'PDSCH-ConfigDedicated-v1130': {'members': [{'name': 'dmrs-ConfigPDSCH-r11',
                                                                                  'optional': True,
                                                                                  'type': 'DMRS-Config-r11'},
                                                                                 {'name': 'qcl-Operation',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('typeA',
                                                                                              0),
                                                                                             ('typeB',
                                                                                              1)]},
                                                                                 {'name': 're-MappingQCLConfigToReleaseList-r11',
                                                                                  'optional': True,
                                                                                  'type': 'RE-MappingQCLConfigToReleaseList-r11'},
                                                                                 {'name': 're-MappingQCLConfigToAddModList-r11',
                                                                                  'optional': True,
                                                                                  'type': 'RE-MappingQCLConfigToAddModList-r11'}],
                                                                     'type': 'SEQUENCE'},
                                     'PDSCH-ConfigDedicated-v1280': {'members': [{'name': 'tbsIndexAlt-r12',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('a26',
                                                                                              0),
                                                                                             ('a33',
                                                                                              1)]}],
                                                                     'type': 'SEQUENCE'},
                                     'PDSCH-ConfigDedicated-v1310': {'members': [{'name': 'dmrs-ConfigPDSCH-v1310',
                                                                                  'optional': True,
                                                                                  'type': 'DMRS-Config-v1310'}],
                                                                     'type': 'SEQUENCE'},
                                     'PDSCH-ConfigDedicated-v1430': {'members': [{'name': 'ce-PDSCH-MaxBandwidth-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('bw5',
                                                                                              0),
                                                                                             ('bw20',
                                                                                              1)]},
                                                                                 {'name': 'ce-PDSCH-TenProcesses-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('on',
                                                                                              0)]},
                                                                                 {'name': 'ce-HARQ-AckBundling-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('on',
                                                                                              0)]},
                                                                                 {'name': 'ce-SchedulingEnhancement-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('range1',
                                                                                              0),
                                                                                             ('range2',
                                                                                              1)]},
                                                                                 {'name': 'tbsIndexAlt2-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('b33',
                                                                                              0)]}],
                                                                     'type': 'SEQUENCE'},
                                     'PDSCH-ConfigDedicatedSCell-v1430': {'members': [{'name': 'tbsIndexAlt2-r14',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('b33',
                                                                                                   0)]}],
                                                                          'type': 'SEQUENCE'},
                                     'PDSCH-RE-MappingQCL-Config-r11': {'members': [{'name': 'pdsch-RE-MappingQCL-ConfigId-r11',
                                                                                     'type': 'PDSCH-RE-MappingQCL-ConfigId-r11'},
                                                                                    {'members': [{'name': 'crs-PortsCount-r11',
                                                                                                  'type': 'ENUMERATED',
                                                                                                  'values': [('n1',
                                                                                                              0),
                                                                                                             ('n2',
                                                                                                              1),
                                                                                                             ('n4',
                                                                                                              2),
                                                                                                             ('spare1',
                                                                                                              3)]},
                                                                                                 {'name': 'crs-FreqShift-r11',
                                                                                                  'restricted-to': [(0,
                                                                                                                     5)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'members': [{'name': 'release',
                                                                                                               'type': 'NULL'},
                                                                                                              {'members': [{'name': 'subframeConfigList',
                                                                                                                            'type': 'MBSFN-SubframeConfigList'}],
                                                                                                               'name': 'setup',
                                                                                                               'type': 'SEQUENCE'}],
                                                                                                  'name': 'mbsfn-SubframeConfigList-r11',
                                                                                                  'optional': True,
                                                                                                  'type': 'CHOICE'},
                                                                                                 {'name': 'pdsch-Start-r11',
                                                                                                  'type': 'ENUMERATED',
                                                                                                  'values': [('reserved',
                                                                                                              0),
                                                                                                             ('n1',
                                                                                                              1),
                                                                                                             ('n2',
                                                                                                              2),
                                                                                                             ('n3',
                                                                                                              3),
                                                                                                             ('n4',
                                                                                                              4),
                                                                                                             ('assigned',
                                                                                                              5)]}],
                                                                                     'name': 'optionalSetOfFields-r11',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'},
                                                                                    {'name': 'csi-RS-ConfigZPId-r11',
                                                                                     'type': 'CSI-RS-ConfigZPId-r11'},
                                                                                    {'name': 'qcl-CSI-RS-ConfigNZPId-r11',
                                                                                     'optional': True,
                                                                                     'type': 'CSI-RS-ConfigNZPId-r11'},
                                                                                    None,
                                                                                    [{'members': [{'name': 'release',
                                                                                                   'type': 'NULL'},
                                                                                                  {'members': [{'name': 'subframeConfigList-v1430',
                                                                                                                'type': 'MBSFN-SubframeConfigList-v1430'}],
                                                                                                   'name': 'setup',
                                                                                                   'type': 'SEQUENCE'}],
                                                                                      'name': 'mbsfn-SubframeConfigList-v1430',
                                                                                      'optional': True,
                                                                                      'type': 'CHOICE'}]],
                                                                        'type': 'SEQUENCE'},
                                     'PDSCH-RE-MappingQCL-ConfigId-r11': {'restricted-to': [(1,
                                                                                             'maxRE-MapQCL-r11')],
                                                                          'type': 'INTEGER'},
                                     'PHICH-Config': {'members': [{'name': 'phich-Duration',
                                                                   'type': 'ENUMERATED',
                                                                   'values': [('normal',
                                                                               0),
                                                                              ('extended',
                                                                               1)]},
                                                                  {'name': 'phich-Resource',
                                                                   'type': 'ENUMERATED',
                                                                   'values': [('oneSixth',
                                                                               0),
                                                                              ('half',
                                                                               1),
                                                                              ('one',
                                                                               2),
                                                                              ('two',
                                                                               3)]}],
                                                      'type': 'SEQUENCE'},
                                     'PLMN-Identity': {'members': [{'name': 'mcc',
                                                                    'optional': True,
                                                                    'type': 'MCC'},
                                                                   {'name': 'mnc',
                                                                    'type': 'MNC'}],
                                                       'type': 'SEQUENCE'},
                                     'PLMN-IdentityInfo': {'members': [{'name': 'plmn-Identity',
                                                                        'type': 'PLMN-Identity'},
                                                                       {'name': 'cellReservedForOperatorUse',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('reserved',
                                                                                    0),
                                                                                   ('notReserved',
                                                                                    1)]}],
                                                           'type': 'SEQUENCE'},
                                     'PLMN-IdentityInfo2-r12': {'members': [{'name': 'plmn-Index-r12',
                                                                             'restricted-to': [(1,
                                                                                                'maxPLMN-r11')],
                                                                             'type': 'INTEGER'},
                                                                            {'name': 'plmnIdentity-r12',
                                                                             'type': 'PLMN-Identity'}],
                                                                'type': 'CHOICE'},
                                     'PLMN-IdentityList': {'element': {'type': 'PLMN-IdentityInfo'},
                                                           'size': [(1,
                                                                     'maxPLMN-r11')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'PLMN-IdentityList-MBMS-r14': {'element': {'type': 'PLMN-Identity'},
                                                                    'size': [(1,
                                                                              'maxPLMN-r11')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'PLMN-IdentityList2': {'element': {'type': 'PLMN-Identity'},
                                                            'size': [(1, 5)],
                                                            'type': 'SEQUENCE '
                                                                    'OF'},
                                     'PLMN-IdentityList3-r11': {'element': {'type': 'PLMN-Identity'},
                                                                'size': [(1,
                                                                          16)],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'PLMN-IdentityList4-r12': {'element': {'type': 'PLMN-IdentityInfo2-r12'},
                                                                'size': [(1,
                                                                          'maxPLMN-r11')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'PMCH-Config-r12': {'members': [{'name': 'sf-AllocEnd-r12',
                                                                      'restricted-to': [(0,
                                                                                         1535)],
                                                                      'type': 'INTEGER'},
                                                                     {'members': [{'name': 'normal-r12',
                                                                                   'restricted-to': [(0,
                                                                                                      28)],
                                                                                   'type': 'INTEGER'},
                                                                                  {'name': 'higerOrder-r12',
                                                                                   'restricted-to': [(0,
                                                                                                      27)],
                                                                                   'type': 'INTEGER'}],
                                                                      'name': 'dataMCS-r12',
                                                                      'type': 'CHOICE'},
                                                                     {'name': 'mch-SchedulingPeriod-r12',
                                                                      'type': 'ENUMERATED',
                                                                      'values': [('rf4',
                                                                                  0),
                                                                                 ('rf8',
                                                                                  1),
                                                                                 ('rf16',
                                                                                  2),
                                                                                 ('rf32',
                                                                                  3),
                                                                                 ('rf64',
                                                                                  4),
                                                                                 ('rf128',
                                                                                  5),
                                                                                 ('rf256',
                                                                                  6),
                                                                                 ('rf512',
                                                                                  7),
                                                                                 ('rf1024',
                                                                                  8)]},
                                                                     None,
                                                                     [{'name': 'mch-SchedulingPeriod-v1430',
                                                                       'optional': True,
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('rf1',
                                                                                   0),
                                                                                  ('rf2',
                                                                                   1)]}]],
                                                         'type': 'SEQUENCE'},
                                     'PMCH-Config-r9': {'members': [{'name': 'sf-AllocEnd-r9',
                                                                     'restricted-to': [(0,
                                                                                        1535)],
                                                                     'type': 'INTEGER'},
                                                                    {'name': 'dataMCS-r9',
                                                                     'restricted-to': [(0,
                                                                                        28)],
                                                                     'type': 'INTEGER'},
                                                                    {'name': 'mch-SchedulingPeriod-r9',
                                                                     'type': 'ENUMERATED',
                                                                     'values': [('rf8',
                                                                                 0),
                                                                                ('rf16',
                                                                                 1),
                                                                                ('rf32',
                                                                                 2),
                                                                                ('rf64',
                                                                                 3),
                                                                                ('rf128',
                                                                                 4),
                                                                                ('rf256',
                                                                                 5),
                                                                                ('rf512',
                                                                                 6),
                                                                                ('rf1024',
                                                                                 7)]},
                                                                    None],
                                                        'type': 'SEQUENCE'},
                                     'PMCH-Info-r9': {'members': [{'name': 'pmch-Config-r9',
                                                                   'type': 'PMCH-Config-r9'},
                                                                  {'name': 'mbms-SessionInfoList-r9',
                                                                   'type': 'MBMS-SessionInfoList-r9'},
                                                                  None],
                                                      'type': 'SEQUENCE'},
                                     'PMCH-InfoExt-r12': {'members': [{'name': 'pmch-Config-r12',
                                                                       'type': 'PMCH-Config-r12'},
                                                                      {'name': 'mbms-SessionInfoList-r12',
                                                                       'type': 'MBMS-SessionInfoList-r9'},
                                                                      None],
                                                          'type': 'SEQUENCE'},
                                     'PMCH-InfoList-r9': {'element': {'type': 'PMCH-Info-r9'},
                                                          'size': [(0,
                                                                    'maxPMCH-PerMBSFN')],
                                                          'type': 'SEQUENCE '
                                                                  'OF'},
                                     'PMCH-InfoListExt-r12': {'element': {'type': 'PMCH-InfoExt-r12'},
                                                              'size': [(0,
                                                                        'maxPMCH-PerMBSFN')],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'PRACH-Config': {'members': [{'name': 'rootSequenceIndex',
                                                                   'restricted-to': [(0,
                                                                                      837)],
                                                                   'type': 'INTEGER'},
                                                                  {'name': 'prach-ConfigInfo',
                                                                   'optional': True,
                                                                   'type': 'PRACH-ConfigInfo'}],
                                                      'type': 'SEQUENCE'},
                                     'PRACH-Config-v1310': {'members': [{'name': 'rsrp-ThresholdsPrachInfoList-r13',
                                                                         'optional': True,
                                                                         'type': 'RSRP-ThresholdsPrachInfoList-r13'},
                                                                        {'members': [{'name': 'fdd-r13',
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('v1',
                                                                                                  0),
                                                                                                 ('v1dot5',
                                                                                                  1),
                                                                                                 ('v2',
                                                                                                  2),
                                                                                                 ('v2dot5',
                                                                                                  3),
                                                                                                 ('v4',
                                                                                                  4),
                                                                                                 ('v5',
                                                                                                  5),
                                                                                                 ('v8',
                                                                                                  6),
                                                                                                 ('v10',
                                                                                                  7)]},
                                                                                     {'name': 'tdd-r13',
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('v1',
                                                                                                  0),
                                                                                                 ('v2',
                                                                                                  1),
                                                                                                 ('v4',
                                                                                                  2),
                                                                                                 ('v5',
                                                                                                  3),
                                                                                                 ('v8',
                                                                                                  4),
                                                                                                 ('v10',
                                                                                                  5),
                                                                                                 ('v20',
                                                                                                  6),
                                                                                                 ('spare',
                                                                                                  7)]}],
                                                                         'name': 'mpdcch-startSF-CSS-RA-r13',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'},
                                                                        {'name': 'prach-HoppingOffset-r13',
                                                                         'optional': True,
                                                                         'restricted-to': [(0,
                                                                                            94)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'prach-ParametersListCE-r13',
                                                                         'optional': True,
                                                                         'type': 'PRACH-ParametersListCE-r13'},
                                                                        {'name': 'initial-CE-level-r13',
                                                                         'optional': True,
                                                                         'restricted-to': [(0,
                                                                                            3)],
                                                                         'type': 'INTEGER'}],
                                                            'type': 'SEQUENCE'},
                                     'PRACH-Config-v1430': {'members': [{'name': 'rootSequenceIndexHighSpeed-r14',
                                                                         'restricted-to': [(0,
                                                                                            837)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'zeroCorrelationZoneConfigHighSpeed-r14',
                                                                         'restricted-to': [(0,
                                                                                            12)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'prach-ConfigIndexHighSpeed-r14',
                                                                         'restricted-to': [(0,
                                                                                            63)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'prach-FreqOffsetHighSpeed-r14',
                                                                         'restricted-to': [(0,
                                                                                            94)],
                                                                         'type': 'INTEGER'}],
                                                            'type': 'SEQUENCE'},
                                     'PRACH-ConfigInfo': {'members': [{'name': 'prach-ConfigIndex',
                                                                       'restricted-to': [(0,
                                                                                          63)],
                                                                       'type': 'INTEGER'},
                                                                      {'name': 'highSpeedFlag',
                                                                       'type': 'BOOLEAN'},
                                                                      {'name': 'zeroCorrelationZoneConfig',
                                                                       'restricted-to': [(0,
                                                                                          15)],
                                                                       'type': 'INTEGER'},
                                                                      {'name': 'prach-FreqOffset',
                                                                       'restricted-to': [(0,
                                                                                          94)],
                                                                       'type': 'INTEGER'}],
                                                          'type': 'SEQUENCE'},
                                     'PRACH-ConfigSCell-r10': {'members': [{'name': 'prach-ConfigIndex-r10',
                                                                            'restricted-to': [(0,
                                                                                               63)],
                                                                            'type': 'INTEGER'}],
                                                               'type': 'SEQUENCE'},
                                     'PRACH-ConfigSIB': {'members': [{'name': 'rootSequenceIndex',
                                                                      'restricted-to': [(0,
                                                                                         837)],
                                                                      'type': 'INTEGER'},
                                                                     {'name': 'prach-ConfigInfo',
                                                                      'type': 'PRACH-ConfigInfo'}],
                                                         'type': 'SEQUENCE'},
                                     'PRACH-ConfigSIB-v1310': {'members': [{'name': 'rsrp-ThresholdsPrachInfoList-r13',
                                                                            'type': 'RSRP-ThresholdsPrachInfoList-r13'},
                                                                           {'members': [{'name': 'fdd-r13',
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('v1',
                                                                                                     0),
                                                                                                    ('v1dot5',
                                                                                                     1),
                                                                                                    ('v2',
                                                                                                     2),
                                                                                                    ('v2dot5',
                                                                                                     3),
                                                                                                    ('v4',
                                                                                                     4),
                                                                                                    ('v5',
                                                                                                     5),
                                                                                                    ('v8',
                                                                                                     6),
                                                                                                    ('v10',
                                                                                                     7)]},
                                                                                        {'name': 'tdd-r13',
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('v1',
                                                                                                     0),
                                                                                                    ('v2',
                                                                                                     1),
                                                                                                    ('v4',
                                                                                                     2),
                                                                                                    ('v5',
                                                                                                     3),
                                                                                                    ('v8',
                                                                                                     4),
                                                                                                    ('v10',
                                                                                                     5),
                                                                                                    ('v20',
                                                                                                     6),
                                                                                                    ('spare',
                                                                                                     7)]}],
                                                                            'name': 'mpdcch-startSF-CSS-RA-r13',
                                                                            'optional': True,
                                                                            'type': 'CHOICE'},
                                                                           {'name': 'prach-HoppingOffset-r13',
                                                                            'optional': True,
                                                                            'restricted-to': [(0,
                                                                                               94)],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'prach-ParametersListCE-r13',
                                                                            'type': 'PRACH-ParametersListCE-r13'}],
                                                               'type': 'SEQUENCE'},
                                     'PRACH-ParametersCE-r13': {'members': [{'name': 'prach-ConfigIndex-r13',
                                                                             'restricted-to': [(0,
                                                                                                63)],
                                                                             'type': 'INTEGER'},
                                                                            {'name': 'prach-FreqOffset-r13',
                                                                             'restricted-to': [(0,
                                                                                                94)],
                                                                             'type': 'INTEGER'},
                                                                            {'name': 'prach-StartingSubframe-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('sf2',
                                                                                         0),
                                                                                        ('sf4',
                                                                                         1),
                                                                                        ('sf8',
                                                                                         2),
                                                                                        ('sf16',
                                                                                         3),
                                                                                        ('sf32',
                                                                                         4),
                                                                                        ('sf64',
                                                                                         5),
                                                                                        ('sf128',
                                                                                         6),
                                                                                        ('sf256',
                                                                                         7)]},
                                                                            {'name': 'maxNumPreambleAttemptCE-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('n3',
                                                                                         0),
                                                                                        ('n4',
                                                                                         1),
                                                                                        ('n5',
                                                                                         2),
                                                                                        ('n6',
                                                                                         3),
                                                                                        ('n7',
                                                                                         4),
                                                                                        ('n8',
                                                                                         5),
                                                                                        ('n10',
                                                                                         6)]},
                                                                            {'name': 'numRepetitionPerPreambleAttempt-r13',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('n1',
                                                                                         0),
                                                                                        ('n2',
                                                                                         1),
                                                                                        ('n4',
                                                                                         2),
                                                                                        ('n8',
                                                                                         3),
                                                                                        ('n16',
                                                                                         4),
                                                                                        ('n32',
                                                                                         5),
                                                                                        ('n64',
                                                                                         6),
                                                                                        ('n128',
                                                                                         7)]},
                                                                            {'element': {'restricted-to': [(1,
                                                                                                            'maxAvailNarrowBands-r13')],
                                                                                         'type': 'INTEGER'},
                                                                             'name': 'mpdcch-NarrowbandsToMonitor-r13',
                                                                             'size': [(1,
                                                                                       2)],
                                                                             'type': 'SEQUENCE '
                                                                                     'OF'},
                                                                            {'name': 'mpdcch-NumRepetition-RA-r13',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('r1',
                                                                                         0),
                                                                                        ('r2',
                                                                                         1),
                                                                                        ('r4',
                                                                                         2),
                                                                                        ('r8',
                                                                                         3),
                                                                                        ('r16',
                                                                                         4),
                                                                                        ('r32',
                                                                                         5),
                                                                                        ('r64',
                                                                                         6),
                                                                                        ('r128',
                                                                                         7),
                                                                                        ('r256',
                                                                                         8)]},
                                                                            {'name': 'prach-HoppingConfig-r13',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('on',
                                                                                         0),
                                                                                        ('off',
                                                                                         1)]}],
                                                                'type': 'SEQUENCE'},
                                     'PRACH-ParametersListCE-r13': {'element': {'type': 'PRACH-ParametersCE-r13'},
                                                                    'size': [(1,
                                                                              'maxCE-Level-r13')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'PSCellToAddMod-r12': {'members': [{'name': 'sCellIndex-r12',
                                                                         'type': 'SCellIndex-r10'},
                                                                        {'members': [{'name': 'physCellId-r12',
                                                                                      'type': 'PhysCellId'},
                                                                                     {'name': 'dl-CarrierFreq-r12',
                                                                                      'type': 'ARFCN-ValueEUTRA-r9'}],
                                                                         'name': 'cellIdentification-r12',
                                                                         'optional': True,
                                                                         'type': 'SEQUENCE'},
                                                                        {'name': 'radioResourceConfigCommonPSCell-r12',
                                                                         'optional': True,
                                                                         'type': 'RadioResourceConfigCommonPSCell-r12'},
                                                                        {'name': 'radioResourceConfigDedicatedPSCell-r12',
                                                                         'optional': True,
                                                                         'type': 'RadioResourceConfigDedicatedPSCell-r12'},
                                                                        None,
                                                                        [{'name': 'antennaInfoDedicatedPSCell-v1280',
                                                                          'optional': True,
                                                                          'type': 'AntennaInfoDedicated-v10i0'}],
                                                                        [{'name': 'sCellIndex-r13',
                                                                          'optional': True,
                                                                          'type': 'SCellIndex-r13'}],
                                                                        [{'name': 'radioResourceConfigDedicatedPSCell-v1370',
                                                                          'optional': True,
                                                                          'type': 'RadioResourceConfigDedicatedPSCell-v1370'}]],
                                                            'type': 'SEQUENCE'},
                                     'PSCellToAddMod-v12f0': {'members': [{'name': 'radioResourceConfigCommonPSCell-r12',
                                                                           'optional': True,
                                                                           'type': 'RadioResourceConfigCommonPSCell-v12f0'}],
                                                              'type': 'SEQUENCE'},
                                     'PSCellToAddMod-v1440': {'members': [{'name': 'radioResourceConfigCommonPSCell-r14',
                                                                           'optional': True,
                                                                           'type': 'RadioResourceConfigCommonPSCell-v1440'}],
                                                              'type': 'SEQUENCE'},
                                     'PUCCH-ConfigCommon': {'members': [{'name': 'deltaPUCCH-Shift',
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('ds1',
                                                                                     0),
                                                                                    ('ds2',
                                                                                     1),
                                                                                    ('ds3',
                                                                                     2)]},
                                                                        {'name': 'nRB-CQI',
                                                                         'restricted-to': [(0,
                                                                                            98)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'nCS-AN',
                                                                         'restricted-to': [(0,
                                                                                            7)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'n1PUCCH-AN',
                                                                         'restricted-to': [(0,
                                                                                            2047)],
                                                                         'type': 'INTEGER'}],
                                                            'type': 'SEQUENCE'},
                                     'PUCCH-ConfigCommon-v1310': {'members': [{'name': 'n1PUCCH-AN-InfoList-r13',
                                                                               'optional': True,
                                                                               'type': 'N1PUCCH-AN-InfoList-r13'},
                                                                              {'name': 'pucch-NumRepetitionCE-Msg4-Level0-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n1',
                                                                                           0),
                                                                                          ('n2',
                                                                                           1),
                                                                                          ('n4',
                                                                                           2),
                                                                                          ('n8',
                                                                                           3)]},
                                                                              {'name': 'pucch-NumRepetitionCE-Msg4-Level1-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n1',
                                                                                           0),
                                                                                          ('n2',
                                                                                           1),
                                                                                          ('n4',
                                                                                           2),
                                                                                          ('n8',
                                                                                           3)]},
                                                                              {'name': 'pucch-NumRepetitionCE-Msg4-Level2-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n4',
                                                                                           0),
                                                                                          ('n8',
                                                                                           1),
                                                                                          ('n16',
                                                                                           2),
                                                                                          ('n32',
                                                                                           3)]},
                                                                              {'name': 'pucch-NumRepetitionCE-Msg4-Level3-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n4',
                                                                                           0),
                                                                                          ('n8',
                                                                                           1),
                                                                                          ('n16',
                                                                                           2),
                                                                                          ('n32',
                                                                                           3)]}],
                                                                  'type': 'SEQUENCE'},
                                     'PUCCH-ConfigCommon-v1430': {'members': [{'name': 'pucch-NumRepetitionCE-Msg4-Level3-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n64',
                                                                                           0),
                                                                                          ('n128',
                                                                                           1)]}],
                                                                  'type': 'SEQUENCE'},
                                     'PUCCH-ConfigDedicated': {'members': [{'members': [{'name': 'release',
                                                                                         'type': 'NULL'},
                                                                                        {'members': [{'name': 'repetitionFactor',
                                                                                                      'type': 'ENUMERATED',
                                                                                                      'values': [('n2',
                                                                                                                  0),
                                                                                                                 ('n4',
                                                                                                                  1),
                                                                                                                 ('n6',
                                                                                                                  2),
                                                                                                                 ('spare1',
                                                                                                                  3)]},
                                                                                                     {'name': 'n1PUCCH-AN-Rep',
                                                                                                      'restricted-to': [(0,
                                                                                                                         2047)],
                                                                                                      'type': 'INTEGER'}],
                                                                                         'name': 'setup',
                                                                                         'type': 'SEQUENCE'}],
                                                                            'name': 'ackNackRepetition',
                                                                            'type': 'CHOICE'},
                                                                           {'name': 'tdd-AckNackFeedbackMode',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('bundling',
                                                                                        0),
                                                                                       ('multiplexing',
                                                                                        1)]}],
                                                               'type': 'SEQUENCE'},
                                     'PUCCH-ConfigDedicated-r13': {'members': [{'members': [{'name': 'release',
                                                                                             'type': 'NULL'},
                                                                                            {'members': [{'name': 'repetitionFactor-r13',
                                                                                                          'type': 'ENUMERATED',
                                                                                                          'values': [('n2',
                                                                                                                      0),
                                                                                                                     ('n4',
                                                                                                                      1),
                                                                                                                     ('n6',
                                                                                                                      2),
                                                                                                                     ('spare1',
                                                                                                                      3)]},
                                                                                                         {'name': 'n1PUCCH-AN-Rep-r13',
                                                                                                          'restricted-to': [(0,
                                                                                                                             2047)],
                                                                                                          'type': 'INTEGER'}],
                                                                                             'name': 'setup',
                                                                                             'type': 'SEQUENCE'}],
                                                                                'name': 'ackNackRepetition-r13',
                                                                                'type': 'CHOICE'},
                                                                               {'name': 'tdd-AckNackFeedbackMode-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('bundling',
                                                                                            0),
                                                                                           ('multiplexing',
                                                                                            1)]},
                                                                               {'members': [{'members': [{'element': {'restricted-to': [(0,
                                                                                                                                         549)],
                                                                                                                      'type': 'INTEGER'},
                                                                                                          'name': 'n3PUCCH-AN-List-r13',
                                                                                                          'optional': True,
                                                                                                          'size': [(1,
                                                                                                                    4)],
                                                                                                          'type': 'SEQUENCE '
                                                                                                                  'OF'},
                                                                                                         {'members': [{'name': 'release',
                                                                                                                       'type': 'NULL'},
                                                                                                                      {'members': [{'element': {'restricted-to': [(0,
                                                                                                                                                                   549)],
                                                                                                                                                'type': 'INTEGER'},
                                                                                                                                    'name': 'n3PUCCH-AN-ListP1-r13',
                                                                                                                                    'size': [(1,
                                                                                                                                              4)],
                                                                                                                                    'type': 'SEQUENCE '
                                                                                                                                            'OF'}],
                                                                                                                       'name': 'setup',
                                                                                                                       'type': 'SEQUENCE'}],
                                                                                                          'name': 'twoAntennaPortActivatedPUCCH-Format3-r13',
                                                                                                          'optional': True,
                                                                                                          'type': 'CHOICE'}],
                                                                                             'name': 'format3-r13',
                                                                                             'type': 'SEQUENCE'},
                                                                                            {'members': [{'members': [{'name': 'release',
                                                                                                                       'type': 'NULL'},
                                                                                                                      {'members': [{'element': {'type': 'N1PUCCH-AN-CS-r10'},
                                                                                                                                    'name': 'n1PUCCH-AN-CS-List-r13',
                                                                                                                                    'size': [(1,
                                                                                                                                              2)],
                                                                                                                                    'type': 'SEQUENCE '
                                                                                                                                            'OF'},
                                                                                                                                   {'element': {'restricted-to': [(0,
                                                                                                                                                                   2047)],
                                                                                                                                                'type': 'INTEGER'},
                                                                                                                                    'name': 'n1PUCCH-AN-CS-ListP1-r13',
                                                                                                                                    'size': [(2,
                                                                                                                                              4)],
                                                                                                                                    'type': 'SEQUENCE '
                                                                                                                                            'OF'}],
                                                                                                                       'name': 'setup',
                                                                                                                       'type': 'SEQUENCE'}],
                                                                                                          'name': 'n1PUCCH-AN-CS-r13',
                                                                                                          'optional': True,
                                                                                                          'type': 'CHOICE'}],
                                                                                             'name': 'channelSelection-r13',
                                                                                             'type': 'SEQUENCE'},
                                                                                            {'members': [{'element': {'type': 'Format4-resource-r13'},
                                                                                                          'name': 'format4-resourceConfiguration-r13',
                                                                                                          'size': [4],
                                                                                                          'type': 'SEQUENCE '
                                                                                                                  'OF'},
                                                                                                         {'element': {'type': 'Format4-resource-r13'},
                                                                                                          'name': 'format4-MultiCSI-resourceConfiguration-r13',
                                                                                                          'optional': True,
                                                                                                          'size': [(1,
                                                                                                                    2)],
                                                                                                          'type': 'SEQUENCE '
                                                                                                                  'OF'}],
                                                                                             'name': 'format4-r13',
                                                                                             'type': 'SEQUENCE'},
                                                                                            {'members': [{'element': {'type': 'Format5-resource-r13'},
                                                                                                          'name': 'format5-resourceConfiguration-r13',
                                                                                                          'size': [4],
                                                                                                          'type': 'SEQUENCE '
                                                                                                                  'OF'},
                                                                                                         {'name': 'format5-MultiCSI-resourceConfiguration-r13',
                                                                                                          'optional': True,
                                                                                                          'type': 'Format5-resource-r13'}],
                                                                                             'name': 'format5-r13',
                                                                                             'type': 'SEQUENCE'}],
                                                                                'name': 'pucch-Format-r13',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'},
                                                                               {'name': 'twoAntennaPortActivatedPUCCH-Format1a1b-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('true',
                                                                                            0)]},
                                                                               {'name': 'simultaneousPUCCH-PUSCH-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('true',
                                                                                            0)]},
                                                                               {'name': 'n1PUCCH-AN-RepP1-r13',
                                                                                'optional': True,
                                                                                'restricted-to': [(0,
                                                                                                   2047)],
                                                                                'type': 'INTEGER'},
                                                                               {'members': [{'name': 'release',
                                                                                             'type': 'NULL'},
                                                                                            {'members': [{'name': 'nPUCCH-Identity-r13',
                                                                                                          'restricted-to': [(0,
                                                                                                                             503)],
                                                                                                          'type': 'INTEGER'},
                                                                                                         {'name': 'n1PUCCH-AN-r13',
                                                                                                          'restricted-to': [(0,
                                                                                                                             2047)],
                                                                                                          'type': 'INTEGER'}],
                                                                                             'name': 'setup',
                                                                                             'type': 'SEQUENCE'}],
                                                                                'name': 'nPUCCH-Param-r13',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'},
                                                                               {'members': [{'name': 'release',
                                                                                             'type': 'NULL'},
                                                                                            {'members': [{'name': 'nkaPUCCH-AN-r13',
                                                                                                          'restricted-to': [(0,
                                                                                                                             2047)],
                                                                                                          'type': 'INTEGER'}],
                                                                                             'name': 'setup',
                                                                                             'type': 'SEQUENCE'}],
                                                                                'name': 'nkaPUCCH-Param-r13',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'},
                                                                               {'name': 'spatialBundlingPUCCH-r13',
                                                                                'type': 'BOOLEAN'},
                                                                               {'name': 'spatialBundlingPUSCH-r13',
                                                                                'type': 'BOOLEAN'},
                                                                               {'name': 'harq-TimingTDD-r13',
                                                                                'type': 'BOOLEAN'},
                                                                               {'name': 'codebooksizeDetermination-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('dai',
                                                                                            0),
                                                                                           ('cc',
                                                                                            1)]},
                                                                               {'name': 'maximumPayloadCoderate-r13',
                                                                                'optional': True,
                                                                                'restricted-to': [(0,
                                                                                                   7)],
                                                                                'type': 'INTEGER'},
                                                                               {'members': [{'name': 'release',
                                                                                             'type': 'NULL'},
                                                                                            {'members': [{'members': [{'name': 'pucch-NumRepetitionCE-format1-r13',
                                                                                                                       'type': 'ENUMERATED',
                                                                                                                       'values': [('r1',
                                                                                                                                   0),
                                                                                                                                  ('r2',
                                                                                                                                   1),
                                                                                                                                  ('r4',
                                                                                                                                   2),
                                                                                                                                  ('r8',
                                                                                                                                   3)]},
                                                                                                                      {'name': 'pucch-NumRepetitionCE-format2-r13',
                                                                                                                       'type': 'ENUMERATED',
                                                                                                                       'values': [('r1',
                                                                                                                                   0),
                                                                                                                                  ('r2',
                                                                                                                                   1),
                                                                                                                                  ('r4',
                                                                                                                                   2),
                                                                                                                                  ('r8',
                                                                                                                                   3)]}],
                                                                                                          'name': 'modeA',
                                                                                                          'type': 'SEQUENCE'},
                                                                                                         {'members': [{'name': 'pucch-NumRepetitionCE-format1-r13',
                                                                                                                       'type': 'ENUMERATED',
                                                                                                                       'values': [('r4',
                                                                                                                                   0),
                                                                                                                                  ('r8',
                                                                                                                                   1),
                                                                                                                                  ('r16',
                                                                                                                                   2),
                                                                                                                                  ('r32',
                                                                                                                                   3)]},
                                                                                                                      {'name': 'pucch-NumRepetitionCE-format2-r13',
                                                                                                                       'type': 'ENUMERATED',
                                                                                                                       'values': [('r4',
                                                                                                                                   0),
                                                                                                                                  ('r8',
                                                                                                                                   1),
                                                                                                                                  ('r16',
                                                                                                                                   2),
                                                                                                                                  ('r32',
                                                                                                                                   3)]}],
                                                                                                          'name': 'modeB',
                                                                                                          'type': 'SEQUENCE'}],
                                                                                             'name': 'setup',
                                                                                             'type': 'CHOICE'}],
                                                                                'name': 'pucch-NumRepetitionCE-r13',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'}],
                                                                   'type': 'SEQUENCE'},
                                     'PUCCH-ConfigDedicated-v1020': {'members': [{'members': [{'name': 'format3-r10',
                                                                                               'type': 'PUCCH-Format3-Conf-r13'},
                                                                                              {'members': [{'members': [{'name': 'release',
                                                                                                                         'type': 'NULL'},
                                                                                                                        {'members': [{'element': {'type': 'N1PUCCH-AN-CS-r10'},
                                                                                                                                      'name': 'n1PUCCH-AN-CS-List-r10',
                                                                                                                                      'size': [(1,
                                                                                                                                                2)],
                                                                                                                                      'type': 'SEQUENCE '
                                                                                                                                              'OF'}],
                                                                                                                         'name': 'setup',
                                                                                                                         'type': 'SEQUENCE'}],
                                                                                                            'name': 'n1PUCCH-AN-CS-r10',
                                                                                                            'optional': True,
                                                                                                            'type': 'CHOICE'}],
                                                                                               'name': 'channelSelection-r10',
                                                                                               'type': 'SEQUENCE'}],
                                                                                  'name': 'pucch-Format-r10',
                                                                                  'optional': True,
                                                                                  'type': 'CHOICE'},
                                                                                 {'name': 'twoAntennaPortActivatedPUCCH-Format1a1b-r10',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('true',
                                                                                              0)]},
                                                                                 {'name': 'simultaneousPUCCH-PUSCH-r10',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('true',
                                                                                              0)]},
                                                                                 {'name': 'n1PUCCH-AN-RepP1-r10',
                                                                                  'optional': True,
                                                                                  'restricted-to': [(0,
                                                                                                     2047)],
                                                                                  'type': 'INTEGER'}],
                                                                     'type': 'SEQUENCE'},
                                     'PUCCH-ConfigDedicated-v1130': {'members': [{'members': [{'name': 'release',
                                                                                               'type': 'NULL'},
                                                                                              {'members': [{'element': {'restricted-to': [(0,
                                                                                                                                           2047)],
                                                                                                                        'type': 'INTEGER'},
                                                                                                            'name': 'n1PUCCH-AN-CS-ListP1-r11',
                                                                                                            'size': [(2,
                                                                                                                      4)],
                                                                                                            'type': 'SEQUENCE '
                                                                                                                    'OF'}],
                                                                                               'name': 'setup',
                                                                                               'type': 'SEQUENCE'}],
                                                                                  'name': 'n1PUCCH-AN-CS-v1130',
                                                                                  'optional': True,
                                                                                  'type': 'CHOICE'},
                                                                                 {'members': [{'name': 'release',
                                                                                               'type': 'NULL'},
                                                                                              {'members': [{'name': 'nPUCCH-Identity-r11',
                                                                                                            'restricted-to': [(0,
                                                                                                                               503)],
                                                                                                            'type': 'INTEGER'},
                                                                                                           {'name': 'n1PUCCH-AN-r11',
                                                                                                            'restricted-to': [(0,
                                                                                                                               2047)],
                                                                                                            'type': 'INTEGER'}],
                                                                                               'name': 'setup',
                                                                                               'type': 'SEQUENCE'}],
                                                                                  'name': 'nPUCCH-Param-r11',
                                                                                  'optional': True,
                                                                                  'type': 'CHOICE'}],
                                                                     'type': 'SEQUENCE'},
                                     'PUCCH-ConfigDedicated-v1250': {'members': [{'members': [{'name': 'release',
                                                                                               'type': 'NULL'},
                                                                                              {'members': [{'name': 'nkaPUCCH-AN-r12',
                                                                                                            'restricted-to': [(0,
                                                                                                                               2047)],
                                                                                                            'type': 'INTEGER'}],
                                                                                               'name': 'setup',
                                                                                               'type': 'SEQUENCE'}],
                                                                                  'name': 'nkaPUCCH-Param-r12',
                                                                                  'type': 'CHOICE'}],
                                                                     'type': 'SEQUENCE'},
                                     'PUCCH-ConfigDedicated-v1370': {'members': [{'members': [{'name': 'release',
                                                                                               'type': 'NULL'},
                                                                                              {'name': 'setup',
                                                                                               'type': 'PUCCH-Format3-Conf-r13'}],
                                                                                  'name': 'pucch-Format-v1370',
                                                                                  'type': 'CHOICE'}],
                                                                     'type': 'SEQUENCE'},
                                     'PUCCH-ConfigDedicated-v1430': {'members': [{'name': 'pucch-NumRepetitionCE-format1-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('r64',
                                                                                              0),
                                                                                             ('r128',
                                                                                              1)]}],
                                                                     'type': 'SEQUENCE'},
                                     'PUCCH-Format3-Conf-r13': {'members': [{'element': {'restricted-to': [(0,
                                                                                                            549)],
                                                                                         'type': 'INTEGER'},
                                                                             'name': 'n3PUCCH-AN-List-r13',
                                                                             'optional': True,
                                                                             'size': [(1,
                                                                                       4)],
                                                                             'type': 'SEQUENCE '
                                                                                     'OF'},
                                                                            {'members': [{'name': 'release',
                                                                                          'type': 'NULL'},
                                                                                         {'members': [{'element': {'restricted-to': [(0,
                                                                                                                                      549)],
                                                                                                                   'type': 'INTEGER'},
                                                                                                       'name': 'n3PUCCH-AN-ListP1-r13',
                                                                                                       'size': [(1,
                                                                                                                 4)],
                                                                                                       'type': 'SEQUENCE '
                                                                                                               'OF'}],
                                                                                          'name': 'setup',
                                                                                          'type': 'SEQUENCE'}],
                                                                             'name': 'twoAntennaPortActivatedPUCCH-Format3-r13',
                                                                             'optional': True,
                                                                             'type': 'CHOICE'}],
                                                                'type': 'SEQUENCE'},
                                     'PUSCH-ConfigCommon': {'members': [{'members': [{'name': 'n-SB',
                                                                                      'restricted-to': [(1,
                                                                                                         4)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'hoppingMode',
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('interSubFrame',
                                                                                                  0),
                                                                                                 ('intraAndInterSubFrame',
                                                                                                  1)]},
                                                                                     {'name': 'pusch-HoppingOffset',
                                                                                      'restricted-to': [(0,
                                                                                                         98)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'enable64QAM',
                                                                                      'type': 'BOOLEAN'}],
                                                                         'name': 'pusch-ConfigBasic',
                                                                         'type': 'SEQUENCE'},
                                                                        {'name': 'ul-ReferenceSignalsPUSCH',
                                                                         'type': 'UL-ReferenceSignalsPUSCH'}],
                                                            'type': 'SEQUENCE'},
                                     'PUSCH-ConfigCommon-v1270': {'members': [{'name': 'enable64QAM-v1270',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('true',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'PUSCH-ConfigCommon-v1310': {'members': [{'name': 'pusch-maxNumRepetitionCEmodeA-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('r8',
                                                                                           0),
                                                                                          ('r16',
                                                                                           1),
                                                                                          ('r32',
                                                                                           2)]},
                                                                              {'name': 'pusch-maxNumRepetitionCEmodeB-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('r192',
                                                                                           0),
                                                                                          ('r256',
                                                                                           1),
                                                                                          ('r384',
                                                                                           2),
                                                                                          ('r512',
                                                                                           3),
                                                                                          ('r768',
                                                                                           4),
                                                                                          ('r1024',
                                                                                           5),
                                                                                          ('r1536',
                                                                                           6),
                                                                                          ('r2048',
                                                                                           7)]},
                                                                              {'name': 'pusch-HoppingOffset-v1310',
                                                                               'optional': True,
                                                                               'restricted-to': [(1,
                                                                                                  'maxAvailNarrowBands-r13')],
                                                                               'type': 'INTEGER'}],
                                                                  'type': 'SEQUENCE'},
                                     'PUSCH-ConfigDedicated': {'members': [{'name': 'betaOffset-ACK-Index',
                                                                            'restricted-to': [(0,
                                                                                               15)],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'betaOffset-RI-Index',
                                                                            'restricted-to': [(0,
                                                                                               15)],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'betaOffset-CQI-Index',
                                                                            'restricted-to': [(0,
                                                                                               15)],
                                                                            'type': 'INTEGER'}],
                                                               'type': 'SEQUENCE'},
                                     'PUSCH-ConfigDedicated-r13': {'members': [{'name': 'betaOffset-ACK-Index-r13',
                                                                                'restricted-to': [(0,
                                                                                                   15)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'betaOffset2-ACK-Index-r13',
                                                                                'optional': True,
                                                                                'restricted-to': [(0,
                                                                                                   15)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'betaOffset-RI-Index-r13',
                                                                                'restricted-to': [(0,
                                                                                                   15)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'betaOffset-CQI-Index-r13',
                                                                                'restricted-to': [(0,
                                                                                                   15)],
                                                                                'type': 'INTEGER'},
                                                                               {'members': [{'name': 'betaOffset-ACK-Index-MC-r13',
                                                                                             'restricted-to': [(0,
                                                                                                                15)],
                                                                                             'type': 'INTEGER'},
                                                                                            {'name': 'betaOffset2-ACK-Index-MC-r13',
                                                                                             'optional': True,
                                                                                             'restricted-to': [(0,
                                                                                                                15)],
                                                                                             'type': 'INTEGER'},
                                                                                            {'name': 'betaOffset-RI-Index-MC-r13',
                                                                                             'restricted-to': [(0,
                                                                                                                15)],
                                                                                             'type': 'INTEGER'},
                                                                                            {'name': 'betaOffset-CQI-Index-MC-r13',
                                                                                             'restricted-to': [(0,
                                                                                                                15)],
                                                                                             'type': 'INTEGER'}],
                                                                                'name': 'betaOffsetMC-r13',
                                                                                'optional': True,
                                                                                'type': 'SEQUENCE'},
                                                                               {'name': 'groupHoppingDisabled-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('true',
                                                                                            0)]},
                                                                               {'name': 'dmrs-WithOCC-Activated-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('true',
                                                                                            0)]},
                                                                               {'members': [{'name': 'release',
                                                                                             'type': 'NULL'},
                                                                                            {'members': [{'name': 'nPUSCH-Identity-r13',
                                                                                                          'restricted-to': [(0,
                                                                                                                             509)],
                                                                                                          'type': 'INTEGER'},
                                                                                                         {'name': 'nDMRS-CSH-Identity-r13',
                                                                                                          'restricted-to': [(0,
                                                                                                                             509)],
                                                                                                          'type': 'INTEGER'}],
                                                                                             'name': 'setup',
                                                                                             'type': 'SEQUENCE'}],
                                                                                'name': 'pusch-DMRS-r11',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'},
                                                                               {'members': [{'name': 'release',
                                                                                             'type': 'NULL'},
                                                                                            {'members': [{'name': 'betaOffset-ACK-Index-SubframeSet2-r13',
                                                                                                          'restricted-to': [(0,
                                                                                                                             15)],
                                                                                                          'type': 'INTEGER'},
                                                                                                         {'name': 'betaOffset2-ACK-Index-SubframeSet2-r13',
                                                                                                          'optional': True,
                                                                                                          'restricted-to': [(0,
                                                                                                                             15)],
                                                                                                          'type': 'INTEGER'},
                                                                                                         {'name': 'betaOffset-RI-Index-SubframeSet2-r13',
                                                                                                          'restricted-to': [(0,
                                                                                                                             15)],
                                                                                                          'type': 'INTEGER'},
                                                                                                         {'name': 'betaOffset-CQI-Index-SubframeSet2-r13',
                                                                                                          'restricted-to': [(0,
                                                                                                                             15)],
                                                                                                          'type': 'INTEGER'},
                                                                                                         {'members': [{'name': 'betaOffset-ACK-Index-MC-SubframeSet2-r13',
                                                                                                                       'restricted-to': [(0,
                                                                                                                                          15)],
                                                                                                                       'type': 'INTEGER'},
                                                                                                                      {'name': 'betaOffset2-ACK-Index-MC-SubframeSet2-r13',
                                                                                                                       'optional': True,
                                                                                                                       'restricted-to': [(0,
                                                                                                                                          15)],
                                                                                                                       'type': 'INTEGER'},
                                                                                                                      {'name': 'betaOffset-RI-Index-MC-SubframeSet2-r13',
                                                                                                                       'restricted-to': [(0,
                                                                                                                                          15)],
                                                                                                                       'type': 'INTEGER'},
                                                                                                                      {'name': 'betaOffset-CQI-Index-MC-SubframeSet2-r13',
                                                                                                                       'restricted-to': [(0,
                                                                                                                                          15)],
                                                                                                                       'type': 'INTEGER'}],
                                                                                                          'name': 'betaOffsetMC-r12',
                                                                                                          'optional': True,
                                                                                                          'type': 'SEQUENCE'}],
                                                                                             'name': 'setup',
                                                                                             'type': 'SEQUENCE'}],
                                                                                'name': 'uciOnPUSCH',
                                                                                'optional': True,
                                                                                'type': 'CHOICE'},
                                                                               {'name': 'pusch-HoppingConfig-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('on',
                                                                                            0)]}],
                                                                   'type': 'SEQUENCE'},
                                     'PUSCH-ConfigDedicated-v1020': {'members': [{'members': [{'name': 'betaOffset-ACK-Index-MC-r10',
                                                                                               'restricted-to': [(0,
                                                                                                                  15)],
                                                                                               'type': 'INTEGER'},
                                                                                              {'name': 'betaOffset-RI-Index-MC-r10',
                                                                                               'restricted-to': [(0,
                                                                                                                  15)],
                                                                                               'type': 'INTEGER'},
                                                                                              {'name': 'betaOffset-CQI-Index-MC-r10',
                                                                                               'restricted-to': [(0,
                                                                                                                  15)],
                                                                                               'type': 'INTEGER'}],
                                                                                  'name': 'betaOffsetMC-r10',
                                                                                  'optional': True,
                                                                                  'type': 'SEQUENCE'},
                                                                                 {'name': 'groupHoppingDisabled-r10',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('true',
                                                                                              0)]},
                                                                                 {'name': 'dmrs-WithOCC-Activated-r10',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('true',
                                                                                              0)]}],
                                                                     'type': 'SEQUENCE'},
                                     'PUSCH-ConfigDedicated-v1130': {'members': [{'members': [{'name': 'release',
                                                                                               'type': 'NULL'},
                                                                                              {'members': [{'name': 'nPUSCH-Identity-r11',
                                                                                                            'restricted-to': [(0,
                                                                                                                               509)],
                                                                                                            'type': 'INTEGER'},
                                                                                                           {'name': 'nDMRS-CSH-Identity-r11',
                                                                                                            'restricted-to': [(0,
                                                                                                                               509)],
                                                                                                            'type': 'INTEGER'}],
                                                                                               'name': 'setup',
                                                                                               'type': 'SEQUENCE'}],
                                                                                  'name': 'pusch-DMRS-r11',
                                                                                  'type': 'CHOICE'}],
                                                                     'type': 'SEQUENCE'},
                                     'PUSCH-ConfigDedicated-v1250': {'members': [{'members': [{'name': 'release',
                                                                                               'type': 'NULL'},
                                                                                              {'members': [{'name': 'betaOffset-ACK-Index-SubframeSet2-r12',
                                                                                                            'restricted-to': [(0,
                                                                                                                               15)],
                                                                                                            'type': 'INTEGER'},
                                                                                                           {'name': 'betaOffset-RI-Index-SubframeSet2-r12',
                                                                                                            'restricted-to': [(0,
                                                                                                                               15)],
                                                                                                            'type': 'INTEGER'},
                                                                                                           {'name': 'betaOffset-CQI-Index-SubframeSet2-r12',
                                                                                                            'restricted-to': [(0,
                                                                                                                               15)],
                                                                                                            'type': 'INTEGER'},
                                                                                                           {'members': [{'name': 'betaOffset-ACK-Index-MC-SubframeSet2-r12',
                                                                                                                         'restricted-to': [(0,
                                                                                                                                            15)],
                                                                                                                         'type': 'INTEGER'},
                                                                                                                        {'name': 'betaOffset-RI-Index-MC-SubframeSet2-r12',
                                                                                                                         'restricted-to': [(0,
                                                                                                                                            15)],
                                                                                                                         'type': 'INTEGER'},
                                                                                                                        {'name': 'betaOffset-CQI-Index-MC-SubframeSet2-r12',
                                                                                                                         'restricted-to': [(0,
                                                                                                                                            15)],
                                                                                                                         'type': 'INTEGER'}],
                                                                                                            'name': 'betaOffsetMC-r12',
                                                                                                            'optional': True,
                                                                                                            'type': 'SEQUENCE'}],
                                                                                               'name': 'setup',
                                                                                               'type': 'SEQUENCE'}],
                                                                                  'name': 'uciOnPUSCH',
                                                                                  'type': 'CHOICE'}],
                                                                     'type': 'SEQUENCE'},
                                     'PUSCH-ConfigDedicated-v1430': {'members': [{'name': 'ce-PUSCH-NB-MaxTBS-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('on',
                                                                                              0)]},
                                                                                 {'name': 'ce-PUSCH-MaxBandwidth-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('bw5',
                                                                                              0)]},
                                                                                 {'name': 'tdd-PUSCH-UpPTS-r14',
                                                                                  'optional': True,
                                                                                  'type': 'TDD-PUSCH-UpPTS-r14'},
                                                                                 {'name': 'ul-DMRS-IFDMA-r14',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'enable256QAM-r14',
                                                                                  'optional': True,
                                                                                  'type': 'Enable256QAM-r14'}],
                                                                     'type': 'SEQUENCE'},
                                     'PUSCH-ConfigDedicatedSCell-r10': {'members': [{'name': 'groupHoppingDisabled-r10',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('true',
                                                                                                 0)]},
                                                                                    {'name': 'dmrs-WithOCC-Activated-r10',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('true',
                                                                                                 0)]}],
                                                                        'type': 'SEQUENCE'},
                                     'PUSCH-ConfigDedicatedSCell-v1430': {'members': [{'name': 'enable256QAM-r14',
                                                                                       'optional': True,
                                                                                       'type': 'Enable256QAM-r14'}],
                                                                          'type': 'SEQUENCE'},
                                     'PUSCH-EnhancementsConfig-r14': {'members': [{'name': 'release',
                                                                                   'type': 'NULL'},
                                                                                  {'members': [{'name': 'pusch-HoppingOffsetPUSCH-Enh-r14',
                                                                                                'optional': True,
                                                                                                'restricted-to': [(1,
                                                                                                                   100)],
                                                                                                'type': 'INTEGER'},
                                                                                               {'members': [{'name': 'interval-FDD-PUSCH-Enh-r14',
                                                                                                             'type': 'ENUMERATED',
                                                                                                             'values': [('int1',
                                                                                                                         0),
                                                                                                                        ('int2',
                                                                                                                         1),
                                                                                                                        ('int4',
                                                                                                                         2),
                                                                                                                        ('int8',
                                                                                                                         3)]},
                                                                                                            {'name': 'interval-TDD-PUSCH-Enh-r14',
                                                                                                             'type': 'ENUMERATED',
                                                                                                             'values': [('int1',
                                                                                                                         0),
                                                                                                                        ('int5',
                                                                                                                         1),
                                                                                                                        ('int10',
                                                                                                                         2),
                                                                                                                        ('int20',
                                                                                                                         3)]}],
                                                                                                'name': 'interval-ULHoppingPUSCH-Enh-r14',
                                                                                                'optional': True,
                                                                                                'type': 'CHOICE'}],
                                                                                   'name': 'setup',
                                                                                   'type': 'SEQUENCE'}],
                                                                      'type': 'CHOICE'},
                                     'Paging': {'members': [{'name': 'pagingRecordList',
                                                             'optional': True,
                                                             'type': 'PagingRecordList'},
                                                            {'name': 'systemInfoModification',
                                                             'optional': True,
                                                             'type': 'ENUMERATED',
                                                             'values': [('true',
                                                                         0)]},
                                                            {'name': 'etws-Indication',
                                                             'optional': True,
                                                             'type': 'ENUMERATED',
                                                             'values': [('true',
                                                                         0)]},
                                                            {'name': 'nonCriticalExtension',
                                                             'optional': True,
                                                             'type': 'Paging-v890-IEs'}],
                                                'type': 'SEQUENCE'},
                                     'Paging-v1130-IEs': {'members': [{'name': 'eab-ParamModification-r11',
                                                                       'optional': True,
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('true',
                                                                                   0)]},
                                                                      {'name': 'nonCriticalExtension',
                                                                       'optional': True,
                                                                       'type': 'Paging-v1310-IEs'}],
                                                          'type': 'SEQUENCE'},
                                     'Paging-v1310-IEs': {'members': [{'name': 'redistributionIndication-r13',
                                                                       'optional': True,
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('true',
                                                                                   0)]},
                                                                      {'name': 'systemInfoModification-eDRX-r13',
                                                                       'optional': True,
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('true',
                                                                                   0)]},
                                                                      {'members': [],
                                                                       'name': 'nonCriticalExtension',
                                                                       'optional': True,
                                                                       'type': 'SEQUENCE'}],
                                                          'type': 'SEQUENCE'},
                                     'Paging-v890-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                      'optional': True,
                                                                      'type': 'OCTET '
                                                                              'STRING'},
                                                                     {'name': 'nonCriticalExtension',
                                                                      'optional': True,
                                                                      'type': 'Paging-v920-IEs'}],
                                                         'type': 'SEQUENCE'},
                                     'Paging-v920-IEs': {'members': [{'name': 'cmas-Indication-r9',
                                                                      'optional': True,
                                                                      'type': 'ENUMERATED',
                                                                      'values': [('true',
                                                                                  0)]},
                                                                     {'name': 'nonCriticalExtension',
                                                                      'optional': True,
                                                                      'type': 'Paging-v1130-IEs'}],
                                                         'type': 'SEQUENCE'},
                                     'PagingRecord': {'members': [{'name': 'ue-Identity',
                                                                   'type': 'PagingUE-Identity'},
                                                                  {'name': 'cn-Domain',
                                                                   'type': 'ENUMERATED',
                                                                   'values': [('ps',
                                                                               0),
                                                                              ('cs',
                                                                               1)]},
                                                                  None],
                                                      'type': 'SEQUENCE'},
                                     'PagingRecordList': {'element': {'type': 'PagingRecord'},
                                                          'size': [(1,
                                                                    'maxPageRec')],
                                                          'type': 'SEQUENCE '
                                                                  'OF'},
                                     'PagingUE-Identity': {'members': [{'name': 's-TMSI',
                                                                        'type': 'S-TMSI'},
                                                                       {'name': 'imsi',
                                                                        'type': 'IMSI'},
                                                                       None],
                                                           'type': 'CHOICE'},
                                     'ParametersCDMA2000-r11': {'members': [{'members': [{'name': 'explicitValue',
                                                                                          'type': 'SystemTimeInfoCDMA2000'},
                                                                                         {'name': 'defaultValue',
                                                                                          'type': 'NULL'}],
                                                                             'name': 'systemTimeInfo-r11',
                                                                             'optional': True,
                                                                             'type': 'CHOICE'},
                                                                            {'name': 'searchWindowSize-r11',
                                                                             'restricted-to': [(0,
                                                                                                15)],
                                                                             'type': 'INTEGER'},
                                                                            {'members': [{'name': 'preRegistrationInfoHRPD-r11',
                                                                                          'type': 'PreRegistrationInfoHRPD'},
                                                                                         {'name': 'cellReselectionParametersHRPD-r11',
                                                                                          'optional': True,
                                                                                          'type': 'CellReselectionParametersCDMA2000-r11'}],
                                                                             'name': 'parametersHRPD-r11',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'csfb-RegistrationParam1XRTT-r11',
                                                                                          'optional': True,
                                                                                          'type': 'CSFB-RegistrationParam1XRTT'},
                                                                                         {'name': 'csfb-RegistrationParam1XRTT-Ext-r11',
                                                                                          'optional': True,
                                                                                          'type': 'CSFB-RegistrationParam1XRTT-v920'},
                                                                                         {'name': 'longCodeState1XRTT-r11',
                                                                                          'optional': True,
                                                                                          'size': [42],
                                                                                          'type': 'BIT '
                                                                                                  'STRING'},
                                                                                         {'name': 'cellReselectionParameters1XRTT-r11',
                                                                                          'optional': True,
                                                                                          'type': 'CellReselectionParametersCDMA2000-r11'},
                                                                                         {'name': 'ac-BarringConfig1XRTT-r11',
                                                                                          'optional': True,
                                                                                          'type': 'AC-BarringConfig1XRTT-r9'},
                                                                                         {'name': 'csfb-SupportForDualRxUEs-r11',
                                                                                          'optional': True,
                                                                                          'type': 'BOOLEAN'},
                                                                                         {'name': 'csfb-DualRxTxSupport-r11',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('true',
                                                                                                      0)]}],
                                                                             'name': 'parameters1XRTT-r11',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            None],
                                                                'type': 'SEQUENCE'},
                                     'PerCC-GapIndication-r14': {'members': [{'name': 'servCellId-r14',
                                                                              'type': 'ServCellIndex-r13'},
                                                                             {'name': 'gapIndication-r14',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('gap',
                                                                                          0),
                                                                                         ('ncsg',
                                                                                          1),
                                                                                         ('nogap-noNcsg',
                                                                                          2)]}],
                                                                 'type': 'SEQUENCE'},
                                     'PerCC-GapIndicationList-r14': {'element': {'type': 'PerCC-GapIndication-r14'},
                                                                     'size': [(1,
                                                                               'maxServCell-r13')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'PeriodicBSR-Timer-r12': {'type': 'ENUMERATED',
                                                               'values': [('sf5',
                                                                           0),
                                                                          ('sf10',
                                                                           1),
                                                                          ('sf16',
                                                                           2),
                                                                          ('sf20',
                                                                           3),
                                                                          ('sf32',
                                                                           4),
                                                                          ('sf40',
                                                                           5),
                                                                          ('sf64',
                                                                           6),
                                                                          ('sf80',
                                                                           7),
                                                                          ('sf128',
                                                                           8),
                                                                          ('sf160',
                                                                           9),
                                                                          ('sf320',
                                                                           10),
                                                                          ('sf640',
                                                                           11),
                                                                          ('sf1280',
                                                                           12),
                                                                          ('sf2560',
                                                                           13),
                                                                          ('infinity',
                                                                           14),
                                                                          ('spare1',
                                                                           15)]},
                                     'PhyLayerParameters': {'members': [{'name': 'ue-TxAntennaSelectionSupported',
                                                                         'type': 'BOOLEAN'},
                                                                        {'name': 'ue-SpecificRefSigsSupported',
                                                                         'type': 'BOOLEAN'}],
                                                            'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v1020': {'members': [{'name': 'twoAntennaPortsForPUCCH-r10',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'tm9-With-8Tx-FDD-r10',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'pmi-Disabling-r10',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'crossCarrierScheduling-r10',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'simultaneousPUCCH-PUSCH-r10',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'multiClusterPUSCH-WithinCC-r10',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'nonContiguousUL-RA-WithinCC-List-r10',
                                                                               'optional': True,
                                                                               'type': 'NonContiguousUL-RA-WithinCC-List-r10'}],
                                                                  'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v1130': {'members': [{'name': 'crs-InterfHandl-r11',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ePDCCH-r11',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'multiACK-CSI-Reporting-r11',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ss-CCH-InterfHandl-r11',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'tdd-SpecialSubframe-r11',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'txDiv-PUCCH1b-ChSelect-r11',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ul-CoMP-r11',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v1170': {'members': [{'name': 'interBandTDD-CA-WithDifferentConfig-r11',
                                                                               'optional': True,
                                                                               'size': [2],
                                                                               'type': 'BIT '
                                                                                       'STRING'}],
                                                                  'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v1250': {'members': [{'name': 'e-HARQ-Pattern-FDD-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'enhanced-4TxCodebook-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'tdd-FDD-CA-PCellDuplex-r12',
                                                                               'optional': True,
                                                                               'size': [2],
                                                                               'type': 'BIT '
                                                                                       'STRING'},
                                                                              {'name': 'phy-TDD-ReConfig-TDD-PCell-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'phy-TDD-ReConfig-FDD-PCell-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'pusch-FeedbackMode-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'pusch-SRS-PowerControl-SubframeSet-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'csi-SubframeSet-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'noResourceRestrictionForTTIBundling-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'discoverySignalsInDeactSCell-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'naics-Capability-List-r12',
                                                                               'optional': True,
                                                                               'type': 'NAICS-Capability-List-r12'}],
                                                                  'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v1280': {'members': [{'name': 'alternativeTBS-Indices-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v1310': {'members': [{'name': 'aperiodicCSI-Reporting-r13',
                                                                               'optional': True,
                                                                               'size': [2],
                                                                               'type': 'BIT '
                                                                                       'STRING'},
                                                                              {'name': 'codebook-HARQ-ACK-r13',
                                                                               'optional': True,
                                                                               'size': [2],
                                                                               'type': 'BIT '
                                                                                       'STRING'},
                                                                              {'name': 'crossCarrierScheduling-B5C-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'fdd-HARQ-TimingTDD-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'maxNumberUpdatedCSI-Proc-r13',
                                                                               'optional': True,
                                                                               'restricted-to': [(5,
                                                                                                  32)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'pucch-Format4-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'pucch-Format5-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'pucch-SCell-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'spatialBundling-HARQ-ACK-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'members': [{'name': 'maxNumberDecoding-r13',
                                                                                            'optional': True,
                                                                                            'restricted-to': [(1,
                                                                                                               32)],
                                                                                            'type': 'INTEGER'},
                                                                                           {'name': 'pdcch-CandidateReductions-r13',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('supported',
                                                                                                        0)]},
                                                                                           {'name': 'skipMonitoringDCI-Format0-1A-r13',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('supported',
                                                                                                        0)]}],
                                                                               'name': 'supportedBlindDecoding-r13',
                                                                               'optional': True,
                                                                               'type': 'SEQUENCE'},
                                                                              {'name': 'uci-PUSCH-Ext-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'crs-InterfMitigationTM10-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'pdsch-CollisionHandling-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v1320': {'members': [{'name': 'mimo-UE-Parameters-r13',
                                                                               'optional': True,
                                                                               'type': 'MIMO-UE-Parameters-r13'}],
                                                                  'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v1330': {'members': [{'name': 'cch-InterfMitigation-RefRecTypeA-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'cch-InterfMitigation-RefRecTypeB-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'cch-InterfMitigation-MaxNumCCs-r13',
                                                                               'optional': True,
                                                                               'restricted-to': [(1,
                                                                                                  'maxServCell-r13')],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'crs-InterfMitigationTM1toTM9-r13',
                                                                               'optional': True,
                                                                               'restricted-to': [(1,
                                                                                                  'maxServCell-r13')],
                                                                               'type': 'INTEGER'}],
                                                                  'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v1430': {'members': [{'name': 'ce-PUSCH-NB-MaxTBS-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ce-PDSCH-PUSCH-MaxBandwidth-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('bw5',
                                                                                           0),
                                                                                          ('bw20',
                                                                                           1)]},
                                                                              {'name': 'ce-HARQ-AckBundling-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ce-PDSCH-TenProcesses-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ce-RetuningSymbols-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n0',
                                                                                           0),
                                                                                          ('n1',
                                                                                           1)]},
                                                                              {'name': 'ce-PDSCH-PUSCH-Enhancement-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ce-SchedulingEnhancement-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ce-SRS-Enhancement-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ce-PUCCH-Enhancement-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ce-ClosedLoopTxAntennaSelection-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'tdd-SpecialSubframe-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'tdd-TTI-Bundling-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'dmrs-LessUpPTS-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'mimo-UE-Parameters-v1430',
                                                                               'optional': True,
                                                                               'type': 'MIMO-UE-Parameters-v1430'},
                                                                              {'name': 'alternativeTBS-Index-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'feMBMS-Unicast-Parameters-r14',
                                                                               'optional': True,
                                                                               'type': 'FeMBMS-Unicast-Parameters-r14'}],
                                                                  'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v920': {'members': [{'name': 'enhancedDualLayerFDD-r9',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('supported',
                                                                                          0)]},
                                                                             {'name': 'enhancedDualLayerTDD-r9',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('supported',
                                                                                          0)]}],
                                                                 'type': 'SEQUENCE'},
                                     'PhyLayerParameters-v9d0': {'members': [{'name': 'tm5-FDD-r9',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('supported',
                                                                                          0)]},
                                                                             {'name': 'tm5-TDD-r9',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('supported',
                                                                                          0)]}],
                                                                 'type': 'SEQUENCE'},
                                     'PhysCellId': {'restricted-to': [(0, 503)],
                                                    'type': 'INTEGER'},
                                     'PhysCellIdCDMA2000': {'restricted-to': [(0,
                                                                               'maxPNOffset')],
                                                            'type': 'INTEGER'},
                                     'PhysCellIdGERAN': {'members': [{'name': 'networkColourCode',
                                                                      'size': [3],
                                                                      'type': 'BIT '
                                                                              'STRING'},
                                                                     {'name': 'baseStationColourCode',
                                                                      'size': [3],
                                                                      'type': 'BIT '
                                                                              'STRING'}],
                                                         'type': 'SEQUENCE'},
                                     'PhysCellIdList-r13': {'element': {'type': 'PhysCellId'},
                                                            'size': [(1,
                                                                      'maxSL-DiscCells-r13')],
                                                            'type': 'SEQUENCE '
                                                                    'OF'},
                                     'PhysCellIdListCDMA2000': {'element': {'type': 'PhysCellIdCDMA2000'},
                                                                'size': [(1,
                                                                          16)],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'PhysCellIdListCDMA2000-v920': {'element': {'type': 'PhysCellIdCDMA2000'},
                                                                     'size': [(0,
                                                                               24)],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'PhysCellIdRange': {'members': [{'name': 'start',
                                                                      'type': 'PhysCellId'},
                                                                     {'name': 'range',
                                                                      'optional': True,
                                                                      'type': 'ENUMERATED',
                                                                      'values': [('n4',
                                                                                  0),
                                                                                 ('n8',
                                                                                  1),
                                                                                 ('n12',
                                                                                  2),
                                                                                 ('n16',
                                                                                  3),
                                                                                 ('n24',
                                                                                  4),
                                                                                 ('n32',
                                                                                  5),
                                                                                 ('n48',
                                                                                  6),
                                                                                 ('n64',
                                                                                  7),
                                                                                 ('n84',
                                                                                  8),
                                                                                 ('n96',
                                                                                  9),
                                                                                 ('n128',
                                                                                  10),
                                                                                 ('n168',
                                                                                  11),
                                                                                 ('n252',
                                                                                  12),
                                                                                 ('n504',
                                                                                  13),
                                                                                 ('spare2',
                                                                                  14),
                                                                                 ('spare1',
                                                                                  15)]}],
                                                         'type': 'SEQUENCE'},
                                     'PhysCellIdRangeUTRA-FDD-r9': {'members': [{'name': 'start-r9',
                                                                                 'type': 'PhysCellIdUTRA-FDD'},
                                                                                {'name': 'range-r9',
                                                                                 'optional': True,
                                                                                 'restricted-to': [(2,
                                                                                                    512)],
                                                                                 'type': 'INTEGER'}],
                                                                    'type': 'SEQUENCE'},
                                     'PhysCellIdRangeUTRA-FDDList-r9': {'element': {'type': 'PhysCellIdRangeUTRA-FDD-r9'},
                                                                        'size': [(1,
                                                                                  'maxPhysCellIdRange-r9')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'PhysCellIdUTRA-FDD': {'restricted-to': [(0,
                                                                               511)],
                                                            'type': 'INTEGER'},
                                     'PhysCellIdUTRA-TDD': {'restricted-to': [(0,
                                                                               127)],
                                                            'type': 'INTEGER'},
                                     'PhysicalConfigDedicated': {'members': [{'name': 'pdsch-ConfigDedicated',
                                                                              'optional': True,
                                                                              'type': 'PDSCH-ConfigDedicated'},
                                                                             {'name': 'pucch-ConfigDedicated',
                                                                              'optional': True,
                                                                              'type': 'PUCCH-ConfigDedicated'},
                                                                             {'name': 'pusch-ConfigDedicated',
                                                                              'optional': True,
                                                                              'type': 'PUSCH-ConfigDedicated'},
                                                                             {'name': 'uplinkPowerControlDedicated',
                                                                              'optional': True,
                                                                              'type': 'UplinkPowerControlDedicated'},
                                                                             {'name': 'tpc-PDCCH-ConfigPUCCH',
                                                                              'optional': True,
                                                                              'type': 'TPC-PDCCH-Config'},
                                                                             {'name': 'tpc-PDCCH-ConfigPUSCH',
                                                                              'optional': True,
                                                                              'type': 'TPC-PDCCH-Config'},
                                                                             {'name': 'cqi-ReportConfig',
                                                                              'optional': True,
                                                                              'type': 'CQI-ReportConfig'},
                                                                             {'name': 'soundingRS-UL-ConfigDedicated',
                                                                              'optional': True,
                                                                              'type': 'SoundingRS-UL-ConfigDedicated'},
                                                                             {'members': [{'name': 'explicitValue',
                                                                                           'type': 'AntennaInfoDedicated'},
                                                                                          {'name': 'defaultValue',
                                                                                           'type': 'NULL'}],
                                                                              'name': 'antennaInfo',
                                                                              'optional': True,
                                                                              'type': 'CHOICE'},
                                                                             {'name': 'schedulingRequestConfig',
                                                                              'optional': True,
                                                                              'type': 'SchedulingRequestConfig'},
                                                                             None,
                                                                             [{'name': 'cqi-ReportConfig-v920',
                                                                               'optional': True,
                                                                               'type': 'CQI-ReportConfig-v920'},
                                                                              {'name': 'antennaInfo-v920',
                                                                               'optional': True,
                                                                               'type': 'AntennaInfoDedicated-v920'}],
                                                                             [{'members': [{'name': 'explicitValue-r10',
                                                                                            'type': 'AntennaInfoDedicated-r10'},
                                                                                           {'name': 'defaultValue',
                                                                                            'type': 'NULL'}],
                                                                               'name': 'antennaInfo-r10',
                                                                               'optional': True,
                                                                               'type': 'CHOICE'},
                                                                              {'name': 'antennaInfoUL-r10',
                                                                               'optional': True,
                                                                               'type': 'AntennaInfoUL-r10'},
                                                                              {'name': 'cif-Presence-r10',
                                                                               'optional': True,
                                                                               'type': 'BOOLEAN'},
                                                                              {'name': 'cqi-ReportConfig-r10',
                                                                               'optional': True,
                                                                               'type': 'CQI-ReportConfig-r10'},
                                                                              {'name': 'csi-RS-Config-r10',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-Config-r10'},
                                                                              {'name': 'pucch-ConfigDedicated-v1020',
                                                                               'optional': True,
                                                                               'type': 'PUCCH-ConfigDedicated-v1020'},
                                                                              {'name': 'pusch-ConfigDedicated-v1020',
                                                                               'optional': True,
                                                                               'type': 'PUSCH-ConfigDedicated-v1020'},
                                                                              {'name': 'schedulingRequestConfig-v1020',
                                                                               'optional': True,
                                                                               'type': 'SchedulingRequestConfig-v1020'},
                                                                              {'name': 'soundingRS-UL-ConfigDedicated-v1020',
                                                                               'optional': True,
                                                                               'type': 'SoundingRS-UL-ConfigDedicated-v1020'},
                                                                              {'name': 'soundingRS-UL-ConfigDedicatedAperiodic-r10',
                                                                               'optional': True,
                                                                               'type': 'SoundingRS-UL-ConfigDedicatedAperiodic-r10'},
                                                                              {'name': 'uplinkPowerControlDedicated-v1020',
                                                                               'optional': True,
                                                                               'type': 'UplinkPowerControlDedicated-v1020'}],
                                                                             [{'members': [{'name': 'release',
                                                                                            'type': 'NULL'},
                                                                                           {'members': [{'name': 'additionalSpectrumEmissionPCell-r10',
                                                                                                         'type': 'AdditionalSpectrumEmission'}],
                                                                                            'name': 'setup',
                                                                                            'type': 'SEQUENCE'}],
                                                                               'name': 'additionalSpectrumEmissionCA-r10',
                                                                               'optional': True,
                                                                               'type': 'CHOICE'}],
                                                                             [{'name': 'csi-RS-ConfigNZPToReleaseList-r11',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-ConfigNZPToReleaseList-r11'},
                                                                              {'name': 'csi-RS-ConfigNZPToAddModList-r11',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-ConfigNZPToAddModList-r11'},
                                                                              {'name': 'csi-RS-ConfigZPToReleaseList-r11',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-ConfigZPToReleaseList-r11'},
                                                                              {'name': 'csi-RS-ConfigZPToAddModList-r11',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-ConfigZPToAddModList-r11'},
                                                                              {'name': 'epdcch-Config-r11',
                                                                               'optional': True,
                                                                               'type': 'EPDCCH-Config-r11'},
                                                                              {'name': 'pdsch-ConfigDedicated-v1130',
                                                                               'optional': True,
                                                                               'type': 'PDSCH-ConfigDedicated-v1130'},
                                                                              {'name': 'cqi-ReportConfig-v1130',
                                                                               'optional': True,
                                                                               'type': 'CQI-ReportConfig-v1130'},
                                                                              {'name': 'pucch-ConfigDedicated-v1130',
                                                                               'optional': True,
                                                                               'type': 'PUCCH-ConfigDedicated-v1130'},
                                                                              {'name': 'pusch-ConfigDedicated-v1130',
                                                                               'optional': True,
                                                                               'type': 'PUSCH-ConfigDedicated-v1130'},
                                                                              {'name': 'uplinkPowerControlDedicated-v1130',
                                                                               'optional': True,
                                                                               'type': 'UplinkPowerControlDedicated-v1130'}],
                                                                             [{'name': 'antennaInfo-v1250',
                                                                               'optional': True,
                                                                               'type': 'AntennaInfoDedicated-v1250'},
                                                                              {'name': 'eimta-MainConfig-r12',
                                                                               'optional': True,
                                                                               'type': 'EIMTA-MainConfig-r12'},
                                                                              {'name': 'eimta-MainConfigPCell-r12',
                                                                               'optional': True,
                                                                               'type': 'EIMTA-MainConfigServCell-r12'},
                                                                              {'name': 'pucch-ConfigDedicated-v1250',
                                                                               'optional': True,
                                                                               'type': 'PUCCH-ConfigDedicated-v1250'},
                                                                              {'name': 'cqi-ReportConfigPCell-v1250',
                                                                               'optional': True,
                                                                               'type': 'CQI-ReportConfig-v1250'},
                                                                              {'name': 'uplinkPowerControlDedicated-v1250',
                                                                               'optional': True,
                                                                               'type': 'UplinkPowerControlDedicated-v1250'},
                                                                              {'name': 'pusch-ConfigDedicated-v1250',
                                                                               'optional': True,
                                                                               'type': 'PUSCH-ConfigDedicated-v1250'},
                                                                              {'name': 'csi-RS-Config-v1250',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-Config-v1250'}],
                                                                             [{'name': 'pdsch-ConfigDedicated-v1280',
                                                                               'optional': True,
                                                                               'type': 'PDSCH-ConfigDedicated-v1280'}],
                                                                             [{'name': 'pdsch-ConfigDedicated-v1310',
                                                                               'optional': True,
                                                                               'type': 'PDSCH-ConfigDedicated-v1310'},
                                                                              {'name': 'pucch-ConfigDedicated-r13',
                                                                               'optional': True,
                                                                               'type': 'PUCCH-ConfigDedicated-r13'},
                                                                              {'name': 'pusch-ConfigDedicated-r13',
                                                                               'optional': True,
                                                                               'type': 'PUSCH-ConfigDedicated-r13'},
                                                                              {'name': 'pdcch-CandidateReductions-r13',
                                                                               'optional': True,
                                                                               'type': 'PDCCH-CandidateReductions-r13'},
                                                                              {'name': 'cqi-ReportConfig-v1310',
                                                                               'optional': True,
                                                                               'type': 'CQI-ReportConfig-v1310'},
                                                                              {'name': 'soundingRS-UL-ConfigDedicated-v1310',
                                                                               'optional': True,
                                                                               'type': 'SoundingRS-UL-ConfigDedicated-v1310'},
                                                                              {'name': 'soundingRS-UL-ConfigDedicatedUpPTsExt-r13',
                                                                               'optional': True,
                                                                               'type': 'SoundingRS-UL-ConfigDedicatedUpPTsExt-r13'},
                                                                              {'name': 'soundingRS-UL-ConfigDedicatedAperiodic-v1310',
                                                                               'optional': True,
                                                                               'type': 'SoundingRS-UL-ConfigDedicatedAperiodic-v1310'},
                                                                              {'name': 'soundingRS-UL-ConfigDedicatedAperiodicUpPTsExt-r13',
                                                                               'optional': True,
                                                                               'type': 'SoundingRS-UL-ConfigDedicatedAperiodicUpPTsExt-r13'},
                                                                              {'name': 'csi-RS-Config-v1310',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-Config-v1310'},
                                                                              {'members': [{'name': 'release',
                                                                                            'type': 'NULL'},
                                                                                           {'name': 'setup',
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('ce-ModeA',
                                                                                                        0),
                                                                                                       ('ce-ModeB',
                                                                                                        1)]}],
                                                                               'name': 'ce-Mode-r13',
                                                                               'optional': True,
                                                                               'type': 'CHOICE'},
                                                                              {'name': 'csi-RS-ConfigNZPToAddModListExt-r13',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-ConfigNZPToAddModListExt-r13'},
                                                                              {'name': 'csi-RS-ConfigNZPToReleaseListExt-r13',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-ConfigNZPToReleaseListExt-r13'}],
                                                                             [{'name': 'cqi-ReportConfig-v1320',
                                                                               'optional': True,
                                                                               'type': 'CQI-ReportConfig-v1320'}],
                                                                             [{'members': [{'name': 'release',
                                                                                            'type': 'NULL'},
                                                                                           {'element': {'type': 'SRS-TPC-PDCCH-Config-r14'},
                                                                                            'name': 'setup',
                                                                                            'size': [(1,
                                                                                                      32)],
                                                                                            'type': 'SEQUENCE '
                                                                                                    'OF'}],
                                                                               'name': 'typeA-SRS-TPC-PDCCH-Group-r14',
                                                                               'optional': True,
                                                                               'type': 'CHOICE'},
                                                                              {'members': [{'name': 'release',
                                                                                            'type': 'NULL'},
                                                                                           {'members': [{'name': 'k-max-r14',
                                                                                                         'type': 'ENUMERATED',
                                                                                                         'values': [('l1',
                                                                                                                     0),
                                                                                                                    ('l3',
                                                                                                                     1)]},
                                                                                                        {'name': 'p-a-must-r14',
                                                                                                         'optional': True,
                                                                                                         'type': 'ENUMERATED',
                                                                                                         'values': [('dB-6',
                                                                                                                     0),
                                                                                                                    ('dB-4dot77',
                                                                                                                     1),
                                                                                                                    ('dB-3',
                                                                                                                     2),
                                                                                                                    ('dB-1dot77',
                                                                                                                     3),
                                                                                                                    ('dB0',
                                                                                                                     4),
                                                                                                                    ('dB1',
                                                                                                                     5),
                                                                                                                    ('dB2',
                                                                                                                     6),
                                                                                                                    ('dB3',
                                                                                                                     7)]}],
                                                                                            'name': 'setup',
                                                                                            'type': 'SEQUENCE'}],
                                                                               'name': 'must-Config-r14',
                                                                               'optional': True,
                                                                               'type': 'CHOICE'},
                                                                              {'name': 'pusch-EnhancementsConfig-r14',
                                                                               'optional': True,
                                                                               'type': 'PUSCH-EnhancementsConfig-r14'},
                                                                              {'name': 'ce-pdsch-pusch-EnhancementConfig-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('on',
                                                                                           0)]},
                                                                              {'name': 'antennaInfo-v1430',
                                                                               'optional': True,
                                                                               'type': 'AntennaInfoDedicated-v1430'},
                                                                              {'name': 'pucch-ConfigDedicated-v1430',
                                                                               'optional': True,
                                                                               'type': 'PUCCH-ConfigDedicated-v1430'},
                                                                              {'name': 'pdsch-ConfigDedicated-v1430',
                                                                               'optional': True,
                                                                               'type': 'PDSCH-ConfigDedicated-v1430'},
                                                                              {'name': 'pusch-ConfigDedicated-v1430',
                                                                               'optional': True,
                                                                               'type': 'PUSCH-ConfigDedicated-v1430'},
                                                                              {'element': {'type': 'SoundingRS-UL-ConfigDedicated'},
                                                                               'name': 'soundingRS-UL-PeriodicConfigDedicatedList-r14',
                                                                               'optional': True,
                                                                               'size': [(1,
                                                                                         2)],
                                                                               'type': 'SEQUENCE '
                                                                                       'OF'},
                                                                              {'element': {'type': 'SoundingRS-UL-ConfigDedicatedUpPTsExt-r13'},
                                                                               'name': 'soundingRS-UL-PeriodicConfigDedicatedUpPTsExtList-r14',
                                                                               'optional': True,
                                                                               'size': [(1,
                                                                                         4)],
                                                                               'type': 'SEQUENCE '
                                                                                       'OF'},
                                                                              {'element': {'type': 'SoundingRS-UL-ConfigDedicatedAperiodic-r10'},
                                                                               'name': 'soundingRS-UL-AperiodicConfigDedicatedList-r14',
                                                                               'optional': True,
                                                                               'size': [(1,
                                                                                         2)],
                                                                               'type': 'SEQUENCE '
                                                                                       'OF'},
                                                                              {'element': {'type': 'SoundingRS-UL-ConfigDedicatedAperiodicUpPTsExt-r13'},
                                                                               'name': 'soundingRS-UL-ConfigDedicatedApUpPTsExtList-r14',
                                                                               'optional': True,
                                                                               'size': [(1,
                                                                                         4)],
                                                                               'type': 'SEQUENCE '
                                                                                       'OF'},
                                                                              {'name': 'csi-RS-Config-v1430',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-Config-v1430'},
                                                                              {'name': 'csi-RS-ConfigZP-ApList-r14',
                                                                               'optional': True,
                                                                               'type': 'CSI-RS-ConfigZP-ApList-r14'},
                                                                              {'name': 'cqi-ReportConfig-v1430',
                                                                               'optional': True,
                                                                               'type': 'CQI-ReportConfig-v1430'},
                                                                              {'name': 'semiOpenLoop-r14',
                                                                               'optional': True,
                                                                               'type': 'BOOLEAN'}]],
                                                                 'type': 'SEQUENCE'},
                                     'PhysicalConfigDedicated-v1370': {'members': [{'name': 'pucch-ConfigDedicated-v1370',
                                                                                    'optional': True,
                                                                                    'type': 'PUCCH-ConfigDedicated-v1370'}],
                                                                       'type': 'SEQUENCE'},
                                     'PhysicalConfigDedicatedSCell-r10': {'members': [{'members': [{'name': 'antennaInfo-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'AntennaInfoDedicated-r10'},
                                                                                                   {'name': 'crossCarrierSchedulingConfig-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'CrossCarrierSchedulingConfig-r10'},
                                                                                                   {'name': 'csi-RS-Config-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'CSI-RS-Config-r10'},
                                                                                                   {'name': 'pdsch-ConfigDedicated-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'PDSCH-ConfigDedicated'}],
                                                                                       'name': 'nonUL-Configuration-r10',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'},
                                                                                      {'members': [{'name': 'antennaInfoUL-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'AntennaInfoUL-r10'},
                                                                                                   {'name': 'pusch-ConfigDedicatedSCell-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'PUSCH-ConfigDedicatedSCell-r10'},
                                                                                                   {'name': 'uplinkPowerControlDedicatedSCell-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'UplinkPowerControlDedicatedSCell-r10'},
                                                                                                   {'name': 'cqi-ReportConfigSCell-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'CQI-ReportConfigSCell-r10'},
                                                                                                   {'name': 'soundingRS-UL-ConfigDedicated-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'SoundingRS-UL-ConfigDedicated'},
                                                                                                   {'name': 'soundingRS-UL-ConfigDedicated-v1020',
                                                                                                    'optional': True,
                                                                                                    'type': 'SoundingRS-UL-ConfigDedicated-v1020'},
                                                                                                   {'name': 'soundingRS-UL-ConfigDedicatedAperiodic-r10',
                                                                                                    'optional': True,
                                                                                                    'type': 'SoundingRS-UL-ConfigDedicatedAperiodic-r10'}],
                                                                                       'name': 'ul-Configuration-r10',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'},
                                                                                      None,
                                                                                      [{'name': 'csi-RS-ConfigNZPToReleaseList-r11',
                                                                                        'optional': True,
                                                                                        'type': 'CSI-RS-ConfigNZPToReleaseList-r11'},
                                                                                       {'name': 'csi-RS-ConfigNZPToAddModList-r11',
                                                                                        'optional': True,
                                                                                        'type': 'CSI-RS-ConfigNZPToAddModList-r11'},
                                                                                       {'name': 'csi-RS-ConfigZPToReleaseList-r11',
                                                                                        'optional': True,
                                                                                        'type': 'CSI-RS-ConfigZPToReleaseList-r11'},
                                                                                       {'name': 'csi-RS-ConfigZPToAddModList-r11',
                                                                                        'optional': True,
                                                                                        'type': 'CSI-RS-ConfigZPToAddModList-r11'},
                                                                                       {'name': 'epdcch-Config-r11',
                                                                                        'optional': True,
                                                                                        'type': 'EPDCCH-Config-r11'},
                                                                                       {'name': 'pdsch-ConfigDedicated-v1130',
                                                                                        'optional': True,
                                                                                        'type': 'PDSCH-ConfigDedicated-v1130'},
                                                                                       {'name': 'cqi-ReportConfig-v1130',
                                                                                        'optional': True,
                                                                                        'type': 'CQI-ReportConfig-v1130'},
                                                                                       {'name': 'pusch-ConfigDedicated-v1130',
                                                                                        'optional': True,
                                                                                        'type': 'PUSCH-ConfigDedicated-v1130'},
                                                                                       {'name': 'uplinkPowerControlDedicatedSCell-v1130',
                                                                                        'optional': True,
                                                                                        'type': 'UplinkPowerControlDedicated-v1130'}],
                                                                                      [{'name': 'antennaInfo-v1250',
                                                                                        'optional': True,
                                                                                        'type': 'AntennaInfoDedicated-v1250'},
                                                                                       {'name': 'eimta-MainConfigSCell-r12',
                                                                                        'optional': True,
                                                                                        'type': 'EIMTA-MainConfigServCell-r12'},
                                                                                       {'name': 'cqi-ReportConfigSCell-v1250',
                                                                                        'optional': True,
                                                                                        'type': 'CQI-ReportConfig-v1250'},
                                                                                       {'name': 'uplinkPowerControlDedicatedSCell-v1250',
                                                                                        'optional': True,
                                                                                        'type': 'UplinkPowerControlDedicated-v1250'},
                                                                                       {'name': 'csi-RS-Config-v1250',
                                                                                        'optional': True,
                                                                                        'type': 'CSI-RS-Config-v1250'}],
                                                                                      [{'name': 'pdsch-ConfigDedicated-v1280',
                                                                                        'optional': True,
                                                                                        'type': 'PDSCH-ConfigDedicated-v1280'}],
                                                                                      [{'name': 'pucch-Cell-r13',
                                                                                        'optional': True,
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('true',
                                                                                                    0)]},
                                                                                       {'members': [{'name': 'release',
                                                                                                     'type': 'NULL'},
                                                                                                    {'members': [{'name': 'pucch-ConfigDedicated-r13',
                                                                                                                  'optional': True,
                                                                                                                  'type': 'PUCCH-ConfigDedicated-r13'},
                                                                                                                 {'name': 'schedulingRequestConfig-r13',
                                                                                                                  'optional': True,
                                                                                                                  'type': 'SchedulingRequestConfigSCell-r13'},
                                                                                                                 {'name': 'tpc-PDCCH-ConfigPUCCH-SCell-r13',
                                                                                                                  'optional': True,
                                                                                                                  'type': 'TPC-PDCCH-ConfigSCell-r13'},
                                                                                                                 {'name': 'pusch-ConfigDedicated-r13',
                                                                                                                  'optional': True,
                                                                                                                  'type': 'PUSCH-ConfigDedicated-r13'},
                                                                                                                 {'name': 'uplinkPowerControlDedicated-r13',
                                                                                                                  'optional': True,
                                                                                                                  'type': 'UplinkPowerControlDedicatedSCell-v1310'}],
                                                                                                     'name': 'setup',
                                                                                                     'type': 'SEQUENCE'}],
                                                                                        'name': 'pucch-SCell',
                                                                                        'optional': True,
                                                                                        'type': 'CHOICE'},
                                                                                       {'name': 'crossCarrierSchedulingConfig-r13',
                                                                                        'optional': True,
                                                                                        'type': 'CrossCarrierSchedulingConfig-r13'},
                                                                                       {'name': 'pdcch-ConfigSCell-r13',
                                                                                        'optional': True,
                                                                                        'type': 'PDCCH-ConfigSCell-r13'},
                                                                                       {'name': 'cqi-ReportConfig-v1310',
                                                                                        'optional': True,
                                                                                        'type': 'CQI-ReportConfig-v1310'},
                                                                                       {'name': 'pdsch-ConfigDedicated-v1310',
                                                                                        'optional': True,
                                                                                        'type': 'PDSCH-ConfigDedicated-v1310'},
                                                                                       {'name': 'soundingRS-UL-ConfigDedicated-v1310',
                                                                                        'optional': True,
                                                                                        'type': 'SoundingRS-UL-ConfigDedicated-v1310'},
                                                                                       {'name': 'soundingRS-UL-ConfigDedicatedUpPTsExt-r13',
                                                                                        'optional': True,
                                                                                        'type': 'SoundingRS-UL-ConfigDedicatedUpPTsExt-r13'},
                                                                                       {'name': 'soundingRS-UL-ConfigDedicatedAperiodic-v1310',
                                                                                        'optional': True,
                                                                                        'type': 'SoundingRS-UL-ConfigDedicatedAperiodic-v1310'},
                                                                                       {'name': 'soundingRS-UL-ConfigDedicatedAperiodicUpPTsExt-r13',
                                                                                        'optional': True,
                                                                                        'type': 'SoundingRS-UL-ConfigDedicatedAperiodicUpPTsExt-r13'},
                                                                                       {'name': 'csi-RS-Config-v1310',
                                                                                        'optional': True,
                                                                                        'type': 'CSI-RS-Config-v1310'},
                                                                                       {'name': 'laa-SCellConfiguration-r13',
                                                                                        'optional': True,
                                                                                        'type': 'LAA-SCellConfiguration-r13'},
                                                                                       {'name': 'csi-RS-ConfigNZPToAddModListExt-r13',
                                                                                        'optional': True,
                                                                                        'type': 'CSI-RS-ConfigNZPToAddModListExt-r13'},
                                                                                       {'name': 'csi-RS-ConfigNZPToReleaseListExt-r13',
                                                                                        'optional': True,
                                                                                        'type': 'CSI-RS-ConfigNZPToReleaseListExt-r13'}],
                                                                                      [{'name': 'cqi-ReportConfig-v1320',
                                                                                        'optional': True,
                                                                                        'type': 'CQI-ReportConfig-v1320'}],
                                                                                      [{'name': 'laa-SCellConfiguration-v1430',
                                                                                        'optional': True,
                                                                                        'type': 'LAA-SCellConfiguration-v1430'},
                                                                                       {'name': 'typeB-SRS-TPC-PDCCH-Config-r14',
                                                                                        'optional': True,
                                                                                        'type': 'SRS-TPC-PDCCH-Config-r14'},
                                                                                       {'name': 'uplinkPUSCH-LessPowerControlDedicated-v1430',
                                                                                        'optional': True,
                                                                                        'type': 'UplinkPUSCH-LessPowerControlDedicated-v1430'},
                                                                                       {'element': {'type': 'SoundingRS-UL-ConfigDedicated'},
                                                                                        'name': 'soundingRS-UL-PeriodicConfigDedicatedList-r14',
                                                                                        'optional': True,
                                                                                        'size': [(1,
                                                                                                  2)],
                                                                                        'type': 'SEQUENCE '
                                                                                                'OF'},
                                                                                       {'element': {'type': 'SoundingRS-UL-ConfigDedicatedUpPTsExt-r13'},
                                                                                        'name': 'soundingRS-UL-PeriodicConfigDedicatedUpPTsExtList-r14',
                                                                                        'optional': True,
                                                                                        'size': [(1,
                                                                                                  4)],
                                                                                        'type': 'SEQUENCE '
                                                                                                'OF'},
                                                                                       {'element': {'type': 'SoundingRS-AperiodicSet-r14'},
                                                                                        'name': 'soundingRS-UL-AperiodicConfigDedicatedList-r14',
                                                                                        'optional': True,
                                                                                        'size': [(1,
                                                                                                  2)],
                                                                                        'type': 'SEQUENCE '
                                                                                                'OF'},
                                                                                       {'element': {'type': 'SoundingRS-AperiodicSetUpPTsExt-r14'},
                                                                                        'name': 'soundingRS-UL-ConfigDedicatedApUpPTsExtList-r14',
                                                                                        'optional': True,
                                                                                        'size': [(1,
                                                                                                  4)],
                                                                                        'type': 'SEQUENCE '
                                                                                                'OF'},
                                                                                       {'members': [{'name': 'release',
                                                                                                     'type': 'NULL'},
                                                                                                    {'members': [{'name': 'k-max-r14',
                                                                                                                  'type': 'ENUMERATED',
                                                                                                                  'values': [('l1',
                                                                                                                              0),
                                                                                                                             ('l3',
                                                                                                                              1)]},
                                                                                                                 {'name': 'p-a-must-r14',
                                                                                                                  'optional': True,
                                                                                                                  'type': 'ENUMERATED',
                                                                                                                  'values': [('dB-6',
                                                                                                                              0),
                                                                                                                             ('dB-4dot77',
                                                                                                                              1),
                                                                                                                             ('dB-3',
                                                                                                                              2),
                                                                                                                             ('dB-1dot77',
                                                                                                                              3),
                                                                                                                             ('dB0',
                                                                                                                              4),
                                                                                                                             ('dB1',
                                                                                                                              5),
                                                                                                                             ('dB2',
                                                                                                                              6),
                                                                                                                             ('dB3',
                                                                                                                              7)]}],
                                                                                                     'name': 'setup',
                                                                                                     'type': 'SEQUENCE'}],
                                                                                        'name': 'must-Config-r14',
                                                                                        'optional': True,
                                                                                        'type': 'CHOICE'},
                                                                                       {'name': 'pusch-ConfigDedicated-v1430',
                                                                                        'optional': True,
                                                                                        'type': 'PUSCH-ConfigDedicatedSCell-v1430'},
                                                                                       {'name': 'csi-RS-Config-v1430',
                                                                                        'optional': True,
                                                                                        'type': 'CSI-RS-Config-v1430'},
                                                                                       {'name': 'csi-RS-ConfigZP-ApList-r14',
                                                                                        'optional': True,
                                                                                        'type': 'CSI-RS-ConfigZP-ApList-r14'},
                                                                                       {'name': 'cqi-ReportConfig-v1430',
                                                                                        'optional': True,
                                                                                        'type': 'CQI-ReportConfig-v1430'},
                                                                                       {'name': 'semiOpenLoop-r14',
                                                                                        'optional': True,
                                                                                        'type': 'BOOLEAN'},
                                                                                       {'name': 'pdsch-ConfigDedicatedSCell-v1430',
                                                                                        'optional': True,
                                                                                        'type': 'PDSCH-ConfigDedicatedSCell-v1430'}]],
                                                                          'type': 'SEQUENCE'},
                                     'PhysicalConfigDedicatedSCell-v1370': {'members': [{'members': [{'name': 'release',
                                                                                                      'type': 'NULL'},
                                                                                                     {'members': [{'name': 'pucch-ConfigDedicated-v1370',
                                                                                                                   'optional': True,
                                                                                                                   'type': 'PUCCH-ConfigDedicated-v1370'}],
                                                                                                      'name': 'setup',
                                                                                                      'type': 'SEQUENCE'}],
                                                                                         'name': 'pucch-SCell-v1370',
                                                                                         'type': 'CHOICE'}],
                                                                            'type': 'SEQUENCE'},
                                     'PollByte': {'type': 'ENUMERATED',
                                                  'values': [('kB25', 0),
                                                             ('kB50', 1),
                                                             ('kB75', 2),
                                                             ('kB100', 3),
                                                             ('kB125', 4),
                                                             ('kB250', 5),
                                                             ('kB375', 6),
                                                             ('kB500', 7),
                                                             ('kB750', 8),
                                                             ('kB1000', 9),
                                                             ('kB1250', 10),
                                                             ('kB1500', 11),
                                                             ('kB2000', 12),
                                                             ('kB3000', 13),
                                                             ('kBinfinity', 14),
                                                             ('spare1', 15)]},
                                     'PollByte-r14': {'type': 'ENUMERATED',
                                                      'values': [('kB1', 0),
                                                                 ('kB2', 1),
                                                                 ('kB5', 2),
                                                                 ('kB8', 3),
                                                                 ('kB10', 4),
                                                                 ('kB15', 5),
                                                                 ('kB3500', 6),
                                                                 ('kB4000', 7),
                                                                 ('kB4500', 8),
                                                                 ('kB5000', 9),
                                                                 ('kB5500', 10),
                                                                 ('kB6000', 11),
                                                                 ('kB6500', 12),
                                                                 ('kB7000', 13),
                                                                 ('kB7500', 14),
                                                                 ('kB8000', 15),
                                                                 ('kB9000', 16),
                                                                 ('kB10000',
                                                                  17),
                                                                 ('kB11000',
                                                                  18),
                                                                 ('kB12000',
                                                                  19),
                                                                 ('kB13000',
                                                                  20),
                                                                 ('kB14000',
                                                                  21),
                                                                 ('kB15000',
                                                                  22),
                                                                 ('kB16000',
                                                                  23),
                                                                 ('kB17000',
                                                                  24),
                                                                 ('kB18000',
                                                                  25),
                                                                 ('kB19000',
                                                                  26),
                                                                 ('kB20000',
                                                                  27),
                                                                 ('kB25000',
                                                                  28),
                                                                 ('kB30000',
                                                                  29),
                                                                 ('kB35000',
                                                                  30),
                                                                 ('kB40000',
                                                                  31)]},
                                     'PollPDU': {'type': 'ENUMERATED',
                                                 'values': [('p4', 0),
                                                            ('p8', 1),
                                                            ('p16', 2),
                                                            ('p32', 3),
                                                            ('p64', 4),
                                                            ('p128', 5),
                                                            ('p256', 6),
                                                            ('pInfinity', 7)]},
                                     'PollPDU-v1310': {'type': 'ENUMERATED',
                                                       'values': [('p512', 0),
                                                                  ('p1024', 1),
                                                                  ('p2048', 2),
                                                                  ('p4096', 3),
                                                                  ('p6144', 4),
                                                                  ('p8192', 5),
                                                                  ('p12288', 6),
                                                                  ('p16384',
                                                                   7)]},
                                     'PowerCoordinationInfo-r12': {'members': [{'name': 'p-MeNB-r12',
                                                                                'restricted-to': [(1,
                                                                                                   16)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'p-SeNB-r12',
                                                                                'restricted-to': [(1,
                                                                                                   16)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'powerControlMode-r12',
                                                                                'restricted-to': [(1,
                                                                                                   2)],
                                                                                'type': 'INTEGER'}],
                                                                   'type': 'SEQUENCE'},
                                     'PowerPrefIndicationConfig-r11': {'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'members': [{'name': 'powerPrefIndicationTimer-r11',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('s0',
                                                                                                             0),
                                                                                                            ('s0dot5',
                                                                                                             1),
                                                                                                            ('s1',
                                                                                                             2),
                                                                                                            ('s2',
                                                                                                             3),
                                                                                                            ('s5',
                                                                                                             4),
                                                                                                            ('s10',
                                                                                                             5),
                                                                                                            ('s20',
                                                                                                             6),
                                                                                                            ('s30',
                                                                                                             7),
                                                                                                            ('s60',
                                                                                                             8),
                                                                                                            ('s90',
                                                                                                             9),
                                                                                                            ('s120',
                                                                                                             10),
                                                                                                            ('s300',
                                                                                                             11),
                                                                                                            ('s600',
                                                                                                             12),
                                                                                                            ('spare3',
                                                                                                             13),
                                                                                                            ('spare2',
                                                                                                             14),
                                                                                                            ('spare1',
                                                                                                             15)]}],
                                                                                    'name': 'setup',
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'CHOICE'},
                                     'PowerRampingParameters': {'members': [{'name': 'powerRampingStep',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('dB0',
                                                                                         0),
                                                                                        ('dB2',
                                                                                         1),
                                                                                        ('dB4',
                                                                                         2),
                                                                                        ('dB6',
                                                                                         3)]},
                                                                            {'name': 'preambleInitialReceivedTargetPower',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('dBm-120',
                                                                                         0),
                                                                                        ('dBm-118',
                                                                                         1),
                                                                                        ('dBm-116',
                                                                                         2),
                                                                                        ('dBm-114',
                                                                                         3),
                                                                                        ('dBm-112',
                                                                                         4),
                                                                                        ('dBm-110',
                                                                                         5),
                                                                                        ('dBm-108',
                                                                                         6),
                                                                                        ('dBm-106',
                                                                                         7),
                                                                                        ('dBm-104',
                                                                                         8),
                                                                                        ('dBm-102',
                                                                                         9),
                                                                                        ('dBm-100',
                                                                                         10),
                                                                                        ('dBm-98',
                                                                                         11),
                                                                                        ('dBm-96',
                                                                                         12),
                                                                                        ('dBm-94',
                                                                                         13),
                                                                                        ('dBm-92',
                                                                                         14),
                                                                                        ('dBm-90',
                                                                                         15)]}],
                                                                'type': 'SEQUENCE'},
                                     'PreRegistrationInfoHRPD': {'members': [{'name': 'preRegistrationAllowed',
                                                                              'type': 'BOOLEAN'},
                                                                             {'name': 'preRegistrationZoneId',
                                                                              'optional': True,
                                                                              'type': 'PreRegistrationZoneIdHRPD'},
                                                                             {'name': 'secondaryPreRegistrationZoneIdList',
                                                                              'optional': True,
                                                                              'type': 'SecondaryPreRegistrationZoneIdListHRPD'}],
                                                                 'type': 'SEQUENCE'},
                                     'PreRegistrationZoneIdHRPD': {'restricted-to': [(0,
                                                                                      255)],
                                                                   'type': 'INTEGER'},
                                     'PreambleTransMax': {'type': 'ENUMERATED',
                                                          'values': [('n3', 0),
                                                                     ('n4', 1),
                                                                     ('n5', 2),
                                                                     ('n6', 3),
                                                                     ('n7', 4),
                                                                     ('n8', 5),
                                                                     ('n10', 6),
                                                                     ('n20', 7),
                                                                     ('n50', 8),
                                                                     ('n100',
                                                                      9),
                                                                     ('n200',
                                                                      10)]},
                                     'PresenceAntennaPort1': {'type': 'BOOLEAN'},
                                     'ProximityIndication-r9': {'members': [{'members': [{'members': [{'name': 'proximityIndication-r9',
                                                                                                       'type': 'ProximityIndication-r9-IEs'},
                                                                                                      {'name': 'spare3',
                                                                                                       'type': 'NULL'},
                                                                                                      {'name': 'spare2',
                                                                                                       'type': 'NULL'},
                                                                                                      {'name': 'spare1',
                                                                                                       'type': 'NULL'}],
                                                                                          'name': 'c1',
                                                                                          'type': 'CHOICE'},
                                                                                         {'members': [],
                                                                                          'name': 'criticalExtensionsFuture',
                                                                                          'type': 'SEQUENCE'}],
                                                                             'name': 'criticalExtensions',
                                                                             'type': 'CHOICE'}],
                                                                'type': 'SEQUENCE'},
                                     'ProximityIndication-r9-IEs': {'members': [{'name': 'type-r9',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('entering',
                                                                                             0),
                                                                                            ('leaving',
                                                                                             1)]},
                                                                                {'members': [{'name': 'eutra-r9',
                                                                                              'type': 'ARFCN-ValueEUTRA'},
                                                                                             {'name': 'utra-r9',
                                                                                              'type': 'ARFCN-ValueUTRA'},
                                                                                             None,
                                                                                             {'name': 'eutra2-v9e0',
                                                                                              'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                                                 'name': 'carrierFreq-r9',
                                                                                 'type': 'CHOICE'},
                                                                                {'name': 'nonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'ProximityIndication-v930-IEs'}],
                                                                    'type': 'SEQUENCE'},
                                     'ProximityIndication-v930-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'members': [],
                                                                                   'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'SEQUENCE'}],
                                                                      'type': 'SEQUENCE'},
                                     'Q-OffsetRange': {'type': 'ENUMERATED',
                                                       'values': [('dB-24', 0),
                                                                  ('dB-22', 1),
                                                                  ('dB-20', 2),
                                                                  ('dB-18', 3),
                                                                  ('dB-16', 4),
                                                                  ('dB-14', 5),
                                                                  ('dB-12', 6),
                                                                  ('dB-10', 7),
                                                                  ('dB-8', 8),
                                                                  ('dB-6', 9),
                                                                  ('dB-5', 10),
                                                                  ('dB-4', 11),
                                                                  ('dB-3', 12),
                                                                  ('dB-2', 13),
                                                                  ('dB-1', 14),
                                                                  ('dB0', 15),
                                                                  ('dB1', 16),
                                                                  ('dB2', 17),
                                                                  ('dB3', 18),
                                                                  ('dB4', 19),
                                                                  ('dB5', 20),
                                                                  ('dB6', 21),
                                                                  ('dB8', 22),
                                                                  ('dB10', 23),
                                                                  ('dB12', 24),
                                                                  ('dB14', 25),
                                                                  ('dB16', 26),
                                                                  ('dB18', 27),
                                                                  ('dB20', 28),
                                                                  ('dB22', 29),
                                                                  ('dB24',
                                                                   30)]},
                                     'Q-OffsetRangeInterRAT': {'restricted-to': [(-15,
                                                                                  15)],
                                                               'type': 'INTEGER'},
                                     'Q-QualMin-r9': {'restricted-to': [(-34,
                                                                         -3)],
                                                      'type': 'INTEGER'},
                                     'Q-RxLevMin': {'restricted-to': [(-70,
                                                                       -22)],
                                                    'type': 'INTEGER'},
                                     'QuantityConfig': {'members': [{'name': 'quantityConfigEUTRA',
                                                                     'optional': True,
                                                                     'type': 'QuantityConfigEUTRA'},
                                                                    {'name': 'quantityConfigUTRA',
                                                                     'optional': True,
                                                                     'type': 'QuantityConfigUTRA'},
                                                                    {'name': 'quantityConfigGERAN',
                                                                     'optional': True,
                                                                     'type': 'QuantityConfigGERAN'},
                                                                    {'name': 'quantityConfigCDMA2000',
                                                                     'optional': True,
                                                                     'type': 'QuantityConfigCDMA2000'},
                                                                    None,
                                                                    [{'name': 'quantityConfigUTRA-v1020',
                                                                      'optional': True,
                                                                      'type': 'QuantityConfigUTRA-v1020'}],
                                                                    [{'name': 'quantityConfigEUTRA-v1250',
                                                                      'optional': True,
                                                                      'type': 'QuantityConfigEUTRA-v1250'}],
                                                                    [{'name': 'quantityConfigEUTRA-v1310',
                                                                      'optional': True,
                                                                      'type': 'QuantityConfigEUTRA-v1310'},
                                                                     {'name': 'quantityConfigWLAN-r13',
                                                                      'optional': True,
                                                                      'type': 'QuantityConfigWLAN-r13'}]],
                                                        'type': 'SEQUENCE'},
                                     'QuantityConfigCDMA2000': {'members': [{'name': 'measQuantityCDMA2000',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('pilotStrength',
                                                                                         0),
                                                                                        ('pilotPnPhaseAndPilotStrength',
                                                                                         1)]}],
                                                                'type': 'SEQUENCE'},
                                     'QuantityConfigEUTRA': {'members': [{'default': 'fc4',
                                                                          'name': 'filterCoefficientRSRP',
                                                                          'type': 'FilterCoefficient'},
                                                                         {'default': 'fc4',
                                                                          'name': 'filterCoefficientRSRQ',
                                                                          'type': 'FilterCoefficient'}],
                                                             'type': 'SEQUENCE'},
                                     'QuantityConfigEUTRA-v1250': {'members': [{'name': 'filterCoefficientCSI-RSRP-r12',
                                                                                'optional': True,
                                                                                'type': 'FilterCoefficient'}],
                                                                   'type': 'SEQUENCE'},
                                     'QuantityConfigEUTRA-v1310': {'members': [{'default': 'fc4',
                                                                                'name': 'filterCoefficientRS-SINR-r13',
                                                                                'type': 'FilterCoefficient'}],
                                                                   'type': 'SEQUENCE'},
                                     'QuantityConfigGERAN': {'members': [{'name': 'measQuantityGERAN',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('rssi',
                                                                                      0)]},
                                                                         {'default': 'fc2',
                                                                          'name': 'filterCoefficient',
                                                                          'type': 'FilterCoefficient'}],
                                                             'type': 'SEQUENCE'},
                                     'QuantityConfigUTRA': {'members': [{'name': 'measQuantityUTRA-FDD',
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('cpich-RSCP',
                                                                                     0),
                                                                                    ('cpich-EcN0',
                                                                                     1)]},
                                                                        {'name': 'measQuantityUTRA-TDD',
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('pccpch-RSCP',
                                                                                     0)]},
                                                                        {'default': 'fc4',
                                                                         'name': 'filterCoefficient',
                                                                         'type': 'FilterCoefficient'}],
                                                            'type': 'SEQUENCE'},
                                     'QuantityConfigUTRA-v1020': {'members': [{'default': 'fc4',
                                                                               'name': 'filterCoefficient2-FDD-r10',
                                                                               'type': 'FilterCoefficient'}],
                                                                  'type': 'SEQUENCE'},
                                     'QuantityConfigWLAN-r13': {'members': [{'name': 'measQuantityWLAN-r13',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('rssiWLAN',
                                                                                         0)]},
                                                                            {'default': 'fc4',
                                                                             'name': 'filterCoefficient-r13',
                                                                             'type': 'FilterCoefficient'}],
                                                                'type': 'SEQUENCE'},
                                     'RACH-CE-LevelInfo-r13': {'members': [{'members': [{'name': 'firstPreamble-r13',
                                                                                         'restricted-to': [(0,
                                                                                                            63)],
                                                                                         'type': 'INTEGER'},
                                                                                        {'name': 'lastPreamble-r13',
                                                                                         'restricted-to': [(0,
                                                                                                            63)],
                                                                                         'type': 'INTEGER'}],
                                                                            'name': 'preambleMappingInfo-r13',
                                                                            'type': 'SEQUENCE'},
                                                                           {'name': 'ra-ResponseWindowSize-r13',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('sf20',
                                                                                        0),
                                                                                       ('sf50',
                                                                                        1),
                                                                                       ('sf80',
                                                                                        2),
                                                                                       ('sf120',
                                                                                        3),
                                                                                       ('sf180',
                                                                                        4),
                                                                                       ('sf240',
                                                                                        5),
                                                                                       ('sf320',
                                                                                        6),
                                                                                       ('sf400',
                                                                                        7)]},
                                                                           {'name': 'mac-ContentionResolutionTimer-r13',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('sf80',
                                                                                        0),
                                                                                       ('sf100',
                                                                                        1),
                                                                                       ('sf120',
                                                                                        2),
                                                                                       ('sf160',
                                                                                        3),
                                                                                       ('sf200',
                                                                                        4),
                                                                                       ('sf240',
                                                                                        5),
                                                                                       ('sf480',
                                                                                        6),
                                                                                       ('sf960',
                                                                                        7)]},
                                                                           {'name': 'rar-HoppingConfig-r13',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('on',
                                                                                        0),
                                                                                       ('off',
                                                                                        1)]},
                                                                           None],
                                                               'type': 'SEQUENCE'},
                                     'RACH-CE-LevelInfoList-r13': {'element': {'type': 'RACH-CE-LevelInfo-r13'},
                                                                   'size': [(1,
                                                                             'maxCE-Level-r13')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'RACH-ConfigCommon': {'members': [{'members': [{'name': 'numberOfRA-Preambles',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('n4',
                                                                                                 0),
                                                                                                ('n8',
                                                                                                 1),
                                                                                                ('n12',
                                                                                                 2),
                                                                                                ('n16',
                                                                                                 3),
                                                                                                ('n20',
                                                                                                 4),
                                                                                                ('n24',
                                                                                                 5),
                                                                                                ('n28',
                                                                                                 6),
                                                                                                ('n32',
                                                                                                 7),
                                                                                                ('n36',
                                                                                                 8),
                                                                                                ('n40',
                                                                                                 9),
                                                                                                ('n44',
                                                                                                 10),
                                                                                                ('n48',
                                                                                                 11),
                                                                                                ('n52',
                                                                                                 12),
                                                                                                ('n56',
                                                                                                 13),
                                                                                                ('n60',
                                                                                                 14),
                                                                                                ('n64',
                                                                                                 15)]},
                                                                                    {'members': [{'name': 'sizeOfRA-PreamblesGroupA',
                                                                                                  'type': 'ENUMERATED',
                                                                                                  'values': [('n4',
                                                                                                              0),
                                                                                                             ('n8',
                                                                                                              1),
                                                                                                             ('n12',
                                                                                                              2),
                                                                                                             ('n16',
                                                                                                              3),
                                                                                                             ('n20',
                                                                                                              4),
                                                                                                             ('n24',
                                                                                                              5),
                                                                                                             ('n28',
                                                                                                              6),
                                                                                                             ('n32',
                                                                                                              7),
                                                                                                             ('n36',
                                                                                                              8),
                                                                                                             ('n40',
                                                                                                              9),
                                                                                                             ('n44',
                                                                                                              10),
                                                                                                             ('n48',
                                                                                                              11),
                                                                                                             ('n52',
                                                                                                              12),
                                                                                                             ('n56',
                                                                                                              13),
                                                                                                             ('n60',
                                                                                                              14)]},
                                                                                                 {'name': 'messageSizeGroupA',
                                                                                                  'type': 'ENUMERATED',
                                                                                                  'values': [('b56',
                                                                                                              0),
                                                                                                             ('b144',
                                                                                                              1),
                                                                                                             ('b208',
                                                                                                              2),
                                                                                                             ('b256',
                                                                                                              3)]},
                                                                                                 {'name': 'messagePowerOffsetGroupB',
                                                                                                  'type': 'ENUMERATED',
                                                                                                  'values': [('minusinfinity',
                                                                                                              0),
                                                                                                             ('dB0',
                                                                                                              1),
                                                                                                             ('dB5',
                                                                                                              2),
                                                                                                             ('dB8',
                                                                                                              3),
                                                                                                             ('dB10',
                                                                                                              4),
                                                                                                             ('dB12',
                                                                                                              5),
                                                                                                             ('dB15',
                                                                                                              6),
                                                                                                             ('dB18',
                                                                                                              7)]},
                                                                                                 None],
                                                                                     'name': 'preamblesGroupAConfig',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'}],
                                                                        'name': 'preambleInfo',
                                                                        'type': 'SEQUENCE'},
                                                                       {'name': 'powerRampingParameters',
                                                                        'type': 'PowerRampingParameters'},
                                                                       {'members': [{'name': 'preambleTransMax',
                                                                                     'type': 'PreambleTransMax'},
                                                                                    {'name': 'ra-ResponseWindowSize',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('sf2',
                                                                                                 0),
                                                                                                ('sf3',
                                                                                                 1),
                                                                                                ('sf4',
                                                                                                 2),
                                                                                                ('sf5',
                                                                                                 3),
                                                                                                ('sf6',
                                                                                                 4),
                                                                                                ('sf7',
                                                                                                 5),
                                                                                                ('sf8',
                                                                                                 6),
                                                                                                ('sf10',
                                                                                                 7)]},
                                                                                    {'name': 'mac-ContentionResolutionTimer',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('sf8',
                                                                                                 0),
                                                                                                ('sf16',
                                                                                                 1),
                                                                                                ('sf24',
                                                                                                 2),
                                                                                                ('sf32',
                                                                                                 3),
                                                                                                ('sf40',
                                                                                                 4),
                                                                                                ('sf48',
                                                                                                 5),
                                                                                                ('sf56',
                                                                                                 6),
                                                                                                ('sf64',
                                                                                                 7)]}],
                                                                        'name': 'ra-SupervisionInfo',
                                                                        'type': 'SEQUENCE'},
                                                                       {'name': 'maxHARQ-Msg3Tx',
                                                                        'restricted-to': [(1,
                                                                                           8)],
                                                                        'type': 'INTEGER'},
                                                                       None,
                                                                       [{'name': 'preambleTransMax-CE-r13',
                                                                         'optional': True,
                                                                         'type': 'PreambleTransMax'},
                                                                        {'name': 'rach-CE-LevelInfoList-r13',
                                                                         'optional': True,
                                                                         'type': 'RACH-CE-LevelInfoList-r13'}]],
                                                           'type': 'SEQUENCE'},
                                     'RACH-ConfigCommon-v1250': {'members': [{'members': [{'name': 'connEstFailCount-r12',
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('n1',
                                                                                                       0),
                                                                                                      ('n2',
                                                                                                       1),
                                                                                                      ('n3',
                                                                                                       2),
                                                                                                      ('n4',
                                                                                                       3)]},
                                                                                          {'name': 'connEstFailOffsetValidity-r12',
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('s30',
                                                                                                       0),
                                                                                                      ('s60',
                                                                                                       1),
                                                                                                      ('s120',
                                                                                                       2),
                                                                                                      ('s240',
                                                                                                       3),
                                                                                                      ('s300',
                                                                                                       4),
                                                                                                      ('s420',
                                                                                                       5),
                                                                                                      ('s600',
                                                                                                       6),
                                                                                                      ('s900',
                                                                                                       7)]},
                                                                                          {'name': 'connEstFailOffset-r12',
                                                                                           'optional': True,
                                                                                           'restricted-to': [(0,
                                                                                                              15)],
                                                                                           'type': 'INTEGER'}],
                                                                              'name': 'txFailParams-r12',
                                                                              'type': 'SEQUENCE'}],
                                                                 'type': 'SEQUENCE'},
                                     'RACH-ConfigCommonSCell-r11': {'members': [{'name': 'powerRampingParameters-r11',
                                                                                 'type': 'PowerRampingParameters'},
                                                                                {'members': [{'name': 'preambleTransMax-r11',
                                                                                              'type': 'PreambleTransMax'}],
                                                                                 'name': 'ra-SupervisionInfo-r11',
                                                                                 'type': 'SEQUENCE'},
                                                                                None],
                                                                    'type': 'SEQUENCE'},
                                     'RACH-ConfigDedicated': {'members': [{'name': 'ra-PreambleIndex',
                                                                           'restricted-to': [(0,
                                                                                              63)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'ra-PRACH-MaskIndex',
                                                                           'restricted-to': [(0,
                                                                                              15)],
                                                                           'type': 'INTEGER'}],
                                                              'type': 'SEQUENCE'},
                                     'RACH-Skip-r14': {'members': [{'members': [{'name': 'ta0-r14',
                                                                                 'type': 'NULL'},
                                                                                {'name': 'mcg-PTAG-r14',
                                                                                 'type': 'NULL'},
                                                                                {'name': 'scg-PTAG-r14',
                                                                                 'type': 'NULL'},
                                                                                {'name': 'mcg-STAG-r14',
                                                                                 'type': 'STAG-Id-r11'},
                                                                                {'name': 'scg-STAG-r14',
                                                                                 'type': 'STAG-Id-r11'}],
                                                                    'name': 'targetTA-r14',
                                                                    'type': 'CHOICE'},
                                                                   {'members': [{'name': 'numberOfConfUL-Processes-r14',
                                                                                 'restricted-to': [(1,
                                                                                                    8)],
                                                                                 'type': 'INTEGER'},
                                                                                {'name': 'ul-SchedInterval-r14',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('sf2',
                                                                                             0),
                                                                                            ('sf5',
                                                                                             1),
                                                                                            ('sf10',
                                                                                             2)]},
                                                                                {'name': 'ul-StartSubframe-r14',
                                                                                 'restricted-to': [(0,
                                                                                                    9)],
                                                                                 'type': 'INTEGER'},
                                                                                {'name': 'ul-Grant-r14',
                                                                                 'size': [16],
                                                                                 'type': 'BIT '
                                                                                         'STRING'}],
                                                                    'name': 'ul-ConfigInfo-r14',
                                                                    'optional': True,
                                                                    'type': 'SEQUENCE'}],
                                                       'type': 'SEQUENCE'},
                                     'RAND-CDMA2000': {'size': [32],
                                                       'type': 'BIT STRING'},
                                     'RAT-Type': {'type': 'ENUMERATED',
                                                  'values': [('eutra', 0),
                                                             ('utra', 1),
                                                             ('geran-cs', 2),
                                                             ('geran-ps', 3),
                                                             ('cdma2000-1XRTT',
                                                              4),
                                                             ('spare3', 5),
                                                             ('spare2', 6),
                                                             ('spare1', 7),
                                                             None]},
                                     'RCLWI-Config-r13': {'members': [{'members': [{'members': [{'name': 'mobilityConfig-r13',
                                                                                                 'type': 'WLAN-Id-List-r12'}],
                                                                                    'name': 'steerToWLAN-r13',
                                                                                    'type': 'SEQUENCE'},
                                                                                   {'name': 'steerToLTE-r13',
                                                                                    'type': 'NULL'}],
                                                                       'name': 'command',
                                                                       'type': 'CHOICE'},
                                                                      None],
                                                          'type': 'SEQUENCE'},
                                     'RCLWI-Configuration-r13': {'members': [{'name': 'release',
                                                                              'type': 'NULL'},
                                                                             {'members': [{'name': 'rclwi-Config-r13',
                                                                                           'type': 'RCLWI-Config-r13'}],
                                                                              'name': 'setup',
                                                                              'type': 'SEQUENCE'}],
                                                                 'type': 'CHOICE'},
                                     'RE-MappingQCLConfigToAddModList-r11': {'element': {'type': 'PDSCH-RE-MappingQCL-Config-r11'},
                                                                             'size': [(1,
                                                                                       'maxRE-MapQCL-r11')],
                                                                             'type': 'SEQUENCE '
                                                                                     'OF'},
                                     'RE-MappingQCLConfigToReleaseList-r11': {'element': {'type': 'PDSCH-RE-MappingQCL-ConfigId-r11'},
                                                                              'size': [(1,
                                                                                        'maxRE-MapQCL-r11')],
                                                                              'type': 'SEQUENCE '
                                                                                      'OF'},
                                     'RF-Parameters': {'members': [{'name': 'supportedBandListEUTRA',
                                                                    'type': 'SupportedBandListEUTRA'}],
                                                       'type': 'SEQUENCE'},
                                     'RF-Parameters-v1020': {'members': [{'name': 'supportedBandCombination-r10',
                                                                          'type': 'SupportedBandCombination-r10'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v1060': {'members': [{'name': 'supportedBandCombinationExt-r10',
                                                                          'type': 'SupportedBandCombinationExt-r10'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v1090': {'members': [{'name': 'supportedBandCombination-v1090',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombination-v1090'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v10f0': {'members': [{'name': 'modifiedMPR-Behavior-r10',
                                                                          'optional': True,
                                                                          'size': [32],
                                                                          'type': 'BIT '
                                                                                  'STRING'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v10i0': {'members': [{'name': 'supportedBandCombination-v10i0',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombination-v10i0'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v10j0': {'members': [{'name': 'multiNS-Pmax-r10',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v1130': {'members': [{'name': 'supportedBandCombination-v1130',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombination-v1130'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v1180': {'members': [{'name': 'freqBandRetrieval-r11',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'element': {'type': 'FreqBandIndicator-r11'},
                                                                          'name': 'requestedBands-r11',
                                                                          'optional': True,
                                                                          'size': [(1,
                                                                                    'maxBands')],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                                                         {'name': 'supportedBandCombinationAdd-r11',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombinationAdd-r11'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v11d0': {'members': [{'name': 'supportedBandCombinationAdd-v11d0',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombinationAdd-v11d0'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v1250': {'members': [{'name': 'supportedBandListEUTRA-v1250',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandListEUTRA-v1250'},
                                                                         {'name': 'supportedBandCombination-v1250',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombination-v1250'},
                                                                         {'name': 'supportedBandCombinationAdd-v1250',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombinationAdd-v1250'},
                                                                         {'name': 'freqBandPriorityAdjustment-r12',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v1270': {'members': [{'name': 'supportedBandCombination-v1270',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombination-v1270'},
                                                                         {'name': 'supportedBandCombinationAdd-v1270',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombinationAdd-v1270'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v12b0': {'members': [{'name': 'maxLayersMIMO-Indication-r12',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v1310': {'members': [{'members': [{'name': 'reducedIntNonContCombRequested-r13',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('true',
                                                                                                   0)]},
                                                                                      {'name': 'requestedCCsDL-r13',
                                                                                       'optional': True,
                                                                                       'restricted-to': [(2,
                                                                                                          32)],
                                                                                       'type': 'INTEGER'},
                                                                                      {'name': 'requestedCCsUL-r13',
                                                                                       'optional': True,
                                                                                       'restricted-to': [(2,
                                                                                                          32)],
                                                                                       'type': 'INTEGER'},
                                                                                      {'name': 'skipFallbackCombRequested-r13',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('true',
                                                                                                   0)]}],
                                                                          'name': 'eNB-RequestedParameters-r13',
                                                                          'optional': True,
                                                                          'type': 'SEQUENCE'},
                                                                         {'name': 'maximumCCsRetrieval-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'skipFallbackCombinations-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'reducedIntNonContComb-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'supportedBandListEUTRA-v1310',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandListEUTRA-v1310'},
                                                                         {'name': 'supportedBandCombinationReduced-r13',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombinationReduced-r13'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v1320': {'members': [{'name': 'supportedBandListEUTRA-v1320',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandListEUTRA-v1320'},
                                                                         {'name': 'supportedBandCombination-v1320',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombination-v1320'},
                                                                         {'name': 'supportedBandCombinationAdd-v1320',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombinationAdd-v1320'},
                                                                         {'name': 'supportedBandCombinationReduced-v1320',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombinationReduced-v1320'}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v1430': {'members': [{'name': 'supportedBandCombination-v1430',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombination-v1430'},
                                                                         {'name': 'supportedBandCombinationAdd-v1430',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombinationAdd-v1430'},
                                                                         {'name': 'supportedBandCombinationReduced-v1430',
                                                                          'optional': True,
                                                                          'type': 'SupportedBandCombinationReduced-v1430'},
                                                                         {'members': [{'name': 'requestedDiffFallbackCombList-r14',
                                                                                       'type': 'BandCombinationList-r14'}],
                                                                          'name': 'eNB-RequestedParameters-v1430',
                                                                          'optional': True,
                                                                          'type': 'SEQUENCE'},
                                                                         {'name': 'diffFallbackCombReport-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'RF-Parameters-v9e0': {'members': [{'name': 'supportedBandListEUTRA-v9e0',
                                                                         'optional': True,
                                                                         'type': 'SupportedBandListEUTRA-v9e0'}],
                                                            'type': 'SEQUENCE'},
                                     'RLC-Config': {'members': [{'members': [{'name': 'ul-AM-RLC',
                                                                              'type': 'UL-AM-RLC'},
                                                                             {'name': 'dl-AM-RLC',
                                                                              'type': 'DL-AM-RLC'}],
                                                                 'name': 'am',
                                                                 'type': 'SEQUENCE'},
                                                                {'members': [{'name': 'ul-UM-RLC',
                                                                              'type': 'UL-UM-RLC'},
                                                                             {'name': 'dl-UM-RLC',
                                                                              'type': 'DL-UM-RLC'}],
                                                                 'name': 'um-Bi-Directional',
                                                                 'type': 'SEQUENCE'},
                                                                {'members': [{'name': 'ul-UM-RLC',
                                                                              'type': 'UL-UM-RLC'}],
                                                                 'name': 'um-Uni-Directional-UL',
                                                                 'type': 'SEQUENCE'},
                                                                {'members': [{'name': 'dl-UM-RLC',
                                                                              'type': 'DL-UM-RLC'}],
                                                                 'name': 'um-Uni-Directional-DL',
                                                                 'type': 'SEQUENCE'},
                                                                None],
                                                    'type': 'CHOICE'},
                                     'RLC-Config-v1250': {'members': [{'name': 'ul-extended-RLC-LI-Field-r12',
                                                                       'type': 'BOOLEAN'},
                                                                      {'name': 'dl-extended-RLC-LI-Field-r12',
                                                                       'type': 'BOOLEAN'}],
                                                          'type': 'SEQUENCE'},
                                     'RLC-Config-v1310': {'members': [{'name': 'ul-extended-RLC-AM-SN-r13',
                                                                       'type': 'BOOLEAN'},
                                                                      {'name': 'dl-extended-RLC-AM-SN-r13',
                                                                       'type': 'BOOLEAN'},
                                                                      {'name': 'pollPDU-v1310',
                                                                       'optional': True,
                                                                       'type': 'PollPDU-v1310'}],
                                                          'type': 'SEQUENCE'},
                                     'RLC-Config-v1430': {'members': [{'name': 'release',
                                                                       'type': 'NULL'},
                                                                      {'members': [{'name': 'pollByte-r14',
                                                                                    'type': 'PollByte-r14'}],
                                                                       'name': 'setup',
                                                                       'type': 'SEQUENCE'}],
                                                          'type': 'CHOICE'},
                                     'RLC-Parameters-r12': {'members': [{'name': 'extended-RLC-LI-Field-r12',
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('supported',
                                                                                     0)]}],
                                                            'type': 'SEQUENCE'},
                                     'RLC-Parameters-v1310': {'members': [{'name': 'extendedRLC-SN-SO-Field-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'RLC-Parameters-v1430': {'members': [{'name': 'extendedPollByte-r14',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'RLF-Report-r9': {'members': [{'members': [{'name': 'rsrpResult-r9',
                                                                                 'type': 'RSRP-Range'},
                                                                                {'name': 'rsrqResult-r9',
                                                                                 'optional': True,
                                                                                 'type': 'RSRQ-Range'}],
                                                                    'name': 'measResultLastServCell-r9',
                                                                    'type': 'SEQUENCE'},
                                                                   {'members': [{'name': 'measResultListEUTRA-r9',
                                                                                 'optional': True,
                                                                                 'type': 'MeasResultList2EUTRA-r9'},
                                                                                {'name': 'measResultListUTRA-r9',
                                                                                 'optional': True,
                                                                                 'type': 'MeasResultList2UTRA-r9'},
                                                                                {'name': 'measResultListGERAN-r9',
                                                                                 'optional': True,
                                                                                 'type': 'MeasResultListGERAN'},
                                                                                {'name': 'measResultsCDMA2000-r9',
                                                                                 'optional': True,
                                                                                 'type': 'MeasResultList2CDMA2000-r9'}],
                                                                    'name': 'measResultNeighCells-r9',
                                                                    'optional': True,
                                                                    'type': 'SEQUENCE'},
                                                                   None,
                                                                   [{'name': 'locationInfo-r10',
                                                                     'optional': True,
                                                                     'type': 'LocationInfo-r10'},
                                                                    {'members': [{'name': 'cellGlobalId-r10',
                                                                                  'type': 'CellGlobalIdEUTRA'},
                                                                                 {'members': [{'name': 'physCellId-r10',
                                                                                               'type': 'PhysCellId'},
                                                                                              {'name': 'carrierFreq-r10',
                                                                                               'type': 'ARFCN-ValueEUTRA'}],
                                                                                  'name': 'pci-arfcn-r10',
                                                                                  'type': 'SEQUENCE'}],
                                                                     'name': 'failedPCellId-r10',
                                                                     'optional': True,
                                                                     'type': 'CHOICE'},
                                                                    {'name': 'reestablishmentCellId-r10',
                                                                     'optional': True,
                                                                     'type': 'CellGlobalIdEUTRA'},
                                                                    {'name': 'timeConnFailure-r10',
                                                                     'optional': True,
                                                                     'restricted-to': [(0,
                                                                                        1023)],
                                                                     'type': 'INTEGER'},
                                                                    {'name': 'connectionFailureType-r10',
                                                                     'optional': True,
                                                                     'type': 'ENUMERATED',
                                                                     'values': [('rlf',
                                                                                 0),
                                                                                ('hof',
                                                                                 1)]},
                                                                    {'name': 'previousPCellId-r10',
                                                                     'optional': True,
                                                                     'type': 'CellGlobalIdEUTRA'}],
                                                                   [{'members': [{'name': 'carrierFreq-v1090',
                                                                                  'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                                     'name': 'failedPCellId-v1090',
                                                                     'optional': True,
                                                                     'type': 'SEQUENCE'}],
                                                                   [{'members': [{'name': 'c-RNTI-r11',
                                                                                  'type': 'C-RNTI'},
                                                                                 {'name': 'rlf-Cause-r11',
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('t310-Expiry',
                                                                                              0),
                                                                                             ('randomAccessProblem',
                                                                                              1),
                                                                                             ('rlc-MaxNumRetx',
                                                                                              2),
                                                                                             ('t312-Expiry-r12',
                                                                                              3)]},
                                                                                 {'name': 'timeSinceFailure-r11',
                                                                                  'type': 'TimeSinceFailure-r11'}],
                                                                     'name': 'basicFields-r11',
                                                                     'optional': True,
                                                                     'type': 'SEQUENCE'},
                                                                    {'members': [{'name': 'carrierFreq-r11',
                                                                                  'type': 'ARFCN-ValueUTRA'},
                                                                                 {'members': [{'name': 'fdd-r11',
                                                                                               'type': 'PhysCellIdUTRA-FDD'},
                                                                                              {'name': 'tdd-r11',
                                                                                               'type': 'PhysCellIdUTRA-TDD'}],
                                                                                  'name': 'physCellId-r11',
                                                                                  'type': 'CHOICE'},
                                                                                 {'name': 'cellGlobalId-r11',
                                                                                  'optional': True,
                                                                                  'type': 'CellGlobalIdUTRA'}],
                                                                     'name': 'previousUTRA-CellId-r11',
                                                                     'optional': True,
                                                                     'type': 'SEQUENCE'},
                                                                    {'members': [{'name': 'carrierFreq-r11',
                                                                                  'type': 'ARFCN-ValueUTRA'},
                                                                                 {'members': [{'name': 'fdd-r11',
                                                                                               'type': 'PhysCellIdUTRA-FDD'},
                                                                                              {'name': 'tdd-r11',
                                                                                               'type': 'PhysCellIdUTRA-TDD'}],
                                                                                  'name': 'physCellId-r11',
                                                                                  'type': 'CHOICE'}],
                                                                     'name': 'selectedUTRA-CellId-r11',
                                                                     'optional': True,
                                                                     'type': 'SEQUENCE'}],
                                                                   [{'members': [{'name': 'tac-FailedPCell-r12',
                                                                                  'type': 'TrackingAreaCode'}],
                                                                     'name': 'failedPCellId-v1250',
                                                                     'optional': True,
                                                                     'type': 'SEQUENCE'},
                                                                    {'name': 'measResultLastServCell-v1250',
                                                                     'optional': True,
                                                                     'type': 'RSRQ-Range-v1250'},
                                                                    {'name': 'lastServCellRSRQ-Type-r12',
                                                                     'optional': True,
                                                                     'type': 'RSRQ-Type-r12'},
                                                                    {'name': 'measResultListEUTRA-v1250',
                                                                     'optional': True,
                                                                     'type': 'MeasResultList2EUTRA-v1250'}],
                                                                   [{'name': 'drb-EstablishedWithQCI-1-r13',
                                                                     'optional': True,
                                                                     'type': 'ENUMERATED',
                                                                     'values': [('qci1',
                                                                                 0)]}],
                                                                   [{'name': 'measResultLastServCell-v1360',
                                                                     'optional': True,
                                                                     'type': 'RSRP-Range-v1360'}]],
                                                       'type': 'SEQUENCE'},
                                     'RLF-Report-v9e0': {'members': [{'name': 'measResultListEUTRA-v9e0',
                                                                      'type': 'MeasResultList2EUTRA-v9e0'}],
                                                         'type': 'SEQUENCE'},
                                     'RLF-TimersAndConstants-r13': {'members': [{'name': 'release',
                                                                                 'type': 'NULL'},
                                                                                {'members': [{'name': 't301-v1310',
                                                                                              'type': 'ENUMERATED',
                                                                                              'values': [('ms2500',
                                                                                                          0),
                                                                                                         ('ms3000',
                                                                                                          1),
                                                                                                         ('ms3500',
                                                                                                          2),
                                                                                                         ('ms4000',
                                                                                                          3),
                                                                                                         ('ms5000',
                                                                                                          4),
                                                                                                         ('ms6000',
                                                                                                          5),
                                                                                                         ('ms8000',
                                                                                                          6),
                                                                                                         ('ms10000',
                                                                                                          7)]},
                                                                                             None,
                                                                                             [{'name': 't310-v1330',
                                                                                               'optional': True,
                                                                                               'type': 'ENUMERATED',
                                                                                               'values': [('ms4000',
                                                                                                           0),
                                                                                                          ('ms6000',
                                                                                                           1)]}]],
                                                                                 'name': 'setup',
                                                                                 'type': 'SEQUENCE'}],
                                                                    'type': 'CHOICE'},
                                     'RLF-TimersAndConstants-r9': {'members': [{'name': 'release',
                                                                                'type': 'NULL'},
                                                                               {'members': [{'name': 't301-r9',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('ms100',
                                                                                                         0),
                                                                                                        ('ms200',
                                                                                                         1),
                                                                                                        ('ms300',
                                                                                                         2),
                                                                                                        ('ms400',
                                                                                                         3),
                                                                                                        ('ms600',
                                                                                                         4),
                                                                                                        ('ms1000',
                                                                                                         5),
                                                                                                        ('ms1500',
                                                                                                         6),
                                                                                                        ('ms2000',
                                                                                                         7)]},
                                                                                            {'name': 't310-r9',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('ms0',
                                                                                                         0),
                                                                                                        ('ms50',
                                                                                                         1),
                                                                                                        ('ms100',
                                                                                                         2),
                                                                                                        ('ms200',
                                                                                                         3),
                                                                                                        ('ms500',
                                                                                                         4),
                                                                                                        ('ms1000',
                                                                                                         5),
                                                                                                        ('ms2000',
                                                                                                         6)]},
                                                                                            {'name': 'n310-r9',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('n1',
                                                                                                         0),
                                                                                                        ('n2',
                                                                                                         1),
                                                                                                        ('n3',
                                                                                                         2),
                                                                                                        ('n4',
                                                                                                         3),
                                                                                                        ('n6',
                                                                                                         4),
                                                                                                        ('n8',
                                                                                                         5),
                                                                                                        ('n10',
                                                                                                         6),
                                                                                                        ('n20',
                                                                                                         7)]},
                                                                                            {'name': 't311-r9',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('ms1000',
                                                                                                         0),
                                                                                                        ('ms3000',
                                                                                                         1),
                                                                                                        ('ms5000',
                                                                                                         2),
                                                                                                        ('ms10000',
                                                                                                         3),
                                                                                                        ('ms15000',
                                                                                                         4),
                                                                                                        ('ms20000',
                                                                                                         5),
                                                                                                        ('ms30000',
                                                                                                         6)]},
                                                                                            {'name': 'n311-r9',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('n1',
                                                                                                         0),
                                                                                                        ('n2',
                                                                                                         1),
                                                                                                        ('n3',
                                                                                                         2),
                                                                                                        ('n4',
                                                                                                         3),
                                                                                                        ('n5',
                                                                                                         4),
                                                                                                        ('n6',
                                                                                                         5),
                                                                                                        ('n8',
                                                                                                         6),
                                                                                                        ('n10',
                                                                                                         7)]},
                                                                                            None],
                                                                                'name': 'setup',
                                                                                'type': 'SEQUENCE'}],
                                                                   'type': 'CHOICE'},
                                     'RLF-TimersAndConstantsSCG-r12': {'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'members': [{'name': 't313-r12',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('ms0',
                                                                                                             0),
                                                                                                            ('ms50',
                                                                                                             1),
                                                                                                            ('ms100',
                                                                                                             2),
                                                                                                            ('ms200',
                                                                                                             3),
                                                                                                            ('ms500',
                                                                                                             4),
                                                                                                            ('ms1000',
                                                                                                             5),
                                                                                                            ('ms2000',
                                                                                                             6)]},
                                                                                                {'name': 'n313-r12',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('n1',
                                                                                                             0),
                                                                                                            ('n2',
                                                                                                             1),
                                                                                                            ('n3',
                                                                                                             2),
                                                                                                            ('n4',
                                                                                                             3),
                                                                                                            ('n6',
                                                                                                             4),
                                                                                                            ('n8',
                                                                                                             5),
                                                                                                            ('n10',
                                                                                                             6),
                                                                                                            ('n20',
                                                                                                             7)]},
                                                                                                {'name': 'n314-r12',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('n1',
                                                                                                             0),
                                                                                                            ('n2',
                                                                                                             1),
                                                                                                            ('n3',
                                                                                                             2),
                                                                                                            ('n4',
                                                                                                             3),
                                                                                                            ('n5',
                                                                                                             4),
                                                                                                            ('n6',
                                                                                                             5),
                                                                                                            ('n8',
                                                                                                             6),
                                                                                                            ('n10',
                                                                                                             7)]},
                                                                                                None],
                                                                                    'name': 'setup',
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'CHOICE'},
                                     'RMTC-Config-r13': {'members': [{'name': 'release',
                                                                      'type': 'NULL'},
                                                                     {'members': [{'name': 'rmtc-Period-r13',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('ms40',
                                                                                               0),
                                                                                              ('ms80',
                                                                                               1),
                                                                                              ('ms160',
                                                                                               2),
                                                                                              ('ms320',
                                                                                               3),
                                                                                              ('ms640',
                                                                                               4)]},
                                                                                  {'name': 'rmtc-SubframeOffset-r13',
                                                                                   'optional': True,
                                                                                   'restricted-to': [(0,
                                                                                                      639)],
                                                                                   'type': 'INTEGER'},
                                                                                  {'name': 'measDuration-r13',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('sym1',
                                                                                               0),
                                                                                              ('sym14',
                                                                                               1),
                                                                                              ('sym28',
                                                                                               2),
                                                                                              ('sym42',
                                                                                               3),
                                                                                              ('sym70',
                                                                                               4)]},
                                                                                  None],
                                                                      'name': 'setup',
                                                                      'type': 'SEQUENCE'}],
                                                         'type': 'CHOICE'},
                                     'RN-SubframeConfig-r10': {'members': [{'members': [{'name': 'subframeConfigPatternFDD-r10',
                                                                                         'size': [8],
                                                                                         'type': 'BIT '
                                                                                                 'STRING'},
                                                                                        {'name': 'subframeConfigPatternTDD-r10',
                                                                                         'restricted-to': [(0,
                                                                                                            31)],
                                                                                         'type': 'INTEGER'}],
                                                                            'name': 'subframeConfigPattern-r10',
                                                                            'optional': True,
                                                                            'type': 'CHOICE'},
                                                                           {'members': [{'name': 'resourceAllocationType-r10',
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('type0',
                                                                                                     0),
                                                                                                    ('type1',
                                                                                                     1),
                                                                                                    ('type2Localized',
                                                                                                     2),
                                                                                                    ('type2Distributed',
                                                                                                     3),
                                                                                                    ('spare4',
                                                                                                     4),
                                                                                                    ('spare3',
                                                                                                     5),
                                                                                                    ('spare2',
                                                                                                     6),
                                                                                                    ('spare1',
                                                                                                     7)]},
                                                                                        {'members': [{'members': [{'name': 'nrb6-r10',
                                                                                                                   'size': [6],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'},
                                                                                                                  {'name': 'nrb15-r10',
                                                                                                                   'size': [8],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'},
                                                                                                                  {'name': 'nrb25-r10',
                                                                                                                   'size': [13],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'},
                                                                                                                  {'name': 'nrb50-r10',
                                                                                                                   'size': [17],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'},
                                                                                                                  {'name': 'nrb75-r10',
                                                                                                                   'size': [19],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'},
                                                                                                                  {'name': 'nrb100-r10',
                                                                                                                   'size': [25],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'}],
                                                                                                      'name': 'type01-r10',
                                                                                                      'type': 'CHOICE'},
                                                                                                     {'members': [{'name': 'nrb6-r10',
                                                                                                                   'size': [5],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'},
                                                                                                                  {'name': 'nrb15-r10',
                                                                                                                   'size': [7],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'},
                                                                                                                  {'name': 'nrb25-r10',
                                                                                                                   'size': [9],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'},
                                                                                                                  {'name': 'nrb50-r10',
                                                                                                                   'size': [11],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'},
                                                                                                                  {'name': 'nrb75-r10',
                                                                                                                   'size': [12],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'},
                                                                                                                  {'name': 'nrb100-r10',
                                                                                                                   'size': [13],
                                                                                                                   'type': 'BIT '
                                                                                                                           'STRING'}],
                                                                                                      'name': 'type2-r10',
                                                                                                      'type': 'CHOICE'},
                                                                                                     None],
                                                                                         'name': 'resourceBlockAssignment-r10',
                                                                                         'type': 'CHOICE'},
                                                                                        {'members': [{'name': 'interleaving-r10',
                                                                                                      'type': 'ENUMERATED',
                                                                                                      'values': [('crs',
                                                                                                                  0)]},
                                                                                                     {'name': 'noInterleaving-r10',
                                                                                                      'type': 'ENUMERATED',
                                                                                                      'values': [('crs',
                                                                                                                  0),
                                                                                                                 ('dmrs',
                                                                                                                  1)]}],
                                                                                         'name': 'demodulationRS-r10',
                                                                                         'type': 'CHOICE'},
                                                                                        {'name': 'pdsch-Start-r10',
                                                                                         'restricted-to': [(1,
                                                                                                            3)],
                                                                                         'type': 'INTEGER'},
                                                                                        {'members': [{'members': [{'members': [{'element': {'restricted-to': [(0,
                                                                                                                                                               2047)],
                                                                                                                                            'type': 'INTEGER'},
                                                                                                                                'name': 'n1PUCCH-AN-List-r10',
                                                                                                                                'size': [(1,
                                                                                                                                          4)],
                                                                                                                                'type': 'SEQUENCE '
                                                                                                                                        'OF'}],
                                                                                                                   'name': 'channelSelectionMultiplexingBundling',
                                                                                                                   'type': 'SEQUENCE'},
                                                                                                                  {'members': [{'name': 'n1PUCCH-AN-P0-r10',
                                                                                                                                'restricted-to': [(0,
                                                                                                                                                   2047)],
                                                                                                                                'type': 'INTEGER'},
                                                                                                                               {'name': 'n1PUCCH-AN-P1-r10',
                                                                                                                                'optional': True,
                                                                                                                                'restricted-to': [(0,
                                                                                                                                                   2047)],
                                                                                                                                'type': 'INTEGER'}],
                                                                                                                   'name': 'fallbackForFormat3',
                                                                                                                   'type': 'SEQUENCE'}],
                                                                                                      'name': 'tdd',
                                                                                                      'type': 'CHOICE'},
                                                                                                     {'members': [{'name': 'n1PUCCH-AN-P0-r10',
                                                                                                                   'restricted-to': [(0,
                                                                                                                                      2047)],
                                                                                                                   'type': 'INTEGER'},
                                                                                                                  {'name': 'n1PUCCH-AN-P1-r10',
                                                                                                                   'optional': True,
                                                                                                                   'restricted-to': [(0,
                                                                                                                                      2047)],
                                                                                                                   'type': 'INTEGER'}],
                                                                                                      'name': 'fdd',
                                                                                                      'type': 'SEQUENCE'}],
                                                                                         'name': 'pucch-Config-r10',
                                                                                         'type': 'CHOICE'},
                                                                                        None],
                                                                            'name': 'rpdcch-Config-r10',
                                                                            'optional': True,
                                                                            'type': 'SEQUENCE'},
                                                                           None],
                                                               'type': 'SEQUENCE'},
                                     'RN-SystemInfo-r10': {'members': [{'name': 'systemInformationBlockType1-r10',
                                                                        'optional': True,
                                                                        'type': 'OCTET '
                                                                                'STRING'},
                                                                       {'name': 'systemInformationBlockType2-r10',
                                                                        'optional': True,
                                                                        'type': 'SystemInformationBlockType2'},
                                                                       None],
                                                           'type': 'SEQUENCE'},
                                     'RNReconfiguration-r10': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                            'type': 'RRC-TransactionIdentifier'},
                                                                           {'members': [{'members': [{'name': 'rnReconfiguration-r10',
                                                                                                      'type': 'RNReconfiguration-r10-IEs'},
                                                                                                     {'name': 'spare3',
                                                                                                      'type': 'NULL'},
                                                                                                     {'name': 'spare2',
                                                                                                      'type': 'NULL'},
                                                                                                     {'name': 'spare1',
                                                                                                      'type': 'NULL'}],
                                                                                         'name': 'c1',
                                                                                         'type': 'CHOICE'},
                                                                                        {'members': [],
                                                                                         'name': 'criticalExtensionsFuture',
                                                                                         'type': 'SEQUENCE'}],
                                                                            'name': 'criticalExtensions',
                                                                            'type': 'CHOICE'}],
                                                               'type': 'SEQUENCE'},
                                     'RNReconfiguration-r10-IEs': {'members': [{'name': 'rn-SystemInfo-r10',
                                                                                'optional': True,
                                                                                'type': 'RN-SystemInfo-r10'},
                                                                               {'name': 'rn-SubframeConfig-r10',
                                                                                'optional': True,
                                                                                'type': 'RN-SubframeConfig-r10'},
                                                                               {'name': 'lateNonCriticalExtension',
                                                                                'optional': True,
                                                                                'type': 'OCTET '
                                                                                        'STRING'},
                                                                               {'members': [],
                                                                                'name': 'nonCriticalExtension',
                                                                                'optional': True,
                                                                                'type': 'SEQUENCE'}],
                                                                   'type': 'SEQUENCE'},
                                     'RNReconfigurationComplete-r10': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                    'type': 'RRC-TransactionIdentifier'},
                                                                                   {'members': [{'members': [{'name': 'rnReconfigurationComplete-r10',
                                                                                                              'type': 'RNReconfigurationComplete-r10-IEs'},
                                                                                                             {'name': 'spare3',
                                                                                                              'type': 'NULL'},
                                                                                                             {'name': 'spare2',
                                                                                                              'type': 'NULL'},
                                                                                                             {'name': 'spare1',
                                                                                                              'type': 'NULL'}],
                                                                                                 'name': 'c1',
                                                                                                 'type': 'CHOICE'},
                                                                                                {'members': [],
                                                                                                 'name': 'criticalExtensionsFuture',
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'name': 'criticalExtensions',
                                                                                    'type': 'CHOICE'}],
                                                                       'type': 'SEQUENCE'},
                                     'RNReconfigurationComplete-r10-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'OCTET '
                                                                                                'STRING'},
                                                                                       {'members': [],
                                                                                        'name': 'nonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'}],
                                                                           'type': 'SEQUENCE'},
                                     'RRC-TransactionIdentifier': {'restricted-to': [(0,
                                                                                      3)],
                                                                   'type': 'INTEGER'},
                                     'RRCConnectionReconfiguration': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                   'type': 'RRC-TransactionIdentifier'},
                                                                                  {'members': [{'members': [{'name': 'rrcConnectionReconfiguration-r8',
                                                                                                             'type': 'RRCConnectionReconfiguration-r8-IEs'},
                                                                                                            {'name': 'spare7',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare6',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare5',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare4',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare3',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare2',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare1',
                                                                                                             'type': 'NULL'}],
                                                                                                'name': 'c1',
                                                                                                'type': 'CHOICE'},
                                                                                               {'members': [],
                                                                                                'name': 'criticalExtensionsFuture',
                                                                                                'type': 'SEQUENCE'}],
                                                                                   'name': 'criticalExtensions',
                                                                                   'type': 'CHOICE'}],
                                                                      'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-r8-IEs': {'members': [{'name': 'measConfig',
                                                                                          'optional': True,
                                                                                          'type': 'MeasConfig'},
                                                                                         {'name': 'mobilityControlInfo',
                                                                                          'optional': True,
                                                                                          'type': 'MobilityControlInfo'},
                                                                                         {'element': {'type': 'DedicatedInfoNAS'},
                                                                                          'name': 'dedicatedInfoNASList',
                                                                                          'optional': True,
                                                                                          'size': [(1,
                                                                                                    'maxDRB')],
                                                                                          'type': 'SEQUENCE '
                                                                                                  'OF'},
                                                                                         {'name': 'radioResourceConfigDedicated',
                                                                                          'optional': True,
                                                                                          'type': 'RadioResourceConfigDedicated'},
                                                                                         {'name': 'securityConfigHO',
                                                                                          'optional': True,
                                                                                          'type': 'SecurityConfigHO'},
                                                                                         {'name': 'nonCriticalExtension',
                                                                                          'optional': True,
                                                                                          'type': 'RRCConnectionReconfiguration-v890-IEs'}],
                                                                             'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v1020-IEs': {'members': [{'name': 'sCellToReleaseList-r10',
                                                                                             'optional': True,
                                                                                             'type': 'SCellToReleaseList-r10'},
                                                                                            {'name': 'sCellToAddModList-r10',
                                                                                             'optional': True,
                                                                                             'type': 'SCellToAddModList-r10'},
                                                                                            {'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'RRCConnectionReconfiguration-v1130-IEs'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v10i0-IEs': {'members': [{'name': 'antennaInfoDedicatedPCell-v10i0',
                                                                                             'optional': True,
                                                                                             'type': 'AntennaInfoDedicated-v10i0'},
                                                                                            {'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'RRCConnectionReconfiguration-v10l0-IEs'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v10l0-IEs': {'members': [{'name': 'mobilityControlInfo-v10l0',
                                                                                             'optional': True,
                                                                                             'type': 'MobilityControlInfo-v10l0'},
                                                                                            {'name': 'sCellToAddModList-v10l0',
                                                                                             'optional': True,
                                                                                             'type': 'SCellToAddModList-v10l0'},
                                                                                            {'name': 'lateNonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'OCTET '
                                                                                                     'STRING'},
                                                                                            {'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'RRCConnectionReconfiguration-v12f0-IEs'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v1130-IEs': {'members': [{'name': 'systemInformationBlockType1Dedicated-r11',
                                                                                             'optional': True,
                                                                                             'type': 'OCTET '
                                                                                                     'STRING'},
                                                                                            {'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'RRCConnectionReconfiguration-v1250-IEs'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v1250-IEs': {'members': [{'members': [{'name': 'release',
                                                                                                          'type': 'NULL'},
                                                                                                         {'members': [{'name': 'wlan-OffloadConfigDedicated-r12',
                                                                                                                       'type': 'WLAN-OffloadConfig-r12'},
                                                                                                                      {'name': 't350-r12',
                                                                                                                       'optional': True,
                                                                                                                       'type': 'ENUMERATED',
                                                                                                                       'values': [('min5',
                                                                                                                                   0),
                                                                                                                                  ('min10',
                                                                                                                                   1),
                                                                                                                                  ('min20',
                                                                                                                                   2),
                                                                                                                                  ('min30',
                                                                                                                                   3),
                                                                                                                                  ('min60',
                                                                                                                                   4),
                                                                                                                                  ('min120',
                                                                                                                                   5),
                                                                                                                                  ('min180',
                                                                                                                                   6),
                                                                                                                                  ('spare1',
                                                                                                                                   7)]}],
                                                                                                          'name': 'setup',
                                                                                                          'type': 'SEQUENCE'}],
                                                                                             'name': 'wlan-OffloadInfo-r12',
                                                                                             'optional': True,
                                                                                             'type': 'CHOICE'},
                                                                                            {'name': 'scg-Configuration-r12',
                                                                                             'optional': True,
                                                                                             'type': 'SCG-Configuration-r12'},
                                                                                            {'name': 'sl-SyncTxControl-r12',
                                                                                             'optional': True,
                                                                                             'type': 'SL-SyncTxControl-r12'},
                                                                                            {'name': 'sl-DiscConfig-r12',
                                                                                             'optional': True,
                                                                                             'type': 'SL-DiscConfig-r12'},
                                                                                            {'name': 'sl-CommConfig-r12',
                                                                                             'optional': True,
                                                                                             'type': 'SL-CommConfig-r12'},
                                                                                            {'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'RRCConnectionReconfiguration-v1310-IEs'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v12f0-IEs': {'members': [{'name': 'scg-Configuration-v12f0',
                                                                                             'optional': True,
                                                                                             'type': 'SCG-Configuration-v12f0'},
                                                                                            {'name': 'lateNonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'OCTET '
                                                                                                     'STRING'},
                                                                                            {'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'RRCConnectionReconfiguration-v1370-IEs'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v1310-IEs': {'members': [{'name': 'sCellToReleaseListExt-r13',
                                                                                             'optional': True,
                                                                                             'type': 'SCellToReleaseListExt-r13'},
                                                                                            {'name': 'sCellToAddModListExt-r13',
                                                                                             'optional': True,
                                                                                             'type': 'SCellToAddModListExt-r13'},
                                                                                            {'name': 'lwa-Configuration-r13',
                                                                                             'optional': True,
                                                                                             'type': 'LWA-Configuration-r13'},
                                                                                            {'name': 'lwip-Configuration-r13',
                                                                                             'optional': True,
                                                                                             'type': 'LWIP-Configuration-r13'},
                                                                                            {'name': 'rclwi-Configuration-r13',
                                                                                             'optional': True,
                                                                                             'type': 'RCLWI-Configuration-r13'},
                                                                                            {'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'RRCConnectionReconfiguration-v1430-IEs'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v1370-IEs': {'members': [{'name': 'radioResourceConfigDedicated-v1370',
                                                                                             'optional': True,
                                                                                             'type': 'RadioResourceConfigDedicated-v1370'},
                                                                                            {'name': 'sCellToAddModListExt-v1370',
                                                                                             'optional': True,
                                                                                             'type': 'SCellToAddModListExt-v1370'},
                                                                                            {'members': [],
                                                                                             'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'SEQUENCE'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v1430-IEs': {'members': [{'name': 'sl-V2X-ConfigDedicated-r14',
                                                                                             'optional': True,
                                                                                             'type': 'SL-V2X-ConfigDedicated-r14'},
                                                                                            {'name': 'sCellToAddModListExt-v1430',
                                                                                             'optional': True,
                                                                                             'type': 'SCellToAddModListExt-v1430'},
                                                                                            {'name': 'perCC-GapIndicationRequest-r14',
                                                                                             'optional': True,
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('true',
                                                                                                         0)]},
                                                                                            {'name': 'systemInformationBlockType2Dedicated-r14',
                                                                                             'optional': True,
                                                                                             'type': 'OCTET '
                                                                                                     'STRING'},
                                                                                            {'members': [],
                                                                                             'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'SEQUENCE'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v890-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'OCTET '
                                                                                                    'STRING'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'RRCConnectionReconfiguration-v920-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v8m0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'OCTET '
                                                                                                    'STRING'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'RRCConnectionReconfiguration-v10i0-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-v920-IEs': {'members': [{'name': 'otherConfig-r9',
                                                                                            'optional': True,
                                                                                            'type': 'OtherConfig-r9'},
                                                                                           {'name': 'fullConfig-r9',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('true',
                                                                                                        0)]},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'RRCConnectionReconfiguration-v1020-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'RRCConnectionReconfigurationComplete': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                           'type': 'RRC-TransactionIdentifier'},
                                                                                          {'members': [{'name': 'rrcConnectionReconfigurationComplete-r8',
                                                                                                        'type': 'RRCConnectionReconfigurationComplete-r8-IEs'},
                                                                                                       {'members': [],
                                                                                                        'name': 'criticalExtensionsFuture',
                                                                                                        'type': 'SEQUENCE'}],
                                                                                           'name': 'criticalExtensions',
                                                                                           'type': 'CHOICE'}],
                                                                              'type': 'SEQUENCE'},
                                     'RRCConnectionReconfigurationComplete-r8-IEs': {'members': [{'name': 'nonCriticalExtension',
                                                                                                  'optional': True,
                                                                                                  'type': 'RRCConnectionReconfigurationComplete-v8a0-IEs'}],
                                                                                     'type': 'SEQUENCE'},
                                     'RRCConnectionReconfigurationComplete-v1020-IEs': {'members': [{'name': 'rlf-InfoAvailable-r10',
                                                                                                     'optional': True,
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('true',
                                                                                                                 0)]},
                                                                                                    {'name': 'logMeasAvailable-r10',
                                                                                                     'optional': True,
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('true',
                                                                                                                 0)]},
                                                                                                    {'name': 'nonCriticalExtension',
                                                                                                     'optional': True,
                                                                                                     'type': 'RRCConnectionReconfigurationComplete-v1130-IEs'}],
                                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionReconfigurationComplete-v1130-IEs': {'members': [{'name': 'connEstFailInfoAvailable-r11',
                                                                                                     'optional': True,
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('true',
                                                                                                                 0)]},
                                                                                                    {'name': 'nonCriticalExtension',
                                                                                                     'optional': True,
                                                                                                     'type': 'RRCConnectionReconfigurationComplete-v1250-IEs'}],
                                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionReconfigurationComplete-v1250-IEs': {'members': [{'name': 'logMeasAvailableMBSFN-r12',
                                                                                                     'optional': True,
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('true',
                                                                                                                 0)]},
                                                                                                    {'name': 'nonCriticalExtension',
                                                                                                     'optional': True,
                                                                                                     'type': 'RRCConnectionReconfigurationComplete-v1430-IEs'}],
                                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionReconfigurationComplete-v1430-IEs': {'members': [{'name': 'perCC-GapIndicationList-r14',
                                                                                                     'optional': True,
                                                                                                     'type': 'PerCC-GapIndicationList-r14'},
                                                                                                    {'name': 'numFreqEffective-r14',
                                                                                                     'optional': True,
                                                                                                     'restricted-to': [(1,
                                                                                                                        12)],
                                                                                                     'type': 'INTEGER'},
                                                                                                    {'name': 'numFreqEffectiveReduced-r14',
                                                                                                     'optional': True,
                                                                                                     'restricted-to': [(1,
                                                                                                                        12)],
                                                                                                     'type': 'INTEGER'},
                                                                                                    {'members': [],
                                                                                                     'name': 'nonCriticalExtension',
                                                                                                     'optional': True,
                                                                                                     'type': 'SEQUENCE'}],
                                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionReconfigurationComplete-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'OCTET '
                                                                                                            'STRING'},
                                                                                                   {'name': 'nonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'RRCConnectionReconfigurationComplete-v1020-IEs'}],
                                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishment': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                   'type': 'RRC-TransactionIdentifier'},
                                                                                  {'members': [{'members': [{'name': 'rrcConnectionReestablishment-r8',
                                                                                                             'type': 'RRCConnectionReestablishment-r8-IEs'},
                                                                                                            {'name': 'spare7',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare6',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare5',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare4',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare3',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare2',
                                                                                                             'type': 'NULL'},
                                                                                                            {'name': 'spare1',
                                                                                                             'type': 'NULL'}],
                                                                                                'name': 'c1',
                                                                                                'type': 'CHOICE'},
                                                                                               {'members': [],
                                                                                                'name': 'criticalExtensionsFuture',
                                                                                                'type': 'SEQUENCE'}],
                                                                                   'name': 'criticalExtensions',
                                                                                   'type': 'CHOICE'}],
                                                                      'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishment-r8-IEs': {'members': [{'name': 'radioResourceConfigDedicated',
                                                                                          'type': 'RadioResourceConfigDedicated'},
                                                                                         {'name': 'nextHopChainingCount',
                                                                                          'type': 'NextHopChainingCount'},
                                                                                         {'name': 'nonCriticalExtension',
                                                                                          'optional': True,
                                                                                          'type': 'RRCConnectionReestablishment-v8a0-IEs'}],
                                                                             'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishment-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'OCTET '
                                                                                                    'STRING'},
                                                                                           {'members': [],
                                                                                            'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SEQUENCE'}],
                                                                               'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentComplete': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                           'type': 'RRC-TransactionIdentifier'},
                                                                                          {'members': [{'name': 'rrcConnectionReestablishmentComplete-r8',
                                                                                                        'type': 'RRCConnectionReestablishmentComplete-r8-IEs'},
                                                                                                       {'members': [],
                                                                                                        'name': 'criticalExtensionsFuture',
                                                                                                        'type': 'SEQUENCE'}],
                                                                                           'name': 'criticalExtensions',
                                                                                           'type': 'CHOICE'}],
                                                                              'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentComplete-r8-IEs': {'members': [{'name': 'nonCriticalExtension',
                                                                                                  'optional': True,
                                                                                                  'type': 'RRCConnectionReestablishmentComplete-v920-IEs'}],
                                                                                     'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentComplete-v1020-IEs': {'members': [{'name': 'logMeasAvailable-r10',
                                                                                                     'optional': True,
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('true',
                                                                                                                 0)]},
                                                                                                    {'name': 'nonCriticalExtension',
                                                                                                     'optional': True,
                                                                                                     'type': 'RRCConnectionReestablishmentComplete-v1130-IEs'}],
                                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentComplete-v1130-IEs': {'members': [{'name': 'connEstFailInfoAvailable-r11',
                                                                                                     'optional': True,
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('true',
                                                                                                                 0)]},
                                                                                                    {'name': 'nonCriticalExtension',
                                                                                                     'optional': True,
                                                                                                     'type': 'RRCConnectionReestablishmentComplete-v1250-IEs'}],
                                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentComplete-v1250-IEs': {'members': [{'name': 'logMeasAvailableMBSFN-r12',
                                                                                                     'optional': True,
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('true',
                                                                                                                 0)]},
                                                                                                    {'members': [],
                                                                                                     'name': 'nonCriticalExtension',
                                                                                                     'optional': True,
                                                                                                     'type': 'SEQUENCE'}],
                                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentComplete-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'OCTET '
                                                                                                            'STRING'},
                                                                                                   {'name': 'nonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'RRCConnectionReestablishmentComplete-v1020-IEs'}],
                                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentComplete-v920-IEs': {'members': [{'name': 'rlf-InfoAvailable-r9',
                                                                                                    'optional': True,
                                                                                                    'type': 'ENUMERATED',
                                                                                                    'values': [('true',
                                                                                                                0)]},
                                                                                                   {'name': 'nonCriticalExtension',
                                                                                                    'optional': True,
                                                                                                    'type': 'RRCConnectionReestablishmentComplete-v8a0-IEs'}],
                                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentReject': {'members': [{'members': [{'name': 'rrcConnectionReestablishmentReject-r8',
                                                                                                      'type': 'RRCConnectionReestablishmentReject-r8-IEs'},
                                                                                                     {'members': [],
                                                                                                      'name': 'criticalExtensionsFuture',
                                                                                                      'type': 'SEQUENCE'}],
                                                                                         'name': 'criticalExtensions',
                                                                                         'type': 'CHOICE'}],
                                                                            'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentReject-r8-IEs': {'members': [{'name': 'nonCriticalExtension',
                                                                                                'optional': True,
                                                                                                'type': 'RRCConnectionReestablishmentReject-v8a0-IEs'}],
                                                                                   'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentReject-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                                  'optional': True,
                                                                                                  'type': 'OCTET '
                                                                                                          'STRING'},
                                                                                                 {'members': [],
                                                                                                  'name': 'nonCriticalExtension',
                                                                                                  'optional': True,
                                                                                                  'type': 'SEQUENCE'}],
                                                                                     'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentRequest': {'members': [{'members': [{'name': 'rrcConnectionReestablishmentRequest-r8',
                                                                                                       'type': 'RRCConnectionReestablishmentRequest-r8-IEs'},
                                                                                                      {'members': [],
                                                                                                       'name': 'criticalExtensionsFuture',
                                                                                                       'type': 'SEQUENCE'}],
                                                                                          'name': 'criticalExtensions',
                                                                                          'type': 'CHOICE'}],
                                                                             'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentRequest-r8-IEs': {'members': [{'name': 'ue-Identity',
                                                                                                 'type': 'ReestabUE-Identity'},
                                                                                                {'name': 'reestablishmentCause',
                                                                                                 'type': 'ReestablishmentCause'},
                                                                                                {'name': 'spare',
                                                                                                 'size': [2],
                                                                                                 'type': 'BIT '
                                                                                                         'STRING'}],
                                                                                    'type': 'SEQUENCE'},
                                     'RRCConnectionReject': {'members': [{'members': [{'members': [{'name': 'rrcConnectionReject-r8',
                                                                                                    'type': 'RRCConnectionReject-r8-IEs'},
                                                                                                   {'name': 'spare3',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare2',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare1',
                                                                                                    'type': 'NULL'}],
                                                                                       'name': 'c1',
                                                                                       'type': 'CHOICE'},
                                                                                      {'members': [],
                                                                                       'name': 'criticalExtensionsFuture',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'name': 'criticalExtensions',
                                                                          'type': 'CHOICE'}],
                                                             'type': 'SEQUENCE'},
                                     'RRCConnectionReject-r8-IEs': {'members': [{'name': 'waitTime',
                                                                                 'restricted-to': [(1,
                                                                                                    16)],
                                                                                 'type': 'INTEGER'},
                                                                                {'name': 'nonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'RRCConnectionReject-v8a0-IEs'}],
                                                                    'type': 'SEQUENCE'},
                                     'RRCConnectionReject-v1020-IEs': {'members': [{'name': 'extendedWaitTime-r10',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(1,
                                                                                                       1800)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'RRCConnectionReject-v1130-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionReject-v1130-IEs': {'members': [{'members': [{'name': 'deprioritisationType-r11',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('frequency',
                                                                                                             0),
                                                                                                            ('e-utra',
                                                                                                             1)]},
                                                                                                {'name': 'deprioritisationTimer-r11',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('min5',
                                                                                                             0),
                                                                                                            ('min10',
                                                                                                             1),
                                                                                                            ('min15',
                                                                                                             2),
                                                                                                            ('min30',
                                                                                                             3)]}],
                                                                                    'name': 'deprioritisationReq-r11',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'RRCConnectionReject-v1320-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionReject-v1320-IEs': {'members': [{'name': 'rrc-SuspendIndication-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionReject-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'RRCConnectionReject-v1020-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'RRCConnectionRelease': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                           'type': 'RRC-TransactionIdentifier'},
                                                                          {'members': [{'members': [{'name': 'rrcConnectionRelease-r8',
                                                                                                     'type': 'RRCConnectionRelease-r8-IEs'},
                                                                                                    {'name': 'spare3',
                                                                                                     'type': 'NULL'},
                                                                                                    {'name': 'spare2',
                                                                                                     'type': 'NULL'},
                                                                                                    {'name': 'spare1',
                                                                                                     'type': 'NULL'}],
                                                                                        'name': 'c1',
                                                                                        'type': 'CHOICE'},
                                                                                       {'members': [],
                                                                                        'name': 'criticalExtensionsFuture',
                                                                                        'type': 'SEQUENCE'}],
                                                                           'name': 'criticalExtensions',
                                                                           'type': 'CHOICE'}],
                                                              'type': 'SEQUENCE'},
                                     'RRCConnectionRelease-r8-IEs': {'members': [{'name': 'releaseCause',
                                                                                  'type': 'ReleaseCause'},
                                                                                 {'name': 'redirectedCarrierInfo',
                                                                                  'optional': True,
                                                                                  'type': 'RedirectedCarrierInfo'},
                                                                                 {'name': 'idleModeMobilityControlInfo',
                                                                                  'optional': True,
                                                                                  'type': 'IdleModeMobilityControlInfo'},
                                                                                 {'name': 'nonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'RRCConnectionRelease-v890-IEs'}],
                                                                     'type': 'SEQUENCE'},
                                     'RRCConnectionRelease-v1020-IEs': {'members': [{'name': 'extendedWaitTime-r10',
                                                                                     'optional': True,
                                                                                     'restricted-to': [(1,
                                                                                                        1800)],
                                                                                     'type': 'INTEGER'},
                                                                                    {'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'RRCConnectionRelease-v1320-IEs'}],
                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionRelease-v1320-IEs': {'members': [{'name': 'resumeIdentity-r13',
                                                                                     'optional': True,
                                                                                     'type': 'ResumeIdentity-r13'},
                                                                                    {'members': [],
                                                                                     'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'}],
                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionRelease-v890-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'RRCConnectionRelease-v920-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionRelease-v920-IEs': {'members': [{'members': [{'name': 'geran-r9',
                                                                                                 'type': 'CellInfoListGERAN-r9'},
                                                                                                {'name': 'utra-FDD-r9',
                                                                                                 'type': 'CellInfoListUTRA-FDD-r9'},
                                                                                                {'name': 'utra-TDD-r9',
                                                                                                 'type': 'CellInfoListUTRA-TDD-r9'},
                                                                                                None,
                                                                                                {'name': 'utra-TDD-r10',
                                                                                                 'type': 'CellInfoListUTRA-TDD-r10'}],
                                                                                    'name': 'cellInfoList-r9',
                                                                                    'optional': True,
                                                                                    'type': 'CHOICE'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'RRCConnectionRelease-v1020-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionRelease-v9e0-IEs': {'members': [{'name': 'redirectedCarrierInfo-v9e0',
                                                                                    'optional': True,
                                                                                    'type': 'RedirectedCarrierInfo-v9e0'},
                                                                                   {'name': 'idleModeMobilityControlInfo-v9e0',
                                                                                    'optional': True,
                                                                                    'type': 'IdleModeMobilityControlInfo-v9e0'},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionRequest': {'members': [{'members': [{'name': 'rrcConnectionRequest-r8',
                                                                                        'type': 'RRCConnectionRequest-r8-IEs'},
                                                                                       {'members': [],
                                                                                        'name': 'criticalExtensionsFuture',
                                                                                        'type': 'SEQUENCE'}],
                                                                           'name': 'criticalExtensions',
                                                                           'type': 'CHOICE'}],
                                                              'type': 'SEQUENCE'},
                                     'RRCConnectionRequest-r8-IEs': {'members': [{'name': 'ue-Identity',
                                                                                  'type': 'InitialUE-Identity'},
                                                                                 {'name': 'establishmentCause',
                                                                                  'type': 'EstablishmentCause'},
                                                                                 {'name': 'spare',
                                                                                  'size': [1],
                                                                                  'type': 'BIT '
                                                                                          'STRING'}],
                                                                     'type': 'SEQUENCE'},
                                     'RRCConnectionResume-r13': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                              'type': 'RRC-TransactionIdentifier'},
                                                                             {'members': [{'members': [{'name': 'rrcConnectionResume-r13',
                                                                                                        'type': 'RRCConnectionResume-r13-IEs'},
                                                                                                       {'name': 'spare3',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare2',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare1',
                                                                                                        'type': 'NULL'}],
                                                                                           'name': 'c1',
                                                                                           'type': 'CHOICE'},
                                                                                          {'members': [],
                                                                                           'name': 'criticalExtensionsFuture',
                                                                                           'type': 'SEQUENCE'}],
                                                                              'name': 'criticalExtensions',
                                                                              'type': 'CHOICE'}],
                                                                 'type': 'SEQUENCE'},
                                     'RRCConnectionResume-r13-IEs': {'members': [{'name': 'radioResourceConfigDedicated-r13',
                                                                                  'optional': True,
                                                                                  'type': 'RadioResourceConfigDedicated'},
                                                                                 {'name': 'nextHopChainingCount-r13',
                                                                                  'type': 'NextHopChainingCount'},
                                                                                 {'name': 'measConfig-r13',
                                                                                  'optional': True,
                                                                                  'type': 'MeasConfig'},
                                                                                 {'name': 'antennaInfoDedicatedPCell-r13',
                                                                                  'optional': True,
                                                                                  'type': 'AntennaInfoDedicated-v10i0'},
                                                                                 {'name': 'drb-ContinueROHC-r13',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('true',
                                                                                              0)]},
                                                                                 {'name': 'lateNonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 {'name': 'rrcConnectionResume-v1430-IEs',
                                                                                  'optional': True,
                                                                                  'type': 'RRCConnectionResume-v1430-IEs'}],
                                                                     'type': 'SEQUENCE'},
                                     'RRCConnectionResume-v1430-IEs': {'members': [{'name': 'otherConfig-r14',
                                                                                    'optional': True,
                                                                                    'type': 'OtherConfig-r9'},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionResumeComplete-r13': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                      'type': 'RRC-TransactionIdentifier'},
                                                                                     {'members': [{'name': 'rrcConnectionResumeComplete-r13',
                                                                                                   'type': 'RRCConnectionResumeComplete-r13-IEs'},
                                                                                                  {'members': [],
                                                                                                   'name': 'criticalExtensionsFuture',
                                                                                                   'type': 'SEQUENCE'}],
                                                                                      'name': 'criticalExtensions',
                                                                                      'type': 'CHOICE'}],
                                                                         'type': 'SEQUENCE'},
                                     'RRCConnectionResumeComplete-r13-IEs': {'members': [{'name': 'selectedPLMN-Identity-r13',
                                                                                          'optional': True,
                                                                                          'restricted-to': [(1,
                                                                                                             'maxPLMN-r11')],
                                                                                          'type': 'INTEGER'},
                                                                                         {'name': 'dedicatedInfoNAS-r13',
                                                                                          'optional': True,
                                                                                          'type': 'DedicatedInfoNAS'},
                                                                                         {'name': 'rlf-InfoAvailable-r13',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('true',
                                                                                                      0)]},
                                                                                         {'name': 'logMeasAvailable-r13',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('true',
                                                                                                      0)]},
                                                                                         {'name': 'connEstFailInfoAvailable-r13',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('true',
                                                                                                      0)]},
                                                                                         {'name': 'mobilityState-r13',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('normal',
                                                                                                      0),
                                                                                                     ('medium',
                                                                                                      1),
                                                                                                     ('high',
                                                                                                      2),
                                                                                                     ('spare',
                                                                                                      3)]},
                                                                                         {'name': 'mobilityHistoryAvail-r13',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('true',
                                                                                                      0)]},
                                                                                         {'name': 'logMeasAvailableMBSFN-r13',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('true',
                                                                                                      0)]},
                                                                                         {'name': 'lateNonCriticalExtension',
                                                                                          'optional': True,
                                                                                          'type': 'OCTET '
                                                                                                  'STRING'},
                                                                                         {'members': [],
                                                                                          'name': 'nonCriticalExtension',
                                                                                          'optional': True,
                                                                                          'type': 'SEQUENCE'}],
                                                                             'type': 'SEQUENCE'},
                                     'RRCConnectionResumeRequest-r13': {'members': [{'members': [{'name': 'rrcConnectionResumeRequest-r13',
                                                                                                  'type': 'RRCConnectionResumeRequest-r13-IEs'},
                                                                                                 {'members': [],
                                                                                                  'name': 'criticalExtensionsFuture',
                                                                                                  'type': 'SEQUENCE'}],
                                                                                     'name': 'criticalExtensions',
                                                                                     'type': 'CHOICE'}],
                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionResumeRequest-r13-IEs': {'members': [{'members': [{'name': 'resumeID-r13',
                                                                                                      'type': 'ResumeIdentity-r13'},
                                                                                                     {'name': 'truncatedResumeID-r13',
                                                                                                      'size': [24],
                                                                                                      'type': 'BIT '
                                                                                                              'STRING'}],
                                                                                         'name': 'resumeIdentity-r13',
                                                                                         'type': 'CHOICE'},
                                                                                        {'name': 'shortResumeMAC-I-r13',
                                                                                         'size': [16],
                                                                                         'type': 'BIT '
                                                                                                 'STRING'},
                                                                                        {'name': 'resumeCause-r13',
                                                                                         'type': 'ResumeCause'},
                                                                                        {'name': 'spare',
                                                                                         'size': [1],
                                                                                         'type': 'BIT '
                                                                                                 'STRING'}],
                                                                            'type': 'SEQUENCE'},
                                     'RRCConnectionSetup': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                         'type': 'RRC-TransactionIdentifier'},
                                                                        {'members': [{'members': [{'name': 'rrcConnectionSetup-r8',
                                                                                                   'type': 'RRCConnectionSetup-r8-IEs'},
                                                                                                  {'name': 'spare7',
                                                                                                   'type': 'NULL'},
                                                                                                  {'name': 'spare6',
                                                                                                   'type': 'NULL'},
                                                                                                  {'name': 'spare5',
                                                                                                   'type': 'NULL'},
                                                                                                  {'name': 'spare4',
                                                                                                   'type': 'NULL'},
                                                                                                  {'name': 'spare3',
                                                                                                   'type': 'NULL'},
                                                                                                  {'name': 'spare2',
                                                                                                   'type': 'NULL'},
                                                                                                  {'name': 'spare1',
                                                                                                   'type': 'NULL'}],
                                                                                      'name': 'c1',
                                                                                      'type': 'CHOICE'},
                                                                                     {'members': [],
                                                                                      'name': 'criticalExtensionsFuture',
                                                                                      'type': 'SEQUENCE'}],
                                                                         'name': 'criticalExtensions',
                                                                         'type': 'CHOICE'}],
                                                            'type': 'SEQUENCE'},
                                     'RRCConnectionSetup-r8-IEs': {'members': [{'name': 'radioResourceConfigDedicated',
                                                                                'type': 'RadioResourceConfigDedicated'},
                                                                               {'name': 'nonCriticalExtension',
                                                                                'optional': True,
                                                                                'type': 'RRCConnectionSetup-v8a0-IEs'}],
                                                                   'type': 'SEQUENCE'},
                                     'RRCConnectionSetup-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 {'members': [],
                                                                                  'name': 'nonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'SEQUENCE'}],
                                                                     'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                 'type': 'RRC-TransactionIdentifier'},
                                                                                {'members': [{'members': [{'name': 'rrcConnectionSetupComplete-r8',
                                                                                                           'type': 'RRCConnectionSetupComplete-r8-IEs'},
                                                                                                          {'name': 'spare3',
                                                                                                           'type': 'NULL'},
                                                                                                          {'name': 'spare2',
                                                                                                           'type': 'NULL'},
                                                                                                          {'name': 'spare1',
                                                                                                           'type': 'NULL'}],
                                                                                              'name': 'c1',
                                                                                              'type': 'CHOICE'},
                                                                                             {'members': [],
                                                                                              'name': 'criticalExtensionsFuture',
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'name': 'criticalExtensions',
                                                                                 'type': 'CHOICE'}],
                                                                    'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-r8-IEs': {'members': [{'name': 'selectedPLMN-Identity',
                                                                                        'restricted-to': [(1,
                                                                                                           'maxPLMN-r11')],
                                                                                        'type': 'INTEGER'},
                                                                                       {'name': 'registeredMME',
                                                                                        'optional': True,
                                                                                        'type': 'RegisteredMME'},
                                                                                       {'name': 'dedicatedInfoNAS',
                                                                                        'type': 'DedicatedInfoNAS'},
                                                                                       {'name': 'nonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'RRCConnectionSetupComplete-v8a0-IEs'}],
                                                                           'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-v1020-IEs': {'members': [{'name': 'gummei-Type-r10',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('native',
                                                                                                       0),
                                                                                                      ('mapped',
                                                                                                       1)]},
                                                                                          {'name': 'rlf-InfoAvailable-r10',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('true',
                                                                                                       0)]},
                                                                                          {'name': 'logMeasAvailable-r10',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('true',
                                                                                                       0)]},
                                                                                          {'name': 'rn-SubframeConfigReq-r10',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('required',
                                                                                                       0),
                                                                                                      ('notRequired',
                                                                                                       1)]},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'RRCConnectionSetupComplete-v1130-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-v1130-IEs': {'members': [{'name': 'connEstFailInfoAvailable-r11',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('true',
                                                                                                       0)]},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'RRCConnectionSetupComplete-v1250-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-v1250-IEs': {'members': [{'name': 'mobilityState-r12',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('normal',
                                                                                                       0),
                                                                                                      ('medium',
                                                                                                       1),
                                                                                                      ('high',
                                                                                                       2),
                                                                                                      ('spare',
                                                                                                       3)]},
                                                                                          {'name': 'mobilityHistoryAvail-r12',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('true',
                                                                                                       0)]},
                                                                                          {'name': 'logMeasAvailableMBSFN-r12',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('true',
                                                                                                       0)]},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'RRCConnectionSetupComplete-v1320-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-v1320-IEs': {'members': [{'name': 'ce-ModeB-r13',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('supported',
                                                                                                       0)]},
                                                                                          {'name': 's-TMSI-r13',
                                                                                           'optional': True,
                                                                                           'type': 'S-TMSI'},
                                                                                          {'name': 'attachWithoutPDN-Connectivity-r13',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('true',
                                                                                                       0)]},
                                                                                          {'name': 'up-CIoT-EPS-Optimisation-r13',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('true',
                                                                                                       0)]},
                                                                                          {'name': 'cp-CIoT-EPS-Optimisation-r13',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('true',
                                                                                                       0)]},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'RRCConnectionSetupComplete-v1330-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-v1330-IEs': {'members': [{'name': 'ue-CE-NeedULGaps-r13',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('true',
                                                                                                       0)]},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'RRCConnectionSetupComplete-v1430-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-v1430-IEs': {'members': [{'name': 'dcn-ID-r14',
                                                                                           'optional': True,
                                                                                           'restricted-to': [(0,
                                                                                                              65535)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'members': [],
                                                                                           'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SEQUENCE'}],
                                                                              'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                          'optional': True,
                                                                                          'type': 'OCTET '
                                                                                                  'STRING'},
                                                                                         {'name': 'nonCriticalExtension',
                                                                                          'optional': True,
                                                                                          'type': 'RRCConnectionSetupComplete-v1020-IEs'}],
                                                                             'type': 'SEQUENCE'},
                                     'RS-SINR-Range-r13': {'restricted-to': [(0,
                                                                              127)],
                                                           'type': 'INTEGER'},
                                     'RSRP-Range': {'restricted-to': [(0, 97)],
                                                    'type': 'INTEGER'},
                                     'RSRP-Range-v1360': {'restricted-to': [(-17,
                                                                             -1)],
                                                          'type': 'INTEGER'},
                                     'RSRP-RangeSL-r12': {'restricted-to': [(0,
                                                                             13)],
                                                          'type': 'INTEGER'},
                                     'RSRP-RangeSL2-r12': {'restricted-to': [(0,
                                                                              7)],
                                                           'type': 'INTEGER'},
                                     'RSRP-RangeSL3-r12': {'restricted-to': [(0,
                                                                              11)],
                                                           'type': 'INTEGER'},
                                     'RSRP-RangeSL4-r13': {'restricted-to': [(0,
                                                                              49)],
                                                           'type': 'INTEGER'},
                                     'RSRP-ThresholdsPrachInfoList-r13': {'element': {'type': 'RSRP-Range'},
                                                                          'size': [(1,
                                                                                    3)],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                     'RSRQ-Range': {'restricted-to': [(0, 34)],
                                                    'type': 'INTEGER'},
                                     'RSRQ-Range-r13': {'restricted-to': [(-30,
                                                                           46)],
                                                        'type': 'INTEGER'},
                                     'RSRQ-Range-v1250': {'restricted-to': [(-30,
                                                                             46)],
                                                          'type': 'INTEGER'},
                                     'RSRQ-RangeConfig-r12': {'members': [{'name': 'release',
                                                                           'type': 'NULL'},
                                                                          {'name': 'setup',
                                                                           'type': 'RSRQ-Range-v1250'}],
                                                              'type': 'CHOICE'},
                                     'RSRQ-Type-r12': {'members': [{'name': 'allSymbols-r12',
                                                                    'type': 'BOOLEAN'},
                                                                   {'name': 'wideBand-r12',
                                                                    'type': 'BOOLEAN'}],
                                                       'type': 'SEQUENCE'},
                                     'RSSI-Range-r13': {'restricted-to': [(0,
                                                                           76)],
                                                        'type': 'INTEGER'},
                                     'RSTD-InterFreqInfo-r10': {'members': [{'name': 'carrierFreq-r10',
                                                                             'type': 'ARFCN-ValueEUTRA'},
                                                                            {'name': 'measPRS-Offset-r10',
                                                                             'restricted-to': [(0,
                                                                                                39)],
                                                                             'type': 'INTEGER'},
                                                                            None,
                                                                            [{'name': 'carrierFreq-v1090',
                                                                              'optional': True,
                                                                              'type': 'ARFCN-ValueEUTRA-v9e0'}]],
                                                                'type': 'SEQUENCE'},
                                     'RSTD-InterFreqInfoList-r10': {'element': {'type': 'RSTD-InterFreqInfo-r10'},
                                                                    'size': [(1,
                                                                              'maxRSTD-Freq-r10')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'RadioResourceConfigCommon': {'members': [{'name': 'rach-ConfigCommon',
                                                                                'optional': True,
                                                                                'type': 'RACH-ConfigCommon'},
                                                                               {'name': 'prach-Config',
                                                                                'type': 'PRACH-Config'},
                                                                               {'name': 'pdsch-ConfigCommon',
                                                                                'optional': True,
                                                                                'type': 'PDSCH-ConfigCommon'},
                                                                               {'name': 'pusch-ConfigCommon',
                                                                                'type': 'PUSCH-ConfigCommon'},
                                                                               {'name': 'phich-Config',
                                                                                'optional': True,
                                                                                'type': 'PHICH-Config'},
                                                                               {'name': 'pucch-ConfigCommon',
                                                                                'optional': True,
                                                                                'type': 'PUCCH-ConfigCommon'},
                                                                               {'name': 'soundingRS-UL-ConfigCommon',
                                                                                'optional': True,
                                                                                'type': 'SoundingRS-UL-ConfigCommon'},
                                                                               {'name': 'uplinkPowerControlCommon',
                                                                                'optional': True,
                                                                                'type': 'UplinkPowerControlCommon'},
                                                                               {'name': 'antennaInfoCommon',
                                                                                'optional': True,
                                                                                'type': 'AntennaInfoCommon'},
                                                                               {'name': 'p-Max',
                                                                                'optional': True,
                                                                                'type': 'P-Max'},
                                                                               {'name': 'tdd-Config',
                                                                                'optional': True,
                                                                                'type': 'TDD-Config'},
                                                                               {'name': 'ul-CyclicPrefixLength',
                                                                                'type': 'UL-CyclicPrefixLength'},
                                                                               None,
                                                                               [{'name': 'uplinkPowerControlCommon-v1020',
                                                                                 'optional': True,
                                                                                 'type': 'UplinkPowerControlCommon-v1020'}],
                                                                               [{'name': 'tdd-Config-v1130',
                                                                                 'optional': True,
                                                                                 'type': 'TDD-Config-v1130'}],
                                                                               [{'name': 'pusch-ConfigCommon-v1270',
                                                                                 'optional': True,
                                                                                 'type': 'PUSCH-ConfigCommon-v1270'}],
                                                                               [{'name': 'prach-Config-v1310',
                                                                                 'optional': True,
                                                                                 'type': 'PRACH-Config-v1310'},
                                                                                {'name': 'freqHoppingParameters-r13',
                                                                                 'optional': True,
                                                                                 'type': 'FreqHoppingParameters-r13'},
                                                                                {'name': 'pdsch-ConfigCommon-v1310',
                                                                                 'optional': True,
                                                                                 'type': 'PDSCH-ConfigCommon-v1310'},
                                                                                {'name': 'pucch-ConfigCommon-v1310',
                                                                                 'optional': True,
                                                                                 'type': 'PUCCH-ConfigCommon-v1310'},
                                                                                {'name': 'pusch-ConfigCommon-v1310',
                                                                                 'optional': True,
                                                                                 'type': 'PUSCH-ConfigCommon-v1310'},
                                                                                {'name': 'uplinkPowerControlCommon-v1310',
                                                                                 'optional': True,
                                                                                 'type': 'UplinkPowerControlCommon-v1310'}],
                                                                               [{'name': 'highSpeedConfig-r14',
                                                                                 'optional': True,
                                                                                 'type': 'HighSpeedConfig-r14'},
                                                                                {'name': 'prach-Config-v1430',
                                                                                 'optional': True,
                                                                                 'type': 'PRACH-Config-v1430'},
                                                                                {'name': 'pucch-ConfigCommon-v1430',
                                                                                 'optional': True,
                                                                                 'type': 'PUCCH-ConfigCommon-v1430'},
                                                                                {'name': 'tdd-Config-v1430',
                                                                                 'optional': True,
                                                                                 'type': 'TDD-Config-v1430'}]],
                                                                   'type': 'SEQUENCE'},
                                     'RadioResourceConfigCommonPSCell-r12': {'members': [{'name': 'basicFields-r12',
                                                                                          'type': 'RadioResourceConfigCommonSCell-r10'},
                                                                                         {'name': 'pucch-ConfigCommon-r12',
                                                                                          'type': 'PUCCH-ConfigCommon'},
                                                                                         {'name': 'rach-ConfigCommon-r12',
                                                                                          'type': 'RACH-ConfigCommon'},
                                                                                         {'name': 'uplinkPowerControlCommonPSCell-r12',
                                                                                          'type': 'UplinkPowerControlCommonPSCell-r12'},
                                                                                         None,
                                                                                         [{'name': 'uplinkPowerControlCommonPSCell-v1310',
                                                                                           'optional': True,
                                                                                           'type': 'UplinkPowerControlCommon-v1310'}]],
                                                                             'type': 'SEQUENCE'},
                                     'RadioResourceConfigCommonPSCell-v12f0': {'members': [{'name': 'basicFields-v12f0',
                                                                                            'type': 'RadioResourceConfigCommonSCell-v10l0'}],
                                                                               'type': 'SEQUENCE'},
                                     'RadioResourceConfigCommonPSCell-v1440': {'members': [{'name': 'basicFields-v1440',
                                                                                            'type': 'RadioResourceConfigCommonSCell-v1440'}],
                                                                               'type': 'SEQUENCE'},
                                     'RadioResourceConfigCommonSCell-r10': {'members': [{'members': [{'name': 'dl-Bandwidth-r10',
                                                                                                      'type': 'ENUMERATED',
                                                                                                      'values': [('n6',
                                                                                                                  0),
                                                                                                                 ('n15',
                                                                                                                  1),
                                                                                                                 ('n25',
                                                                                                                  2),
                                                                                                                 ('n50',
                                                                                                                  3),
                                                                                                                 ('n75',
                                                                                                                  4),
                                                                                                                 ('n100',
                                                                                                                  5)]},
                                                                                                     {'name': 'antennaInfoCommon-r10',
                                                                                                      'type': 'AntennaInfoCommon'},
                                                                                                     {'name': 'mbsfn-SubframeConfigList-r10',
                                                                                                      'optional': True,
                                                                                                      'type': 'MBSFN-SubframeConfigList'},
                                                                                                     {'name': 'phich-Config-r10',
                                                                                                      'type': 'PHICH-Config'},
                                                                                                     {'name': 'pdsch-ConfigCommon-r10',
                                                                                                      'type': 'PDSCH-ConfigCommon'},
                                                                                                     {'name': 'tdd-Config-r10',
                                                                                                      'optional': True,
                                                                                                      'type': 'TDD-Config'}],
                                                                                         'name': 'nonUL-Configuration-r10',
                                                                                         'type': 'SEQUENCE'},
                                                                                        {'members': [{'members': [{'name': 'ul-CarrierFreq-r10',
                                                                                                                   'optional': True,
                                                                                                                   'type': 'ARFCN-ValueEUTRA'},
                                                                                                                  {'name': 'ul-Bandwidth-r10',
                                                                                                                   'optional': True,
                                                                                                                   'type': 'ENUMERATED',
                                                                                                                   'values': [('n6',
                                                                                                                               0),
                                                                                                                              ('n15',
                                                                                                                               1),
                                                                                                                              ('n25',
                                                                                                                               2),
                                                                                                                              ('n50',
                                                                                                                               3),
                                                                                                                              ('n75',
                                                                                                                               4),
                                                                                                                              ('n100',
                                                                                                                               5)]},
                                                                                                                  {'name': 'additionalSpectrumEmissionSCell-r10',
                                                                                                                   'type': 'AdditionalSpectrumEmission'}],
                                                                                                      'name': 'ul-FreqInfo-r10',
                                                                                                      'type': 'SEQUENCE'},
                                                                                                     {'name': 'p-Max-r10',
                                                                                                      'optional': True,
                                                                                                      'type': 'P-Max'},
                                                                                                     {'name': 'uplinkPowerControlCommonSCell-r10',
                                                                                                      'type': 'UplinkPowerControlCommonSCell-r10'},
                                                                                                     {'name': 'soundingRS-UL-ConfigCommon-r10',
                                                                                                      'type': 'SoundingRS-UL-ConfigCommon'},
                                                                                                     {'name': 'ul-CyclicPrefixLength-r10',
                                                                                                      'type': 'UL-CyclicPrefixLength'},
                                                                                                     {'name': 'prach-ConfigSCell-r10',
                                                                                                      'optional': True,
                                                                                                      'type': 'PRACH-ConfigSCell-r10'},
                                                                                                     {'name': 'pusch-ConfigCommon-r10',
                                                                                                      'type': 'PUSCH-ConfigCommon'}],
                                                                                         'name': 'ul-Configuration-r10',
                                                                                         'optional': True,
                                                                                         'type': 'SEQUENCE'},
                                                                                        None,
                                                                                        [{'name': 'ul-CarrierFreq-v1090',
                                                                                          'optional': True,
                                                                                          'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                                                        [{'name': 'rach-ConfigCommonSCell-r11',
                                                                                          'optional': True,
                                                                                          'type': 'RACH-ConfigCommonSCell-r11'},
                                                                                         {'name': 'prach-ConfigSCell-r11',
                                                                                          'optional': True,
                                                                                          'type': 'PRACH-Config'},
                                                                                         {'name': 'tdd-Config-v1130',
                                                                                          'optional': True,
                                                                                          'type': 'TDD-Config-v1130'},
                                                                                         {'name': 'uplinkPowerControlCommonSCell-v1130',
                                                                                          'optional': True,
                                                                                          'type': 'UplinkPowerControlCommonSCell-v1130'}],
                                                                                        [{'name': 'pusch-ConfigCommon-v1270',
                                                                                          'optional': True,
                                                                                          'type': 'PUSCH-ConfigCommon-v1270'}],
                                                                                        [{'name': 'pucch-ConfigCommon-r13',
                                                                                          'optional': True,
                                                                                          'type': 'PUCCH-ConfigCommon'},
                                                                                         {'name': 'uplinkPowerControlCommonSCell-v1310',
                                                                                          'optional': True,
                                                                                          'type': 'UplinkPowerControlCommonSCell-v1310'}],
                                                                                        [{'name': 'highSpeedConfigSCell-r14',
                                                                                          'optional': True,
                                                                                          'type': 'HighSpeedConfigSCell-r14'},
                                                                                         {'name': 'prach-Config-v1430',
                                                                                          'optional': True,
                                                                                          'type': 'PRACH-Config-v1430'},
                                                                                         {'members': [{'members': [{'name': 'ul-CarrierFreq-r14',
                                                                                                                    'optional': True,
                                                                                                                    'type': 'ARFCN-ValueEUTRA-r9'},
                                                                                                                   {'name': 'ul-Bandwidth-r14',
                                                                                                                    'optional': True,
                                                                                                                    'type': 'ENUMERATED',
                                                                                                                    'values': [('n6',
                                                                                                                                0),
                                                                                                                               ('n15',
                                                                                                                                1),
                                                                                                                               ('n25',
                                                                                                                                2),
                                                                                                                               ('n50',
                                                                                                                                3),
                                                                                                                               ('n75',
                                                                                                                                4),
                                                                                                                               ('n100',
                                                                                                                                5)]},
                                                                                                                   {'name': 'additionalSpectrumEmissionSCell-r14',
                                                                                                                    'type': 'AdditionalSpectrumEmission'}],
                                                                                                       'name': 'ul-FreqInfo-r14',
                                                                                                       'type': 'SEQUENCE'},
                                                                                                      {'name': 'p-Max-r14',
                                                                                                       'optional': True,
                                                                                                       'type': 'P-Max'},
                                                                                                      {'name': 'soundingRS-UL-ConfigCommon-r14',
                                                                                                       'type': 'SoundingRS-UL-ConfigCommon'},
                                                                                                      {'name': 'ul-CyclicPrefixLength-r14',
                                                                                                       'type': 'UL-CyclicPrefixLength'},
                                                                                                      {'name': 'prach-ConfigSCell-r14',
                                                                                                       'optional': True,
                                                                                                       'type': 'PRACH-ConfigSCell-r10'},
                                                                                                      {'name': 'uplinkPowerControlCommonPUSCH-LessCell-v1430',
                                                                                                       'optional': True,
                                                                                                       'type': 'UplinkPowerControlCommonPUSCH-LessCell-v1430'}],
                                                                                          'name': 'ul-Configuration-r14',
                                                                                          'optional': True,
                                                                                          'type': 'SEQUENCE'},
                                                                                         {'name': 'harq-ReferenceConfig-r14',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('sa2',
                                                                                                      0),
                                                                                                     ('sa4',
                                                                                                      1),
                                                                                                     ('sa5',
                                                                                                      2)]},
                                                                                         {'name': 'soundingRS-FlexibleTiming-r14',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('true',
                                                                                                      0)]}],
                                                                                        [{'name': 'mbsfn-SubframeConfigList-v1430',
                                                                                          'optional': True,
                                                                                          'type': 'MBSFN-SubframeConfigList-v1430'}]],
                                                                            'type': 'SEQUENCE'},
                                     'RadioResourceConfigCommonSCell-v10l0': {'members': [{'members': [{'name': 'additionalSpectrumEmissionSCell-v10l0',
                                                                                                        'type': 'AdditionalSpectrumEmission-v10l0'}],
                                                                                           'name': 'ul-Configuration-v10l0',
                                                                                           'type': 'SEQUENCE'}],
                                                                              'type': 'SEQUENCE'},
                                     'RadioResourceConfigCommonSCell-v1440': {'members': [{'members': [{'members': [{'name': 'additionalSpectrumEmissionSCell-v1440',
                                                                                                                     'type': 'AdditionalSpectrumEmission-v10l0'}],
                                                                                                        'name': 'ul-FreqInfo-v1440',
                                                                                                        'type': 'SEQUENCE'}],
                                                                                           'name': 'ul-Configuration-v1440',
                                                                                           'type': 'SEQUENCE'}],
                                                                              'type': 'SEQUENCE'},
                                     'RadioResourceConfigCommonSIB': {'members': [{'name': 'rach-ConfigCommon',
                                                                                   'type': 'RACH-ConfigCommon'},
                                                                                  {'name': 'bcch-Config',
                                                                                   'type': 'BCCH-Config'},
                                                                                  {'name': 'pcch-Config',
                                                                                   'type': 'PCCH-Config'},
                                                                                  {'name': 'prach-Config',
                                                                                   'type': 'PRACH-ConfigSIB'},
                                                                                  {'name': 'pdsch-ConfigCommon',
                                                                                   'type': 'PDSCH-ConfigCommon'},
                                                                                  {'name': 'pusch-ConfigCommon',
                                                                                   'type': 'PUSCH-ConfigCommon'},
                                                                                  {'name': 'pucch-ConfigCommon',
                                                                                   'type': 'PUCCH-ConfigCommon'},
                                                                                  {'name': 'soundingRS-UL-ConfigCommon',
                                                                                   'type': 'SoundingRS-UL-ConfigCommon'},
                                                                                  {'name': 'uplinkPowerControlCommon',
                                                                                   'type': 'UplinkPowerControlCommon'},
                                                                                  {'name': 'ul-CyclicPrefixLength',
                                                                                   'type': 'UL-CyclicPrefixLength'},
                                                                                  None,
                                                                                  [{'name': 'uplinkPowerControlCommon-v1020',
                                                                                    'optional': True,
                                                                                    'type': 'UplinkPowerControlCommon-v1020'}],
                                                                                  [{'name': 'rach-ConfigCommon-v1250',
                                                                                    'optional': True,
                                                                                    'type': 'RACH-ConfigCommon-v1250'}],
                                                                                  [{'name': 'pusch-ConfigCommon-v1270',
                                                                                    'optional': True,
                                                                                    'type': 'PUSCH-ConfigCommon-v1270'}],
                                                                                  [{'name': 'bcch-Config-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'BCCH-Config-v1310'},
                                                                                   {'name': 'pcch-Config-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'PCCH-Config-v1310'},
                                                                                   {'name': 'freqHoppingParameters-r13',
                                                                                    'optional': True,
                                                                                    'type': 'FreqHoppingParameters-r13'},
                                                                                   {'name': 'pdsch-ConfigCommon-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'PDSCH-ConfigCommon-v1310'},
                                                                                   {'name': 'pusch-ConfigCommon-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'PUSCH-ConfigCommon-v1310'},
                                                                                   {'name': 'prach-ConfigCommon-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'PRACH-ConfigSIB-v1310'},
                                                                                   {'name': 'pucch-ConfigCommon-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'PUCCH-ConfigCommon-v1310'}],
                                                                                  [{'name': 'highSpeedConfig-r14',
                                                                                    'optional': True,
                                                                                    'type': 'HighSpeedConfig-r14'},
                                                                                   {'name': 'prach-Config-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'PRACH-Config-v1430'},
                                                                                   {'name': 'pucch-ConfigCommon-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'PUCCH-ConfigCommon-v1430'}]],
                                                                      'type': 'SEQUENCE'},
                                     'RadioResourceConfigDedicated': {'members': [{'name': 'srb-ToAddModList',
                                                                                   'optional': True,
                                                                                   'type': 'SRB-ToAddModList'},
                                                                                  {'name': 'drb-ToAddModList',
                                                                                   'optional': True,
                                                                                   'type': 'DRB-ToAddModList'},
                                                                                  {'name': 'drb-ToReleaseList',
                                                                                   'optional': True,
                                                                                   'type': 'DRB-ToReleaseList'},
                                                                                  {'members': [{'name': 'explicitValue',
                                                                                                'type': 'MAC-MainConfig'},
                                                                                               {'name': 'defaultValue',
                                                                                                'type': 'NULL'}],
                                                                                   'name': 'mac-MainConfig',
                                                                                   'optional': True,
                                                                                   'type': 'CHOICE'},
                                                                                  {'name': 'sps-Config',
                                                                                   'optional': True,
                                                                                   'type': 'SPS-Config'},
                                                                                  {'name': 'physicalConfigDedicated',
                                                                                   'optional': True,
                                                                                   'type': 'PhysicalConfigDedicated'},
                                                                                  None,
                                                                                  [{'name': 'rlf-TimersAndConstants-r9',
                                                                                    'optional': True,
                                                                                    'type': 'RLF-TimersAndConstants-r9'}],
                                                                                  [{'name': 'measSubframePatternPCell-r10',
                                                                                    'optional': True,
                                                                                    'type': 'MeasSubframePatternPCell-r10'}],
                                                                                  [{'name': 'neighCellsCRS-Info-r11',
                                                                                    'optional': True,
                                                                                    'type': 'NeighCellsCRS-Info-r11'}],
                                                                                  [{'name': 'naics-Info-r12',
                                                                                    'optional': True,
                                                                                    'type': 'NAICS-AssistanceInfo-r12'}],
                                                                                  [{'name': 'neighCellsCRS-Info-r13',
                                                                                    'optional': True,
                                                                                    'type': 'NeighCellsCRS-Info-r13'},
                                                                                   {'name': 'rlf-TimersAndConstants-r13',
                                                                                    'optional': True,
                                                                                    'type': 'RLF-TimersAndConstants-r13'}],
                                                                                  [{'name': 'sps-Config-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'SPS-Config-v1430'}]],
                                                                      'type': 'SEQUENCE'},
                                     'RadioResourceConfigDedicated-v1370': {'members': [{'name': 'physicalConfigDedicated-v1370',
                                                                                         'optional': True,
                                                                                         'type': 'PhysicalConfigDedicated-v1370'}],
                                                                            'type': 'SEQUENCE'},
                                     'RadioResourceConfigDedicatedPSCell-r12': {'members': [{'name': 'physicalConfigDedicatedPSCell-r12',
                                                                                             'optional': True,
                                                                                             'type': 'PhysicalConfigDedicated'},
                                                                                            {'name': 'sps-Config-r12',
                                                                                             'optional': True,
                                                                                             'type': 'SPS-Config'},
                                                                                            {'name': 'naics-Info-r12',
                                                                                             'optional': True,
                                                                                             'type': 'NAICS-AssistanceInfo-r12'},
                                                                                            None,
                                                                                            [{'name': 'neighCellsCRS-InfoPSCell-r13',
                                                                                              'optional': True,
                                                                                              'type': 'NeighCellsCRS-Info-r13'}],
                                                                                            [{'name': 'sps-Config-v1430',
                                                                                              'optional': True,
                                                                                              'type': 'SPS-Config-v1430'}]],
                                                                                'type': 'SEQUENCE'},
                                     'RadioResourceConfigDedicatedPSCell-v1370': {'members': [{'name': 'physicalConfigDedicatedPSCell-v1370',
                                                                                               'optional': True,
                                                                                               'type': 'PhysicalConfigDedicated-v1370'}],
                                                                                  'type': 'SEQUENCE'},
                                     'RadioResourceConfigDedicatedSCG-r12': {'members': [{'name': 'drb-ToAddModListSCG-r12',
                                                                                          'optional': True,
                                                                                          'type': 'DRB-ToAddModListSCG-r12'},
                                                                                         {'name': 'mac-MainConfigSCG-r12',
                                                                                          'optional': True,
                                                                                          'type': 'MAC-MainConfig'},
                                                                                         {'name': 'rlf-TimersAndConstantsSCG-r12',
                                                                                          'optional': True,
                                                                                          'type': 'RLF-TimersAndConstantsSCG-r12'},
                                                                                         None],
                                                                             'type': 'SEQUENCE'},
                                     'RadioResourceConfigDedicatedSCell-r10': {'members': [{'name': 'physicalConfigDedicatedSCell-r10',
                                                                                            'optional': True,
                                                                                            'type': 'PhysicalConfigDedicatedSCell-r10'},
                                                                                           None,
                                                                                           [{'name': 'mac-MainConfigSCell-r11',
                                                                                             'optional': True,
                                                                                             'type': 'MAC-MainConfigSCell-r11'}],
                                                                                           [{'name': 'naics-Info-r12',
                                                                                             'optional': True,
                                                                                             'type': 'NAICS-AssistanceInfo-r12'}],
                                                                                           [{'name': 'neighCellsCRS-InfoSCell-r13',
                                                                                             'optional': True,
                                                                                             'type': 'NeighCellsCRS-Info-r13'}],
                                                                                           [{'name': 'physicalConfigDedicatedSCell-v1370',
                                                                                             'optional': True,
                                                                                             'type': 'PhysicalConfigDedicatedSCell-v1370'}]],
                                                                               'type': 'SEQUENCE'},
                                     'RedirectedCarrierInfo': {'members': [{'name': 'eutra',
                                                                            'type': 'ARFCN-ValueEUTRA'},
                                                                           {'name': 'geran',
                                                                            'type': 'CarrierFreqsGERAN'},
                                                                           {'name': 'utra-FDD',
                                                                            'type': 'ARFCN-ValueUTRA'},
                                                                           {'name': 'utra-TDD',
                                                                            'type': 'ARFCN-ValueUTRA'},
                                                                           {'name': 'cdma2000-HRPD',
                                                                            'type': 'CarrierFreqCDMA2000'},
                                                                           {'name': 'cdma2000-1xRTT',
                                                                            'type': 'CarrierFreqCDMA2000'},
                                                                           None,
                                                                           {'name': 'utra-TDD-r10',
                                                                            'type': 'CarrierFreqListUTRA-TDD-r10'}],
                                                               'type': 'CHOICE'},
                                     'RedirectedCarrierInfo-v9e0': {'members': [{'name': 'eutra-v9e0',
                                                                                 'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                                    'type': 'SEQUENCE'},
                                     'RedistributionFactor-r13': {'restricted-to': [(1,
                                                                                     10)],
                                                                  'type': 'INTEGER'},
                                     'RedistributionInterFreqInfo-r13': {'members': [{'name': 'redistributionFactorFreq-r13',
                                                                                      'optional': True,
                                                                                      'type': 'RedistributionFactor-r13'},
                                                                                     {'name': 'redistributionNeighCellList-r13',
                                                                                      'optional': True,
                                                                                      'type': 'RedistributionNeighCellList-r13'}],
                                                                         'type': 'SEQUENCE'},
                                     'RedistributionNeighCell-r13': {'members': [{'name': 'physCellId-r13',
                                                                                  'type': 'PhysCellId'},
                                                                                 {'name': 'redistributionFactorCell-r13',
                                                                                  'type': 'RedistributionFactor-r13'}],
                                                                     'type': 'SEQUENCE'},
                                     'RedistributionNeighCellList-r13': {'element': {'type': 'RedistributionNeighCell-r13'},
                                                                         'size': [(1,
                                                                                   'maxCellInter')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'RedistributionServingInfo-r13': {'members': [{'name': 'redistributionFactorServing-r13',
                                                                                    'restricted-to': [(0,
                                                                                                       10)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'redistributionFactorCell-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]},
                                                                                   {'name': 't360-r13',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('min4',
                                                                                                0),
                                                                                               ('min8',
                                                                                                1),
                                                                                               ('min16',
                                                                                                2),
                                                                                               ('min32',
                                                                                                3),
                                                                                               ('infinity',
                                                                                                4),
                                                                                               ('spare3',
                                                                                                5),
                                                                                               ('spare2',
                                                                                                6),
                                                                                               ('spare1',
                                                                                                7)]},
                                                                                   {'name': 'redistrOnPagingOnly-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]}],
                                                                       'type': 'SEQUENCE'},
                                     'ReestabUE-Identity': {'members': [{'name': 'c-RNTI',
                                                                         'type': 'C-RNTI'},
                                                                        {'name': 'physCellId',
                                                                         'type': 'PhysCellId'},
                                                                        {'name': 'shortMAC-I',
                                                                         'type': 'ShortMAC-I'}],
                                                            'type': 'SEQUENCE'},
                                     'ReestablishmentCause': {'type': 'ENUMERATED',
                                                              'values': [('reconfigurationFailure',
                                                                          0),
                                                                         ('handoverFailure',
                                                                          1),
                                                                         ('otherFailure',
                                                                          2),
                                                                         ('spare1',
                                                                          3)]},
                                     'RegisteredMME': {'members': [{'name': 'plmn-Identity',
                                                                    'optional': True,
                                                                    'type': 'PLMN-Identity'},
                                                                   {'name': 'mmegi',
                                                                    'size': [16],
                                                                    'type': 'BIT '
                                                                            'STRING'},
                                                                   {'name': 'mmec',
                                                                    'type': 'MMEC'}],
                                                       'type': 'SEQUENCE'},
                                     'ReleaseCause': {'type': 'ENUMERATED',
                                                      'values': [('loadBalancingTAUrequired',
                                                                  0),
                                                                 ('other', 1),
                                                                 ('cs-FallbackHighPriority-v1020',
                                                                  2),
                                                                 ('rrc-Suspend-v1320',
                                                                  3)]},
                                     'ReportConfigEUTRA': {'members': [{'members': [{'members': [{'members': [{'members': [{'name': 'a1-Threshold',
                                                                                                                            'type': 'ThresholdEUTRA'}],
                                                                                                               'name': 'eventA1',
                                                                                                               'type': 'SEQUENCE'},
                                                                                                              {'members': [{'name': 'a2-Threshold',
                                                                                                                            'type': 'ThresholdEUTRA'}],
                                                                                                               'name': 'eventA2',
                                                                                                               'type': 'SEQUENCE'},
                                                                                                              {'members': [{'name': 'a3-Offset',
                                                                                                                            'restricted-to': [(-30,
                                                                                                                                               30)],
                                                                                                                            'type': 'INTEGER'},
                                                                                                                           {'name': 'reportOnLeave',
                                                                                                                            'type': 'BOOLEAN'}],
                                                                                                               'name': 'eventA3',
                                                                                                               'type': 'SEQUENCE'},
                                                                                                              {'members': [{'name': 'a4-Threshold',
                                                                                                                            'type': 'ThresholdEUTRA'}],
                                                                                                               'name': 'eventA4',
                                                                                                               'type': 'SEQUENCE'},
                                                                                                              {'members': [{'name': 'a5-Threshold1',
                                                                                                                            'type': 'ThresholdEUTRA'},
                                                                                                                           {'name': 'a5-Threshold2',
                                                                                                                            'type': 'ThresholdEUTRA'}],
                                                                                                               'name': 'eventA5',
                                                                                                               'type': 'SEQUENCE'},
                                                                                                              None,
                                                                                                              {'members': [{'name': 'a6-Offset-r10',
                                                                                                                            'restricted-to': [(-30,
                                                                                                                                               30)],
                                                                                                                            'type': 'INTEGER'},
                                                                                                                           {'name': 'a6-ReportOnLeave-r10',
                                                                                                                            'type': 'BOOLEAN'}],
                                                                                                               'name': 'eventA6-r10',
                                                                                                               'type': 'SEQUENCE'},
                                                                                                              {'members': [{'name': 'c1-Threshold-r12',
                                                                                                                            'type': 'ThresholdEUTRA-v1250'},
                                                                                                                           {'name': 'c1-ReportOnLeave-r12',
                                                                                                                            'type': 'BOOLEAN'}],
                                                                                                               'name': 'eventC1-r12',
                                                                                                               'type': 'SEQUENCE'},
                                                                                                              {'members': [{'name': 'c2-RefCSI-RS-r12',
                                                                                                                            'type': 'MeasCSI-RS-Id-r12'},
                                                                                                                           {'name': 'c2-Offset-r12',
                                                                                                                            'restricted-to': [(-30,
                                                                                                                                               30)],
                                                                                                                            'type': 'INTEGER'},
                                                                                                                           {'name': 'c2-ReportOnLeave-r12',
                                                                                                                            'type': 'BOOLEAN'}],
                                                                                                               'name': 'eventC2-r12',
                                                                                                               'type': 'SEQUENCE'},
                                                                                                              {'members': [{'name': 'v1-Threshold-r14',
                                                                                                                            'type': 'SL-CBR-r14'}],
                                                                                                               'name': 'eventV1-r14',
                                                                                                               'type': 'SEQUENCE'},
                                                                                                              {'members': [{'name': 'v2-Threshold-r14',
                                                                                                                            'type': 'SL-CBR-r14'}],
                                                                                                               'name': 'eventV2-r14',
                                                                                                               'type': 'SEQUENCE'}],
                                                                                                  'name': 'eventId',
                                                                                                  'type': 'CHOICE'},
                                                                                                 {'name': 'hysteresis',
                                                                                                  'type': 'Hysteresis'},
                                                                                                 {'name': 'timeToTrigger',
                                                                                                  'type': 'TimeToTrigger'}],
                                                                                     'name': 'event',
                                                                                     'type': 'SEQUENCE'},
                                                                                    {'members': [{'name': 'purpose',
                                                                                                  'type': 'ENUMERATED',
                                                                                                  'values': [('reportStrongestCells',
                                                                                                              0),
                                                                                                             ('reportCGI',
                                                                                                              1)]}],
                                                                                     'name': 'periodical',
                                                                                     'type': 'SEQUENCE'}],
                                                                        'name': 'triggerType',
                                                                        'type': 'CHOICE'},
                                                                       {'name': 'triggerQuantity',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('rsrp',
                                                                                    0),
                                                                                   ('rsrq',
                                                                                    1)]},
                                                                       {'name': 'reportQuantity',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('sameAsTriggerQuantity',
                                                                                    0),
                                                                                   ('both',
                                                                                    1)]},
                                                                       {'name': 'maxReportCells',
                                                                        'restricted-to': [(1,
                                                                                           'maxCellReport')],
                                                                        'type': 'INTEGER'},
                                                                       {'name': 'reportInterval',
                                                                        'type': 'ReportInterval'},
                                                                       {'name': 'reportAmount',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('r1',
                                                                                    0),
                                                                                   ('r2',
                                                                                    1),
                                                                                   ('r4',
                                                                                    2),
                                                                                   ('r8',
                                                                                    3),
                                                                                   ('r16',
                                                                                    4),
                                                                                   ('r32',
                                                                                    5),
                                                                                   ('r64',
                                                                                    6),
                                                                                   ('infinity',
                                                                                    7)]},
                                                                       None,
                                                                       [{'name': 'si-RequestForHO-r9',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('setup',
                                                                                     0)]},
                                                                        {'name': 'ue-RxTxTimeDiffPeriodical-r9',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('setup',
                                                                                     0)]}],
                                                                       [{'name': 'includeLocationInfo-r10',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('true',
                                                                                     0)]},
                                                                        {'name': 'reportAddNeighMeas-r10',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('setup',
                                                                                     0)]}],
                                                                       [{'members': [{'name': 'release',
                                                                                      'type': 'NULL'},
                                                                                     {'name': 'setup',
                                                                                      'type': 'TimeToTrigger'}],
                                                                         'name': 'alternativeTimeToTrigger-r12',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'},
                                                                        {'name': 'useT312-r12',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'},
                                                                        {'name': 'usePSCell-r12',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'},
                                                                        {'name': 'aN-Threshold1-v1250',
                                                                         'optional': True,
                                                                         'type': 'RSRQ-RangeConfig-r12'},
                                                                        {'name': 'a5-Threshold2-v1250',
                                                                         'optional': True,
                                                                         'type': 'RSRQ-RangeConfig-r12'},
                                                                        {'name': 'reportStrongestCSI-RSs-r12',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'},
                                                                        {'name': 'reportCRS-Meas-r12',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'},
                                                                        {'name': 'triggerQuantityCSI-RS-r12',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'}],
                                                                       [{'name': 'reportSSTD-Meas-r13',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'},
                                                                        {'members': [{'name': 'release',
                                                                                      'type': 'NULL'},
                                                                                     {'members': [{'name': 'triggerQuantity-v1310',
                                                                                                   'optional': True,
                                                                                                   'type': 'ENUMERATED',
                                                                                                   'values': [('sinr',
                                                                                                               0)]},
                                                                                                  {'name': 'aN-Threshold1-r13',
                                                                                                   'optional': True,
                                                                                                   'type': 'RS-SINR-Range-r13'},
                                                                                                  {'name': 'a5-Threshold2-r13',
                                                                                                   'optional': True,
                                                                                                   'type': 'RS-SINR-Range-r13'},
                                                                                                  {'name': 'reportQuantity-v1310',
                                                                                                   'type': 'ENUMERATED',
                                                                                                   'values': [('rsrpANDsinr',
                                                                                                               0),
                                                                                                              ('rsrqANDsinr',
                                                                                                               1),
                                                                                                              ('all',
                                                                                                               2)]}],
                                                                                      'name': 'setup',
                                                                                      'type': 'SEQUENCE'}],
                                                                         'name': 'rs-sinr-Config-r13',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'},
                                                                        {'name': 'useWhiteCellList-r13',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'},
                                                                        {'name': 'measRSSI-ReportConfig-r13',
                                                                         'optional': True,
                                                                         'type': 'MeasRSSI-ReportConfig-r13'},
                                                                        {'name': 'includeMultiBandInfo-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('true',
                                                                                     0)]},
                                                                        {'name': 'ul-DelayConfig-r13',
                                                                         'optional': True,
                                                                         'type': 'UL-DelayConfig-r13'}],
                                                                       [{'name': 'ue-RxTxTimeDiffPeriodicalTDD-r13',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'}],
                                                                       [{'name': 'purpose-v1430',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('reportLocation',
                                                                                     0),
                                                                                    ('sidelink',
                                                                                     1),
                                                                                    ('spare2',
                                                                                     2),
                                                                                    ('spare1',
                                                                                     3)]}]],
                                                           'type': 'SEQUENCE'},
                                     'ReportConfigId': {'restricted-to': [(1,
                                                                           'maxReportConfigId')],
                                                        'type': 'INTEGER'},
                                     'ReportConfigInterRAT': {'members': [{'members': [{'members': [{'members': [{'members': [{'members': [{'name': 'b1-ThresholdUTRA',
                                                                                                                                            'type': 'ThresholdUTRA'},
                                                                                                                                           {'name': 'b1-ThresholdGERAN',
                                                                                                                                            'type': 'ThresholdGERAN'},
                                                                                                                                           {'name': 'b1-ThresholdCDMA2000',
                                                                                                                                            'type': 'ThresholdCDMA2000'}],
                                                                                                                               'name': 'b1-Threshold',
                                                                                                                               'type': 'CHOICE'}],
                                                                                                                  'name': 'eventB1',
                                                                                                                  'type': 'SEQUENCE'},
                                                                                                                 {'members': [{'name': 'b2-Threshold1',
                                                                                                                               'type': 'ThresholdEUTRA'},
                                                                                                                              {'members': [{'name': 'b2-Threshold2UTRA',
                                                                                                                                            'type': 'ThresholdUTRA'},
                                                                                                                                           {'name': 'b2-Threshold2GERAN',
                                                                                                                                            'type': 'ThresholdGERAN'},
                                                                                                                                           {'name': 'b2-Threshold2CDMA2000',
                                                                                                                                            'type': 'ThresholdCDMA2000'}],
                                                                                                                               'name': 'b2-Threshold2',
                                                                                                                               'type': 'CHOICE'}],
                                                                                                                  'name': 'eventB2',
                                                                                                                  'type': 'SEQUENCE'},
                                                                                                                 None,
                                                                                                                 {'members': [{'name': 'w1-Threshold-r13',
                                                                                                                               'type': 'WLAN-RSSI-Range-r13'}],
                                                                                                                  'name': 'eventW1-r13',
                                                                                                                  'type': 'SEQUENCE'},
                                                                                                                 {'members': [{'name': 'w2-Threshold1-r13',
                                                                                                                               'type': 'WLAN-RSSI-Range-r13'},
                                                                                                                              {'name': 'w2-Threshold2-r13',
                                                                                                                               'type': 'WLAN-RSSI-Range-r13'}],
                                                                                                                  'name': 'eventW2-r13',
                                                                                                                  'type': 'SEQUENCE'},
                                                                                                                 {'members': [{'name': 'w3-Threshold-r13',
                                                                                                                               'type': 'WLAN-RSSI-Range-r13'}],
                                                                                                                  'name': 'eventW3-r13',
                                                                                                                  'type': 'SEQUENCE'}],
                                                                                                     'name': 'eventId',
                                                                                                     'type': 'CHOICE'},
                                                                                                    {'name': 'hysteresis',
                                                                                                     'type': 'Hysteresis'},
                                                                                                    {'name': 'timeToTrigger',
                                                                                                     'type': 'TimeToTrigger'}],
                                                                                        'name': 'event',
                                                                                        'type': 'SEQUENCE'},
                                                                                       {'members': [{'name': 'purpose',
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('reportStrongestCells',
                                                                                                                 0),
                                                                                                                ('reportStrongestCellsForSON',
                                                                                                                 1),
                                                                                                                ('reportCGI',
                                                                                                                 2)]}],
                                                                                        'name': 'periodical',
                                                                                        'type': 'SEQUENCE'}],
                                                                           'name': 'triggerType',
                                                                           'type': 'CHOICE'},
                                                                          {'name': 'maxReportCells',
                                                                           'restricted-to': [(1,
                                                                                              'maxCellReport')],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'reportInterval',
                                                                           'type': 'ReportInterval'},
                                                                          {'name': 'reportAmount',
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('r1',
                                                                                       0),
                                                                                      ('r2',
                                                                                       1),
                                                                                      ('r4',
                                                                                       2),
                                                                                      ('r8',
                                                                                       3),
                                                                                      ('r16',
                                                                                       4),
                                                                                      ('r32',
                                                                                       5),
                                                                                      ('r64',
                                                                                       6),
                                                                                      ('infinity',
                                                                                       7)]},
                                                                          None,
                                                                          [{'name': 'si-RequestForHO-r9',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('setup',
                                                                                        0)]}],
                                                                          [{'name': 'reportQuantityUTRA-FDD-r10',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('both',
                                                                                        0)]}],
                                                                          [{'name': 'includeLocationInfo-r11',
                                                                            'optional': True,
                                                                            'type': 'BOOLEAN'}],
                                                                          [{'members': [{'name': 'release',
                                                                                         'type': 'NULL'},
                                                                                        {'name': 'setup',
                                                                                         'type': 'RSRQ-Range-v1250'}],
                                                                            'name': 'b2-Threshold1-v1250',
                                                                            'optional': True,
                                                                            'type': 'CHOICE'}],
                                                                          [{'name': 'reportQuantityWLAN-r13',
                                                                            'optional': True,
                                                                            'type': 'ReportQuantityWLAN-r13'}],
                                                                          [{'name': 'reportAnyWLAN-r14',
                                                                            'optional': True,
                                                                            'type': 'BOOLEAN'}]],
                                                              'type': 'SEQUENCE'},
                                     'ReportConfigToAddMod': {'members': [{'name': 'reportConfigId',
                                                                           'type': 'ReportConfigId'},
                                                                          {'members': [{'name': 'reportConfigEUTRA',
                                                                                        'type': 'ReportConfigEUTRA'},
                                                                                       {'name': 'reportConfigInterRAT',
                                                                                        'type': 'ReportConfigInterRAT'}],
                                                                           'name': 'reportConfig',
                                                                           'type': 'CHOICE'}],
                                                              'type': 'SEQUENCE'},
                                     'ReportConfigToAddModList': {'element': {'type': 'ReportConfigToAddMod'},
                                                                  'size': [(1,
                                                                            'maxReportConfigId')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'ReportConfigToRemoveList': {'element': {'type': 'ReportConfigId'},
                                                                  'size': [(1,
                                                                            'maxReportConfigId')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'ReportInterval': {'type': 'ENUMERATED',
                                                        'values': [('ms120', 0),
                                                                   ('ms240', 1),
                                                                   ('ms480', 2),
                                                                   ('ms640', 3),
                                                                   ('ms1024',
                                                                    4),
                                                                   ('ms2048',
                                                                    5),
                                                                   ('ms5120',
                                                                    6),
                                                                   ('ms10240',
                                                                    7),
                                                                   ('min1', 8),
                                                                   ('min6', 9),
                                                                   ('min12',
                                                                    10),
                                                                   ('min30',
                                                                    11),
                                                                   ('min60',
                                                                    12),
                                                                   ('spare3',
                                                                    13),
                                                                   ('spare2',
                                                                    14),
                                                                   ('spare1',
                                                                    15)]},
                                     'ReportProximityConfig-r9': {'members': [{'name': 'proximityIndicationEUTRA-r9',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('enabled',
                                                                                           0)]},
                                                                              {'name': 'proximityIndicationUTRA-r9',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('enabled',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'ReportQuantityWLAN-r13': {'members': [{'name': 'bandRequestWLAN-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('true',
                                                                                         0)]},
                                                                            {'name': 'carrierInfoRequestWLAN-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('true',
                                                                                         0)]},
                                                                            {'name': 'availableAdmissionCapacityRequestWLAN-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('true',
                                                                                         0)]},
                                                                            {'name': 'backhaulDL-BandwidthRequestWLAN-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('true',
                                                                                         0)]},
                                                                            {'name': 'backhaulUL-BandwidthRequestWLAN-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('true',
                                                                                         0)]},
                                                                            {'name': 'channelUtilizationRequestWLAN-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('true',
                                                                                         0)]},
                                                                            {'name': 'stationCountRequestWLAN-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('true',
                                                                                         0)]},
                                                                            None],
                                                                'type': 'SEQUENCE'},
                                     'ReselectionInfoRelay-r13': {'members': [{'name': 'q-RxLevMin-r13',
                                                                               'type': 'Q-RxLevMin'},
                                                                              {'name': 'filterCoefficient-r13',
                                                                               'type': 'FilterCoefficient'},
                                                                              {'name': 'minHyst-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('dB0',
                                                                                           0),
                                                                                          ('dB3',
                                                                                           1),
                                                                                          ('dB6',
                                                                                           2),
                                                                                          ('dB9',
                                                                                           3),
                                                                                          ('dB12',
                                                                                           4),
                                                                                          ('dBinf',
                                                                                           5)]}],
                                                                  'type': 'SEQUENCE'},
                                     'ReselectionThreshold': {'restricted-to': [(0,
                                                                                 31)],
                                                              'type': 'INTEGER'},
                                     'ReselectionThresholdQ-r9': {'restricted-to': [(0,
                                                                                     31)],
                                                                  'type': 'INTEGER'},
                                     'ResourceConfig-r13': {'restricted-to': [(0,
                                                                               31)],
                                                            'type': 'INTEGER'},
                                     'ResumeCause': {'type': 'ENUMERATED',
                                                     'values': [('emergency',
                                                                 0),
                                                                ('highPriorityAccess',
                                                                 1),
                                                                ('mt-Access',
                                                                 2),
                                                                ('mo-Signalling',
                                                                 3),
                                                                ('mo-Data', 4),
                                                                ('delayTolerantAccess-v1020',
                                                                 5),
                                                                ('mo-VoiceCall-v1280',
                                                                 6),
                                                                ('spare1', 7)]},
                                     'ResumeIdentity-r13': {'size': [40],
                                                            'type': 'BIT '
                                                                    'STRING'},
                                     'RetuningTimeInfo-r14': {'members': [{'members': [{'name': 'rf-RetuningTimeDL-r14',
                                                                                        'optional': True,
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('n0',
                                                                                                    0),
                                                                                                   ('n0dot5',
                                                                                                    1),
                                                                                                   ('n1',
                                                                                                    2),
                                                                                                   ('n1dot5',
                                                                                                    3),
                                                                                                   ('n2',
                                                                                                    4),
                                                                                                   ('n2dot5',
                                                                                                    5),
                                                                                                   ('n3',
                                                                                                    6),
                                                                                                   ('n3dot5',
                                                                                                    7),
                                                                                                   ('n4',
                                                                                                    8),
                                                                                                   ('n4dot5',
                                                                                                    9),
                                                                                                   ('n5',
                                                                                                    10),
                                                                                                   ('n5dot5',
                                                                                                    11),
                                                                                                   ('n6',
                                                                                                    12),
                                                                                                   ('n6dot5',
                                                                                                    13),
                                                                                                   ('n7',
                                                                                                    14),
                                                                                                   ('spare1',
                                                                                                    15)]},
                                                                                       {'name': 'rf-RetuningTimeUL-r14',
                                                                                        'optional': True,
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('n0',
                                                                                                    0),
                                                                                                   ('n0dot5',
                                                                                                    1),
                                                                                                   ('n1',
                                                                                                    2),
                                                                                                   ('n1dot5',
                                                                                                    3),
                                                                                                   ('n2',
                                                                                                    4),
                                                                                                   ('n2dot5',
                                                                                                    5),
                                                                                                   ('n3',
                                                                                                    6),
                                                                                                   ('n3dot5',
                                                                                                    7),
                                                                                                   ('n4',
                                                                                                    8),
                                                                                                   ('n4dot5',
                                                                                                    9),
                                                                                                   ('n5',
                                                                                                    10),
                                                                                                   ('n5dot5',
                                                                                                    11),
                                                                                                   ('n6',
                                                                                                    12),
                                                                                                   ('n6dot5',
                                                                                                    13),
                                                                                                   ('n7',
                                                                                                    14),
                                                                                                   ('spare1',
                                                                                                    15)]}],
                                                                           'name': 'retuningInfo',
                                                                           'type': 'SEQUENCE'}],
                                                              'type': 'SEQUENCE'},
                                     'RetxBSR-Timer-r12': {'type': 'ENUMERATED',
                                                           'values': [('sf320',
                                                                       0),
                                                                      ('sf640',
                                                                       1),
                                                                      ('sf1280',
                                                                       2),
                                                                      ('sf2560',
                                                                       3),
                                                                      ('sf5120',
                                                                       4),
                                                                      ('sf10240',
                                                                       5),
                                                                      ('spare2',
                                                                       6),
                                                                      ('spare1',
                                                                       7)]},
                                     'S-TMSI': {'members': [{'name': 'mmec',
                                                             'type': 'MMEC'},
                                                            {'name': 'm-TMSI',
                                                             'size': [32],
                                                             'type': 'BIT '
                                                                     'STRING'}],
                                                'type': 'SEQUENCE'},
                                     'SC-MCCH-Message-r13': {'members': [{'name': 'message',
                                                                          'type': 'SC-MCCH-MessageType-r13'}],
                                                             'type': 'SEQUENCE'},
                                     'SC-MCCH-MessageType-r13': {'members': [{'members': [{'name': 'scptmConfiguration-r13',
                                                                                           'type': 'SCPTMConfiguration-r13'}],
                                                                              'name': 'c1',
                                                                              'type': 'CHOICE'},
                                                                             {'members': [{'members': [{'name': 'scptmConfiguration-BR-r14',
                                                                                                        'type': 'SCPTMConfiguration-BR-r14'},
                                                                                                       {'name': 'spare',
                                                                                                        'type': 'NULL'}],
                                                                                           'name': 'c2',
                                                                                           'type': 'CHOICE'},
                                                                                          {'members': [],
                                                                                           'name': 'messageClassExtensionFuture-r14',
                                                                                           'type': 'SEQUENCE'}],
                                                                              'name': 'messageClassExtension',
                                                                              'type': 'CHOICE'}],
                                                                 'type': 'CHOICE'},
                                     'SC-MCCH-SchedulingInfo-r14': {'members': [{'name': 'onDurationTimerSCPTM-r14',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('psf10',
                                                                                             0),
                                                                                            ('psf20',
                                                                                             1),
                                                                                            ('psf100',
                                                                                             2),
                                                                                            ('psf300',
                                                                                             3),
                                                                                            ('psf500',
                                                                                             4),
                                                                                            ('psf1000',
                                                                                             5),
                                                                                            ('psf1200',
                                                                                             6),
                                                                                            ('psf1600',
                                                                                             7)]},
                                                                                {'name': 'drx-InactivityTimerSCPTM-r14',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('psf0',
                                                                                             0),
                                                                                            ('psf1',
                                                                                             1),
                                                                                            ('psf2',
                                                                                             2),
                                                                                            ('psf4',
                                                                                             3),
                                                                                            ('psf8',
                                                                                             4),
                                                                                            ('psf16',
                                                                                             5),
                                                                                            ('psf32',
                                                                                             6),
                                                                                            ('psf64',
                                                                                             7),
                                                                                            ('psf128',
                                                                                             8),
                                                                                            ('psf256',
                                                                                             9),
                                                                                            ('ps512',
                                                                                             10),
                                                                                            ('psf1024',
                                                                                             11),
                                                                                            ('psf2048',
                                                                                             12),
                                                                                            ('psf4096',
                                                                                             13),
                                                                                            ('psf8192',
                                                                                             14),
                                                                                            ('psf16384',
                                                                                             15)]},
                                                                                {'members': [{'name': 'sf10',
                                                                                              'restricted-to': [(0,
                                                                                                                 9)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf20',
                                                                                              'restricted-to': [(0,
                                                                                                                 19)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf32',
                                                                                              'restricted-to': [(0,
                                                                                                                 31)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf40',
                                                                                              'restricted-to': [(0,
                                                                                                                 39)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf64',
                                                                                              'restricted-to': [(0,
                                                                                                                 63)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf80',
                                                                                              'restricted-to': [(0,
                                                                                                                 79)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf128',
                                                                                              'restricted-to': [(0,
                                                                                                                 127)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf160',
                                                                                              'restricted-to': [(0,
                                                                                                                 159)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf256',
                                                                                              'restricted-to': [(0,
                                                                                                                 255)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf320',
                                                                                              'restricted-to': [(0,
                                                                                                                 319)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf512',
                                                                                              'restricted-to': [(0,
                                                                                                                 511)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf640',
                                                                                              'restricted-to': [(0,
                                                                                                                 639)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf1024',
                                                                                              'restricted-to': [(0,
                                                                                                                 1023)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf2048',
                                                                                              'restricted-to': [(0,
                                                                                                                 2047)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf4096',
                                                                                              'restricted-to': [(0,
                                                                                                                 4095)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf8192',
                                                                                              'restricted-to': [(0,
                                                                                                                 8191)],
                                                                                              'type': 'INTEGER'}],
                                                                                 'name': 'schedulingPeriodStartOffsetSCPTM-r14',
                                                                                 'type': 'CHOICE'},
                                                                                None],
                                                                    'type': 'SEQUENCE'},
                                     'SC-MTCH-Info-BR-r14': {'members': [{'name': 'sc-mtch-CarrierFreq-r14',
                                                                          'type': 'ARFCN-ValueEUTRA-r9'},
                                                                         {'name': 'mbmsSessionInfo-r14',
                                                                          'type': 'MBMSSessionInfo-r13'},
                                                                         {'name': 'g-RNTI-r14',
                                                                          'size': [16],
                                                                          'type': 'BIT '
                                                                                  'STRING'},
                                                                         {'name': 'sc-mtch-schedulingInfo-r14',
                                                                          'optional': True,
                                                                          'type': 'SC-MTCH-SchedulingInfo-BR-r14'},
                                                                         {'name': 'sc-mtch-neighbourCell-r14',
                                                                          'optional': True,
                                                                          'size': ['maxNeighCell-SCPTM-r13'],
                                                                          'type': 'BIT '
                                                                                  'STRING'},
                                                                         {'name': 'mpdcch-Narrowband-SC-MTCH-r14',
                                                                          'restricted-to': [(1,
                                                                                             'maxAvailNarrowBands-r13')],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'mpdcch-NumRepetition-SC-MTCH-r14',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('r1',
                                                                                      0),
                                                                                     ('r2',
                                                                                      1),
                                                                                     ('r4',
                                                                                      2),
                                                                                     ('r8',
                                                                                      3),
                                                                                     ('r16',
                                                                                      4),
                                                                                     ('r32',
                                                                                      5),
                                                                                     ('r64',
                                                                                      6),
                                                                                     ('r128',
                                                                                      7),
                                                                                     ('r256',
                                                                                      8)]},
                                                                         {'members': [{'name': 'fdd-r14',
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('v1',
                                                                                                   0),
                                                                                                  ('v1dot5',
                                                                                                   1),
                                                                                                  ('v2',
                                                                                                   2),
                                                                                                  ('v2dot5',
                                                                                                   3),
                                                                                                  ('v4',
                                                                                                   4),
                                                                                                  ('v5',
                                                                                                   5),
                                                                                                  ('v8',
                                                                                                   6),
                                                                                                  ('v10',
                                                                                                   7)]},
                                                                                      {'name': 'tdd-r14',
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('v1',
                                                                                                   0),
                                                                                                  ('v2',
                                                                                                   1),
                                                                                                  ('v4',
                                                                                                   2),
                                                                                                  ('v5',
                                                                                                   3),
                                                                                                  ('v8',
                                                                                                   4),
                                                                                                  ('v10',
                                                                                                   5),
                                                                                                  ('v20',
                                                                                                   6)]}],
                                                                          'name': 'mpdcch-StartSF-SC-MTCH-r14',
                                                                          'type': 'CHOICE'},
                                                                         {'name': 'mpdcch-PDSCH-HoppingConfig-SC-MTCH-r14',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('on',
                                                                                      0),
                                                                                     ('off',
                                                                                      1)]},
                                                                         {'name': 'mpdcch-PDSCH-CEmodeConfig-SC-MTCH-r14',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('ce-ModeA',
                                                                                      0),
                                                                                     ('ce-ModeB',
                                                                                      1)]},
                                                                         {'name': 'mpdcch-PDSCH-MaxBandwidth-SC-MTCH-r14',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('bw1dot4',
                                                                                      0),
                                                                                     ('bw5',
                                                                                      1)]},
                                                                         {'name': 'mpdcch-Offset-SC-MTCH-r14',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('zero',
                                                                                      0),
                                                                                     ('oneEighth',
                                                                                      1),
                                                                                     ('oneQuarter',
                                                                                      2),
                                                                                     ('threeEighth',
                                                                                      3),
                                                                                     ('oneHalf',
                                                                                      4),
                                                                                     ('fiveEighth',
                                                                                      5),
                                                                                     ('threeQuarter',
                                                                                      6),
                                                                                     ('sevenEighth',
                                                                                      7)]},
                                                                         {'name': 'p-a-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('dB-6',
                                                                                      0),
                                                                                     ('dB-4dot77',
                                                                                      1),
                                                                                     ('dB-3',
                                                                                      2),
                                                                                     ('dB-1dot77',
                                                                                      3),
                                                                                     ('dB0',
                                                                                      4),
                                                                                     ('dB1',
                                                                                      5),
                                                                                     ('dB2',
                                                                                      6),
                                                                                     ('dB3',
                                                                                      7)]},
                                                                         None],
                                                             'type': 'SEQUENCE'},
                                     'SC-MTCH-Info-r13': {'members': [{'name': 'mbmsSessionInfo-r13',
                                                                       'type': 'MBMSSessionInfo-r13'},
                                                                      {'name': 'g-RNTI-r13',
                                                                       'size': [16],
                                                                       'type': 'BIT '
                                                                               'STRING'},
                                                                      {'name': 'sc-mtch-schedulingInfo-r13',
                                                                       'optional': True,
                                                                       'type': 'SC-MTCH-SchedulingInfo-r13'},
                                                                      {'name': 'sc-mtch-neighbourCell-r13',
                                                                       'optional': True,
                                                                       'size': ['maxNeighCell-SCPTM-r13'],
                                                                       'type': 'BIT '
                                                                               'STRING'},
                                                                      None,
                                                                      [{'name': 'p-a-r13',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('dB-6',
                                                                                    0),
                                                                                   ('dB-4dot77',
                                                                                    1),
                                                                                   ('dB-3',
                                                                                    2),
                                                                                   ('dB-1dot77',
                                                                                    3),
                                                                                   ('dB0',
                                                                                    4),
                                                                                   ('dB1',
                                                                                    5),
                                                                                   ('dB2',
                                                                                    6),
                                                                                   ('dB3',
                                                                                    7)]}]],
                                                          'type': 'SEQUENCE'},
                                     'SC-MTCH-InfoList-BR-r14': {'element': {'type': 'SC-MTCH-Info-BR-r14'},
                                                                 'size': [(0,
                                                                           'maxSC-MTCH-BR-r14')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'SC-MTCH-InfoList-r13': {'element': {'type': 'SC-MTCH-Info-r13'},
                                                              'size': [(0,
                                                                        'maxSC-MTCH-r13')],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'SC-MTCH-SchedulingInfo-BR-r14': {'members': [{'name': 'onDurationTimerSCPTM-r14',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('psf300',
                                                                                                0),
                                                                                               ('psf400',
                                                                                                1),
                                                                                               ('psf500',
                                                                                                2),
                                                                                               ('psf600',
                                                                                                3),
                                                                                               ('psf800',
                                                                                                4),
                                                                                               ('psf1000',
                                                                                                5),
                                                                                               ('psf1200',
                                                                                                6),
                                                                                               ('psf1600',
                                                                                                7)]},
                                                                                   {'name': 'drx-InactivityTimerSCPTM-r14',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('psf0',
                                                                                                0),
                                                                                               ('psf1',
                                                                                                1),
                                                                                               ('psf2',
                                                                                                2),
                                                                                               ('psf4',
                                                                                                3),
                                                                                               ('psf8',
                                                                                                4),
                                                                                               ('psf16',
                                                                                                5),
                                                                                               ('psf32',
                                                                                                6),
                                                                                               ('psf64',
                                                                                                7),
                                                                                               ('psf128',
                                                                                                8),
                                                                                               ('psf256',
                                                                                                9),
                                                                                               ('ps512',
                                                                                                10),
                                                                                               ('psf1024',
                                                                                                11),
                                                                                               ('psf2048',
                                                                                                12),
                                                                                               ('psf4096',
                                                                                                13),
                                                                                               ('psf8192',
                                                                                                14),
                                                                                               ('psf16384',
                                                                                                15)]},
                                                                                   {'members': [{'name': 'sf10',
                                                                                                 'restricted-to': [(0,
                                                                                                                    9)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf20',
                                                                                                 'restricted-to': [(0,
                                                                                                                    19)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf32',
                                                                                                 'restricted-to': [(0,
                                                                                                                    31)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf40',
                                                                                                 'restricted-to': [(0,
                                                                                                                    39)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf64',
                                                                                                 'restricted-to': [(0,
                                                                                                                    63)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf80',
                                                                                                 'restricted-to': [(0,
                                                                                                                    79)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf128',
                                                                                                 'restricted-to': [(0,
                                                                                                                    127)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf160',
                                                                                                 'restricted-to': [(0,
                                                                                                                    159)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf256',
                                                                                                 'restricted-to': [(0,
                                                                                                                    255)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf320',
                                                                                                 'restricted-to': [(0,
                                                                                                                    319)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf512',
                                                                                                 'restricted-to': [(0,
                                                                                                                    511)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf640',
                                                                                                 'restricted-to': [(0,
                                                                                                                    639)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf1024',
                                                                                                 'restricted-to': [(0,
                                                                                                                    1023)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf2048',
                                                                                                 'restricted-to': [(0,
                                                                                                                    2047)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf4096',
                                                                                                 'restricted-to': [(0,
                                                                                                                    4095)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf8192',
                                                                                                 'restricted-to': [(0,
                                                                                                                    8191)],
                                                                                                 'type': 'INTEGER'}],
                                                                                    'name': 'schedulingPeriodStartOffsetSCPTM-r14',
                                                                                    'type': 'CHOICE'},
                                                                                   None],
                                                                       'type': 'SEQUENCE'},
                                     'SC-MTCH-SchedulingInfo-r13': {'members': [{'name': 'onDurationTimerSCPTM-r13',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('psf1',
                                                                                             0),
                                                                                            ('psf2',
                                                                                             1),
                                                                                            ('psf3',
                                                                                             2),
                                                                                            ('psf4',
                                                                                             3),
                                                                                            ('psf5',
                                                                                             4),
                                                                                            ('psf6',
                                                                                             5),
                                                                                            ('psf8',
                                                                                             6),
                                                                                            ('psf10',
                                                                                             7),
                                                                                            ('psf20',
                                                                                             8),
                                                                                            ('psf30',
                                                                                             9),
                                                                                            ('psf40',
                                                                                             10),
                                                                                            ('psf50',
                                                                                             11),
                                                                                            ('psf60',
                                                                                             12),
                                                                                            ('psf80',
                                                                                             13),
                                                                                            ('psf100',
                                                                                             14),
                                                                                            ('psf200',
                                                                                             15)]},
                                                                                {'name': 'drx-InactivityTimerSCPTM-r13',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('psf0',
                                                                                             0),
                                                                                            ('psf1',
                                                                                             1),
                                                                                            ('psf2',
                                                                                             2),
                                                                                            ('psf4',
                                                                                             3),
                                                                                            ('psf8',
                                                                                             4),
                                                                                            ('psf10',
                                                                                             5),
                                                                                            ('psf20',
                                                                                             6),
                                                                                            ('psf40',
                                                                                             7),
                                                                                            ('psf80',
                                                                                             8),
                                                                                            ('psf160',
                                                                                             9),
                                                                                            ('ps320',
                                                                                             10),
                                                                                            ('psf640',
                                                                                             11),
                                                                                            ('psf960',
                                                                                             12),
                                                                                            ('psf1280',
                                                                                             13),
                                                                                            ('psf1920',
                                                                                             14),
                                                                                            ('psf2560',
                                                                                             15)]},
                                                                                {'members': [{'name': 'sf10',
                                                                                              'restricted-to': [(0,
                                                                                                                 9)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf20',
                                                                                              'restricted-to': [(0,
                                                                                                                 19)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf32',
                                                                                              'restricted-to': [(0,
                                                                                                                 31)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf40',
                                                                                              'restricted-to': [(0,
                                                                                                                 39)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf64',
                                                                                              'restricted-to': [(0,
                                                                                                                 63)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf80',
                                                                                              'restricted-to': [(0,
                                                                                                                 79)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf128',
                                                                                              'restricted-to': [(0,
                                                                                                                 127)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf160',
                                                                                              'restricted-to': [(0,
                                                                                                                 159)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf256',
                                                                                              'restricted-to': [(0,
                                                                                                                 255)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf320',
                                                                                              'restricted-to': [(0,
                                                                                                                 319)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf512',
                                                                                              'restricted-to': [(0,
                                                                                                                 511)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf640',
                                                                                              'restricted-to': [(0,
                                                                                                                 639)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf1024',
                                                                                              'restricted-to': [(0,
                                                                                                                 1023)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf2048',
                                                                                              'restricted-to': [(0,
                                                                                                                 2048)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf4096',
                                                                                              'restricted-to': [(0,
                                                                                                                 4096)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sf8192',
                                                                                              'restricted-to': [(0,
                                                                                                                 8192)],
                                                                                              'type': 'INTEGER'}],
                                                                                 'name': 'schedulingPeriodStartOffsetSCPTM-r13',
                                                                                 'type': 'CHOICE'},
                                                                                None],
                                                                    'type': 'SEQUENCE'},
                                     'SCG-ConfigPartSCG-r12': {'members': [{'name': 'radioResourceConfigDedicatedSCG-r12',
                                                                            'optional': True,
                                                                            'type': 'RadioResourceConfigDedicatedSCG-r12'},
                                                                           {'name': 'sCellToReleaseListSCG-r12',
                                                                            'optional': True,
                                                                            'type': 'SCellToReleaseList-r10'},
                                                                           {'name': 'pSCellToAddMod-r12',
                                                                            'optional': True,
                                                                            'type': 'PSCellToAddMod-r12'},
                                                                           {'name': 'sCellToAddModListSCG-r12',
                                                                            'optional': True,
                                                                            'type': 'SCellToAddModList-r10'},
                                                                           {'name': 'mobilityControlInfoSCG-r12',
                                                                            'optional': True,
                                                                            'type': 'MobilityControlInfoSCG-r12'},
                                                                           None,
                                                                           [{'name': 'sCellToReleaseListSCG-Ext-r13',
                                                                             'optional': True,
                                                                             'type': 'SCellToReleaseListExt-r13'},
                                                                            {'name': 'sCellToAddModListSCG-Ext-r13',
                                                                             'optional': True,
                                                                             'type': 'SCellToAddModListExt-r13'}],
                                                                           [{'name': 'sCellToAddModListSCG-Ext-v1370',
                                                                             'optional': True,
                                                                             'type': 'SCellToAddModListExt-v1370'}],
                                                                           [{'name': 'pSCellToAddMod-v1440',
                                                                             'optional': True,
                                                                             'type': 'PSCellToAddMod-v1440'}]],
                                                               'type': 'SEQUENCE'},
                                     'SCG-ConfigPartSCG-v12f0': {'members': [{'name': 'pSCellToAddMod-v12f0',
                                                                              'optional': True,
                                                                              'type': 'PSCellToAddMod-v12f0'},
                                                                             {'name': 'sCellToAddModListSCG-v12f0',
                                                                              'optional': True,
                                                                              'type': 'SCellToAddModList-v10l0'}],
                                                                 'type': 'SEQUENCE'},
                                     'SCG-Configuration-r12': {'members': [{'name': 'release',
                                                                            'type': 'NULL'},
                                                                           {'members': [{'members': [{'name': 'scg-Counter-r12',
                                                                                                      'optional': True,
                                                                                                      'restricted-to': [(0,
                                                                                                                         65535)],
                                                                                                      'type': 'INTEGER'},
                                                                                                     {'name': 'powerCoordinationInfo-r12',
                                                                                                      'optional': True,
                                                                                                      'type': 'PowerCoordinationInfo-r12'},
                                                                                                     None],
                                                                                         'name': 'scg-ConfigPartMCG-r12',
                                                                                         'optional': True,
                                                                                         'type': 'SEQUENCE'},
                                                                                        {'name': 'scg-ConfigPartSCG-r12',
                                                                                         'optional': True,
                                                                                         'type': 'SCG-ConfigPartSCG-r12'}],
                                                                            'name': 'setup',
                                                                            'type': 'SEQUENCE'}],
                                                               'type': 'CHOICE'},
                                     'SCG-Configuration-v12f0': {'members': [{'name': 'release',
                                                                              'type': 'NULL'},
                                                                             {'members': [{'name': 'scg-ConfigPartSCG-v12f0',
                                                                                           'optional': True,
                                                                                           'type': 'SCG-ConfigPartSCG-v12f0'}],
                                                                              'name': 'setup',
                                                                              'type': 'SEQUENCE'}],
                                                                 'type': 'CHOICE'},
                                     'SCGFailureInformation-r12': {'members': [{'members': [{'members': [{'name': 'scgFailureInformation-r12',
                                                                                                          'type': 'SCGFailureInformation-r12-IEs'},
                                                                                                         {'name': 'spare3',
                                                                                                          'type': 'NULL'},
                                                                                                         {'name': 'spare2',
                                                                                                          'type': 'NULL'},
                                                                                                         {'name': 'spare1',
                                                                                                          'type': 'NULL'}],
                                                                                             'name': 'c1',
                                                                                             'type': 'CHOICE'},
                                                                                            {'members': [],
                                                                                             'name': 'criticalExtensionsFuture',
                                                                                             'type': 'SEQUENCE'}],
                                                                                'name': 'criticalExtensions',
                                                                                'type': 'CHOICE'}],
                                                                   'type': 'SEQUENCE'},
                                     'SCGFailureInformation-r12-IEs': {'members': [{'name': 'failureReportSCG-r12',
                                                                                    'optional': True,
                                                                                    'type': 'FailureReportSCG-r12'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SCGFailureInformation-v1310-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'SCGFailureInformation-v12d0-IEs': {'members': [{'name': 'failureReportSCG-v12d0',
                                                                                      'optional': True,
                                                                                      'type': 'FailureReportSCG-v12d0'},
                                                                                     {'members': [],
                                                                                      'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'SEQUENCE'}],
                                                                         'type': 'SEQUENCE'},
                                     'SCGFailureInformation-v1310-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'OCTET '
                                                                                              'STRING'},
                                                                                     {'members': [],
                                                                                      'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'SEQUENCE'}],
                                                                         'type': 'SEQUENCE'},
                                     'SCPTM-NeighbourCellList-r13': {'element': {'type': 'PCI-ARFCN-r13'},
                                                                     'size': [(1,
                                                                               'maxNeighCell-SCPTM-r13')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'SCPTM-Parameters-r13': {'members': [{'name': 'scptm-ParallelReception-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'scptm-SCell-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'scptm-NonServingCell-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'scptm-AsyncDC-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'SCPTMConfiguration-BR-r14': {'members': [{'name': 'sc-mtch-InfoList-r14',
                                                                                'type': 'SC-MTCH-InfoList-BR-r14'},
                                                                               {'name': 'scptm-NeighbourCellList-r14',
                                                                                'optional': True,
                                                                                'type': 'SCPTM-NeighbourCellList-r13'},
                                                                               {'name': 'p-b-r14',
                                                                                'optional': True,
                                                                                'restricted-to': [(0,
                                                                                                   3)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'lateNonCriticalExtension',
                                                                                'optional': True,
                                                                                'type': 'OCTET '
                                                                                        'STRING'},
                                                                               {'members': [],
                                                                                'name': 'nonCriticalExtension',
                                                                                'optional': True,
                                                                                'type': 'SEQUENCE'}],
                                                                   'type': 'SEQUENCE'},
                                     'SCPTMConfiguration-r13': {'members': [{'name': 'sc-mtch-InfoList-r13',
                                                                             'type': 'SC-MTCH-InfoList-r13'},
                                                                            {'name': 'scptm-NeighbourCellList-r13',
                                                                             'optional': True,
                                                                             'type': 'SCPTM-NeighbourCellList-r13'},
                                                                            {'name': 'lateNonCriticalExtension',
                                                                             'optional': True,
                                                                             'type': 'OCTET '
                                                                                     'STRING'},
                                                                            {'name': 'nonCriticalExtension',
                                                                             'optional': True,
                                                                             'type': 'SCPTMConfiguration-v1340'}],
                                                                'type': 'SEQUENCE'},
                                     'SCPTMConfiguration-v1340': {'members': [{'name': 'p-b-r13',
                                                                               'optional': True,
                                                                               'restricted-to': [(0,
                                                                                                  3)],
                                                                               'type': 'INTEGER'},
                                                                              {'members': [],
                                                                               'name': 'nonCriticalExtension',
                                                                               'optional': True,
                                                                               'type': 'SEQUENCE'}],
                                                                  'type': 'SEQUENCE'},
                                     'SCellIndex-r10': {'restricted-to': [(1,
                                                                           7)],
                                                        'type': 'INTEGER'},
                                     'SCellIndex-r13': {'restricted-to': [(1,
                                                                           31)],
                                                        'type': 'INTEGER'},
                                     'SCellToAddMod-r10': {'members': [{'name': 'sCellIndex-r10',
                                                                        'type': 'SCellIndex-r10'},
                                                                       {'members': [{'name': 'physCellId-r10',
                                                                                     'type': 'PhysCellId'},
                                                                                    {'name': 'dl-CarrierFreq-r10',
                                                                                     'type': 'ARFCN-ValueEUTRA'}],
                                                                        'name': 'cellIdentification-r10',
                                                                        'optional': True,
                                                                        'type': 'SEQUENCE'},
                                                                       {'name': 'radioResourceConfigCommonSCell-r10',
                                                                        'optional': True,
                                                                        'type': 'RadioResourceConfigCommonSCell-r10'},
                                                                       {'name': 'radioResourceConfigDedicatedSCell-r10',
                                                                        'optional': True,
                                                                        'type': 'RadioResourceConfigDedicatedSCell-r10'},
                                                                       None,
                                                                       [{'name': 'dl-CarrierFreq-v1090',
                                                                         'optional': True,
                                                                         'type': 'ARFCN-ValueEUTRA-v9e0'}],
                                                                       [{'name': 'antennaInfoDedicatedSCell-v10i0',
                                                                         'optional': True,
                                                                         'type': 'AntennaInfoDedicated-v10i0'}],
                                                                       [{'name': 'srs-SwitchFromServCellIndex-r14',
                                                                         'optional': True,
                                                                         'restricted-to': [(0,
                                                                                            31)],
                                                                         'type': 'INTEGER'}]],
                                                           'type': 'SEQUENCE'},
                                     'SCellToAddMod-v10l0': {'members': [{'name': 'radioResourceConfigCommonSCell-v10l0',
                                                                          'optional': True,
                                                                          'type': 'RadioResourceConfigCommonSCell-v10l0'}],
                                                             'type': 'SEQUENCE'},
                                     'SCellToAddModExt-r13': {'members': [{'name': 'sCellIndex-r13',
                                                                           'type': 'SCellIndex-r13'},
                                                                          {'members': [{'name': 'physCellId-r13',
                                                                                        'type': 'PhysCellId'},
                                                                                       {'name': 'dl-CarrierFreq-r13',
                                                                                        'type': 'ARFCN-ValueEUTRA-r9'}],
                                                                           'name': 'cellIdentification-r13',
                                                                           'optional': True,
                                                                           'type': 'SEQUENCE'},
                                                                          {'name': 'radioResourceConfigCommonSCell-r13',
                                                                           'optional': True,
                                                                           'type': 'RadioResourceConfigCommonSCell-r10'},
                                                                          {'name': 'radioResourceConfigDedicatedSCell-r13',
                                                                           'optional': True,
                                                                           'type': 'RadioResourceConfigDedicatedSCell-r10'},
                                                                          {'name': 'antennaInfoDedicatedSCell-r13',
                                                                           'optional': True,
                                                                           'type': 'AntennaInfoDedicated-v10i0'}],
                                                              'type': 'SEQUENCE'},
                                     'SCellToAddModExt-v1370': {'members': [{'name': 'radioResourceConfigCommonSCell-v1370',
                                                                             'optional': True,
                                                                             'type': 'RadioResourceConfigCommonSCell-v10l0'}],
                                                                'type': 'SEQUENCE'},
                                     'SCellToAddModExt-v1430': {'members': [{'name': 'srs-SwitchFromServCellIndex-r14',
                                                                             'optional': True,
                                                                             'restricted-to': [(0,
                                                                                                31)],
                                                                             'type': 'INTEGER'},
                                                                            None],
                                                                'type': 'SEQUENCE'},
                                     'SCellToAddModList-r10': {'element': {'type': 'SCellToAddMod-r10'},
                                                               'size': [(1,
                                                                         'maxSCell-r10')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'SCellToAddModList-v10l0': {'element': {'type': 'SCellToAddMod-v10l0'},
                                                                 'size': [(1,
                                                                           'maxSCell-r10')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'SCellToAddModListExt-r13': {'element': {'type': 'SCellToAddModExt-r13'},
                                                                  'size': [(1,
                                                                            'maxSCell-r13')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'SCellToAddModListExt-v1370': {'element': {'type': 'SCellToAddModExt-v1370'},
                                                                    'size': [(1,
                                                                              'maxSCell-r13')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'SCellToAddModListExt-v1430': {'element': {'type': 'SCellToAddModExt-v1430'},
                                                                    'size': [(1,
                                                                              'maxSCell-r13')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'SCellToReleaseList-r10': {'element': {'type': 'SCellIndex-r10'},
                                                                'size': [(1,
                                                                          'maxSCell-r10')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'SCellToReleaseListExt-r13': {'element': {'type': 'SCellIndex-r13'},
                                                                   'size': [(1,
                                                                             'maxSCell-r13')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'SI-OrPSI-GERAN': {'members': [{'name': 'si',
                                                                     'type': 'SystemInfoListGERAN'},
                                                                    {'name': 'psi',
                                                                     'type': 'SystemInfoListGERAN'}],
                                                        'type': 'CHOICE'},
                                     'SIB-MappingInfo': {'element': {'type': 'SIB-Type'},
                                                         'size': [(0,
                                                                   'maxSIB-1')],
                                                         'type': 'SEQUENCE OF'},
                                     'SIB-MappingInfo-MBMS-r14': {'element': {'type': 'SIB-Type-MBMS-r14'},
                                                                  'size': [(0,
                                                                            'maxSIB-1')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'SIB-Type': {'type': 'ENUMERATED',
                                                  'values': [('sibType3', 0),
                                                             ('sibType4', 1),
                                                             ('sibType5', 2),
                                                             ('sibType6', 3),
                                                             ('sibType7', 4),
                                                             ('sibType8', 5),
                                                             ('sibType9', 6),
                                                             ('sibType10', 7),
                                                             ('sibType11', 8),
                                                             ('sibType12-v920',
                                                              9),
                                                             ('sibType13-v920',
                                                              10),
                                                             ('sibType14-v1130',
                                                              11),
                                                             ('sibType15-v1130',
                                                              12),
                                                             ('sibType16-v1130',
                                                              13),
                                                             ('sibType17-v1250',
                                                              14),
                                                             ('sibType18-v1250',
                                                              15),
                                                             None,
                                                             ('sibType19-v1250',
                                                              16),
                                                             ('sibType20-v1310',
                                                              17),
                                                             ('sibType21-v1430',
                                                              18)]},
                                     'SIB-Type-MBMS-r14': {'type': 'ENUMERATED',
                                                           'values': [('sibType10',
                                                                       0),
                                                                      ('sibType11',
                                                                       1),
                                                                      ('sibType12-v920',
                                                                       2),
                                                                      ('sibType13-v920',
                                                                       3),
                                                                      ('sibType15-v1130',
                                                                       4),
                                                                      ('sibType16-v1130',
                                                                       5),
                                                                      None]},
                                     'SIB8-PerPLMN-List-r11': {'element': {'type': 'SIB8-PerPLMN-r11'},
                                                               'size': [(1,
                                                                         'maxPLMN-r11')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'SIB8-PerPLMN-r11': {'members': [{'name': 'plmn-Identity-r11',
                                                                       'restricted-to': [(1,
                                                                                          'maxPLMN-r11')],
                                                                       'type': 'INTEGER'},
                                                                      {'members': [{'name': 'explicitValue',
                                                                                    'type': 'ParametersCDMA2000-r11'},
                                                                                   {'name': 'defaultValue',
                                                                                    'type': 'NULL'}],
                                                                       'name': 'parametersCDMA2000-r11',
                                                                       'type': 'CHOICE'}],
                                                          'type': 'SEQUENCE'},
                                     'SL-AnchorCarrierFreqList-V2X-r14': {'element': {'type': 'ARFCN-ValueEUTRA-r9'},
                                                                          'size': [(1,
                                                                                    'maxFreqV2X-r14')],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                     'SL-CBR-CommonTxConfigList-r14': {'members': [{'element': {'type': 'SL-CBR-Levels-Config-r14'},
                                                                                    'name': 'cbr-RangeCommonConfigList-r14',
                                                                                    'size': [(1,
                                                                                              'maxSL-V2X-CBRConfig-r14')],
                                                                                    'type': 'SEQUENCE '
                                                                                            'OF'},
                                                                                   {'element': {'type': 'SL-CBR-PSSCH-TxConfig-r14'},
                                                                                    'name': 'sl-CBR-PSSCH-TxConfigList-r14',
                                                                                    'size': [(1,
                                                                                              'maxSL-V2X-TxConfig-r14')],
                                                                                    'type': 'SEQUENCE '
                                                                                            'OF'}],
                                                                       'type': 'SEQUENCE'},
                                     'SL-CBR-Levels-Config-r14': {'element': {'type': 'SL-CBR-r14'},
                                                                  'size': [(1,
                                                                            'maxCBR-Level-r14')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'SL-CBR-PPPP-TxConfigList-r14': {'element': {'type': 'SL-PPPP-TxConfigIndex-r14'},
                                                                      'size': [(1,
                                                                                8)],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'SL-CBR-PSSCH-TxConfig-r14': {'members': [{'name': 'cr-Limit-r14',
                                                                                'restricted-to': [(0,
                                                                                                   10000)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'tx-Parameters-r14',
                                                                                'type': 'SL-PSSCH-TxParameters-r14'}],
                                                                   'type': 'SEQUENCE'},
                                     'SL-CBR-r14': {'restricted-to': [(0, 100)],
                                                    'type': 'INTEGER'},
                                     'SL-CP-Len-r12': {'type': 'ENUMERATED',
                                                       'values': [('normal', 0),
                                                                  ('extended',
                                                                   1)]},
                                     'SL-CarrierFreqInfo-r12': {'members': [{'name': 'carrierFreq-r12',
                                                                             'type': 'ARFCN-ValueEUTRA-r9'},
                                                                            {'name': 'plmn-IdentityList-r12',
                                                                             'optional': True,
                                                                             'type': 'PLMN-IdentityList4-r12'}],
                                                                'type': 'SEQUENCE'},
                                     'SL-CarrierFreqInfo-v1310': {'members': [{'name': 'discResourcesNonPS-r13',
                                                                               'optional': True,
                                                                               'type': 'SL-ResourcesInterFreq-r13'},
                                                                              {'name': 'discResourcesPS-r13',
                                                                               'optional': True,
                                                                               'type': 'SL-ResourcesInterFreq-r13'},
                                                                              {'name': 'discConfigOther-r13',
                                                                               'optional': True,
                                                                               'type': 'SL-DiscConfigOtherInterFreq-r13'},
                                                                              None],
                                                                  'type': 'SEQUENCE'},
                                     'SL-CarrierFreqInfoList-r12': {'element': {'type': 'SL-CarrierFreqInfo-r12'},
                                                                    'size': [(1,
                                                                              'maxFreq')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'SL-CarrierFreqInfoList-v1310': {'element': {'type': 'SL-CarrierFreqInfo-v1310'},
                                                                      'size': [(1,
                                                                                'maxFreq')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'SL-CommConfig-r12': {'members': [{'members': [{'name': 'release',
                                                                                     'type': 'NULL'},
                                                                                    {'members': [{'members': [{'name': 'sl-RNTI-r12',
                                                                                                               'type': 'C-RNTI'},
                                                                                                              {'name': 'mac-MainConfig-r12',
                                                                                                               'type': 'MAC-MainConfigSL-r12'},
                                                                                                              {'name': 'sc-CommTxConfig-r12',
                                                                                                               'type': 'SL-CommResourcePool-r12'},
                                                                                                              {'name': 'mcs-r12',
                                                                                                               'optional': True,
                                                                                                               'restricted-to': [(0,
                                                                                                                                  28)],
                                                                                                               'type': 'INTEGER'}],
                                                                                                  'name': 'scheduled-r12',
                                                                                                  'type': 'SEQUENCE'},
                                                                                                 {'members': [{'members': [{'name': 'poolToReleaseList-r12',
                                                                                                                            'optional': True,
                                                                                                                            'type': 'SL-TxPoolToReleaseList-r12'},
                                                                                                                           {'name': 'poolToAddModList-r12',
                                                                                                                            'optional': True,
                                                                                                                            'type': 'SL-CommTxPoolToAddModList-r12'}],
                                                                                                               'name': 'commTxPoolNormalDedicated-r12',
                                                                                                               'type': 'SEQUENCE'}],
                                                                                                  'name': 'ue-Selected-r12',
                                                                                                  'type': 'SEQUENCE'}],
                                                                                     'name': 'setup',
                                                                                     'type': 'CHOICE'}],
                                                                        'name': 'commTxResources-r12',
                                                                        'optional': True,
                                                                        'type': 'CHOICE'},
                                                                       None,
                                                                       [{'members': [{'name': 'release',
                                                                                      'type': 'NULL'},
                                                                                     {'members': [{'members': [{'name': 'logicalChGroupInfoList-r13',
                                                                                                                'type': 'LogicalChGroupInfoList-r13'},
                                                                                                               {'name': 'multipleTx-r13',
                                                                                                                'type': 'BOOLEAN'}],
                                                                                                   'name': 'scheduled-v1310',
                                                                                                   'type': 'SEQUENCE'},
                                                                                                  {'members': [{'members': [{'name': 'poolToReleaseListExt-r13',
                                                                                                                             'optional': True,
                                                                                                                             'type': 'SL-TxPoolToReleaseListExt-r13'},
                                                                                                                            {'name': 'poolToAddModListExt-r13',
                                                                                                                             'optional': True,
                                                                                                                             'type': 'SL-CommTxPoolToAddModListExt-r13'}],
                                                                                                                'name': 'commTxPoolNormalDedicatedExt-r13',
                                                                                                                'type': 'SEQUENCE'}],
                                                                                                   'name': 'ue-Selected-v1310',
                                                                                                   'type': 'SEQUENCE'}],
                                                                                      'name': 'setup',
                                                                                      'type': 'CHOICE'}],
                                                                         'name': 'commTxResources-v1310',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'},
                                                                        {'name': 'commTxAllowRelayDedicated-r13',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'}]],
                                                           'type': 'SEQUENCE'},
                                     'SL-CommResourcePool-r12': {'members': [{'name': 'sc-CP-Len-r12',
                                                                              'type': 'SL-CP-Len-r12'},
                                                                             {'name': 'sc-Period-r12',
                                                                              'type': 'SL-PeriodComm-r12'},
                                                                             {'name': 'sc-TF-ResourceConfig-r12',
                                                                              'type': 'SL-TF-ResourceConfig-r12'},
                                                                             {'name': 'data-CP-Len-r12',
                                                                              'type': 'SL-CP-Len-r12'},
                                                                             {'name': 'dataHoppingConfig-r12',
                                                                              'type': 'SL-HoppingConfigComm-r12'},
                                                                             {'members': [{'name': 'data-TF-ResourceConfig-r12',
                                                                                           'type': 'SL-TF-ResourceConfig-r12'},
                                                                                          {'name': 'trpt-Subset-r12',
                                                                                           'optional': True,
                                                                                           'type': 'SL-TRPT-Subset-r12'}],
                                                                              'name': 'ue-SelectedResourceConfig-r12',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'},
                                                                             {'members': [{'name': 'tdd-Config-r12',
                                                                                           'optional': True,
                                                                                           'type': 'TDD-Config'},
                                                                                          {'name': 'syncConfigIndex-r12',
                                                                                           'restricted-to': [(0,
                                                                                                              15)],
                                                                                           'type': 'INTEGER'}],
                                                                              'name': 'rxParametersNCell-r12',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'},
                                                                             {'members': [{'name': 'sc-TxParameters-r12',
                                                                                           'type': 'SL-TxParameters-r12'},
                                                                                          {'name': 'dataTxParameters-r12',
                                                                                           'type': 'SL-TxParameters-r12'}],
                                                                              'name': 'txParameters-r12',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'},
                                                                             None,
                                                                             [{'name': 'priorityList-r13',
                                                                               'optional': True,
                                                                               'type': 'SL-PriorityList-r13'}]],
                                                                 'type': 'SEQUENCE'},
                                     'SL-CommResourcePoolV2X-r14': {'members': [{'name': 'sl-OffsetIndicator-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-OffsetIndicator-r12'},
                                                                                {'name': 'sl-Subframe-r14',
                                                                                 'type': 'SubframeBitmapSL-r14'},
                                                                                {'name': 'adjacencyPSCCH-PSSCH-r14',
                                                                                 'type': 'BOOLEAN'},
                                                                                {'name': 'sizeSubchannel-r14',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('n4',
                                                                                             0),
                                                                                            ('n5',
                                                                                             1),
                                                                                            ('n6',
                                                                                             2),
                                                                                            ('n8',
                                                                                             3),
                                                                                            ('n9',
                                                                                             4),
                                                                                            ('n10',
                                                                                             5),
                                                                                            ('n12',
                                                                                             6),
                                                                                            ('n15',
                                                                                             7),
                                                                                            ('n16',
                                                                                             8),
                                                                                            ('n18',
                                                                                             9),
                                                                                            ('n20',
                                                                                             10),
                                                                                            ('n25',
                                                                                             11),
                                                                                            ('n30',
                                                                                             12),
                                                                                            ('n48',
                                                                                             13),
                                                                                            ('n50',
                                                                                             14),
                                                                                            ('n72',
                                                                                             15),
                                                                                            ('n75',
                                                                                             16),
                                                                                            ('n96',
                                                                                             17),
                                                                                            ('n100',
                                                                                             18),
                                                                                            ('spare13',
                                                                                             19),
                                                                                            ('spare12',
                                                                                             20),
                                                                                            ('spare11',
                                                                                             21),
                                                                                            ('spare10',
                                                                                             22),
                                                                                            ('spare9',
                                                                                             23),
                                                                                            ('spare8',
                                                                                             24),
                                                                                            ('spare7',
                                                                                             25),
                                                                                            ('spare6',
                                                                                             26),
                                                                                            ('spare5',
                                                                                             27),
                                                                                            ('spare4',
                                                                                             28),
                                                                                            ('spare3',
                                                                                             29),
                                                                                            ('spare2',
                                                                                             30),
                                                                                            ('spare1',
                                                                                             31)]},
                                                                                {'name': 'numSubchannel-r14',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('n1',
                                                                                             0),
                                                                                            ('n3',
                                                                                             1),
                                                                                            ('n5',
                                                                                             2),
                                                                                            ('n8',
                                                                                             3),
                                                                                            ('n10',
                                                                                             4),
                                                                                            ('n15',
                                                                                             5),
                                                                                            ('n20',
                                                                                             6),
                                                                                            ('spare1',
                                                                                             7)]},
                                                                                {'name': 'startRB-Subchannel-r14',
                                                                                 'restricted-to': [(0,
                                                                                                    99)],
                                                                                 'type': 'INTEGER'},
                                                                                {'name': 'startRB-PSCCH-Pool-r14',
                                                                                 'optional': True,
                                                                                 'restricted-to': [(0,
                                                                                                    99)],
                                                                                 'type': 'INTEGER'},
                                                                                {'members': [{'name': 'tdd-Config-r14',
                                                                                              'optional': True,
                                                                                              'type': 'TDD-Config'},
                                                                                             {'name': 'syncConfigIndex-r14',
                                                                                              'restricted-to': [(0,
                                                                                                                 15)],
                                                                                              'type': 'INTEGER'}],
                                                                                 'name': 'rxParametersNCell-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SEQUENCE'},
                                                                                {'name': 'dataTxParameters-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-TxParameters-r12'},
                                                                                {'name': 'zoneID-r14',
                                                                                 'optional': True,
                                                                                 'restricted-to': [(0,
                                                                                                    7)],
                                                                                 'type': 'INTEGER'},
                                                                                {'name': 'threshS-RSSI-CBR-r14',
                                                                                 'optional': True,
                                                                                 'restricted-to': [(0,
                                                                                                    45)],
                                                                                 'type': 'INTEGER'},
                                                                                {'name': 'poolReportId-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-V2X-TxPoolReportIdentity-r14'},
                                                                                {'name': 'cbr-pssch-TxConfigList-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-CBR-PPPP-TxConfigList-r14'},
                                                                                {'name': 'resourceSelectionConfigP2X-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-P2X-ResourceSelectionConfig-r14'},
                                                                                {'name': 'syncAllowed-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-SyncAllowed-r14'},
                                                                                {'name': 'restrictResourceReservationPeriod-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-RestrictResourceReservationPeriodList-r14'},
                                                                                None],
                                                                    'type': 'SEQUENCE'},
                                     'SL-CommRxPoolList-r12': {'element': {'type': 'SL-CommResourcePool-r12'},
                                                               'size': [(1,
                                                                         'maxSL-RxPool-r12')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'SL-CommRxPoolListV2X-r14': {'element': {'type': 'SL-CommResourcePoolV2X-r14'},
                                                                  'size': [(1,
                                                                            'maxSL-V2X-RxPool-r14')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'SL-CommTxPoolList-r12': {'element': {'type': 'SL-CommResourcePool-r12'},
                                                               'size': [(1,
                                                                         'maxSL-TxPool-r12')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'SL-CommTxPoolListExt-r13': {'element': {'type': 'SL-CommResourcePool-r12'},
                                                                  'size': [(1,
                                                                            'maxSL-TxPool-v1310')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'SL-CommTxPoolListV2X-r14': {'element': {'type': 'SL-CommResourcePoolV2X-r14'},
                                                                  'size': [(1,
                                                                            'maxSL-V2X-TxPool-r14')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'SL-CommTxPoolSensingConfig-r14': {'members': [{'name': 'pssch-TxConfigList-r14',
                                                                                     'type': 'SL-PSSCH-TxConfigList-r14'},
                                                                                    {'name': 'thresPSSCH-RSRP-List-r14',
                                                                                     'type': 'SL-ThresPSSCH-RSRP-List-r14'},
                                                                                    {'name': 'restrictResourceReservationPeriod-r14',
                                                                                     'optional': True,
                                                                                     'type': 'SL-RestrictResourceReservationPeriodList-r14'},
                                                                                    {'name': 'probResourceKeep-r14',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('v0',
                                                                                                 0),
                                                                                                ('v0dot2',
                                                                                                 1),
                                                                                                ('v0dot4',
                                                                                                 2),
                                                                                                ('v0dot6',
                                                                                                 3),
                                                                                                ('v0dot8',
                                                                                                 4),
                                                                                                ('spare3',
                                                                                                 5),
                                                                                                ('spare2',
                                                                                                 6),
                                                                                                ('spare1',
                                                                                                 7)]},
                                                                                    {'members': [{'name': 'minNumCandidateSF-r14',
                                                                                                  'restricted-to': [(1,
                                                                                                                     13)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'name': 'gapCandidateSensing-r14',
                                                                                                  'size': [10],
                                                                                                  'type': 'BIT '
                                                                                                          'STRING'}],
                                                                                     'name': 'p2x-SensingConfig-r14',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'},
                                                                                    {'name': 'sl-ReselectAfter-r14',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('n1',
                                                                                                 0),
                                                                                                ('n2',
                                                                                                 1),
                                                                                                ('n3',
                                                                                                 2),
                                                                                                ('n4',
                                                                                                 3),
                                                                                                ('n5',
                                                                                                 4),
                                                                                                ('n6',
                                                                                                 5),
                                                                                                ('n7',
                                                                                                 6),
                                                                                                ('n8',
                                                                                                 7),
                                                                                                ('n9',
                                                                                                 8),
                                                                                                ('spare7',
                                                                                                 9),
                                                                                                ('spare6',
                                                                                                 10),
                                                                                                ('spare5',
                                                                                                 11),
                                                                                                ('spare4',
                                                                                                 12),
                                                                                                ('spare3',
                                                                                                 13),
                                                                                                ('spare2',
                                                                                                 14),
                                                                                                ('spare1',
                                                                                                 15)]}],
                                                                        'type': 'SEQUENCE'},
                                     'SL-CommTxPoolToAddMod-r12': {'members': [{'name': 'poolIdentity-r12',
                                                                                'type': 'SL-TxPoolIdentity-r12'},
                                                                               {'name': 'pool-r12',
                                                                                'type': 'SL-CommResourcePool-r12'}],
                                                                   'type': 'SEQUENCE'},
                                     'SL-CommTxPoolToAddModExt-r13': {'members': [{'name': 'poolIdentity-v1310',
                                                                                   'type': 'SL-TxPoolIdentity-v1310'},
                                                                                  {'name': 'pool-r13',
                                                                                   'type': 'SL-CommResourcePool-r12'}],
                                                                      'type': 'SEQUENCE'},
                                     'SL-CommTxPoolToAddModList-r12': {'element': {'type': 'SL-CommTxPoolToAddMod-r12'},
                                                                       'size': [(1,
                                                                                 'maxSL-TxPool-r12')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'SL-CommTxPoolToAddModListExt-r13': {'element': {'type': 'SL-CommTxPoolToAddModExt-r13'},
                                                                          'size': [(1,
                                                                                    'maxSL-TxPool-v1310')],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                     'SL-CommTxResourceReq-r12': {'members': [{'name': 'carrierFreq-r12',
                                                                               'optional': True,
                                                                               'type': 'ARFCN-ValueEUTRA-r9'},
                                                                              {'name': 'destinationInfoList-r12',
                                                                               'type': 'SL-DestinationInfoList-r12'}],
                                                                  'type': 'SEQUENCE'},
                                     'SL-DestinationIdentity-r12': {'size': [24],
                                                                    'type': 'BIT '
                                                                            'STRING'},
                                     'SL-DestinationInfoList-r12': {'element': {'type': 'SL-DestinationIdentity-r12'},
                                                                    'size': [(1,
                                                                              'maxSL-Dest-r12')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'SL-DiscConfig-r12': {'members': [{'members': [{'name': 'release',
                                                                                     'type': 'NULL'},
                                                                                    {'members': [{'members': [{'name': 'discTxConfig-r12',
                                                                                                               'optional': True,
                                                                                                               'type': 'SL-DiscResourcePool-r12'},
                                                                                                              {'name': 'discTF-IndexList-r12',
                                                                                                               'optional': True,
                                                                                                               'type': 'SL-TF-IndexPairList-r12'},
                                                                                                              {'name': 'discHoppingConfig-r12',
                                                                                                               'optional': True,
                                                                                                               'type': 'SL-HoppingConfigDisc-r12'}],
                                                                                                  'name': 'scheduled-r12',
                                                                                                  'type': 'SEQUENCE'},
                                                                                                 {'members': [{'members': [{'name': 'poolToReleaseList-r12',
                                                                                                                            'optional': True,
                                                                                                                            'type': 'SL-TxPoolToReleaseList-r12'},
                                                                                                                           {'name': 'poolToAddModList-r12',
                                                                                                                            'optional': True,
                                                                                                                            'type': 'SL-DiscTxPoolToAddModList-r12'}],
                                                                                                               'name': 'discTxPoolDedicated-r12',
                                                                                                               'optional': True,
                                                                                                               'type': 'SEQUENCE'}],
                                                                                                  'name': 'ue-Selected-r12',
                                                                                                  'type': 'SEQUENCE'}],
                                                                                     'name': 'setup',
                                                                                     'type': 'CHOICE'}],
                                                                        'name': 'discTxResources-r12',
                                                                        'optional': True,
                                                                        'type': 'CHOICE'},
                                                                       None,
                                                                       [{'members': [{'name': 'release',
                                                                                      'type': 'NULL'},
                                                                                     {'members': [{'name': 'discTF-IndexList-r12b',
                                                                                                   'type': 'SL-TF-IndexPairList-r12b'}],
                                                                                      'name': 'setup',
                                                                                      'type': 'SEQUENCE'}],
                                                                         'name': 'discTF-IndexList-v1260',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'}],
                                                                       [{'members': [{'name': 'release',
                                                                                      'type': 'NULL'},
                                                                                     {'members': [{'name': 'scheduled-r13',
                                                                                                   'type': 'SL-DiscTxConfigScheduled-r13'},
                                                                                                  {'members': [{'name': 'discTxPoolPS-Dedicated-r13',
                                                                                                                'type': 'SL-DiscTxPoolDedicated-r13'}],
                                                                                                   'name': 'ue-Selected-r13',
                                                                                                   'type': 'SEQUENCE'}],
                                                                                      'name': 'setup',
                                                                                      'type': 'CHOICE'}],
                                                                         'name': 'discTxResourcesPS-r13',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'},
                                                                        {'members': [{'name': 'release',
                                                                                      'type': 'NULL'},
                                                                                     {'members': [{'name': 'discTxCarrierFreq-r13',
                                                                                                   'optional': True,
                                                                                                   'type': 'ARFCN-ValueEUTRA-r9'},
                                                                                                  {'name': 'discTxRefCarrierDedicated-r13',
                                                                                                   'optional': True,
                                                                                                   'type': 'SL-DiscTxRefCarrierDedicated-r13'},
                                                                                                  {'name': 'discTxInfoInterFreqListAdd-r13',
                                                                                                   'optional': True,
                                                                                                   'type': 'SL-DiscTxInfoInterFreqListAdd-r13'}],
                                                                                      'name': 'setup',
                                                                                      'type': 'SEQUENCE'}],
                                                                         'name': 'discTxInterFreqInfo-r13',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'},
                                                                        {'name': 'gapRequestsAllowedDedicated-r13',
                                                                         'optional': True,
                                                                         'type': 'BOOLEAN'},
                                                                        {'members': [{'name': 'release',
                                                                                      'type': 'NULL'},
                                                                                     {'name': 'setup',
                                                                                      'type': 'SL-GapConfig-r13'}],
                                                                         'name': 'discRxGapConfig-r13',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'},
                                                                        {'members': [{'name': 'release',
                                                                                      'type': 'NULL'},
                                                                                     {'name': 'setup',
                                                                                      'type': 'SL-GapConfig-r13'}],
                                                                         'name': 'discTxGapConfig-r13',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'},
                                                                        {'members': [{'name': 'release',
                                                                                      'type': 'NULL'},
                                                                                     {'name': 'setup',
                                                                                      'type': 'SL-DiscSysInfoToReportFreqList-r13'}],
                                                                         'name': 'discSysInfoToReportConfig-r13',
                                                                         'optional': True,
                                                                         'type': 'CHOICE'}]],
                                                           'type': 'SEQUENCE'},
                                     'SL-DiscConfigOtherInterFreq-r13': {'members': [{'name': 'txPowerInfo-r13',
                                                                                      'optional': True,
                                                                                      'type': 'SL-DiscTxPowerInfoList-r12'},
                                                                                     {'name': 'refCarrierCommon-r13',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('pCell',
                                                                                                  0)]},
                                                                                     {'name': 'discSyncConfig-r13',
                                                                                      'optional': True,
                                                                                      'type': 'SL-SyncConfigListNFreq-r13'},
                                                                                     {'name': 'discCellSelectionInfo-r13',
                                                                                      'optional': True,
                                                                                      'type': 'CellSelectionInfoNFreq-r13'}],
                                                                         'type': 'SEQUENCE'},
                                     'SL-DiscConfigRelayUE-r13': {'members': [{'name': 'threshHigh-r13',
                                                                               'optional': True,
                                                                               'type': 'RSRP-RangeSL4-r13'},
                                                                              {'name': 'threshLow-r13',
                                                                               'optional': True,
                                                                               'type': 'RSRP-RangeSL4-r13'},
                                                                              {'name': 'hystMax-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('dB0',
                                                                                           0),
                                                                                          ('dB3',
                                                                                           1),
                                                                                          ('dB6',
                                                                                           2),
                                                                                          ('dB9',
                                                                                           3),
                                                                                          ('dB12',
                                                                                           4),
                                                                                          ('dBinf',
                                                                                           5)]},
                                                                              {'name': 'hystMin-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('dB0',
                                                                                           0),
                                                                                          ('dB3',
                                                                                           1),
                                                                                          ('dB6',
                                                                                           2),
                                                                                          ('dB9',
                                                                                           3),
                                                                                          ('dB12',
                                                                                           4)]}],
                                                                  'type': 'SEQUENCE'},
                                     'SL-DiscConfigRemoteUE-r13': {'members': [{'name': 'threshHigh-r13',
                                                                                'optional': True,
                                                                                'type': 'RSRP-RangeSL4-r13'},
                                                                               {'name': 'hystMax-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('dB0',
                                                                                            0),
                                                                                           ('dB3',
                                                                                            1),
                                                                                           ('dB6',
                                                                                            2),
                                                                                           ('dB9',
                                                                                            3),
                                                                                           ('dB12',
                                                                                            4)]},
                                                                               {'name': 'reselectionInfoIC-r13',
                                                                                'type': 'ReselectionInfoRelay-r13'}],
                                                                   'type': 'SEQUENCE'},
                                     'SL-DiscResourcePool-r12': {'members': [{'name': 'cp-Len-r12',
                                                                              'type': 'SL-CP-Len-r12'},
                                                                             {'name': 'discPeriod-r12',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('rf32',
                                                                                          0),
                                                                                         ('rf64',
                                                                                          1),
                                                                                         ('rf128',
                                                                                          2),
                                                                                         ('rf256',
                                                                                          3),
                                                                                         ('rf512',
                                                                                          4),
                                                                                         ('rf1024',
                                                                                          5),
                                                                                         ('rf16-v1310',
                                                                                          6),
                                                                                         ('spare',
                                                                                          7)]},
                                                                             {'name': 'numRetx-r12',
                                                                              'restricted-to': [(0,
                                                                                                 3)],
                                                                              'type': 'INTEGER'},
                                                                             {'name': 'numRepetition-r12',
                                                                              'restricted-to': [(1,
                                                                                                 50)],
                                                                              'type': 'INTEGER'},
                                                                             {'name': 'tf-ResourceConfig-r12',
                                                                              'type': 'SL-TF-ResourceConfig-r12'},
                                                                             {'members': [{'name': 'txParametersGeneral-r12',
                                                                                           'type': 'SL-TxParameters-r12'},
                                                                                          {'members': [{'members': [{'name': 'rsrpBased-r12',
                                                                                                                     'type': 'SL-PoolSelectionConfig-r12'},
                                                                                                                    {'name': 'random-r12',
                                                                                                                     'type': 'NULL'}],
                                                                                                        'name': 'poolSelection-r12',
                                                                                                        'type': 'CHOICE'},
                                                                                                       {'name': 'txProbability-r12',
                                                                                                        'type': 'ENUMERATED',
                                                                                                        'values': [('p25',
                                                                                                                    0),
                                                                                                                   ('p50',
                                                                                                                    1),
                                                                                                                   ('p75',
                                                                                                                    2),
                                                                                                                   ('p100',
                                                                                                                    3)]}],
                                                                                           'name': 'ue-SelectedResourceConfig-r12',
                                                                                           'optional': True,
                                                                                           'type': 'SEQUENCE'}],
                                                                              'name': 'txParameters-r12',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'},
                                                                             {'members': [{'name': 'tdd-Config-r12',
                                                                                           'optional': True,
                                                                                           'type': 'TDD-Config'},
                                                                                          {'name': 'syncConfigIndex-r12',
                                                                                           'restricted-to': [(0,
                                                                                                              15)],
                                                                                           'type': 'INTEGER'}],
                                                                              'name': 'rxParameters-r12',
                                                                              'optional': True,
                                                                              'type': 'SEQUENCE'},
                                                                             None,
                                                                             [{'members': [{'name': 'release',
                                                                                            'type': 'NULL'},
                                                                                           {'name': 'setup',
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('rf4',
                                                                                                        0),
                                                                                                       ('rf6',
                                                                                                        1),
                                                                                                       ('rf7',
                                                                                                        2),
                                                                                                       ('rf8',
                                                                                                        3),
                                                                                                       ('rf12',
                                                                                                        4),
                                                                                                       ('rf14',
                                                                                                        5),
                                                                                                       ('rf24',
                                                                                                        6),
                                                                                                       ('rf28',
                                                                                                        7)]}],
                                                                               'name': 'discPeriod-v1310',
                                                                               'optional': True,
                                                                               'type': 'CHOICE'},
                                                                              {'members': [{'name': 'release',
                                                                                            'type': 'NULL'},
                                                                                           {'members': [{'name': 'physCellId-r13',
                                                                                                         'type': 'PhysCellIdList-r13'}],
                                                                                            'name': 'setup',
                                                                                            'type': 'SEQUENCE'}],
                                                                               'name': 'rxParamsAddNeighFreq-r13',
                                                                               'optional': True,
                                                                               'type': 'CHOICE'},
                                                                              {'members': [{'name': 'release',
                                                                                            'type': 'NULL'},
                                                                                           {'members': [{'name': 'physCellId-r13',
                                                                                                         'type': 'PhysCellIdList-r13'},
                                                                                                        {'name': 'p-Max',
                                                                                                         'optional': True,
                                                                                                         'type': 'P-Max'},
                                                                                                        {'name': 'tdd-Config-r13',
                                                                                                         'optional': True,
                                                                                                         'type': 'TDD-Config'},
                                                                                                        {'name': 'tdd-Config-v1130',
                                                                                                         'optional': True,
                                                                                                         'type': 'TDD-Config-v1130'},
                                                                                                        {'members': [{'name': 'ul-CarrierFreq',
                                                                                                                      'optional': True,
                                                                                                                      'type': 'ARFCN-ValueEUTRA'},
                                                                                                                     {'name': 'ul-Bandwidth',
                                                                                                                      'optional': True,
                                                                                                                      'type': 'ENUMERATED',
                                                                                                                      'values': [('n6',
                                                                                                                                  0),
                                                                                                                                 ('n15',
                                                                                                                                  1),
                                                                                                                                 ('n25',
                                                                                                                                  2),
                                                                                                                                 ('n50',
                                                                                                                                  3),
                                                                                                                                 ('n75',
                                                                                                                                  4),
                                                                                                                                 ('n100',
                                                                                                                                  5)]},
                                                                                                                     {'name': 'additionalSpectrumEmission',
                                                                                                                      'type': 'AdditionalSpectrumEmission'}],
                                                                                                         'name': 'freqInfo',
                                                                                                         'type': 'SEQUENCE'},
                                                                                                        {'name': 'referenceSignalPower',
                                                                                                         'restricted-to': [(-60,
                                                                                                                            50)],
                                                                                                         'type': 'INTEGER'},
                                                                                                        {'name': 'syncConfigIndex-r13',
                                                                                                         'optional': True,
                                                                                                         'restricted-to': [(0,
                                                                                                                            15)],
                                                                                                         'type': 'INTEGER'}],
                                                                                            'name': 'setup',
                                                                                            'type': 'SEQUENCE'}],
                                                                               'name': 'txParamsAddNeighFreq-r13',
                                                                               'optional': True,
                                                                               'type': 'CHOICE'}],
                                                                             [{'members': [{'name': 'release',
                                                                                            'type': 'NULL'},
                                                                                           {'members': [{'members': [{'name': 'additionalSpectrumEmission-v1370',
                                                                                                                      'type': 'AdditionalSpectrumEmission-v10l0'}],
                                                                                                         'name': 'freqInfo-v1370',
                                                                                                         'type': 'SEQUENCE'}],
                                                                                            'name': 'setup',
                                                                                            'type': 'SEQUENCE'}],
                                                                               'name': 'txParamsAddNeighFreq-v1370',
                                                                               'optional': True,
                                                                               'type': 'CHOICE'}]],
                                                                 'type': 'SEQUENCE'},
                                     'SL-DiscRxPoolList-r12': {'element': {'type': 'SL-DiscResourcePool-r12'},
                                                               'size': [(1,
                                                                         'maxSL-RxPool-r12')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'SL-DiscSysInfoReport-r13': {'members': [{'name': 'plmn-IdentityList-r13',
                                                                               'optional': True,
                                                                               'type': 'PLMN-IdentityList'},
                                                                              {'name': 'cellIdentity-13',
                                                                               'optional': True,
                                                                               'type': 'CellIdentity'},
                                                                              {'name': 'carrierFreqInfo-13',
                                                                               'optional': True,
                                                                               'type': 'ARFCN-ValueEUTRA-r9'},
                                                                              {'name': 'discRxResources-r13',
                                                                               'optional': True,
                                                                               'type': 'SL-DiscRxPoolList-r12'},
                                                                              {'name': 'discTxPoolCommon-r13',
                                                                               'optional': True,
                                                                               'type': 'SL-DiscTxPoolList-r12'},
                                                                              {'name': 'discTxPowerInfo-r13',
                                                                               'optional': True,
                                                                               'type': 'SL-DiscTxPowerInfoList-r12'},
                                                                              {'name': 'discSyncConfig-r13',
                                                                               'optional': True,
                                                                               'type': 'SL-SyncConfigNFreq-r13'},
                                                                              {'members': [{'name': 'q-RxLevMin-r13',
                                                                                            'type': 'Q-RxLevMin'},
                                                                                           {'name': 'q-RxLevMinOffset-r13',
                                                                                            'optional': True,
                                                                                            'restricted-to': [(1,
                                                                                                               8)],
                                                                                            'type': 'INTEGER'}],
                                                                               'name': 'discCellSelectionInfo-r13',
                                                                               'optional': True,
                                                                               'type': 'SEQUENCE'},
                                                                              {'members': [{'name': 'q-Hyst-r13',
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('dB0',
                                                                                                        0),
                                                                                                       ('dB1',
                                                                                                        1),
                                                                                                       ('dB2',
                                                                                                        2),
                                                                                                       ('dB3',
                                                                                                        3),
                                                                                                       ('dB4',
                                                                                                        4),
                                                                                                       ('dB5',
                                                                                                        5),
                                                                                                       ('dB6',
                                                                                                        6),
                                                                                                       ('dB8',
                                                                                                        7),
                                                                                                       ('dB10',
                                                                                                        8),
                                                                                                       ('dB12',
                                                                                                        9),
                                                                                                       ('dB14',
                                                                                                        10),
                                                                                                       ('dB16',
                                                                                                        11),
                                                                                                       ('dB18',
                                                                                                        12),
                                                                                                       ('dB20',
                                                                                                        13),
                                                                                                       ('dB22',
                                                                                                        14),
                                                                                                       ('dB24',
                                                                                                        15)]},
                                                                                           {'name': 'q-RxLevMin-r13',
                                                                                            'type': 'Q-RxLevMin'},
                                                                                           {'name': 't-ReselectionEUTRA-r13',
                                                                                            'type': 'T-Reselection'}],
                                                                               'name': 'cellReselectionInfo-r13',
                                                                               'optional': True,
                                                                               'type': 'SEQUENCE'},
                                                                              {'name': 'tdd-Config-r13',
                                                                               'optional': True,
                                                                               'type': 'TDD-Config'},
                                                                              {'members': [{'name': 'ul-CarrierFreq-r13',
                                                                                            'optional': True,
                                                                                            'type': 'ARFCN-ValueEUTRA'},
                                                                                           {'name': 'ul-Bandwidth-r13',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('n6',
                                                                                                        0),
                                                                                                       ('n15',
                                                                                                        1),
                                                                                                       ('n25',
                                                                                                        2),
                                                                                                       ('n50',
                                                                                                        3),
                                                                                                       ('n75',
                                                                                                        4),
                                                                                                       ('n100',
                                                                                                        5)]},
                                                                                           {'name': 'additionalSpectrumEmission-r13',
                                                                                            'optional': True,
                                                                                            'type': 'AdditionalSpectrumEmission'}],
                                                                               'name': 'freqInfo-r13',
                                                                               'optional': True,
                                                                               'type': 'SEQUENCE'},
                                                                              {'name': 'p-Max-r13',
                                                                               'optional': True,
                                                                               'type': 'P-Max'},
                                                                              {'name': 'referenceSignalPower-r13',
                                                                               'optional': True,
                                                                               'restricted-to': [(-60,
                                                                                                  50)],
                                                                               'type': 'INTEGER'},
                                                                              None,
                                                                              [{'members': [{'name': 'additionalSpectrumEmission-v1370',
                                                                                             'type': 'AdditionalSpectrumEmission-v10l0'}],
                                                                                'name': 'freqInfo-v1370',
                                                                                'optional': True,
                                                                                'type': 'SEQUENCE'}]],
                                                                  'type': 'SEQUENCE'},
                                     'SL-DiscSysInfoReportFreqList-r13': {'element': {'type': 'SL-DiscSysInfoReport-r13'},
                                                                          'size': [(1,
                                                                                    'maxSL-DiscSysInfoReportFreq-r13')],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                     'SL-DiscSysInfoToReportFreqList-r13': {'element': {'type': 'ARFCN-ValueEUTRA-r9'},
                                                                            'size': [(1,
                                                                                      'maxFreq')],
                                                                            'type': 'SEQUENCE '
                                                                                    'OF'},
                                     'SL-DiscTxConfigScheduled-r13': {'members': [{'name': 'discTxConfig-r13',
                                                                                   'optional': True,
                                                                                   'type': 'SL-DiscResourcePool-r12'},
                                                                                  {'name': 'discTF-IndexList-r13',
                                                                                   'optional': True,
                                                                                   'type': 'SL-TF-IndexPairList-r12b'},
                                                                                  {'name': 'discHoppingConfig-r13',
                                                                                   'optional': True,
                                                                                   'type': 'SL-HoppingConfigDisc-r12'},
                                                                                  None],
                                                                      'type': 'SEQUENCE'},
                                     'SL-DiscTxInfoInterFreqListAdd-r13': {'members': [{'element': {'type': 'SL-DiscTxResourceInfoPerFreq-r13'},
                                                                                        'name': 'discTxFreqToAddModList-r13',
                                                                                        'optional': True,
                                                                                        'size': [(1,
                                                                                                  'maxFreq')],
                                                                                        'type': 'SEQUENCE '
                                                                                                'OF'},
                                                                                       {'element': {'type': 'ARFCN-ValueEUTRA-r9'},
                                                                                        'name': 'discTxFreqToReleaseList-r13',
                                                                                        'optional': True,
                                                                                        'size': [(1,
                                                                                                  'maxFreq')],
                                                                                        'type': 'SEQUENCE '
                                                                                                'OF'},
                                                                                       None],
                                                                           'type': 'SEQUENCE'},
                                     'SL-DiscTxPoolDedicated-r13': {'members': [{'name': 'poolToReleaseList-r13',
                                                                                 'optional': True,
                                                                                 'type': 'SL-TxPoolToReleaseList-r12'},
                                                                                {'name': 'poolToAddModList-r13',
                                                                                 'optional': True,
                                                                                 'type': 'SL-DiscTxPoolToAddModList-r12'}],
                                                                    'type': 'SEQUENCE'},
                                     'SL-DiscTxPoolList-r12': {'element': {'type': 'SL-DiscResourcePool-r12'},
                                                               'size': [(1,
                                                                         'maxSL-TxPool-r12')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'SL-DiscTxPoolToAddMod-r12': {'members': [{'name': 'poolIdentity-r12',
                                                                                'type': 'SL-TxPoolIdentity-r12'},
                                                                               {'name': 'pool-r12',
                                                                                'type': 'SL-DiscResourcePool-r12'}],
                                                                   'type': 'SEQUENCE'},
                                     'SL-DiscTxPoolToAddModList-r12': {'element': {'type': 'SL-DiscTxPoolToAddMod-r12'},
                                                                       'size': [(1,
                                                                                 'maxSL-TxPool-r12')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'SL-DiscTxPowerInfo-r12': {'members': [{'name': 'discMaxTxPower-r12',
                                                                             'type': 'P-Max'},
                                                                            None],
                                                                'type': 'SEQUENCE'},
                                     'SL-DiscTxPowerInfoList-r12': {'element': {'type': 'SL-DiscTxPowerInfo-r12'},
                                                                    'size': ['maxSL-DiscPowerClass-r12'],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'SL-DiscTxRefCarrierDedicated-r13': {'members': [{'name': 'pCell',
                                                                                       'type': 'NULL'},
                                                                                      {'name': 'sCell',
                                                                                       'type': 'SCellIndex-r10'}],
                                                                          'type': 'CHOICE'},
                                     'SL-DiscTxResource-r13': {'members': [{'name': 'release',
                                                                            'type': 'NULL'},
                                                                           {'members': [{'name': 'scheduled-r13',
                                                                                         'type': 'SL-DiscTxConfigScheduled-r13'},
                                                                                        {'name': 'ue-Selected-r13',
                                                                                         'type': 'SL-DiscTxPoolDedicated-r13'}],
                                                                            'name': 'setup',
                                                                            'type': 'CHOICE'}],
                                                               'type': 'CHOICE'},
                                     'SL-DiscTxResourceInfoPerFreq-r13': {'members': [{'name': 'discTxCarrierFreq-r13',
                                                                                       'type': 'ARFCN-ValueEUTRA-r9'},
                                                                                      {'name': 'discTxResources-r13',
                                                                                       'optional': True,
                                                                                       'type': 'SL-DiscTxResource-r13'},
                                                                                      {'name': 'discTxResourcesPS-r13',
                                                                                       'optional': True,
                                                                                       'type': 'SL-DiscTxResource-r13'},
                                                                                      {'name': 'discTxRefCarrierDedicated-r13',
                                                                                       'optional': True,
                                                                                       'type': 'SL-DiscTxRefCarrierDedicated-r13'},
                                                                                      {'name': 'discCellSelectionInfo-r13',
                                                                                       'optional': True,
                                                                                       'type': 'CellSelectionInfoNFreq-r13'},
                                                                                      None],
                                                                          'type': 'SEQUENCE'},
                                     'SL-DiscTxResourceReq-r13': {'members': [{'name': 'carrierFreqDiscTx-r13',
                                                                               'optional': True,
                                                                               'restricted-to': [(1,
                                                                                                  'maxFreq')],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'discTxResourceReq-r13',
                                                                               'restricted-to': [(1,
                                                                                                  63)],
                                                                               'type': 'INTEGER'}],
                                                                  'type': 'SEQUENCE'},
                                     'SL-DiscTxResourceReqPerFreqList-r13': {'element': {'type': 'SL-DiscTxResourceReq-r13'},
                                                                             'size': [(1,
                                                                                       'maxFreq')],
                                                                             'type': 'SEQUENCE '
                                                                                     'OF'},
                                     'SL-DiscTxResourcesInterFreq-r13': {'members': [{'name': 'acquireSI-FromCarrier-r13',
                                                                                      'type': 'NULL'},
                                                                                     {'name': 'discTxPoolCommon-r13',
                                                                                      'type': 'SL-DiscTxPoolList-r12'},
                                                                                     {'name': 'requestDedicated-r13',
                                                                                      'type': 'NULL'},
                                                                                     {'name': 'noTxOnCarrier-r13',
                                                                                      'type': 'NULL'}],
                                                                         'type': 'CHOICE'},
                                     'SL-GapConfig-r13': {'members': [{'name': 'gapPatternList-r13',
                                                                       'type': 'SL-GapPatternList-r13'}],
                                                          'type': 'SEQUENCE'},
                                     'SL-GapFreqInfo-r13': {'members': [{'name': 'carrierFreq-r13',
                                                                         'optional': True,
                                                                         'type': 'ARFCN-ValueEUTRA-r9'},
                                                                        {'name': 'gapPatternList-r13',
                                                                         'type': 'SL-GapPatternList-r13'}],
                                                            'type': 'SEQUENCE'},
                                     'SL-GapPattern-r13': {'members': [{'name': 'gapPeriod-r13',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('sf40',
                                                                                    0),
                                                                                   ('sf60',
                                                                                    1),
                                                                                   ('sf70',
                                                                                    2),
                                                                                   ('sf80',
                                                                                    3),
                                                                                   ('sf120',
                                                                                    4),
                                                                                   ('sf140',
                                                                                    5),
                                                                                   ('sf160',
                                                                                    6),
                                                                                   ('sf240',
                                                                                    7),
                                                                                   ('sf280',
                                                                                    8),
                                                                                   ('sf320',
                                                                                    9),
                                                                                   ('sf640',
                                                                                    10),
                                                                                   ('sf1280',
                                                                                    11),
                                                                                   ('sf2560',
                                                                                    12),
                                                                                   ('sf5120',
                                                                                    13),
                                                                                   ('sf10240',
                                                                                    14)]},
                                                                       {'name': 'gapOffset-r12',
                                                                        'type': 'SL-OffsetIndicator-r12'},
                                                                       {'name': 'gapSubframeBitmap-r13',
                                                                        'size': [(1,
                                                                                  10240)],
                                                                        'type': 'BIT '
                                                                                'STRING'},
                                                                       None],
                                                           'type': 'SEQUENCE'},
                                     'SL-GapPatternList-r13': {'element': {'type': 'SL-GapPattern-r13'},
                                                               'size': [(1,
                                                                         'maxSL-GP-r13')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'SL-GapRequest-r13': {'element': {'type': 'SL-GapFreqInfo-r13'},
                                                           'size': [(1,
                                                                     'maxFreq')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                     'SL-HoppingConfigComm-r12': {'members': [{'name': 'hoppingParameter-r12',
                                                                               'restricted-to': [(0,
                                                                                                  504)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'numSubbands-r12',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('ns1',
                                                                                           0),
                                                                                          ('ns2',
                                                                                           1),
                                                                                          ('ns4',
                                                                                           2)]},
                                                                              {'name': 'rb-Offset-r12',
                                                                               'restricted-to': [(0,
                                                                                                  110)],
                                                                               'type': 'INTEGER'}],
                                                                  'type': 'SEQUENCE'},
                                     'SL-HoppingConfigDisc-r12': {'members': [{'name': 'a-r12',
                                                                               'restricted-to': [(1,
                                                                                                  200)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'b-r12',
                                                                               'restricted-to': [(1,
                                                                                                  10)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'c-r12',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n1',
                                                                                           0),
                                                                                          ('n5',
                                                                                           1)]}],
                                                                  'type': 'SEQUENCE'},
                                     'SL-InterFreqInfoListV2X-r14': {'element': {'type': 'SL-InterFreqInfoV2X-r14'},
                                                                     'size': [(0,
                                                                               'maxFreqV2X-1-r14')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'SL-InterFreqInfoV2X-r14': {'members': [{'name': 'plmn-IdentityList-r14',
                                                                              'optional': True,
                                                                              'type': 'PLMN-IdentityList'},
                                                                             {'name': 'v2x-CommCarrierFreq-r14',
                                                                              'type': 'ARFCN-ValueEUTRA-r9'},
                                                                             {'name': 'sl-MaxTxPower-r14',
                                                                              'optional': True,
                                                                              'type': 'P-Max'},
                                                                             {'name': 'sl-Bandwidth-r14',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('n6',
                                                                                          0),
                                                                                         ('n15',
                                                                                          1),
                                                                                         ('n25',
                                                                                          2),
                                                                                         ('n50',
                                                                                          3),
                                                                                         ('n75',
                                                                                          4),
                                                                                         ('n100',
                                                                                          5)]},
                                                                             {'name': 'v2x-SchedulingPool-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-CommResourcePoolV2X-r14'},
                                                                             {'name': 'v2x-UE-ConfigList-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-V2X-UE-ConfigList-r14'},
                                                                             None,
                                                                             [{'members': [{'name': 'additionalSpectrumEmission-r14',
                                                                                            'type': 'AdditionalSpectrumEmission'},
                                                                                           {'name': 'additionalSpectrumEmission-v1440',
                                                                                            'type': 'AdditionalSpectrumEmission-v10l0'}],
                                                                               'name': 'additionalSpectrumEmissionV2X-r14',
                                                                               'optional': True,
                                                                               'type': 'CHOICE'}]],
                                                                 'type': 'SEQUENCE'},
                                     'SL-OffsetIndicator-r12': {'members': [{'name': 'small-r12',
                                                                             'restricted-to': [(0,
                                                                                                319)],
                                                                             'type': 'INTEGER'},
                                                                            {'name': 'large-r12',
                                                                             'restricted-to': [(0,
                                                                                                10239)],
                                                                             'type': 'INTEGER'}],
                                                                'type': 'CHOICE'},
                                     'SL-OffsetIndicatorSync-r12': {'restricted-to': [(0,
                                                                                       39)],
                                                                    'type': 'INTEGER'},
                                     'SL-OffsetIndicatorSync-r14': {'restricted-to': [(0,
                                                                                       159)],
                                                                    'type': 'INTEGER'},
                                     'SL-OffsetIndicatorSync-v1430': {'restricted-to': [(40,
                                                                                         159)],
                                                                      'type': 'INTEGER'},
                                     'SL-P2X-ResourceSelectionConfig-r14': {'members': [{'name': 'partialSensing-r14',
                                                                                         'optional': True,
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('true',
                                                                                                     0)]},
                                                                                        {'name': 'randomSelection-r14',
                                                                                         'optional': True,
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('true',
                                                                                                     0)]}],
                                                                            'type': 'SEQUENCE'},
                                     'SL-PPPP-TxConfigIndex-r14': {'members': [{'name': 'priorityThreshold-r14',
                                                                                'type': 'SL-Priority-r13'},
                                                                               {'name': 'defaultTxConfigIndex-r14',
                                                                                'restricted-to': [(0,
                                                                                                   'maxCBR-Level-1-r14')],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'cbr-ConfigIndex-r14',
                                                                                'restricted-to': [(0,
                                                                                                   'maxSL-V2X-CBRConfig-1-r14')],
                                                                                'type': 'INTEGER'},
                                                                               {'element': {'type': 'Tx-ConfigIndex-r14'},
                                                                                'name': 'tx-ConfigIndexList-r14',
                                                                                'size': [(1,
                                                                                          'maxCBR-Level-r14')],
                                                                                'type': 'SEQUENCE '
                                                                                        'OF'}],
                                                                   'type': 'SEQUENCE'},
                                     'SL-PSSCH-TxConfig-r14': {'members': [{'name': 'typeTxSync-r14',
                                                                            'optional': True,
                                                                            'type': 'SL-TypeTxSync-r14'},
                                                                           {'name': 'thresUE-Speed-r14',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('kmph60',
                                                                                        0),
                                                                                       ('kmph80',
                                                                                        1),
                                                                                       ('kmph100',
                                                                                        2),
                                                                                       ('kmph120',
                                                                                        3),
                                                                                       ('kmph140',
                                                                                        4),
                                                                                       ('kmph160',
                                                                                        5),
                                                                                       ('kmph180',
                                                                                        6),
                                                                                       ('kmph200',
                                                                                        7)]},
                                                                           {'name': 'parametersAboveThres-r14',
                                                                            'type': 'SL-PSSCH-TxParameters-r14'},
                                                                           {'name': 'parametersBelowThres-r14',
                                                                            'type': 'SL-PSSCH-TxParameters-r14'},
                                                                           None],
                                                               'type': 'SEQUENCE'},
                                     'SL-PSSCH-TxConfigList-r14': {'element': {'type': 'SL-PSSCH-TxConfig-r14'},
                                                                   'size': [(1,
                                                                             'maxPSSCH-TxConfig-r14')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'SL-PSSCH-TxParameters-r14': {'members': [{'name': 'minMCS-PSSCH-r14',
                                                                                'restricted-to': [(0,
                                                                                                   31)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'maxMCS-PSSCH-r14',
                                                                                'restricted-to': [(0,
                                                                                                   31)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'minSubChannel-NumberPSSCH-r14',
                                                                                'restricted-to': [(1,
                                                                                                   20)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'maxSubchannel-NumberPSSCH-r14',
                                                                                'restricted-to': [(1,
                                                                                                   20)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'allowedRetxNumberPSSCH-r14',
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('n0',
                                                                                            0),
                                                                                           ('n1',
                                                                                            1),
                                                                                           ('both',
                                                                                            2),
                                                                                           ('spare1',
                                                                                            3)]},
                                                                               {'name': 'maxTxPower-r14',
                                                                                'optional': True,
                                                                                'type': 'SL-TxPower-r14'}],
                                                                   'type': 'SEQUENCE'},
                                     'SL-Parameters-r12': {'members': [{'name': 'commSimultaneousTx-r12',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('supported',
                                                                                    0)]},
                                                                       {'name': 'commSupportedBands-r12',
                                                                        'optional': True,
                                                                        'type': 'FreqBandIndicatorListEUTRA-r12'},
                                                                       {'name': 'discSupportedBands-r12',
                                                                        'optional': True,
                                                                        'type': 'SupportedBandInfoList-r12'},
                                                                       {'name': 'discScheduledResourceAlloc-r12',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('supported',
                                                                                    0)]},
                                                                       {'name': 'disc-UE-SelectedResourceAlloc-r12',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('supported',
                                                                                    0)]},
                                                                       {'name': 'disc-SLSS-r12',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('supported',
                                                                                    0)]},
                                                                       {'name': 'discSupportedProc-r12',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('n50',
                                                                                    0),
                                                                                   ('n400',
                                                                                    1)]}],
                                                           'type': 'SEQUENCE'},
                                     'SL-Parameters-v1310': {'members': [{'name': 'discSysInfoReporting-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'commMultipleTx-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'discInterFreqTx-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'discPeriodicSLSS-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]}],
                                                             'type': 'SEQUENCE'},
                                     'SL-Parameters-v1430': {'members': [{'name': 'zoneBasedPoolSelection-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'ue-AutonomousWithFullSensing-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'ue-AutonomousWithPartialSensing-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'sl-CongestionControl-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'v2x-TxWithShortResvInterval-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'v2x-numberTxRxTiming-r14',
                                                                          'optional': True,
                                                                          'restricted-to': [(1,
                                                                                             16)],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'v2x-nonAdjacentPSCCH-PSSCH-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'slss-TxRx-r14',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('supported',
                                                                                      0)]},
                                                                         {'name': 'v2x-SupportedBandCombinationList-r14',
                                                                          'optional': True,
                                                                          'type': 'V2X-SupportedBandCombination-r14'}],
                                                             'type': 'SEQUENCE'},
                                     'SL-PeriodComm-r12': {'type': 'ENUMERATED',
                                                           'values': [('sf40',
                                                                       0),
                                                                      ('sf60',
                                                                       1),
                                                                      ('sf70',
                                                                       2),
                                                                      ('sf80',
                                                                       3),
                                                                      ('sf120',
                                                                       4),
                                                                      ('sf140',
                                                                       5),
                                                                      ('sf160',
                                                                       6),
                                                                      ('sf240',
                                                                       7),
                                                                      ('sf280',
                                                                       8),
                                                                      ('sf320',
                                                                       9),
                                                                      ('spare6',
                                                                       10),
                                                                      ('spare5',
                                                                       11),
                                                                      ('spare4',
                                                                       12),
                                                                      ('spare3',
                                                                       13),
                                                                      ('spare2',
                                                                       14),
                                                                      ('spare',
                                                                       15)]},
                                     'SL-PoolSelectionConfig-r12': {'members': [{'name': 'threshLow-r12',
                                                                                 'type': 'RSRP-RangeSL2-r12'},
                                                                                {'name': 'threshHigh-r12',
                                                                                 'type': 'RSRP-RangeSL2-r12'}],
                                                                    'type': 'SEQUENCE'},
                                     'SL-Priority-r13': {'restricted-to': [(1,
                                                                            8)],
                                                         'type': 'INTEGER'},
                                     'SL-PriorityList-r13': {'element': {'type': 'SL-Priority-r13'},
                                                             'size': [(1,
                                                                       'maxSL-Prio-r13')],
                                                             'type': 'SEQUENCE '
                                                                     'OF'},
                                     'SL-ResourcesInterFreq-r13': {'members': [{'name': 'discRxResourcesInterFreq-r13',
                                                                                'optional': True,
                                                                                'type': 'SL-DiscRxPoolList-r12'},
                                                                               {'name': 'discTxResourcesInterFreq-r13',
                                                                                'optional': True,
                                                                                'type': 'SL-DiscTxResourcesInterFreq-r13'}],
                                                                   'type': 'SEQUENCE'},
                                     'SL-RestrictResourceReservationPeriod-r14': {'type': 'ENUMERATED',
                                                                                  'values': [('v0dot2',
                                                                                              0),
                                                                                             ('v0dot5',
                                                                                              1),
                                                                                             ('v1',
                                                                                              2),
                                                                                             ('v2',
                                                                                              3),
                                                                                             ('v3',
                                                                                              4),
                                                                                             ('v4',
                                                                                              5),
                                                                                             ('v5',
                                                                                              6),
                                                                                             ('v6',
                                                                                              7),
                                                                                             ('v7',
                                                                                              8),
                                                                                             ('v8',
                                                                                              9),
                                                                                             ('v9',
                                                                                              10),
                                                                                             ('v10',
                                                                                              11),
                                                                                             ('spare4',
                                                                                              12),
                                                                                             ('spare3',
                                                                                              13),
                                                                                             ('spare2',
                                                                                              14),
                                                                                             ('spare1',
                                                                                              15)]},
                                     'SL-RestrictResourceReservationPeriodList-r14': {'element': {'type': 'SL-RestrictResourceReservationPeriod-r14'},
                                                                                      'size': [(1,
                                                                                                'maxReservationPeriod-r14')],
                                                                                      'type': 'SEQUENCE '
                                                                                              'OF'},
                                     'SL-SyncAllowed-r14': {'members': [{'name': 'gnss-Sync-r14',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('true',
                                                                                     0)]},
                                                                        {'name': 'enb-Sync-r14',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('true',
                                                                                     0)]},
                                                                        {'name': 'ue-Sync-r14',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('true',
                                                                                     0)]}],
                                                            'type': 'SEQUENCE'},
                                     'SL-SyncConfig-r12': {'members': [{'name': 'syncCP-Len-r12',
                                                                        'type': 'SL-CP-Len-r12'},
                                                                       {'name': 'syncOffsetIndicator-r12',
                                                                        'type': 'SL-OffsetIndicatorSync-r12'},
                                                                       {'name': 'slssid-r12',
                                                                        'type': 'SLSSID-r12'},
                                                                       {'members': [{'name': 'syncTxParameters-r12',
                                                                                     'type': 'SL-TxParameters-r12'},
                                                                                    {'name': 'syncTxThreshIC-r12',
                                                                                     'type': 'RSRP-RangeSL-r12'},
                                                                                    {'name': 'syncInfoReserved-r12',
                                                                                     'optional': True,
                                                                                     'size': [19],
                                                                                     'type': 'BIT '
                                                                                             'STRING'}],
                                                                        'name': 'txParameters-r12',
                                                                        'optional': True,
                                                                        'type': 'SEQUENCE'},
                                                                       {'members': [{'name': 'physCellId-r12',
                                                                                     'type': 'PhysCellId'},
                                                                                    {'name': 'discSyncWindow-r12',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('w1',
                                                                                                 0),
                                                                                                ('w2',
                                                                                                 1)]}],
                                                                        'name': 'rxParamsNCell-r12',
                                                                        'optional': True,
                                                                        'type': 'SEQUENCE'},
                                                                       None,
                                                                       [{'name': 'syncTxPeriodic-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('true',
                                                                                     0)]}],
                                                                       [{'name': 'syncOffsetIndicator-v1430',
                                                                         'optional': True,
                                                                         'type': 'SL-OffsetIndicatorSync-v1430'},
                                                                        {'name': 'gnss-Sync-r14',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('true',
                                                                                     0)]}]],
                                                           'type': 'SEQUENCE'},
                                     'SL-SyncConfigList-r12': {'element': {'type': 'SL-SyncConfig-r12'},
                                                               'size': [(1,
                                                                         'maxSL-SyncConfig-r12')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'SL-SyncConfigListNFreq-r13': {'element': {'type': 'SL-SyncConfigNFreq-r13'},
                                                                    'size': [(1,
                                                                              'maxSL-SyncConfig-r12')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'SL-SyncConfigListNFreqV2X-r14': {'element': {'type': 'SL-SyncConfigNFreq-r13'},
                                                                       'size': [(1,
                                                                                 'maxSL-V2X-SyncConfig-r14')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'SL-SyncConfigListV2X-r14': {'element': {'type': 'SL-SyncConfig-r12'},
                                                                  'size': [(1,
                                                                            'maxSL-V2X-SyncConfig-r14')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'SL-SyncConfigNFreq-r13': {'members': [{'members': [{'name': 'syncCP-Len-r13',
                                                                                          'type': 'SL-CP-Len-r12'},
                                                                                         {'name': 'syncOffsetIndicator-r13',
                                                                                          'type': 'SL-OffsetIndicatorSync-r12'},
                                                                                         {'name': 'slssid-r13',
                                                                                          'type': 'SLSSID-r12'}],
                                                                             'name': 'asyncParameters-r13',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'syncTxParameters-r13',
                                                                                          'type': 'SL-TxParameters-r12'},
                                                                                         {'name': 'syncTxThreshIC-r13',
                                                                                          'type': 'RSRP-RangeSL-r12'},
                                                                                         {'name': 'syncInfoReserved-r13',
                                                                                          'optional': True,
                                                                                          'size': [19],
                                                                                          'type': 'BIT '
                                                                                                  'STRING'},
                                                                                         {'name': 'syncTxPeriodic-r13',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('true',
                                                                                                      0)]}],
                                                                             'name': 'txParameters-r13',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'discSyncWindow-r13',
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('w1',
                                                                                                      0),
                                                                                                     ('w2',
                                                                                                      1)]}],
                                                                             'name': 'rxParameters-r13',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            None,
                                                                            [{'name': 'syncOffsetIndicator-v1430',
                                                                              'optional': True,
                                                                              'type': 'SL-OffsetIndicatorSync-v1430'},
                                                                             {'name': 'gnss-Sync-r14',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('true',
                                                                                          0)]}]],
                                                                'type': 'SEQUENCE'},
                                     'SL-SyncTxControl-r12': {'members': [{'name': 'networkControlledSyncTx-r12',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('on',
                                                                                       0),
                                                                                      ('off',
                                                                                       1)]}],
                                                              'type': 'SEQUENCE'},
                                     'SL-TF-IndexPair-r12': {'members': [{'name': 'discSF-Index-r12',
                                                                          'optional': True,
                                                                          'restricted-to': [(1,
                                                                                             200)],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'discPRB-Index-r12',
                                                                          'optional': True,
                                                                          'restricted-to': [(1,
                                                                                             50)],
                                                                          'type': 'INTEGER'}],
                                                             'type': 'SEQUENCE'},
                                     'SL-TF-IndexPair-r12b': {'members': [{'name': 'discSF-Index-r12b',
                                                                           'optional': True,
                                                                           'restricted-to': [(0,
                                                                                              209)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'discPRB-Index-r12b',
                                                                           'optional': True,
                                                                           'restricted-to': [(0,
                                                                                              49)],
                                                                           'type': 'INTEGER'}],
                                                              'type': 'SEQUENCE'},
                                     'SL-TF-IndexPairList-r12': {'element': {'type': 'SL-TF-IndexPair-r12'},
                                                                 'size': [(1,
                                                                           'maxSL-TF-IndexPair-r12')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'SL-TF-IndexPairList-r12b': {'element': {'type': 'SL-TF-IndexPair-r12b'},
                                                                  'size': [(1,
                                                                            'maxSL-TF-IndexPair-r12')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'SL-TF-ResourceConfig-r12': {'members': [{'name': 'prb-Num-r12',
                                                                               'restricted-to': [(1,
                                                                                                  100)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'prb-Start-r12',
                                                                               'restricted-to': [(0,
                                                                                                  99)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'prb-End-r12',
                                                                               'restricted-to': [(0,
                                                                                                  99)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'offsetIndicator-r12',
                                                                               'type': 'SL-OffsetIndicator-r12'},
                                                                              {'name': 'subframeBitmap-r12',
                                                                               'type': 'SubframeBitmapSL-r12'}],
                                                                  'type': 'SEQUENCE'},
                                     'SL-TRPT-Subset-r12': {'size': [(3, 5)],
                                                            'type': 'BIT '
                                                                    'STRING'},
                                     'SL-ThresPSSCH-RSRP-List-r14': {'element': {'type': 'SL-ThresPSSCH-RSRP-r14'},
                                                                     'size': [64],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'SL-ThresPSSCH-RSRP-r14': {'restricted-to': [(0,
                                                                                   66)],
                                                                'type': 'INTEGER'},
                                     'SL-TxParameters-r12': {'members': [{'name': 'alpha-r12',
                                                                          'type': 'Alpha-r12'},
                                                                         {'name': 'p0-r12',
                                                                          'type': 'P0-SL-r12'}],
                                                             'type': 'SEQUENCE'},
                                     'SL-TxPoolIdentity-r12': {'restricted-to': [(1,
                                                                                  'maxSL-TxPool-r12')],
                                                               'type': 'INTEGER'},
                                     'SL-TxPoolIdentity-v1310': {'restricted-to': [('maxSL-TxPool-r12Plus1-r13',
                                                                                    'maxSL-TxPool-r13')],
                                                                 'type': 'INTEGER'},
                                     'SL-TxPoolToAddMod-r14': {'members': [{'name': 'poolIdentity-r14',
                                                                            'type': 'SL-V2X-TxPoolIdentity-r14'},
                                                                           {'name': 'pool-r14',
                                                                            'type': 'SL-CommResourcePoolV2X-r14'}],
                                                               'type': 'SEQUENCE'},
                                     'SL-TxPoolToAddModListV2X-r14': {'element': {'type': 'SL-TxPoolToAddMod-r14'},
                                                                      'size': [(1,
                                                                                'maxSL-V2X-TxPool-r14')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'SL-TxPoolToReleaseList-r12': {'element': {'type': 'SL-TxPoolIdentity-r12'},
                                                                    'size': [(1,
                                                                              'maxSL-TxPool-r12')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'SL-TxPoolToReleaseListExt-r13': {'element': {'type': 'SL-TxPoolIdentity-v1310'},
                                                                       'size': [(1,
                                                                                 'maxSL-TxPool-v1310')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'SL-TxPoolToReleaseListV2X-r14': {'element': {'type': 'SL-V2X-TxPoolIdentity-r14'},
                                                                       'size': [(1,
                                                                                 'maxSL-V2X-TxPool-r14')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'SL-TxPower-r14': {'members': [{'name': 'minusinfinity-r14',
                                                                     'type': 'NULL'},
                                                                    {'name': 'txPower-r14',
                                                                     'restricted-to': [(-41,
                                                                                        31)],
                                                                     'type': 'INTEGER'}],
                                                        'type': 'CHOICE'},
                                     'SL-TypeTxSync-r14': {'type': 'ENUMERATED',
                                                           'values': [('gnss',
                                                                       0),
                                                                      ('enb',
                                                                       1),
                                                                      ('ue',
                                                                       2)]},
                                     'SL-V2X-CommFreqList-r14': {'element': {'restricted-to': [(0,
                                                                                                'maxFreqV2X-1-r14')],
                                                                             'type': 'INTEGER'},
                                                                 'size': [(1,
                                                                           'maxFreqV2X-r14')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'SL-V2X-CommTxFreqList-r14': {'element': {'type': 'SL-V2X-CommTxResourceReq-r14'},
                                                                   'size': [(1,
                                                                             'maxFreqV2X-r14')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'SL-V2X-CommTxResourceReq-r14': {'members': [{'name': 'carrierFreqCommTx-r14',
                                                                                   'optional': True,
                                                                                   'restricted-to': [(0,
                                                                                                      'maxFreqV2X-1-r14')],
                                                                                   'type': 'INTEGER'},
                                                                                  {'name': 'v2x-TypeTxSync-r14',
                                                                                   'optional': True,
                                                                                   'type': 'SL-TypeTxSync-r14'},
                                                                                  {'name': 'v2x-DestinationInfoList-r14',
                                                                                   'optional': True,
                                                                                   'type': 'SL-DestinationInfoList-r12'}],
                                                                      'type': 'SEQUENCE'},
                                     'SL-V2X-ConfigCommon-r14': {'members': [{'name': 'v2x-CommRxPool-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-CommRxPoolListV2X-r14'},
                                                                             {'name': 'v2x-CommTxPoolNormalCommon-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-CommTxPoolListV2X-r14'},
                                                                             {'name': 'p2x-CommTxPoolNormalCommon-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-CommTxPoolListV2X-r14'},
                                                                             {'name': 'v2x-CommTxPoolExceptional-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-CommResourcePoolV2X-r14'},
                                                                             {'name': 'v2x-SyncConfig-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-SyncConfigListV2X-r14'},
                                                                             {'name': 'v2x-InterFreqInfoList-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-InterFreqInfoListV2X-r14'},
                                                                             {'name': 'v2x-ResourceSelectionConfig-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-CommTxPoolSensingConfig-r14'},
                                                                             {'name': 'zoneConfig-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-ZoneConfig-r14'},
                                                                             {'name': 'typeTxSync-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-TypeTxSync-r14'},
                                                                             {'name': 'thresSL-TxPrioritization-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-Priority-r13'},
                                                                             {'name': 'anchorCarrierFreqList-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-AnchorCarrierFreqList-V2X-r14'},
                                                                             {'name': 'offsetDFN-r14',
                                                                              'optional': True,
                                                                              'restricted-to': [(0,
                                                                                                 1000)],
                                                                              'type': 'INTEGER'},
                                                                             {'name': 'cbr-CommonTxConfigList-r14',
                                                                              'optional': True,
                                                                              'type': 'SL-CBR-CommonTxConfigList-r14'}],
                                                                 'type': 'SEQUENCE'},
                                     'SL-V2X-ConfigDedicated-r14': {'members': [{'members': [{'name': 'release',
                                                                                              'type': 'NULL'},
                                                                                             {'members': [{'members': [{'name': 'sl-V-RNTI-r14',
                                                                                                                        'type': 'C-RNTI'},
                                                                                                                       {'name': 'mac-MainConfig-r14',
                                                                                                                        'type': 'MAC-MainConfigSL-r12'},
                                                                                                                       {'name': 'v2x-SchedulingPool-r14',
                                                                                                                        'optional': True,
                                                                                                                        'type': 'SL-CommResourcePoolV2X-r14'},
                                                                                                                       {'name': 'mcs-r14',
                                                                                                                        'optional': True,
                                                                                                                        'restricted-to': [(0,
                                                                                                                                           31)],
                                                                                                                        'type': 'INTEGER'},
                                                                                                                       {'name': 'logicalChGroupInfoList-r14',
                                                                                                                        'type': 'LogicalChGroupInfoList-r13'}],
                                                                                                           'name': 'scheduled-r14',
                                                                                                           'type': 'SEQUENCE'},
                                                                                                          {'members': [{'members': [{'name': 'poolToReleaseList-r14',
                                                                                                                                     'optional': True,
                                                                                                                                     'type': 'SL-TxPoolToReleaseListV2X-r14'},
                                                                                                                                    {'name': 'poolToAddModList-r14',
                                                                                                                                     'optional': True,
                                                                                                                                     'type': 'SL-TxPoolToAddModListV2X-r14'},
                                                                                                                                    {'name': 'v2x-CommTxPoolSensingConfig-r14',
                                                                                                                                     'optional': True,
                                                                                                                                     'type': 'SL-CommTxPoolSensingConfig-r14'}],
                                                                                                                        'name': 'v2x-CommTxPoolNormalDedicated-r14',
                                                                                                                        'type': 'SEQUENCE'}],
                                                                                                           'name': 'ue-Selected-r14',
                                                                                                           'type': 'SEQUENCE'}],
                                                                                              'name': 'setup',
                                                                                              'type': 'CHOICE'}],
                                                                                 'name': 'commTxResources-r14',
                                                                                 'optional': True,
                                                                                 'type': 'CHOICE'},
                                                                                {'name': 'v2x-InterFreqInfoList-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-InterFreqInfoListV2X-r14'},
                                                                                {'name': 'thresSL-TxPrioritization-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-Priority-r13'},
                                                                                {'name': 'typeTxSync-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-TypeTxSync-r14'},
                                                                                {'name': 'cbr-DedicatedTxConfigList-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SL-CBR-CommonTxConfigList-r14'},
                                                                                None],
                                                                    'type': 'SEQUENCE'},
                                     'SL-V2X-InterFreqUE-Config-r14': {'members': [{'name': 'physCellIdList-r14',
                                                                                    'optional': True,
                                                                                    'type': 'PhysCellIdList-r13'},
                                                                                   {'name': 'typeTxSync-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-TypeTxSync-r14'},
                                                                                   {'name': 'v2x-SyncConfig-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-SyncConfigListNFreqV2X-r14'},
                                                                                   {'name': 'v2x-CommRxPool-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-CommRxPoolListV2X-r14'},
                                                                                   {'name': 'v2x-CommTxPoolNormal-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-CommTxPoolListV2X-r14'},
                                                                                   {'name': 'p2x-CommTxPoolNormal-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-CommTxPoolListV2X-r14'},
                                                                                   {'name': 'v2x-CommTxPoolExceptional-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-CommResourcePoolV2X-r14'},
                                                                                   {'name': 'v2x-ResourceSelectionConfig-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-CommTxPoolSensingConfig-r14'},
                                                                                   {'name': 'zoneConfig-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-ZoneConfig-r14'},
                                                                                   {'name': 'offsetDFN-r14',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(0,
                                                                                                       1000)],
                                                                                    'type': 'INTEGER'},
                                                                                   None],
                                                                       'type': 'SEQUENCE'},
                                     'SL-V2X-TxPoolIdentity-r14': {'restricted-to': [(1,
                                                                                      'maxSL-V2X-TxPool-r14')],
                                                                   'type': 'INTEGER'},
                                     'SL-V2X-TxPoolReportIdentity-r14': {'restricted-to': [(1,
                                                                                            'maxSL-PoolToMeasure-r14')],
                                                                         'type': 'INTEGER'},
                                     'SL-V2X-UE-ConfigList-r14': {'element': {'type': 'SL-V2X-InterFreqUE-Config-r14'},
                                                                  'size': [(1,
                                                                            'maxCellIntra')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'SL-ZoneConfig-r14': {'members': [{'name': 'zoneLength-r14',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('m5',
                                                                                    0),
                                                                                   ('m10',
                                                                                    1),
                                                                                   ('m20',
                                                                                    2),
                                                                                   ('m50',
                                                                                    3),
                                                                                   ('m100',
                                                                                    4),
                                                                                   ('m200',
                                                                                    5),
                                                                                   ('m500',
                                                                                    6),
                                                                                   ('spare1',
                                                                                    7)]},
                                                                       {'name': 'zoneWidth-r14',
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('m5',
                                                                                    0),
                                                                                   ('m10',
                                                                                    1),
                                                                                   ('m20',
                                                                                    2),
                                                                                   ('m50',
                                                                                    3),
                                                                                   ('m100',
                                                                                    4),
                                                                                   ('m200',
                                                                                    5),
                                                                                   ('m500',
                                                                                    6),
                                                                                   ('spare1',
                                                                                    7)]},
                                                                       {'name': 'zoneIdLongiMod-r14',
                                                                        'restricted-to': [(1,
                                                                                           4)],
                                                                        'type': 'INTEGER'},
                                                                       {'name': 'zoneIdLatiMod-r14',
                                                                        'restricted-to': [(1,
                                                                                           4)],
                                                                        'type': 'INTEGER'}],
                                                           'type': 'SEQUENCE'},
                                     'SLSSID-r12': {'restricted-to': [(0, 167)],
                                                    'type': 'INTEGER'},
                                     'SN-FieldLength': {'type': 'ENUMERATED',
                                                        'values': [('size5', 0),
                                                                   ('size10',
                                                                    1)]},
                                     'SON-Parameters-r9': {'members': [{'name': 'rach-Report-r9',
                                                                        'optional': True,
                                                                        'type': 'ENUMERATED',
                                                                        'values': [('supported',
                                                                                    0)]}],
                                                           'type': 'SEQUENCE'},
                                     'SPS-Config': {'members': [{'name': 'semiPersistSchedC-RNTI',
                                                                 'optional': True,
                                                                 'type': 'C-RNTI'},
                                                                {'name': 'sps-ConfigDL',
                                                                 'optional': True,
                                                                 'type': 'SPS-ConfigDL'},
                                                                {'name': 'sps-ConfigUL',
                                                                 'optional': True,
                                                                 'type': 'SPS-ConfigUL'}],
                                                    'type': 'SEQUENCE'},
                                     'SPS-Config-v1430': {'members': [{'name': 'ul-SPS-V-RNTI-r14',
                                                                       'optional': True,
                                                                       'type': 'C-RNTI'},
                                                                      {'name': 'sl-SPS-V-RNTI-r14',
                                                                       'optional': True,
                                                                       'type': 'C-RNTI'},
                                                                      {'name': 'sps-ConfigUL-ToAddModList-r14',
                                                                       'optional': True,
                                                                       'type': 'SPS-ConfigUL-ToAddModList-r14'},
                                                                      {'name': 'sps-ConfigUL-ToReleaseList-r14',
                                                                       'optional': True,
                                                                       'type': 'SPS-ConfigUL-ToReleaseList-r14'},
                                                                      {'name': 'sps-ConfigSL-ToAddModList-r14',
                                                                       'optional': True,
                                                                       'type': 'SPS-ConfigSL-ToAddModList-r14'},
                                                                      {'name': 'sps-ConfigSL-ToReleaseList-r14',
                                                                       'optional': True,
                                                                       'type': 'SPS-ConfigSL-ToReleaseList-r14'}],
                                                          'type': 'SEQUENCE'},
                                     'SPS-ConfigDL': {'members': [{'name': 'release',
                                                                   'type': 'NULL'},
                                                                  {'members': [{'name': 'semiPersistSchedIntervalDL',
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('sf10',
                                                                                            0),
                                                                                           ('sf20',
                                                                                            1),
                                                                                           ('sf32',
                                                                                            2),
                                                                                           ('sf40',
                                                                                            3),
                                                                                           ('sf64',
                                                                                            4),
                                                                                           ('sf80',
                                                                                            5),
                                                                                           ('sf128',
                                                                                            6),
                                                                                           ('sf160',
                                                                                            7),
                                                                                           ('sf320',
                                                                                            8),
                                                                                           ('sf640',
                                                                                            9),
                                                                                           ('spare6',
                                                                                            10),
                                                                                           ('spare5',
                                                                                            11),
                                                                                           ('spare4',
                                                                                            12),
                                                                                           ('spare3',
                                                                                            13),
                                                                                           ('spare2',
                                                                                            14),
                                                                                           ('spare1',
                                                                                            15)]},
                                                                               {'name': 'numberOfConfSPS-Processes',
                                                                                'restricted-to': [(1,
                                                                                                   8)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'n1PUCCH-AN-PersistentList',
                                                                                'type': 'N1PUCCH-AN-PersistentList'},
                                                                               None,
                                                                               [{'members': [{'name': 'release',
                                                                                              'type': 'NULL'},
                                                                                             {'members': [{'name': 'n1PUCCH-AN-PersistentListP1-r10',
                                                                                                           'type': 'N1PUCCH-AN-PersistentList'}],
                                                                                              'name': 'setup',
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'name': 'twoAntennaPortActivated-r10',
                                                                                 'optional': True,
                                                                                 'type': 'CHOICE'}]],
                                                                   'name': 'setup',
                                                                   'type': 'SEQUENCE'}],
                                                      'type': 'CHOICE'},
                                     'SPS-ConfigIndex-r14': {'restricted-to': [(1,
                                                                                'maxConfigSPS-r14')],
                                                             'type': 'INTEGER'},
                                     'SPS-ConfigSL-ToAddModList-r14': {'element': {'type': 'SPS-ConfigSL-r14'},
                                                                       'size': [(1,
                                                                                 'maxConfigSPS-r14')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'SPS-ConfigSL-ToReleaseList-r14': {'element': {'type': 'SPS-ConfigIndex-r14'},
                                                                        'size': [(1,
                                                                                  'maxConfigSPS-r14')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'SPS-ConfigSL-r14': {'members': [{'name': 'sps-ConfigIndex-r14',
                                                                       'type': 'SPS-ConfigIndex-r14'},
                                                                      {'name': 'semiPersistSchedIntervalSL-r14',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('sf20',
                                                                                   0),
                                                                                  ('sf50',
                                                                                   1),
                                                                                  ('sf100',
                                                                                   2),
                                                                                  ('sf200',
                                                                                   3),
                                                                                  ('sf300',
                                                                                   4),
                                                                                  ('sf400',
                                                                                   5),
                                                                                  ('sf500',
                                                                                   6),
                                                                                  ('sf600',
                                                                                   7),
                                                                                  ('sf700',
                                                                                   8),
                                                                                  ('sf800',
                                                                                   9),
                                                                                  ('sf900',
                                                                                   10),
                                                                                  ('sf1000',
                                                                                   11),
                                                                                  ('spare4',
                                                                                   12),
                                                                                  ('spare3',
                                                                                   13),
                                                                                  ('spare2',
                                                                                   14),
                                                                                  ('spare1',
                                                                                   15)]}],
                                                          'type': 'SEQUENCE'},
                                     'SPS-ConfigUL': {'members': [{'name': 'release',
                                                                   'type': 'NULL'},
                                                                  {'members': [{'name': 'semiPersistSchedIntervalUL',
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('sf10',
                                                                                            0),
                                                                                           ('sf20',
                                                                                            1),
                                                                                           ('sf32',
                                                                                            2),
                                                                                           ('sf40',
                                                                                            3),
                                                                                           ('sf64',
                                                                                            4),
                                                                                           ('sf80',
                                                                                            5),
                                                                                           ('sf128',
                                                                                            6),
                                                                                           ('sf160',
                                                                                            7),
                                                                                           ('sf320',
                                                                                            8),
                                                                                           ('sf640',
                                                                                            9),
                                                                                           ('sf1-v1430',
                                                                                            10),
                                                                                           ('sf2-v1430',
                                                                                            11),
                                                                                           ('sf3-v1430',
                                                                                            12),
                                                                                           ('sf4-v1430',
                                                                                            13),
                                                                                           ('sf5-v1430',
                                                                                            14),
                                                                                           ('spare1',
                                                                                            15)]},
                                                                               {'name': 'implicitReleaseAfter',
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('e2',
                                                                                            0),
                                                                                           ('e3',
                                                                                            1),
                                                                                           ('e4',
                                                                                            2),
                                                                                           ('e8',
                                                                                            3)]},
                                                                               {'members': [{'name': 'p0-NominalPUSCH-Persistent',
                                                                                             'restricted-to': [(-126,
                                                                                                                24)],
                                                                                             'type': 'INTEGER'},
                                                                                            {'name': 'p0-UE-PUSCH-Persistent',
                                                                                             'restricted-to': [(-8,
                                                                                                                7)],
                                                                                             'type': 'INTEGER'}],
                                                                                'name': 'p0-Persistent',
                                                                                'optional': True,
                                                                                'type': 'SEQUENCE'},
                                                                               {'name': 'twoIntervalsConfig',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('true',
                                                                                            0)]},
                                                                               None,
                                                                               [{'members': [{'name': 'release',
                                                                                              'type': 'NULL'},
                                                                                             {'members': [{'name': 'p0-NominalPUSCH-PersistentSubframeSet2-r12',
                                                                                                           'restricted-to': [(-126,
                                                                                                                              24)],
                                                                                                           'type': 'INTEGER'},
                                                                                                          {'name': 'p0-UE-PUSCH-PersistentSubframeSet2-r12',
                                                                                                           'restricted-to': [(-8,
                                                                                                                              7)],
                                                                                                           'type': 'INTEGER'}],
                                                                                              'name': 'setup',
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'name': 'p0-PersistentSubframeSet2-r12',
                                                                                 'optional': True,
                                                                                 'type': 'CHOICE'}],
                                                                               [{'name': 'numberOfConfUlSPS-Processes-r13',
                                                                                 'optional': True,
                                                                                 'restricted-to': [(1,
                                                                                                    8)],
                                                                                 'type': 'INTEGER'}],
                                                                               [{'name': 'fixedRV-NonAdaptive-r14',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('true',
                                                                                             0)]},
                                                                                {'name': 'sps-ConfigIndex-r14',
                                                                                 'optional': True,
                                                                                 'type': 'SPS-ConfigIndex-r14'},
                                                                                {'name': 'semiPersistSchedIntervalUL-v1430',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('sf50',
                                                                                             0),
                                                                                            ('sf100',
                                                                                             1),
                                                                                            ('sf200',
                                                                                             2),
                                                                                            ('sf300',
                                                                                             3),
                                                                                            ('sf400',
                                                                                             4),
                                                                                            ('sf500',
                                                                                             5),
                                                                                            ('sf600',
                                                                                             6),
                                                                                            ('sf700',
                                                                                             7),
                                                                                            ('sf800',
                                                                                             8),
                                                                                            ('sf900',
                                                                                             9),
                                                                                            ('sf1000',
                                                                                             10),
                                                                                            ('spare5',
                                                                                             11),
                                                                                            ('spare4',
                                                                                             12),
                                                                                            ('spare3',
                                                                                             13),
                                                                                            ('spare2',
                                                                                             14),
                                                                                            ('spare1',
                                                                                             15)]}]],
                                                                   'name': 'setup',
                                                                   'type': 'SEQUENCE'}],
                                                      'type': 'CHOICE'},
                                     'SPS-ConfigUL-ToAddModList-r14': {'element': {'type': 'SPS-ConfigUL'},
                                                                       'size': [(1,
                                                                                 'maxConfigSPS-r14')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'SPS-ConfigUL-ToReleaseList-r14': {'element': {'type': 'SPS-ConfigIndex-r14'},
                                                                        'size': [(1,
                                                                                  'maxConfigSPS-r14')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'SRB-ToAddMod': {'members': [{'name': 'srb-Identity',
                                                                   'restricted-to': [(1,
                                                                                      2)],
                                                                   'type': 'INTEGER'},
                                                                  {'members': [{'name': 'explicitValue',
                                                                                'type': 'RLC-Config'},
                                                                               {'name': 'defaultValue',
                                                                                'type': 'NULL'}],
                                                                   'name': 'rlc-Config',
                                                                   'optional': True,
                                                                   'type': 'CHOICE'},
                                                                  {'members': [{'name': 'explicitValue',
                                                                                'type': 'LogicalChannelConfig'},
                                                                               {'name': 'defaultValue',
                                                                                'type': 'NULL'}],
                                                                   'name': 'logicalChannelConfig',
                                                                   'optional': True,
                                                                   'type': 'CHOICE'},
                                                                  None],
                                                      'type': 'SEQUENCE'},
                                     'SRB-ToAddModList': {'element': {'type': 'SRB-ToAddMod'},
                                                          'size': [(1, 2)],
                                                          'type': 'SEQUENCE '
                                                                  'OF'},
                                     'SRS-AntennaPort': {'type': 'ENUMERATED',
                                                         'values': [('an1', 0),
                                                                    ('an2', 1),
                                                                    ('an4', 2),
                                                                    ('spare1',
                                                                     3)]},
                                     'SRS-CC-SetIndex-r14': {'members': [{'name': 'cc-SetIndex-r14',
                                                                          'restricted-to': [(0,
                                                                                             3)],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'cc-IndexInOneCC-Set-r14',
                                                                          'restricted-to': [(0,
                                                                                             7)],
                                                                          'type': 'INTEGER'}],
                                                             'type': 'SEQUENCE'},
                                     'SRS-ConfigAp-r10': {'members': [{'name': 'srs-AntennaPortAp-r10',
                                                                       'type': 'SRS-AntennaPort'},
                                                                      {'name': 'srs-BandwidthAp-r10',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('bw0',
                                                                                   0),
                                                                                  ('bw1',
                                                                                   1),
                                                                                  ('bw2',
                                                                                   2),
                                                                                  ('bw3',
                                                                                   3)]},
                                                                      {'name': 'freqDomainPositionAp-r10',
                                                                       'restricted-to': [(0,
                                                                                          23)],
                                                                       'type': 'INTEGER'},
                                                                      {'name': 'transmissionCombAp-r10',
                                                                       'restricted-to': [(0,
                                                                                          1)],
                                                                       'type': 'INTEGER'},
                                                                      {'name': 'cyclicShiftAp-r10',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('cs0',
                                                                                   0),
                                                                                  ('cs1',
                                                                                   1),
                                                                                  ('cs2',
                                                                                   2),
                                                                                  ('cs3',
                                                                                   3),
                                                                                  ('cs4',
                                                                                   4),
                                                                                  ('cs5',
                                                                                   5),
                                                                                  ('cs6',
                                                                                   6),
                                                                                  ('cs7',
                                                                                   7)]}],
                                                          'type': 'SEQUENCE'},
                                     'SRS-ConfigAp-r13': {'members': [{'name': 'srs-AntennaPortAp-r13',
                                                                       'type': 'SRS-AntennaPort'},
                                                                      {'name': 'srs-BandwidthAp-r13',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('bw0',
                                                                                   0),
                                                                                  ('bw1',
                                                                                   1),
                                                                                  ('bw2',
                                                                                   2),
                                                                                  ('bw3',
                                                                                   3)]},
                                                                      {'name': 'freqDomainPositionAp-r13',
                                                                       'restricted-to': [(0,
                                                                                          23)],
                                                                       'type': 'INTEGER'},
                                                                      {'name': 'transmissionCombAp-r13',
                                                                       'restricted-to': [(0,
                                                                                          3)],
                                                                       'type': 'INTEGER'},
                                                                      {'name': 'cyclicShiftAp-r13',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('cs0',
                                                                                   0),
                                                                                  ('cs1',
                                                                                   1),
                                                                                  ('cs2',
                                                                                   2),
                                                                                  ('cs3',
                                                                                   3),
                                                                                  ('cs4',
                                                                                   4),
                                                                                  ('cs5',
                                                                                   5),
                                                                                  ('cs6',
                                                                                   6),
                                                                                  ('cs7',
                                                                                   7),
                                                                                  ('cs8',
                                                                                   8),
                                                                                  ('cs9',
                                                                                   9),
                                                                                  ('cs10',
                                                                                   10),
                                                                                  ('cs11',
                                                                                   11)]},
                                                                      {'name': 'transmissionCombNum-r13',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('n2',
                                                                                   0),
                                                                                  ('n4',
                                                                                   1)]}],
                                                          'type': 'SEQUENCE'},
                                     'SRS-ConfigAp-v1310': {'members': [{'name': 'transmissionCombAp-v1310',
                                                                         'optional': True,
                                                                         'restricted-to': [(2,
                                                                                            3)],
                                                                         'type': 'INTEGER'},
                                                                        {'name': 'cyclicShiftAp-v1310',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('cs8',
                                                                                     0),
                                                                                    ('cs9',
                                                                                     1),
                                                                                    ('cs10',
                                                                                     2),
                                                                                    ('cs11',
                                                                                     3)]},
                                                                        {'name': 'transmissionCombNum-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('n2',
                                                                                     0),
                                                                                    ('n4',
                                                                                     1)]}],
                                                            'type': 'SEQUENCE'},
                                     'SRS-TPC-PDCCH-Config-r14': {'members': [{'name': 'release',
                                                                               'type': 'NULL'},
                                                                              {'members': [{'name': 'srs-TPC-RNTI-r14',
                                                                                            'size': [16],
                                                                                            'type': 'BIT '
                                                                                                    'STRING'},
                                                                                           {'name': 'startingBitOfFormat3B-r14',
                                                                                            'restricted-to': [(0,
                                                                                                               31)],
                                                                                            'type': 'INTEGER'},
                                                                                           {'name': 'fieldTypeFormat3B-r14',
                                                                                            'restricted-to': [(1,
                                                                                                               4)],
                                                                                            'type': 'INTEGER'},
                                                                                           {'element': {'type': 'SRS-CC-SetIndex-r14'},
                                                                                            'name': 'srs-CC-SetIndexlist-r14',
                                                                                            'optional': True,
                                                                                            'size': [(1,
                                                                                                      4)],
                                                                                            'type': 'SEQUENCE '
                                                                                                    'OF'}],
                                                                               'name': 'setup',
                                                                               'type': 'SEQUENCE'}],
                                                                  'type': 'CHOICE'},
                                     'STAG-Id-r11': {'restricted-to': [(1,
                                                                        'maxSTAG-r11')],
                                                     'type': 'INTEGER'},
                                     'STAG-ToAddMod-r11': {'members': [{'name': 'stag-Id-r11',
                                                                        'type': 'STAG-Id-r11'},
                                                                       {'name': 'timeAlignmentTimerSTAG-r11',
                                                                        'type': 'TimeAlignmentTimer'},
                                                                       None],
                                                           'type': 'SEQUENCE'},
                                     'STAG-ToAddModList-r11': {'element': {'type': 'STAG-ToAddMod-r11'},
                                                               'size': [(1,
                                                                         'maxSTAG-r11')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'STAG-ToReleaseList-r11': {'element': {'type': 'STAG-Id-r11'},
                                                                'size': [(1,
                                                                          'maxSTAG-r11')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'SchedulingInfo': {'members': [{'name': 'si-Periodicity',
                                                                     'type': 'ENUMERATED',
                                                                     'values': [('rf8',
                                                                                 0),
                                                                                ('rf16',
                                                                                 1),
                                                                                ('rf32',
                                                                                 2),
                                                                                ('rf64',
                                                                                 3),
                                                                                ('rf128',
                                                                                 4),
                                                                                ('rf256',
                                                                                 5),
                                                                                ('rf512',
                                                                                 6)]},
                                                                    {'name': 'sib-MappingInfo',
                                                                     'type': 'SIB-MappingInfo'}],
                                                        'type': 'SEQUENCE'},
                                     'SchedulingInfo-BR-r13': {'members': [{'name': 'si-Narrowband-r13',
                                                                            'restricted-to': [(1,
                                                                                               'maxAvailNarrowBands-r13')],
                                                                            'type': 'INTEGER'},
                                                                           {'name': 'si-TBS-r13',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('b152',
                                                                                        0),
                                                                                       ('b208',
                                                                                        1),
                                                                                       ('b256',
                                                                                        2),
                                                                                       ('b328',
                                                                                        3),
                                                                                       ('b408',
                                                                                        4),
                                                                                       ('b504',
                                                                                        5),
                                                                                       ('b600',
                                                                                        6),
                                                                                       ('b712',
                                                                                        7),
                                                                                       ('b808',
                                                                                        8),
                                                                                       ('b936',
                                                                                        9)]}],
                                                               'type': 'SEQUENCE'},
                                     'SchedulingInfo-MBMS-r14': {'members': [{'name': 'si-Periodicity-r14',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('rf16',
                                                                                          0),
                                                                                         ('rf32',
                                                                                          1),
                                                                                         ('rf64',
                                                                                          2),
                                                                                         ('rf128',
                                                                                          3),
                                                                                         ('rf256',
                                                                                          4),
                                                                                         ('rf512',
                                                                                          5)]},
                                                                             {'name': 'sib-MappingInfo-r14',
                                                                              'type': 'SIB-MappingInfo-MBMS-r14'}],
                                                                 'type': 'SEQUENCE'},
                                     'SchedulingInfoList': {'element': {'type': 'SchedulingInfo'},
                                                            'size': [(1,
                                                                      'maxSI-Message')],
                                                            'type': 'SEQUENCE '
                                                                    'OF'},
                                     'SchedulingInfoList-BR-r13': {'element': {'type': 'SchedulingInfo-BR-r13'},
                                                                   'size': [(1,
                                                                             'maxSI-Message')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'SchedulingInfoList-MBMS-r14': {'element': {'type': 'SchedulingInfo-MBMS-r14'},
                                                                     'size': [(1,
                                                                               'maxSI-Message')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'SchedulingRequestConfig': {'members': [{'name': 'release',
                                                                              'type': 'NULL'},
                                                                             {'members': [{'name': 'sr-PUCCH-ResourceIndex',
                                                                                           'restricted-to': [(0,
                                                                                                              2047)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'sr-ConfigIndex',
                                                                                           'restricted-to': [(0,
                                                                                                              157)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'dsr-TransMax',
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('n4',
                                                                                                       0),
                                                                                                      ('n8',
                                                                                                       1),
                                                                                                      ('n16',
                                                                                                       2),
                                                                                                      ('n32',
                                                                                                       3),
                                                                                                      ('n64',
                                                                                                       4),
                                                                                                      ('spare3',
                                                                                                       5),
                                                                                                      ('spare2',
                                                                                                       6),
                                                                                                      ('spare1',
                                                                                                       7)]}],
                                                                              'name': 'setup',
                                                                              'type': 'SEQUENCE'}],
                                                                 'type': 'CHOICE'},
                                     'SchedulingRequestConfig-v1020': {'members': [{'name': 'sr-PUCCH-ResourceIndexP1-r10',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(0,
                                                                                                       2047)],
                                                                                    'type': 'INTEGER'}],
                                                                       'type': 'SEQUENCE'},
                                     'SchedulingRequestConfigSCell-r13': {'members': [{'name': 'release',
                                                                                       'type': 'NULL'},
                                                                                      {'members': [{'name': 'sr-PUCCH-ResourceIndex-r13',
                                                                                                    'restricted-to': [(0,
                                                                                                                       2047)],
                                                                                                    'type': 'INTEGER'},
                                                                                                   {'name': 'sr-PUCCH-ResourceIndexP1-r13',
                                                                                                    'optional': True,
                                                                                                    'restricted-to': [(0,
                                                                                                                       2047)],
                                                                                                    'type': 'INTEGER'},
                                                                                                   {'name': 'sr-ConfigIndex-r13',
                                                                                                    'restricted-to': [(0,
                                                                                                                       157)],
                                                                                                    'type': 'INTEGER'},
                                                                                                   {'name': 'dsr-TransMax-r13',
                                                                                                    'type': 'ENUMERATED',
                                                                                                    'values': [('n4',
                                                                                                                0),
                                                                                                               ('n8',
                                                                                                                1),
                                                                                                               ('n16',
                                                                                                                2),
                                                                                                               ('n32',
                                                                                                                3),
                                                                                                               ('n64',
                                                                                                                4),
                                                                                                               ('spare3',
                                                                                                                5),
                                                                                                               ('spare2',
                                                                                                                6),
                                                                                                               ('spare1',
                                                                                                                7)]}],
                                                                                       'name': 'setup',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'type': 'CHOICE'},
                                     'SecondaryPreRegistrationZoneIdListHRPD': {'element': {'type': 'PreRegistrationZoneIdHRPD'},
                                                                                'size': [(1,
                                                                                          2)],
                                                                                'type': 'SEQUENCE '
                                                                                        'OF'},
                                     'SecurityAlgorithmConfig': {'members': [{'name': 'cipheringAlgorithm',
                                                                              'type': 'CipheringAlgorithm-r12'},
                                                                             {'name': 'integrityProtAlgorithm',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('eia0-v920',
                                                                                          0),
                                                                                         ('eia1',
                                                                                          1),
                                                                                         ('eia2',
                                                                                          2),
                                                                                         ('eia3-v1130',
                                                                                          3),
                                                                                         ('spare4',
                                                                                          4),
                                                                                         ('spare3',
                                                                                          5),
                                                                                         ('spare2',
                                                                                          6),
                                                                                         ('spare1',
                                                                                          7),
                                                                                         None]}],
                                                                 'type': 'SEQUENCE'},
                                     'SecurityConfigHO': {'members': [{'members': [{'members': [{'name': 'securityAlgorithmConfig',
                                                                                                 'optional': True,
                                                                                                 'type': 'SecurityAlgorithmConfig'},
                                                                                                {'name': 'keyChangeIndicator',
                                                                                                 'type': 'BOOLEAN'},
                                                                                                {'name': 'nextHopChainingCount',
                                                                                                 'type': 'NextHopChainingCount'}],
                                                                                    'name': 'intraLTE',
                                                                                    'type': 'SEQUENCE'},
                                                                                   {'members': [{'name': 'securityAlgorithmConfig',
                                                                                                 'type': 'SecurityAlgorithmConfig'},
                                                                                                {'name': 'nas-SecurityParamToEUTRA',
                                                                                                 'size': [6],
                                                                                                 'type': 'OCTET '
                                                                                                         'STRING'}],
                                                                                    'name': 'interRAT',
                                                                                    'type': 'SEQUENCE'}],
                                                                       'name': 'handoverType',
                                                                       'type': 'CHOICE'},
                                                                      None],
                                                          'type': 'SEQUENCE'},
                                     'SecurityConfigSMC': {'members': [{'name': 'securityAlgorithmConfig',
                                                                        'type': 'SecurityAlgorithmConfig'},
                                                                       None],
                                                           'type': 'SEQUENCE'},
                                     'SecurityModeCommand': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                          'type': 'RRC-TransactionIdentifier'},
                                                                         {'members': [{'members': [{'name': 'securityModeCommand-r8',
                                                                                                    'type': 'SecurityModeCommand-r8-IEs'},
                                                                                                   {'name': 'spare3',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare2',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare1',
                                                                                                    'type': 'NULL'}],
                                                                                       'name': 'c1',
                                                                                       'type': 'CHOICE'},
                                                                                      {'members': [],
                                                                                       'name': 'criticalExtensionsFuture',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'name': 'criticalExtensions',
                                                                          'type': 'CHOICE'}],
                                                             'type': 'SEQUENCE'},
                                     'SecurityModeCommand-r8-IEs': {'members': [{'name': 'securityConfigSMC',
                                                                                 'type': 'SecurityConfigSMC'},
                                                                                {'name': 'nonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'SecurityModeCommand-v8a0-IEs'}],
                                                                    'type': 'SEQUENCE'},
                                     'SecurityModeCommand-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'members': [],
                                                                                   'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'SEQUENCE'}],
                                                                      'type': 'SEQUENCE'},
                                     'SecurityModeComplete': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                           'type': 'RRC-TransactionIdentifier'},
                                                                          {'members': [{'name': 'securityModeComplete-r8',
                                                                                        'type': 'SecurityModeComplete-r8-IEs'},
                                                                                       {'members': [],
                                                                                        'name': 'criticalExtensionsFuture',
                                                                                        'type': 'SEQUENCE'}],
                                                                           'name': 'criticalExtensions',
                                                                           'type': 'CHOICE'}],
                                                              'type': 'SEQUENCE'},
                                     'SecurityModeComplete-r8-IEs': {'members': [{'name': 'nonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'SecurityModeComplete-v8a0-IEs'}],
                                                                     'type': 'SEQUENCE'},
                                     'SecurityModeComplete-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                     'SecurityModeFailure': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                          'type': 'RRC-TransactionIdentifier'},
                                                                         {'members': [{'name': 'securityModeFailure-r8',
                                                                                       'type': 'SecurityModeFailure-r8-IEs'},
                                                                                      {'members': [],
                                                                                       'name': 'criticalExtensionsFuture',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'name': 'criticalExtensions',
                                                                          'type': 'CHOICE'}],
                                                             'type': 'SEQUENCE'},
                                     'SecurityModeFailure-r8-IEs': {'members': [{'name': 'nonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'SecurityModeFailure-v8a0-IEs'}],
                                                                    'type': 'SEQUENCE'},
                                     'SecurityModeFailure-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'members': [],
                                                                                   'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'SEQUENCE'}],
                                                                      'type': 'SEQUENCE'},
                                     'ServCellIndex-r10': {'restricted-to': [(0,
                                                                              7)],
                                                           'type': 'INTEGER'},
                                     'ServCellIndex-r13': {'restricted-to': [(0,
                                                                              31)],
                                                           'type': 'INTEGER'},
                                     'ShortMAC-I': {'size': [16],
                                                    'type': 'BIT STRING'},
                                     'SidelinkUEInformation-r12': {'members': [{'members': [{'members': [{'name': 'sidelinkUEInformation-r12',
                                                                                                          'type': 'SidelinkUEInformation-r12-IEs'},
                                                                                                         {'name': 'spare3',
                                                                                                          'type': 'NULL'},
                                                                                                         {'name': 'spare2',
                                                                                                          'type': 'NULL'},
                                                                                                         {'name': 'spare1',
                                                                                                          'type': 'NULL'}],
                                                                                             'name': 'c1',
                                                                                             'type': 'CHOICE'},
                                                                                            {'members': [],
                                                                                             'name': 'criticalExtensionsFuture',
                                                                                             'type': 'SEQUENCE'}],
                                                                                'name': 'criticalExtensions',
                                                                                'type': 'CHOICE'}],
                                                                   'type': 'SEQUENCE'},
                                     'SidelinkUEInformation-r12-IEs': {'members': [{'name': 'commRxInterestedFreq-r12',
                                                                                    'optional': True,
                                                                                    'type': 'ARFCN-ValueEUTRA-r9'},
                                                                                   {'name': 'commTxResourceReq-r12',
                                                                                    'optional': True,
                                                                                    'type': 'SL-CommTxResourceReq-r12'},
                                                                                   {'name': 'discRxInterest-r12',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]},
                                                                                   {'name': 'discTxResourceReq-r12',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(1,
                                                                                                       63)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'lateNonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SidelinkUEInformation-v1310-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'SidelinkUEInformation-v1310-IEs': {'members': [{'name': 'commTxResourceReqUC-r13',
                                                                                      'optional': True,
                                                                                      'type': 'SL-CommTxResourceReq-r12'},
                                                                                     {'members': [{'name': 'commTxResourceReqRelay-r13',
                                                                                                   'optional': True,
                                                                                                   'type': 'SL-CommTxResourceReq-r12'},
                                                                                                  {'name': 'commTxResourceReqRelayUC-r13',
                                                                                                   'optional': True,
                                                                                                   'type': 'SL-CommTxResourceReq-r12'},
                                                                                                  {'name': 'ue-Type-r13',
                                                                                                   'type': 'ENUMERATED',
                                                                                                   'values': [('relayUE',
                                                                                                               0),
                                                                                                              ('remoteUE',
                                                                                                               1)]}],
                                                                                      'name': 'commTxResourceInfoReqRelay-r13',
                                                                                      'optional': True,
                                                                                      'type': 'SEQUENCE'},
                                                                                     {'members': [{'name': 'carrierFreqDiscTx-r13',
                                                                                                   'optional': True,
                                                                                                   'restricted-to': [(1,
                                                                                                                      'maxFreq')],
                                                                                                   'type': 'INTEGER'},
                                                                                                  {'name': 'discTxResourceReqAddFreq-r13',
                                                                                                   'optional': True,
                                                                                                   'type': 'SL-DiscTxResourceReqPerFreqList-r13'}],
                                                                                      'name': 'discTxResourceReq-v1310',
                                                                                      'optional': True,
                                                                                      'type': 'SEQUENCE'},
                                                                                     {'name': 'discTxResourceReqPS-r13',
                                                                                      'optional': True,
                                                                                      'type': 'SL-DiscTxResourceReq-r13'},
                                                                                     {'name': 'discRxGapReq-r13',
                                                                                      'optional': True,
                                                                                      'type': 'SL-GapRequest-r13'},
                                                                                     {'name': 'discTxGapReq-r13',
                                                                                      'optional': True,
                                                                                      'type': 'SL-GapRequest-r13'},
                                                                                     {'name': 'discSysInfoReportFreqList-r13',
                                                                                      'optional': True,
                                                                                      'type': 'SL-DiscSysInfoReportFreqList-r13'},
                                                                                     {'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'SidelinkUEInformation-v1430-IEs'}],
                                                                         'type': 'SEQUENCE'},
                                     'SidelinkUEInformation-v1430-IEs': {'members': [{'name': 'v2x-CommRxInterestedFreqList-r14',
                                                                                      'optional': True,
                                                                                      'type': 'SL-V2X-CommFreqList-r14'},
                                                                                     {'name': 'p2x-CommTxType-r14',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('true',
                                                                                                  0)]},
                                                                                     {'name': 'v2x-CommTxResourceReq-r14',
                                                                                      'optional': True,
                                                                                      'type': 'SL-V2X-CommTxFreqList-r14'},
                                                                                     {'members': [],
                                                                                      'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'SEQUENCE'}],
                                                                         'type': 'SEQUENCE'},
                                     'SoundingRS-AperiodicSet-r14': {'members': [{'element': {'type': 'SRS-CC-SetIndex-r14'},
                                                                                  'name': 'srs-CC-SetIndexList-r14',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            4)],
                                                                                  'type': 'SEQUENCE '
                                                                                          'OF'},
                                                                                 {'name': 'soundingRS-UL-ConfigDedicatedAperiodic-r14',
                                                                                  'type': 'SoundingRS-UL-ConfigDedicatedAperiodic-r10'}],
                                                                     'type': 'SEQUENCE'},
                                     'SoundingRS-AperiodicSetUpPTsExt-r14': {'members': [{'element': {'type': 'SRS-CC-SetIndex-r14'},
                                                                                          'name': 'srs-CC-SetIndexList-r14',
                                                                                          'optional': True,
                                                                                          'size': [(1,
                                                                                                    4)],
                                                                                          'type': 'SEQUENCE '
                                                                                                  'OF'},
                                                                                         {'name': 'soundingRS-UL-ConfigDedicatedAperiodicUpPTsExt-r14',
                                                                                          'type': 'SoundingRS-UL-ConfigDedicatedAperiodicUpPTsExt-r13'}],
                                                                             'type': 'SEQUENCE'},
                                     'SoundingRS-UL-ConfigCommon': {'members': [{'name': 'release',
                                                                                 'type': 'NULL'},
                                                                                {'members': [{'name': 'srs-BandwidthConfig',
                                                                                              'type': 'ENUMERATED',
                                                                                              'values': [('bw0',
                                                                                                          0),
                                                                                                         ('bw1',
                                                                                                          1),
                                                                                                         ('bw2',
                                                                                                          2),
                                                                                                         ('bw3',
                                                                                                          3),
                                                                                                         ('bw4',
                                                                                                          4),
                                                                                                         ('bw5',
                                                                                                          5),
                                                                                                         ('bw6',
                                                                                                          6),
                                                                                                         ('bw7',
                                                                                                          7)]},
                                                                                             {'name': 'srs-SubframeConfig',
                                                                                              'type': 'ENUMERATED',
                                                                                              'values': [('sc0',
                                                                                                          0),
                                                                                                         ('sc1',
                                                                                                          1),
                                                                                                         ('sc2',
                                                                                                          2),
                                                                                                         ('sc3',
                                                                                                          3),
                                                                                                         ('sc4',
                                                                                                          4),
                                                                                                         ('sc5',
                                                                                                          5),
                                                                                                         ('sc6',
                                                                                                          6),
                                                                                                         ('sc7',
                                                                                                          7),
                                                                                                         ('sc8',
                                                                                                          8),
                                                                                                         ('sc9',
                                                                                                          9),
                                                                                                         ('sc10',
                                                                                                          10),
                                                                                                         ('sc11',
                                                                                                          11),
                                                                                                         ('sc12',
                                                                                                          12),
                                                                                                         ('sc13',
                                                                                                          13),
                                                                                                         ('sc14',
                                                                                                          14),
                                                                                                         ('sc15',
                                                                                                          15)]},
                                                                                             {'name': 'ackNackSRS-SimultaneousTransmission',
                                                                                              'type': 'BOOLEAN'},
                                                                                             {'name': 'srs-MaxUpPts',
                                                                                              'optional': True,
                                                                                              'type': 'ENUMERATED',
                                                                                              'values': [('true',
                                                                                                          0)]}],
                                                                                 'name': 'setup',
                                                                                 'type': 'SEQUENCE'}],
                                                                    'type': 'CHOICE'},
                                     'SoundingRS-UL-ConfigDedicated': {'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'members': [{'name': 'srs-Bandwidth',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('bw0',
                                                                                                             0),
                                                                                                            ('bw1',
                                                                                                             1),
                                                                                                            ('bw2',
                                                                                                             2),
                                                                                                            ('bw3',
                                                                                                             3)]},
                                                                                                {'name': 'srs-HoppingBandwidth',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('hbw0',
                                                                                                             0),
                                                                                                            ('hbw1',
                                                                                                             1),
                                                                                                            ('hbw2',
                                                                                                             2),
                                                                                                            ('hbw3',
                                                                                                             3)]},
                                                                                                {'name': 'freqDomainPosition',
                                                                                                 'restricted-to': [(0,
                                                                                                                    23)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'duration',
                                                                                                 'type': 'BOOLEAN'},
                                                                                                {'name': 'srs-ConfigIndex',
                                                                                                 'restricted-to': [(0,
                                                                                                                    1023)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'transmissionComb',
                                                                                                 'restricted-to': [(0,
                                                                                                                    1)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'cyclicShift',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('cs0',
                                                                                                             0),
                                                                                                            ('cs1',
                                                                                                             1),
                                                                                                            ('cs2',
                                                                                                             2),
                                                                                                            ('cs3',
                                                                                                             3),
                                                                                                            ('cs4',
                                                                                                             4),
                                                                                                            ('cs5',
                                                                                                             5),
                                                                                                            ('cs6',
                                                                                                             6),
                                                                                                            ('cs7',
                                                                                                             7)]}],
                                                                                    'name': 'setup',
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'CHOICE'},
                                     'SoundingRS-UL-ConfigDedicated-v1020': {'members': [{'name': 'srs-AntennaPort-r10',
                                                                                          'type': 'SRS-AntennaPort'}],
                                                                             'type': 'SEQUENCE'},
                                     'SoundingRS-UL-ConfigDedicated-v1310': {'members': [{'name': 'release',
                                                                                          'type': 'NULL'},
                                                                                         {'members': [{'name': 'transmissionComb-v1310',
                                                                                                       'optional': True,
                                                                                                       'restricted-to': [(2,
                                                                                                                          3)],
                                                                                                       'type': 'INTEGER'},
                                                                                                      {'name': 'cyclicShift-v1310',
                                                                                                       'optional': True,
                                                                                                       'type': 'ENUMERATED',
                                                                                                       'values': [('cs8',
                                                                                                                   0),
                                                                                                                  ('cs9',
                                                                                                                   1),
                                                                                                                  ('cs10',
                                                                                                                   2),
                                                                                                                  ('cs11',
                                                                                                                   3)]},
                                                                                                      {'name': 'transmissionCombNum-r13',
                                                                                                       'optional': True,
                                                                                                       'type': 'ENUMERATED',
                                                                                                       'values': [('n2',
                                                                                                                   0),
                                                                                                                  ('n4',
                                                                                                                   1)]}],
                                                                                          'name': 'setup',
                                                                                          'type': 'SEQUENCE'}],
                                                                             'type': 'CHOICE'},
                                     'SoundingRS-UL-ConfigDedicatedAperiodic-r10': {'members': [{'name': 'release',
                                                                                                 'type': 'NULL'},
                                                                                                {'members': [{'name': 'srs-ConfigIndexAp-r10',
                                                                                                              'restricted-to': [(0,
                                                                                                                                 31)],
                                                                                                              'type': 'INTEGER'},
                                                                                                             {'element': {'type': 'SRS-ConfigAp-r10'},
                                                                                                              'name': 'srs-ConfigApDCI-Format4-r10',
                                                                                                              'optional': True,
                                                                                                              'size': [(1,
                                                                                                                        3)],
                                                                                                              'type': 'SEQUENCE '
                                                                                                                      'OF'},
                                                                                                             {'members': [{'name': 'release',
                                                                                                                           'type': 'NULL'},
                                                                                                                          {'members': [{'name': 'srs-ConfigApDCI-Format0-r10',
                                                                                                                                        'type': 'SRS-ConfigAp-r10'},
                                                                                                                                       {'name': 'srs-ConfigApDCI-Format1a2b2c-r10',
                                                                                                                                        'type': 'SRS-ConfigAp-r10'},
                                                                                                                                       None],
                                                                                                                           'name': 'setup',
                                                                                                                           'type': 'SEQUENCE'}],
                                                                                                              'name': 'srs-ActivateAp-r10',
                                                                                                              'optional': True,
                                                                                                              'type': 'CHOICE'}],
                                                                                                 'name': 'setup',
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'type': 'CHOICE'},
                                     'SoundingRS-UL-ConfigDedicatedAperiodic-v1310': {'members': [{'name': 'release',
                                                                                                   'type': 'NULL'},
                                                                                                  {'members': [{'element': {'type': 'SRS-ConfigAp-v1310'},
                                                                                                                'name': 'srs-ConfigApDCI-Format4-v1310',
                                                                                                                'optional': True,
                                                                                                                'size': [(1,
                                                                                                                          3)],
                                                                                                                'type': 'SEQUENCE '
                                                                                                                        'OF'},
                                                                                                               {'members': [{'name': 'release',
                                                                                                                             'type': 'NULL'},
                                                                                                                            {'members': [{'name': 'srs-ConfigApDCI-Format0-v1310',
                                                                                                                                          'optional': True,
                                                                                                                                          'type': 'SRS-ConfigAp-v1310'},
                                                                                                                                         {'name': 'srs-ConfigApDCI-Format1a2b2c-v1310',
                                                                                                                                          'optional': True,
                                                                                                                                          'type': 'SRS-ConfigAp-v1310'}],
                                                                                                                             'name': 'setup',
                                                                                                                             'type': 'SEQUENCE'}],
                                                                                                                'name': 'srs-ActivateAp-v1310',
                                                                                                                'optional': True,
                                                                                                                'type': 'CHOICE'}],
                                                                                                   'name': 'setup',
                                                                                                   'type': 'SEQUENCE'}],
                                                                                      'type': 'CHOICE'},
                                     'SoundingRS-UL-ConfigDedicatedAperiodic-v1430': {'members': [{'name': 'release',
                                                                                                   'type': 'NULL'},
                                                                                                  {'members': [{'name': 'srs-SubframeIndication-r14',
                                                                                                                'optional': True,
                                                                                                                'restricted-to': [(1,
                                                                                                                                   4)],
                                                                                                                'type': 'INTEGER'}],
                                                                                                   'name': 'setup',
                                                                                                   'type': 'SEQUENCE'}],
                                                                                      'type': 'CHOICE'},
                                     'SoundingRS-UL-ConfigDedicatedAperiodicUpPTsExt-r13': {'members': [{'name': 'release',
                                                                                                         'type': 'NULL'},
                                                                                                        {'members': [{'name': 'srs-UpPtsAdd-r13',
                                                                                                                      'type': 'ENUMERATED',
                                                                                                                      'values': [('sym2',
                                                                                                                                  0),
                                                                                                                                 ('sym4',
                                                                                                                                  1)]},
                                                                                                                     {'name': 'srs-ConfigIndexAp-r13',
                                                                                                                      'restricted-to': [(0,
                                                                                                                                         31)],
                                                                                                                      'type': 'INTEGER'},
                                                                                                                     {'element': {'type': 'SRS-ConfigAp-r13'},
                                                                                                                      'name': 'srs-ConfigApDCI-Format4-r13',
                                                                                                                      'optional': True,
                                                                                                                      'size': [(1,
                                                                                                                                3)],
                                                                                                                      'type': 'SEQUENCE '
                                                                                                                              'OF'},
                                                                                                                     {'members': [{'name': 'release',
                                                                                                                                   'type': 'NULL'},
                                                                                                                                  {'members': [{'name': 'srs-ConfigApDCI-Format0-r13',
                                                                                                                                                'type': 'SRS-ConfigAp-r13'},
                                                                                                                                               {'name': 'srs-ConfigApDCI-Format1a2b2c-r13',
                                                                                                                                                'type': 'SRS-ConfigAp-r13'}],
                                                                                                                                   'name': 'setup',
                                                                                                                                   'type': 'SEQUENCE'}],
                                                                                                                      'name': 'srs-ActivateAp-r13',
                                                                                                                      'optional': True,
                                                                                                                      'type': 'CHOICE'}],
                                                                                                         'name': 'setup',
                                                                                                         'type': 'SEQUENCE'}],
                                                                                            'type': 'CHOICE'},
                                     'SoundingRS-UL-ConfigDedicatedUpPTsExt-r13': {'members': [{'name': 'release',
                                                                                                'type': 'NULL'},
                                                                                               {'members': [{'name': 'srs-UpPtsAdd-r13',
                                                                                                             'type': 'ENUMERATED',
                                                                                                             'values': [('sym2',
                                                                                                                         0),
                                                                                                                        ('sym4',
                                                                                                                         1)]},
                                                                                                            {'name': 'srs-Bandwidth-r13',
                                                                                                             'type': 'ENUMERATED',
                                                                                                             'values': [('bw0',
                                                                                                                         0),
                                                                                                                        ('bw1',
                                                                                                                         1),
                                                                                                                        ('bw2',
                                                                                                                         2),
                                                                                                                        ('bw3',
                                                                                                                         3)]},
                                                                                                            {'name': 'srs-HoppingBandwidth-r13',
                                                                                                             'type': 'ENUMERATED',
                                                                                                             'values': [('hbw0',
                                                                                                                         0),
                                                                                                                        ('hbw1',
                                                                                                                         1),
                                                                                                                        ('hbw2',
                                                                                                                         2),
                                                                                                                        ('hbw3',
                                                                                                                         3)]},
                                                                                                            {'name': 'freqDomainPosition-r13',
                                                                                                             'restricted-to': [(0,
                                                                                                                                23)],
                                                                                                             'type': 'INTEGER'},
                                                                                                            {'name': 'duration-r13',
                                                                                                             'type': 'BOOLEAN'},
                                                                                                            {'name': 'srs-ConfigIndex-r13',
                                                                                                             'restricted-to': [(0,
                                                                                                                                1023)],
                                                                                                             'type': 'INTEGER'},
                                                                                                            {'name': 'transmissionComb-r13',
                                                                                                             'restricted-to': [(0,
                                                                                                                                3)],
                                                                                                             'type': 'INTEGER'},
                                                                                                            {'name': 'cyclicShift-r13',
                                                                                                             'type': 'ENUMERATED',
                                                                                                             'values': [('cs0',
                                                                                                                         0),
                                                                                                                        ('cs1',
                                                                                                                         1),
                                                                                                                        ('cs2',
                                                                                                                         2),
                                                                                                                        ('cs3',
                                                                                                                         3),
                                                                                                                        ('cs4',
                                                                                                                         4),
                                                                                                                        ('cs5',
                                                                                                                         5),
                                                                                                                        ('cs6',
                                                                                                                         6),
                                                                                                                        ('cs7',
                                                                                                                         7),
                                                                                                                        ('cs8',
                                                                                                                         8),
                                                                                                                        ('cs9',
                                                                                                                         9),
                                                                                                                        ('cs10',
                                                                                                                         10),
                                                                                                                        ('cs11',
                                                                                                                         11)]},
                                                                                                            {'name': 'srs-AntennaPort-r13',
                                                                                                             'type': 'SRS-AntennaPort'},
                                                                                                            {'name': 'transmissionCombNum-r13',
                                                                                                             'type': 'ENUMERATED',
                                                                                                             'values': [('n2',
                                                                                                                         0),
                                                                                                                        ('n4',
                                                                                                                         1)]}],
                                                                                                'name': 'setup',
                                                                                                'type': 'SEQUENCE'}],
                                                                                   'type': 'CHOICE'},
                                     'SpeedStateScaleFactors': {'members': [{'name': 'sf-Medium',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('oDot25',
                                                                                         0),
                                                                                        ('oDot5',
                                                                                         1),
                                                                                        ('oDot75',
                                                                                         2),
                                                                                        ('lDot0',
                                                                                         3)]},
                                                                            {'name': 'sf-High',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('oDot25',
                                                                                         0),
                                                                                        ('oDot5',
                                                                                         1),
                                                                                        ('oDot75',
                                                                                         2),
                                                                                        ('lDot0',
                                                                                         3)]}],
                                                                'type': 'SEQUENCE'},
                                     'SubframeBitmapSL-r12': {'members': [{'name': 'bs4-r12',
                                                                           'size': [4],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs8-r12',
                                                                           'size': [8],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs12-r12',
                                                                           'size': [12],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs16-r12',
                                                                           'size': [16],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs30-r12',
                                                                           'size': [30],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs40-r12',
                                                                           'size': [40],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs42-r12',
                                                                           'size': [42],
                                                                           'type': 'BIT '
                                                                                   'STRING'}],
                                                              'type': 'CHOICE'},
                                     'SubframeBitmapSL-r14': {'members': [{'name': 'bs10-r14',
                                                                           'size': [10],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs16-r14',
                                                                           'size': [16],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs20-r14',
                                                                           'size': [20],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs30-r14',
                                                                           'size': [30],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs40-r14',
                                                                           'size': [40],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs50-r14',
                                                                           'size': [50],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs60-r14',
                                                                           'size': [60],
                                                                           'type': 'BIT '
                                                                                   'STRING'},
                                                                          {'name': 'bs100-r14',
                                                                           'size': [100],
                                                                           'type': 'BIT '
                                                                                   'STRING'}],
                                                              'type': 'CHOICE'},
                                     'SupportedBandCombination-r10': {'element': {'type': 'BandCombinationParameters-r10'},
                                                                      'size': [(1,
                                                                                'maxBandComb-r10')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'SupportedBandCombination-v1090': {'element': {'type': 'BandCombinationParameters-v1090'},
                                                                        'size': [(1,
                                                                                  'maxBandComb-r10')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'SupportedBandCombination-v10i0': {'element': {'type': 'BandCombinationParameters-v10i0'},
                                                                        'size': [(1,
                                                                                  'maxBandComb-r10')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'SupportedBandCombination-v1130': {'element': {'type': 'BandCombinationParameters-v1130'},
                                                                        'size': [(1,
                                                                                  'maxBandComb-r10')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'SupportedBandCombination-v1250': {'element': {'type': 'BandCombinationParameters-v1250'},
                                                                        'size': [(1,
                                                                                  'maxBandComb-r10')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'SupportedBandCombination-v1270': {'element': {'type': 'BandCombinationParameters-v1270'},
                                                                        'size': [(1,
                                                                                  'maxBandComb-r10')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'SupportedBandCombination-v1320': {'element': {'type': 'BandCombinationParameters-v1320'},
                                                                        'size': [(1,
                                                                                  'maxBandComb-r10')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'SupportedBandCombination-v1430': {'element': {'type': 'BandCombinationParameters-v1430'},
                                                                        'size': [(1,
                                                                                  'maxBandComb-r10')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'SupportedBandCombinationAdd-r11': {'element': {'type': 'BandCombinationParameters-r11'},
                                                                         'size': [(1,
                                                                                   'maxBandComb-r11')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'SupportedBandCombinationAdd-v11d0': {'element': {'type': 'BandCombinationParameters-v10i0'},
                                                                           'size': [(1,
                                                                                     'maxBandComb-r11')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'SupportedBandCombinationAdd-v1250': {'element': {'type': 'BandCombinationParameters-v1250'},
                                                                           'size': [(1,
                                                                                     'maxBandComb-r11')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'SupportedBandCombinationAdd-v1270': {'element': {'type': 'BandCombinationParameters-v1270'},
                                                                           'size': [(1,
                                                                                     'maxBandComb-r11')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'SupportedBandCombinationAdd-v1320': {'element': {'type': 'BandCombinationParameters-v1320'},
                                                                           'size': [(1,
                                                                                     'maxBandComb-r11')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'SupportedBandCombinationAdd-v1430': {'element': {'type': 'BandCombinationParameters-v1430'},
                                                                           'size': [(1,
                                                                                     'maxBandComb-r11')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'SupportedBandCombinationExt-r10': {'element': {'type': 'BandCombinationParametersExt-r10'},
                                                                         'size': [(1,
                                                                                   'maxBandComb-r10')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'SupportedBandCombinationReduced-r13': {'element': {'type': 'BandCombinationParameters-r13'},
                                                                             'size': [(1,
                                                                                       'maxBandComb-r13')],
                                                                             'type': 'SEQUENCE '
                                                                                     'OF'},
                                     'SupportedBandCombinationReduced-v1320': {'element': {'type': 'BandCombinationParameters-v1320'},
                                                                               'size': [(1,
                                                                                         'maxBandComb-r13')],
                                                                               'type': 'SEQUENCE '
                                                                                       'OF'},
                                     'SupportedBandCombinationReduced-v1430': {'element': {'type': 'BandCombinationParameters-v1430'},
                                                                               'size': [(1,
                                                                                         'maxBandComb-r13')],
                                                                               'type': 'SEQUENCE '
                                                                                       'OF'},
                                     'SupportedBandEUTRA': {'members': [{'name': 'bandEUTRA',
                                                                         'type': 'FreqBandIndicator'},
                                                                        {'name': 'halfDuplex',
                                                                         'type': 'BOOLEAN'}],
                                                            'type': 'SEQUENCE'},
                                     'SupportedBandEUTRA-v1250': {'members': [{'name': 'dl-256QAM-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ul-64QAM-r12',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'SupportedBandEUTRA-v1310': {'members': [{'name': 'ue-PowerClass-5-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'SupportedBandEUTRA-v1320': {'members': [{'name': 'intraFreq-CE-NeedForGaps-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]},
                                                                              {'name': 'ue-PowerClass-N-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('class1',
                                                                                           0),
                                                                                          ('class2',
                                                                                           1),
                                                                                          ('class4',
                                                                                           2)]}],
                                                                  'type': 'SEQUENCE'},
                                     'SupportedBandEUTRA-v9e0': {'members': [{'name': 'bandEUTRA-v9e0',
                                                                              'optional': True,
                                                                              'type': 'FreqBandIndicator-v9e0'}],
                                                                 'type': 'SEQUENCE'},
                                     'SupportedBandGERAN': {'type': 'ENUMERATED',
                                                            'values': [('gsm450',
                                                                        0),
                                                                       ('gsm480',
                                                                        1),
                                                                       ('gsm710',
                                                                        2),
                                                                       ('gsm750',
                                                                        3),
                                                                       ('gsm810',
                                                                        4),
                                                                       ('gsm850',
                                                                        5),
                                                                       ('gsm900P',
                                                                        6),
                                                                       ('gsm900E',
                                                                        7),
                                                                       ('gsm900R',
                                                                        8),
                                                                       ('gsm1800',
                                                                        9),
                                                                       ('gsm1900',
                                                                        10),
                                                                       ('spare5',
                                                                        11),
                                                                       ('spare4',
                                                                        12),
                                                                       ('spare3',
                                                                        13),
                                                                       ('spare2',
                                                                        14),
                                                                       ('spare1',
                                                                        15),
                                                                       None]},
                                     'SupportedBandInfo-r12': {'members': [{'name': 'support-r12',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]}],
                                                               'type': 'SEQUENCE'},
                                     'SupportedBandInfoList-r12': {'element': {'type': 'SupportedBandInfo-r12'},
                                                                   'size': [(1,
                                                                             'maxBands')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'SupportedBandList1XRTT': {'element': {'type': 'BandclassCDMA2000'},
                                                                'size': [(1,
                                                                          'maxCDMA-BandClass')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'SupportedBandListEUTRA': {'element': {'type': 'SupportedBandEUTRA'},
                                                                'size': [(1,
                                                                          'maxBands')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'SupportedBandListEUTRA-v1250': {'element': {'type': 'SupportedBandEUTRA-v1250'},
                                                                      'size': [(1,
                                                                                'maxBands')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'SupportedBandListEUTRA-v1310': {'element': {'type': 'SupportedBandEUTRA-v1310'},
                                                                      'size': [(1,
                                                                                'maxBands')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'SupportedBandListEUTRA-v1320': {'element': {'type': 'SupportedBandEUTRA-v1320'},
                                                                      'size': [(1,
                                                                                'maxBands')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'SupportedBandListEUTRA-v9e0': {'element': {'type': 'SupportedBandEUTRA-v9e0'},
                                                                     'size': [(1,
                                                                               'maxBands')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'SupportedBandListGERAN': {'element': {'type': 'SupportedBandGERAN'},
                                                                'size': [(1,
                                                                          'maxBands')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'SupportedBandListHRPD': {'element': {'type': 'BandclassCDMA2000'},
                                                               'size': [(1,
                                                                         'maxCDMA-BandClass')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                     'SupportedBandListUTRA-FDD': {'element': {'type': 'SupportedBandUTRA-FDD'},
                                                                   'size': [(1,
                                                                             'maxBands')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'SupportedBandListUTRA-TDD128': {'element': {'type': 'SupportedBandUTRA-TDD128'},
                                                                      'size': [(1,
                                                                                'maxBands')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'SupportedBandListUTRA-TDD384': {'element': {'type': 'SupportedBandUTRA-TDD384'},
                                                                      'size': [(1,
                                                                                'maxBands')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'SupportedBandListUTRA-TDD768': {'element': {'type': 'SupportedBandUTRA-TDD768'},
                                                                      'size': [(1,
                                                                                'maxBands')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'SupportedBandUTRA-FDD': {'type': 'ENUMERATED',
                                                               'values': [('bandI',
                                                                           0),
                                                                          ('bandII',
                                                                           1),
                                                                          ('bandIII',
                                                                           2),
                                                                          ('bandIV',
                                                                           3),
                                                                          ('bandV',
                                                                           4),
                                                                          ('bandVI',
                                                                           5),
                                                                          ('bandVII',
                                                                           6),
                                                                          ('bandVIII',
                                                                           7),
                                                                          ('bandIX',
                                                                           8),
                                                                          ('bandX',
                                                                           9),
                                                                          ('bandXI',
                                                                           10),
                                                                          ('bandXII',
                                                                           11),
                                                                          ('bandXIII',
                                                                           12),
                                                                          ('bandXIV',
                                                                           13),
                                                                          ('bandXV',
                                                                           14),
                                                                          ('bandXVI',
                                                                           15),
                                                                          None,
                                                                          ('bandXVII-8a0',
                                                                           16),
                                                                          ('bandXVIII-8a0',
                                                                           17),
                                                                          ('bandXIX-8a0',
                                                                           18),
                                                                          ('bandXX-8a0',
                                                                           19),
                                                                          ('bandXXI-8a0',
                                                                           20),
                                                                          ('bandXXII-8a0',
                                                                           21),
                                                                          ('bandXXIII-8a0',
                                                                           22),
                                                                          ('bandXXIV-8a0',
                                                                           23),
                                                                          ('bandXXV-8a0',
                                                                           24),
                                                                          ('bandXXVI-8a0',
                                                                           25),
                                                                          ('bandXXVII-8a0',
                                                                           26),
                                                                          ('bandXXVIII-8a0',
                                                                           27),
                                                                          ('bandXXIX-8a0',
                                                                           28),
                                                                          ('bandXXX-8a0',
                                                                           29),
                                                                          ('bandXXXI-8a0',
                                                                           30),
                                                                          ('bandXXXII-8a0',
                                                                           31)]},
                                     'SupportedBandUTRA-TDD128': {'type': 'ENUMERATED',
                                                                  'values': [('a',
                                                                              0),
                                                                             ('b',
                                                                              1),
                                                                             ('c',
                                                                              2),
                                                                             ('d',
                                                                              3),
                                                                             ('e',
                                                                              4),
                                                                             ('f',
                                                                              5),
                                                                             ('g',
                                                                              6),
                                                                             ('h',
                                                                              7),
                                                                             ('i',
                                                                              8),
                                                                             ('j',
                                                                              9),
                                                                             ('k',
                                                                              10),
                                                                             ('l',
                                                                              11),
                                                                             ('m',
                                                                              12),
                                                                             ('n',
                                                                              13),
                                                                             ('o',
                                                                              14),
                                                                             ('p',
                                                                              15),
                                                                             None]},
                                     'SupportedBandUTRA-TDD384': {'type': 'ENUMERATED',
                                                                  'values': [('a',
                                                                              0),
                                                                             ('b',
                                                                              1),
                                                                             ('c',
                                                                              2),
                                                                             ('d',
                                                                              3),
                                                                             ('e',
                                                                              4),
                                                                             ('f',
                                                                              5),
                                                                             ('g',
                                                                              6),
                                                                             ('h',
                                                                              7),
                                                                             ('i',
                                                                              8),
                                                                             ('j',
                                                                              9),
                                                                             ('k',
                                                                              10),
                                                                             ('l',
                                                                              11),
                                                                             ('m',
                                                                              12),
                                                                             ('n',
                                                                              13),
                                                                             ('o',
                                                                              14),
                                                                             ('p',
                                                                              15),
                                                                             None]},
                                     'SupportedBandUTRA-TDD768': {'type': 'ENUMERATED',
                                                                  'values': [('a',
                                                                              0),
                                                                             ('b',
                                                                              1),
                                                                             ('c',
                                                                              2),
                                                                             ('d',
                                                                              3),
                                                                             ('e',
                                                                              4),
                                                                             ('f',
                                                                              5),
                                                                             ('g',
                                                                              6),
                                                                             ('h',
                                                                              7),
                                                                             ('i',
                                                                              8),
                                                                             ('j',
                                                                              9),
                                                                             ('k',
                                                                              10),
                                                                             ('l',
                                                                              11),
                                                                             ('m',
                                                                              12),
                                                                             ('n',
                                                                              13),
                                                                             ('o',
                                                                              14),
                                                                             ('p',
                                                                              15),
                                                                             None]},
                                     'SupportedBandwidthCombinationSet-r10': {'size': [(1,
                                                                                        'maxBandwidthCombSet-r10')],
                                                                              'type': 'BIT '
                                                                                      'STRING'},
                                     'SystemInfoListGERAN': {'element': {'size': [(1,
                                                                                   23)],
                                                                         'type': 'OCTET '
                                                                                 'STRING'},
                                                             'size': [(1,
                                                                       'maxGERAN-SI')],
                                                             'type': 'SEQUENCE '
                                                                     'OF'},
                                     'SystemInfoValueTagList-r13': {'element': {'type': 'SystemInfoValueTagSI-r13'},
                                                                    'size': [(1,
                                                                              'maxSI-Message')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'SystemInfoValueTagSI-r13': {'restricted-to': [(0,
                                                                                     3)],
                                                                  'type': 'INTEGER'},
                                     'SystemInformation': {'members': [{'members': [{'name': 'systemInformation-r8',
                                                                                     'type': 'SystemInformation-r8-IEs'},
                                                                                    {'members': [],
                                                                                     'name': 'criticalExtensionsFuture',
                                                                                     'type': 'SEQUENCE'}],
                                                                        'name': 'criticalExtensions',
                                                                        'type': 'CHOICE'}],
                                                           'type': 'SEQUENCE'},
                                     'SystemInformation-BR-r13': {'type': 'SystemInformation'},
                                     'SystemInformation-MBMS-r14': {'type': 'SystemInformation'},
                                     'SystemInformation-r8-IEs': {'members': [{'element': {'members': [{'name': 'sib2',
                                                                                                        'type': 'SystemInformationBlockType2'},
                                                                                                       {'name': 'sib3',
                                                                                                        'type': 'SystemInformationBlockType3'},
                                                                                                       {'name': 'sib4',
                                                                                                        'type': 'SystemInformationBlockType4'},
                                                                                                       {'name': 'sib5',
                                                                                                        'type': 'SystemInformationBlockType5'},
                                                                                                       {'name': 'sib6',
                                                                                                        'type': 'SystemInformationBlockType6'},
                                                                                                       {'name': 'sib7',
                                                                                                        'type': 'SystemInformationBlockType7'},
                                                                                                       {'name': 'sib8',
                                                                                                        'type': 'SystemInformationBlockType8'},
                                                                                                       {'name': 'sib9',
                                                                                                        'type': 'SystemInformationBlockType9'},
                                                                                                       {'name': 'sib10',
                                                                                                        'type': 'SystemInformationBlockType10'},
                                                                                                       {'name': 'sib11',
                                                                                                        'type': 'SystemInformationBlockType11'},
                                                                                                       None,
                                                                                                       {'name': 'sib12-v920',
                                                                                                        'type': 'SystemInformationBlockType12-r9'},
                                                                                                       {'name': 'sib13-v920',
                                                                                                        'type': 'SystemInformationBlockType13-r9'},
                                                                                                       {'name': 'sib14-v1130',
                                                                                                        'type': 'SystemInformationBlockType14-r11'},
                                                                                                       {'name': 'sib15-v1130',
                                                                                                        'type': 'SystemInformationBlockType15-r11'},
                                                                                                       {'name': 'sib16-v1130',
                                                                                                        'type': 'SystemInformationBlockType16-r11'},
                                                                                                       {'name': 'sib17-v1250',
                                                                                                        'type': 'SystemInformationBlockType17-r12'},
                                                                                                       {'name': 'sib18-v1250',
                                                                                                        'type': 'SystemInformationBlockType18-r12'},
                                                                                                       {'name': 'sib19-v1250',
                                                                                                        'type': 'SystemInformationBlockType19-r12'},
                                                                                                       {'name': 'sib20-v1310',
                                                                                                        'type': 'SystemInformationBlockType20-r13'},
                                                                                                       {'name': 'sib21-v1430',
                                                                                                        'type': 'SystemInformationBlockType21-r14'}],
                                                                                           'type': 'CHOICE'},
                                                                               'name': 'sib-TypeAndInfo',
                                                                               'size': [(1,
                                                                                         'maxSIB')],
                                                                               'type': 'SEQUENCE '
                                                                                       'OF'},
                                                                              {'name': 'nonCriticalExtension',
                                                                               'optional': True,
                                                                               'type': 'SystemInformation-v8a0-IEs'}],
                                                                  'type': 'SEQUENCE'},
                                     'SystemInformation-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'OCTET '
                                                                                         'STRING'},
                                                                                {'members': [],
                                                                                 'name': 'nonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'SEQUENCE'}],
                                                                    'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1': {'members': [{'members': [{'name': 'plmn-IdentityList',
                                                                                               'type': 'PLMN-IdentityList'},
                                                                                              {'name': 'trackingAreaCode',
                                                                                               'type': 'TrackingAreaCode'},
                                                                                              {'name': 'cellIdentity',
                                                                                               'type': 'CellIdentity'},
                                                                                              {'name': 'cellBarred',
                                                                                               'type': 'ENUMERATED',
                                                                                               'values': [('barred',
                                                                                                           0),
                                                                                                          ('notBarred',
                                                                                                           1)]},
                                                                                              {'name': 'intraFreqReselection',
                                                                                               'type': 'ENUMERATED',
                                                                                               'values': [('allowed',
                                                                                                           0),
                                                                                                          ('notAllowed',
                                                                                                           1)]},
                                                                                              {'name': 'csg-Indication',
                                                                                               'type': 'BOOLEAN'},
                                                                                              {'name': 'csg-Identity',
                                                                                               'optional': True,
                                                                                               'type': 'CSG-Identity'}],
                                                                                  'name': 'cellAccessRelatedInfo',
                                                                                  'type': 'SEQUENCE'},
                                                                                 {'members': [{'name': 'q-RxLevMin',
                                                                                               'type': 'Q-RxLevMin'},
                                                                                              {'name': 'q-RxLevMinOffset',
                                                                                               'optional': True,
                                                                                               'restricted-to': [(1,
                                                                                                                  8)],
                                                                                               'type': 'INTEGER'}],
                                                                                  'name': 'cellSelectionInfo',
                                                                                  'type': 'SEQUENCE'},
                                                                                 {'name': 'p-Max',
                                                                                  'optional': True,
                                                                                  'type': 'P-Max'},
                                                                                 {'name': 'freqBandIndicator',
                                                                                  'type': 'FreqBandIndicator'},
                                                                                 {'name': 'schedulingInfoList',
                                                                                  'type': 'SchedulingInfoList'},
                                                                                 {'name': 'tdd-Config',
                                                                                  'optional': True,
                                                                                  'type': 'TDD-Config'},
                                                                                 {'name': 'si-WindowLength',
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('ms1',
                                                                                              0),
                                                                                             ('ms2',
                                                                                              1),
                                                                                             ('ms5',
                                                                                              2),
                                                                                             ('ms10',
                                                                                              3),
                                                                                             ('ms15',
                                                                                              4),
                                                                                             ('ms20',
                                                                                              5),
                                                                                             ('ms40',
                                                                                              6)]},
                                                                                 {'name': 'systemInfoValueTag',
                                                                                  'restricted-to': [(0,
                                                                                                     31)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'name': 'nonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'SystemInformationBlockType1-v890-IEs'}],
                                                                     'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-BR-r13': {'type': 'SystemInformationBlockType1'},
                                     'SystemInformationBlockType1-MBMS-r14': {'members': [{'members': [{'name': 'plmn-IdentityList-r14',
                                                                                                        'type': 'PLMN-IdentityList-MBMS-r14'},
                                                                                                       {'name': 'trackingAreaCode-r14',
                                                                                                        'type': 'TrackingAreaCode'},
                                                                                                       {'name': 'cellIdentity-r14',
                                                                                                        'type': 'CellIdentity'}],
                                                                                           'name': 'cellAccessRelatedInfo-r14',
                                                                                           'type': 'SEQUENCE'},
                                                                                          {'name': 'freqBandIndicator-r14',
                                                                                           'type': 'FreqBandIndicator-r11'},
                                                                                          {'name': 'multiBandInfoList-r14',
                                                                                           'optional': True,
                                                                                           'type': 'MultiBandInfoList-r11'},
                                                                                          {'name': 'schedulingInfoList-MBMS-r14',
                                                                                           'type': 'SchedulingInfoList-MBMS-r14'},
                                                                                          {'name': 'si-WindowLength-r14',
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('ms1',
                                                                                                       0),
                                                                                                      ('ms2',
                                                                                                       1),
                                                                                                      ('ms5',
                                                                                                       2),
                                                                                                      ('ms10',
                                                                                                       3),
                                                                                                      ('ms15',
                                                                                                       4),
                                                                                                      ('ms20',
                                                                                                       5),
                                                                                                      ('ms40',
                                                                                                       6),
                                                                                                      ('ms80',
                                                                                                       7)]},
                                                                                          {'name': 'systemInfoValueTag-r14',
                                                                                           'restricted-to': [(0,
                                                                                                              31)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'nonMBSFN-SubframeConfig-r14',
                                                                                           'optional': True,
                                                                                           'type': 'NonMBSFN-SubframeConfig-r14'},
                                                                                          {'name': 'pdsch-ConfigCommon-r14',
                                                                                           'type': 'PDSCH-ConfigCommon'},
                                                                                          {'name': 'systemInformationBlockType13-r14',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType13-r9'},
                                                                                          {'element': {'type': 'CellAccessRelatedInfo-r14'},
                                                                                           'name': 'cellAccessRelatedInfoList-r14',
                                                                                           'optional': True,
                                                                                           'size': [(1,
                                                                                                     'maxPLMN-1-r14')],
                                                                                           'type': 'SEQUENCE '
                                                                                                   'OF'},
                                                                                          {'members': [],
                                                                                           'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SEQUENCE'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v10j0-IEs': {'members': [{'name': 'freqBandInfo-r10',
                                                                                            'optional': True,
                                                                                            'type': 'NS-PmaxList-r10'},
                                                                                           {'name': 'multiBandInfoList-v10j0',
                                                                                            'optional': True,
                                                                                            'type': 'MultiBandInfoList-v10j0'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SystemInformationBlockType1-v10l0-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v10l0-IEs': {'members': [{'name': 'freqBandInfo-v10l0',
                                                                                            'optional': True,
                                                                                            'type': 'NS-PmaxList-v10l0'},
                                                                                           {'name': 'multiBandInfoList-v10l0',
                                                                                            'optional': True,
                                                                                            'type': 'MultiBandInfoList-v10l0'},
                                                                                           {'members': [],
                                                                                            'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SEQUENCE'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v1130-IEs': {'members': [{'name': 'tdd-Config-v1130',
                                                                                            'optional': True,
                                                                                            'type': 'TDD-Config-v1130'},
                                                                                           {'name': 'cellSelectionInfo-v1130',
                                                                                            'optional': True,
                                                                                            'type': 'CellSelectionInfo-v1130'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SystemInformationBlockType1-v1250-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v1250-IEs': {'members': [{'members': [{'name': 'category0Allowed-r12',
                                                                                                         'optional': True,
                                                                                                         'type': 'ENUMERATED',
                                                                                                         'values': [('true',
                                                                                                                     0)]}],
                                                                                            'name': 'cellAccessRelatedInfo-v1250',
                                                                                            'type': 'SEQUENCE'},
                                                                                           {'name': 'cellSelectionInfo-v1250',
                                                                                            'optional': True,
                                                                                            'type': 'CellSelectionInfo-v1250'},
                                                                                           {'name': 'freqBandIndicatorPriority-r12',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('true',
                                                                                                        0)]},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SystemInformationBlockType1-v1310-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v1310-IEs': {'members': [{'name': 'hyperSFN-r13',
                                                                                            'optional': True,
                                                                                            'size': [10],
                                                                                            'type': 'BIT '
                                                                                                    'STRING'},
                                                                                           {'name': 'eDRX-Allowed-r13',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('true',
                                                                                                        0)]},
                                                                                           {'name': 'cellSelectionInfoCE-r13',
                                                                                            'optional': True,
                                                                                            'type': 'CellSelectionInfoCE-r13'},
                                                                                           {'members': [{'name': 'si-WindowLength-BR-r13',
                                                                                                         'type': 'ENUMERATED',
                                                                                                         'values': [('ms20',
                                                                                                                     0),
                                                                                                                    ('ms40',
                                                                                                                     1),
                                                                                                                    ('ms60',
                                                                                                                     2),
                                                                                                                    ('ms80',
                                                                                                                     3),
                                                                                                                    ('ms120',
                                                                                                                     4),
                                                                                                                    ('ms160',
                                                                                                                     5),
                                                                                                                    ('ms200',
                                                                                                                     6),
                                                                                                                    ('spare',
                                                                                                                     7)]},
                                                                                                        {'name': 'si-RepetitionPattern-r13',
                                                                                                         'type': 'ENUMERATED',
                                                                                                         'values': [('everyRF',
                                                                                                                     0),
                                                                                                                    ('every2ndRF',
                                                                                                                     1),
                                                                                                                    ('every4thRF',
                                                                                                                     2),
                                                                                                                    ('every8thRF',
                                                                                                                     3)]},
                                                                                                        {'name': 'schedulingInfoList-BR-r13',
                                                                                                         'optional': True,
                                                                                                         'type': 'SchedulingInfoList-BR-r13'},
                                                                                                        {'members': [{'name': 'subframePattern10-r13',
                                                                                                                      'size': [10],
                                                                                                                      'type': 'BIT '
                                                                                                                              'STRING'},
                                                                                                                     {'name': 'subframePattern40-r13',
                                                                                                                      'size': [40],
                                                                                                                      'type': 'BIT '
                                                                                                                              'STRING'}],
                                                                                                         'name': 'fdd-DownlinkOrTddSubframeBitmapBR-r13',
                                                                                                         'optional': True,
                                                                                                         'type': 'CHOICE'},
                                                                                                        {'name': 'fdd-UplinkSubframeBitmapBR-r13',
                                                                                                         'optional': True,
                                                                                                         'size': [10],
                                                                                                         'type': 'BIT '
                                                                                                                 'STRING'},
                                                                                                        {'name': 'startSymbolBR-r13',
                                                                                                         'restricted-to': [(1,
                                                                                                                            4)],
                                                                                                         'type': 'INTEGER'},
                                                                                                        {'name': 'si-HoppingConfigCommon-r13',
                                                                                                         'type': 'ENUMERATED',
                                                                                                         'values': [('on',
                                                                                                                     0),
                                                                                                                    ('off',
                                                                                                                     1)]},
                                                                                                        {'name': 'si-ValidityTime-r13',
                                                                                                         'optional': True,
                                                                                                         'type': 'ENUMERATED',
                                                                                                         'values': [('true',
                                                                                                                     0)]},
                                                                                                        {'name': 'systemInfoValueTagList-r13',
                                                                                                         'optional': True,
                                                                                                         'type': 'SystemInfoValueTagList-r13'}],
                                                                                            'name': 'bandwidthReducedAccessRelatedInfo-r13',
                                                                                            'optional': True,
                                                                                            'type': 'SEQUENCE'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SystemInformationBlockType1-v1320-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v1320-IEs': {'members': [{'members': [{'name': 'mpdcch-pdsch-HoppingNB-r13',
                                                                                                         'optional': True,
                                                                                                         'type': 'ENUMERATED',
                                                                                                         'values': [('nb2',
                                                                                                                     0),
                                                                                                                    ('nb4',
                                                                                                                     1)]},
                                                                                                        {'members': [{'name': 'interval-FDD-r13',
                                                                                                                      'type': 'ENUMERATED',
                                                                                                                      'values': [('int1',
                                                                                                                                  0),
                                                                                                                                 ('int2',
                                                                                                                                  1),
                                                                                                                                 ('int4',
                                                                                                                                  2),
                                                                                                                                 ('int8',
                                                                                                                                  3)]},
                                                                                                                     {'name': 'interval-TDD-r13',
                                                                                                                      'type': 'ENUMERATED',
                                                                                                                      'values': [('int1',
                                                                                                                                  0),
                                                                                                                                 ('int5',
                                                                                                                                  1),
                                                                                                                                 ('int10',
                                                                                                                                  2),
                                                                                                                                 ('int20',
                                                                                                                                  3)]}],
                                                                                                         'name': 'interval-DLHoppingConfigCommonModeA-r13',
                                                                                                         'optional': True,
                                                                                                         'type': 'CHOICE'},
                                                                                                        {'members': [{'name': 'interval-FDD-r13',
                                                                                                                      'type': 'ENUMERATED',
                                                                                                                      'values': [('int2',
                                                                                                                                  0),
                                                                                                                                 ('int4',
                                                                                                                                  1),
                                                                                                                                 ('int8',
                                                                                                                                  2),
                                                                                                                                 ('int16',
                                                                                                                                  3)]},
                                                                                                                     {'name': 'interval-TDD-r13',
                                                                                                                      'type': 'ENUMERATED',
                                                                                                                      'values': [('int5',
                                                                                                                                  0),
                                                                                                                                 ('int10',
                                                                                                                                  1),
                                                                                                                                 ('int20',
                                                                                                                                  2),
                                                                                                                                 ('int40',
                                                                                                                                  3)]}],
                                                                                                         'name': 'interval-DLHoppingConfigCommonModeB-r13',
                                                                                                         'optional': True,
                                                                                                         'type': 'CHOICE'},
                                                                                                        {'name': 'mpdcch-pdsch-HoppingOffset-r13',
                                                                                                         'optional': True,
                                                                                                         'restricted-to': [(1,
                                                                                                                            'maxAvailNarrowBands-r13')],
                                                                                                         'type': 'INTEGER'}],
                                                                                            'name': 'freqHoppingParametersDL-r13',
                                                                                            'optional': True,
                                                                                            'type': 'SEQUENCE'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SystemInformationBlockType1-v1350-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v1350-IEs': {'members': [{'name': 'cellSelectionInfoCE1-r13',
                                                                                            'optional': True,
                                                                                            'type': 'CellSelectionInfoCE1-r13'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SystemInformationBlockType1-v1360-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v1360-IEs': {'members': [{'name': 'cellSelectionInfoCE1-v1360',
                                                                                            'optional': True,
                                                                                            'type': 'CellSelectionInfoCE1-v1360'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SystemInformationBlockType1-v1430-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v1430-IEs': {'members': [{'name': 'eCallOverIMS-Support-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('true',
                                                                                                        0)]},
                                                                                           {'name': 'tdd-Config-v1430',
                                                                                            'optional': True,
                                                                                            'type': 'TDD-Config-v1430'},
                                                                                           {'element': {'type': 'CellAccessRelatedInfo-r14'},
                                                                                            'name': 'cellAccessRelatedInfoList-r14',
                                                                                            'optional': True,
                                                                                            'size': [(1,
                                                                                                      'maxPLMN-1-r14')],
                                                                                            'type': 'SEQUENCE '
                                                                                                    'OF'},
                                                                                           {'members': [],
                                                                                            'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SEQUENCE'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v890-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'OCTET '
                                                                                                   'STRING'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType1-v920-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v8h0-IEs': {'members': [{'name': 'multiBandInfoList',
                                                                                           'optional': True,
                                                                                           'type': 'MultiBandInfoList'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType1-v9e0-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v920-IEs': {'members': [{'name': 'ims-EmergencySupport-r9',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('true',
                                                                                                       0)]},
                                                                                          {'name': 'cellSelectionInfo-v920',
                                                                                           'optional': True,
                                                                                           'type': 'CellSelectionInfo-v920'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType1-v1130-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-v9e0-IEs': {'members': [{'name': 'freqBandIndicator-v9e0',
                                                                                           'optional': True,
                                                                                           'type': 'FreqBandIndicator-v9e0'},
                                                                                          {'name': 'multiBandInfoList-v9e0',
                                                                                           'optional': True,
                                                                                           'type': 'MultiBandInfoList-v9e0'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType1-v10j0-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType10': {'members': [{'name': 'messageIdentifier',
                                                                                   'size': [16],
                                                                                   'type': 'BIT '
                                                                                           'STRING'},
                                                                                  {'name': 'serialNumber',
                                                                                   'size': [16],
                                                                                   'type': 'BIT '
                                                                                           'STRING'},
                                                                                  {'name': 'warningType',
                                                                                   'size': [2],
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'name': 'dummy',
                                                                                   'optional': True,
                                                                                   'size': [50],
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  None,
                                                                                  {'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'}],
                                                                      'type': 'SEQUENCE'},
                                     'SystemInformationBlockType11': {'members': [{'name': 'messageIdentifier',
                                                                                   'size': [16],
                                                                                   'type': 'BIT '
                                                                                           'STRING'},
                                                                                  {'name': 'serialNumber',
                                                                                   'size': [16],
                                                                                   'type': 'BIT '
                                                                                           'STRING'},
                                                                                  {'name': 'warningMessageSegmentType',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('notLastSegment',
                                                                                               0),
                                                                                              ('lastSegment',
                                                                                               1)]},
                                                                                  {'name': 'warningMessageSegmentNumber',
                                                                                   'restricted-to': [(0,
                                                                                                      63)],
                                                                                   'type': 'INTEGER'},
                                                                                  {'name': 'warningMessageSegment',
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'name': 'dataCodingScheme',
                                                                                   'optional': True,
                                                                                   'size': [1],
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  None,
                                                                                  {'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'}],
                                                                      'type': 'SEQUENCE'},
                                     'SystemInformationBlockType12-r9': {'members': [{'name': 'messageIdentifier-r9',
                                                                                      'size': [16],
                                                                                      'type': 'BIT '
                                                                                              'STRING'},
                                                                                     {'name': 'serialNumber-r9',
                                                                                      'size': [16],
                                                                                      'type': 'BIT '
                                                                                              'STRING'},
                                                                                     {'name': 'warningMessageSegmentType-r9',
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('notLastSegment',
                                                                                                  0),
                                                                                                 ('lastSegment',
                                                                                                  1)]},
                                                                                     {'name': 'warningMessageSegmentNumber-r9',
                                                                                      'restricted-to': [(0,
                                                                                                         63)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'warningMessageSegment-r9',
                                                                                      'type': 'OCTET '
                                                                                              'STRING'},
                                                                                     {'name': 'dataCodingScheme-r9',
                                                                                      'optional': True,
                                                                                      'size': [1],
                                                                                      'type': 'OCTET '
                                                                                              'STRING'},
                                                                                     {'name': 'lateNonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'OCTET '
                                                                                              'STRING'},
                                                                                     None],
                                                                         'type': 'SEQUENCE'},
                                     'SystemInformationBlockType13-r9': {'members': [{'name': 'mbsfn-AreaInfoList-r9',
                                                                                      'type': 'MBSFN-AreaInfoList-r9'},
                                                                                     {'name': 'notificationConfig-r9',
                                                                                      'type': 'MBMS-NotificationConfig-r9'},
                                                                                     {'name': 'lateNonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'OCTET '
                                                                                              'STRING'},
                                                                                     None,
                                                                                     [{'name': 'notificationConfig-v1430',
                                                                                       'optional': True,
                                                                                       'type': 'MBMS-NotificationConfig-v1430'}]],
                                                                         'type': 'SEQUENCE'},
                                     'SystemInformationBlockType14-r11': {'members': [{'members': [{'name': 'eab-Common-r11',
                                                                                                    'type': 'EAB-Config-r11'},
                                                                                                   {'element': {'type': 'EAB-ConfigPLMN-r11'},
                                                                                                    'name': 'eab-PerPLMN-List-r11',
                                                                                                    'size': [(1,
                                                                                                              'maxPLMN-r11')],
                                                                                                    'type': 'SEQUENCE '
                                                                                                            'OF'}],
                                                                                       'name': 'eab-Param-r11',
                                                                                       'optional': True,
                                                                                       'type': 'CHOICE'},
                                                                                      {'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      None],
                                                                          'type': 'SEQUENCE'},
                                     'SystemInformationBlockType15-r11': {'members': [{'name': 'mbms-SAI-IntraFreq-r11',
                                                                                       'optional': True,
                                                                                       'type': 'MBMS-SAI-List-r11'},
                                                                                      {'name': 'mbms-SAI-InterFreqList-r11',
                                                                                       'optional': True,
                                                                                       'type': 'MBMS-SAI-InterFreqList-r11'},
                                                                                      {'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      None,
                                                                                      [{'name': 'mbms-SAI-InterFreqList-v1140',
                                                                                        'optional': True,
                                                                                        'type': 'MBMS-SAI-InterFreqList-v1140'}],
                                                                                      [{'name': 'mbms-IntraFreqCarrierType-r14',
                                                                                        'optional': True,
                                                                                        'type': 'MBMS-CarrierType-r14'},
                                                                                       {'name': 'mbms-InterFreqCarrierTypeList-r14',
                                                                                        'optional': True,
                                                                                        'type': 'MBMS-InterFreqCarrierTypeList-r14'}]],
                                                                          'type': 'SEQUENCE'},
                                     'SystemInformationBlockType16-r11': {'members': [{'members': [{'name': 'timeInfoUTC-r11',
                                                                                                    'restricted-to': [(0,
                                                                                                                       549755813887)],
                                                                                                    'type': 'INTEGER'},
                                                                                                   {'name': 'dayLightSavingTime-r11',
                                                                                                    'optional': True,
                                                                                                    'size': [2],
                                                                                                    'type': 'BIT '
                                                                                                            'STRING'},
                                                                                                   {'name': 'leapSeconds-r11',
                                                                                                    'optional': True,
                                                                                                    'restricted-to': [(-127,
                                                                                                                       128)],
                                                                                                    'type': 'INTEGER'},
                                                                                                   {'name': 'localTimeOffset-r11',
                                                                                                    'optional': True,
                                                                                                    'restricted-to': [(-63,
                                                                                                                       64)],
                                                                                                    'type': 'INTEGER'}],
                                                                                       'name': 'timeInfo-r11',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'},
                                                                                      {'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      None],
                                                                          'type': 'SEQUENCE'},
                                     'SystemInformationBlockType17-r12': {'members': [{'element': {'type': 'WLAN-OffloadInfoPerPLMN-r12'},
                                                                                       'name': 'wlan-OffloadInfoPerPLMN-List-r12',
                                                                                       'optional': True,
                                                                                       'size': [(1,
                                                                                                 'maxPLMN-r11')],
                                                                                       'type': 'SEQUENCE '
                                                                                               'OF'},
                                                                                      {'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      None],
                                                                          'type': 'SEQUENCE'},
                                     'SystemInformationBlockType18-r12': {'members': [{'members': [{'name': 'commRxPool-r12',
                                                                                                    'type': 'SL-CommRxPoolList-r12'},
                                                                                                   {'name': 'commTxPoolNormalCommon-r12',
                                                                                                    'optional': True,
                                                                                                    'type': 'SL-CommTxPoolList-r12'},
                                                                                                   {'name': 'commTxPoolExceptional-r12',
                                                                                                    'optional': True,
                                                                                                    'type': 'SL-CommTxPoolList-r12'},
                                                                                                   {'name': 'commSyncConfig-r12',
                                                                                                    'optional': True,
                                                                                                    'type': 'SL-SyncConfigList-r12'}],
                                                                                       'name': 'commConfig-r12',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'},
                                                                                      {'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      None,
                                                                                      [{'name': 'commTxPoolNormalCommonExt-r13',
                                                                                        'optional': True,
                                                                                        'type': 'SL-CommTxPoolListExt-r13'},
                                                                                       {'name': 'commTxResourceUC-ReqAllowed-r13',
                                                                                        'optional': True,
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('true',
                                                                                                    0)]},
                                                                                       {'name': 'commTxAllowRelayCommon-r13',
                                                                                        'optional': True,
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('true',
                                                                                                    0)]}]],
                                                                          'type': 'SEQUENCE'},
                                     'SystemInformationBlockType19-r12': {'members': [{'members': [{'name': 'discRxPool-r12',
                                                                                                    'type': 'SL-DiscRxPoolList-r12'},
                                                                                                   {'name': 'discTxPoolCommon-r12',
                                                                                                    'optional': True,
                                                                                                    'type': 'SL-DiscTxPoolList-r12'},
                                                                                                   {'name': 'discTxPowerInfo-r12',
                                                                                                    'optional': True,
                                                                                                    'type': 'SL-DiscTxPowerInfoList-r12'},
                                                                                                   {'name': 'discSyncConfig-r12',
                                                                                                    'optional': True,
                                                                                                    'type': 'SL-SyncConfigList-r12'}],
                                                                                       'name': 'discConfig-r12',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'},
                                                                                      {'name': 'discInterFreqList-r12',
                                                                                       'optional': True,
                                                                                       'type': 'SL-CarrierFreqInfoList-r12'},
                                                                                      {'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      None,
                                                                                      [{'members': [{'name': 'discInterFreqList-v1310',
                                                                                                     'optional': True,
                                                                                                     'type': 'SL-CarrierFreqInfoList-v1310'},
                                                                                                    {'name': 'gapRequestsAllowedCommon',
                                                                                                     'optional': True,
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('true',
                                                                                                                 0)]}],
                                                                                        'name': 'discConfig-v1310',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'},
                                                                                       {'members': [{'name': 'relayUE-Config-r13',
                                                                                                     'type': 'SL-DiscConfigRelayUE-r13'},
                                                                                                    {'name': 'remoteUE-Config-r13',
                                                                                                     'type': 'SL-DiscConfigRemoteUE-r13'}],
                                                                                        'name': 'discConfigRelay-r13',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'},
                                                                                       {'members': [{'name': 'discRxPoolPS-r13',
                                                                                                     'type': 'SL-DiscRxPoolList-r12'},
                                                                                                    {'name': 'discTxPoolPS-Common-r13',
                                                                                                     'optional': True,
                                                                                                     'type': 'SL-DiscTxPoolList-r12'}],
                                                                                        'name': 'discConfigPS-13',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'}]],
                                                                          'type': 'SEQUENCE'},
                                     'SystemInformationBlockType2': {'members': [{'members': [{'name': 'ac-BarringForEmergency',
                                                                                               'type': 'BOOLEAN'},
                                                                                              {'name': 'ac-BarringForMO-Signalling',
                                                                                               'optional': True,
                                                                                               'type': 'AC-BarringConfig'},
                                                                                              {'name': 'ac-BarringForMO-Data',
                                                                                               'optional': True,
                                                                                               'type': 'AC-BarringConfig'}],
                                                                                  'name': 'ac-BarringInfo',
                                                                                  'optional': True,
                                                                                  'type': 'SEQUENCE'},
                                                                                 {'name': 'radioResourceConfigCommon',
                                                                                  'type': 'RadioResourceConfigCommonSIB'},
                                                                                 {'name': 'ue-TimersAndConstants',
                                                                                  'type': 'UE-TimersAndConstants'},
                                                                                 {'members': [{'name': 'ul-CarrierFreq',
                                                                                               'optional': True,
                                                                                               'type': 'ARFCN-ValueEUTRA'},
                                                                                              {'name': 'ul-Bandwidth',
                                                                                               'optional': True,
                                                                                               'type': 'ENUMERATED',
                                                                                               'values': [('n6',
                                                                                                           0),
                                                                                                          ('n15',
                                                                                                           1),
                                                                                                          ('n25',
                                                                                                           2),
                                                                                                          ('n50',
                                                                                                           3),
                                                                                                          ('n75',
                                                                                                           4),
                                                                                                          ('n100',
                                                                                                           5)]},
                                                                                              {'name': 'additionalSpectrumEmission',
                                                                                               'type': 'AdditionalSpectrumEmission'}],
                                                                                  'name': 'freqInfo',
                                                                                  'type': 'SEQUENCE'},
                                                                                 {'name': 'mbsfn-SubframeConfigList',
                                                                                  'optional': True,
                                                                                  'type': 'MBSFN-SubframeConfigList'},
                                                                                 {'name': 'timeAlignmentTimerCommon',
                                                                                  'type': 'TimeAlignmentTimer'},
                                                                                 None,
                                                                                 {'name': 'lateNonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 [{'name': 'ssac-BarringForMMTEL-Voice-r9',
                                                                                   'optional': True,
                                                                                   'type': 'AC-BarringConfig'},
                                                                                  {'name': 'ssac-BarringForMMTEL-Video-r9',
                                                                                   'optional': True,
                                                                                   'type': 'AC-BarringConfig'}],
                                                                                 [{'name': 'ac-BarringForCSFB-r10',
                                                                                   'optional': True,
                                                                                   'type': 'AC-BarringConfig'}],
                                                                                 [{'name': 'ac-BarringSkipForMMTELVoice-r12',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]},
                                                                                  {'name': 'ac-BarringSkipForMMTELVideo-r12',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]},
                                                                                  {'name': 'ac-BarringSkipForSMS-r12',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]},
                                                                                  {'name': 'ac-BarringPerPLMN-List-r12',
                                                                                   'optional': True,
                                                                                   'type': 'AC-BarringPerPLMN-List-r12'}],
                                                                                 [{'name': 'voiceServiceCauseIndication-r12',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]}],
                                                                                 [{'name': 'acdc-BarringForCommon-r13',
                                                                                   'optional': True,
                                                                                   'type': 'ACDC-BarringForCommon-r13'},
                                                                                  {'name': 'acdc-BarringPerPLMN-List-r13',
                                                                                   'optional': True,
                                                                                   'type': 'ACDC-BarringPerPLMN-List-r13'}],
                                                                                 [{'name': 'udt-RestrictingForCommon-r13',
                                                                                   'optional': True,
                                                                                   'type': 'UDT-Restricting-r13'},
                                                                                  {'name': 'udt-RestrictingPerPLMN-List-r13',
                                                                                   'optional': True,
                                                                                   'type': 'UDT-RestrictingPerPLMN-List-r13'},
                                                                                  {'name': 'cIoT-EPS-OptimisationInfo-r13',
                                                                                   'optional': True,
                                                                                   'type': 'CIOT-EPS-OptimisationInfo-r13'},
                                                                                  {'name': 'useFullResumeID-r13',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]}],
                                                                                 [{'name': 'unicastFreqHoppingInd-r13',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]}],
                                                                                 [{'name': 'mbsfn-SubframeConfigList-v1430',
                                                                                   'optional': True,
                                                                                   'type': 'MBSFN-SubframeConfigList-v1430'},
                                                                                  {'name': 'videoServiceCauseIndication-r14',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]}]],
                                                                     'type': 'SEQUENCE'},
                                     'SystemInformationBlockType2-v10l0-IEs': {'members': [{'members': [{'name': 'additionalSpectrumEmission-v10l0',
                                                                                                         'type': 'AdditionalSpectrumEmission-v10l0'}],
                                                                                            'name': 'freqInfo-v10l0',
                                                                                            'type': 'SEQUENCE'},
                                                                                           {'element': {'type': 'AdditionalSpectrumEmission-v10l0'},
                                                                                            'name': 'multiBandInfoList-v10l0',
                                                                                            'size': [(1,
                                                                                                      'maxMultiBands')],
                                                                                            'type': 'SEQUENCE '
                                                                                                    'OF'},
                                                                                           {'members': [],
                                                                                            'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SEQUENCE'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType2-v8h0-IEs': {'members': [{'element': {'type': 'AdditionalSpectrumEmission'},
                                                                                           'name': 'multiBandInfoList',
                                                                                           'optional': True,
                                                                                           'size': [(1,
                                                                                                     'maxMultiBands')],
                                                                                           'type': 'SEQUENCE '
                                                                                                   'OF'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType2-v9e0-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType2-v9e0-IEs': {'members': [{'name': 'ul-CarrierFreq-v9e0',
                                                                                           'optional': True,
                                                                                           'type': 'ARFCN-ValueEUTRA-v9e0'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType2-v9i0-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType2-v9i0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'OCTET '
                                                                                                   'STRING'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType2-v10l0-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType20-r13': {'members': [{'name': 'sc-mcch-RepetitionPeriod-r13',
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('rf2',
                                                                                                   0),
                                                                                                  ('rf4',
                                                                                                   1),
                                                                                                  ('rf8',
                                                                                                   2),
                                                                                                  ('rf16',
                                                                                                   3),
                                                                                                  ('rf32',
                                                                                                   4),
                                                                                                  ('rf64',
                                                                                                   5),
                                                                                                  ('rf128',
                                                                                                   6),
                                                                                                  ('rf256',
                                                                                                   7)]},
                                                                                      {'name': 'sc-mcch-Offset-r13',
                                                                                       'restricted-to': [(0,
                                                                                                          10)],
                                                                                       'type': 'INTEGER'},
                                                                                      {'name': 'sc-mcch-FirstSubframe-r13',
                                                                                       'restricted-to': [(0,
                                                                                                          9)],
                                                                                       'type': 'INTEGER'},
                                                                                      {'name': 'sc-mcch-duration-r13',
                                                                                       'optional': True,
                                                                                       'restricted-to': [(2,
                                                                                                          9)],
                                                                                       'type': 'INTEGER'},
                                                                                      {'name': 'sc-mcch-ModificationPeriod-r13',
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('rf2',
                                                                                                   0),
                                                                                                  ('rf4',
                                                                                                   1),
                                                                                                  ('rf8',
                                                                                                   2),
                                                                                                  ('rf16',
                                                                                                   3),
                                                                                                  ('rf32',
                                                                                                   4),
                                                                                                  ('rf64',
                                                                                                   5),
                                                                                                  ('rf128',
                                                                                                   6),
                                                                                                  ('rf256',
                                                                                                   7),
                                                                                                  ('rf512',
                                                                                                   8),
                                                                                                  ('rf1024',
                                                                                                   9),
                                                                                                  ('r2048',
                                                                                                   10),
                                                                                                  ('rf4096',
                                                                                                   11),
                                                                                                  ('rf8192',
                                                                                                   12),
                                                                                                  ('rf16384',
                                                                                                   13),
                                                                                                  ('rf32768',
                                                                                                   14),
                                                                                                  ('rf65536',
                                                                                                   15)]},
                                                                                      {'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      None,
                                                                                      [{'members': [{'name': 'sc-mcch-RepetitionPeriod-v1430',
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('rf1',
                                                                                                                 0)]},
                                                                                                    {'name': 'sc-mcch-ModificationPeriod-v1430',
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('rf1',
                                                                                                                 0)]},
                                                                                                    {'name': 'mpdcch-Narrowband-SC-MCCH-r14',
                                                                                                     'restricted-to': [(1,
                                                                                                                        'maxAvailNarrowBands-r13')],
                                                                                                     'type': 'INTEGER'},
                                                                                                    {'name': 'mpdcch-NumRepetition-SC-MCCH-r14',
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('r1',
                                                                                                                 0),
                                                                                                                ('r2',
                                                                                                                 1),
                                                                                                                ('r4',
                                                                                                                 2),
                                                                                                                ('r8',
                                                                                                                 3),
                                                                                                                ('r16',
                                                                                                                 4),
                                                                                                                ('r32',
                                                                                                                 5),
                                                                                                                ('r64',
                                                                                                                 6),
                                                                                                                ('r128',
                                                                                                                 7),
                                                                                                                ('r256',
                                                                                                                 8)]},
                                                                                                    {'members': [{'name': 'fdd-r14',
                                                                                                                  'type': 'ENUMERATED',
                                                                                                                  'values': [('v1',
                                                                                                                              0),
                                                                                                                             ('v1dot5',
                                                                                                                              1),
                                                                                                                             ('v2',
                                                                                                                              2),
                                                                                                                             ('v2dot5',
                                                                                                                              3),
                                                                                                                             ('v4',
                                                                                                                              4),
                                                                                                                             ('v5',
                                                                                                                              5),
                                                                                                                             ('v8',
                                                                                                                              6),
                                                                                                                             ('v10',
                                                                                                                              7)]},
                                                                                                                 {'name': 'tdd-r14',
                                                                                                                  'type': 'ENUMERATED',
                                                                                                                  'values': [('v1',
                                                                                                                              0),
                                                                                                                             ('v2',
                                                                                                                              1),
                                                                                                                             ('v4',
                                                                                                                              2),
                                                                                                                             ('v5',
                                                                                                                              3),
                                                                                                                             ('v8',
                                                                                                                              4),
                                                                                                                             ('v10',
                                                                                                                              5),
                                                                                                                             ('v20',
                                                                                                                              6)]}],
                                                                                                     'name': 'mpdcch-StartSF-SC-MCCH-r14',
                                                                                                     'type': 'CHOICE'},
                                                                                                    {'name': 'mpdcch-PDSCH-HoppingConfig-SC-MCCH-r14',
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('off',
                                                                                                                 0),
                                                                                                                ('ce-ModeA',
                                                                                                                 1),
                                                                                                                ('ce-ModeB',
                                                                                                                 2)]},
                                                                                                    {'name': 'sc-mcch-CarrierFreq-r14',
                                                                                                     'type': 'ARFCN-ValueEUTRA-r9'},
                                                                                                    {'name': 'sc-mcch-Offset-BR-r14',
                                                                                                     'restricted-to': [(0,
                                                                                                                        10)],
                                                                                                     'type': 'INTEGER'},
                                                                                                    {'name': 'sc-mcch-RepetitionPeriod-BR-r14',
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('rf32',
                                                                                                                 0),
                                                                                                                ('rf128',
                                                                                                                 1),
                                                                                                                ('rf512',
                                                                                                                 2),
                                                                                                                ('rf1024',
                                                                                                                 3),
                                                                                                                ('rf2048',
                                                                                                                 4),
                                                                                                                ('rf4096',
                                                                                                                 5),
                                                                                                                ('rf8192',
                                                                                                                 6),
                                                                                                                ('rf16384',
                                                                                                                 7)]},
                                                                                                    {'name': 'sc-mcch-ModificationPeriod-BR-r14',
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('rf32',
                                                                                                                 0),
                                                                                                                ('rf128',
                                                                                                                 1),
                                                                                                                ('rf256',
                                                                                                                 2),
                                                                                                                ('rf512',
                                                                                                                 3),
                                                                                                                ('rf1024',
                                                                                                                 4),
                                                                                                                ('rf2048',
                                                                                                                 5),
                                                                                                                ('rf4096',
                                                                                                                 6),
                                                                                                                ('rf8192',
                                                                                                                 7),
                                                                                                                ('rf16384',
                                                                                                                 8),
                                                                                                                ('rf32768',
                                                                                                                 9),
                                                                                                                ('rf65536',
                                                                                                                 10),
                                                                                                                ('rf131072',
                                                                                                                 11),
                                                                                                                ('rf262144',
                                                                                                                 12),
                                                                                                                ('rf524288',
                                                                                                                 13),
                                                                                                                ('rf1048576',
                                                                                                                 14)]}],
                                                                                        'name': 'br-BCCH-Config-r14',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'},
                                                                                       {'name': 'sc-mcch-SchedulingInfo-r14',
                                                                                        'optional': True,
                                                                                        'type': 'SC-MCCH-SchedulingInfo-r14'},
                                                                                       {'name': 'pdsch-maxNumRepetitionCEmodeA-SC-MTCH-r14',
                                                                                        'optional': True,
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('r16',
                                                                                                    0),
                                                                                                   ('r32',
                                                                                                    1)]},
                                                                                       {'name': 'pdsch-maxNumRepetitionCEmodeB-SC-MTCH-r14',
                                                                                        'optional': True,
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('r192',
                                                                                                    0),
                                                                                                   ('r256',
                                                                                                    1),
                                                                                                   ('r384',
                                                                                                    2),
                                                                                                   ('r512',
                                                                                                    3),
                                                                                                   ('r768',
                                                                                                    4),
                                                                                                   ('r1024',
                                                                                                    5),
                                                                                                   ('r1536',
                                                                                                    6),
                                                                                                   ('r2048',
                                                                                                    7)]}]],
                                                                          'type': 'SEQUENCE'},
                                     'SystemInformationBlockType21-r14': {'members': [{'name': 'sl-V2X-ConfigCommon-r14',
                                                                                       'optional': True,
                                                                                       'type': 'SL-V2X-ConfigCommon-r14'},
                                                                                      {'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      None],
                                                                          'type': 'SEQUENCE'},
                                     'SystemInformationBlockType3': {'members': [{'members': [{'name': 'q-Hyst',
                                                                                               'type': 'ENUMERATED',
                                                                                               'values': [('dB0',
                                                                                                           0),
                                                                                                          ('dB1',
                                                                                                           1),
                                                                                                          ('dB2',
                                                                                                           2),
                                                                                                          ('dB3',
                                                                                                           3),
                                                                                                          ('dB4',
                                                                                                           4),
                                                                                                          ('dB5',
                                                                                                           5),
                                                                                                          ('dB6',
                                                                                                           6),
                                                                                                          ('dB8',
                                                                                                           7),
                                                                                                          ('dB10',
                                                                                                           8),
                                                                                                          ('dB12',
                                                                                                           9),
                                                                                                          ('dB14',
                                                                                                           10),
                                                                                                          ('dB16',
                                                                                                           11),
                                                                                                          ('dB18',
                                                                                                           12),
                                                                                                          ('dB20',
                                                                                                           13),
                                                                                                          ('dB22',
                                                                                                           14),
                                                                                                          ('dB24',
                                                                                                           15)]},
                                                                                              {'members': [{'name': 'mobilityStateParameters',
                                                                                                            'type': 'MobilityStateParameters'},
                                                                                                           {'members': [{'name': 'sf-Medium',
                                                                                                                         'type': 'ENUMERATED',
                                                                                                                         'values': [('dB-6',
                                                                                                                                     0),
                                                                                                                                    ('dB-4',
                                                                                                                                     1),
                                                                                                                                    ('dB-2',
                                                                                                                                     2),
                                                                                                                                    ('dB0',
                                                                                                                                     3)]},
                                                                                                                        {'name': 'sf-High',
                                                                                                                         'type': 'ENUMERATED',
                                                                                                                         'values': [('dB-6',
                                                                                                                                     0),
                                                                                                                                    ('dB-4',
                                                                                                                                     1),
                                                                                                                                    ('dB-2',
                                                                                                                                     2),
                                                                                                                                    ('dB0',
                                                                                                                                     3)]}],
                                                                                                            'name': 'q-HystSF',
                                                                                                            'type': 'SEQUENCE'}],
                                                                                               'name': 'speedStateReselectionPars',
                                                                                               'optional': True,
                                                                                               'type': 'SEQUENCE'}],
                                                                                  'name': 'cellReselectionInfoCommon',
                                                                                  'type': 'SEQUENCE'},
                                                                                 {'members': [{'name': 's-NonIntraSearch',
                                                                                               'optional': True,
                                                                                               'type': 'ReselectionThreshold'},
                                                                                              {'name': 'threshServingLow',
                                                                                               'type': 'ReselectionThreshold'},
                                                                                              {'name': 'cellReselectionPriority',
                                                                                               'type': 'CellReselectionPriority'}],
                                                                                  'name': 'cellReselectionServingFreqInfo',
                                                                                  'type': 'SEQUENCE'},
                                                                                 {'members': [{'name': 'q-RxLevMin',
                                                                                               'type': 'Q-RxLevMin'},
                                                                                              {'name': 'p-Max',
                                                                                               'optional': True,
                                                                                               'type': 'P-Max'},
                                                                                              {'name': 's-IntraSearch',
                                                                                               'optional': True,
                                                                                               'type': 'ReselectionThreshold'},
                                                                                              {'name': 'allowedMeasBandwidth',
                                                                                               'optional': True,
                                                                                               'type': 'AllowedMeasBandwidth'},
                                                                                              {'name': 'presenceAntennaPort1',
                                                                                               'type': 'PresenceAntennaPort1'},
                                                                                              {'name': 'neighCellConfig',
                                                                                               'type': 'NeighCellConfig'},
                                                                                              {'name': 't-ReselectionEUTRA',
                                                                                               'type': 'T-Reselection'},
                                                                                              {'name': 't-ReselectionEUTRA-SF',
                                                                                               'optional': True,
                                                                                               'type': 'SpeedStateScaleFactors'}],
                                                                                  'name': 'intraFreqCellReselectionInfo',
                                                                                  'type': 'SEQUENCE'},
                                                                                 None,
                                                                                 {'name': 'lateNonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 [{'members': [{'name': 's-IntraSearchP-r9',
                                                                                                'type': 'ReselectionThreshold'},
                                                                                               {'name': 's-IntraSearchQ-r9',
                                                                                                'type': 'ReselectionThresholdQ-r9'}],
                                                                                   'name': 's-IntraSearch-v920',
                                                                                   'optional': True,
                                                                                   'type': 'SEQUENCE'},
                                                                                  {'members': [{'name': 's-NonIntraSearchP-r9',
                                                                                                'type': 'ReselectionThreshold'},
                                                                                               {'name': 's-NonIntraSearchQ-r9',
                                                                                                'type': 'ReselectionThresholdQ-r9'}],
                                                                                   'name': 's-NonIntraSearch-v920',
                                                                                   'optional': True,
                                                                                   'type': 'SEQUENCE'},
                                                                                  {'name': 'q-QualMin-r9',
                                                                                   'optional': True,
                                                                                   'type': 'Q-QualMin-r9'},
                                                                                  {'name': 'threshServingLowQ-r9',
                                                                                   'optional': True,
                                                                                   'type': 'ReselectionThresholdQ-r9'}],
                                                                                 [{'name': 'q-QualMinWB-r11',
                                                                                   'optional': True,
                                                                                   'type': 'Q-QualMin-r9'}],
                                                                                 [{'name': 'q-QualMinRSRQ-OnAllSymbols-r12',
                                                                                   'optional': True,
                                                                                   'type': 'Q-QualMin-r9'}],
                                                                                 [{'name': 'cellReselectionServingFreqInfo-v1310',
                                                                                   'optional': True,
                                                                                   'type': 'CellReselectionServingFreqInfo-v1310'},
                                                                                  {'name': 'redistributionServingInfo-r13',
                                                                                   'optional': True,
                                                                                   'type': 'RedistributionServingInfo-r13'},
                                                                                  {'name': 'cellSelectionInfoCE-r13',
                                                                                   'optional': True,
                                                                                   'type': 'CellSelectionInfoCE-r13'},
                                                                                  {'name': 't-ReselectionEUTRA-CE-r13',
                                                                                   'optional': True,
                                                                                   'type': 'T-ReselectionEUTRA-CE-r13'}],
                                                                                 [{'name': 'cellSelectionInfoCE1-r13',
                                                                                   'optional': True,
                                                                                   'type': 'CellSelectionInfoCE1-r13'}],
                                                                                 [{'name': 'cellSelectionInfoCE1-v1360',
                                                                                   'optional': True,
                                                                                   'type': 'CellSelectionInfoCE1-v1360'}]],
                                                                     'type': 'SEQUENCE'},
                                     'SystemInformationBlockType3-v10j0-IEs': {'members': [{'name': 'freqBandInfo-r10',
                                                                                            'optional': True,
                                                                                            'type': 'NS-PmaxList-r10'},
                                                                                           {'name': 'multiBandInfoList-v10j0',
                                                                                            'optional': True,
                                                                                            'type': 'MultiBandInfoList-v10j0'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SystemInformationBlockType3-v10l0-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType3-v10l0-IEs': {'members': [{'name': 'freqBandInfo-v10l0',
                                                                                            'optional': True,
                                                                                            'type': 'NS-PmaxList-v10l0'},
                                                                                           {'name': 'multiBandInfoList-v10l0',
                                                                                            'optional': True,
                                                                                            'type': 'MultiBandInfoList-v10l0'},
                                                                                           {'members': [],
                                                                                            'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SEQUENCE'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType4': {'members': [{'name': 'intraFreqNeighCellList',
                                                                                  'optional': True,
                                                                                  'type': 'IntraFreqNeighCellList'},
                                                                                 {'name': 'intraFreqBlackCellList',
                                                                                  'optional': True,
                                                                                  'type': 'IntraFreqBlackCellList'},
                                                                                 {'name': 'csg-PhysCellIdRange',
                                                                                  'optional': True,
                                                                                  'type': 'PhysCellIdRange'},
                                                                                 None,
                                                                                 {'name': 'lateNonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'}],
                                                                     'type': 'SEQUENCE'},
                                     'SystemInformationBlockType5': {'members': [{'name': 'interFreqCarrierFreqList',
                                                                                  'type': 'InterFreqCarrierFreqList'},
                                                                                 None,
                                                                                 {'name': 'lateNonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 [{'name': 'interFreqCarrierFreqList-v1250',
                                                                                   'optional': True,
                                                                                   'type': 'InterFreqCarrierFreqList-v1250'},
                                                                                  {'name': 'interFreqCarrierFreqListExt-r12',
                                                                                   'optional': True,
                                                                                   'type': 'InterFreqCarrierFreqListExt-r12'}],
                                                                                 [{'name': 'interFreqCarrierFreqListExt-v1280',
                                                                                   'optional': True,
                                                                                   'type': 'InterFreqCarrierFreqListExt-v1280'}],
                                                                                 [{'name': 'interFreqCarrierFreqList-v1310',
                                                                                   'optional': True,
                                                                                   'type': 'InterFreqCarrierFreqList-v1310'},
                                                                                  {'name': 'interFreqCarrierFreqListExt-v1310',
                                                                                   'optional': True,
                                                                                   'type': 'InterFreqCarrierFreqListExt-v1310'}],
                                                                                 [{'name': 'interFreqCarrierFreqList-v1350',
                                                                                   'optional': True,
                                                                                   'type': 'InterFreqCarrierFreqList-v1350'},
                                                                                  {'name': 'interFreqCarrierFreqListExt-v1350',
                                                                                   'optional': True,
                                                                                   'type': 'InterFreqCarrierFreqListExt-v1350'}],
                                                                                 [{'name': 'interFreqCarrierFreqListExt-v1360',
                                                                                   'optional': True,
                                                                                   'type': 'InterFreqCarrierFreqListExt-v1360'}],
                                                                                 [{'name': 'scptm-FreqOffset-r14',
                                                                                   'optional': True,
                                                                                   'restricted-to': [(1,
                                                                                                      8)],
                                                                                   'type': 'INTEGER'}]],
                                                                     'type': 'SEQUENCE'},
                                     'SystemInformationBlockType5-v10j0-IEs': {'members': [{'element': {'type': 'InterFreqCarrierFreqInfo-v10j0'},
                                                                                            'name': 'interFreqCarrierFreqList-v10j0',
                                                                                            'optional': True,
                                                                                            'size': [(1,
                                                                                                      'maxFreq')],
                                                                                            'type': 'SEQUENCE '
                                                                                                    'OF'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SystemInformationBlockType5-v10l0-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType5-v10l0-IEs': {'members': [{'element': {'type': 'InterFreqCarrierFreqInfo-v10l0'},
                                                                                            'name': 'interFreqCarrierFreqList-v10l0',
                                                                                            'optional': True,
                                                                                            'size': [(1,
                                                                                                      'maxFreq')],
                                                                                            'type': 'SEQUENCE '
                                                                                                    'OF'},
                                                                                           {'members': [],
                                                                                            'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'SEQUENCE'}],
                                                                               'type': 'SEQUENCE'},
                                     'SystemInformationBlockType5-v8h0-IEs': {'members': [{'element': {'type': 'InterFreqCarrierFreqInfo-v8h0'},
                                                                                           'name': 'interFreqCarrierFreqList-v8h0',
                                                                                           'optional': True,
                                                                                           'size': [(1,
                                                                                                     'maxFreq')],
                                                                                           'type': 'SEQUENCE '
                                                                                                   'OF'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType5-v9e0-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType5-v9e0-IEs': {'members': [{'element': {'type': 'InterFreqCarrierFreqInfo-v9e0'},
                                                                                           'name': 'interFreqCarrierFreqList-v9e0',
                                                                                           'optional': True,
                                                                                           'size': [(1,
                                                                                                     'maxFreq')],
                                                                                           'type': 'SEQUENCE '
                                                                                                   'OF'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType5-v10j0-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType6': {'members': [{'name': 'carrierFreqListUTRA-FDD',
                                                                                  'optional': True,
                                                                                  'type': 'CarrierFreqListUTRA-FDD'},
                                                                                 {'name': 'carrierFreqListUTRA-TDD',
                                                                                  'optional': True,
                                                                                  'type': 'CarrierFreqListUTRA-TDD'},
                                                                                 {'name': 't-ReselectionUTRA',
                                                                                  'type': 'T-Reselection'},
                                                                                 {'name': 't-ReselectionUTRA-SF',
                                                                                  'optional': True,
                                                                                  'type': 'SpeedStateScaleFactors'},
                                                                                 None,
                                                                                 {'name': 'lateNonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 [{'element': {'type': 'CarrierFreqInfoUTRA-v1250'},
                                                                                   'name': 'carrierFreqListUTRA-FDD-v1250',
                                                                                   'optional': True,
                                                                                   'size': [(1,
                                                                                             'maxUTRA-FDD-Carrier')],
                                                                                   'type': 'SEQUENCE '
                                                                                           'OF'},
                                                                                  {'element': {'type': 'CarrierFreqInfoUTRA-v1250'},
                                                                                   'name': 'carrierFreqListUTRA-TDD-v1250',
                                                                                   'optional': True,
                                                                                   'size': [(1,
                                                                                             'maxUTRA-TDD-Carrier')],
                                                                                   'type': 'SEQUENCE '
                                                                                           'OF'},
                                                                                  {'name': 'carrierFreqListUTRA-FDD-Ext-r12',
                                                                                   'optional': True,
                                                                                   'type': 'CarrierFreqListUTRA-FDD-Ext-r12'},
                                                                                  {'name': 'carrierFreqListUTRA-TDD-Ext-r12',
                                                                                   'optional': True,
                                                                                   'type': 'CarrierFreqListUTRA-TDD-Ext-r12'}]],
                                                                     'type': 'SEQUENCE'},
                                     'SystemInformationBlockType6-v8h0-IEs': {'members': [{'element': {'type': 'CarrierFreqInfoUTRA-FDD-v8h0'},
                                                                                           'name': 'carrierFreqListUTRA-FDD-v8h0',
                                                                                           'optional': True,
                                                                                           'size': [(1,
                                                                                                     'maxUTRA-FDD-Carrier')],
                                                                                           'type': 'SEQUENCE '
                                                                                                   'OF'},
                                                                                          {'members': [],
                                                                                           'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SEQUENCE'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType7': {'members': [{'name': 't-ReselectionGERAN',
                                                                                  'type': 'T-Reselection'},
                                                                                 {'name': 't-ReselectionGERAN-SF',
                                                                                  'optional': True,
                                                                                  'type': 'SpeedStateScaleFactors'},
                                                                                 {'name': 'carrierFreqsInfoList',
                                                                                  'optional': True,
                                                                                  'type': 'CarrierFreqsInfoListGERAN'},
                                                                                 None,
                                                                                 {'name': 'lateNonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'}],
                                                                     'type': 'SEQUENCE'},
                                     'SystemInformationBlockType8': {'members': [{'name': 'systemTimeInfo',
                                                                                  'optional': True,
                                                                                  'type': 'SystemTimeInfoCDMA2000'},
                                                                                 {'name': 'searchWindowSize',
                                                                                  'optional': True,
                                                                                  'restricted-to': [(0,
                                                                                                     15)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'members': [{'name': 'preRegistrationInfoHRPD',
                                                                                               'type': 'PreRegistrationInfoHRPD'},
                                                                                              {'name': 'cellReselectionParametersHRPD',
                                                                                               'optional': True,
                                                                                               'type': 'CellReselectionParametersCDMA2000'}],
                                                                                  'name': 'parametersHRPD',
                                                                                  'optional': True,
                                                                                  'type': 'SEQUENCE'},
                                                                                 {'members': [{'name': 'csfb-RegistrationParam1XRTT',
                                                                                               'optional': True,
                                                                                               'type': 'CSFB-RegistrationParam1XRTT'},
                                                                                              {'name': 'longCodeState1XRTT',
                                                                                               'optional': True,
                                                                                               'size': [42],
                                                                                               'type': 'BIT '
                                                                                                       'STRING'},
                                                                                              {'name': 'cellReselectionParameters1XRTT',
                                                                                               'optional': True,
                                                                                               'type': 'CellReselectionParametersCDMA2000'}],
                                                                                  'name': 'parameters1XRTT',
                                                                                  'optional': True,
                                                                                  'type': 'SEQUENCE'},
                                                                                 None,
                                                                                 {'name': 'lateNonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 [{'name': 'csfb-SupportForDualRxUEs-r9',
                                                                                   'optional': True,
                                                                                   'type': 'BOOLEAN'},
                                                                                  {'name': 'cellReselectionParametersHRPD-v920',
                                                                                   'optional': True,
                                                                                   'type': 'CellReselectionParametersCDMA2000-v920'},
                                                                                  {'name': 'cellReselectionParameters1XRTT-v920',
                                                                                   'optional': True,
                                                                                   'type': 'CellReselectionParametersCDMA2000-v920'},
                                                                                  {'name': 'csfb-RegistrationParam1XRTT-v920',
                                                                                   'optional': True,
                                                                                   'type': 'CSFB-RegistrationParam1XRTT-v920'},
                                                                                  {'name': 'ac-BarringConfig1XRTT-r9',
                                                                                   'optional': True,
                                                                                   'type': 'AC-BarringConfig1XRTT-r9'}],
                                                                                 [{'name': 'csfb-DualRxTxSupport-r10',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('true',
                                                                                               0)]}],
                                                                                 [{'name': 'sib8-PerPLMN-List-r11',
                                                                                   'optional': True,
                                                                                   'type': 'SIB8-PerPLMN-List-r11'}]],
                                                                     'type': 'SEQUENCE'},
                                     'SystemInformationBlockType9': {'members': [{'name': 'hnb-Name',
                                                                                  'optional': True,
                                                                                  'size': [(1,
                                                                                            48)],
                                                                                  'type': 'OCTET '
                                                                                          'STRING'},
                                                                                 None,
                                                                                 {'name': 'lateNonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'OCTET '
                                                                                          'STRING'}],
                                                                     'type': 'SEQUENCE'},
                                     'SystemTimeInfoCDMA2000': {'members': [{'name': 'cdma-EUTRA-Synchronisation',
                                                                             'type': 'BOOLEAN'},
                                                                            {'members': [{'name': 'synchronousSystemTime',
                                                                                          'size': [39],
                                                                                          'type': 'BIT '
                                                                                                  'STRING'},
                                                                                         {'name': 'asynchronousSystemTime',
                                                                                          'size': [49],
                                                                                          'type': 'BIT '
                                                                                                  'STRING'}],
                                                                             'name': 'cdma-SystemTime',
                                                                             'type': 'CHOICE'}],
                                                                'type': 'SEQUENCE'},
                                     'T-PollRetransmit': {'type': 'ENUMERATED',
                                                          'values': [('ms5', 0),
                                                                     ('ms10',
                                                                      1),
                                                                     ('ms15',
                                                                      2),
                                                                     ('ms20',
                                                                      3),
                                                                     ('ms25',
                                                                      4),
                                                                     ('ms30',
                                                                      5),
                                                                     ('ms35',
                                                                      6),
                                                                     ('ms40',
                                                                      7),
                                                                     ('ms45',
                                                                      8),
                                                                     ('ms50',
                                                                      9),
                                                                     ('ms55',
                                                                      10),
                                                                     ('ms60',
                                                                      11),
                                                                     ('ms65',
                                                                      12),
                                                                     ('ms70',
                                                                      13),
                                                                     ('ms75',
                                                                      14),
                                                                     ('ms80',
                                                                      15),
                                                                     ('ms85',
                                                                      16),
                                                                     ('ms90',
                                                                      17),
                                                                     ('ms95',
                                                                      18),
                                                                     ('ms100',
                                                                      19),
                                                                     ('ms105',
                                                                      20),
                                                                     ('ms110',
                                                                      21),
                                                                     ('ms115',
                                                                      22),
                                                                     ('ms120',
                                                                      23),
                                                                     ('ms125',
                                                                      24),
                                                                     ('ms130',
                                                                      25),
                                                                     ('ms135',
                                                                      26),
                                                                     ('ms140',
                                                                      27),
                                                                     ('ms145',
                                                                      28),
                                                                     ('ms150',
                                                                      29),
                                                                     ('ms155',
                                                                      30),
                                                                     ('ms160',
                                                                      31),
                                                                     ('ms165',
                                                                      32),
                                                                     ('ms170',
                                                                      33),
                                                                     ('ms175',
                                                                      34),
                                                                     ('ms180',
                                                                      35),
                                                                     ('ms185',
                                                                      36),
                                                                     ('ms190',
                                                                      37),
                                                                     ('ms195',
                                                                      38),
                                                                     ('ms200',
                                                                      39),
                                                                     ('ms205',
                                                                      40),
                                                                     ('ms210',
                                                                      41),
                                                                     ('ms215',
                                                                      42),
                                                                     ('ms220',
                                                                      43),
                                                                     ('ms225',
                                                                      44),
                                                                     ('ms230',
                                                                      45),
                                                                     ('ms235',
                                                                      46),
                                                                     ('ms240',
                                                                      47),
                                                                     ('ms245',
                                                                      48),
                                                                     ('ms250',
                                                                      49),
                                                                     ('ms300',
                                                                      50),
                                                                     ('ms350',
                                                                      51),
                                                                     ('ms400',
                                                                      52),
                                                                     ('ms450',
                                                                      53),
                                                                     ('ms500',
                                                                      54),
                                                                     ('ms800-v1310',
                                                                      55),
                                                                     ('ms1000-v1310',
                                                                      56),
                                                                     ('ms2000-v1310',
                                                                      57),
                                                                     ('ms4000-v1310',
                                                                      58),
                                                                     ('spare5',
                                                                      59),
                                                                     ('spare4',
                                                                      60),
                                                                     ('spare3',
                                                                      61),
                                                                     ('spare2',
                                                                      62),
                                                                     ('spare1',
                                                                      63)]},
                                     'T-Reordering': {'type': 'ENUMERATED',
                                                      'values': [('ms0', 0),
                                                                 ('ms5', 1),
                                                                 ('ms10', 2),
                                                                 ('ms15', 3),
                                                                 ('ms20', 4),
                                                                 ('ms25', 5),
                                                                 ('ms30', 6),
                                                                 ('ms35', 7),
                                                                 ('ms40', 8),
                                                                 ('ms45', 9),
                                                                 ('ms50', 10),
                                                                 ('ms55', 11),
                                                                 ('ms60', 12),
                                                                 ('ms65', 13),
                                                                 ('ms70', 14),
                                                                 ('ms75', 15),
                                                                 ('ms80', 16),
                                                                 ('ms85', 17),
                                                                 ('ms90', 18),
                                                                 ('ms95', 19),
                                                                 ('ms100', 20),
                                                                 ('ms110', 21),
                                                                 ('ms120', 22),
                                                                 ('ms130', 23),
                                                                 ('ms140', 24),
                                                                 ('ms150', 25),
                                                                 ('ms160', 26),
                                                                 ('ms170', 27),
                                                                 ('ms180', 28),
                                                                 ('ms190', 29),
                                                                 ('ms200', 30),
                                                                 ('ms1600-v1310',
                                                                  31)]},
                                     'T-Reselection': {'restricted-to': [(0,
                                                                          7)],
                                                       'type': 'INTEGER'},
                                     'T-ReselectionEUTRA-CE-r13': {'restricted-to': [(0,
                                                                                      15)],
                                                                   'type': 'INTEGER'},
                                     'T-StatusProhibit': {'type': 'ENUMERATED',
                                                          'values': [('ms0', 0),
                                                                     ('ms5', 1),
                                                                     ('ms10',
                                                                      2),
                                                                     ('ms15',
                                                                      3),
                                                                     ('ms20',
                                                                      4),
                                                                     ('ms25',
                                                                      5),
                                                                     ('ms30',
                                                                      6),
                                                                     ('ms35',
                                                                      7),
                                                                     ('ms40',
                                                                      8),
                                                                     ('ms45',
                                                                      9),
                                                                     ('ms50',
                                                                      10),
                                                                     ('ms55',
                                                                      11),
                                                                     ('ms60',
                                                                      12),
                                                                     ('ms65',
                                                                      13),
                                                                     ('ms70',
                                                                      14),
                                                                     ('ms75',
                                                                      15),
                                                                     ('ms80',
                                                                      16),
                                                                     ('ms85',
                                                                      17),
                                                                     ('ms90',
                                                                      18),
                                                                     ('ms95',
                                                                      19),
                                                                     ('ms100',
                                                                      20),
                                                                     ('ms105',
                                                                      21),
                                                                     ('ms110',
                                                                      22),
                                                                     ('ms115',
                                                                      23),
                                                                     ('ms120',
                                                                      24),
                                                                     ('ms125',
                                                                      25),
                                                                     ('ms130',
                                                                      26),
                                                                     ('ms135',
                                                                      27),
                                                                     ('ms140',
                                                                      28),
                                                                     ('ms145',
                                                                      29),
                                                                     ('ms150',
                                                                      30),
                                                                     ('ms155',
                                                                      31),
                                                                     ('ms160',
                                                                      32),
                                                                     ('ms165',
                                                                      33),
                                                                     ('ms170',
                                                                      34),
                                                                     ('ms175',
                                                                      35),
                                                                     ('ms180',
                                                                      36),
                                                                     ('ms185',
                                                                      37),
                                                                     ('ms190',
                                                                      38),
                                                                     ('ms195',
                                                                      39),
                                                                     ('ms200',
                                                                      40),
                                                                     ('ms205',
                                                                      41),
                                                                     ('ms210',
                                                                      42),
                                                                     ('ms215',
                                                                      43),
                                                                     ('ms220',
                                                                      44),
                                                                     ('ms225',
                                                                      45),
                                                                     ('ms230',
                                                                      46),
                                                                     ('ms235',
                                                                      47),
                                                                     ('ms240',
                                                                      48),
                                                                     ('ms245',
                                                                      49),
                                                                     ('ms250',
                                                                      50),
                                                                     ('ms300',
                                                                      51),
                                                                     ('ms350',
                                                                      52),
                                                                     ('ms400',
                                                                      53),
                                                                     ('ms450',
                                                                      54),
                                                                     ('ms500',
                                                                      55),
                                                                     ('ms800-v1310',
                                                                      56),
                                                                     ('ms1000-v1310',
                                                                      57),
                                                                     ('ms1200-v1310',
                                                                      58),
                                                                     ('ms1600-v1310',
                                                                      59),
                                                                     ('ms2000-v1310',
                                                                      60),
                                                                     ('ms2400-v1310',
                                                                      61),
                                                                     ('spare2',
                                                                      62),
                                                                     ('spare1',
                                                                      63)]},
                                     'TDD-Config': {'members': [{'name': 'subframeAssignment',
                                                                 'type': 'ENUMERATED',
                                                                 'values': [('sa0',
                                                                             0),
                                                                            ('sa1',
                                                                             1),
                                                                            ('sa2',
                                                                             2),
                                                                            ('sa3',
                                                                             3),
                                                                            ('sa4',
                                                                             4),
                                                                            ('sa5',
                                                                             5),
                                                                            ('sa6',
                                                                             6)]},
                                                                {'name': 'specialSubframePatterns',
                                                                 'type': 'ENUMERATED',
                                                                 'values': [('ssp0',
                                                                             0),
                                                                            ('ssp1',
                                                                             1),
                                                                            ('ssp2',
                                                                             2),
                                                                            ('ssp3',
                                                                             3),
                                                                            ('ssp4',
                                                                             4),
                                                                            ('ssp5',
                                                                             5),
                                                                            ('ssp6',
                                                                             6),
                                                                            ('ssp7',
                                                                             7),
                                                                            ('ssp8',
                                                                             8)]}],
                                                    'type': 'SEQUENCE'},
                                     'TDD-Config-v1130': {'members': [{'name': 'specialSubframePatterns-v1130',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('ssp7',
                                                                                   0),
                                                                                  ('ssp9',
                                                                                   1)]}],
                                                          'type': 'SEQUENCE'},
                                     'TDD-Config-v1430': {'members': [{'name': 'specialSubframePatterns-v1430',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('ssp10',
                                                                                   0)]}],
                                                          'type': 'SEQUENCE'},
                                     'TDD-ConfigSL-r12': {'members': [{'name': 'subframeAssignmentSL-r12',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('none',
                                                                                   0),
                                                                                  ('sa0',
                                                                                   1),
                                                                                  ('sa1',
                                                                                   2),
                                                                                  ('sa2',
                                                                                   3),
                                                                                  ('sa3',
                                                                                   4),
                                                                                  ('sa4',
                                                                                   5),
                                                                                  ('sa5',
                                                                                   6),
                                                                                  ('sa6',
                                                                                   7)]}],
                                                          'type': 'SEQUENCE'},
                                     'TDD-PUSCH-UpPTS-r14': {'members': [{'name': 'release',
                                                                          'type': 'NULL'},
                                                                         {'members': [{'name': 'symPUSCH-UpPTS-r14',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('sym1',
                                                                                                   0),
                                                                                                  ('sym2',
                                                                                                   1),
                                                                                                  ('sym3',
                                                                                                   2),
                                                                                                  ('sym4',
                                                                                                   3),
                                                                                                  ('sym5',
                                                                                                   4),
                                                                                                  ('sym6',
                                                                                                   5)]},
                                                                                      {'name': 'dmrs-LessUpPTS-Config-r14',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('true',
                                                                                                   0)]}],
                                                                          'name': 'setup',
                                                                          'type': 'SEQUENCE'}],
                                                             'type': 'CHOICE'},
                                     'TDM-AssistanceInfo-r11': {'members': [{'members': [{'name': 'drx-CycleLength-r11',
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('sf40',
                                                                                                      0),
                                                                                                     ('sf64',
                                                                                                      1),
                                                                                                     ('sf80',
                                                                                                      2),
                                                                                                     ('sf128',
                                                                                                      3),
                                                                                                     ('sf160',
                                                                                                      4),
                                                                                                     ('sf256',
                                                                                                      5),
                                                                                                     ('spare2',
                                                                                                      6),
                                                                                                     ('spare1',
                                                                                                      7)]},
                                                                                         {'name': 'drx-Offset-r11',
                                                                                          'optional': True,
                                                                                          'restricted-to': [(0,
                                                                                                             255)],
                                                                                          'type': 'INTEGER'},
                                                                                         {'name': 'drx-ActiveTime-r11',
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('sf20',
                                                                                                      0),
                                                                                                     ('sf30',
                                                                                                      1),
                                                                                                     ('sf40',
                                                                                                      2),
                                                                                                     ('sf60',
                                                                                                      3),
                                                                                                     ('sf80',
                                                                                                      4),
                                                                                                     ('sf100',
                                                                                                      5),
                                                                                                     ('spare2',
                                                                                                      6),
                                                                                                     ('spare1',
                                                                                                      7)]}],
                                                                             'name': 'drx-AssistanceInfo-r11',
                                                                             'type': 'SEQUENCE'},
                                                                            {'name': 'idc-SubframePatternList-r11',
                                                                             'type': 'IDC-SubframePatternList-r11'},
                                                                            None],
                                                                'type': 'CHOICE'},
                                     'TMGI-r9': {'members': [{'members': [{'name': 'plmn-Index-r9',
                                                                           'restricted-to': [(1,
                                                                                              'maxPLMN-r11')],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'explicitValue-r9',
                                                                           'type': 'PLMN-Identity'}],
                                                              'name': 'plmn-Id-r9',
                                                              'type': 'CHOICE'},
                                                             {'name': 'serviceId-r9',
                                                              'size': [3],
                                                              'type': 'OCTET '
                                                                      'STRING'}],
                                                 'type': 'SEQUENCE'},
                                     'TPC-Index': {'members': [{'name': 'indexOfFormat3',
                                                                'restricted-to': [(1,
                                                                                   15)],
                                                                'type': 'INTEGER'},
                                                               {'name': 'indexOfFormat3A',
                                                                'restricted-to': [(1,
                                                                                   31)],
                                                                'type': 'INTEGER'}],
                                                   'type': 'CHOICE'},
                                     'TPC-PDCCH-Config': {'members': [{'name': 'release',
                                                                       'type': 'NULL'},
                                                                      {'members': [{'name': 'tpc-RNTI',
                                                                                    'size': [16],
                                                                                    'type': 'BIT '
                                                                                            'STRING'},
                                                                                   {'name': 'tpc-Index',
                                                                                    'type': 'TPC-Index'}],
                                                                       'name': 'setup',
                                                                       'type': 'SEQUENCE'}],
                                                          'type': 'CHOICE'},
                                     'TPC-PDCCH-ConfigSCell-r13': {'members': [{'name': 'release',
                                                                                'type': 'NULL'},
                                                                               {'members': [{'name': 'tpc-Index-PUCCH-SCell-r13',
                                                                                             'type': 'TPC-Index'}],
                                                                                'name': 'setup',
                                                                                'type': 'SEQUENCE'}],
                                                                   'type': 'CHOICE'},
                                     'TargetMBSFN-Area-r12': {'members': [{'name': 'mbsfn-AreaId-r12',
                                                                           'optional': True,
                                                                           'type': 'MBSFN-AreaId-r12'},
                                                                          {'name': 'carrierFreq-r12',
                                                                           'type': 'ARFCN-ValueEUTRA-r9'},
                                                                          None],
                                                              'type': 'SEQUENCE'},
                                     'TargetMBSFN-AreaList-r12': {'element': {'type': 'TargetMBSFN-Area-r12'},
                                                                  'size': [(0,
                                                                            'maxMBSFN-Area')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'ThresholdCDMA2000': {'restricted-to': [(0,
                                                                              63)],
                                                           'type': 'INTEGER'},
                                     'ThresholdEUTRA': {'members': [{'name': 'threshold-RSRP',
                                                                     'type': 'RSRP-Range'},
                                                                    {'name': 'threshold-RSRQ',
                                                                     'type': 'RSRQ-Range'}],
                                                        'type': 'CHOICE'},
                                     'ThresholdEUTRA-v1250': {'type': 'CSI-RSRP-Range-r12'},
                                     'ThresholdGERAN': {'restricted-to': [(0,
                                                                           63)],
                                                        'type': 'INTEGER'},
                                     'ThresholdUTRA': {'members': [{'name': 'utra-RSCP',
                                                                    'restricted-to': [(-5,
                                                                                       91)],
                                                                    'type': 'INTEGER'},
                                                                   {'name': 'utra-EcN0',
                                                                    'restricted-to': [(0,
                                                                                       49)],
                                                                    'type': 'INTEGER'}],
                                                       'type': 'CHOICE'},
                                     'TimeAlignmentTimer': {'type': 'ENUMERATED',
                                                            'values': [('sf500',
                                                                        0),
                                                                       ('sf750',
                                                                        1),
                                                                       ('sf1280',
                                                                        2),
                                                                       ('sf1920',
                                                                        3),
                                                                       ('sf2560',
                                                                        4),
                                                                       ('sf5120',
                                                                        5),
                                                                       ('sf10240',
                                                                        6),
                                                                       ('infinity',
                                                                        7)]},
                                     'TimeSinceFailure-r11': {'restricted-to': [(0,
                                                                                 172800)],
                                                              'type': 'INTEGER'},
                                     'TimeToTrigger': {'type': 'ENUMERATED',
                                                       'values': [('ms0', 0),
                                                                  ('ms40', 1),
                                                                  ('ms64', 2),
                                                                  ('ms80', 3),
                                                                  ('ms100', 4),
                                                                  ('ms128', 5),
                                                                  ('ms160', 6),
                                                                  ('ms256', 7),
                                                                  ('ms320', 8),
                                                                  ('ms480', 9),
                                                                  ('ms512', 10),
                                                                  ('ms640', 11),
                                                                  ('ms1024',
                                                                   12),
                                                                  ('ms1280',
                                                                   13),
                                                                  ('ms2560',
                                                                   14),
                                                                  ('ms5120',
                                                                   15)]},
                                     'TraceReference-r10': {'members': [{'name': 'plmn-Identity-r10',
                                                                         'type': 'PLMN-Identity'},
                                                                        {'name': 'traceId-r10',
                                                                         'size': [3],
                                                                         'type': 'OCTET '
                                                                                 'STRING'}],
                                                            'type': 'SEQUENCE'},
                                     'TrackingAreaCode': {'size': [16],
                                                          'type': 'BIT STRING'},
                                     'TrackingAreaCodeList-r10': {'element': {'type': 'TrackingAreaCode'},
                                                                  'size': [(1,
                                                                            8)],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'TrackingAreaCodeList-v1130': {'members': [{'element': {'type': 'PLMN-Identity'},
                                                                                 'name': 'plmn-Identity-perTAC-List-r11',
                                                                                 'size': [(1,
                                                                                           8)],
                                                                                 'type': 'SEQUENCE '
                                                                                         'OF'}],
                                                                    'type': 'SEQUENCE'},
                                     'TrafficPatternInfo-r14': {'members': [{'name': 'trafficPeriodicity-r14',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('sf20',
                                                                                         0),
                                                                                        ('sf50',
                                                                                         1),
                                                                                        ('sf100',
                                                                                         2),
                                                                                        ('sf200',
                                                                                         3),
                                                                                        ('sf300',
                                                                                         4),
                                                                                        ('sf400',
                                                                                         5),
                                                                                        ('sf500',
                                                                                         6),
                                                                                        ('sf600',
                                                                                         7),
                                                                                        ('sf700',
                                                                                         8),
                                                                                        ('sf800',
                                                                                         9),
                                                                                        ('sf900',
                                                                                         10),
                                                                                        ('sf1000',
                                                                                         11)]},
                                                                            {'name': 'timingOffset-r14',
                                                                             'restricted-to': [(0,
                                                                                                10239)],
                                                                             'type': 'INTEGER'},
                                                                            {'name': 'priorityInfoSL-r14',
                                                                             'optional': True,
                                                                             'type': 'SL-Priority-r13'},
                                                                            {'name': 'logicalChannelIdentityUL-r14',
                                                                             'optional': True,
                                                                             'restricted-to': [(3,
                                                                                                10)],
                                                                             'type': 'INTEGER'},
                                                                            {'name': 'messageSize-r14',
                                                                             'size': [6],
                                                                             'type': 'BIT '
                                                                                     'STRING'}],
                                                                'type': 'SEQUENCE'},
                                     'TrafficPatternInfoList-r14': {'element': {'type': 'TrafficPatternInfo-r14'},
                                                                    'size': [(1,
                                                                              'maxTrafficPattern-r14')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'TunnelConfigLWIP-r13': {'members': [{'name': 'ip-Address-r13',
                                                                           'type': 'IP-Address-r13'},
                                                                          {'name': 'ike-Identity-r13',
                                                                           'type': 'IKE-Identity-r13'},
                                                                          None,
                                                                          [{'name': 'lwip-Counter-r13',
                                                                            'optional': True,
                                                                            'restricted-to': [(0,
                                                                                               65535)],
                                                                            'type': 'INTEGER'}]],
                                                              'type': 'SEQUENCE'},
                                     'Tx-ConfigIndex-r14': {'restricted-to': [(0,
                                                                               'maxSL-V2X-TxConfig-1-r14')],
                                                            'type': 'INTEGER'},
                                     'Tx-ResourcePoolMeasList-r14': {'element': {'type': 'SL-V2X-TxPoolReportIdentity-r14'},
                                                                     'size': [(1,
                                                                               'maxSL-PoolToMeasure-r14')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'UDT-Restricting-r13': {'members': [{'name': 'udt-Restricting-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('true',
                                                                                      0)]},
                                                                         {'name': 'udt-RestrictingTime-r13',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('s4',
                                                                                      0),
                                                                                     ('s8',
                                                                                      1),
                                                                                     ('s16',
                                                                                      2),
                                                                                     ('s32',
                                                                                      3),
                                                                                     ('s64',
                                                                                      4),
                                                                                     ('s128',
                                                                                      5),
                                                                                     ('s256',
                                                                                      6),
                                                                                     ('s512',
                                                                                      7)]}],
                                                             'type': 'SEQUENCE'},
                                     'UDT-RestrictingPerPLMN-List-r13': {'element': {'type': 'UDT-RestrictingPerPLMN-r13'},
                                                                         'size': [(1,
                                                                                   'maxPLMN-r11')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'UDT-RestrictingPerPLMN-r13': {'members': [{'name': 'plmn-IdentityIndex-r13',
                                                                                 'restricted-to': [(1,
                                                                                                    'maxPLMN-r11')],
                                                                                 'type': 'INTEGER'},
                                                                                {'name': 'udt-Restricting-r13',
                                                                                 'optional': True,
                                                                                 'type': 'UDT-Restricting-r13'}],
                                                                    'type': 'SEQUENCE'},
                                     'UE-BasedNetwPerfMeasParameters-r10': {'members': [{'name': 'loggedMeasurementsIdle-r10',
                                                                                         'optional': True,
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('supported',
                                                                                                     0)]},
                                                                                        {'name': 'standaloneGNSS-Location-r10',
                                                                                         'optional': True,
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('supported',
                                                                                                     0)]}],
                                                                            'type': 'SEQUENCE'},
                                     'UE-BasedNetwPerfMeasParameters-v1250': {'members': [{'name': 'loggedMBSFNMeasurements-r12',
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('supported',
                                                                                                       0)]}],
                                                                              'type': 'SEQUENCE'},
                                     'UE-BasedNetwPerfMeasParameters-v1430': {'members': [{'name': 'locationReport-r14',
                                                                                           'optional': True,
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('supported',
                                                                                                       0)]}],
                                                                              'type': 'SEQUENCE'},
                                     'UE-CapabilityRAT-Container': {'members': [{'name': 'rat-Type',
                                                                                 'type': 'RAT-Type'},
                                                                                {'name': 'ueCapabilityRAT-Container',
                                                                                 'type': 'OCTET '
                                                                                         'STRING'}],
                                                                    'type': 'SEQUENCE'},
                                     'UE-CapabilityRAT-ContainerList': {'element': {'type': 'UE-CapabilityRAT-Container'},
                                                                        'size': [(0,
                                                                                  'maxRAT-Capabilities')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'UE-CapabilityRequest': {'element': {'type': 'RAT-Type'},
                                                              'size': [(1,
                                                                        'maxRAT-Capabilities')],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'UE-EUTRA-Capability': {'members': [{'name': 'accessStratumRelease',
                                                                          'type': 'AccessStratumRelease'},
                                                                         {'name': 'ue-Category',
                                                                          'restricted-to': [(1,
                                                                                             5)],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'pdcp-Parameters',
                                                                          'type': 'PDCP-Parameters'},
                                                                         {'name': 'phyLayerParameters',
                                                                          'type': 'PhyLayerParameters'},
                                                                         {'name': 'rf-Parameters',
                                                                          'type': 'RF-Parameters'},
                                                                         {'name': 'measParameters',
                                                                          'type': 'MeasParameters'},
                                                                         {'name': 'featureGroupIndicators',
                                                                          'optional': True,
                                                                          'size': [32],
                                                                          'type': 'BIT '
                                                                                  'STRING'},
                                                                         {'members': [{'name': 'utraFDD',
                                                                                       'optional': True,
                                                                                       'type': 'IRAT-ParametersUTRA-FDD'},
                                                                                      {'name': 'utraTDD128',
                                                                                       'optional': True,
                                                                                       'type': 'IRAT-ParametersUTRA-TDD128'},
                                                                                      {'name': 'utraTDD384',
                                                                                       'optional': True,
                                                                                       'type': 'IRAT-ParametersUTRA-TDD384'},
                                                                                      {'name': 'utraTDD768',
                                                                                       'optional': True,
                                                                                       'type': 'IRAT-ParametersUTRA-TDD768'},
                                                                                      {'name': 'geran',
                                                                                       'optional': True,
                                                                                       'type': 'IRAT-ParametersGERAN'},
                                                                                      {'name': 'cdma2000-HRPD',
                                                                                       'optional': True,
                                                                                       'type': 'IRAT-ParametersCDMA2000-HRPD'},
                                                                                      {'name': 'cdma2000-1xRTT',
                                                                                       'optional': True,
                                                                                       'type': 'IRAT-ParametersCDMA2000-1XRTT'}],
                                                                          'name': 'interRAT-Parameters',
                                                                          'type': 'SEQUENCE'},
                                                                         {'name': 'nonCriticalExtension',
                                                                          'optional': True,
                                                                          'type': 'UE-EUTRA-Capability-v920-IEs'}],
                                                             'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1020-IEs': {'members': [{'name': 'ue-Category-v1020',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(6,
                                                                                                       8)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'phyLayerParameters-v1020',
                                                                                    'optional': True,
                                                                                    'type': 'PhyLayerParameters-v1020'},
                                                                                   {'name': 'rf-Parameters-v1020',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v1020'},
                                                                                   {'name': 'measParameters-v1020',
                                                                                    'optional': True,
                                                                                    'type': 'MeasParameters-v1020'},
                                                                                   {'name': 'featureGroupIndRel10-r10',
                                                                                    'optional': True,
                                                                                    'size': [32],
                                                                                    'type': 'BIT '
                                                                                            'STRING'},
                                                                                   {'name': 'interRAT-ParametersCDMA2000-v1020',
                                                                                    'optional': True,
                                                                                    'type': 'IRAT-ParametersCDMA2000-1XRTT-v1020'},
                                                                                   {'name': 'ue-BasedNetwPerfMeasParameters-r10',
                                                                                    'optional': True,
                                                                                    'type': 'UE-BasedNetwPerfMeasParameters-r10'},
                                                                                   {'name': 'interRAT-ParametersUTRA-TDD-v1020',
                                                                                    'optional': True,
                                                                                    'type': 'IRAT-ParametersUTRA-TDD-v1020'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1060-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1060-IEs': {'members': [{'name': 'fdd-Add-UE-EUTRA-Capabilities-v1060',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1060'},
                                                                                   {'name': 'tdd-Add-UE-EUTRA-Capabilities-v1060',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1060'},
                                                                                   {'name': 'rf-Parameters-v1060',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v1060'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1090-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1090-IEs': {'members': [{'name': 'rf-Parameters-v1090',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v1090'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1130-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v10c0-IEs': {'members': [{'name': 'otdoa-PositioningCapabilities-r10',
                                                                                    'optional': True,
                                                                                    'type': 'OTDOA-PositioningCapabilities-r10'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v10f0-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v10f0-IEs': {'members': [{'name': 'rf-Parameters-v10f0',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v10f0'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v10i0-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v10i0-IEs': {'members': [{'name': 'rf-Parameters-v10i0',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v10i0'},
                                                                                   {'name': 'lateNonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v11d0-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v10j0-IEs': {'members': [{'name': 'rf-Parameters-v10j0',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v10j0'},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1130-IEs': {'members': [{'name': 'pdcp-Parameters-v1130',
                                                                                    'type': 'PDCP-Parameters-v1130'},
                                                                                   {'name': 'phyLayerParameters-v1130',
                                                                                    'optional': True,
                                                                                    'type': 'PhyLayerParameters-v1130'},
                                                                                   {'name': 'rf-Parameters-v1130',
                                                                                    'type': 'RF-Parameters-v1130'},
                                                                                   {'name': 'measParameters-v1130',
                                                                                    'type': 'MeasParameters-v1130'},
                                                                                   {'name': 'interRAT-ParametersCDMA2000-v1130',
                                                                                    'type': 'IRAT-ParametersCDMA2000-v1130'},
                                                                                   {'name': 'otherParameters-r11',
                                                                                    'type': 'Other-Parameters-r11'},
                                                                                   {'name': 'fdd-Add-UE-EUTRA-Capabilities-v1130',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1130'},
                                                                                   {'name': 'tdd-Add-UE-EUTRA-Capabilities-v1130',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1130'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1170-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1170-IEs': {'members': [{'name': 'phyLayerParameters-v1170',
                                                                                    'optional': True,
                                                                                    'type': 'PhyLayerParameters-v1170'},
                                                                                   {'name': 'ue-Category-v1170',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(9,
                                                                                                       10)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1180-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1180-IEs': {'members': [{'name': 'rf-Parameters-v1180',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v1180'},
                                                                                   {'name': 'mbms-Parameters-r11',
                                                                                    'optional': True,
                                                                                    'type': 'MBMS-Parameters-r11'},
                                                                                   {'name': 'fdd-Add-UE-EUTRA-Capabilities-v1180',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1180'},
                                                                                   {'name': 'tdd-Add-UE-EUTRA-Capabilities-v1180',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1180'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v11a0-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v11a0-IEs': {'members': [{'name': 'ue-Category-v11a0',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(11,
                                                                                                       12)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'measParameters-v11a0',
                                                                                    'optional': True,
                                                                                    'type': 'MeasParameters-v11a0'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1250-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v11d0-IEs': {'members': [{'name': 'rf-Parameters-v11d0',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v11d0'},
                                                                                   {'name': 'otherParameters-v11d0',
                                                                                    'optional': True,
                                                                                    'type': 'Other-Parameters-v11d0'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v11x0-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v11x0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v12b0-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1250-IEs': {'members': [{'name': 'phyLayerParameters-v1250',
                                                                                    'optional': True,
                                                                                    'type': 'PhyLayerParameters-v1250'},
                                                                                   {'name': 'rf-Parameters-v1250',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v1250'},
                                                                                   {'name': 'rlc-Parameters-r12',
                                                                                    'optional': True,
                                                                                    'type': 'RLC-Parameters-r12'},
                                                                                   {'name': 'ue-BasedNetwPerfMeasParameters-v1250',
                                                                                    'optional': True,
                                                                                    'type': 'UE-BasedNetwPerfMeasParameters-v1250'},
                                                                                   {'name': 'ue-CategoryDL-r12',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(0,
                                                                                                       14)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'ue-CategoryUL-r12',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(0,
                                                                                                       13)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'wlan-IW-Parameters-r12',
                                                                                    'optional': True,
                                                                                    'type': 'WLAN-IW-Parameters-r12'},
                                                                                   {'name': 'measParameters-v1250',
                                                                                    'optional': True,
                                                                                    'type': 'MeasParameters-v1250'},
                                                                                   {'name': 'dc-Parameters-r12',
                                                                                    'optional': True,
                                                                                    'type': 'DC-Parameters-r12'},
                                                                                   {'name': 'mbms-Parameters-v1250',
                                                                                    'optional': True,
                                                                                    'type': 'MBMS-Parameters-v1250'},
                                                                                   {'name': 'mac-Parameters-r12',
                                                                                    'optional': True,
                                                                                    'type': 'MAC-Parameters-r12'},
                                                                                   {'name': 'fdd-Add-UE-EUTRA-Capabilities-v1250',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1250'},
                                                                                   {'name': 'tdd-Add-UE-EUTRA-Capabilities-v1250',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1250'},
                                                                                   {'name': 'sl-Parameters-r12',
                                                                                    'optional': True,
                                                                                    'type': 'SL-Parameters-r12'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1260-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1260-IEs': {'members': [{'name': 'ue-CategoryDL-v1260',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(15,
                                                                                                       16)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1270-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1270-IEs': {'members': [{'name': 'rf-Parameters-v1270',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v1270'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1280-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1280-IEs': {'members': [{'name': 'phyLayerParameters-v1280',
                                                                                    'optional': True,
                                                                                    'type': 'PhyLayerParameters-v1280'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1310-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v12b0-IEs': {'members': [{'name': 'rf-Parameters-v12b0',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v12b0'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v12x0-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v12x0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1370-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1310-IEs': {'members': [{'name': 'ue-CategoryDL-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('n17',
                                                                                                0),
                                                                                               ('m1',
                                                                                                1)]},
                                                                                   {'name': 'ue-CategoryUL-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('n14',
                                                                                                0),
                                                                                               ('m1',
                                                                                                1)]},
                                                                                   {'name': 'pdcp-Parameters-v1310',
                                                                                    'type': 'PDCP-Parameters-v1310'},
                                                                                   {'name': 'rlc-Parameters-v1310',
                                                                                    'type': 'RLC-Parameters-v1310'},
                                                                                   {'name': 'mac-Parameters-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'MAC-Parameters-v1310'},
                                                                                   {'name': 'phyLayerParameters-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'PhyLayerParameters-v1310'},
                                                                                   {'name': 'rf-Parameters-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v1310'},
                                                                                   {'name': 'measParameters-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'MeasParameters-v1310'},
                                                                                   {'name': 'dc-Parameters-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'DC-Parameters-v1310'},
                                                                                   {'name': 'sl-Parameters-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'SL-Parameters-v1310'},
                                                                                   {'name': 'scptm-Parameters-r13',
                                                                                    'optional': True,
                                                                                    'type': 'SCPTM-Parameters-r13'},
                                                                                   {'name': 'ce-Parameters-r13',
                                                                                    'optional': True,
                                                                                    'type': 'CE-Parameters-r13'},
                                                                                   {'name': 'interRAT-ParametersWLAN-r13',
                                                                                    'type': 'IRAT-ParametersWLAN-r13'},
                                                                                   {'name': 'laa-Parameters-r13',
                                                                                    'optional': True,
                                                                                    'type': 'LAA-Parameters-r13'},
                                                                                   {'name': 'lwa-Parameters-r13',
                                                                                    'optional': True,
                                                                                    'type': 'LWA-Parameters-r13'},
                                                                                   {'name': 'wlan-IW-Parameters-v1310',
                                                                                    'type': 'WLAN-IW-Parameters-v1310'},
                                                                                   {'name': 'lwip-Parameters-r13',
                                                                                    'type': 'LWIP-Parameters-r13'},
                                                                                   {'name': 'fdd-Add-UE-EUTRA-Capabilities-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1310'},
                                                                                   {'name': 'tdd-Add-UE-EUTRA-Capabilities-v1310',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1310'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1320-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1320-IEs': {'members': [{'name': 'ce-Parameters-v1320',
                                                                                    'optional': True,
                                                                                    'type': 'CE-Parameters-v1320'},
                                                                                   {'name': 'phyLayerParameters-v1320',
                                                                                    'optional': True,
                                                                                    'type': 'PhyLayerParameters-v1320'},
                                                                                   {'name': 'rf-Parameters-v1320',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v1320'},
                                                                                   {'name': 'fdd-Add-UE-EUTRA-Capabilities-v1320',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1320'},
                                                                                   {'name': 'tdd-Add-UE-EUTRA-Capabilities-v1320',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1320'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1330-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1330-IEs': {'members': [{'name': 'ue-CategoryDL-v1330',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(18,
                                                                                                       19)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'phyLayerParameters-v1330',
                                                                                    'optional': True,
                                                                                    'type': 'PhyLayerParameters-v1330'},
                                                                                   {'name': 'ue-CE-NeedULGaps-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1340-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1340-IEs': {'members': [{'name': 'ue-CategoryUL-v1340',
                                                                                    'optional': True,
                                                                                    'restricted-to': [15],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1350-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1350-IEs': {'members': [{'name': 'ue-CategoryDL-v1350',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('oneBis',
                                                                                                0)]},
                                                                                   {'name': 'ue-CategoryUL-v1350',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('oneBis',
                                                                                                0)]},
                                                                                   {'name': 'ce-Parameters-v1350',
                                                                                    'type': 'CE-Parameters-v1350'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1360-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1360-IEs': {'members': [{'name': 'other-Parameters-v1360',
                                                                                    'optional': True,
                                                                                    'type': 'Other-Parameters-v1360'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1430-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1370-IEs': {'members': [{'name': 'ce-Parameters-v1370',
                                                                                    'optional': True,
                                                                                    'type': 'CE-Parameters-v1370'},
                                                                                   {'name': 'fdd-Add-UE-EUTRA-Capabilities-v1370',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1370'},
                                                                                   {'name': 'tdd-Add-UE-EUTRA-Capabilities-v1370',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1370'},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1430-IEs': {'members': [{'name': 'phyLayerParameters-v1430',
                                                                                    'type': 'PhyLayerParameters-v1430'},
                                                                                   {'name': 'ue-CategoryDL-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('m2',
                                                                                                0)]},
                                                                                   {'name': 'ue-CategoryUL-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('n16',
                                                                                                0),
                                                                                               ('n17',
                                                                                                1),
                                                                                               ('n18',
                                                                                                2),
                                                                                               ('n19',
                                                                                                3),
                                                                                               ('n20',
                                                                                                4),
                                                                                               ('m2',
                                                                                                5)]},
                                                                                   {'name': 'ue-CategoryUL-v1430b',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('n21',
                                                                                                0)]},
                                                                                   {'name': 'mac-Parameters-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'MAC-Parameters-v1430'},
                                                                                   {'name': 'measParameters-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'MeasParameters-v1430'},
                                                                                   {'name': 'pdcp-Parameters-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'PDCP-Parameters-v1430'},
                                                                                   {'name': 'rlc-Parameters-v1430',
                                                                                    'type': 'RLC-Parameters-v1430'},
                                                                                   {'name': 'rf-Parameters-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'RF-Parameters-v1430'},
                                                                                   {'name': 'laa-Parameters-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'LAA-Parameters-v1430'},
                                                                                   {'name': 'lwa-Parameters-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'LWA-Parameters-v1430'},
                                                                                   {'name': 'lwip-Parameters-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'LWIP-Parameters-v1430'},
                                                                                   {'name': 'otherParameters-v1430',
                                                                                    'type': 'Other-Parameters-v1430'},
                                                                                   {'name': 'mmtel-Parameters-r14',
                                                                                    'optional': True,
                                                                                    'type': 'MMTEL-Parameters-r14'},
                                                                                   {'name': 'mobilityParameters-r14',
                                                                                    'optional': True,
                                                                                    'type': 'MobilityParameters-r14'},
                                                                                   {'name': 'ce-Parameters-v1430',
                                                                                    'type': 'CE-Parameters-v1430'},
                                                                                   {'name': 'fdd-Add-UE-EUTRA-Capabilities-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1430'},
                                                                                   {'name': 'tdd-Add-UE-EUTRA-Capabilities-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-CapabilityAddXDD-Mode-v1430'},
                                                                                   {'name': 'mbms-Parameters-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'MBMS-Parameters-v1430'},
                                                                                   {'name': 'sl-Parameters-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'SL-Parameters-v1430'},
                                                                                   {'name': 'ue-BasedNetwPerfMeasParameters-v1430',
                                                                                    'optional': True,
                                                                                    'type': 'UE-BasedNetwPerfMeasParameters-v1430'},
                                                                                   {'name': 'highSpeedEnhParameters-r14',
                                                                                    'optional': True,
                                                                                    'type': 'HighSpeedEnhParameters-r14'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UE-EUTRA-Capability-v1440-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v1440-IEs': {'members': [{'name': 'lwa-Parameters-v1440',
                                                                                    'type': 'LWA-Parameters-v1440'},
                                                                                   {'name': 'mac-Parameters-v1440',
                                                                                    'type': 'MAC-Parameters-v1440'},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v920-IEs': {'members': [{'name': 'phyLayerParameters-v920',
                                                                                   'type': 'PhyLayerParameters-v920'},
                                                                                  {'name': 'interRAT-ParametersGERAN-v920',
                                                                                   'type': 'IRAT-ParametersGERAN-v920'},
                                                                                  {'name': 'interRAT-ParametersUTRA-v920',
                                                                                   'optional': True,
                                                                                   'type': 'IRAT-ParametersUTRA-v920'},
                                                                                  {'name': 'interRAT-ParametersCDMA2000-v920',
                                                                                   'optional': True,
                                                                                   'type': 'IRAT-ParametersCDMA2000-1XRTT-v920'},
                                                                                  {'name': 'deviceType-r9',
                                                                                   'optional': True,
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('noBenFromBatConsumpOpt',
                                                                                               0)]},
                                                                                  {'name': 'csg-ProximityIndicationParameters-r9',
                                                                                   'type': 'CSG-ProximityIndicationParameters-r9'},
                                                                                  {'name': 'neighCellSI-AcquisitionParameters-r9',
                                                                                   'type': 'NeighCellSI-AcquisitionParameters-r9'},
                                                                                  {'name': 'son-Parameters-r9',
                                                                                   'type': 'SON-Parameters-r9'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'UE-EUTRA-Capability-v940-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v940-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'UE-EUTRA-Capability-v1020-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v9a0-IEs': {'members': [{'name': 'featureGroupIndRel9Add-r9',
                                                                                   'optional': True,
                                                                                   'size': [32],
                                                                                   'type': 'BIT '
                                                                                           'STRING'},
                                                                                  {'name': 'fdd-Add-UE-EUTRA-Capabilities-r9',
                                                                                   'optional': True,
                                                                                   'type': 'UE-EUTRA-CapabilityAddXDD-Mode-r9'},
                                                                                  {'name': 'tdd-Add-UE-EUTRA-Capabilities-r9',
                                                                                   'optional': True,
                                                                                   'type': 'UE-EUTRA-CapabilityAddXDD-Mode-r9'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'UE-EUTRA-Capability-v9c0-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v9c0-IEs': {'members': [{'name': 'interRAT-ParametersUTRA-v9c0',
                                                                                   'optional': True,
                                                                                   'type': 'IRAT-ParametersUTRA-v9c0'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'UE-EUTRA-Capability-v9d0-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v9d0-IEs': {'members': [{'name': 'phyLayerParameters-v9d0',
                                                                                   'optional': True,
                                                                                   'type': 'PhyLayerParameters-v9d0'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'UE-EUTRA-Capability-v9e0-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v9e0-IEs': {'members': [{'name': 'rf-Parameters-v9e0',
                                                                                   'optional': True,
                                                                                   'type': 'RF-Parameters-v9e0'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'UE-EUTRA-Capability-v9h0-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'UE-EUTRA-Capability-v9h0-IEs': {'members': [{'name': 'interRAT-ParametersUTRA-v9h0',
                                                                                   'optional': True,
                                                                                   'type': 'IRAT-ParametersUTRA-v9h0'},
                                                                                  {'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'UE-EUTRA-Capability-v10c0-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'UE-EUTRA-CapabilityAddXDD-Mode-r9': {'members': [{'name': 'phyLayerParameters-r9',
                                                                                        'optional': True,
                                                                                        'type': 'PhyLayerParameters'},
                                                                                       {'name': 'featureGroupIndicators-r9',
                                                                                        'optional': True,
                                                                                        'size': [32],
                                                                                        'type': 'BIT '
                                                                                                'STRING'},
                                                                                       {'name': 'featureGroupIndRel9Add-r9',
                                                                                        'optional': True,
                                                                                        'size': [32],
                                                                                        'type': 'BIT '
                                                                                                'STRING'},
                                                                                       {'name': 'interRAT-ParametersGERAN-r9',
                                                                                        'optional': True,
                                                                                        'type': 'IRAT-ParametersGERAN'},
                                                                                       {'name': 'interRAT-ParametersUTRA-r9',
                                                                                        'optional': True,
                                                                                        'type': 'IRAT-ParametersUTRA-v920'},
                                                                                       {'name': 'interRAT-ParametersCDMA2000-r9',
                                                                                        'optional': True,
                                                                                        'type': 'IRAT-ParametersCDMA2000-1XRTT-v920'},
                                                                                       {'name': 'neighCellSI-AcquisitionParameters-r9',
                                                                                        'optional': True,
                                                                                        'type': 'NeighCellSI-AcquisitionParameters-r9'},
                                                                                       None],
                                                                           'type': 'SEQUENCE'},
                                     'UE-EUTRA-CapabilityAddXDD-Mode-v1060': {'members': [{'name': 'phyLayerParameters-v1060',
                                                                                           'optional': True,
                                                                                           'type': 'PhyLayerParameters-v1020'},
                                                                                          {'name': 'featureGroupIndRel10-v1060',
                                                                                           'optional': True,
                                                                                           'size': [32],
                                                                                           'type': 'BIT '
                                                                                                   'STRING'},
                                                                                          {'name': 'interRAT-ParametersCDMA2000-v1060',
                                                                                           'optional': True,
                                                                                           'type': 'IRAT-ParametersCDMA2000-1XRTT-v1020'},
                                                                                          {'name': 'interRAT-ParametersUTRA-TDD-v1060',
                                                                                           'optional': True,
                                                                                           'type': 'IRAT-ParametersUTRA-TDD-v1020'},
                                                                                          None,
                                                                                          [{'name': 'otdoa-PositioningCapabilities-r10',
                                                                                            'optional': True,
                                                                                            'type': 'OTDOA-PositioningCapabilities-r10'}]],
                                                                              'type': 'SEQUENCE'},
                                     'UE-EUTRA-CapabilityAddXDD-Mode-v1130': {'members': [{'name': 'phyLayerParameters-v1130',
                                                                                           'optional': True,
                                                                                           'type': 'PhyLayerParameters-v1130'},
                                                                                          {'name': 'measParameters-v1130',
                                                                                           'optional': True,
                                                                                           'type': 'MeasParameters-v1130'},
                                                                                          {'name': 'otherParameters-r11',
                                                                                           'optional': True,
                                                                                           'type': 'Other-Parameters-r11'},
                                                                                          None],
                                                                              'type': 'SEQUENCE'},
                                     'UE-EUTRA-CapabilityAddXDD-Mode-v1180': {'members': [{'name': 'mbms-Parameters-r11',
                                                                                           'type': 'MBMS-Parameters-r11'}],
                                                                              'type': 'SEQUENCE'},
                                     'UE-EUTRA-CapabilityAddXDD-Mode-v1250': {'members': [{'name': 'phyLayerParameters-v1250',
                                                                                           'optional': True,
                                                                                           'type': 'PhyLayerParameters-v1250'},
                                                                                          {'name': 'measParameters-v1250',
                                                                                           'optional': True,
                                                                                           'type': 'MeasParameters-v1250'}],
                                                                              'type': 'SEQUENCE'},
                                     'UE-EUTRA-CapabilityAddXDD-Mode-v1310': {'members': [{'name': 'phyLayerParameters-v1310',
                                                                                           'optional': True,
                                                                                           'type': 'PhyLayerParameters-v1310'}],
                                                                              'type': 'SEQUENCE'},
                                     'UE-EUTRA-CapabilityAddXDD-Mode-v1320': {'members': [{'name': 'phyLayerParameters-v1320',
                                                                                           'optional': True,
                                                                                           'type': 'PhyLayerParameters-v1320'},
                                                                                          {'name': 'scptm-Parameters-r13',
                                                                                           'optional': True,
                                                                                           'type': 'SCPTM-Parameters-r13'}],
                                                                              'type': 'SEQUENCE'},
                                     'UE-EUTRA-CapabilityAddXDD-Mode-v1370': {'members': [{'name': 'ce-Parameters-v1370',
                                                                                           'optional': True,
                                                                                           'type': 'CE-Parameters-v1370'}],
                                                                              'type': 'SEQUENCE'},
                                     'UE-EUTRA-CapabilityAddXDD-Mode-v1430': {'members': [{'name': 'phyLayerParameters-v1430',
                                                                                           'optional': True,
                                                                                           'type': 'PhyLayerParameters-v1430'},
                                                                                          {'name': 'mmtel-Parameters-r14',
                                                                                           'optional': True,
                                                                                           'type': 'MMTEL-Parameters-r14'}],
                                                                              'type': 'SEQUENCE'},
                                     'UE-RadioPagingInfo-r12': {'members': [{'name': 'ue-Category-v1250',
                                                                             'optional': True,
                                                                             'restricted-to': [0],
                                                                             'type': 'INTEGER'},
                                                                            None,
                                                                            [{'name': 'ue-CategoryDL-v1310',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('m1',
                                                                                          0)]},
                                                                             {'name': 'ce-ModeA-r13',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('true',
                                                                                          0)]},
                                                                             {'name': 'ce-ModeB-r13',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('true',
                                                                                          0)]}]],
                                                                'type': 'SEQUENCE'},
                                     'UE-TimersAndConstants': {'members': [{'name': 't300',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('ms100',
                                                                                        0),
                                                                                       ('ms200',
                                                                                        1),
                                                                                       ('ms300',
                                                                                        2),
                                                                                       ('ms400',
                                                                                        3),
                                                                                       ('ms600',
                                                                                        4),
                                                                                       ('ms1000',
                                                                                        5),
                                                                                       ('ms1500',
                                                                                        6),
                                                                                       ('ms2000',
                                                                                        7)]},
                                                                           {'name': 't301',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('ms100',
                                                                                        0),
                                                                                       ('ms200',
                                                                                        1),
                                                                                       ('ms300',
                                                                                        2),
                                                                                       ('ms400',
                                                                                        3),
                                                                                       ('ms600',
                                                                                        4),
                                                                                       ('ms1000',
                                                                                        5),
                                                                                       ('ms1500',
                                                                                        6),
                                                                                       ('ms2000',
                                                                                        7)]},
                                                                           {'name': 't310',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('ms0',
                                                                                        0),
                                                                                       ('ms50',
                                                                                        1),
                                                                                       ('ms100',
                                                                                        2),
                                                                                       ('ms200',
                                                                                        3),
                                                                                       ('ms500',
                                                                                        4),
                                                                                       ('ms1000',
                                                                                        5),
                                                                                       ('ms2000',
                                                                                        6)]},
                                                                           {'name': 'n310',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('n1',
                                                                                        0),
                                                                                       ('n2',
                                                                                        1),
                                                                                       ('n3',
                                                                                        2),
                                                                                       ('n4',
                                                                                        3),
                                                                                       ('n6',
                                                                                        4),
                                                                                       ('n8',
                                                                                        5),
                                                                                       ('n10',
                                                                                        6),
                                                                                       ('n20',
                                                                                        7)]},
                                                                           {'name': 't311',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('ms1000',
                                                                                        0),
                                                                                       ('ms3000',
                                                                                        1),
                                                                                       ('ms5000',
                                                                                        2),
                                                                                       ('ms10000',
                                                                                        3),
                                                                                       ('ms15000',
                                                                                        4),
                                                                                       ('ms20000',
                                                                                        5),
                                                                                       ('ms30000',
                                                                                        6)]},
                                                                           {'name': 'n311',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('n1',
                                                                                        0),
                                                                                       ('n2',
                                                                                        1),
                                                                                       ('n3',
                                                                                        2),
                                                                                       ('n4',
                                                                                        3),
                                                                                       ('n5',
                                                                                        4),
                                                                                       ('n6',
                                                                                        5),
                                                                                       ('n8',
                                                                                        6),
                                                                                       ('n10',
                                                                                        7)]},
                                                                           None,
                                                                           [{'name': 't300-v1310',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('ms2500',
                                                                                         0),
                                                                                        ('ms3000',
                                                                                         1),
                                                                                        ('ms3500',
                                                                                         2),
                                                                                        ('ms4000',
                                                                                         3),
                                                                                        ('ms5000',
                                                                                         4),
                                                                                        ('ms6000',
                                                                                         5),
                                                                                        ('ms8000',
                                                                                         6),
                                                                                        ('ms10000',
                                                                                         7)]},
                                                                            {'name': 't301-v1310',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('ms2500',
                                                                                         0),
                                                                                        ('ms3000',
                                                                                         1),
                                                                                        ('ms3500',
                                                                                         2),
                                                                                        ('ms4000',
                                                                                         3),
                                                                                        ('ms5000',
                                                                                         4),
                                                                                        ('ms6000',
                                                                                         5),
                                                                                        ('ms8000',
                                                                                         6),
                                                                                        ('ms10000',
                                                                                         7)]}],
                                                                           [{'name': 't310-v1330',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('ms4000',
                                                                                         0),
                                                                                        ('ms6000',
                                                                                         1)]}]],
                                                               'type': 'SEQUENCE'},
                                     'UEAssistanceInformation-r11': {'members': [{'members': [{'members': [{'name': 'ueAssistanceInformation-r11',
                                                                                                            'type': 'UEAssistanceInformation-r11-IEs'},
                                                                                                           {'name': 'spare3',
                                                                                                            'type': 'NULL'},
                                                                                                           {'name': 'spare2',
                                                                                                            'type': 'NULL'},
                                                                                                           {'name': 'spare1',
                                                                                                            'type': 'NULL'}],
                                                                                               'name': 'c1',
                                                                                               'type': 'CHOICE'},
                                                                                              {'members': [],
                                                                                               'name': 'criticalExtensionsFuture',
                                                                                               'type': 'SEQUENCE'}],
                                                                                  'name': 'criticalExtensions',
                                                                                  'type': 'CHOICE'}],
                                                                     'type': 'SEQUENCE'},
                                     'UEAssistanceInformation-r11-IEs': {'members': [{'name': 'powerPrefIndication-r11',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('normal',
                                                                                                  0),
                                                                                                 ('lowPowerConsumption',
                                                                                                  1)]},
                                                                                     {'name': 'lateNonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'OCTET '
                                                                                              'STRING'},
                                                                                     {'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'UEAssistanceInformation-v1430-IEs'}],
                                                                         'type': 'SEQUENCE'},
                                     'UEAssistanceInformation-v1430-IEs': {'members': [{'name': 'bw-Preference-r14',
                                                                                        'optional': True,
                                                                                        'type': 'BW-Preference-r14'},
                                                                                       {'members': [{'name': 'trafficPatternInfoListSL-r14',
                                                                                                     'optional': True,
                                                                                                     'type': 'TrafficPatternInfoList-r14'},
                                                                                                    {'name': 'trafficPatternInfoListUL-r14',
                                                                                                     'optional': True,
                                                                                                     'type': 'TrafficPatternInfoList-r14'}],
                                                                                        'name': 'sps-AssistanceInformation-r14',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'},
                                                                                       {'members': [{'name': 'rlm-Event-r14',
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('earlyOutOfSync',
                                                                                                                 0),
                                                                                                                ('earlyInSync',
                                                                                                                 1)]},
                                                                                                    {'name': 'excessRep-MPDCCH-r14',
                                                                                                     'optional': True,
                                                                                                     'type': 'ENUMERATED',
                                                                                                     'values': [('excessRep1',
                                                                                                                 0),
                                                                                                                ('excessRep2',
                                                                                                                 1)]}],
                                                                                        'name': 'rlm-Report-r14',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'},
                                                                                       {'name': 'delayBudgetReport-r14',
                                                                                        'optional': True,
                                                                                        'type': 'DelayBudgetReport-r14'},
                                                                                       {'members': [],
                                                                                        'name': 'nonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'}],
                                                                           'type': 'SEQUENCE'},
                                     'UECapabilityEnquiry': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                          'type': 'RRC-TransactionIdentifier'},
                                                                         {'members': [{'members': [{'name': 'ueCapabilityEnquiry-r8',
                                                                                                    'type': 'UECapabilityEnquiry-r8-IEs'},
                                                                                                   {'name': 'spare3',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare2',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare1',
                                                                                                    'type': 'NULL'}],
                                                                                       'name': 'c1',
                                                                                       'type': 'CHOICE'},
                                                                                      {'members': [],
                                                                                       'name': 'criticalExtensionsFuture',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'name': 'criticalExtensions',
                                                                          'type': 'CHOICE'}],
                                                             'type': 'SEQUENCE'},
                                     'UECapabilityEnquiry-r8-IEs': {'members': [{'name': 'ue-CapabilityRequest',
                                                                                 'type': 'UE-CapabilityRequest'},
                                                                                {'name': 'nonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'UECapabilityEnquiry-v8a0-IEs'}],
                                                                    'type': 'SEQUENCE'},
                                     'UECapabilityEnquiry-v1180-IEs': {'members': [{'element': {'type': 'FreqBandIndicator-r11'},
                                                                                    'name': 'requestedFrequencyBands-r11',
                                                                                    'optional': True,
                                                                                    'size': [(1,
                                                                                              16)],
                                                                                    'type': 'SEQUENCE '
                                                                                            'OF'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UECapabilityEnquiry-v1310-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UECapabilityEnquiry-v1310-IEs': {'members': [{'name': 'requestReducedFormat-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]},
                                                                                   {'name': 'requestSkipFallbackComb-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]},
                                                                                   {'name': 'requestedMaxCCsDL-r13',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(2,
                                                                                                       32)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'requestedMaxCCsUL-r13',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(2,
                                                                                                       32)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'requestReducedIntNonContComb-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UECapabilityEnquiry-v1430-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UECapabilityEnquiry-v1430-IEs': {'members': [{'name': 'requestDiffFallbackCombList-r14',
                                                                                    'optional': True,
                                                                                    'type': 'BandCombinationList-r14'},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                     'UECapabilityEnquiry-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'UECapabilityEnquiry-v1180-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'UECapabilityInformation': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                              'type': 'RRC-TransactionIdentifier'},
                                                                             {'members': [{'members': [{'name': 'ueCapabilityInformation-r8',
                                                                                                        'type': 'UECapabilityInformation-r8-IEs'},
                                                                                                       {'name': 'spare7',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare6',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare5',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare4',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare3',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare2',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare1',
                                                                                                        'type': 'NULL'}],
                                                                                           'name': 'c1',
                                                                                           'type': 'CHOICE'},
                                                                                          {'members': [],
                                                                                           'name': 'criticalExtensionsFuture',
                                                                                           'type': 'SEQUENCE'}],
                                                                              'name': 'criticalExtensions',
                                                                              'type': 'CHOICE'}],
                                                                 'type': 'SEQUENCE'},
                                     'UECapabilityInformation-r8-IEs': {'members': [{'name': 'ue-CapabilityRAT-ContainerList',
                                                                                     'type': 'UE-CapabilityRAT-ContainerList'},
                                                                                    {'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'UECapabilityInformation-v8a0-IEs'}],
                                                                        'type': 'SEQUENCE'},
                                     'UECapabilityInformation-v1250-IEs': {'members': [{'name': 'ue-RadioPagingInfo-r12',
                                                                                        'optional': True,
                                                                                        'type': 'UE-RadioPagingInfo-r12'},
                                                                                       {'members': [],
                                                                                        'name': 'nonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'}],
                                                                           'type': 'SEQUENCE'},
                                     'UECapabilityInformation-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      {'name': 'nonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'UECapabilityInformation-v1250-IEs'}],
                                                                          'type': 'SEQUENCE'},
                                     'UEInformationRequest-r9': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                              'type': 'RRC-TransactionIdentifier'},
                                                                             {'members': [{'members': [{'name': 'ueInformationRequest-r9',
                                                                                                        'type': 'UEInformationRequest-r9-IEs'},
                                                                                                       {'name': 'spare3',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare2',
                                                                                                        'type': 'NULL'},
                                                                                                       {'name': 'spare1',
                                                                                                        'type': 'NULL'}],
                                                                                           'name': 'c1',
                                                                                           'type': 'CHOICE'},
                                                                                          {'members': [],
                                                                                           'name': 'criticalExtensionsFuture',
                                                                                           'type': 'SEQUENCE'}],
                                                                              'name': 'criticalExtensions',
                                                                              'type': 'CHOICE'}],
                                                                 'type': 'SEQUENCE'},
                                     'UEInformationRequest-r9-IEs': {'members': [{'name': 'rach-ReportReq-r9',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'rlf-ReportReq-r9',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'nonCriticalExtension',
                                                                                  'optional': True,
                                                                                  'type': 'UEInformationRequest-v930-IEs'}],
                                                                     'type': 'SEQUENCE'},
                                     'UEInformationRequest-v1020-IEs': {'members': [{'name': 'logMeasReportReq-r10',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('true',
                                                                                                 0)]},
                                                                                    {'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'UEInformationRequest-v1130-IEs'}],
                                                                        'type': 'SEQUENCE'},
                                     'UEInformationRequest-v1130-IEs': {'members': [{'name': 'connEstFailReportReq-r11',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('true',
                                                                                                 0)]},
                                                                                    {'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'UEInformationRequest-v1250-IEs'}],
                                                                        'type': 'SEQUENCE'},
                                     'UEInformationRequest-v1250-IEs': {'members': [{'name': 'mobilityHistoryReportReq-r12',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('true',
                                                                                                 0)]},
                                                                                    {'members': [],
                                                                                     'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'}],
                                                                        'type': 'SEQUENCE'},
                                     'UEInformationRequest-v930-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'UEInformationRequest-v1020-IEs'}],
                                                                       'type': 'SEQUENCE'},
                                     'UEInformationResponse-r9': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                               'type': 'RRC-TransactionIdentifier'},
                                                                              {'members': [{'members': [{'name': 'ueInformationResponse-r9',
                                                                                                         'type': 'UEInformationResponse-r9-IEs'},
                                                                                                        {'name': 'spare3',
                                                                                                         'type': 'NULL'},
                                                                                                        {'name': 'spare2',
                                                                                                         'type': 'NULL'},
                                                                                                        {'name': 'spare1',
                                                                                                         'type': 'NULL'}],
                                                                                            'name': 'c1',
                                                                                            'type': 'CHOICE'},
                                                                                           {'members': [],
                                                                                            'name': 'criticalExtensionsFuture',
                                                                                            'type': 'SEQUENCE'}],
                                                                               'name': 'criticalExtensions',
                                                                               'type': 'CHOICE'}],
                                                                  'type': 'SEQUENCE'},
                                     'UEInformationResponse-r9-IEs': {'members': [{'members': [{'name': 'numberOfPreamblesSent-r9',
                                                                                                'type': 'NumberOfPreamblesSent-r11'},
                                                                                               {'name': 'contentionDetected-r9',
                                                                                                'type': 'BOOLEAN'}],
                                                                                   'name': 'rach-Report-r9',
                                                                                   'optional': True,
                                                                                   'type': 'SEQUENCE'},
                                                                                  {'name': 'rlf-Report-r9',
                                                                                   'optional': True,
                                                                                   'type': 'RLF-Report-r9'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'UEInformationResponse-v930-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'UEInformationResponse-v1020-IEs': {'members': [{'name': 'logMeasReport-r10',
                                                                                      'optional': True,
                                                                                      'type': 'LogMeasReport-r10'},
                                                                                     {'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'UEInformationResponse-v1130-IEs'}],
                                                                         'type': 'SEQUENCE'},
                                     'UEInformationResponse-v1130-IEs': {'members': [{'name': 'connEstFailReport-r11',
                                                                                      'optional': True,
                                                                                      'type': 'ConnEstFailReport-r11'},
                                                                                     {'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'UEInformationResponse-v1250-IEs'}],
                                                                         'type': 'SEQUENCE'},
                                     'UEInformationResponse-v1250-IEs': {'members': [{'name': 'mobilityHistoryReport-r12',
                                                                                      'optional': True,
                                                                                      'type': 'MobilityHistoryReport-r12'},
                                                                                     {'members': [],
                                                                                      'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'SEQUENCE'}],
                                                                         'type': 'SEQUENCE'},
                                     'UEInformationResponse-v930-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'OCTET '
                                                                                             'STRING'},
                                                                                    {'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'UEInformationResponse-v1020-IEs'}],
                                                                        'type': 'SEQUENCE'},
                                     'UEInformationResponse-v9e0-IEs': {'members': [{'name': 'rlf-Report-v9e0',
                                                                                     'optional': True,
                                                                                     'type': 'RLF-Report-v9e0'},
                                                                                    {'members': [],
                                                                                     'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'}],
                                                                        'type': 'SEQUENCE'},
                                     'UL-256QAM-perCC-Info-r14': {'members': [{'name': 'ul-256QAM-perCC-r14',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'UL-AM-RLC': {'members': [{'name': 't-PollRetransmit',
                                                                'type': 'T-PollRetransmit'},
                                                               {'name': 'pollPDU',
                                                                'type': 'PollPDU'},
                                                               {'name': 'pollByte',
                                                                'type': 'PollByte'},
                                                               {'name': 'maxRetxThreshold',
                                                                'type': 'ENUMERATED',
                                                                'values': [('t1',
                                                                            0),
                                                                           ('t2',
                                                                            1),
                                                                           ('t3',
                                                                            2),
                                                                           ('t4',
                                                                            3),
                                                                           ('t6',
                                                                            4),
                                                                           ('t8',
                                                                            5),
                                                                           ('t16',
                                                                            6),
                                                                           ('t32',
                                                                            7)]}],
                                                   'type': 'SEQUENCE'},
                                     'UL-CCCH-Message': {'members': [{'name': 'message',
                                                                      'type': 'UL-CCCH-MessageType'}],
                                                         'type': 'SEQUENCE'},
                                     'UL-CCCH-MessageType': {'members': [{'members': [{'name': 'rrcConnectionReestablishmentRequest',
                                                                                       'type': 'RRCConnectionReestablishmentRequest'},
                                                                                      {'name': 'rrcConnectionRequest',
                                                                                       'type': 'RRCConnectionRequest'}],
                                                                          'name': 'c1',
                                                                          'type': 'CHOICE'},
                                                                         {'members': [{'members': [{'name': 'rrcConnectionResumeRequest-r13',
                                                                                                    'type': 'RRCConnectionResumeRequest-r13'}],
                                                                                       'name': 'c2',
                                                                                       'type': 'CHOICE'},
                                                                                      {'members': [],
                                                                                       'name': 'messageClassExtensionFuture-r13',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'name': 'messageClassExtension',
                                                                          'type': 'CHOICE'}],
                                                             'type': 'CHOICE'},
                                     'UL-CyclicPrefixLength': {'type': 'ENUMERATED',
                                                               'values': [('len1',
                                                                           0),
                                                                          ('len2',
                                                                           1)]},
                                     'UL-DCCH-Message': {'members': [{'name': 'message',
                                                                      'type': 'UL-DCCH-MessageType'}],
                                                         'type': 'SEQUENCE'},
                                     'UL-DCCH-MessageType': {'members': [{'members': [{'name': 'csfbParametersRequestCDMA2000',
                                                                                       'type': 'CSFBParametersRequestCDMA2000'},
                                                                                      {'name': 'measurementReport',
                                                                                       'type': 'MeasurementReport'},
                                                                                      {'name': 'rrcConnectionReconfigurationComplete',
                                                                                       'type': 'RRCConnectionReconfigurationComplete'},
                                                                                      {'name': 'rrcConnectionReestablishmentComplete',
                                                                                       'type': 'RRCConnectionReestablishmentComplete'},
                                                                                      {'name': 'rrcConnectionSetupComplete',
                                                                                       'type': 'RRCConnectionSetupComplete'},
                                                                                      {'name': 'securityModeComplete',
                                                                                       'type': 'SecurityModeComplete'},
                                                                                      {'name': 'securityModeFailure',
                                                                                       'type': 'SecurityModeFailure'},
                                                                                      {'name': 'ueCapabilityInformation',
                                                                                       'type': 'UECapabilityInformation'},
                                                                                      {'name': 'ulHandoverPreparationTransfer',
                                                                                       'type': 'ULHandoverPreparationTransfer'},
                                                                                      {'name': 'ulInformationTransfer',
                                                                                       'type': 'ULInformationTransfer'},
                                                                                      {'name': 'counterCheckResponse',
                                                                                       'type': 'CounterCheckResponse'},
                                                                                      {'name': 'ueInformationResponse-r9',
                                                                                       'type': 'UEInformationResponse-r9'},
                                                                                      {'name': 'proximityIndication-r9',
                                                                                       'type': 'ProximityIndication-r9'},
                                                                                      {'name': 'rnReconfigurationComplete-r10',
                                                                                       'type': 'RNReconfigurationComplete-r10'},
                                                                                      {'name': 'mbmsCountingResponse-r10',
                                                                                       'type': 'MBMSCountingResponse-r10'},
                                                                                      {'name': 'interFreqRSTDMeasurementIndication-r10',
                                                                                       'type': 'InterFreqRSTDMeasurementIndication-r10'}],
                                                                          'name': 'c1',
                                                                          'type': 'CHOICE'},
                                                                         {'members': [{'members': [{'name': 'ueAssistanceInformation-r11',
                                                                                                    'type': 'UEAssistanceInformation-r11'},
                                                                                                   {'name': 'inDeviceCoexIndication-r11',
                                                                                                    'type': 'InDeviceCoexIndication-r11'},
                                                                                                   {'name': 'mbmsInterestIndication-r11',
                                                                                                    'type': 'MBMSInterestIndication-r11'},
                                                                                                   {'name': 'scgFailureInformation-r12',
                                                                                                    'type': 'SCGFailureInformation-r12'},
                                                                                                   {'name': 'sidelinkUEInformation-r12',
                                                                                                    'type': 'SidelinkUEInformation-r12'},
                                                                                                   {'name': 'wlanConnectionStatusReport-r13',
                                                                                                    'type': 'WLANConnectionStatusReport-r13'},
                                                                                                   {'name': 'rrcConnectionResumeComplete-r13',
                                                                                                    'type': 'RRCConnectionResumeComplete-r13'},
                                                                                                   {'name': 'spare9',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare8',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare7',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare6',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare5',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare4',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare3',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare2',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'spare1',
                                                                                                    'type': 'NULL'}],
                                                                                       'name': 'c2',
                                                                                       'type': 'CHOICE'},
                                                                                      {'members': [],
                                                                                       'name': 'messageClassExtensionFuture-r11',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'name': 'messageClassExtension',
                                                                          'type': 'CHOICE'}],
                                                             'type': 'CHOICE'},
                                     'UL-DelayConfig-r13': {'members': [{'name': 'release',
                                                                         'type': 'NULL'},
                                                                        {'members': [{'name': 'delayThreshold-r13',
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('ms30',
                                                                                                  0),
                                                                                                 ('ms40',
                                                                                                  1),
                                                                                                 ('ms50',
                                                                                                  2),
                                                                                                 ('ms60',
                                                                                                  3),
                                                                                                 ('ms70',
                                                                                                  4),
                                                                                                 ('ms80',
                                                                                                  5),
                                                                                                 ('ms90',
                                                                                                  6),
                                                                                                 ('ms100',
                                                                                                  7),
                                                                                                 ('ms150',
                                                                                                  8),
                                                                                                 ('ms300',
                                                                                                  9),
                                                                                                 ('ms500',
                                                                                                  10),
                                                                                                 ('ms750',
                                                                                                  11),
                                                                                                 ('spare4',
                                                                                                  12),
                                                                                                 ('spare3',
                                                                                                  13),
                                                                                                 ('spare2',
                                                                                                  14),
                                                                                                 ('spare1',
                                                                                                  15)]}],
                                                                         'name': 'setup',
                                                                         'type': 'SEQUENCE'}],
                                                            'type': 'CHOICE'},
                                     'UL-PDCP-DelayResult-r13': {'members': [{'name': 'qci-Id-r13',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('qci1',
                                                                                          0),
                                                                                         ('qci2',
                                                                                          1),
                                                                                         ('qci3',
                                                                                          2),
                                                                                         ('qci4',
                                                                                          3),
                                                                                         ('spare4',
                                                                                          4),
                                                                                         ('spare3',
                                                                                          5),
                                                                                         ('spare2',
                                                                                          6),
                                                                                         ('spare1',
                                                                                          7)]},
                                                                             {'name': 'excessDelay-r13',
                                                                              'restricted-to': [(0,
                                                                                                 31)],
                                                                              'type': 'INTEGER'},
                                                                             None],
                                                                 'type': 'SEQUENCE'},
                                     'UL-PDCP-DelayResultList-r13': {'element': {'type': 'UL-PDCP-DelayResult-r13'},
                                                                     'size': [(1,
                                                                               'maxQCI-r13')],
                                                                     'type': 'SEQUENCE '
                                                                             'OF'},
                                     'UL-ReferenceSignalsPUSCH': {'members': [{'name': 'groupHoppingEnabled',
                                                                               'type': 'BOOLEAN'},
                                                                              {'name': 'groupAssignmentPUSCH',
                                                                               'restricted-to': [(0,
                                                                                                  29)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'sequenceHoppingEnabled',
                                                                               'type': 'BOOLEAN'},
                                                                              {'name': 'cyclicShift',
                                                                               'restricted-to': [(0,
                                                                                                  7)],
                                                                               'type': 'INTEGER'}],
                                                                  'type': 'SEQUENCE'},
                                     'UL-UM-RLC': {'members': [{'name': 'sn-FieldLength',
                                                                'type': 'SN-FieldLength'}],
                                                   'type': 'SEQUENCE'},
                                     'ULHandoverPreparationTransfer': {'members': [{'members': [{'members': [{'name': 'ulHandoverPreparationTransfer-r8',
                                                                                                              'type': 'ULHandoverPreparationTransfer-r8-IEs'},
                                                                                                             {'name': 'spare3',
                                                                                                              'type': 'NULL'},
                                                                                                             {'name': 'spare2',
                                                                                                              'type': 'NULL'},
                                                                                                             {'name': 'spare1',
                                                                                                              'type': 'NULL'}],
                                                                                                 'name': 'c1',
                                                                                                 'type': 'CHOICE'},
                                                                                                {'members': [],
                                                                                                 'name': 'criticalExtensionsFuture',
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'name': 'criticalExtensions',
                                                                                    'type': 'CHOICE'}],
                                                                       'type': 'SEQUENCE'},
                                     'ULHandoverPreparationTransfer-r8-IEs': {'members': [{'name': 'cdma2000-Type',
                                                                                           'type': 'CDMA2000-Type'},
                                                                                          {'name': 'meid',
                                                                                           'optional': True,
                                                                                           'size': [56],
                                                                                           'type': 'BIT '
                                                                                                   'STRING'},
                                                                                          {'name': 'dedicatedInfo',
                                                                                           'type': 'DedicatedInfoCDMA2000'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'ULHandoverPreparationTransfer-v8a0-IEs'}],
                                                                              'type': 'SEQUENCE'},
                                     'ULHandoverPreparationTransfer-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'OCTET '
                                                                                                     'STRING'},
                                                                                            {'members': [],
                                                                                             'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'SEQUENCE'}],
                                                                                'type': 'SEQUENCE'},
                                     'ULInformationTransfer': {'members': [{'members': [{'members': [{'name': 'ulInformationTransfer-r8',
                                                                                                      'type': 'ULInformationTransfer-r8-IEs'},
                                                                                                     {'name': 'spare3',
                                                                                                      'type': 'NULL'},
                                                                                                     {'name': 'spare2',
                                                                                                      'type': 'NULL'},
                                                                                                     {'name': 'spare1',
                                                                                                      'type': 'NULL'}],
                                                                                         'name': 'c1',
                                                                                         'type': 'CHOICE'},
                                                                                        {'members': [],
                                                                                         'name': 'criticalExtensionsFuture',
                                                                                         'type': 'SEQUENCE'}],
                                                                            'name': 'criticalExtensions',
                                                                            'type': 'CHOICE'}],
                                                               'type': 'SEQUENCE'},
                                     'ULInformationTransfer-r8-IEs': {'members': [{'members': [{'name': 'dedicatedInfoNAS',
                                                                                                'type': 'DedicatedInfoNAS'},
                                                                                               {'name': 'dedicatedInfoCDMA2000-1XRTT',
                                                                                                'type': 'DedicatedInfoCDMA2000'},
                                                                                               {'name': 'dedicatedInfoCDMA2000-HRPD',
                                                                                                'type': 'DedicatedInfoCDMA2000'}],
                                                                                   'name': 'dedicatedInfoType',
                                                                                   'type': 'CHOICE'},
                                                                                  {'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'ULInformationTransfer-v8a0-IEs'}],
                                                                      'type': 'SEQUENCE'},
                                     'ULInformationTransfer-v8a0-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'OCTET '
                                                                                             'STRING'},
                                                                                    {'members': [],
                                                                                     'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'}],
                                                                        'type': 'SEQUENCE'},
                                     'UplinkPUSCH-LessPowerControlDedicated-v1430': {'members': [{'name': 'p0-UE-PeriodicSRS-r14',
                                                                                                  'optional': True,
                                                                                                  'restricted-to': [(-8,
                                                                                                                     7)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'name': 'p0-UE-AperiodicSRS-r14',
                                                                                                  'optional': True,
                                                                                                  'restricted-to': [(-8,
                                                                                                                     7)],
                                                                                                  'type': 'INTEGER'},
                                                                                                 {'name': 'accumulationEnabled-r14',
                                                                                                  'type': 'BOOLEAN'}],
                                                                                     'type': 'SEQUENCE'},
                                     'UplinkPowerControlCommon': {'members': [{'name': 'p0-NominalPUSCH',
                                                                               'restricted-to': [(-126,
                                                                                                  24)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'alpha',
                                                                               'type': 'Alpha-r12'},
                                                                              {'name': 'p0-NominalPUCCH',
                                                                               'restricted-to': [(-127,
                                                                                                  -96)],
                                                                               'type': 'INTEGER'},
                                                                              {'name': 'deltaFList-PUCCH',
                                                                               'type': 'DeltaFList-PUCCH'},
                                                                              {'name': 'deltaPreambleMsg3',
                                                                               'restricted-to': [(-1,
                                                                                                  6)],
                                                                               'type': 'INTEGER'}],
                                                                  'type': 'SEQUENCE'},
                                     'UplinkPowerControlCommon-v1020': {'members': [{'name': 'deltaF-PUCCH-Format3-r10',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('deltaF-1',
                                                                                                 0),
                                                                                                ('deltaF0',
                                                                                                 1),
                                                                                                ('deltaF1',
                                                                                                 2),
                                                                                                ('deltaF2',
                                                                                                 3),
                                                                                                ('deltaF3',
                                                                                                 4),
                                                                                                ('deltaF4',
                                                                                                 5),
                                                                                                ('deltaF5',
                                                                                                 6),
                                                                                                ('deltaF6',
                                                                                                 7)]},
                                                                                    {'name': 'deltaF-PUCCH-Format1bCS-r10',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('deltaF1',
                                                                                                 0),
                                                                                                ('deltaF2',
                                                                                                 1),
                                                                                                ('spare2',
                                                                                                 2),
                                                                                                ('spare1',
                                                                                                 3)]}],
                                                                        'type': 'SEQUENCE'},
                                     'UplinkPowerControlCommon-v1310': {'members': [{'name': 'deltaF-PUCCH-Format4-r13',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('deltaF16',
                                                                                                 0),
                                                                                                ('deltaF15',
                                                                                                 1),
                                                                                                ('deltaF14',
                                                                                                 2),
                                                                                                ('deltaF13',
                                                                                                 3),
                                                                                                ('deltaF12',
                                                                                                 4),
                                                                                                ('deltaF11',
                                                                                                 5),
                                                                                                ('deltaF10',
                                                                                                 6),
                                                                                                ('spare1',
                                                                                                 7)]},
                                                                                    {'name': 'deltaF-PUCCH-Format5-13',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('deltaF13',
                                                                                                 0),
                                                                                                ('deltaF12',
                                                                                                 1),
                                                                                                ('deltaF11',
                                                                                                 2),
                                                                                                ('deltaF10',
                                                                                                 3),
                                                                                                ('deltaF9',
                                                                                                 4),
                                                                                                ('deltaF8',
                                                                                                 5),
                                                                                                ('deltaF7',
                                                                                                 6),
                                                                                                ('spare1',
                                                                                                 7)]}],
                                                                        'type': 'SEQUENCE'},
                                     'UplinkPowerControlCommonPSCell-r12': {'members': [{'name': 'deltaF-PUCCH-Format3-r12',
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('deltaF-1',
                                                                                                     0),
                                                                                                    ('deltaF0',
                                                                                                     1),
                                                                                                    ('deltaF1',
                                                                                                     2),
                                                                                                    ('deltaF2',
                                                                                                     3),
                                                                                                    ('deltaF3',
                                                                                                     4),
                                                                                                    ('deltaF4',
                                                                                                     5),
                                                                                                    ('deltaF5',
                                                                                                     6),
                                                                                                    ('deltaF6',
                                                                                                     7)]},
                                                                                        {'name': 'deltaF-PUCCH-Format1bCS-r12',
                                                                                         'type': 'ENUMERATED',
                                                                                         'values': [('deltaF1',
                                                                                                     0),
                                                                                                    ('deltaF2',
                                                                                                     1),
                                                                                                    ('spare2',
                                                                                                     2),
                                                                                                    ('spare1',
                                                                                                     3)]},
                                                                                        {'name': 'p0-NominalPUCCH-r12',
                                                                                         'restricted-to': [(-127,
                                                                                                            -96)],
                                                                                         'type': 'INTEGER'},
                                                                                        {'name': 'deltaFList-PUCCH-r12',
                                                                                         'type': 'DeltaFList-PUCCH'}],
                                                                            'type': 'SEQUENCE'},
                                     'UplinkPowerControlCommonPUSCH-LessCell-v1430': {'members': [{'name': 'p0-Nominal-PeriodicSRS-r14',
                                                                                                   'optional': True,
                                                                                                   'restricted-to': [(-126,
                                                                                                                      24)],
                                                                                                   'type': 'INTEGER'},
                                                                                                  {'name': 'p0-Nominal-AperiodicSRS-r14',
                                                                                                   'optional': True,
                                                                                                   'restricted-to': [(-126,
                                                                                                                      24)],
                                                                                                   'type': 'INTEGER'},
                                                                                                  {'name': 'alpha-SRS-r14',
                                                                                                   'optional': True,
                                                                                                   'type': 'Alpha-r12'}],
                                                                                      'type': 'SEQUENCE'},
                                     'UplinkPowerControlCommonSCell-r10': {'members': [{'name': 'p0-NominalPUSCH-r10',
                                                                                        'restricted-to': [(-126,
                                                                                                           24)],
                                                                                        'type': 'INTEGER'},
                                                                                       {'name': 'alpha-r10',
                                                                                        'type': 'Alpha-r12'}],
                                                                           'type': 'SEQUENCE'},
                                     'UplinkPowerControlCommonSCell-v1130': {'members': [{'name': 'deltaPreambleMsg3-r11',
                                                                                          'restricted-to': [(-1,
                                                                                                             6)],
                                                                                          'type': 'INTEGER'}],
                                                                             'type': 'SEQUENCE'},
                                     'UplinkPowerControlCommonSCell-v1310': {'members': [{'name': 'p0-NominalPUCCH',
                                                                                          'restricted-to': [(-127,
                                                                                                             -96)],
                                                                                          'type': 'INTEGER'},
                                                                                         {'name': 'deltaFList-PUCCH',
                                                                                          'type': 'DeltaFList-PUCCH'},
                                                                                         {'name': 'deltaF-PUCCH-Format3-r12',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('deltaF-1',
                                                                                                      0),
                                                                                                     ('deltaF0',
                                                                                                      1),
                                                                                                     ('deltaF1',
                                                                                                      2),
                                                                                                     ('deltaF2',
                                                                                                      3),
                                                                                                     ('deltaF3',
                                                                                                      4),
                                                                                                     ('deltaF4',
                                                                                                      5),
                                                                                                     ('deltaF5',
                                                                                                      6),
                                                                                                     ('deltaF6',
                                                                                                      7)]},
                                                                                         {'name': 'deltaF-PUCCH-Format1bCS-r12',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('deltaF1',
                                                                                                      0),
                                                                                                     ('deltaF2',
                                                                                                      1),
                                                                                                     ('spare2',
                                                                                                      2),
                                                                                                     ('spare1',
                                                                                                      3)]},
                                                                                         {'name': 'deltaF-PUCCH-Format4-r13',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('deltaF16',
                                                                                                      0),
                                                                                                     ('deltaF15',
                                                                                                      1),
                                                                                                     ('deltaF14',
                                                                                                      2),
                                                                                                     ('deltaF13',
                                                                                                      3),
                                                                                                     ('deltaF12',
                                                                                                      4),
                                                                                                     ('deltaF11',
                                                                                                      5),
                                                                                                     ('deltaF10',
                                                                                                      6),
                                                                                                     ('spare1',
                                                                                                      7)]},
                                                                                         {'name': 'deltaF-PUCCH-Format5-13',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('deltaF13',
                                                                                                      0),
                                                                                                     ('deltaF12',
                                                                                                      1),
                                                                                                     ('deltaF11',
                                                                                                      2),
                                                                                                     ('deltaF10',
                                                                                                      3),
                                                                                                     ('deltaF9',
                                                                                                      4),
                                                                                                     ('deltaF8',
                                                                                                      5),
                                                                                                     ('deltaF7',
                                                                                                      6),
                                                                                                     ('spare1',
                                                                                                      7)]}],
                                                                             'type': 'SEQUENCE'},
                                     'UplinkPowerControlDedicated': {'members': [{'name': 'p0-UE-PUSCH',
                                                                                  'restricted-to': [(-8,
                                                                                                     7)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'name': 'deltaMCS-Enabled',
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('en0',
                                                                                              0),
                                                                                             ('en1',
                                                                                              1)]},
                                                                                 {'name': 'accumulationEnabled',
                                                                                  'type': 'BOOLEAN'},
                                                                                 {'name': 'p0-UE-PUCCH',
                                                                                  'restricted-to': [(-8,
                                                                                                     7)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'name': 'pSRS-Offset',
                                                                                  'restricted-to': [(0,
                                                                                                     15)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'default': 'fc4',
                                                                                  'name': 'filterCoefficient',
                                                                                  'type': 'FilterCoefficient'}],
                                                                     'type': 'SEQUENCE'},
                                     'UplinkPowerControlDedicated-v1020': {'members': [{'name': 'deltaTxD-OffsetListPUCCH-r10',
                                                                                        'optional': True,
                                                                                        'type': 'DeltaTxD-OffsetListPUCCH-r10'},
                                                                                       {'name': 'pSRS-OffsetAp-r10',
                                                                                        'optional': True,
                                                                                        'restricted-to': [(0,
                                                                                                           15)],
                                                                                        'type': 'INTEGER'}],
                                                                           'type': 'SEQUENCE'},
                                     'UplinkPowerControlDedicated-v1130': {'members': [{'name': 'pSRS-Offset-v1130',
                                                                                        'optional': True,
                                                                                        'restricted-to': [(16,
                                                                                                           31)],
                                                                                        'type': 'INTEGER'},
                                                                                       {'name': 'pSRS-OffsetAp-v1130',
                                                                                        'optional': True,
                                                                                        'restricted-to': [(16,
                                                                                                           31)],
                                                                                        'type': 'INTEGER'},
                                                                                       {'name': 'deltaTxD-OffsetListPUCCH-v1130',
                                                                                        'optional': True,
                                                                                        'type': 'DeltaTxD-OffsetListPUCCH-v1130'}],
                                                                           'type': 'SEQUENCE'},
                                     'UplinkPowerControlDedicated-v1250': {'members': [{'members': [{'name': 'release',
                                                                                                     'type': 'NULL'},
                                                                                                    {'members': [{'name': 'tpc-SubframeSet-r12',
                                                                                                                  'size': [10],
                                                                                                                  'type': 'BIT '
                                                                                                                          'STRING'},
                                                                                                                 {'name': 'p0-NominalPUSCH-SubframeSet2-r12',
                                                                                                                  'restricted-to': [(-126,
                                                                                                                                     24)],
                                                                                                                  'type': 'INTEGER'},
                                                                                                                 {'name': 'alpha-SubframeSet2-r12',
                                                                                                                  'type': 'Alpha-r12'},
                                                                                                                 {'name': 'p0-UE-PUSCH-SubframeSet2-r12',
                                                                                                                  'restricted-to': [(-8,
                                                                                                                                     7)],
                                                                                                                  'type': 'INTEGER'}],
                                                                                                     'name': 'setup',
                                                                                                     'type': 'SEQUENCE'}],
                                                                                        'name': 'set2PowerControlParameter',
                                                                                        'type': 'CHOICE'}],
                                                                           'type': 'SEQUENCE'},
                                     'UplinkPowerControlDedicatedSCell-r10': {'members': [{'name': 'p0-UE-PUSCH-r10',
                                                                                           'restricted-to': [(-8,
                                                                                                              7)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'deltaMCS-Enabled-r10',
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('en0',
                                                                                                       0),
                                                                                                      ('en1',
                                                                                                       1)]},
                                                                                          {'name': 'accumulationEnabled-r10',
                                                                                           'type': 'BOOLEAN'},
                                                                                          {'name': 'pSRS-Offset-r10',
                                                                                           'restricted-to': [(0,
                                                                                                              15)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'name': 'pSRS-OffsetAp-r10',
                                                                                           'optional': True,
                                                                                           'restricted-to': [(0,
                                                                                                              15)],
                                                                                           'type': 'INTEGER'},
                                                                                          {'default': 'fc4',
                                                                                           'name': 'filterCoefficient-r10',
                                                                                           'type': 'FilterCoefficient'},
                                                                                          {'name': 'pathlossReferenceLinking-r10',
                                                                                           'type': 'ENUMERATED',
                                                                                           'values': [('pCell',
                                                                                                       0),
                                                                                                      ('sCell',
                                                                                                       1)]}],
                                                                              'type': 'SEQUENCE'},
                                     'UplinkPowerControlDedicatedSCell-v1310': {'members': [{'name': 'p0-UE-PUCCH',
                                                                                             'restricted-to': [(-8,
                                                                                                                7)],
                                                                                             'type': 'INTEGER'},
                                                                                            {'name': 'deltaTxD-OffsetListPUCCH-r10',
                                                                                             'optional': True,
                                                                                             'type': 'DeltaTxD-OffsetListPUCCH-r10'}],
                                                                                'type': 'SEQUENCE'},
                                     'V2X-BandCombinationParameters-r14': {'element': {'type': 'V2X-BandParameters-r14'},
                                                                           'size': [(1,
                                                                                     'maxSimultaneousBands-r10')],
                                                                           'type': 'SEQUENCE '
                                                                                   'OF'},
                                     'V2X-BandParameters-r14': {'members': [{'name': 'v2x-FreqBandEUTRA-r14',
                                                                             'type': 'FreqBandIndicator-r11'},
                                                                            {'name': 'bandParametersTxSL-r14',
                                                                             'optional': True,
                                                                             'type': 'BandParametersTxSL-r14'},
                                                                            {'name': 'bandParametersRxSL-r14',
                                                                             'optional': True,
                                                                             'type': 'BandParametersRxSL-r14'}],
                                                                'type': 'SEQUENCE'},
                                     'V2X-BandwidthClass-r14': {'type': 'ENUMERATED',
                                                                'values': [('a',
                                                                            0),
                                                                           ('b',
                                                                            1),
                                                                           ('c',
                                                                            2),
                                                                           ('d',
                                                                            3),
                                                                           ('e',
                                                                            4),
                                                                           ('f',
                                                                            5),
                                                                           None]},
                                     'V2X-BandwidthClassSL-r14': {'element': {'type': 'V2X-BandwidthClass-r14'},
                                                                  'size': [(1,
                                                                            'maxBandwidthClass-r10')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'V2X-SupportedBandCombination-r14': {'element': {'type': 'V2X-BandCombinationParameters-r14'},
                                                                          'size': [(1,
                                                                                    'maxBandComb-r13')],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                     'VictimSystemType-r11': {'members': [{'name': 'gps-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('true',
                                                                                       0)]},
                                                                          {'name': 'glonass-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('true',
                                                                                       0)]},
                                                                          {'name': 'bds-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('true',
                                                                                       0)]},
                                                                          {'name': 'galileo-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('true',
                                                                                       0)]},
                                                                          {'name': 'wlan-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('true',
                                                                                       0)]},
                                                                          {'name': 'bluetooth-r11',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('true',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'VisitedCellInfo-r12': {'members': [{'members': [{'name': 'cellGlobalId-r12',
                                                                                       'type': 'CellGlobalIdEUTRA'},
                                                                                      {'members': [{'name': 'physCellId-r12',
                                                                                                    'type': 'PhysCellId'},
                                                                                                   {'name': 'carrierFreq-r12',
                                                                                                    'type': 'ARFCN-ValueEUTRA-r9'}],
                                                                                       'name': 'pci-arfcn-r12',
                                                                                       'type': 'SEQUENCE'}],
                                                                          'name': 'visitedCellId-r12',
                                                                          'optional': True,
                                                                          'type': 'CHOICE'},
                                                                         {'name': 'timeSpent-r12',
                                                                          'restricted-to': [(0,
                                                                                             4095)],
                                                                          'type': 'INTEGER'},
                                                                         None],
                                                             'type': 'SEQUENCE'},
                                     'VisitedCellInfoList-r12': {'element': {'type': 'VisitedCellInfo-r12'},
                                                                 'size': [(1,
                                                                           'maxCellHistory-r12')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'WLAN-BandIndicator-r13': {'type': 'ENUMERATED',
                                                                'values': [('band2dot4',
                                                                            0),
                                                                           ('band5',
                                                                            1),
                                                                           ('band60-v1430',
                                                                            2),
                                                                           ('spare5',
                                                                            3),
                                                                           ('spare4',
                                                                            4),
                                                                           ('spare3',
                                                                            5),
                                                                           ('spare2',
                                                                            6),
                                                                           ('spare1',
                                                                            7),
                                                                           None]},
                                     'WLAN-CarrierInfo-r13': {'members': [{'name': 'operatingClass-r13',
                                                                           'optional': True,
                                                                           'restricted-to': [(0,
                                                                                              255)],
                                                                           'type': 'INTEGER'},
                                                                          {'name': 'countryCode-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('unitedStates',
                                                                                       0),
                                                                                      ('europe',
                                                                                       1),
                                                                                      ('japan',
                                                                                       2),
                                                                                      ('global',
                                                                                       3),
                                                                                      None]},
                                                                          {'name': 'channelNumbers-r13',
                                                                           'optional': True,
                                                                           'type': 'WLAN-ChannelList-r13'},
                                                                          None],
                                                              'type': 'SEQUENCE'},
                                     'WLAN-Channel-r13': {'restricted-to': [(0,
                                                                             255)],
                                                          'type': 'INTEGER'},
                                     'WLAN-ChannelList-r13': {'element': {'type': 'WLAN-Channel-r13'},
                                                              'size': [(1,
                                                                        'maxWLAN-Channels-r13')],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'WLAN-IW-Parameters-r12': {'members': [{'name': 'wlan-IW-RAN-Rules-r12',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]},
                                                                            {'name': 'wlan-IW-ANDSF-Policies-r12',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]}],
                                                                'type': 'SEQUENCE'},
                                     'WLAN-IW-Parameters-v1310': {'members': [{'name': 'rclwi-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('supported',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'WLAN-Id-List-r12': {'element': {'type': 'WLAN-Identifiers-r12'},
                                                          'size': [(1,
                                                                    'maxWLAN-Id-r12')],
                                                          'type': 'SEQUENCE '
                                                                  'OF'},
                                     'WLAN-Id-List-r13': {'element': {'type': 'WLAN-Identifiers-r12'},
                                                          'size': [(1,
                                                                    'maxWLAN-Id-r13')],
                                                          'type': 'SEQUENCE '
                                                                  'OF'},
                                     'WLAN-Identifiers-r12': {'members': [{'name': 'ssid-r12',
                                                                           'optional': True,
                                                                           'size': [(1,
                                                                                     32)],
                                                                           'type': 'OCTET '
                                                                                   'STRING'},
                                                                          {'name': 'bssid-r12',
                                                                           'optional': True,
                                                                           'size': [6],
                                                                           'type': 'OCTET '
                                                                                   'STRING'},
                                                                          {'name': 'hessid-r12',
                                                                           'optional': True,
                                                                           'size': [6],
                                                                           'type': 'OCTET '
                                                                                   'STRING'},
                                                                          None],
                                                              'type': 'SEQUENCE'},
                                     'WLAN-MobilityConfig-r13': {'members': [{'name': 'wlan-ToReleaseList-r13',
                                                                              'optional': True,
                                                                              'type': 'WLAN-Id-List-r13'},
                                                                             {'name': 'wlan-ToAddList-r13',
                                                                              'optional': True,
                                                                              'type': 'WLAN-Id-List-r13'},
                                                                             {'name': 'associationTimer-r13',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('s10',
                                                                                          0),
                                                                                         ('s30',
                                                                                          1),
                                                                                         ('s60',
                                                                                          2),
                                                                                         ('s120',
                                                                                          3),
                                                                                         ('s240',
                                                                                          4)]},
                                                                             {'name': 'successReportRequested-r13',
                                                                              'optional': True,
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('true',
                                                                                          0)]},
                                                                             None,
                                                                             [{'name': 'wlan-SuspendConfig-r14',
                                                                               'optional': True,
                                                                               'type': 'WLAN-SuspendConfig-r14'}]],
                                                                 'type': 'SEQUENCE'},
                                     'WLAN-OffloadConfig-r12': {'members': [{'members': [{'name': 'thresholdRSRP-Low-r12',
                                                                                          'type': 'RSRP-Range'},
                                                                                         {'name': 'thresholdRSRP-High-r12',
                                                                                          'type': 'RSRP-Range'}],
                                                                             'name': 'thresholdRSRP-r12',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'thresholdRSRQ-Low-r12',
                                                                                          'type': 'RSRQ-Range'},
                                                                                         {'name': 'thresholdRSRQ-High-r12',
                                                                                          'type': 'RSRQ-Range'}],
                                                                             'name': 'thresholdRSRQ-r12',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'thresholdRSRQ-OnAllSymbolsWithWB-Low-r12',
                                                                                          'type': 'RSRQ-Range'},
                                                                                         {'name': 'thresholdRSRQ-OnAllSymbolsWithWB-High-r12',
                                                                                          'type': 'RSRQ-Range'}],
                                                                             'name': 'thresholdRSRQ-OnAllSymbolsWithWB-r12',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'thresholdRSRQ-OnAllSymbolsLow-r12',
                                                                                          'type': 'RSRQ-Range'},
                                                                                         {'name': 'thresholdRSRQ-OnAllSymbolsHigh-r12',
                                                                                          'type': 'RSRQ-Range'}],
                                                                             'name': 'thresholdRSRQ-OnAllSymbols-r12',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'thresholdRSRQ-WB-Low-r12',
                                                                                          'type': 'RSRQ-Range'},
                                                                                         {'name': 'thresholdRSRQ-WB-High-r12',
                                                                                          'type': 'RSRQ-Range'}],
                                                                             'name': 'thresholdRSRQ-WB-r12',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'thresholdChannelUtilizationLow-r12',
                                                                                          'restricted-to': [(0,
                                                                                                             255)],
                                                                                          'type': 'INTEGER'},
                                                                                         {'name': 'thresholdChannelUtilizationHigh-r12',
                                                                                          'restricted-to': [(0,
                                                                                                             255)],
                                                                                          'type': 'INTEGER'}],
                                                                             'name': 'thresholdChannelUtilization-r12',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'thresholdBackhaulDL-BandwidthLow-r12',
                                                                                          'type': 'WLAN-backhaulRate-r12'},
                                                                                         {'name': 'thresholdBackhaulDL-BandwidthHigh-r12',
                                                                                          'type': 'WLAN-backhaulRate-r12'},
                                                                                         {'name': 'thresholdBackhaulUL-BandwidthLow-r12',
                                                                                          'type': 'WLAN-backhaulRate-r12'},
                                                                                         {'name': 'thresholdBackhaulUL-BandwidthHigh-r12',
                                                                                          'type': 'WLAN-backhaulRate-r12'}],
                                                                             'name': 'thresholdBackhaul-Bandwidth-r12',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'members': [{'name': 'thresholdWLAN-RSSI-Low-r12',
                                                                                          'restricted-to': [(0,
                                                                                                             255)],
                                                                                          'type': 'INTEGER'},
                                                                                         {'name': 'thresholdWLAN-RSSI-High-r12',
                                                                                          'restricted-to': [(0,
                                                                                                             255)],
                                                                                          'type': 'INTEGER'}],
                                                                             'name': 'thresholdWLAN-RSSI-r12',
                                                                             'optional': True,
                                                                             'type': 'SEQUENCE'},
                                                                            {'name': 'offloadPreferenceIndicator-r12',
                                                                             'optional': True,
                                                                             'size': [16],
                                                                             'type': 'BIT '
                                                                                     'STRING'},
                                                                            {'name': 't-SteeringWLAN-r12',
                                                                             'optional': True,
                                                                             'type': 'T-Reselection'},
                                                                            None],
                                                                'type': 'SEQUENCE'},
                                     'WLAN-OffloadInfoPerPLMN-r12': {'members': [{'name': 'wlan-OffloadConfigCommon-r12',
                                                                                  'optional': True,
                                                                                  'type': 'WLAN-OffloadConfig-r12'},
                                                                                 {'name': 'wlan-Id-List-r12',
                                                                                  'optional': True,
                                                                                  'type': 'WLAN-Id-List-r12'},
                                                                                 None],
                                                                     'type': 'SEQUENCE'},
                                     'WLAN-RSSI-Range-r13': {'restricted-to': [(0,
                                                                                141)],
                                                             'type': 'INTEGER'},
                                     'WLAN-Status-r13': {'type': 'ENUMERATED',
                                                         'values': [('successfulAssociation',
                                                                     0),
                                                                    ('failureWlanRadioLink',
                                                                     1),
                                                                    ('failureWlanUnavailable',
                                                                     2),
                                                                    ('failureTimeout',
                                                                     3)]},
                                     'WLAN-Status-v1430': {'type': 'ENUMERATED',
                                                           'values': [('suspended',
                                                                       0),
                                                                      ('resumed',
                                                                       1)]},
                                     'WLAN-SuspendConfig-r14': {'members': [{'name': 'wlan-SuspendResumeAllowed-r14',
                                                                             'optional': True,
                                                                             'type': 'BOOLEAN'},
                                                                            {'name': 'wlan-SuspendTriggersStatusReport-r14',
                                                                             'optional': True,
                                                                             'type': 'BOOLEAN'}],
                                                                'type': 'SEQUENCE'},
                                     'WLAN-backhaulRate-r12': {'type': 'ENUMERATED',
                                                               'values': [('r0',
                                                                           0),
                                                                          ('r4',
                                                                           1),
                                                                          ('r8',
                                                                           2),
                                                                          ('r16',
                                                                           3),
                                                                          ('r32',
                                                                           4),
                                                                          ('r64',
                                                                           5),
                                                                          ('r128',
                                                                           6),
                                                                          ('r256',
                                                                           7),
                                                                          ('r512',
                                                                           8),
                                                                          ('r1024',
                                                                           9),
                                                                          ('r2048',
                                                                           10),
                                                                          ('r4096',
                                                                           11),
                                                                          ('r8192',
                                                                           12),
                                                                          ('r16384',
                                                                           13),
                                                                          ('r32768',
                                                                           14),
                                                                          ('r65536',
                                                                           15),
                                                                          ('r131072',
                                                                           16),
                                                                          ('r262144',
                                                                           17),
                                                                          ('r524288',
                                                                           18),
                                                                          ('r1048576',
                                                                           19),
                                                                          ('r2097152',
                                                                           20),
                                                                          ('r4194304',
                                                                           21),
                                                                          ('r8388608',
                                                                           22),
                                                                          ('r16777216',
                                                                           23),
                                                                          ('r33554432',
                                                                           24),
                                                                          ('r67108864',
                                                                           25),
                                                                          ('r134217728',
                                                                           26),
                                                                          ('r268435456',
                                                                           27),
                                                                          ('r536870912',
                                                                           28),
                                                                          ('r1073741824',
                                                                           29),
                                                                          ('r2147483648',
                                                                           30),
                                                                          ('r4294967296',
                                                                           31)]},
                                     'WLANConnectionStatusReport-r13': {'members': [{'members': [{'members': [{'name': 'wlanConnectionStatusReport-r13',
                                                                                                               'type': 'WLANConnectionStatusReport-r13-IEs'},
                                                                                                              {'name': 'spare3',
                                                                                                               'type': 'NULL'},
                                                                                                              {'name': 'spare2',
                                                                                                               'type': 'NULL'},
                                                                                                              {'name': 'spare1',
                                                                                                               'type': 'NULL'}],
                                                                                                  'name': 'c1',
                                                                                                  'type': 'CHOICE'},
                                                                                                 {'members': [],
                                                                                                  'name': 'criticalExtensionsFuture',
                                                                                                  'type': 'SEQUENCE'}],
                                                                                     'name': 'criticalExtensions',
                                                                                     'type': 'CHOICE'}],
                                                                        'type': 'SEQUENCE'},
                                     'WLANConnectionStatusReport-r13-IEs': {'members': [{'name': 'wlan-Status-r13',
                                                                                         'type': 'WLAN-Status-r13'},
                                                                                        {'name': 'lateNonCriticalExtension',
                                                                                         'optional': True,
                                                                                         'type': 'OCTET '
                                                                                                 'STRING'},
                                                                                        {'name': 'nonCriticalExtension',
                                                                                         'optional': True,
                                                                                         'type': 'WLANConnectionStatusReport-v1430-IEs'}],
                                                                            'type': 'SEQUENCE'},
                                     'WLANConnectionStatusReport-v1430-IEs': {'members': [{'name': 'wlan-Status-v1430',
                                                                                           'type': 'WLAN-Status-v1430'},
                                                                                          {'members': [],
                                                                                           'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SEQUENCE'}],
                                                                              'type': 'SEQUENCE'},
                                     'WhiteCellsToAddMod-r13': {'members': [{'name': 'cellIndex-r13',
                                                                             'restricted-to': [(1,
                                                                                                'maxCellMeas')],
                                                                             'type': 'INTEGER'},
                                                                            {'name': 'physCellIdRange-r13',
                                                                             'type': 'PhysCellIdRange'}],
                                                                'type': 'SEQUENCE'},
                                     'WhiteCellsToAddModList-r13': {'element': {'type': 'WhiteCellsToAddMod-r13'},
                                                                    'size': [(1,
                                                                              'maxCellMeas')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'ZeroTxPowerCSI-RS-Conf-r12': {'members': [{'name': 'release',
                                                                                 'type': 'NULL'},
                                                                                {'name': 'setup',
                                                                                 'type': 'ZeroTxPowerCSI-RS-r12'}],
                                                                    'type': 'CHOICE'},
                                     'ZeroTxPowerCSI-RS-r12': {'members': [{'name': 'zeroTxPowerResourceConfigList-r12',
                                                                            'size': [16],
                                                                            'type': 'BIT '
                                                                                    'STRING'},
                                                                           {'name': 'zeroTxPowerSubframeConfig-r12',
                                                                            'restricted-to': [(0,
                                                                                               154)],
                                                                            'type': 'INTEGER'}],
                                                               'type': 'SEQUENCE'}},
                           'values': {'maxACDC-Cat-r13': {'type': 'INTEGER',
                                                          'value': 16},
                                      'maxAvailNarrowBands-r13': {'type': 'INTEGER',
                                                                  'value': 16},
                                      'maxBandComb-r10': {'type': 'INTEGER',
                                                          'value': 128},
                                      'maxBandComb-r11': {'type': 'INTEGER',
                                                          'value': 256},
                                      'maxBandComb-r13': {'type': 'INTEGER',
                                                          'value': 384},
                                      'maxBands': {'type': 'INTEGER',
                                                   'value': 64},
                                      'maxBandwidthClass-r10': {'type': 'INTEGER',
                                                                'value': 16},
                                      'maxBandwidthCombSet-r10': {'type': 'INTEGER',
                                                                  'value': 32},
                                      'maxCBR-Level-1-r14': {'type': 'INTEGER',
                                                             'value': 15},
                                      'maxCBR-Level-r14': {'type': 'INTEGER',
                                                           'value': 16},
                                      'maxCBR-Report-r14': {'type': 'INTEGER',
                                                            'value': 72},
                                      'maxCDMA-BandClass': {'type': 'INTEGER',
                                                            'value': 32},
                                      'maxCE-Level-r13': {'type': 'INTEGER',
                                                          'value': 4},
                                      'maxCQI-ProcExt-r11': {'type': 'INTEGER',
                                                             'value': 3},
                                      'maxCSI-IM-r11': {'type': 'INTEGER',
                                                        'value': 3},
                                      'maxCSI-IM-r12': {'type': 'INTEGER',
                                                        'value': 4},
                                      'maxCSI-IM-r13': {'type': 'INTEGER',
                                                        'value': 24},
                                      'maxCSI-IM-v1310': {'type': 'INTEGER',
                                                          'value': 20},
                                      'maxCSI-Proc-r11': {'type': 'INTEGER',
                                                          'value': 4},
                                      'maxCSI-RS-Meas-r12': {'type': 'INTEGER',
                                                             'value': 96},
                                      'maxCSI-RS-NZP-r11': {'type': 'INTEGER',
                                                            'value': 3},
                                      'maxCSI-RS-NZP-r13': {'type': 'INTEGER',
                                                            'value': 24},
                                      'maxCSI-RS-NZP-v1310': {'type': 'INTEGER',
                                                              'value': 21},
                                      'maxCSI-RS-ZP-r11': {'type': 'INTEGER',
                                                           'value': 4},
                                      'maxCellBlack': {'type': 'INTEGER',
                                                       'value': 16},
                                      'maxCellHistory-r12': {'type': 'INTEGER',
                                                             'value': 16},
                                      'maxCellInfoGERAN-r9': {'type': 'INTEGER',
                                                              'value': 32},
                                      'maxCellInfoUTRA-r9': {'type': 'INTEGER',
                                                             'value': 16},
                                      'maxCellInter': {'type': 'INTEGER',
                                                       'value': 16},
                                      'maxCellIntra': {'type': 'INTEGER',
                                                       'value': 16},
                                      'maxCellListGERAN': {'type': 'INTEGER',
                                                           'value': 3},
                                      'maxCellMeas': {'type': 'INTEGER',
                                                      'value': 32},
                                      'maxCellReport': {'type': 'INTEGER',
                                                        'value': 8},
                                      'maxCombIDC-r11': {'type': 'INTEGER',
                                                         'value': 128},
                                      'maxConfigSPS-r14': {'type': 'INTEGER',
                                                           'value': 8},
                                      'maxDRB': {'type': 'INTEGER',
                                                 'value': 11},
                                      'maxDS-Duration-r12': {'type': 'INTEGER',
                                                             'value': 5},
                                      'maxDS-ZTP-CSI-RS-r12': {'type': 'INTEGER',
                                                               'value': 5},
                                      'maxEARFCN': {'type': 'INTEGER',
                                                    'value': 65535},
                                      'maxEARFCN-Plus1': {'type': 'INTEGER',
                                                          'value': 65536},
                                      'maxEARFCN2': {'type': 'INTEGER',
                                                     'value': 262143},
                                      'maxEPDCCH-Set-r11': {'type': 'INTEGER',
                                                            'value': 2},
                                      'maxFBI': {'type': 'INTEGER',
                                                 'value': 64},
                                      'maxFBI-Plus1': {'type': 'INTEGER',
                                                       'value': 65},
                                      'maxFBI2': {'type': 'INTEGER',
                                                  'value': 256},
                                      'maxFreq': {'type': 'INTEGER',
                                                  'value': 8},
                                      'maxFreqIDC-r11': {'type': 'INTEGER',
                                                         'value': 32},
                                      'maxFreqMBMS-r11': {'type': 'INTEGER',
                                                          'value': 5},
                                      'maxFreqUTRA-TDD-r10': {'type': 'INTEGER',
                                                              'value': 6},
                                      'maxFreqV2X-1-r14': {'type': 'INTEGER',
                                                           'value': 7},
                                      'maxFreqV2X-r14': {'type': 'INTEGER',
                                                         'value': 8},
                                      'maxGERAN-SI': {'type': 'INTEGER',
                                                      'value': 10},
                                      'maxGNFG': {'type': 'INTEGER',
                                                  'value': 16},
                                      'maxLCG-r13': {'type': 'INTEGER',
                                                     'value': 4},
                                      'maxLogMeasReport-r10': {'type': 'INTEGER',
                                                               'value': 520},
                                      'maxMBMS-ServiceListPerUE-r13': {'type': 'INTEGER',
                                                                       'value': 15},
                                      'maxMBSFN-Allocations': {'type': 'INTEGER',
                                                               'value': 8},
                                      'maxMBSFN-Area': {'type': 'INTEGER',
                                                        'value': 8},
                                      'maxMBSFN-Area-1': {'type': 'INTEGER',
                                                          'value': 7},
                                      'maxMeasId': {'type': 'INTEGER',
                                                    'value': 32},
                                      'maxMeasId-Plus1': {'type': 'INTEGER',
                                                          'value': 33},
                                      'maxMeasId-r12': {'type': 'INTEGER',
                                                        'value': 64},
                                      'maxMultiBands': {'type': 'INTEGER',
                                                        'value': 8},
                                      'maxNAICS-Entries-r12': {'type': 'INTEGER',
                                                               'value': 8},
                                      'maxNS-Pmax-r10': {'type': 'INTEGER',
                                                         'value': 8},
                                      'maxNeighCell-SCPTM-r13': {'type': 'INTEGER',
                                                                 'value': 8},
                                      'maxNeighCell-r12': {'type': 'INTEGER',
                                                           'value': 8},
                                      'maxObjectId': {'type': 'INTEGER',
                                                      'value': 32},
                                      'maxObjectId-Plus1-r13': {'type': 'INTEGER',
                                                                'value': 33},
                                      'maxObjectId-r13': {'type': 'INTEGER',
                                                          'value': 64},
                                      'maxP-a-PerNeighCell-r12': {'type': 'INTEGER',
                                                                  'value': 3},
                                      'maxPLMN-1-r14': {'type': 'INTEGER',
                                                        'value': 5},
                                      'maxPLMN-r11': {'type': 'INTEGER',
                                                      'value': 6},
                                      'maxPMCH-PerMBSFN': {'type': 'INTEGER',
                                                           'value': 15},
                                      'maxPNOffset': {'type': 'INTEGER',
                                                      'value': 511},
                                      'maxPSSCH-TxConfig-r14': {'type': 'INTEGER',
                                                                'value': 16},
                                      'maxPageRec': {'type': 'INTEGER',
                                                     'value': 16},
                                      'maxPhysCellIdRange-r9': {'type': 'INTEGER',
                                                                'value': 4},
                                      'maxQCI-r13': {'type': 'INTEGER',
                                                     'value': 6},
                                      'maxRAT-Capabilities': {'type': 'INTEGER',
                                                              'value': 8},
                                      'maxRE-MapQCL-r11': {'type': 'INTEGER',
                                                           'value': 4},
                                      'maxRSTD-Freq-r10': {'type': 'INTEGER',
                                                           'value': 3},
                                      'maxReportConfigId': {'type': 'INTEGER',
                                                            'value': 32},
                                      'maxReservationPeriod-r14': {'type': 'INTEGER',
                                                                   'value': 16},
                                      'maxSAI-MBMS-r11': {'type': 'INTEGER',
                                                          'value': 64},
                                      'maxSC-MTCH-BR-r14': {'type': 'INTEGER',
                                                            'value': 128},
                                      'maxSC-MTCH-r13': {'type': 'INTEGER',
                                                         'value': 1023},
                                      'maxSCell-r10': {'type': 'INTEGER',
                                                       'value': 4},
                                      'maxSCell-r13': {'type': 'INTEGER',
                                                       'value': 31},
                                      'maxSI-Message': {'type': 'INTEGER',
                                                        'value': 32},
                                      'maxSIB': {'type': 'INTEGER',
                                                 'value': 32},
                                      'maxSIB-1': {'type': 'INTEGER',
                                                   'value': 31},
                                      'maxSL-CommRxPoolNFreq-r13': {'type': 'INTEGER',
                                                                    'value': 32},
                                      'maxSL-CommRxPoolPreconf-v1310': {'type': 'INTEGER',
                                                                        'value': 12},
                                      'maxSL-CommTxPoolPreconf-v1310': {'type': 'INTEGER',
                                                                        'value': 7},
                                      'maxSL-Dest-r12': {'type': 'INTEGER',
                                                         'value': 16},
                                      'maxSL-DiscCells-r13': {'type': 'INTEGER',
                                                              'value': 16},
                                      'maxSL-DiscPowerClass-r12': {'type': 'INTEGER',
                                                                   'value': 3},
                                      'maxSL-DiscRxPoolPreconf-r13': {'type': 'INTEGER',
                                                                      'value': 16},
                                      'maxSL-DiscSysInfoReportFreq-r13': {'type': 'INTEGER',
                                                                          'value': 8},
                                      'maxSL-DiscTxPoolPreconf-r13': {'type': 'INTEGER',
                                                                      'value': 4},
                                      'maxSL-GP-r13': {'type': 'INTEGER',
                                                       'value': 8},
                                      'maxSL-PoolToMeasure-r14': {'type': 'INTEGER',
                                                                  'value': 72},
                                      'maxSL-Prio-r13': {'type': 'INTEGER',
                                                         'value': 8},
                                      'maxSL-RxPool-r12': {'type': 'INTEGER',
                                                           'value': 16},
                                      'maxSL-SyncConfig-r12': {'type': 'INTEGER',
                                                               'value': 16},
                                      'maxSL-TF-IndexPair-r12': {'type': 'INTEGER',
                                                                 'value': 64},
                                      'maxSL-TxPool-r12': {'type': 'INTEGER',
                                                           'value': 4},
                                      'maxSL-TxPool-r12Plus1-r13': {'type': 'INTEGER',
                                                                    'value': 5},
                                      'maxSL-TxPool-r13': {'type': 'INTEGER',
                                                           'value': 8},
                                      'maxSL-TxPool-v1310': {'type': 'INTEGER',
                                                             'value': 4},
                                      'maxSL-V2X-CBRConfig-1-r14': {'type': 'INTEGER',
                                                                    'value': 3},
                                      'maxSL-V2X-CBRConfig-r14': {'type': 'INTEGER',
                                                                  'value': 4},
                                      'maxSL-V2X-CBRConfig2-1-r14': {'type': 'INTEGER',
                                                                     'value': 7},
                                      'maxSL-V2X-CBRConfig2-r14': {'type': 'INTEGER',
                                                                   'value': 8},
                                      'maxSL-V2X-RxPool-r14': {'type': 'INTEGER',
                                                               'value': 16},
                                      'maxSL-V2X-RxPoolPreconf-r14': {'type': 'INTEGER',
                                                                      'value': 16},
                                      'maxSL-V2X-SyncConfig-r14': {'type': 'INTEGER',
                                                                   'value': 16},
                                      'maxSL-V2X-TxConfig-1-r14': {'type': 'INTEGER',
                                                                   'value': 63},
                                      'maxSL-V2X-TxConfig-r14': {'type': 'INTEGER',
                                                                 'value': 64},
                                      'maxSL-V2X-TxConfig2-1-r14': {'type': 'INTEGER',
                                                                    'value': 127},
                                      'maxSL-V2X-TxConfig2-r14': {'type': 'INTEGER',
                                                                  'value': 128},
                                      'maxSL-V2X-TxPool-r14': {'type': 'INTEGER',
                                                               'value': 8},
                                      'maxSL-V2X-TxPoolPreconf-r14': {'type': 'INTEGER',
                                                                      'value': 8},
                                      'maxSTAG-r11': {'type': 'INTEGER',
                                                      'value': 3},
                                      'maxServCell-r10': {'type': 'INTEGER',
                                                          'value': 5},
                                      'maxServCell-r13': {'type': 'INTEGER',
                                                          'value': 32},
                                      'maxServiceCount': {'type': 'INTEGER',
                                                          'value': 16},
                                      'maxServiceCount-1': {'type': 'INTEGER',
                                                            'value': 15},
                                      'maxSessionPerPMCH': {'type': 'INTEGER',
                                                            'value': 29},
                                      'maxSessionPerPMCH-1': {'type': 'INTEGER',
                                                              'value': 28},
                                      'maxSimultaneousBands-r10': {'type': 'INTEGER',
                                                                   'value': 64},
                                      'maxSubframePatternIDC-r11': {'type': 'INTEGER',
                                                                    'value': 8},
                                      'maxTrafficPattern-r14': {'type': 'INTEGER',
                                                                'value': 8},
                                      'maxUTRA-FDD-Carrier': {'type': 'INTEGER',
                                                              'value': 16},
                                      'maxUTRA-TDD-Carrier': {'type': 'INTEGER',
                                                              'value': 16},
                                      'maxWLAN-Bands-r13': {'type': 'INTEGER',
                                                            'value': 8},
                                      'maxWLAN-CarrierInfo-r13': {'type': 'INTEGER',
                                                                  'value': 8},
                                      'maxWLAN-Channels-r13': {'type': 'INTEGER',
                                                               'value': 16},
                                      'maxWLAN-Id-Report-r14': {'type': 'INTEGER',
                                                                'value': 32},
                                      'maxWLAN-Id-r12': {'type': 'INTEGER',
                                                         'value': 16},
                                      'maxWLAN-Id-r13': {'type': 'INTEGER',
                                                         'value': 32},
                                      'minCSI-IM-r13': {'type': 'INTEGER',
                                                        'value': 5},
                                      'minCSI-RS-NZP-r13': {'type': 'INTEGER',
                                                            'value': 4}}},
 'EUTRA-Sidelink-Preconf': {'extensibility-implied': False,
                            'imports': {'EUTRA-RRC-Definitions': ['AdditionalSpectrumEmission',
                                                                  'AdditionalSpectrumEmission-v10l0',
                                                                  'ARFCN-ValueEUTRA-r9',
                                                                  'FilterCoefficient',
                                                                  'maxCBR-Level-r14',
                                                                  'maxCBR-Level-1-r14',
                                                                  'maxFreq',
                                                                  'maxFreqV2X-r14',
                                                                  'maxSL-TxPool-r12',
                                                                  'maxSL-CommRxPoolPreconf-v1310',
                                                                  'maxSL-CommTxPoolPreconf-v1310',
                                                                  'maxSL-DiscRxPoolPreconf-r13',
                                                                  'maxSL-DiscTxPoolPreconf-r13',
                                                                  'maxSL-V2X-CBRConfig2-r14',
                                                                  'maxSL-V2X-CBRConfig2-1-r14',
                                                                  'maxSL-V2X-RxPoolPreconf-r14',
                                                                  'maxSL-V2X-TxConfig2-r14',
                                                                  'maxSL-V2X-TxConfig2-1-r14',
                                                                  'maxSL-V2X-TxPoolPreconf-r14',
                                                                  'P-Max',
                                                                  'ReselectionInfoRelay-r13',
                                                                  'SL-AnchorCarrierFreqList-V2X-r14',
                                                                  'SL-CBR-Levels-Config-r14',
                                                                  'SL-CBR-PSSCH-TxConfig-r14',
                                                                  'SL-CommTxPoolSensingConfig-r14',
                                                                  'SL-CP-Len-r12',
                                                                  'SL-HoppingConfigComm-r12',
                                                                  'SL-OffsetIndicator-r12',
                                                                  'SL-OffsetIndicatorSync-r12',
                                                                  'SL-OffsetIndicatorSync-v1430',
                                                                  'SL-PeriodComm-r12',
                                                                  'RSRP-RangeSL3-r12',
                                                                  'SL-PriorityList-r13',
                                                                  'SL-TF-ResourceConfig-r12',
                                                                  'SL-TRPT-Subset-r12',
                                                                  'SL-TxParameters-r12',
                                                                  'SL-ZoneConfig-r14',
                                                                  'P0-SL-r12',
                                                                  'TDD-ConfigSL-r12',
                                                                  'SubframeBitmapSL-r14',
                                                                  'SL-P2X-ResourceSelectionConfig-r14',
                                                                  'SL-RestrictResourceReservationPeriodList-r14',
                                                                  'SL-SyncAllowed-r14',
                                                                  'SL-OffsetIndicatorSync-r14',
                                                                  'SL-Priority-r13']},
                            'object-classes': {},
                            'object-sets': {},
                            'tags': 'AUTOMATIC',
                            'types': {'SL-CBR-PPPP-TxPreconfigList-r14': {'element': {'type': 'SL-PPPP-TxPreconfigIndex-r14'},
                                                                          'size': [(1,
                                                                                    8)],
                                                                          'type': 'SEQUENCE '
                                                                                  'OF'},
                                      'SL-CBR-PreconfigTxConfigList-r14': {'members': [{'element': {'type': 'SL-CBR-Levels-Config-r14'},
                                                                                        'name': 'cbr-RangeCommonConfigList-r14',
                                                                                        'size': [(1,
                                                                                                  'maxSL-V2X-CBRConfig2-r14')],
                                                                                        'type': 'SEQUENCE '
                                                                                                'OF'},
                                                                                       {'element': {'type': 'SL-CBR-PSSCH-TxConfig-r14'},
                                                                                        'name': 'sl-CBR-PSSCH-TxConfigList-r14',
                                                                                        'size': [(1,
                                                                                                  'maxSL-V2X-TxConfig2-r14')],
                                                                                        'type': 'SEQUENCE '
                                                                                                'OF'}],
                                                                           'type': 'SEQUENCE'},
                                      'SL-PPPP-TxPreconfigIndex-r14': {'members': [{'name': 'priorityThreshold-r14',
                                                                                    'type': 'SL-Priority-r13'},
                                                                                   {'name': 'defaultTxConfigIndex-r14',
                                                                                    'restricted-to': [(0,
                                                                                                       'maxCBR-Level-1-r14')],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'cbr-ConfigIndex-r14',
                                                                                    'restricted-to': [(0,
                                                                                                       'maxSL-V2X-CBRConfig2-1-r14')],
                                                                                    'type': 'INTEGER'},
                                                                                   {'element': {'type': 'Tx-PreconfigIndex-r14'},
                                                                                    'name': 'tx-ConfigIndexList-r14',
                                                                                    'size': [(1,
                                                                                              'maxCBR-Level-r14')],
                                                                                    'type': 'SEQUENCE '
                                                                                            'OF'}],
                                                                       'type': 'SEQUENCE'},
                                      'SL-PreconfigCommPool-r12': {'members': [{'name': 'sc-CP-Len-r12',
                                                                                'type': 'SL-CP-Len-r12'},
                                                                               {'name': 'sc-Period-r12',
                                                                                'type': 'SL-PeriodComm-r12'},
                                                                               {'name': 'sc-TF-ResourceConfig-r12',
                                                                                'type': 'SL-TF-ResourceConfig-r12'},
                                                                               {'name': 'sc-TxParameters-r12',
                                                                                'type': 'P0-SL-r12'},
                                                                               {'name': 'data-CP-Len-r12',
                                                                                'type': 'SL-CP-Len-r12'},
                                                                               {'name': 'data-TF-ResourceConfig-r12',
                                                                                'type': 'SL-TF-ResourceConfig-r12'},
                                                                               {'name': 'dataHoppingConfig-r12',
                                                                                'type': 'SL-HoppingConfigComm-r12'},
                                                                               {'name': 'dataTxParameters-r12',
                                                                                'type': 'P0-SL-r12'},
                                                                               {'name': 'trpt-Subset-r12',
                                                                                'type': 'SL-TRPT-Subset-r12'},
                                                                               None,
                                                                               [{'name': 'priorityList-r13',
                                                                                 'optional': True,
                                                                                 'type': 'SL-PriorityList-r13'}]],
                                                                   'type': 'SEQUENCE'},
                                      'SL-PreconfigCommPoolList4-r12': {'element': {'type': 'SL-PreconfigCommPool-r12'},
                                                                        'size': [(1,
                                                                                  'maxSL-TxPool-r12')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                      'SL-PreconfigCommRxPoolList-r13': {'element': {'type': 'SL-PreconfigCommPool-r12'},
                                                                         'size': [(1,
                                                                                   'maxSL-CommRxPoolPreconf-v1310')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                      'SL-PreconfigCommTxPoolList-r13': {'element': {'type': 'SL-PreconfigCommPool-r12'},
                                                                         'size': [(1,
                                                                                   'maxSL-CommTxPoolPreconf-v1310')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                      'SL-PreconfigDiscPool-r13': {'members': [{'name': 'cp-Len-r13',
                                                                                'type': 'SL-CP-Len-r12'},
                                                                               {'name': 'discPeriod-r13',
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('rf4',
                                                                                            0),
                                                                                           ('rf6',
                                                                                            1),
                                                                                           ('rf7',
                                                                                            2),
                                                                                           ('rf8',
                                                                                            3),
                                                                                           ('rf12',
                                                                                            4),
                                                                                           ('rf14',
                                                                                            5),
                                                                                           ('rf16',
                                                                                            6),
                                                                                           ('rf24',
                                                                                            7),
                                                                                           ('rf28',
                                                                                            8),
                                                                                           ('rf32',
                                                                                            9),
                                                                                           ('rf64',
                                                                                            10),
                                                                                           ('rf128',
                                                                                            11),
                                                                                           ('rf256',
                                                                                            12),
                                                                                           ('rf512',
                                                                                            13),
                                                                                           ('rf1024',
                                                                                            14),
                                                                                           ('spare',
                                                                                            15)]},
                                                                               {'name': 'numRetx-r13',
                                                                                'restricted-to': [(0,
                                                                                                   3)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'numRepetition-r13',
                                                                                'restricted-to': [(1,
                                                                                                   50)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'tf-ResourceConfig-r13',
                                                                                'type': 'SL-TF-ResourceConfig-r12'},
                                                                               {'members': [{'name': 'txParametersGeneral-r13',
                                                                                             'type': 'P0-SL-r12'},
                                                                                            {'name': 'txProbability-r13',
                                                                                             'type': 'ENUMERATED',
                                                                                             'values': [('p25',
                                                                                                         0),
                                                                                                        ('p50',
                                                                                                         1),
                                                                                                        ('p75',
                                                                                                         2),
                                                                                                        ('p100',
                                                                                                         3)]}],
                                                                                'name': 'txParameters-r13',
                                                                                'optional': True,
                                                                                'type': 'SEQUENCE'},
                                                                               None],
                                                                   'type': 'SEQUENCE'},
                                      'SL-PreconfigDiscRxPoolList-r13': {'element': {'type': 'SL-PreconfigDiscPool-r13'},
                                                                         'size': [(1,
                                                                                   'maxSL-DiscRxPoolPreconf-r13')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                      'SL-PreconfigDiscTxPoolList-r13': {'element': {'type': 'SL-PreconfigDiscPool-r13'},
                                                                         'size': [(1,
                                                                                   'maxSL-DiscTxPoolPreconf-r13')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                      'SL-PreconfigGeneral-r12': {'members': [{'members': [{'name': 'profile0x0001-r12',
                                                                                            'type': 'BOOLEAN'},
                                                                                           {'name': 'profile0x0002-r12',
                                                                                            'type': 'BOOLEAN'},
                                                                                           {'name': 'profile0x0004-r12',
                                                                                            'type': 'BOOLEAN'},
                                                                                           {'name': 'profile0x0006-r12',
                                                                                            'type': 'BOOLEAN'},
                                                                                           {'name': 'profile0x0101-r12',
                                                                                            'type': 'BOOLEAN'},
                                                                                           {'name': 'profile0x0102-r12',
                                                                                            'type': 'BOOLEAN'},
                                                                                           {'name': 'profile0x0104-r12',
                                                                                            'type': 'BOOLEAN'}],
                                                                               'name': 'rohc-Profiles-r12',
                                                                               'type': 'SEQUENCE'},
                                                                              {'name': 'carrierFreq-r12',
                                                                               'type': 'ARFCN-ValueEUTRA-r9'},
                                                                              {'name': 'maxTxPower-r12',
                                                                               'type': 'P-Max'},
                                                                              {'name': 'additionalSpectrumEmission-r12',
                                                                               'type': 'AdditionalSpectrumEmission'},
                                                                              {'name': 'sl-bandwidth-r12',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n6',
                                                                                           0),
                                                                                          ('n15',
                                                                                           1),
                                                                                          ('n25',
                                                                                           2),
                                                                                          ('n50',
                                                                                           3),
                                                                                          ('n75',
                                                                                           4),
                                                                                          ('n100',
                                                                                           5)]},
                                                                              {'name': 'tdd-ConfigSL-r12',
                                                                               'type': 'TDD-ConfigSL-r12'},
                                                                              {'name': 'reserved-r12',
                                                                               'size': [19],
                                                                               'type': 'BIT '
                                                                                       'STRING'},
                                                                              None,
                                                                              [{'name': 'additionalSpectrumEmission-v1440',
                                                                                'optional': True,
                                                                                'type': 'AdditionalSpectrumEmission-v10l0'}]],
                                                                  'type': 'SEQUENCE'},
                                      'SL-PreconfigRelay-r13': {'members': [{'name': 'reselectionInfoOoC-r13',
                                                                             'type': 'ReselectionInfoRelay-r13'}],
                                                                'type': 'SEQUENCE'},
                                      'SL-PreconfigSync-r12': {'members': [{'name': 'syncCP-Len-r12',
                                                                            'type': 'SL-CP-Len-r12'},
                                                                           {'name': 'syncOffsetIndicator1-r12',
                                                                            'type': 'SL-OffsetIndicatorSync-r12'},
                                                                           {'name': 'syncOffsetIndicator2-r12',
                                                                            'type': 'SL-OffsetIndicatorSync-r12'},
                                                                           {'name': 'syncTxParameters-r12',
                                                                            'type': 'P0-SL-r12'},
                                                                           {'name': 'syncTxThreshOoC-r12',
                                                                            'type': 'RSRP-RangeSL3-r12'},
                                                                           {'name': 'filterCoefficient-r12',
                                                                            'type': 'FilterCoefficient'},
                                                                           {'name': 'syncRefMinHyst-r12',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('dB0',
                                                                                        0),
                                                                                       ('dB3',
                                                                                        1),
                                                                                       ('dB6',
                                                                                        2),
                                                                                       ('dB9',
                                                                                        3),
                                                                                       ('dB12',
                                                                                        4)]},
                                                                           {'name': 'syncRefDiffHyst-r12',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('dB0',
                                                                                        0),
                                                                                       ('dB3',
                                                                                        1),
                                                                                       ('dB6',
                                                                                        2),
                                                                                       ('dB9',
                                                                                        3),
                                                                                       ('dB12',
                                                                                        4),
                                                                                       ('dBinf',
                                                                                        5)]},
                                                                           None,
                                                                           [{'name': 'syncTxPeriodic-r13',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('true',
                                                                                         0)]}]],
                                                               'type': 'SEQUENCE'},
                                      'SL-PreconfigV2X-RxPoolList-r14': {'element': {'type': 'SL-V2X-PreconfigCommPool-r14'},
                                                                         'size': [(1,
                                                                                   'maxSL-V2X-RxPoolPreconf-r14')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                      'SL-PreconfigV2X-Sync-r14': {'members': [{'name': 'syncOffsetIndicators-r14',
                                                                                'type': 'SL-V2X-SyncOffsetIndicators-r14'},
                                                                               {'name': 'syncTxParameters-r14',
                                                                                'type': 'P0-SL-r12'},
                                                                               {'name': 'syncTxThreshOoC-r14',
                                                                                'type': 'RSRP-RangeSL3-r12'},
                                                                               {'name': 'filterCoefficient-r14',
                                                                                'type': 'FilterCoefficient'},
                                                                               {'name': 'syncRefMinHyst-r14',
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('dB0',
                                                                                            0),
                                                                                           ('dB3',
                                                                                            1),
                                                                                           ('dB6',
                                                                                            2),
                                                                                           ('dB9',
                                                                                            3),
                                                                                           ('dB12',
                                                                                            4)]},
                                                                               {'name': 'syncRefDiffHyst-r14',
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('dB0',
                                                                                            0),
                                                                                           ('dB3',
                                                                                            1),
                                                                                           ('dB6',
                                                                                            2),
                                                                                           ('dB9',
                                                                                            3),
                                                                                           ('dB12',
                                                                                            4),
                                                                                           ('dBinf',
                                                                                            5)]},
                                                                               None],
                                                                   'type': 'SEQUENCE'},
                                      'SL-PreconfigV2X-TxPoolList-r14': {'element': {'type': 'SL-V2X-PreconfigCommPool-r14'},
                                                                         'size': [(1,
                                                                                   'maxSL-V2X-TxPoolPreconf-r14')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                      'SL-Preconfiguration-r12': {'members': [{'name': 'preconfigGeneral-r12',
                                                                               'type': 'SL-PreconfigGeneral-r12'},
                                                                              {'name': 'preconfigSync-r12',
                                                                               'type': 'SL-PreconfigSync-r12'},
                                                                              {'name': 'preconfigComm-r12',
                                                                               'type': 'SL-PreconfigCommPoolList4-r12'},
                                                                              None,
                                                                              [{'members': [{'name': 'commRxPoolList-r13',
                                                                                             'type': 'SL-PreconfigCommRxPoolList-r13'},
                                                                                            {'name': 'commTxPoolList-r13',
                                                                                             'optional': True,
                                                                                             'type': 'SL-PreconfigCommTxPoolList-r13'}],
                                                                                'name': 'preconfigComm-v1310',
                                                                                'optional': True,
                                                                                'type': 'SEQUENCE'},
                                                                               {'members': [{'name': 'discRxPoolList-r13',
                                                                                             'type': 'SL-PreconfigDiscRxPoolList-r13'},
                                                                                            {'name': 'discTxPoolList-r13',
                                                                                             'optional': True,
                                                                                             'type': 'SL-PreconfigDiscTxPoolList-r13'}],
                                                                                'name': 'preconfigDisc-r13',
                                                                                'optional': True,
                                                                                'type': 'SEQUENCE'},
                                                                               {'name': 'preconfigRelay-r13',
                                                                                'optional': True,
                                                                                'type': 'SL-PreconfigRelay-r13'}]],
                                                                  'type': 'SEQUENCE'},
                                      'SL-V2X-PreconfigCommPool-r14': {'members': [{'name': 'sl-OffsetIndicator-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-OffsetIndicator-r12'},
                                                                                   {'name': 'sl-Subframe-r14',
                                                                                    'type': 'SubframeBitmapSL-r14'},
                                                                                   {'name': 'adjacencyPSCCH-PSSCH-r14',
                                                                                    'type': 'BOOLEAN'},
                                                                                   {'name': 'sizeSubchannel-r14',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('n4',
                                                                                                0),
                                                                                               ('n5',
                                                                                                1),
                                                                                               ('n6',
                                                                                                2),
                                                                                               ('n8',
                                                                                                3),
                                                                                               ('n9',
                                                                                                4),
                                                                                               ('n10',
                                                                                                5),
                                                                                               ('n12',
                                                                                                6),
                                                                                               ('n15',
                                                                                                7),
                                                                                               ('n16',
                                                                                                8),
                                                                                               ('n18',
                                                                                                9),
                                                                                               ('n20',
                                                                                                10),
                                                                                               ('n25',
                                                                                                11),
                                                                                               ('n30',
                                                                                                12),
                                                                                               ('n48',
                                                                                                13),
                                                                                               ('n50',
                                                                                                14),
                                                                                               ('n72',
                                                                                                15),
                                                                                               ('n75',
                                                                                                16),
                                                                                               ('n96',
                                                                                                17),
                                                                                               ('n100',
                                                                                                18),
                                                                                               ('spare13',
                                                                                                19),
                                                                                               ('spare12',
                                                                                                20),
                                                                                               ('spare11',
                                                                                                21),
                                                                                               ('spare10',
                                                                                                22),
                                                                                               ('spare9',
                                                                                                23),
                                                                                               ('spare8',
                                                                                                24),
                                                                                               ('spare7',
                                                                                                25),
                                                                                               ('spare6',
                                                                                                26),
                                                                                               ('spare5',
                                                                                                27),
                                                                                               ('spare4',
                                                                                                28),
                                                                                               ('spare3',
                                                                                                29),
                                                                                               ('spare2',
                                                                                                30),
                                                                                               ('spare1',
                                                                                                31)]},
                                                                                   {'name': 'numSubchannel-r14',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('n1',
                                                                                                0),
                                                                                               ('n3',
                                                                                                1),
                                                                                               ('n5',
                                                                                                2),
                                                                                               ('n8',
                                                                                                3),
                                                                                               ('n10',
                                                                                                4),
                                                                                               ('n15',
                                                                                                5),
                                                                                               ('n20',
                                                                                                6),
                                                                                               ('spare1',
                                                                                                7)]},
                                                                                   {'name': 'startRB-Subchannel-r14',
                                                                                    'restricted-to': [(0,
                                                                                                       99)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'startRB-PSCCH-Pool-r14',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(0,
                                                                                                       99)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'dataTxParameters-r14',
                                                                                    'type': 'P0-SL-r12'},
                                                                                   {'name': 'zoneID-r14',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(0,
                                                                                                       7)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'threshS-RSSI-CBR-r14',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(0,
                                                                                                       45)],
                                                                                    'type': 'INTEGER'},
                                                                                   {'name': 'cbr-pssch-TxConfigList-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-CBR-PPPP-TxPreconfigList-r14'},
                                                                                   {'name': 'resourceSelectionConfigP2X-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-P2X-ResourceSelectionConfig-r14'},
                                                                                   {'name': 'syncAllowed-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-SyncAllowed-r14'},
                                                                                   {'name': 'restrictResourceReservationPeriod-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-RestrictResourceReservationPeriodList-r14'},
                                                                                   None],
                                                                       'type': 'SEQUENCE'},
                                      'SL-V2X-PreconfigFreqInfo-r14': {'members': [{'name': 'v2x-CommPreconfigGeneral-r14',
                                                                                    'type': 'SL-PreconfigGeneral-r12'},
                                                                                   {'name': 'v2x-CommPreconfigSync-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-PreconfigV2X-Sync-r14'},
                                                                                   {'name': 'v2x-CommRxPoolList-r14',
                                                                                    'type': 'SL-PreconfigV2X-RxPoolList-r14'},
                                                                                   {'name': 'v2x-CommTxPoolList-r14',
                                                                                    'type': 'SL-PreconfigV2X-TxPoolList-r14'},
                                                                                   {'name': 'p2x-CommTxPoolList-r14',
                                                                                    'type': 'SL-PreconfigV2X-TxPoolList-r14'},
                                                                                   {'name': 'v2x-ResourceSelectionConfig-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-CommTxPoolSensingConfig-r14'},
                                                                                   {'name': 'zoneConfig-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-ZoneConfig-r14'},
                                                                                   {'name': 'syncPriority-r14',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('gnss',
                                                                                                0),
                                                                                               ('enb',
                                                                                                1)]},
                                                                                   {'name': 'thresSL-TxPrioritization-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SL-Priority-r13'},
                                                                                   {'name': 'offsetDFN-r14',
                                                                                    'optional': True,
                                                                                    'restricted-to': [(0,
                                                                                                       1000)],
                                                                                    'type': 'INTEGER'},
                                                                                   None],
                                                                       'type': 'SEQUENCE'},
                                      'SL-V2X-PreconfigFreqList-r14': {'element': {'type': 'SL-V2X-PreconfigFreqInfo-r14'},
                                                                       'size': [(1,
                                                                                 'maxFreqV2X-r14')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                      'SL-V2X-Preconfiguration-r14': {'members': [{'name': 'v2x-PreconfigFreqList-r14',
                                                                                   'type': 'SL-V2X-PreconfigFreqList-r14'},
                                                                                  {'name': 'anchorCarrierFreqList-r14',
                                                                                   'optional': True,
                                                                                   'type': 'SL-AnchorCarrierFreqList-V2X-r14'},
                                                                                  {'name': 'cbr-PreconfigList-r14',
                                                                                   'optional': True,
                                                                                   'type': 'SL-CBR-PreconfigTxConfigList-r14'},
                                                                                  None],
                                                                      'type': 'SEQUENCE'},
                                      'SL-V2X-SyncOffsetIndicators-r14': {'members': [{'name': 'syncOffsetIndicator1-r14',
                                                                                       'type': 'SL-OffsetIndicatorSync-r14'},
                                                                                      {'name': 'syncOffsetIndicator2-r14',
                                                                                       'type': 'SL-OffsetIndicatorSync-r14'},
                                                                                      {'name': 'syncOffsetIndicator3-r14',
                                                                                       'optional': True,
                                                                                       'type': 'SL-OffsetIndicatorSync-r14'}],
                                                                          'type': 'SEQUENCE'},
                                      'Tx-PreconfigIndex-r14': {'restricted-to': [(0,
                                                                                   'maxSL-V2X-TxConfig2-1-r14')],
                                                                'type': 'INTEGER'}},
                            'values': {}},
 'EUTRA-UE-Variables': {'extensibility-implied': False,
                        'imports': {'EUTRA-RRC-Definitions': ['AbsoluteTimeInfo-r10',
                                                              'AreaConfiguration-r10',
                                                              'AreaConfiguration-v1130',
                                                              'CarrierFreqGERAN',
                                                              'CellIdentity',
                                                              'ConnEstFailReport-r11',
                                                              'SpeedStateScaleFactors',
                                                              'C-RNTI',
                                                              'LoggingDuration-r10',
                                                              'LoggingInterval-r10',
                                                              'LogMeasInfo-r10',
                                                              'MeasCSI-RS-Id-r12',
                                                              'MeasId',
                                                              'MeasId-v1250',
                                                              'MeasIdToAddModList',
                                                              'MeasIdToAddModListExt-r12',
                                                              'MeasIdToAddModList-v1310',
                                                              'MeasIdToAddModListExt-v1310',
                                                              'MeasObjectToAddModList',
                                                              'MeasObjectToAddModList-v9e0',
                                                              'MeasObjectToAddModListExt-r13',
                                                              'MeasScaleFactor-r12',
                                                              'MobilityStateParameters',
                                                              'NeighCellConfig',
                                                              'PhysCellId',
                                                              'PhysCellIdCDMA2000',
                                                              'PhysCellIdGERAN',
                                                              'PhysCellIdUTRA-FDD',
                                                              'PhysCellIdUTRA-TDD',
                                                              'PLMN-Identity',
                                                              'PLMN-IdentityList3-r11',
                                                              'QuantityConfig',
                                                              'ReportConfigToAddModList',
                                                              'RLF-Report-r9',
                                                              'TargetMBSFN-AreaList-r12',
                                                              'TraceReference-r10',
                                                              'Tx-ResourcePoolMeasList-r14',
                                                              'VisitedCellInfoList-r12',
                                                              'maxCellMeas',
                                                              'maxCSI-RS-Meas-r12',
                                                              'maxMeasId',
                                                              'maxMeasId-r12',
                                                              'UL-DelayConfig-r13',
                                                              'WLAN-CarrierInfo-r13',
                                                              'WLAN-Identifiers-r12',
                                                              'WLAN-Id-List-r13',
                                                              'WLAN-Status-r13',
                                                              'WLAN-Status-v1430',
                                                              'WLAN-SuspendConfig-r14']},
                        'object-classes': {},
                        'object-sets': {},
                        'tags': 'AUTOMATIC',
                        'types': {'CSI-RS-TriggeredList-r12': {'element': {'type': 'MeasCSI-RS-Id-r12'},
                                                               'size': [(1,
                                                                         'maxCSI-RS-Meas-r12')],
                                                               'type': 'SEQUENCE '
                                                                       'OF'},
                                  'CellsTriggeredList': {'element': {'members': [{'name': 'physCellIdEUTRA',
                                                                                  'type': 'PhysCellId'},
                                                                                 {'members': [{'name': 'fdd',
                                                                                               'type': 'PhysCellIdUTRA-FDD'},
                                                                                              {'name': 'tdd',
                                                                                               'type': 'PhysCellIdUTRA-TDD'}],
                                                                                  'name': 'physCellIdUTRA',
                                                                                  'type': 'CHOICE'},
                                                                                 {'members': [{'name': 'carrierFreq',
                                                                                               'type': 'CarrierFreqGERAN'},
                                                                                              {'name': 'physCellId',
                                                                                               'type': 'PhysCellIdGERAN'}],
                                                                                  'name': 'physCellIdGERAN',
                                                                                  'type': 'SEQUENCE'},
                                                                                 {'name': 'physCellIdCDMA2000',
                                                                                  'type': 'PhysCellIdCDMA2000'},
                                                                                 {'name': 'wlan-Identifiers-r13',
                                                                                  'type': 'WLAN-Identifiers-r12'}],
                                                                     'type': 'CHOICE'},
                                                         'size': [(1,
                                                                   'maxCellMeas')],
                                                         'type': 'SEQUENCE OF'},
                                  'LogMeasInfoList2-r10': {'element': {'type': 'LogMeasInfo-r10'},
                                                           'size': [(1,
                                                                     'maxLogMeas-r10')],
                                                           'type': 'SEQUENCE '
                                                                   'OF'},
                                  'VarConnEstFailReport-r11': {'members': [{'name': 'connEstFailReport-r11',
                                                                            'type': 'ConnEstFailReport-r11'},
                                                                           {'name': 'plmn-Identity-r11',
                                                                            'type': 'PLMN-Identity'}],
                                                               'type': 'SEQUENCE'},
                                  'VarLogMeasConfig-r10': {'members': [{'name': 'areaConfiguration-r10',
                                                                        'optional': True,
                                                                        'type': 'AreaConfiguration-r10'},
                                                                       {'name': 'loggingDuration-r10',
                                                                        'type': 'LoggingDuration-r10'},
                                                                       {'name': 'loggingInterval-r10',
                                                                        'type': 'LoggingInterval-r10'}],
                                                           'type': 'SEQUENCE'},
                                  'VarLogMeasConfig-r11': {'members': [{'name': 'areaConfiguration-r10',
                                                                        'optional': True,
                                                                        'type': 'AreaConfiguration-r10'},
                                                                       {'name': 'areaConfiguration-v1130',
                                                                        'optional': True,
                                                                        'type': 'AreaConfiguration-v1130'},
                                                                       {'name': 'loggingDuration-r10',
                                                                        'type': 'LoggingDuration-r10'},
                                                                       {'name': 'loggingInterval-r10',
                                                                        'type': 'LoggingInterval-r10'}],
                                                           'type': 'SEQUENCE'},
                                  'VarLogMeasConfig-r12': {'members': [{'name': 'areaConfiguration-r10',
                                                                        'optional': True,
                                                                        'type': 'AreaConfiguration-r10'},
                                                                       {'name': 'areaConfiguration-v1130',
                                                                        'optional': True,
                                                                        'type': 'AreaConfiguration-v1130'},
                                                                       {'name': 'loggingDuration-r10',
                                                                        'type': 'LoggingDuration-r10'},
                                                                       {'name': 'loggingInterval-r10',
                                                                        'type': 'LoggingInterval-r10'},
                                                                       {'name': 'targetMBSFN-AreaList-r12',
                                                                        'optional': True,
                                                                        'type': 'TargetMBSFN-AreaList-r12'}],
                                                           'type': 'SEQUENCE'},
                                  'VarLogMeasReport-r10': {'members': [{'name': 'traceReference-r10',
                                                                        'type': 'TraceReference-r10'},
                                                                       {'name': 'traceRecordingSessionRef-r10',
                                                                        'size': [2],
                                                                        'type': 'OCTET '
                                                                                'STRING'},
                                                                       {'name': 'tce-Id-r10',
                                                                        'size': [1],
                                                                        'type': 'OCTET '
                                                                                'STRING'},
                                                                       {'name': 'plmn-Identity-r10',
                                                                        'type': 'PLMN-Identity'},
                                                                       {'name': 'absoluteTimeInfo-r10',
                                                                        'type': 'AbsoluteTimeInfo-r10'},
                                                                       {'name': 'logMeasInfoList-r10',
                                                                        'type': 'LogMeasInfoList2-r10'}],
                                                           'type': 'SEQUENCE'},
                                  'VarLogMeasReport-r11': {'members': [{'name': 'traceReference-r10',
                                                                        'type': 'TraceReference-r10'},
                                                                       {'name': 'traceRecordingSessionRef-r10',
                                                                        'size': [2],
                                                                        'type': 'OCTET '
                                                                                'STRING'},
                                                                       {'name': 'tce-Id-r10',
                                                                        'size': [1],
                                                                        'type': 'OCTET '
                                                                                'STRING'},
                                                                       {'name': 'plmn-IdentityList-r11',
                                                                        'type': 'PLMN-IdentityList3-r11'},
                                                                       {'name': 'absoluteTimeInfo-r10',
                                                                        'type': 'AbsoluteTimeInfo-r10'},
                                                                       {'name': 'logMeasInfoList-r10',
                                                                        'type': 'LogMeasInfoList2-r10'}],
                                                           'type': 'SEQUENCE'},
                                  'VarMeasConfig': {'members': [{'name': 'measIdList',
                                                                 'optional': True,
                                                                 'type': 'MeasIdToAddModList'},
                                                                {'name': 'measIdListExt-r12',
                                                                 'optional': True,
                                                                 'type': 'MeasIdToAddModListExt-r12'},
                                                                {'name': 'measIdList-v1310',
                                                                 'optional': True,
                                                                 'type': 'MeasIdToAddModList-v1310'},
                                                                {'name': 'measIdListExt-v1310',
                                                                 'optional': True,
                                                                 'type': 'MeasIdToAddModListExt-v1310'},
                                                                {'name': 'measObjectList',
                                                                 'optional': True,
                                                                 'type': 'MeasObjectToAddModList'},
                                                                {'name': 'measObjectListExt-r13',
                                                                 'optional': True,
                                                                 'type': 'MeasObjectToAddModListExt-r13'},
                                                                {'name': 'measObjectList-v9i0',
                                                                 'optional': True,
                                                                 'type': 'MeasObjectToAddModList-v9e0'},
                                                                {'name': 'reportConfigList',
                                                                 'optional': True,
                                                                 'type': 'ReportConfigToAddModList'},
                                                                {'name': 'quantityConfig',
                                                                 'optional': True,
                                                                 'type': 'QuantityConfig'},
                                                                {'name': 'measScaleFactor-r12',
                                                                 'optional': True,
                                                                 'type': 'MeasScaleFactor-r12'},
                                                                {'name': 's-Measure',
                                                                 'optional': True,
                                                                 'restricted-to': [(-140,
                                                                                    -44)],
                                                                 'type': 'INTEGER'},
                                                                {'members': [{'name': 'release',
                                                                              'type': 'NULL'},
                                                                             {'members': [{'name': 'mobilityStateParameters',
                                                                                           'type': 'MobilityStateParameters'},
                                                                                          {'name': 'timeToTrigger-SF',
                                                                                           'type': 'SpeedStateScaleFactors'}],
                                                                              'name': 'setup',
                                                                              'type': 'SEQUENCE'}],
                                                                 'name': 'speedStatePars',
                                                                 'optional': True,
                                                                 'type': 'CHOICE'},
                                                                {'name': 'allowInterruptions-r11',
                                                                 'optional': True,
                                                                 'type': 'BOOLEAN'}],
                                                    'type': 'SEQUENCE'},
                                  'VarMeasReport': {'members': [{'name': 'measId',
                                                                 'type': 'MeasId'},
                                                                {'name': 'measId-v1250',
                                                                 'optional': True,
                                                                 'type': 'MeasId-v1250'},
                                                                {'name': 'cellsTriggeredList',
                                                                 'optional': True,
                                                                 'type': 'CellsTriggeredList'},
                                                                {'name': 'csi-RS-TriggeredList-r12',
                                                                 'optional': True,
                                                                 'type': 'CSI-RS-TriggeredList-r12'},
                                                                {'name': 'poolsTriggeredList-r14',
                                                                 'optional': True,
                                                                 'type': 'Tx-ResourcePoolMeasList-r14'},
                                                                {'name': 'numberOfReportsSent',
                                                                 'type': 'INTEGER'}],
                                                    'type': 'SEQUENCE'},
                                  'VarMeasReportList': {'element': {'type': 'VarMeasReport'},
                                                        'size': [(1,
                                                                  'maxMeasId')],
                                                        'type': 'SEQUENCE OF'},
                                  'VarMeasReportList-r12': {'element': {'type': 'VarMeasReport'},
                                                            'size': [(1,
                                                                      'maxMeasId-r12')],
                                                            'type': 'SEQUENCE '
                                                                    'OF'},
                                  'VarMobilityHistoryReport-r12': {'type': 'VisitedCellInfoList-r12'},
                                  'VarRLF-Report-r10': {'members': [{'name': 'rlf-Report-r10',
                                                                     'type': 'RLF-Report-r9'},
                                                                    {'name': 'plmn-Identity-r10',
                                                                     'type': 'PLMN-Identity'}],
                                                        'type': 'SEQUENCE'},
                                  'VarRLF-Report-r11': {'members': [{'name': 'rlf-Report-r10',
                                                                     'type': 'RLF-Report-r9'},
                                                                    {'name': 'plmn-IdentityList-r11',
                                                                     'type': 'PLMN-IdentityList3-r11'}],
                                                        'type': 'SEQUENCE'},
                                  'VarShortMAC-Input': {'members': [{'name': 'cellIdentity',
                                                                     'type': 'CellIdentity'},
                                                                    {'name': 'physCellId',
                                                                     'type': 'PhysCellId'},
                                                                    {'name': 'c-RNTI',
                                                                     'type': 'C-RNTI'}],
                                                        'type': 'SEQUENCE'},
                                  'VarShortResumeMAC-Input-r13': {'members': [{'name': 'cellIdentity-r13',
                                                                               'type': 'CellIdentity'},
                                                                              {'name': 'physCellId-r13',
                                                                               'type': 'PhysCellId'},
                                                                              {'name': 'c-RNTI-r13',
                                                                               'type': 'C-RNTI'},
                                                                              {'name': 'resumeDiscriminator-r13',
                                                                               'size': [1],
                                                                               'type': 'BIT '
                                                                                       'STRING'}],
                                                                  'type': 'SEQUENCE'},
                                  'VarWLAN-MobilityConfig': {'members': [{'name': 'wlan-MobilitySet-r13',
                                                                          'optional': True,
                                                                          'type': 'WLAN-Id-List-r13'},
                                                                         {'name': 'successReportRequested',
                                                                          'optional': True,
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('true',
                                                                                      0)]},
                                                                         {'name': 'wlan-SuspendConfig-r14',
                                                                          'optional': True,
                                                                          'type': 'WLAN-SuspendConfig-r14'}],
                                                             'type': 'SEQUENCE'},
                                  'VarWLAN-Status-r13': {'members': [{'name': 'status-r13',
                                                                      'type': 'WLAN-Status-r13'},
                                                                     {'name': 'status-r14',
                                                                      'optional': True,
                                                                      'type': 'WLAN-Status-v1430'}],
                                                         'type': 'SEQUENCE'}},
                        'values': {'maxLogMeas-r10': {'type': 'INTEGER',
                                                      'value': 4060}}},
 'NBIOT-InterNodeDefinitions': {'extensibility-implied': False,
                                'imports': {'EUTRA-InterNodeDefinitions': ['AdditionalReestabInfoList'],
                                            'EUTRA-RRC-Definitions': ['C-RNTI',
                                                                      'PhysCellId',
                                                                      'SecurityAlgorithmConfig',
                                                                      'ShortMAC-I'],
                                            'NBIOT-RRC-Definitions': ['CarrierFreq-NB-r13',
                                                                      'RadioResourceConfigDedicated-NB-r13',
                                                                      'UE-Capability-NB-r13',
                                                                      'UE-RadioPagingInfo-NB-r13']},
                                'object-classes': {},
                                'object-sets': {},
                                'tags': 'AUTOMATIC',
                                'types': {'AS-Config-NB': {'members': [{'name': 'sourceRadioResourceConfig-r13',
                                                                        'type': 'RadioResourceConfigDedicated-NB-r13'},
                                                                       {'name': 'sourceSecurityAlgorithmConfig-r13',
                                                                        'type': 'SecurityAlgorithmConfig'},
                                                                       {'name': 'sourceUE-Identity-r13',
                                                                        'type': 'C-RNTI'},
                                                                       {'name': 'sourceDl-CarrierFreq-r13',
                                                                        'type': 'CarrierFreq-NB-r13'},
                                                                       None],
                                                           'type': 'SEQUENCE'},
                                          'AS-Context-NB': {'members': [{'name': 'reestablishmentInfo-r13',
                                                                         'optional': True,
                                                                         'type': 'ReestablishmentInfo-NB'},
                                                                        None],
                                                            'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-NB': {'members': [{'members': [{'members': [{'name': 'handoverPreparationInformation-r13',
                                                                                                                       'type': 'HandoverPreparationInformation-NB-IEs'},
                                                                                                                      {'name': 'spare3',
                                                                                                                       'type': 'NULL'},
                                                                                                                      {'name': 'spare2',
                                                                                                                       'type': 'NULL'},
                                                                                                                      {'name': 'spare1',
                                                                                                                       'type': 'NULL'}],
                                                                                                          'name': 'c1',
                                                                                                          'type': 'CHOICE'},
                                                                                                         {'members': [],
                                                                                                          'name': 'criticalExtensionsFuture',
                                                                                                          'type': 'SEQUENCE'}],
                                                                                             'name': 'criticalExtensions',
                                                                                             'type': 'CHOICE'}],
                                                                                'type': 'SEQUENCE'},
                                          'HandoverPreparationInformation-NB-IEs': {'members': [{'name': 'ue-RadioAccessCapabilityInfo-r13',
                                                                                                 'type': 'UE-Capability-NB-r13'},
                                                                                                {'name': 'as-Config-r13',
                                                                                                 'type': 'AS-Config-NB'},
                                                                                                {'name': 'rrm-Config-r13',
                                                                                                 'optional': True,
                                                                                                 'type': 'RRM-Config-NB'},
                                                                                                {'name': 'as-Context-r13',
                                                                                                 'optional': True,
                                                                                                 'type': 'AS-Context-NB'},
                                                                                                {'members': [],
                                                                                                 'name': 'nonCriticalExtension',
                                                                                                 'optional': True,
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'type': 'SEQUENCE'},
                                          'RRM-Config-NB': {'members': [{'name': 'ue-InactiveTime',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('s1',
                                                                                     0),
                                                                                    ('s2',
                                                                                     1),
                                                                                    ('s3',
                                                                                     2),
                                                                                    ('s5',
                                                                                     3),
                                                                                    ('s7',
                                                                                     4),
                                                                                    ('s10',
                                                                                     5),
                                                                                    ('s15',
                                                                                     6),
                                                                                    ('s20',
                                                                                     7),
                                                                                    ('s25',
                                                                                     8),
                                                                                    ('s30',
                                                                                     9),
                                                                                    ('s40',
                                                                                     10),
                                                                                    ('s50',
                                                                                     11),
                                                                                    ('min1',
                                                                                     12),
                                                                                    ('min1s20',
                                                                                     13),
                                                                                    ('min1s40',
                                                                                     14),
                                                                                    ('min2',
                                                                                     15),
                                                                                    ('min2s30',
                                                                                     16),
                                                                                    ('min3',
                                                                                     17),
                                                                                    ('min3s30',
                                                                                     18),
                                                                                    ('min4',
                                                                                     19),
                                                                                    ('min5',
                                                                                     20),
                                                                                    ('min6',
                                                                                     21),
                                                                                    ('min7',
                                                                                     22),
                                                                                    ('min8',
                                                                                     23),
                                                                                    ('min9',
                                                                                     24),
                                                                                    ('min10',
                                                                                     25),
                                                                                    ('min12',
                                                                                     26),
                                                                                    ('min14',
                                                                                     27),
                                                                                    ('min17',
                                                                                     28),
                                                                                    ('min20',
                                                                                     29),
                                                                                    ('min24',
                                                                                     30),
                                                                                    ('min28',
                                                                                     31),
                                                                                    ('min33',
                                                                                     32),
                                                                                    ('min38',
                                                                                     33),
                                                                                    ('min44',
                                                                                     34),
                                                                                    ('min50',
                                                                                     35),
                                                                                    ('hr1',
                                                                                     36),
                                                                                    ('hr1min30',
                                                                                     37),
                                                                                    ('hr2',
                                                                                     38),
                                                                                    ('hr2min30',
                                                                                     39),
                                                                                    ('hr3',
                                                                                     40),
                                                                                    ('hr3min30',
                                                                                     41),
                                                                                    ('hr4',
                                                                                     42),
                                                                                    ('hr5',
                                                                                     43),
                                                                                    ('hr6',
                                                                                     44),
                                                                                    ('hr8',
                                                                                     45),
                                                                                    ('hr10',
                                                                                     46),
                                                                                    ('hr13',
                                                                                     47),
                                                                                    ('hr16',
                                                                                     48),
                                                                                    ('hr20',
                                                                                     49),
                                                                                    ('day1',
                                                                                     50),
                                                                                    ('day1hr12',
                                                                                     51),
                                                                                    ('day2',
                                                                                     52),
                                                                                    ('day2hr12',
                                                                                     53),
                                                                                    ('day3',
                                                                                     54),
                                                                                    ('day4',
                                                                                     55),
                                                                                    ('day5',
                                                                                     56),
                                                                                    ('day7',
                                                                                     57),
                                                                                    ('day10',
                                                                                     58),
                                                                                    ('day14',
                                                                                     59),
                                                                                    ('day19',
                                                                                     60),
                                                                                    ('day24',
                                                                                     61),
                                                                                    ('day30',
                                                                                     62),
                                                                                    ('dayMoreThan30',
                                                                                     63)]},
                                                                        None],
                                                            'type': 'SEQUENCE'},
                                          'ReestablishmentInfo-NB': {'members': [{'name': 'sourcePhysCellId-r13',
                                                                                  'type': 'PhysCellId'},
                                                                                 {'name': 'targetCellShortMAC-I-r13',
                                                                                  'type': 'ShortMAC-I'},
                                                                                 {'name': 'additionalReestabInfoList-r13',
                                                                                  'optional': True,
                                                                                  'type': 'AdditionalReestabInfoList'},
                                                                                 None],
                                                                     'type': 'SEQUENCE'},
                                          'UEPagingCoverageInformation-NB': {'members': [{'members': [{'members': [{'name': 'uePagingCoverageInformation-r13',
                                                                                                                    'type': 'UEPagingCoverageInformation-NB-IEs'},
                                                                                                                   {'name': 'spare3',
                                                                                                                    'type': 'NULL'},
                                                                                                                   {'name': 'spare2',
                                                                                                                    'type': 'NULL'},
                                                                                                                   {'name': 'spare1',
                                                                                                                    'type': 'NULL'}],
                                                                                                       'name': 'c1',
                                                                                                       'type': 'CHOICE'},
                                                                                                      {'members': [],
                                                                                                       'name': 'criticalExtensionsFuture',
                                                                                                       'type': 'SEQUENCE'}],
                                                                                          'name': 'criticalExtensions',
                                                                                          'type': 'CHOICE'}],
                                                                             'type': 'SEQUENCE'},
                                          'UEPagingCoverageInformation-NB-IEs': {'members': [{'name': 'npdcch-NumRepetitionPaging-r13',
                                                                                              'optional': True,
                                                                                              'restricted-to': [(1,
                                                                                                                 2048)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'members': [],
                                                                                              'name': 'nonCriticalExtension',
                                                                                              'optional': True,
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'type': 'SEQUENCE'},
                                          'UERadioAccessCapabilityInformation-NB': {'members': [{'members': [{'members': [{'name': 'ueRadioAccessCapabilityInformation-r13',
                                                                                                                           'type': 'UERadioAccessCapabilityInformation-NB-IEs'},
                                                                                                                          {'name': 'spare3',
                                                                                                                           'type': 'NULL'},
                                                                                                                          {'name': 'spare2',
                                                                                                                           'type': 'NULL'},
                                                                                                                          {'name': 'spare1',
                                                                                                                           'type': 'NULL'}],
                                                                                                              'name': 'c1',
                                                                                                              'type': 'CHOICE'},
                                                                                                             {'members': [],
                                                                                                              'name': 'criticalExtensionsFuture',
                                                                                                              'type': 'SEQUENCE'}],
                                                                                                 'name': 'criticalExtensions',
                                                                                                 'type': 'CHOICE'}],
                                                                                    'type': 'SEQUENCE'},
                                          'UERadioAccessCapabilityInformation-NB-IEs': {'members': [{'name': 'ue-RadioAccessCapabilityInfo-r13',
                                                                                                     'type': 'OCTET '
                                                                                                             'STRING'},
                                                                                                    {'members': [],
                                                                                                     'name': 'nonCriticalExtension',
                                                                                                     'optional': True,
                                                                                                     'type': 'SEQUENCE'}],
                                                                                        'type': 'SEQUENCE'},
                                          'UERadioPagingInformation-NB': {'members': [{'members': [{'members': [{'name': 'ueRadioPagingInformation-r13',
                                                                                                                 'type': 'UERadioPagingInformation-NB-IEs'},
                                                                                                                {'name': 'spare3',
                                                                                                                 'type': 'NULL'},
                                                                                                                {'name': 'spare2',
                                                                                                                 'type': 'NULL'},
                                                                                                                {'name': 'spare1',
                                                                                                                 'type': 'NULL'}],
                                                                                                    'name': 'c1',
                                                                                                    'type': 'CHOICE'},
                                                                                                   {'members': [],
                                                                                                    'name': 'criticalExtensionsFuture',
                                                                                                    'type': 'SEQUENCE'}],
                                                                                       'name': 'criticalExtensions',
                                                                                       'type': 'CHOICE'}],
                                                                          'type': 'SEQUENCE'},
                                          'UERadioPagingInformation-NB-IEs': {'members': [{'name': 'ue-RadioPagingInfo-r13',
                                                                                           'type': 'OCTET '
                                                                                                   'STRING'},
                                                                                          {'members': [],
                                                                                           'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SEQUENCE'}],
                                                                              'type': 'SEQUENCE'}},
                                'values': {}},
 'NBIOT-RRC-Definitions': {'extensibility-implied': False,
                           'imports': {'EUTRA-RRC-Definitions': ['RRCConnectionReestablishmentReject',
                                                                 'SecurityModeCommand',
                                                                 'SecurityModeComplete',
                                                                 'SecurityModeFailure',
                                                                 'AdditionalSpectrumEmission',
                                                                 'ARFCN-ValueEUTRA-r9',
                                                                 'CellIdentity',
                                                                 'DedicatedInfoNAS',
                                                                 'DRB-Identity',
                                                                 'InitialUE-Identity',
                                                                 'IntraFreqBlackCellList',
                                                                 'IntraFreqNeighCellList',
                                                                 'maxBands',
                                                                 'maxCellBlack',
                                                                 'maxCellInter',
                                                                 'maxFBI2',
                                                                 'maxFreq',
                                                                 'maxMultiBands',
                                                                 'maxPageRec',
                                                                 'maxPLMN-r11',
                                                                 'maxSAI-MBMS-r11',
                                                                 'maxSIB',
                                                                 'maxSIB-1',
                                                                 'MBMS-SAI-r11',
                                                                 'MBMS-SAI-List-r11',
                                                                 'MBMSSessionInfo-r13',
                                                                 'NextHopChainingCount',
                                                                 'PagingUE-Identity',
                                                                 'PLMN-Identity',
                                                                 'P-Max',
                                                                 'PowerRampingParameters',
                                                                 'PreambleTransMax',
                                                                 'PhysCellId',
                                                                 'Q-OffsetRange',
                                                                 'Q-QualMin-r9',
                                                                 'Q-RxLevMin',
                                                                 'ReestabUE-Identity',
                                                                 'RegisteredMME',
                                                                 'ReselectionThreshold',
                                                                 'ResumeIdentity-r13',
                                                                 'RRC-TransactionIdentifier',
                                                                 'RSRP-Range',
                                                                 'ShortMAC-I',
                                                                 'S-TMSI',
                                                                 'SystemInformationBlockType16-r11',
                                                                 'SystemInfoValueTagSI-r13',
                                                                 'T-Reordering',
                                                                 'TimeAlignmentTimer',
                                                                 'TMGI-r9',
                                                                 'TrackingAreaCode',
                                                                 'DataInactivityTimer-r14']},
                           'object-classes': {},
                           'object-sets': {},
                           'tags': 'AUTOMATIC',
                           'types': {'AB-Config-NB-r13': {'members': [{'name': 'ab-Category-r13',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('a',
                                                                                   0),
                                                                                  ('b',
                                                                                   1),
                                                                                  ('c',
                                                                                   2)]},
                                                                      {'name': 'ab-BarringBitmap-r13',
                                                                       'size': [10],
                                                                       'type': 'BIT '
                                                                               'STRING'},
                                                                      {'name': 'ab-BarringForExceptionData-r13',
                                                                       'optional': True,
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('true',
                                                                                   0)]},
                                                                      {'name': 'ab-BarringForSpecialAC-r13',
                                                                       'size': [5],
                                                                       'type': 'BIT '
                                                                               'STRING'}],
                                                          'type': 'SEQUENCE'},
                                     'AB-ConfigPLMN-NB-r13': {'members': [{'name': 'ab-Config-r13',
                                                                           'optional': True,
                                                                           'type': 'AB-Config-NB-r13'}],
                                                              'type': 'SEQUENCE'},
                                     'ACK-NACK-NumRepetitions-NB-r13': {'type': 'ENUMERATED',
                                                                        'values': [('r1',
                                                                                    0),
                                                                                   ('r2',
                                                                                    1),
                                                                                   ('r4',
                                                                                    2),
                                                                                   ('r8',
                                                                                    3),
                                                                                   ('r16',
                                                                                    4),
                                                                                   ('r32',
                                                                                    5),
                                                                                   ('r64',
                                                                                    6),
                                                                                   ('r128',
                                                                                    7)]},
                                     'AccessStratumRelease-NB-r13': {'type': 'ENUMERATED',
                                                                     'values': [('rel13',
                                                                                 0),
                                                                                ('rel14',
                                                                                 1),
                                                                                ('spare6',
                                                                                 2),
                                                                                ('spare5',
                                                                                 3),
                                                                                ('spare4',
                                                                                 4),
                                                                                ('spare3',
                                                                                 5),
                                                                                ('spare2',
                                                                                 6),
                                                                                ('spare1',
                                                                                 7),
                                                                                None]},
                                     'AdditionalBandInfoList-NB-r14': {'element': {'type': 'FreqBandIndicator-NB-r13'},
                                                                       'size': [(1,
                                                                                 'maxMultiBands')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'BCCH-BCH-Message-NB': {'members': [{'name': 'message',
                                                                          'type': 'BCCH-BCH-MessageType-NB'}],
                                                             'type': 'SEQUENCE'},
                                     'BCCH-BCH-MessageType-NB': {'type': 'MasterInformationBlock-NB'},
                                     'BCCH-Config-NB-r13': {'members': [{'name': 'modificationPeriodCoeff-r13',
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('n16',
                                                                                     0),
                                                                                    ('n32',
                                                                                     1),
                                                                                    ('n64',
                                                                                     2),
                                                                                    ('n128',
                                                                                     3)]}],
                                                            'type': 'SEQUENCE'},
                                     'BCCH-DL-SCH-Message-NB': {'members': [{'name': 'message',
                                                                             'type': 'BCCH-DL-SCH-MessageType-NB'}],
                                                                'type': 'SEQUENCE'},
                                     'BCCH-DL-SCH-MessageType-NB': {'members': [{'members': [{'name': 'systemInformation-r13',
                                                                                              'type': 'SystemInformation-NB'},
                                                                                             {'name': 'systemInformationBlockType1-r13',
                                                                                              'type': 'SystemInformationBlockType1-NB'}],
                                                                                 'name': 'c1',
                                                                                 'type': 'CHOICE'},
                                                                                {'members': [],
                                                                                 'name': 'messageClassExtension',
                                                                                 'type': 'SEQUENCE'}],
                                                                    'type': 'CHOICE'},
                                     'CarrierConfigDedicated-NB-r13': {'members': [{'name': 'dl-CarrierConfig-r13',
                                                                                    'type': 'DL-CarrierConfigDedicated-NB-r13'},
                                                                                   {'name': 'ul-CarrierConfig-r13',
                                                                                    'type': 'UL-CarrierConfigDedicated-NB-r13'}],
                                                                       'type': 'SEQUENCE'},
                                     'CarrierFreq-NB-r13': {'members': [{'name': 'carrierFreq-r13',
                                                                         'type': 'ARFCN-ValueEUTRA-r9'},
                                                                        {'name': 'carrierFreqOffset-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('v-10',
                                                                                     0),
                                                                                    ('v-9',
                                                                                     1),
                                                                                    ('v-8',
                                                                                     2),
                                                                                    ('v-7',
                                                                                     3),
                                                                                    ('v-6',
                                                                                     4),
                                                                                    ('v-5',
                                                                                     5),
                                                                                    ('v-4',
                                                                                     6),
                                                                                    ('v-3',
                                                                                     7),
                                                                                    ('v-2',
                                                                                     8),
                                                                                    ('v-1',
                                                                                     9),
                                                                                    ('v-0dot5',
                                                                                     10),
                                                                                    ('v0',
                                                                                     11),
                                                                                    ('v1',
                                                                                     12),
                                                                                    ('v2',
                                                                                     13),
                                                                                    ('v3',
                                                                                     14),
                                                                                    ('v4',
                                                                                     15),
                                                                                    ('v5',
                                                                                     16),
                                                                                    ('v6',
                                                                                     17),
                                                                                    ('v7',
                                                                                     18),
                                                                                    ('v8',
                                                                                     19),
                                                                                    ('v9',
                                                                                     20)]}],
                                                            'type': 'SEQUENCE'},
                                     'CellSelectionInfo-NB-v1350': {'members': [{'name': 'delta-RxLevMin-v1350',
                                                                                 'restricted-to': [(-8,
                                                                                                    -1)],
                                                                                 'type': 'INTEGER'}],
                                                                    'type': 'SEQUENCE'},
                                     'CellSelectionInfo-NB-v1430': {'members': [{'name': 'powerClass14dBm-Offset-r14',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('dB-6',
                                                                                             0),
                                                                                            ('dB-3',
                                                                                             1),
                                                                                            ('dB3',
                                                                                             2),
                                                                                            ('dB6',
                                                                                             3),
                                                                                            ('dB9',
                                                                                             4),
                                                                                            ('dB12',
                                                                                             5)]},
                                                                                {'name': 'ce-authorisationOffset-r14',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('dB5',
                                                                                             0),
                                                                                            ('dB10',
                                                                                             1),
                                                                                            ('dB15',
                                                                                             2),
                                                                                            ('dB20',
                                                                                             3),
                                                                                            ('dB25',
                                                                                             4),
                                                                                            ('dB30',
                                                                                             5),
                                                                                            ('dB35',
                                                                                             6)]}],
                                                                    'type': 'SEQUENCE'},
                                     'ChannelRasterOffset-NB-r13': {'type': 'ENUMERATED',
                                                                    'values': [('khz-7dot5',
                                                                                0),
                                                                               ('khz-2dot5',
                                                                                1),
                                                                               ('khz2dot5',
                                                                                2),
                                                                               ('khz7dot5',
                                                                                3)]},
                                     'DL-AM-RLC-NB-r13': {'members': [{'name': 'enableStatusReportSN-Gap-r13',
                                                                       'optional': True,
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('true',
                                                                                   0)]}],
                                                          'type': 'SEQUENCE'},
                                     'DL-Bitmap-NB-r13': {'members': [{'name': 'subframePattern10-r13',
                                                                       'size': [10],
                                                                       'type': 'BIT '
                                                                               'STRING'},
                                                                      {'name': 'subframePattern40-r13',
                                                                       'size': [40],
                                                                       'type': 'BIT '
                                                                               'STRING'}],
                                                          'type': 'CHOICE'},
                                     'DL-CCCH-Message-NB': {'members': [{'name': 'message',
                                                                         'type': 'DL-CCCH-MessageType-NB'}],
                                                            'type': 'SEQUENCE'},
                                     'DL-CCCH-MessageType-NB': {'members': [{'members': [{'name': 'rrcConnectionReestablishment-r13',
                                                                                          'type': 'RRCConnectionReestablishment-NB'},
                                                                                         {'name': 'rrcConnectionReestablishmentReject-r13',
                                                                                          'type': 'RRCConnectionReestablishmentReject'},
                                                                                         {'name': 'rrcConnectionReject-r13',
                                                                                          'type': 'RRCConnectionReject-NB'},
                                                                                         {'name': 'rrcConnectionSetup-r13',
                                                                                          'type': 'RRCConnectionSetup-NB'},
                                                                                         {'name': 'spare4',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare3',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare2',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare1',
                                                                                          'type': 'NULL'}],
                                                                             'name': 'c1',
                                                                             'type': 'CHOICE'},
                                                                            {'members': [],
                                                                             'name': 'messageClassExtension',
                                                                             'type': 'SEQUENCE'}],
                                                                'type': 'CHOICE'},
                                     'DL-CarrierConfigCommon-NB-r14': {'members': [{'name': 'dl-CarrierFreq-r14',
                                                                                    'type': 'CarrierFreq-NB-r13'},
                                                                                   {'members': [{'name': 'useNoBitmap-r14',
                                                                                                 'type': 'NULL'},
                                                                                                {'name': 'useAnchorBitmap-r14',
                                                                                                 'type': 'NULL'},
                                                                                                {'name': 'explicitBitmapConfiguration-r14',
                                                                                                 'type': 'DL-Bitmap-NB-r13'}],
                                                                                    'name': 'downlinkBitmapNonAnchor-r14',
                                                                                    'type': 'CHOICE'},
                                                                                   {'members': [{'name': 'useNoGap-r14',
                                                                                                 'type': 'NULL'},
                                                                                                {'name': 'useAnchorGapConfig-r14',
                                                                                                 'type': 'NULL'},
                                                                                                {'name': 'explicitGapConfiguration-r14',
                                                                                                 'type': 'DL-GapConfig-NB-r13'}],
                                                                                    'name': 'dl-GapNonAnchor-r14',
                                                                                    'type': 'CHOICE'},
                                                                                   {'members': [{'members': [{'members': [{'name': 'indexToMidPRB-r14',
                                                                                                                           'restricted-to': [(-55,
                                                                                                                                              54)],
                                                                                                                           'type': 'INTEGER'}],
                                                                                                              'name': 'samePCI-r14',
                                                                                                              'type': 'SEQUENCE'},
                                                                                                             {'members': [{'name': 'eutra-NumCRS-Ports-r14',
                                                                                                                           'type': 'ENUMERATED',
                                                                                                                           'values': [('same',
                                                                                                                                       0),
                                                                                                                                      ('four',
                                                                                                                                       1)]}],
                                                                                                              'name': 'differentPCI-r14',
                                                                                                              'type': 'SEQUENCE'}],
                                                                                                 'name': 'samePCI-Indicator-r14',
                                                                                                 'optional': True,
                                                                                                 'type': 'CHOICE'},
                                                                                                {'name': 'eutraControlRegionSize-r14',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('n1',
                                                                                                             0),
                                                                                                            ('n2',
                                                                                                             1),
                                                                                                            ('n3',
                                                                                                             2)]}],
                                                                                    'name': 'inbandCarrierInfo-r14',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'},
                                                                                   {'default': 'dB0',
                                                                                    'name': 'nrs-PowerOffsetNonAnchor-r14',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('dB-12',
                                                                                                0),
                                                                                               ('dB-10',
                                                                                                1),
                                                                                               ('dB-8',
                                                                                                2),
                                                                                               ('dB-6',
                                                                                                3),
                                                                                               ('dB-4',
                                                                                                4),
                                                                                               ('dB-2',
                                                                                                5),
                                                                                               ('dB0',
                                                                                                6),
                                                                                               ('dB3',
                                                                                                7)]},
                                                                                   None],
                                                                       'type': 'SEQUENCE'},
                                     'DL-CarrierConfigDedicated-NB-r13': {'members': [{'name': 'dl-CarrierFreq-r13',
                                                                                       'type': 'CarrierFreq-NB-r13'},
                                                                                      {'members': [{'name': 'useNoBitmap-r13',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'useAnchorBitmap-r13',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'explicitBitmapConfiguration-r13',
                                                                                                    'type': 'DL-Bitmap-NB-r13'},
                                                                                                   {'name': 'spare',
                                                                                                    'type': 'NULL'}],
                                                                                       'name': 'downlinkBitmapNonAnchor-r13',
                                                                                       'optional': True,
                                                                                       'type': 'CHOICE'},
                                                                                      {'members': [{'name': 'useNoGap-r13',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'useAnchorGapConfig-r13',
                                                                                                    'type': 'NULL'},
                                                                                                   {'name': 'explicitGapConfiguration-r13',
                                                                                                    'type': 'DL-GapConfig-NB-r13'},
                                                                                                   {'name': 'spare',
                                                                                                    'type': 'NULL'}],
                                                                                       'name': 'dl-GapNonAnchor-r13',
                                                                                       'optional': True,
                                                                                       'type': 'CHOICE'},
                                                                                      {'members': [{'members': [{'members': [{'name': 'indexToMidPRB-r13',
                                                                                                                              'restricted-to': [(-55,
                                                                                                                                                 54)],
                                                                                                                              'type': 'INTEGER'}],
                                                                                                                 'name': 'samePCI-r13',
                                                                                                                 'type': 'SEQUENCE'},
                                                                                                                {'members': [{'name': 'eutra-NumCRS-Ports-r13',
                                                                                                                              'type': 'ENUMERATED',
                                                                                                                              'values': [('same',
                                                                                                                                          0),
                                                                                                                                         ('four',
                                                                                                                                          1)]}],
                                                                                                                 'name': 'differentPCI-r13',
                                                                                                                 'type': 'SEQUENCE'}],
                                                                                                    'name': 'samePCI-Indicator-r13',
                                                                                                    'optional': True,
                                                                                                    'type': 'CHOICE'},
                                                                                                   {'name': 'eutraControlRegionSize-r13',
                                                                                                    'type': 'ENUMERATED',
                                                                                                    'values': [('n1',
                                                                                                                0),
                                                                                                               ('n2',
                                                                                                                1),
                                                                                                               ('n3',
                                                                                                                2)]}],
                                                                                       'name': 'inbandCarrierInfo-r13',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'},
                                                                                      None,
                                                                                      [{'name': 'nrs-PowerOffsetNonAnchor-v1330',
                                                                                        'optional': True,
                                                                                        'type': 'ENUMERATED',
                                                                                        'values': [('dB-12',
                                                                                                    0),
                                                                                                   ('dB-10',
                                                                                                    1),
                                                                                                   ('dB-8',
                                                                                                    2),
                                                                                                   ('dB-6',
                                                                                                    3),
                                                                                                   ('dB-4',
                                                                                                    4),
                                                                                                   ('dB-2',
                                                                                                    5),
                                                                                                   ('dB0',
                                                                                                    6),
                                                                                                   ('dB3',
                                                                                                    7)]}]],
                                                                          'type': 'SEQUENCE'},
                                     'DL-ConfigCommon-NB-r14': {'members': [{'name': 'dl-CarrierConfig-r14',
                                                                             'type': 'DL-CarrierConfigCommon-NB-r14'},
                                                                            {'name': 'pcch-Config-r14',
                                                                             'optional': True,
                                                                             'type': 'PCCH-Config-NB-r14'},
                                                                            None],
                                                                'type': 'SEQUENCE'},
                                     'DL-ConfigCommonList-NB-r14': {'element': {'type': 'DL-ConfigCommon-NB-r14'},
                                                                    'size': [(1,
                                                                              'maxNonAnchorCarriers-NB-r14')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'DL-DCCH-Message-NB': {'members': [{'name': 'message',
                                                                         'type': 'DL-DCCH-MessageType-NB'}],
                                                            'type': 'SEQUENCE'},
                                     'DL-DCCH-MessageType-NB': {'members': [{'members': [{'name': 'dlInformationTransfer-r13',
                                                                                          'type': 'DLInformationTransfer-NB'},
                                                                                         {'name': 'rrcConnectionReconfiguration-r13',
                                                                                          'type': 'RRCConnectionReconfiguration-NB'},
                                                                                         {'name': 'rrcConnectionRelease-r13',
                                                                                          'type': 'RRCConnectionRelease-NB'},
                                                                                         {'name': 'securityModeCommand-r13',
                                                                                          'type': 'SecurityModeCommand'},
                                                                                         {'name': 'ueCapabilityEnquiry-r13',
                                                                                          'type': 'UECapabilityEnquiry-NB'},
                                                                                         {'name': 'rrcConnectionResume-r13',
                                                                                          'type': 'RRCConnectionResume-NB'},
                                                                                         {'name': 'spare2',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare1',
                                                                                          'type': 'NULL'}],
                                                                             'name': 'c1',
                                                                             'type': 'CHOICE'},
                                                                            {'members': [],
                                                                             'name': 'messageClassExtension',
                                                                             'type': 'SEQUENCE'}],
                                                                'type': 'CHOICE'},
                                     'DL-GapConfig-NB-r13': {'members': [{'name': 'dl-GapThreshold-r13',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('n32',
                                                                                      0),
                                                                                     ('n64',
                                                                                      1),
                                                                                     ('n128',
                                                                                      2),
                                                                                     ('n256',
                                                                                      3)]},
                                                                         {'name': 'dl-GapPeriodicity-r13',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('sf64',
                                                                                      0),
                                                                                     ('sf128',
                                                                                      1),
                                                                                     ('sf256',
                                                                                      2),
                                                                                     ('sf512',
                                                                                      3)]},
                                                                         {'name': 'dl-GapDurationCoeff-r13',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('oneEighth',
                                                                                      0),
                                                                                     ('oneFourth',
                                                                                      1),
                                                                                     ('threeEighth',
                                                                                      2),
                                                                                     ('oneHalf',
                                                                                      3)]}],
                                                             'type': 'SEQUENCE'},
                                     'DLInformationTransfer-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                               'type': 'RRC-TransactionIdentifier'},
                                                                              {'members': [{'members': [{'name': 'dlInformationTransfer-r13',
                                                                                                         'type': 'DLInformationTransfer-NB-r13-IEs'},
                                                                                                        {'name': 'spare1',
                                                                                                         'type': 'NULL'}],
                                                                                            'name': 'c1',
                                                                                            'type': 'CHOICE'},
                                                                                           {'members': [],
                                                                                            'name': 'criticalExtensionsFuture',
                                                                                            'type': 'SEQUENCE'}],
                                                                               'name': 'criticalExtensions',
                                                                               'type': 'CHOICE'}],
                                                                  'type': 'SEQUENCE'},
                                     'DLInformationTransfer-NB-r13-IEs': {'members': [{'name': 'dedicatedInfoNAS-r13',
                                                                                       'type': 'DedicatedInfoNAS'},
                                                                                      {'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      {'members': [],
                                                                                       'name': 'nonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'}],
                                                                          'type': 'SEQUENCE'},
                                     'DRB-ToAddMod-NB-r13': {'members': [{'name': 'eps-BearerIdentity-r13',
                                                                          'optional': True,
                                                                          'restricted-to': [(0,
                                                                                             15)],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'drb-Identity-r13',
                                                                          'type': 'DRB-Identity'},
                                                                         {'name': 'pdcp-Config-r13',
                                                                          'optional': True,
                                                                          'type': 'PDCP-Config-NB-r13'},
                                                                         {'name': 'rlc-Config-r13',
                                                                          'optional': True,
                                                                          'type': 'RLC-Config-NB-r13'},
                                                                         {'name': 'logicalChannelIdentity-r13',
                                                                          'optional': True,
                                                                          'restricted-to': [(3,
                                                                                             10)],
                                                                          'type': 'INTEGER'},
                                                                         {'name': 'logicalChannelConfig-r13',
                                                                          'optional': True,
                                                                          'type': 'LogicalChannelConfig-NB-r13'},
                                                                         None,
                                                                         [{'name': 'rlc-Config-v1430',
                                                                           'optional': True,
                                                                           'type': 'RLC-Config-NB-v1430'}]],
                                                             'type': 'SEQUENCE'},
                                     'DRB-ToAddModList-NB-r13': {'element': {'type': 'DRB-ToAddMod-NB-r13'},
                                                                 'size': [(1,
                                                                           'maxDRB-NB-r13')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'DRB-ToReleaseList-NB-r13': {'element': {'type': 'DRB-Identity'},
                                                                  'size': [(1,
                                                                            'maxDRB-NB-r13')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'DRX-Config-NB-r13': {'members': [{'name': 'release',
                                                                        'type': 'NULL'},
                                                                       {'members': [{'name': 'onDurationTimer-r13',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('pp1',
                                                                                                 0),
                                                                                                ('pp2',
                                                                                                 1),
                                                                                                ('pp3',
                                                                                                 2),
                                                                                                ('pp4',
                                                                                                 3),
                                                                                                ('pp8',
                                                                                                 4),
                                                                                                ('pp16',
                                                                                                 5),
                                                                                                ('pp32',
                                                                                                 6),
                                                                                                ('spare',
                                                                                                 7)]},
                                                                                    {'name': 'drx-InactivityTimer-r13',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('pp0',
                                                                                                 0),
                                                                                                ('pp1',
                                                                                                 1),
                                                                                                ('pp2',
                                                                                                 2),
                                                                                                ('pp3',
                                                                                                 3),
                                                                                                ('pp4',
                                                                                                 4),
                                                                                                ('pp8',
                                                                                                 5),
                                                                                                ('pp16',
                                                                                                 6),
                                                                                                ('pp32',
                                                                                                 7)]},
                                                                                    {'name': 'drx-RetransmissionTimer-r13',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('pp0',
                                                                                                 0),
                                                                                                ('pp1',
                                                                                                 1),
                                                                                                ('pp2',
                                                                                                 2),
                                                                                                ('pp4',
                                                                                                 3),
                                                                                                ('pp6',
                                                                                                 4),
                                                                                                ('pp8',
                                                                                                 5),
                                                                                                ('pp16',
                                                                                                 6),
                                                                                                ('pp24',
                                                                                                 7),
                                                                                                ('pp33',
                                                                                                 8),
                                                                                                ('spare7',
                                                                                                 9),
                                                                                                ('spare6',
                                                                                                 10),
                                                                                                ('spare5',
                                                                                                 11),
                                                                                                ('spare4',
                                                                                                 12),
                                                                                                ('spare3',
                                                                                                 13),
                                                                                                ('spare2',
                                                                                                 14),
                                                                                                ('spare1',
                                                                                                 15)]},
                                                                                    {'name': 'drx-Cycle-r13',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('sf256',
                                                                                                 0),
                                                                                                ('sf512',
                                                                                                 1),
                                                                                                ('sf1024',
                                                                                                 2),
                                                                                                ('sf1536',
                                                                                                 3),
                                                                                                ('sf2048',
                                                                                                 4),
                                                                                                ('sf3072',
                                                                                                 5),
                                                                                                ('sf4096',
                                                                                                 6),
                                                                                                ('sf4608',
                                                                                                 7),
                                                                                                ('sf6144',
                                                                                                 8),
                                                                                                ('sf7680',
                                                                                                 9),
                                                                                                ('sf8192',
                                                                                                 10),
                                                                                                ('sf9216',
                                                                                                 11),
                                                                                                ('spare4',
                                                                                                 12),
                                                                                                ('spare3',
                                                                                                 13),
                                                                                                ('spare2',
                                                                                                 14),
                                                                                                ('spare1',
                                                                                                 15)]},
                                                                                    {'name': 'drx-StartOffset-r13',
                                                                                     'restricted-to': [(0,
                                                                                                        255)],
                                                                                     'type': 'INTEGER'},
                                                                                    {'name': 'drx-ULRetransmissionTimer-r13',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('pp0',
                                                                                                 0),
                                                                                                ('pp1',
                                                                                                 1),
                                                                                                ('pp2',
                                                                                                 2),
                                                                                                ('pp4',
                                                                                                 3),
                                                                                                ('pp6',
                                                                                                 4),
                                                                                                ('pp8',
                                                                                                 5),
                                                                                                ('pp16',
                                                                                                 6),
                                                                                                ('pp24',
                                                                                                 7),
                                                                                                ('pp33',
                                                                                                 8),
                                                                                                ('pp40',
                                                                                                 9),
                                                                                                ('pp64',
                                                                                                 10),
                                                                                                ('pp80',
                                                                                                 11),
                                                                                                ('pp96',
                                                                                                 12),
                                                                                                ('pp112',
                                                                                                 13),
                                                                                                ('pp128',
                                                                                                 14),
                                                                                                ('pp160',
                                                                                                 15),
                                                                                                ('pp320',
                                                                                                 16)]}],
                                                                        'name': 'setup',
                                                                        'type': 'SEQUENCE'}],
                                                           'type': 'CHOICE'},
                                     'EstablishmentCause-NB-r13': {'type': 'ENUMERATED',
                                                                   'values': [('mt-Access',
                                                                               0),
                                                                              ('mo-Signalling',
                                                                               1),
                                                                              ('mo-Data',
                                                                               2),
                                                                              ('mo-ExceptionData',
                                                                               3),
                                                                              ('delayTolerantAccess-v1330',
                                                                               4),
                                                                              ('spare3',
                                                                               5),
                                                                              ('spare2',
                                                                               6),
                                                                              ('spare1',
                                                                               7)]},
                                     'FreqBandIndicator-NB-r13': {'restricted-to': [(1,
                                                                                     'maxFBI2')],
                                                                  'type': 'INTEGER'},
                                     'Guardband-NB-r13': {'members': [{'name': 'rasterOffset-r13',
                                                                       'type': 'ChannelRasterOffset-NB-r13'},
                                                                      {'name': 'spare',
                                                                       'size': [3],
                                                                       'type': 'BIT '
                                                                               'STRING'}],
                                                          'type': 'SEQUENCE'},
                                     'Inband-DifferentPCI-NB-r13': {'members': [{'name': 'eutra-NumCRS-Ports-r13',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('same',
                                                                                             0),
                                                                                            ('four',
                                                                                             1)]},
                                                                                {'name': 'rasterOffset-r13',
                                                                                 'type': 'ChannelRasterOffset-NB-r13'},
                                                                                {'name': 'spare',
                                                                                 'size': [2],
                                                                                 'type': 'BIT '
                                                                                         'STRING'}],
                                                                    'type': 'SEQUENCE'},
                                     'Inband-SamePCI-NB-r13': {'members': [{'name': 'eutra-CRS-SequenceInfo-r13',
                                                                            'restricted-to': [(0,
                                                                                               31)],
                                                                            'type': 'INTEGER'}],
                                                               'type': 'SEQUENCE'},
                                     'InterFreqBlackCellList-NB-r13': {'element': {'type': 'PhysCellId'},
                                                                       'size': [(1,
                                                                                 'maxCellBlack')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'InterFreqCarrierFreqInfo-NB-r13': {'members': [{'name': 'dl-CarrierFreq-r13',
                                                                                      'type': 'CarrierFreq-NB-r13'},
                                                                                     {'name': 'q-RxLevMin-r13',
                                                                                      'type': 'Q-RxLevMin'},
                                                                                     {'name': 'q-QualMin-r13',
                                                                                      'optional': True,
                                                                                      'type': 'Q-QualMin-r9'},
                                                                                     {'name': 'p-Max-r13',
                                                                                      'optional': True,
                                                                                      'type': 'P-Max'},
                                                                                     {'default': 'dB0',
                                                                                      'name': 'q-OffsetFreq-r13',
                                                                                      'type': 'Q-OffsetRange'},
                                                                                     {'name': 'interFreqNeighCellList-r13',
                                                                                      'optional': True,
                                                                                      'type': 'InterFreqNeighCellList-NB-r13'},
                                                                                     {'name': 'interFreqBlackCellList-r13',
                                                                                      'optional': True,
                                                                                      'type': 'InterFreqBlackCellList-NB-r13'},
                                                                                     {'name': 'multiBandInfoList-r13',
                                                                                      'optional': True,
                                                                                      'type': 'MultiBandInfoList-NB-r13'},
                                                                                     None,
                                                                                     [{'name': 'delta-RxLevMin-v1350',
                                                                                       'optional': True,
                                                                                       'restricted-to': [(-8,
                                                                                                          -1)],
                                                                                       'type': 'INTEGER'}],
                                                                                     [{'name': 'powerClass14dBm-Offset-r14',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('dB-6',
                                                                                                   0),
                                                                                                  ('dB-3',
                                                                                                   1),
                                                                                                  ('dB3',
                                                                                                   2),
                                                                                                  ('dB6',
                                                                                                   3),
                                                                                                  ('dB9',
                                                                                                   4),
                                                                                                  ('dB12',
                                                                                                   5)]},
                                                                                      {'name': 'ce-AuthorisationOffset-r14',
                                                                                       'optional': True,
                                                                                       'type': 'ENUMERATED',
                                                                                       'values': [('dB5',
                                                                                                   0),
                                                                                                  ('dB10',
                                                                                                   1),
                                                                                                  ('dB15',
                                                                                                   2),
                                                                                                  ('dB20',
                                                                                                   3),
                                                                                                  ('dB25',
                                                                                                   4),
                                                                                                  ('dB30',
                                                                                                   5),
                                                                                                  ('dB35',
                                                                                                   6)]}]],
                                                                         'type': 'SEQUENCE'},
                                     'InterFreqCarrierFreqList-NB-r13': {'element': {'type': 'InterFreqCarrierFreqInfo-NB-r13'},
                                                                         'size': [(1,
                                                                                   'maxFreq')],
                                                                         'type': 'SEQUENCE '
                                                                                 'OF'},
                                     'InterFreqNeighCellList-NB-r13': {'element': {'type': 'PhysCellId'},
                                                                       'size': [(1,
                                                                                 'maxCellInter')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'IntraFreqCellReselectionInfo-NB-v1350': {'members': [{'name': 'delta-RxLevMin-v1350',
                                                                                            'restricted-to': [(-8,
                                                                                                               -1)],
                                                                                            'type': 'INTEGER'}],
                                                                               'type': 'SEQUENCE'},
                                     'IntraFreqCellReselectionInfo-NB-v1360': {'members': [{'name': 's-IntraSearchP-v1360',
                                                                                            'type': 'ReselectionThreshold-NB-v1360'}],
                                                                               'type': 'SEQUENCE'},
                                     'IntraFreqCellReselectionInfo-NB-v1430': {'members': [{'name': 'powerClass14dBm-Offset-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('dB-6',
                                                                                                        0),
                                                                                                       ('dB-3',
                                                                                                        1),
                                                                                                       ('dB3',
                                                                                                        2),
                                                                                                       ('dB6',
                                                                                                        3),
                                                                                                       ('dB9',
                                                                                                        4),
                                                                                                       ('dB12',
                                                                                                        5)]},
                                                                                           {'name': 'ce-AuthorisationOffset-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('dB5',
                                                                                                        0),
                                                                                                       ('dB10',
                                                                                                        1),
                                                                                                       ('dB15',
                                                                                                        2),
                                                                                                       ('dB20',
                                                                                                        3),
                                                                                                       ('dB25',
                                                                                                        4),
                                                                                                       ('dB30',
                                                                                                        5),
                                                                                                       ('dB35',
                                                                                                        6)]}],
                                                                               'type': 'SEQUENCE'},
                                     'LogicalChannelConfig-NB-r13': {'members': [{'name': 'priority-r13',
                                                                                  'optional': True,
                                                                                  'restricted-to': [(1,
                                                                                                     16)],
                                                                                  'type': 'INTEGER'},
                                                                                 {'name': 'logicalChannelSR-Prohibit-r13',
                                                                                  'optional': True,
                                                                                  'type': 'BOOLEAN'},
                                                                                 None],
                                                                     'type': 'SEQUENCE'},
                                     'MAC-MainConfig-NB-r13': {'members': [{'members': [{'name': 'periodicBSR-Timer-r13',
                                                                                         'optional': True,
                                                                                         'type': 'PeriodicBSR-Timer-NB-r13'},
                                                                                        {'name': 'retxBSR-Timer-r13',
                                                                                         'type': 'RetxBSR-Timer-NB-r13'}],
                                                                            'name': 'ul-SCH-Config-r13',
                                                                            'optional': True,
                                                                            'type': 'SEQUENCE'},
                                                                           {'name': 'drx-Config-r13',
                                                                            'optional': True,
                                                                            'type': 'DRX-Config-NB-r13'},
                                                                           {'name': 'timeAlignmentTimerDedicated-r13',
                                                                            'type': 'TimeAlignmentTimer'},
                                                                           {'members': [{'name': 'release',
                                                                                         'type': 'NULL'},
                                                                                        {'members': [{'name': 'logicalChannelSR-ProhibitTimer-r13',
                                                                                                      'type': 'ENUMERATED',
                                                                                                      'values': [('pp2',
                                                                                                                  0),
                                                                                                                 ('pp8',
                                                                                                                  1),
                                                                                                                 ('pp32',
                                                                                                                  2),
                                                                                                                 ('pp128',
                                                                                                                  3),
                                                                                                                 ('pp512',
                                                                                                                  4),
                                                                                                                 ('pp1024',
                                                                                                                  5),
                                                                                                                 ('pp2048',
                                                                                                                  6),
                                                                                                                 ('spare',
                                                                                                                  7)]}],
                                                                                         'name': 'setup',
                                                                                         'type': 'SEQUENCE'}],
                                                                            'name': 'logicalChannelSR-Config-r13',
                                                                            'optional': True,
                                                                            'type': 'CHOICE'},
                                                                           None,
                                                                           [{'name': 'rai-Activation-r14',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('true',
                                                                                         0)]},
                                                                            {'members': [{'name': 'release',
                                                                                          'type': 'NULL'},
                                                                                         {'members': [{'name': 'dataInactivityTimer-r14',
                                                                                                       'type': 'DataInactivityTimer-r14'}],
                                                                                          'name': 'setup',
                                                                                          'type': 'SEQUENCE'}],
                                                                             'name': 'dataInactivityTimerConfig-r14',
                                                                             'optional': True,
                                                                             'type': 'CHOICE'}],
                                                                           [{'name': 'drx-Cycle-v1430',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('sf1280',
                                                                                         0),
                                                                                        ('sf2560',
                                                                                         1),
                                                                                        ('sf5120',
                                                                                         2),
                                                                                        ('sf10240',
                                                                                         3)]}],
                                                                           [{'name': 'ra-CFRA-Config-r14',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('true',
                                                                                         0)]}]],
                                                               'type': 'SEQUENCE'},
                                     'MAC-Parameters-NB-r14': {'members': [{'name': 'dataInactMon-r14',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]},
                                                                           {'name': 'rai-Support-r14',
                                                                            'optional': True,
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('supported',
                                                                                        0)]}],
                                                               'type': 'SEQUENCE'},
                                     'MBMS-SAI-InterFreq-NB-r14': {'members': [{'name': 'dl-CarrierFreq-r14',
                                                                                'type': 'CarrierFreq-NB-r13'},
                                                                               {'name': 'mbms-SAI-List-r14',
                                                                                'type': 'MBMS-SAI-List-r11'},
                                                                               {'name': 'multiBandInfoList-r14',
                                                                                'optional': True,
                                                                                'type': 'AdditionalBandInfoList-NB-r14'}],
                                                                   'type': 'SEQUENCE'},
                                     'MBMS-SAI-InterFreqList-NB-r14': {'element': {'type': 'MBMS-SAI-InterFreq-NB-r14'},
                                                                       'size': [(1,
                                                                                 'maxFreq')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'MasterInformationBlock-NB': {'members': [{'name': 'systemFrameNumber-MSB-r13',
                                                                                'size': [4],
                                                                                'type': 'BIT '
                                                                                        'STRING'},
                                                                               {'name': 'hyperSFN-LSB-r13',
                                                                                'size': [2],
                                                                                'type': 'BIT '
                                                                                        'STRING'},
                                                                               {'name': 'schedulingInfoSIB1-r13',
                                                                                'restricted-to': [(0,
                                                                                                   15)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'systemInfoValueTag-r13',
                                                                                'restricted-to': [(0,
                                                                                                   31)],
                                                                                'type': 'INTEGER'},
                                                                               {'name': 'ab-Enabled-r13',
                                                                                'type': 'BOOLEAN'},
                                                                               {'members': [{'name': 'inband-SamePCI-r13',
                                                                                             'type': 'Inband-SamePCI-NB-r13'},
                                                                                            {'name': 'inband-DifferentPCI-r13',
                                                                                             'type': 'Inband-DifferentPCI-NB-r13'},
                                                                                            {'name': 'guardband-r13',
                                                                                             'type': 'Guardband-NB-r13'},
                                                                                            {'name': 'standalone-r13',
                                                                                             'type': 'Standalone-NB-r13'}],
                                                                                'name': 'operationModeInfo-r13',
                                                                                'type': 'CHOICE'},
                                                                               {'name': 'spare',
                                                                                'size': [11],
                                                                                'type': 'BIT '
                                                                                        'STRING'}],
                                                                   'type': 'SEQUENCE'},
                                     'MultiBandInfo-NB-r13': {'members': [{'name': 'freqBandIndicator-r13',
                                                                           'optional': True,
                                                                           'type': 'FreqBandIndicator-NB-r13'},
                                                                          {'name': 'freqBandInfo-r13',
                                                                           'optional': True,
                                                                           'type': 'NS-PmaxList-NB-r13'}],
                                                              'type': 'SEQUENCE'},
                                     'MultiBandInfoList-NB-r13': {'element': {'type': 'MultiBandInfo-NB-r13'},
                                                                  'size': [(1,
                                                                            'maxMultiBands')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'NPDCCH-ConfigDedicated-NB-r13': {'members': [{'name': 'npdcch-NumRepetitions-r13',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('r1',
                                                                                                0),
                                                                                               ('r2',
                                                                                                1),
                                                                                               ('r4',
                                                                                                2),
                                                                                               ('r8',
                                                                                                3),
                                                                                               ('r16',
                                                                                                4),
                                                                                               ('r32',
                                                                                                5),
                                                                                               ('r64',
                                                                                                6),
                                                                                               ('r128',
                                                                                                7),
                                                                                               ('r256',
                                                                                                8),
                                                                                               ('r512',
                                                                                                9),
                                                                                               ('r1024',
                                                                                                10),
                                                                                               ('r2048',
                                                                                                11),
                                                                                               ('spare4',
                                                                                                12),
                                                                                               ('spare3',
                                                                                                13),
                                                                                               ('spare2',
                                                                                                14),
                                                                                               ('spare1',
                                                                                                15)]},
                                                                                   {'name': 'npdcch-StartSF-USS-r13',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('v1dot5',
                                                                                                0),
                                                                                               ('v2',
                                                                                                1),
                                                                                               ('v4',
                                                                                                2),
                                                                                               ('v8',
                                                                                                3),
                                                                                               ('v16',
                                                                                                4),
                                                                                               ('v32',
                                                                                                5),
                                                                                               ('v48',
                                                                                                6),
                                                                                               ('v64',
                                                                                                7)]},
                                                                                   {'name': 'npdcch-Offset-USS-r13',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('zero',
                                                                                                0),
                                                                                               ('oneEighth',
                                                                                                1),
                                                                                               ('oneFourth',
                                                                                                2),
                                                                                               ('threeEighth',
                                                                                                3)]}],
                                                                       'type': 'SEQUENCE'},
                                     'NPDCCH-SC-MCCH-Config-NB-r14': {'members': [{'name': 'npdcch-NumRepetitions-SC-MCCH-r14',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('r1',
                                                                                               0),
                                                                                              ('r2',
                                                                                               1),
                                                                                              ('r4',
                                                                                               2),
                                                                                              ('r8',
                                                                                               3),
                                                                                              ('r16',
                                                                                               4),
                                                                                              ('r32',
                                                                                               5),
                                                                                              ('r64',
                                                                                               6),
                                                                                              ('r128',
                                                                                               7),
                                                                                              ('r256',
                                                                                               8),
                                                                                              ('r512',
                                                                                               9),
                                                                                              ('r1024',
                                                                                               10),
                                                                                              ('r2048',
                                                                                               11)]},
                                                                                  {'name': 'npdcch-StartSF-SC-MCCH-r14',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('v1dot5',
                                                                                               0),
                                                                                              ('v2',
                                                                                               1),
                                                                                              ('v4',
                                                                                               2),
                                                                                              ('v8',
                                                                                               3),
                                                                                              ('v16',
                                                                                               4),
                                                                                              ('v32',
                                                                                               5),
                                                                                              ('v48',
                                                                                               6),
                                                                                              ('v64',
                                                                                               7)]},
                                                                                  {'name': 'npdcch-Offset-SC-MCCH-r14',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('zero',
                                                                                               0),
                                                                                              ('oneEighth',
                                                                                               1),
                                                                                              ('oneQuarter',
                                                                                               2),
                                                                                              ('threeEighth',
                                                                                               3),
                                                                                              ('oneHalf',
                                                                                               4),
                                                                                              ('fiveEighth',
                                                                                               5),
                                                                                              ('threeQuarter',
                                                                                               6),
                                                                                              ('sevenEighth',
                                                                                               7)]}],
                                                                      'type': 'SEQUENCE'},
                                     'NPDSCH-ConfigCommon-NB-r13': {'members': [{'name': 'nrs-Power-r13',
                                                                                 'restricted-to': [(-60,
                                                                                                    50)],
                                                                                 'type': 'INTEGER'}],
                                                                    'type': 'SEQUENCE'},
                                     'NPRACH-ConfigSIB-NB-r13': {'members': [{'name': 'nprach-CP-Length-r13',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('us66dot7',
                                                                                          0),
                                                                                         ('us266dot7',
                                                                                          1)]},
                                                                             {'name': 'rsrp-ThresholdsPrachInfoList-r13',
                                                                              'optional': True,
                                                                              'type': 'RSRP-ThresholdsNPRACH-InfoList-NB-r13'},
                                                                             {'name': 'nprach-ParametersList-r13',
                                                                              'type': 'NPRACH-ParametersList-NB-r13'}],
                                                                 'type': 'SEQUENCE'},
                                     'NPRACH-ConfigSIB-NB-v1330': {'members': [{'name': 'nprach-ParametersList-v1330',
                                                                                'type': 'NPRACH-ParametersList-NB-v1330'}],
                                                                   'type': 'SEQUENCE'},
                                     'NPRACH-Parameters-NB-r13': {'members': [{'name': 'nprach-Periodicity-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('ms40',
                                                                                           0),
                                                                                          ('ms80',
                                                                                           1),
                                                                                          ('ms160',
                                                                                           2),
                                                                                          ('ms240',
                                                                                           3),
                                                                                          ('ms320',
                                                                                           4),
                                                                                          ('ms640',
                                                                                           5),
                                                                                          ('ms1280',
                                                                                           6),
                                                                                          ('ms2560',
                                                                                           7)]},
                                                                              {'name': 'nprach-StartTime-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('ms8',
                                                                                           0),
                                                                                          ('ms16',
                                                                                           1),
                                                                                          ('ms32',
                                                                                           2),
                                                                                          ('ms64',
                                                                                           3),
                                                                                          ('ms128',
                                                                                           4),
                                                                                          ('ms256',
                                                                                           5),
                                                                                          ('ms512',
                                                                                           6),
                                                                                          ('ms1024',
                                                                                           7)]},
                                                                              {'name': 'nprach-SubcarrierOffset-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n0',
                                                                                           0),
                                                                                          ('n12',
                                                                                           1),
                                                                                          ('n24',
                                                                                           2),
                                                                                          ('n36',
                                                                                           3),
                                                                                          ('n2',
                                                                                           4),
                                                                                          ('n18',
                                                                                           5),
                                                                                          ('n34',
                                                                                           6),
                                                                                          ('spare1',
                                                                                           7)]},
                                                                              {'name': 'nprach-NumSubcarriers-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n12',
                                                                                           0),
                                                                                          ('n24',
                                                                                           1),
                                                                                          ('n36',
                                                                                           2),
                                                                                          ('n48',
                                                                                           3)]},
                                                                              {'name': 'nprach-SubcarrierMSG3-RangeStart-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('zero',
                                                                                           0),
                                                                                          ('oneThird',
                                                                                           1),
                                                                                          ('twoThird',
                                                                                           2),
                                                                                          ('one',
                                                                                           3)]},
                                                                              {'name': 'maxNumPreambleAttemptCE-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n3',
                                                                                           0),
                                                                                          ('n4',
                                                                                           1),
                                                                                          ('n5',
                                                                                           2),
                                                                                          ('n6',
                                                                                           3),
                                                                                          ('n7',
                                                                                           4),
                                                                                          ('n8',
                                                                                           5),
                                                                                          ('n10',
                                                                                           6),
                                                                                          ('spare1',
                                                                                           7)]},
                                                                              {'name': 'numRepetitionsPerPreambleAttempt-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('n1',
                                                                                           0),
                                                                                          ('n2',
                                                                                           1),
                                                                                          ('n4',
                                                                                           2),
                                                                                          ('n8',
                                                                                           3),
                                                                                          ('n16',
                                                                                           4),
                                                                                          ('n32',
                                                                                           5),
                                                                                          ('n64',
                                                                                           6),
                                                                                          ('n128',
                                                                                           7)]},
                                                                              {'name': 'npdcch-NumRepetitions-RA-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('r1',
                                                                                           0),
                                                                                          ('r2',
                                                                                           1),
                                                                                          ('r4',
                                                                                           2),
                                                                                          ('r8',
                                                                                           3),
                                                                                          ('r16',
                                                                                           4),
                                                                                          ('r32',
                                                                                           5),
                                                                                          ('r64',
                                                                                           6),
                                                                                          ('r128',
                                                                                           7),
                                                                                          ('r256',
                                                                                           8),
                                                                                          ('r512',
                                                                                           9),
                                                                                          ('r1024',
                                                                                           10),
                                                                                          ('r2048',
                                                                                           11),
                                                                                          ('spare4',
                                                                                           12),
                                                                                          ('spare3',
                                                                                           13),
                                                                                          ('spare2',
                                                                                           14),
                                                                                          ('spare1',
                                                                                           15)]},
                                                                              {'name': 'npdcch-StartSF-CSS-RA-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('v1dot5',
                                                                                           0),
                                                                                          ('v2',
                                                                                           1),
                                                                                          ('v4',
                                                                                           2),
                                                                                          ('v8',
                                                                                           3),
                                                                                          ('v16',
                                                                                           4),
                                                                                          ('v32',
                                                                                           5),
                                                                                          ('v48',
                                                                                           6),
                                                                                          ('v64',
                                                                                           7)]},
                                                                              {'name': 'npdcch-Offset-RA-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('zero',
                                                                                           0),
                                                                                          ('oneEighth',
                                                                                           1),
                                                                                          ('oneFourth',
                                                                                           2),
                                                                                          ('threeEighth',
                                                                                           3)]}],
                                                                  'type': 'SEQUENCE'},
                                     'NPRACH-Parameters-NB-r14': {'members': [{'members': [{'name': 'nprach-Periodicity-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('ms40',
                                                                                                        0),
                                                                                                       ('ms80',
                                                                                                        1),
                                                                                                       ('ms160',
                                                                                                        2),
                                                                                                       ('ms240',
                                                                                                        3),
                                                                                                       ('ms320',
                                                                                                        4),
                                                                                                       ('ms640',
                                                                                                        5),
                                                                                                       ('ms1280',
                                                                                                        6),
                                                                                                       ('ms2560',
                                                                                                        7)]},
                                                                                           {'name': 'nprach-StartTime-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('ms8',
                                                                                                        0),
                                                                                                       ('ms16',
                                                                                                        1),
                                                                                                       ('ms32',
                                                                                                        2),
                                                                                                       ('ms64',
                                                                                                        3),
                                                                                                       ('ms128',
                                                                                                        4),
                                                                                                       ('ms256',
                                                                                                        5),
                                                                                                       ('ms512',
                                                                                                        6),
                                                                                                       ('ms1024',
                                                                                                        7)]},
                                                                                           {'name': 'nprach-SubcarrierOffset-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('n0',
                                                                                                        0),
                                                                                                       ('n12',
                                                                                                        1),
                                                                                                       ('n24',
                                                                                                        2),
                                                                                                       ('n36',
                                                                                                        3),
                                                                                                       ('n2',
                                                                                                        4),
                                                                                                       ('n18',
                                                                                                        5),
                                                                                                       ('n34',
                                                                                                        6),
                                                                                                       ('spare1',
                                                                                                        7)]},
                                                                                           {'name': 'nprach-NumSubcarriers-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('n12',
                                                                                                        0),
                                                                                                       ('n24',
                                                                                                        1),
                                                                                                       ('n36',
                                                                                                        2),
                                                                                                       ('n48',
                                                                                                        3)]},
                                                                                           {'name': 'nprach-SubcarrierMSG3-RangeStart-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('zero',
                                                                                                        0),
                                                                                                       ('oneThird',
                                                                                                        1),
                                                                                                       ('twoThird',
                                                                                                        2),
                                                                                                       ('one',
                                                                                                        3)]},
                                                                                           {'name': 'npdcch-NumRepetitions-RA-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('r1',
                                                                                                        0),
                                                                                                       ('r2',
                                                                                                        1),
                                                                                                       ('r4',
                                                                                                        2),
                                                                                                       ('r8',
                                                                                                        3),
                                                                                                       ('r16',
                                                                                                        4),
                                                                                                       ('r32',
                                                                                                        5),
                                                                                                       ('r64',
                                                                                                        6),
                                                                                                       ('r128',
                                                                                                        7),
                                                                                                       ('r256',
                                                                                                        8),
                                                                                                       ('r512',
                                                                                                        9),
                                                                                                       ('r1024',
                                                                                                        10),
                                                                                                       ('r2048',
                                                                                                        11),
                                                                                                       ('spare4',
                                                                                                        12),
                                                                                                       ('spare3',
                                                                                                        13),
                                                                                                       ('spare2',
                                                                                                        14),
                                                                                                       ('spare1',
                                                                                                        15)]},
                                                                                           {'name': 'npdcch-StartSF-CSS-RA-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('v1dot5',
                                                                                                        0),
                                                                                                       ('v2',
                                                                                                        1),
                                                                                                       ('v4',
                                                                                                        2),
                                                                                                       ('v8',
                                                                                                        3),
                                                                                                       ('v16',
                                                                                                        4),
                                                                                                       ('v32',
                                                                                                        5),
                                                                                                       ('v48',
                                                                                                        6),
                                                                                                       ('v64',
                                                                                                        7)]},
                                                                                           {'name': 'npdcch-Offset-RA-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('zero',
                                                                                                        0),
                                                                                                       ('oneEighth',
                                                                                                        1),
                                                                                                       ('oneFourth',
                                                                                                        2),
                                                                                                       ('threeEighth',
                                                                                                        3)]},
                                                                                           {'name': 'nprach-NumCBRA-StartSubcarriers-r14',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('n8',
                                                                                                        0),
                                                                                                       ('n10',
                                                                                                        1),
                                                                                                       ('n11',
                                                                                                        2),
                                                                                                       ('n12',
                                                                                                        3),
                                                                                                       ('n20',
                                                                                                        4),
                                                                                                       ('n22',
                                                                                                        5),
                                                                                                       ('n23',
                                                                                                        6),
                                                                                                       ('n24',
                                                                                                        7),
                                                                                                       ('n32',
                                                                                                        8),
                                                                                                       ('n34',
                                                                                                        9),
                                                                                                       ('n35',
                                                                                                        10),
                                                                                                       ('n36',
                                                                                                        11),
                                                                                                       ('n40',
                                                                                                        12),
                                                                                                       ('n44',
                                                                                                        13),
                                                                                                       ('n46',
                                                                                                        14),
                                                                                                       ('n48',
                                                                                                        15)]},
                                                                                           {'name': 'npdcch-CarrierIndex-r14',
                                                                                            'optional': True,
                                                                                            'restricted-to': [(1,
                                                                                                               'maxNonAnchorCarriers-NB-r14')],
                                                                                            'type': 'INTEGER'},
                                                                                           None],
                                                                               'name': 'nprach-Parameters-r14',
                                                                               'optional': True,
                                                                               'type': 'SEQUENCE'}],
                                                                  'type': 'SEQUENCE'},
                                     'NPRACH-Parameters-NB-v1330': {'members': [{'name': 'nprach-NumCBRA-StartSubcarriers-r13',
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('n8',
                                                                                             0),
                                                                                            ('n10',
                                                                                             1),
                                                                                            ('n11',
                                                                                             2),
                                                                                            ('n12',
                                                                                             3),
                                                                                            ('n20',
                                                                                             4),
                                                                                            ('n22',
                                                                                             5),
                                                                                            ('n23',
                                                                                             6),
                                                                                            ('n24',
                                                                                             7),
                                                                                            ('n32',
                                                                                             8),
                                                                                            ('n34',
                                                                                             9),
                                                                                            ('n35',
                                                                                             10),
                                                                                            ('n36',
                                                                                             11),
                                                                                            ('n40',
                                                                                             12),
                                                                                            ('n44',
                                                                                             13),
                                                                                            ('n46',
                                                                                             14),
                                                                                            ('n48',
                                                                                             15)]}],
                                                                    'type': 'SEQUENCE'},
                                     'NPRACH-ParametersList-NB-r13': {'element': {'type': 'NPRACH-Parameters-NB-r13'},
                                                                      'size': [(1,
                                                                                'maxNPRACH-Resources-NB-r13')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'NPRACH-ParametersList-NB-r14': {'element': {'type': 'NPRACH-Parameters-NB-r14'},
                                                                      'size': [(1,
                                                                                'maxNPRACH-Resources-NB-r13')],
                                                                      'type': 'SEQUENCE '
                                                                              'OF'},
                                     'NPRACH-ParametersList-NB-v1330': {'element': {'type': 'NPRACH-Parameters-NB-v1330'},
                                                                        'size': [(1,
                                                                                  'maxNPRACH-Resources-NB-r13')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'NPRACH-ProbabilityAnchor-NB-r14': {'members': [{'name': 'nprach-ProbabilityAnchor-r14',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('zero',
                                                                                                  0),
                                                                                                 ('oneSixteenth',
                                                                                                  1),
                                                                                                 ('oneFifteenth',
                                                                                                  2),
                                                                                                 ('oneFourteenth',
                                                                                                  3),
                                                                                                 ('oneThirteenth',
                                                                                                  4),
                                                                                                 ('oneTwelfth',
                                                                                                  5),
                                                                                                 ('oneEleventh',
                                                                                                  6),
                                                                                                 ('oneTenth',
                                                                                                  7),
                                                                                                 ('oneNinth',
                                                                                                  8),
                                                                                                 ('oneEighth',
                                                                                                  9),
                                                                                                 ('oneSeventh',
                                                                                                  10),
                                                                                                 ('oneSixth',
                                                                                                  11),
                                                                                                 ('oneFifth',
                                                                                                  12),
                                                                                                 ('oneFourth',
                                                                                                  13),
                                                                                                 ('oneThird',
                                                                                                  14),
                                                                                                 ('oneHalf',
                                                                                                  15)]}],
                                                                         'type': 'SEQUENCE'},
                                     'NPRACH-ProbabilityAnchorList-NB-r14': {'element': {'type': 'NPRACH-ProbabilityAnchor-NB-r14'},
                                                                             'size': [(1,
                                                                                       'maxNPRACH-Resources-NB-r13')],
                                                                             'type': 'SEQUENCE '
                                                                                     'OF'},
                                     'NPUSCH-ConfigCommon-NB-r13': {'members': [{'element': {'type': 'ACK-NACK-NumRepetitions-NB-r13'},
                                                                                 'name': 'ack-NACK-NumRepetitions-Msg4-r13',
                                                                                 'size': [(1,
                                                                                           'maxNPRACH-Resources-NB-r13')],
                                                                                 'type': 'SEQUENCE '
                                                                                         'OF'},
                                                                                {'name': 'srs-SubframeConfig-r13',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('sc0',
                                                                                             0),
                                                                                            ('sc1',
                                                                                             1),
                                                                                            ('sc2',
                                                                                             2),
                                                                                            ('sc3',
                                                                                             3),
                                                                                            ('sc4',
                                                                                             4),
                                                                                            ('sc5',
                                                                                             5),
                                                                                            ('sc6',
                                                                                             6),
                                                                                            ('sc7',
                                                                                             7),
                                                                                            ('sc8',
                                                                                             8),
                                                                                            ('sc9',
                                                                                             9),
                                                                                            ('sc10',
                                                                                             10),
                                                                                            ('sc11',
                                                                                             11),
                                                                                            ('sc12',
                                                                                             12),
                                                                                            ('sc13',
                                                                                             13),
                                                                                            ('sc14',
                                                                                             14),
                                                                                            ('sc15',
                                                                                             15)]},
                                                                                {'members': [{'name': 'threeTone-BaseSequence-r13',
                                                                                              'optional': True,
                                                                                              'restricted-to': [(0,
                                                                                                                 12)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'threeTone-CyclicShift-r13',
                                                                                              'restricted-to': [(0,
                                                                                                                 2)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sixTone-BaseSequence-r13',
                                                                                              'optional': True,
                                                                                              'restricted-to': [(0,
                                                                                                                 14)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'sixTone-CyclicShift-r13',
                                                                                              'restricted-to': [(0,
                                                                                                                 3)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'name': 'twelveTone-BaseSequence-r13',
                                                                                              'optional': True,
                                                                                              'restricted-to': [(0,
                                                                                                                 30)],
                                                                                              'type': 'INTEGER'}],
                                                                                 'name': 'dmrs-Config-r13',
                                                                                 'optional': True,
                                                                                 'type': 'SEQUENCE'},
                                                                                {'name': 'ul-ReferenceSignalsNPUSCH-r13',
                                                                                 'type': 'UL-ReferenceSignalsNPUSCH-NB-r13'}],
                                                                    'type': 'SEQUENCE'},
                                     'NPUSCH-ConfigDedicated-NB-r13': {'members': [{'name': 'ack-NACK-NumRepetitions-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ACK-NACK-NumRepetitions-NB-r13'},
                                                                                   {'name': 'npusch-AllSymbols-r13',
                                                                                    'optional': True,
                                                                                    'type': 'BOOLEAN'},
                                                                                   {'name': 'groupHoppingDisabled-r13',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('true',
                                                                                                0)]}],
                                                                       'type': 'SEQUENCE'},
                                     'NS-PmaxList-NB-r13': {'element': {'type': 'NS-PmaxValue-NB-r13'},
                                                            'size': [(1,
                                                                      'maxNS-Pmax-NB-r13')],
                                                            'type': 'SEQUENCE '
                                                                    'OF'},
                                     'NS-PmaxValue-NB-r13': {'members': [{'name': 'additionalPmax-r13',
                                                                          'optional': True,
                                                                          'type': 'P-Max'},
                                                                         {'name': 'additionalSpectrumEmission-r13',
                                                                          'type': 'AdditionalSpectrumEmission'}],
                                                             'type': 'SEQUENCE'},
                                     'PCCH-Config-NB-r13': {'members': [{'name': 'defaultPagingCycle-r13',
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('rf128',
                                                                                     0),
                                                                                    ('rf256',
                                                                                     1),
                                                                                    ('rf512',
                                                                                     2),
                                                                                    ('rf1024',
                                                                                     3)]},
                                                                        {'name': 'nB-r13',
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('fourT',
                                                                                     0),
                                                                                    ('twoT',
                                                                                     1),
                                                                                    ('oneT',
                                                                                     2),
                                                                                    ('halfT',
                                                                                     3),
                                                                                    ('quarterT',
                                                                                     4),
                                                                                    ('one8thT',
                                                                                     5),
                                                                                    ('one16thT',
                                                                                     6),
                                                                                    ('one32ndT',
                                                                                     7),
                                                                                    ('one64thT',
                                                                                     8),
                                                                                    ('one128thT',
                                                                                     9),
                                                                                    ('one256thT',
                                                                                     10),
                                                                                    ('one512thT',
                                                                                     11),
                                                                                    ('one1024thT',
                                                                                     12),
                                                                                    ('spare3',
                                                                                     13),
                                                                                    ('spare2',
                                                                                     14),
                                                                                    ('spare1',
                                                                                     15)]},
                                                                        {'name': 'npdcch-NumRepetitionPaging-r13',
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('r1',
                                                                                     0),
                                                                                    ('r2',
                                                                                     1),
                                                                                    ('r4',
                                                                                     2),
                                                                                    ('r8',
                                                                                     3),
                                                                                    ('r16',
                                                                                     4),
                                                                                    ('r32',
                                                                                     5),
                                                                                    ('r64',
                                                                                     6),
                                                                                    ('r128',
                                                                                     7),
                                                                                    ('r256',
                                                                                     8),
                                                                                    ('r512',
                                                                                     9),
                                                                                    ('r1024',
                                                                                     10),
                                                                                    ('r2048',
                                                                                     11),
                                                                                    ('spare4',
                                                                                     12),
                                                                                    ('spare3',
                                                                                     13),
                                                                                    ('spare2',
                                                                                     14),
                                                                                    ('spare1',
                                                                                     15)]}],
                                                            'type': 'SEQUENCE'},
                                     'PCCH-Config-NB-r14': {'members': [{'name': 'npdcch-NumRepetitionPaging-r14',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('r1',
                                                                                     0),
                                                                                    ('r2',
                                                                                     1),
                                                                                    ('r4',
                                                                                     2),
                                                                                    ('r8',
                                                                                     3),
                                                                                    ('r16',
                                                                                     4),
                                                                                    ('r32',
                                                                                     5),
                                                                                    ('r64',
                                                                                     6),
                                                                                    ('r128',
                                                                                     7),
                                                                                    ('r256',
                                                                                     8),
                                                                                    ('r512',
                                                                                     9),
                                                                                    ('r1024',
                                                                                     10),
                                                                                    ('r2048',
                                                                                     11),
                                                                                    ('spare4',
                                                                                     12),
                                                                                    ('spare3',
                                                                                     13),
                                                                                    ('spare2',
                                                                                     14),
                                                                                    ('spare1',
                                                                                     15)]},
                                                                        {'default': 'w1',
                                                                         'name': 'pagingWeight-r14',
                                                                         'type': 'PagingWeight-NB-r14'},
                                                                        None],
                                                            'type': 'SEQUENCE'},
                                     'PCCH-Message-NB': {'members': [{'name': 'message',
                                                                      'type': 'PCCH-MessageType-NB'}],
                                                         'type': 'SEQUENCE'},
                                     'PCCH-MessageType-NB': {'members': [{'members': [{'name': 'paging-r13',
                                                                                       'type': 'Paging-NB'}],
                                                                          'name': 'c1',
                                                                          'type': 'CHOICE'},
                                                                         {'members': [],
                                                                          'name': 'messageClassExtension',
                                                                          'type': 'SEQUENCE'}],
                                                             'type': 'CHOICE'},
                                     'PCI-ARFCN-NB-r14': {'members': [{'name': 'physCellId-r14',
                                                                       'type': 'PhysCellId'},
                                                                      {'name': 'carrierFreq-r14',
                                                                       'optional': True,
                                                                       'type': 'CarrierFreq-NB-r13'}],
                                                          'type': 'SEQUENCE'},
                                     'PDCP-Config-NB-r13': {'members': [{'name': 'discardTimer-r13',
                                                                         'optional': True,
                                                                         'type': 'ENUMERATED',
                                                                         'values': [('ms5120',
                                                                                     0),
                                                                                    ('ms10240',
                                                                                     1),
                                                                                    ('ms20480',
                                                                                     2),
                                                                                    ('ms40960',
                                                                                     3),
                                                                                    ('ms81920',
                                                                                     4),
                                                                                    ('infinity',
                                                                                     5),
                                                                                    ('spare2',
                                                                                     6),
                                                                                    ('spare1',
                                                                                     7)]},
                                                                        {'members': [{'name': 'notUsed',
                                                                                      'type': 'NULL'},
                                                                                     {'members': [{'default': 15,
                                                                                                   'name': 'maxCID-r13',
                                                                                                   'restricted-to': [(1,
                                                                                                                      16383)],
                                                                                                   'type': 'INTEGER'},
                                                                                                  {'members': [{'name': 'profile0x0002',
                                                                                                                'type': 'BOOLEAN'},
                                                                                                               {'name': 'profile0x0003',
                                                                                                                'type': 'BOOLEAN'},
                                                                                                               {'name': 'profile0x0004',
                                                                                                                'type': 'BOOLEAN'},
                                                                                                               {'name': 'profile0x0006',
                                                                                                                'type': 'BOOLEAN'},
                                                                                                               {'name': 'profile0x0102',
                                                                                                                'type': 'BOOLEAN'},
                                                                                                               {'name': 'profile0x0103',
                                                                                                                'type': 'BOOLEAN'},
                                                                                                               {'name': 'profile0x0104',
                                                                                                                'type': 'BOOLEAN'}],
                                                                                                   'name': 'profiles-r13',
                                                                                                   'type': 'SEQUENCE'},
                                                                                                  None],
                                                                                      'name': 'rohc',
                                                                                      'type': 'SEQUENCE'}],
                                                                         'name': 'headerCompression-r13',
                                                                         'type': 'CHOICE'},
                                                                        None],
                                                            'type': 'SEQUENCE'},
                                     'PDCP-Parameters-NB-r13': {'members': [{'members': [{'name': 'profile0x0002',
                                                                                          'type': 'BOOLEAN'},
                                                                                         {'name': 'profile0x0003',
                                                                                          'type': 'BOOLEAN'},
                                                                                         {'name': 'profile0x0004',
                                                                                          'type': 'BOOLEAN'},
                                                                                         {'name': 'profile0x0006',
                                                                                          'type': 'BOOLEAN'},
                                                                                         {'name': 'profile0x0102',
                                                                                          'type': 'BOOLEAN'},
                                                                                         {'name': 'profile0x0103',
                                                                                          'type': 'BOOLEAN'},
                                                                                         {'name': 'profile0x0104',
                                                                                          'type': 'BOOLEAN'}],
                                                                             'name': 'supportedROHC-Profiles-r13',
                                                                             'type': 'SEQUENCE'},
                                                                            {'default': 'cs2',
                                                                             'name': 'maxNumberROHC-ContextSessions-r13',
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('cs2',
                                                                                         0),
                                                                                        ('cs4',
                                                                                         1),
                                                                                        ('cs8',
                                                                                         2),
                                                                                        ('cs12',
                                                                                         3)]},
                                                                            None],
                                                                'type': 'SEQUENCE'},
                                     'PLMN-IdentityInfo-NB-r13': {'members': [{'name': 'plmn-Identity-r13',
                                                                               'type': 'PLMN-Identity'},
                                                                              {'name': 'cellReservedForOperatorUse-r13',
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('reserved',
                                                                                           0),
                                                                                          ('notReserved',
                                                                                           1)]},
                                                                              {'name': 'attachWithoutPDN-Connectivity-r13',
                                                                               'optional': True,
                                                                               'type': 'ENUMERATED',
                                                                               'values': [('true',
                                                                                           0)]}],
                                                                  'type': 'SEQUENCE'},
                                     'PLMN-IdentityList-NB-r13': {'element': {'type': 'PLMN-IdentityInfo-NB-r13'},
                                                                  'size': [(1,
                                                                            'maxPLMN-r11')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'Paging-NB': {'members': [{'name': 'pagingRecordList-r13',
                                                                'optional': True,
                                                                'type': 'PagingRecordList-NB-r13'},
                                                               {'name': 'systemInfoModification-r13',
                                                                'optional': True,
                                                                'type': 'ENUMERATED',
                                                                'values': [('true',
                                                                            0)]},
                                                               {'name': 'systemInfoModification-eDRX-r13',
                                                                'optional': True,
                                                                'type': 'ENUMERATED',
                                                                'values': [('true',
                                                                            0)]},
                                                               {'members': [],
                                                                'name': 'nonCriticalExtension',
                                                                'optional': True,
                                                                'type': 'SEQUENCE'}],
                                                   'type': 'SEQUENCE'},
                                     'PagingRecord-NB-r13': {'members': [{'name': 'ue-Identity-r13',
                                                                          'type': 'PagingUE-Identity'},
                                                                         None],
                                                             'type': 'SEQUENCE'},
                                     'PagingRecordList-NB-r13': {'element': {'type': 'PagingRecord-NB-r13'},
                                                                 'size': [(1,
                                                                           'maxPageRec')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'PagingWeight-NB-r14': {'type': 'ENUMERATED',
                                                             'values': [('w1',
                                                                         0),
                                                                        ('w2',
                                                                         1),
                                                                        ('w3',
                                                                         2),
                                                                        ('w4',
                                                                         3),
                                                                        ('w5',
                                                                         4),
                                                                        ('w6',
                                                                         5),
                                                                        ('w7',
                                                                         6),
                                                                        ('w8',
                                                                         7),
                                                                        ('w9',
                                                                         8),
                                                                        ('w10',
                                                                         9),
                                                                        ('w11',
                                                                         10),
                                                                        ('w12',
                                                                         11),
                                                                        ('w13',
                                                                         12),
                                                                        ('w14',
                                                                         13),
                                                                        ('w15',
                                                                         14),
                                                                        ('w16',
                                                                         15)]},
                                     'PeriodicBSR-Timer-NB-r13': {'type': 'ENUMERATED',
                                                                  'values': [('pp2',
                                                                              0),
                                                                             ('pp4',
                                                                              1),
                                                                             ('pp8',
                                                                              2),
                                                                             ('pp16',
                                                                              3),
                                                                             ('pp64',
                                                                              4),
                                                                             ('pp128',
                                                                              5),
                                                                             ('infinity',
                                                                              6),
                                                                             ('spare',
                                                                              7)]},
                                     'PhyLayerParameters-NB-r13': {'members': [{'name': 'multiTone-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('supported',
                                                                                            0)]},
                                                                               {'name': 'multiCarrier-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('supported',
                                                                                            0)]}],
                                                                   'type': 'SEQUENCE'},
                                     'PhyLayerParameters-NB-v1430': {'members': [{'name': 'multiCarrier-NPRACH-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('supported',
                                                                                              0)]},
                                                                                 {'name': 'twoHARQ-Processes-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('supported',
                                                                                              0)]}],
                                                                     'type': 'SEQUENCE'},
                                     'PhyLayerParameters-NB-v1440': {'members': [{'name': 'interferenceRandomisation-r14',
                                                                                  'optional': True,
                                                                                  'type': 'ENUMERATED',
                                                                                  'values': [('supported',
                                                                                              0)]}],
                                                                     'type': 'SEQUENCE'},
                                     'PhysicalConfigDedicated-NB-r13': {'members': [{'name': 'carrierConfigDedicated-r13',
                                                                                     'optional': True,
                                                                                     'type': 'CarrierConfigDedicated-NB-r13'},
                                                                                    {'name': 'npdcch-ConfigDedicated-r13',
                                                                                     'optional': True,
                                                                                     'type': 'NPDCCH-ConfigDedicated-NB-r13'},
                                                                                    {'name': 'npusch-ConfigDedicated-r13',
                                                                                     'optional': True,
                                                                                     'type': 'NPUSCH-ConfigDedicated-NB-r13'},
                                                                                    {'name': 'uplinkPowerControlDedicated-r13',
                                                                                     'optional': True,
                                                                                     'type': 'UplinkPowerControlDedicated-NB-r13'},
                                                                                    None,
                                                                                    [{'name': 'twoHARQ-ProcessesConfig-r14',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('true',
                                                                                                  0)]}],
                                                                                    [{'name': 'interferenceRandomisationConfig-r14',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('true',
                                                                                                  0)]}]],
                                                                        'type': 'SEQUENCE'},
                                     'RACH-ConfigCommon-NB-r13': {'members': [{'name': 'preambleTransMax-CE-r13',
                                                                               'type': 'PreambleTransMax'},
                                                                              {'name': 'powerRampingParameters-r13',
                                                                               'type': 'PowerRampingParameters'},
                                                                              {'name': 'rach-InfoList-r13',
                                                                               'type': 'RACH-InfoList-NB-r13'},
                                                                              {'name': 'connEstFailOffset-r13',
                                                                               'optional': True,
                                                                               'restricted-to': [(0,
                                                                                                  15)],
                                                                               'type': 'INTEGER'},
                                                                              None],
                                                                  'type': 'SEQUENCE'},
                                     'RACH-Info-NB-r13': {'members': [{'name': 'ra-ResponseWindowSize-r13',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('pp2',
                                                                                   0),
                                                                                  ('pp3',
                                                                                   1),
                                                                                  ('pp4',
                                                                                   2),
                                                                                  ('pp5',
                                                                                   3),
                                                                                  ('pp6',
                                                                                   4),
                                                                                  ('pp7',
                                                                                   5),
                                                                                  ('pp8',
                                                                                   6),
                                                                                  ('pp10',
                                                                                   7)]},
                                                                      {'name': 'mac-ContentionResolutionTimer-r13',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('pp1',
                                                                                   0),
                                                                                  ('pp2',
                                                                                   1),
                                                                                  ('pp3',
                                                                                   2),
                                                                                  ('pp4',
                                                                                   3),
                                                                                  ('pp8',
                                                                                   4),
                                                                                  ('pp16',
                                                                                   5),
                                                                                  ('pp32',
                                                                                   6),
                                                                                  ('pp64',
                                                                                   7)]}],
                                                          'type': 'SEQUENCE'},
                                     'RACH-InfoList-NB-r13': {'element': {'type': 'RACH-Info-NB-r13'},
                                                              'size': [(1,
                                                                        'maxNPRACH-Resources-NB-r13')],
                                                              'type': 'SEQUENCE '
                                                                      'OF'},
                                     'RF-Parameters-NB-r13': {'members': [{'name': 'supportedBandList-r13',
                                                                           'type': 'SupportedBandList-NB-r13'},
                                                                          {'name': 'multiNS-Pmax-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'RF-Parameters-NB-v1430': {'members': [{'name': 'powerClassNB-14dBm-r14',
                                                                             'optional': True,
                                                                             'type': 'ENUMERATED',
                                                                             'values': [('supported',
                                                                                         0)]}],
                                                                'type': 'SEQUENCE'},
                                     'RLC-Config-NB-r13': {'members': [{'members': [{'name': 'ul-AM-RLC-r13',
                                                                                     'type': 'UL-AM-RLC-NB-r13'},
                                                                                    {'name': 'dl-AM-RLC-r13',
                                                                                     'type': 'DL-AM-RLC-NB-r13'}],
                                                                        'name': 'am',
                                                                        'type': 'SEQUENCE'},
                                                                       None],
                                                           'type': 'CHOICE'},
                                     'RLC-Config-NB-v1430': {'members': [{'name': 't-Reordering-r14',
                                                                          'optional': True,
                                                                          'type': 'T-Reordering'}],
                                                             'type': 'SEQUENCE'},
                                     'RLF-TimersAndConstants-NB-r13': {'members': [{'name': 'release',
                                                                                    'type': 'NULL'},
                                                                                   {'members': [{'name': 't301-r13',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('ms2500',
                                                                                                             0),
                                                                                                            ('ms4000',
                                                                                                             1),
                                                                                                            ('ms6000',
                                                                                                             2),
                                                                                                            ('ms10000',
                                                                                                             3),
                                                                                                            ('ms15000',
                                                                                                             4),
                                                                                                            ('ms25000',
                                                                                                             5),
                                                                                                            ('ms40000',
                                                                                                             6),
                                                                                                            ('ms60000',
                                                                                                             7)]},
                                                                                                {'name': 't310-r13',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('ms0',
                                                                                                             0),
                                                                                                            ('ms200',
                                                                                                             1),
                                                                                                            ('ms500',
                                                                                                             2),
                                                                                                            ('ms1000',
                                                                                                             3),
                                                                                                            ('ms2000',
                                                                                                             4),
                                                                                                            ('ms4000',
                                                                                                             5),
                                                                                                            ('ms8000',
                                                                                                             6)]},
                                                                                                {'name': 'n310-r13',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('n1',
                                                                                                             0),
                                                                                                            ('n2',
                                                                                                             1),
                                                                                                            ('n3',
                                                                                                             2),
                                                                                                            ('n4',
                                                                                                             3),
                                                                                                            ('n6',
                                                                                                             4),
                                                                                                            ('n8',
                                                                                                             5),
                                                                                                            ('n10',
                                                                                                             6),
                                                                                                            ('n20',
                                                                                                             7)]},
                                                                                                {'name': 't311-r13',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('ms1000',
                                                                                                             0),
                                                                                                            ('ms3000',
                                                                                                             1),
                                                                                                            ('ms5000',
                                                                                                             2),
                                                                                                            ('ms10000',
                                                                                                             3),
                                                                                                            ('ms15000',
                                                                                                             4),
                                                                                                            ('ms20000',
                                                                                                             5),
                                                                                                            ('ms30000',
                                                                                                             6)]},
                                                                                                {'name': 'n311-r13',
                                                                                                 'type': 'ENUMERATED',
                                                                                                 'values': [('n1',
                                                                                                             0),
                                                                                                            ('n2',
                                                                                                             1),
                                                                                                            ('n3',
                                                                                                             2),
                                                                                                            ('n4',
                                                                                                             3),
                                                                                                            ('n5',
                                                                                                             4),
                                                                                                            ('n6',
                                                                                                             5),
                                                                                                            ('n8',
                                                                                                             6),
                                                                                                            ('n10',
                                                                                                             7)]},
                                                                                                None,
                                                                                                [{'name': 't311-v1350',
                                                                                                  'optional': True,
                                                                                                  'type': 'ENUMERATED',
                                                                                                  'values': [('ms40000',
                                                                                                              0),
                                                                                                             ('ms60000',
                                                                                                              1),
                                                                                                             ('ms90000',
                                                                                                              2),
                                                                                                             ('ms120000',
                                                                                                              3)]}]],
                                                                                    'name': 'setup',
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'CHOICE'},
                                     'RRCConnectionReconfiguration-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                      'type': 'RRC-TransactionIdentifier'},
                                                                                     {'members': [{'members': [{'name': 'rrcConnectionReconfiguration-r13',
                                                                                                                'type': 'RRCConnectionReconfiguration-NB-r13-IEs'},
                                                                                                               {'name': 'spare1',
                                                                                                                'type': 'NULL'}],
                                                                                                   'name': 'c1',
                                                                                                   'type': 'CHOICE'},
                                                                                                  {'members': [],
                                                                                                   'name': 'criticalExtensionsFuture',
                                                                                                   'type': 'SEQUENCE'}],
                                                                                      'name': 'criticalExtensions',
                                                                                      'type': 'CHOICE'}],
                                                                         'type': 'SEQUENCE'},
                                     'RRCConnectionReconfiguration-NB-r13-IEs': {'members': [{'element': {'type': 'DedicatedInfoNAS'},
                                                                                              'name': 'dedicatedInfoNASList-r13',
                                                                                              'optional': True,
                                                                                              'size': [(1,
                                                                                                        'maxDRB-NB-r13')],
                                                                                              'type': 'SEQUENCE '
                                                                                                      'OF'},
                                                                                             {'name': 'radioResourceConfigDedicated-r13',
                                                                                              'optional': True,
                                                                                              'type': 'RadioResourceConfigDedicated-NB-r13'},
                                                                                             {'name': 'fullConfig-r13',
                                                                                              'optional': True,
                                                                                              'type': 'ENUMERATED',
                                                                                              'values': [('true',
                                                                                                          0)]},
                                                                                             {'name': 'lateNonCriticalExtension',
                                                                                              'optional': True,
                                                                                              'type': 'OCTET '
                                                                                                      'STRING'},
                                                                                             {'members': [],
                                                                                              'name': 'nonCriticalExtension',
                                                                                              'optional': True,
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'type': 'SEQUENCE'},
                                     'RRCConnectionReconfigurationComplete-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                              'type': 'RRC-TransactionIdentifier'},
                                                                                             {'members': [{'name': 'rrcConnectionReconfigurationComplete-r13',
                                                                                                           'type': 'RRCConnectionReconfigurationComplete-NB-r13-IEs'},
                                                                                                          {'members': [],
                                                                                                           'name': 'criticalExtensionsFuture',
                                                                                                           'type': 'SEQUENCE'}],
                                                                                              'name': 'criticalExtensions',
                                                                                              'type': 'CHOICE'}],
                                                                                 'type': 'SEQUENCE'},
                                     'RRCConnectionReconfigurationComplete-NB-r13-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                                      'optional': True,
                                                                                                      'type': 'OCTET '
                                                                                                              'STRING'},
                                                                                                     {'members': [],
                                                                                                      'name': 'nonCriticalExtension',
                                                                                                      'optional': True,
                                                                                                      'type': 'SEQUENCE'}],
                                                                                         'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishment-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                      'type': 'RRC-TransactionIdentifier'},
                                                                                     {'members': [{'members': [{'name': 'rrcConnectionReestablishment-r13',
                                                                                                                'type': 'RRCConnectionReestablishment-NB-r13-IEs'},
                                                                                                               {'name': 'spare1',
                                                                                                                'type': 'NULL'}],
                                                                                                   'name': 'c1',
                                                                                                   'type': 'CHOICE'},
                                                                                                  {'members': [],
                                                                                                   'name': 'criticalExtensionsFuture',
                                                                                                   'type': 'SEQUENCE'}],
                                                                                      'name': 'criticalExtensions',
                                                                                      'type': 'CHOICE'}],
                                                                         'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishment-NB-r13-IEs': {'members': [{'name': 'radioResourceConfigDedicated-r13',
                                                                                              'type': 'RadioResourceConfigDedicated-NB-r13'},
                                                                                             {'name': 'nextHopChainingCount-r13',
                                                                                              'type': 'NextHopChainingCount'},
                                                                                             {'name': 'lateNonCriticalExtension',
                                                                                              'optional': True,
                                                                                              'type': 'OCTET '
                                                                                                      'STRING'},
                                                                                             {'name': 'nonCriticalExtension',
                                                                                              'optional': True,
                                                                                              'type': 'RRCConnectionReestablishment-NB-v1430-IEs'}],
                                                                                 'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishment-NB-v1430-IEs': {'members': [{'name': 'dl-NAS-MAC',
                                                                                                'size': [16],
                                                                                                'type': 'BIT '
                                                                                                        'STRING'},
                                                                                               {'members': [],
                                                                                                'name': 'nonCriticalExtension',
                                                                                                'optional': True,
                                                                                                'type': 'SEQUENCE'}],
                                                                                   'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentComplete-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                              'type': 'RRC-TransactionIdentifier'},
                                                                                             {'members': [{'name': 'rrcConnectionReestablishmentComplete-r13',
                                                                                                           'type': 'RRCConnectionReestablishmentComplete-NB-r13-IEs'},
                                                                                                          {'members': [],
                                                                                                           'name': 'criticalExtensionsFuture',
                                                                                                           'type': 'SEQUENCE'}],
                                                                                              'name': 'criticalExtensions',
                                                                                              'type': 'CHOICE'}],
                                                                                 'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentComplete-NB-r13-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                                      'optional': True,
                                                                                                      'type': 'OCTET '
                                                                                                              'STRING'},
                                                                                                     {'members': [],
                                                                                                      'name': 'nonCriticalExtension',
                                                                                                      'optional': True,
                                                                                                      'type': 'SEQUENCE'}],
                                                                                         'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentRequest-NB': {'members': [{'members': [{'name': 'rrcConnectionReestablishmentRequest-r13',
                                                                                                          'type': 'RRCConnectionReestablishmentRequest-NB-r13-IEs'},
                                                                                                         {'members': [{'name': 'rrcConnectionReestablishmentRequest-r14',
                                                                                                                       'type': 'RRCConnectionReestablishmentRequest-NB-r14-IEs'},
                                                                                                                      {'members': [],
                                                                                                                       'name': 'criticalExtensionsFuture',
                                                                                                                       'type': 'SEQUENCE'}],
                                                                                                          'name': 'later',
                                                                                                          'type': 'CHOICE'}],
                                                                                             'name': 'criticalExtensions',
                                                                                             'type': 'CHOICE'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentRequest-NB-r13-IEs': {'members': [{'name': 'ue-Identity-r13',
                                                                                                     'type': 'ReestabUE-Identity'},
                                                                                                    {'name': 'reestablishmentCause-r13',
                                                                                                     'type': 'ReestablishmentCause-NB-r13'},
                                                                                                    {'name': 'spare',
                                                                                                     'size': [25],
                                                                                                     'type': 'BIT '
                                                                                                             'STRING'}],
                                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionReestablishmentRequest-NB-r14-IEs': {'members': [{'name': 'ue-Identity-r14',
                                                                                                     'type': 'ReestabUE-Identity-CP-NB-r14'},
                                                                                                    {'name': 'reestablishmentCause-r14',
                                                                                                     'type': 'ReestablishmentCause-NB-r13'},
                                                                                                    {'name': 'spare',
                                                                                                     'size': [4],
                                                                                                     'type': 'BIT '
                                                                                                             'STRING'}],
                                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionReject-NB': {'members': [{'members': [{'members': [{'name': 'rrcConnectionReject-r13',
                                                                                                       'type': 'RRCConnectionReject-NB-r13-IEs'},
                                                                                                      {'name': 'spare1',
                                                                                                       'type': 'NULL'}],
                                                                                          'name': 'c1',
                                                                                          'type': 'CHOICE'},
                                                                                         {'members': [],
                                                                                          'name': 'criticalExtensionsFuture',
                                                                                          'type': 'SEQUENCE'}],
                                                                             'name': 'criticalExtensions',
                                                                             'type': 'CHOICE'}],
                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionReject-NB-r13-IEs': {'members': [{'name': 'extendedWaitTime-r13',
                                                                                     'restricted-to': [(1,
                                                                                                        1800)],
                                                                                     'type': 'INTEGER'},
                                                                                    {'name': 'rrc-SuspendIndication-r13',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('true',
                                                                                                 0)]},
                                                                                    {'name': 'lateNonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'OCTET '
                                                                                             'STRING'},
                                                                                    {'members': [],
                                                                                     'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'}],
                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionRelease-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                              'type': 'RRC-TransactionIdentifier'},
                                                                             {'members': [{'members': [{'name': 'rrcConnectionRelease-r13',
                                                                                                        'type': 'RRCConnectionRelease-NB-r13-IEs'},
                                                                                                       {'name': 'spare1',
                                                                                                        'type': 'NULL'}],
                                                                                           'name': 'c1',
                                                                                           'type': 'CHOICE'},
                                                                                          {'members': [],
                                                                                           'name': 'criticalExtensionsFuture',
                                                                                           'type': 'SEQUENCE'}],
                                                                              'name': 'criticalExtensions',
                                                                              'type': 'CHOICE'}],
                                                                 'type': 'SEQUENCE'},
                                     'RRCConnectionRelease-NB-r13-IEs': {'members': [{'name': 'releaseCause-r13',
                                                                                      'type': 'ReleaseCause-NB-r13'},
                                                                                     {'name': 'resumeIdentity-r13',
                                                                                      'optional': True,
                                                                                      'type': 'ResumeIdentity-r13'},
                                                                                     {'name': 'extendedWaitTime-r13',
                                                                                      'optional': True,
                                                                                      'restricted-to': [(1,
                                                                                                         1800)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'redirectedCarrierInfo-r13',
                                                                                      'optional': True,
                                                                                      'type': 'RedirectedCarrierInfo-NB-r13'},
                                                                                     {'name': 'lateNonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'OCTET '
                                                                                              'STRING'},
                                                                                     {'name': 'nonCriticalExtension',
                                                                                      'optional': True,
                                                                                      'type': 'RRCConnectionRelease-NB-v1430-IEs'}],
                                                                         'type': 'SEQUENCE'},
                                     'RRCConnectionRelease-NB-v1430-IEs': {'members': [{'name': 'redirectedCarrierInfo-v1430',
                                                                                        'optional': True,
                                                                                        'type': 'RedirectedCarrierInfo-NB-v1430'},
                                                                                       {'name': 'extendedWaitTime-CPdata-r14',
                                                                                        'optional': True,
                                                                                        'restricted-to': [(1,
                                                                                                           1800)],
                                                                                        'type': 'INTEGER'},
                                                                                       {'members': [],
                                                                                        'name': 'nonCriticalExtension',
                                                                                        'optional': True,
                                                                                        'type': 'SEQUENCE'}],
                                                                           'type': 'SEQUENCE'},
                                     'RRCConnectionRequest-NB': {'members': [{'members': [{'name': 'rrcConnectionRequest-r13',
                                                                                           'type': 'RRCConnectionRequest-NB-r13-IEs'},
                                                                                          {'members': [],
                                                                                           'name': 'criticalExtensionsFuture',
                                                                                           'type': 'SEQUENCE'}],
                                                                              'name': 'criticalExtensions',
                                                                              'type': 'CHOICE'}],
                                                                 'type': 'SEQUENCE'},
                                     'RRCConnectionRequest-NB-r13-IEs': {'members': [{'name': 'ue-Identity-r13',
                                                                                      'type': 'InitialUE-Identity'},
                                                                                     {'name': 'establishmentCause-r13',
                                                                                      'type': 'EstablishmentCause-NB-r13'},
                                                                                     {'name': 'multiToneSupport-r13',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('true',
                                                                                                  0)]},
                                                                                     {'name': 'multiCarrierSupport-r13',
                                                                                      'optional': True,
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('true',
                                                                                                  0)]},
                                                                                     {'name': 'spare',
                                                                                      'size': [22],
                                                                                      'type': 'BIT '
                                                                                              'STRING'}],
                                                                         'type': 'SEQUENCE'},
                                     'RRCConnectionResume-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                             'type': 'RRC-TransactionIdentifier'},
                                                                            {'members': [{'members': [{'name': 'rrcConnectionResume-r13',
                                                                                                       'type': 'RRCConnectionResume-NB-r13-IEs'},
                                                                                                      {'name': 'spare1',
                                                                                                       'type': 'NULL'}],
                                                                                          'name': 'c1',
                                                                                          'type': 'CHOICE'},
                                                                                         {'members': [],
                                                                                          'name': 'criticalExtensionsFuture',
                                                                                          'type': 'SEQUENCE'}],
                                                                             'name': 'criticalExtensions',
                                                                             'type': 'CHOICE'}],
                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionResume-NB-r13-IEs': {'members': [{'name': 'radioResourceConfigDedicated-r13',
                                                                                     'optional': True,
                                                                                     'type': 'RadioResourceConfigDedicated-NB-r13'},
                                                                                    {'name': 'nextHopChainingCount-r13',
                                                                                     'type': 'NextHopChainingCount'},
                                                                                    {'name': 'drb-ContinueROHC-r13',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('true',
                                                                                                 0)]},
                                                                                    {'name': 'lateNonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'OCTET '
                                                                                             'STRING'},
                                                                                    {'members': [],
                                                                                     'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'}],
                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionResumeComplete-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                     'type': 'RRC-TransactionIdentifier'},
                                                                                    {'members': [{'name': 'rrcConnectionResumeComplete-r13',
                                                                                                  'type': 'RRCConnectionResumeComplete-NB-r13-IEs'},
                                                                                                 {'members': [],
                                                                                                  'name': 'criticalExtensionsFuture',
                                                                                                  'type': 'SEQUENCE'}],
                                                                                     'name': 'criticalExtensions',
                                                                                     'type': 'CHOICE'}],
                                                                        'type': 'SEQUENCE'},
                                     'RRCConnectionResumeComplete-NB-r13-IEs': {'members': [{'name': 'selectedPLMN-Identity-r13',
                                                                                             'optional': True,
                                                                                             'restricted-to': [(1,
                                                                                                                'maxPLMN-r11')],
                                                                                             'type': 'INTEGER'},
                                                                                            {'name': 'dedicatedInfoNAS-r13',
                                                                                             'optional': True,
                                                                                             'type': 'DedicatedInfoNAS'},
                                                                                            {'name': 'lateNonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'OCTET '
                                                                                                     'STRING'},
                                                                                            {'members': [],
                                                                                             'name': 'nonCriticalExtension',
                                                                                             'optional': True,
                                                                                             'type': 'SEQUENCE'}],
                                                                                'type': 'SEQUENCE'},
                                     'RRCConnectionResumeRequest-NB': {'members': [{'members': [{'name': 'rrcConnectionResumeRequest-r13',
                                                                                                 'type': 'RRCConnectionResumeRequest-NB-r13-IEs'},
                                                                                                {'members': [],
                                                                                                 'name': 'criticalExtensionsFuture',
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'name': 'criticalExtensions',
                                                                                    'type': 'CHOICE'}],
                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionResumeRequest-NB-r13-IEs': {'members': [{'name': 'resumeID-r13',
                                                                                            'type': 'ResumeIdentity-r13'},
                                                                                           {'name': 'shortResumeMAC-I-r13',
                                                                                            'type': 'ShortMAC-I'},
                                                                                           {'name': 'resumeCause-r13',
                                                                                            'type': 'EstablishmentCause-NB-r13'},
                                                                                           {'name': 'spare',
                                                                                            'size': [9],
                                                                                            'type': 'BIT '
                                                                                                    'STRING'}],
                                                                               'type': 'SEQUENCE'},
                                     'RRCConnectionSetup-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                            'type': 'RRC-TransactionIdentifier'},
                                                                           {'members': [{'members': [{'name': 'rrcConnectionSetup-r13',
                                                                                                      'type': 'RRCConnectionSetup-NB-r13-IEs'},
                                                                                                     {'name': 'spare1',
                                                                                                      'type': 'NULL'}],
                                                                                         'name': 'c1',
                                                                                         'type': 'CHOICE'},
                                                                                        {'members': [],
                                                                                         'name': 'criticalExtensionsFuture',
                                                                                         'type': 'SEQUENCE'}],
                                                                            'name': 'criticalExtensions',
                                                                            'type': 'CHOICE'}],
                                                               'type': 'SEQUENCE'},
                                     'RRCConnectionSetup-NB-r13-IEs': {'members': [{'name': 'radioResourceConfigDedicated-r13',
                                                                                    'type': 'RadioResourceConfigDedicated-NB-r13'},
                                                                                   {'name': 'lateNonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'OCTET '
                                                                                            'STRING'},
                                                                                   {'members': [],
                                                                                    'name': 'nonCriticalExtension',
                                                                                    'optional': True,
                                                                                    'type': 'SEQUENCE'}],
                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                    'type': 'RRC-TransactionIdentifier'},
                                                                                   {'members': [{'name': 'rrcConnectionSetupComplete-r13',
                                                                                                 'type': 'RRCConnectionSetupComplete-NB-r13-IEs'},
                                                                                                {'members': [],
                                                                                                 'name': 'criticalExtensionsFuture',
                                                                                                 'type': 'SEQUENCE'}],
                                                                                    'name': 'criticalExtensions',
                                                                                    'type': 'CHOICE'}],
                                                                       'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-NB-r13-IEs': {'members': [{'name': 'selectedPLMN-Identity-r13',
                                                                                            'restricted-to': [(1,
                                                                                                               'maxPLMN-r11')],
                                                                                            'type': 'INTEGER'},
                                                                                           {'name': 's-TMSI-r13',
                                                                                            'optional': True,
                                                                                            'type': 'S-TMSI'},
                                                                                           {'name': 'registeredMME-r13',
                                                                                            'optional': True,
                                                                                            'type': 'RegisteredMME'},
                                                                                           {'name': 'dedicatedInfoNAS-r13',
                                                                                            'type': 'DedicatedInfoNAS'},
                                                                                           {'name': 'attachWithoutPDN-Connectivity-r13',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('true',
                                                                                                        0)]},
                                                                                           {'name': 'up-CIoT-EPS-Optimisation-r13',
                                                                                            'optional': True,
                                                                                            'type': 'ENUMERATED',
                                                                                            'values': [('true',
                                                                                                        0)]},
                                                                                           {'name': 'lateNonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'OCTET '
                                                                                                    'STRING'},
                                                                                           {'name': 'nonCriticalExtension',
                                                                                            'optional': True,
                                                                                            'type': 'RRCConnectionSetupComplete-NB-v1430-IEs'}],
                                                                               'type': 'SEQUENCE'},
                                     'RRCConnectionSetupComplete-NB-v1430-IEs': {'members': [{'name': 'gummei-Type-r14',
                                                                                              'optional': True,
                                                                                              'type': 'ENUMERATED',
                                                                                              'values': [('mapped',
                                                                                                          0)]},
                                                                                             {'name': 'dcn-ID-r14',
                                                                                              'optional': True,
                                                                                              'restricted-to': [(0,
                                                                                                                 65535)],
                                                                                              'type': 'INTEGER'},
                                                                                             {'members': [],
                                                                                              'name': 'nonCriticalExtension',
                                                                                              'optional': True,
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'type': 'SEQUENCE'},
                                     'RSRP-ThresholdsNPRACH-InfoList-NB-r13': {'element': {'type': 'RSRP-Range'},
                                                                               'size': [(1,
                                                                                         2)],
                                                                               'type': 'SEQUENCE '
                                                                                       'OF'},
                                     'RadioResourceConfigCommonSIB-NB-r13': {'members': [{'name': 'rach-ConfigCommon-r13',
                                                                                          'type': 'RACH-ConfigCommon-NB-r13'},
                                                                                         {'name': 'bcch-Config-r13',
                                                                                          'type': 'BCCH-Config-NB-r13'},
                                                                                         {'name': 'pcch-Config-r13',
                                                                                          'type': 'PCCH-Config-NB-r13'},
                                                                                         {'name': 'nprach-Config-r13',
                                                                                          'type': 'NPRACH-ConfigSIB-NB-r13'},
                                                                                         {'name': 'npdsch-ConfigCommon-r13',
                                                                                          'type': 'NPDSCH-ConfigCommon-NB-r13'},
                                                                                         {'name': 'npusch-ConfigCommon-r13',
                                                                                          'type': 'NPUSCH-ConfigCommon-NB-r13'},
                                                                                         {'name': 'dl-Gap-r13',
                                                                                          'optional': True,
                                                                                          'type': 'DL-GapConfig-NB-r13'},
                                                                                         {'name': 'uplinkPowerControlCommon-r13',
                                                                                          'type': 'UplinkPowerControlCommon-NB-r13'},
                                                                                         None,
                                                                                         [{'name': 'nprach-Config-v1330',
                                                                                           'optional': True,
                                                                                           'type': 'NPRACH-ConfigSIB-NB-v1330'}]],
                                                                             'type': 'SEQUENCE'},
                                     'RadioResourceConfigDedicated-NB-r13': {'members': [{'name': 'srb-ToAddModList-r13',
                                                                                          'optional': True,
                                                                                          'type': 'SRB-ToAddModList-NB-r13'},
                                                                                         {'name': 'drb-ToAddModList-r13',
                                                                                          'optional': True,
                                                                                          'type': 'DRB-ToAddModList-NB-r13'},
                                                                                         {'name': 'drb-ToReleaseList-r13',
                                                                                          'optional': True,
                                                                                          'type': 'DRB-ToReleaseList-NB-r13'},
                                                                                         {'members': [{'name': 'explicitValue-r13',
                                                                                                       'type': 'MAC-MainConfig-NB-r13'},
                                                                                                      {'name': 'defaultValue-r13',
                                                                                                       'type': 'NULL'}],
                                                                                          'name': 'mac-MainConfig-r13',
                                                                                          'optional': True,
                                                                                          'type': 'CHOICE'},
                                                                                         {'name': 'physicalConfigDedicated-r13',
                                                                                          'optional': True,
                                                                                          'type': 'PhysicalConfigDedicated-NB-r13'},
                                                                                         {'name': 'rlf-TimersAndConstants-r13',
                                                                                          'optional': True,
                                                                                          'type': 'RLF-TimersAndConstants-NB-r13'},
                                                                                         None],
                                                                             'type': 'SEQUENCE'},
                                     'RedirectedCarrierInfo-NB-r13': {'type': 'CarrierFreq-NB-r13'},
                                     'RedirectedCarrierInfo-NB-v1430': {'members': [{'name': 'redirectedCarrierOffsetDedicated-r14',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('dB1',
                                                                                                 0),
                                                                                                ('dB2',
                                                                                                 1),
                                                                                                ('dB3',
                                                                                                 2),
                                                                                                ('dB4',
                                                                                                 3),
                                                                                                ('dB5',
                                                                                                 4),
                                                                                                ('dB6',
                                                                                                 5),
                                                                                                ('dB8',
                                                                                                 6),
                                                                                                ('dB10',
                                                                                                 7),
                                                                                                ('dB12',
                                                                                                 8),
                                                                                                ('dB14',
                                                                                                 9),
                                                                                                ('dB16',
                                                                                                 10),
                                                                                                ('dB18',
                                                                                                 11),
                                                                                                ('dB20',
                                                                                                 12),
                                                                                                ('dB22',
                                                                                                 13),
                                                                                                ('dB24',
                                                                                                 14),
                                                                                                ('dB26',
                                                                                                 15)]},
                                                                                    {'name': 't322-r14',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('min5',
                                                                                                 0),
                                                                                                ('min10',
                                                                                                 1),
                                                                                                ('min20',
                                                                                                 2),
                                                                                                ('min30',
                                                                                                 3),
                                                                                                ('min60',
                                                                                                 4),
                                                                                                ('min120',
                                                                                                 5),
                                                                                                ('min180',
                                                                                                 6),
                                                                                                ('spare1',
                                                                                                 7)]}],
                                                                        'type': 'SEQUENCE'},
                                     'ReestabUE-Identity-CP-NB-r14': {'members': [{'name': 's-TMSI-r14',
                                                                                   'type': 'S-TMSI'},
                                                                                  {'name': 'ul-NAS-MAC-r14',
                                                                                   'size': [16],
                                                                                   'type': 'BIT '
                                                                                           'STRING'},
                                                                                  {'name': 'ul-NAS-Count-r14',
                                                                                   'size': [5],
                                                                                   'type': 'BIT '
                                                                                           'STRING'}],
                                                                      'type': 'SEQUENCE'},
                                     'ReestablishmentCause-NB-r13': {'type': 'ENUMERATED',
                                                                     'values': [('reconfigurationFailure',
                                                                                 0),
                                                                                ('otherFailure',
                                                                                 1),
                                                                                ('spare2',
                                                                                 2),
                                                                                ('spare1',
                                                                                 3)]},
                                     'ReleaseCause-NB-r13': {'type': 'ENUMERATED',
                                                             'values': [('loadBalancingTAUrequired',
                                                                         0),
                                                                        ('other',
                                                                         1),
                                                                        ('rrc-Suspend',
                                                                         2),
                                                                        ('spare1',
                                                                         3)]},
                                     'ReselectionThreshold-NB-v1360': {'restricted-to': [(32,
                                                                                          63)],
                                                                       'type': 'INTEGER'},
                                     'RetxBSR-Timer-NB-r13': {'type': 'ENUMERATED',
                                                              'values': [('pp4',
                                                                          0),
                                                                         ('pp16',
                                                                          1),
                                                                         ('pp64',
                                                                          2),
                                                                         ('pp128',
                                                                          3),
                                                                         ('pp256',
                                                                          4),
                                                                         ('pp512',
                                                                          5),
                                                                         ('infinity',
                                                                          6),
                                                                         ('spare',
                                                                          7)]},
                                     'SC-MCCH-Message-NB': {'members': [{'name': 'message',
                                                                         'type': 'SC-MCCH-MessageType-NB'}],
                                                            'type': 'SEQUENCE'},
                                     'SC-MCCH-MessageType-NB': {'members': [{'members': [{'name': 'scptmConfiguration-r14',
                                                                                          'type': 'SCPTMConfiguration-NB-r14'}],
                                                                             'name': 'c1',
                                                                             'type': 'CHOICE'},
                                                                            {'members': [],
                                                                             'name': 'messageClassExtension',
                                                                             'type': 'SEQUENCE'}],
                                                                'type': 'CHOICE'},
                                     'SC-MCCH-SchedulingInfo-NB-r14': {'members': [{'name': 'onDurationTimerSCPTM-r14',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('pp1',
                                                                                                0),
                                                                                               ('pp2',
                                                                                                1),
                                                                                               ('pp3',
                                                                                                2),
                                                                                               ('pp4',
                                                                                                3),
                                                                                               ('pp8',
                                                                                                4),
                                                                                               ('pp16',
                                                                                                5),
                                                                                               ('pp32',
                                                                                                6),
                                                                                               ('spare',
                                                                                                7)]},
                                                                                   {'name': 'drx-InactivityTimerSCPTM-r14',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('pp0',
                                                                                                0),
                                                                                               ('pp1',
                                                                                                1),
                                                                                               ('pp2',
                                                                                                2),
                                                                                               ('pp3',
                                                                                                3),
                                                                                               ('pp4',
                                                                                                4),
                                                                                               ('pp8',
                                                                                                5),
                                                                                               ('pp16',
                                                                                                6),
                                                                                               ('pp32',
                                                                                                7)]},
                                                                                   {'members': [{'name': 'sf10',
                                                                                                 'restricted-to': [(0,
                                                                                                                    9)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf20',
                                                                                                 'restricted-to': [(0,
                                                                                                                    19)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf32',
                                                                                                 'restricted-to': [(0,
                                                                                                                    31)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf40',
                                                                                                 'restricted-to': [(0,
                                                                                                                    39)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf64',
                                                                                                 'restricted-to': [(0,
                                                                                                                    63)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf80',
                                                                                                 'restricted-to': [(0,
                                                                                                                    79)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf128',
                                                                                                 'restricted-to': [(0,
                                                                                                                    127)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf160',
                                                                                                 'restricted-to': [(0,
                                                                                                                    159)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf256',
                                                                                                 'restricted-to': [(0,
                                                                                                                    255)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf320',
                                                                                                 'restricted-to': [(0,
                                                                                                                    319)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf512',
                                                                                                 'restricted-to': [(0,
                                                                                                                    511)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf640',
                                                                                                 'restricted-to': [(0,
                                                                                                                    639)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf1024',
                                                                                                 'restricted-to': [(0,
                                                                                                                    1023)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf2048',
                                                                                                 'restricted-to': [(0,
                                                                                                                    2047)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf4096',
                                                                                                 'restricted-to': [(0,
                                                                                                                    4095)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf8192',
                                                                                                 'restricted-to': [(0,
                                                                                                                    8191)],
                                                                                                 'type': 'INTEGER'}],
                                                                                    'name': 'schedulingPeriodStartOffsetSCPTM-r14',
                                                                                    'type': 'CHOICE'},
                                                                                   None],
                                                                       'type': 'SEQUENCE'},
                                     'SC-MTCH-Info-NB-r14': {'members': [{'members': [{'name': 'dl-CarrierConfig-r14',
                                                                                       'type': 'DL-CarrierConfigCommon-NB-r14'},
                                                                                      {'name': 'dl-CarrierIndex-r14',
                                                                                       'restricted-to': [(0,
                                                                                                          'maxNonAnchorCarriers-NB-r14')],
                                                                                       'type': 'INTEGER'}],
                                                                          'name': 'sc-mtch-CarrierConfig-r14',
                                                                          'type': 'CHOICE'},
                                                                         {'name': 'mbmsSessionInfo-r14',
                                                                          'type': 'MBMSSessionInfo-r13'},
                                                                         {'name': 'g-RNTI-r14',
                                                                          'size': [16],
                                                                          'type': 'BIT '
                                                                                  'STRING'},
                                                                         {'name': 'sc-mtch-SchedulingInfo-r14',
                                                                          'optional': True,
                                                                          'type': 'SC-MTCH-SchedulingInfo-NB-r14'},
                                                                         {'name': 'sc-mtch-NeighbourCell-r14',
                                                                          'optional': True,
                                                                          'size': ['maxNeighCell-SCPTM-NB-r14'],
                                                                          'type': 'BIT '
                                                                                  'STRING'},
                                                                         {'name': 'npdcch-NPDSCH-MaxTBS-SC-MTCH-r14',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('n680',
                                                                                      0),
                                                                                     ('n2536',
                                                                                      1)]},
                                                                         {'name': 'npdcch-NumRepetitions-SC-MTCH-r14',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('r1',
                                                                                      0),
                                                                                     ('r2',
                                                                                      1),
                                                                                     ('r4',
                                                                                      2),
                                                                                     ('r8',
                                                                                      3),
                                                                                     ('r16',
                                                                                      4),
                                                                                     ('r32',
                                                                                      5),
                                                                                     ('r64',
                                                                                      6),
                                                                                     ('r128',
                                                                                      7),
                                                                                     ('r256',
                                                                                      8),
                                                                                     ('r512',
                                                                                      9),
                                                                                     ('r1024',
                                                                                      10),
                                                                                     ('r2048',
                                                                                      11),
                                                                                     ('spare4',
                                                                                      12),
                                                                                     ('spare3',
                                                                                      13),
                                                                                     ('spare2',
                                                                                      14),
                                                                                     ('spare1',
                                                                                      15)]},
                                                                         {'name': 'npdcch-StartSF-SC-MTCH-r14',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('v1dot5',
                                                                                      0),
                                                                                     ('v2',
                                                                                      1),
                                                                                     ('v4',
                                                                                      2),
                                                                                     ('v8',
                                                                                      3),
                                                                                     ('v16',
                                                                                      4),
                                                                                     ('v32',
                                                                                      5),
                                                                                     ('v48',
                                                                                      6),
                                                                                     ('v64',
                                                                                      7)]},
                                                                         {'name': 'npdcch-Offset-SC-MTCH-r14',
                                                                          'type': 'ENUMERATED',
                                                                          'values': [('zero',
                                                                                      0),
                                                                                     ('oneEighth',
                                                                                      1),
                                                                                     ('oneQuarter',
                                                                                      2),
                                                                                     ('threeEighth',
                                                                                      3),
                                                                                     ('oneHalf',
                                                                                      4),
                                                                                     ('fiveEighth',
                                                                                      5),
                                                                                     ('threeQuarter',
                                                                                      6),
                                                                                     ('sevenEighth',
                                                                                      7)]},
                                                                         None],
                                                             'type': 'SEQUENCE'},
                                     'SC-MTCH-InfoList-NB-r14': {'element': {'type': 'SC-MTCH-Info-NB-r14'},
                                                                 'size': [(0,
                                                                           'maxSC-MTCH-NB-r14')],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'SC-MTCH-SchedulingInfo-NB-r14': {'members': [{'name': 'onDurationTimerSCPTM-r14',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('pp1',
                                                                                                0),
                                                                                               ('pp2',
                                                                                                1),
                                                                                               ('pp3',
                                                                                                2),
                                                                                               ('pp4',
                                                                                                3),
                                                                                               ('pp8',
                                                                                                4),
                                                                                               ('pp16',
                                                                                                5),
                                                                                               ('pp32',
                                                                                                6),
                                                                                               ('spare',
                                                                                                7)]},
                                                                                   {'name': 'drx-InactivityTimerSCPTM-r14',
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('pp0',
                                                                                                0),
                                                                                               ('pp1',
                                                                                                1),
                                                                                               ('pp2',
                                                                                                2),
                                                                                               ('pp3',
                                                                                                3),
                                                                                               ('pp4',
                                                                                                4),
                                                                                               ('pp8',
                                                                                                5),
                                                                                               ('pp16',
                                                                                                6),
                                                                                               ('pp32',
                                                                                                7)]},
                                                                                   {'members': [{'name': 'sf10',
                                                                                                 'restricted-to': [(0,
                                                                                                                    9)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf20',
                                                                                                 'restricted-to': [(0,
                                                                                                                    19)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf32',
                                                                                                 'restricted-to': [(0,
                                                                                                                    31)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf40',
                                                                                                 'restricted-to': [(0,
                                                                                                                    39)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf64',
                                                                                                 'restricted-to': [(0,
                                                                                                                    63)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf80',
                                                                                                 'restricted-to': [(0,
                                                                                                                    79)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf128',
                                                                                                 'restricted-to': [(0,
                                                                                                                    127)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf160',
                                                                                                 'restricted-to': [(0,
                                                                                                                    159)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf256',
                                                                                                 'restricted-to': [(0,
                                                                                                                    255)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf320',
                                                                                                 'restricted-to': [(0,
                                                                                                                    319)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf512',
                                                                                                 'restricted-to': [(0,
                                                                                                                    511)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf640',
                                                                                                 'restricted-to': [(0,
                                                                                                                    639)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf1024',
                                                                                                 'restricted-to': [(0,
                                                                                                                    1023)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf2048',
                                                                                                 'restricted-to': [(0,
                                                                                                                    2047)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf4096',
                                                                                                 'restricted-to': [(0,
                                                                                                                    4095)],
                                                                                                 'type': 'INTEGER'},
                                                                                                {'name': 'sf8192',
                                                                                                 'restricted-to': [(0,
                                                                                                                    8191)],
                                                                                                 'type': 'INTEGER'}],
                                                                                    'name': 'schedulingPeriodStartOffsetSCPTM-r14',
                                                                                    'type': 'CHOICE'},
                                                                                   None],
                                                                       'type': 'SEQUENCE'},
                                     'SCPTM-NeighbourCellList-NB-r14': {'element': {'type': 'PCI-ARFCN-NB-r14'},
                                                                        'size': [(1,
                                                                                  'maxNeighCell-SCPTM-NB-r14')],
                                                                        'type': 'SEQUENCE '
                                                                                'OF'},
                                     'SCPTMConfiguration-NB-r14': {'members': [{'name': 'sc-mtch-InfoList-r14',
                                                                                'type': 'SC-MTCH-InfoList-NB-r14'},
                                                                               {'name': 'scptm-NeighbourCellList-r14',
                                                                                'optional': True,
                                                                                'type': 'SCPTM-NeighbourCellList-NB-r14'},
                                                                               {'name': 'lateNonCriticalExtension',
                                                                                'optional': True,
                                                                                'type': 'OCTET '
                                                                                        'STRING'},
                                                                               {'members': [],
                                                                                'name': 'nonCriticalExtension',
                                                                                'optional': True,
                                                                                'type': 'SEQUENCE'}],
                                                                   'type': 'SEQUENCE'},
                                     'SIB-MappingInfo-NB-r13': {'element': {'type': 'SIB-Type-NB-r13'},
                                                                'size': [(0,
                                                                          'maxSIB-1')],
                                                                'type': 'SEQUENCE '
                                                                        'OF'},
                                     'SIB-Type-NB-r13': {'type': 'ENUMERATED',
                                                         'values': [('sibType3-NB-r13',
                                                                     0),
                                                                    ('sibType4-NB-r13',
                                                                     1),
                                                                    ('sibType5-NB-r13',
                                                                     2),
                                                                    ('sibType14-NB-r13',
                                                                     3),
                                                                    ('sibType16-NB-r13',
                                                                     4),
                                                                    ('sibType15-NB-r14',
                                                                     5),
                                                                    ('sibType20-NB-r14',
                                                                     6),
                                                                    ('sibType22-NB-r14',
                                                                     7)]},
                                     'SRB-ToAddMod-NB-r13': {'members': [{'members': [{'name': 'explicitValue',
                                                                                       'type': 'RLC-Config-NB-r13'},
                                                                                      {'name': 'defaultValue',
                                                                                       'type': 'NULL'}],
                                                                          'name': 'rlc-Config-r13',
                                                                          'optional': True,
                                                                          'type': 'CHOICE'},
                                                                         {'members': [{'name': 'explicitValue',
                                                                                       'type': 'LogicalChannelConfig-NB-r13'},
                                                                                      {'name': 'defaultValue',
                                                                                       'type': 'NULL'}],
                                                                          'name': 'logicalChannelConfig-r13',
                                                                          'optional': True,
                                                                          'type': 'CHOICE'},
                                                                         None,
                                                                         [{'name': 'rlc-Config-v1430',
                                                                           'optional': True,
                                                                           'type': 'RLC-Config-NB-v1430'}]],
                                                             'type': 'SEQUENCE'},
                                     'SRB-ToAddModList-NB-r13': {'element': {'type': 'SRB-ToAddMod-NB-r13'},
                                                                 'size': [1],
                                                                 'type': 'SEQUENCE '
                                                                         'OF'},
                                     'SchedulingInfo-NB-r13': {'members': [{'name': 'si-Periodicity-r13',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('rf64',
                                                                                        0),
                                                                                       ('rf128',
                                                                                        1),
                                                                                       ('rf256',
                                                                                        2),
                                                                                       ('rf512',
                                                                                        3),
                                                                                       ('rf1024',
                                                                                        4),
                                                                                       ('rf2048',
                                                                                        5),
                                                                                       ('rf4096',
                                                                                        6),
                                                                                       ('spare',
                                                                                        7)]},
                                                                           {'name': 'si-RepetitionPattern-r13',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('every2ndRF',
                                                                                        0),
                                                                                       ('every4thRF',
                                                                                        1),
                                                                                       ('every8thRF',
                                                                                        2),
                                                                                       ('every16thRF',
                                                                                        3)]},
                                                                           {'name': 'sib-MappingInfo-r13',
                                                                            'type': 'SIB-MappingInfo-NB-r13'},
                                                                           {'name': 'si-TB-r13',
                                                                            'type': 'ENUMERATED',
                                                                            'values': [('b56',
                                                                                        0),
                                                                                       ('b120',
                                                                                        1),
                                                                                       ('b208',
                                                                                        2),
                                                                                       ('b256',
                                                                                        3),
                                                                                       ('b328',
                                                                                        4),
                                                                                       ('b440',
                                                                                        5),
                                                                                       ('b552',
                                                                                        6),
                                                                                       ('b680',
                                                                                        7)]}],
                                                               'type': 'SEQUENCE'},
                                     'SchedulingInfoList-NB-r13': {'element': {'type': 'SchedulingInfo-NB-r13'},
                                                                   'size': [(1,
                                                                             'maxSI-Message-NB-r13')],
                                                                   'type': 'SEQUENCE '
                                                                           'OF'},
                                     'Standalone-NB-r13': {'members': [{'name': 'spare',
                                                                        'size': [5],
                                                                        'type': 'BIT '
                                                                                'STRING'}],
                                                           'type': 'SEQUENCE'},
                                     'SupportedBand-NB-r13': {'members': [{'name': 'band-r13',
                                                                           'type': 'FreqBandIndicator-NB-r13'},
                                                                          {'name': 'powerClassNB-20dBm-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]}],
                                                              'type': 'SEQUENCE'},
                                     'SupportedBandList-NB-r13': {'element': {'type': 'SupportedBand-NB-r13'},
                                                                  'size': [(1,
                                                                            'maxBands')],
                                                                  'type': 'SEQUENCE '
                                                                          'OF'},
                                     'SystemInfoValueTagList-NB-r13': {'element': {'type': 'SystemInfoValueTagSI-r13'},
                                                                       'size': [(1,
                                                                                 'maxSI-Message-NB-r13')],
                                                                       'type': 'SEQUENCE '
                                                                               'OF'},
                                     'SystemInformation-NB': {'members': [{'members': [{'name': 'systemInformation-r13',
                                                                                        'type': 'SystemInformation-NB-r13-IEs'},
                                                                                       {'members': [],
                                                                                        'name': 'criticalExtensionsFuture',
                                                                                        'type': 'SEQUENCE'}],
                                                                           'name': 'criticalExtensions',
                                                                           'type': 'CHOICE'}],
                                                              'type': 'SEQUENCE'},
                                     'SystemInformation-NB-r13-IEs': {'members': [{'element': {'members': [{'name': 'sib2-r13',
                                                                                                            'type': 'SystemInformationBlockType2-NB-r13'},
                                                                                                           {'name': 'sib3-r13',
                                                                                                            'type': 'SystemInformationBlockType3-NB-r13'},
                                                                                                           {'name': 'sib4-r13',
                                                                                                            'type': 'SystemInformationBlockType4-NB-r13'},
                                                                                                           {'name': 'sib5-r13',
                                                                                                            'type': 'SystemInformationBlockType5-NB-r13'},
                                                                                                           {'name': 'sib14-r13',
                                                                                                            'type': 'SystemInformationBlockType14-NB-r13'},
                                                                                                           {'name': 'sib16-r13',
                                                                                                            'type': 'SystemInformationBlockType16-NB-r13'},
                                                                                                           None,
                                                                                                           {'name': 'sib15-v1430',
                                                                                                            'type': 'SystemInformationBlockType15-NB-r14'},
                                                                                                           {'name': 'sib20-v1430',
                                                                                                            'type': 'SystemInformationBlockType20-NB-r14'},
                                                                                                           {'name': 'sib22-v1430',
                                                                                                            'type': 'SystemInformationBlockType22-NB-r14'}],
                                                                                               'type': 'CHOICE'},
                                                                                   'name': 'sib-TypeAndInfo-r13',
                                                                                   'size': [(1,
                                                                                             'maxSIB')],
                                                                                   'type': 'SEQUENCE '
                                                                                           'OF'},
                                                                                  {'name': 'lateNonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'OCTET '
                                                                                           'STRING'},
                                                                                  {'members': [],
                                                                                   'name': 'nonCriticalExtension',
                                                                                   'optional': True,
                                                                                   'type': 'SEQUENCE'}],
                                                                      'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-NB': {'members': [{'name': 'hyperSFN-MSB-r13',
                                                                                     'size': [8],
                                                                                     'type': 'BIT '
                                                                                             'STRING'},
                                                                                    {'members': [{'name': 'plmn-IdentityList-r13',
                                                                                                  'type': 'PLMN-IdentityList-NB-r13'},
                                                                                                 {'name': 'trackingAreaCode-r13',
                                                                                                  'type': 'TrackingAreaCode'},
                                                                                                 {'name': 'cellIdentity-r13',
                                                                                                  'type': 'CellIdentity'},
                                                                                                 {'name': 'cellBarred-r13',
                                                                                                  'type': 'ENUMERATED',
                                                                                                  'values': [('barred',
                                                                                                              0),
                                                                                                             ('notBarred',
                                                                                                              1)]},
                                                                                                 {'name': 'intraFreqReselection-r13',
                                                                                                  'type': 'ENUMERATED',
                                                                                                  'values': [('allowed',
                                                                                                              0),
                                                                                                             ('notAllowed',
                                                                                                              1)]}],
                                                                                     'name': 'cellAccessRelatedInfo-r13',
                                                                                     'type': 'SEQUENCE'},
                                                                                    {'members': [{'name': 'q-RxLevMin-r13',
                                                                                                  'type': 'Q-RxLevMin'},
                                                                                                 {'name': 'q-QualMin-r13',
                                                                                                  'type': 'Q-QualMin-r9'}],
                                                                                     'name': 'cellSelectionInfo-r13',
                                                                                     'type': 'SEQUENCE'},
                                                                                    {'name': 'p-Max-r13',
                                                                                     'optional': True,
                                                                                     'type': 'P-Max'},
                                                                                    {'name': 'freqBandIndicator-r13',
                                                                                     'type': 'FreqBandIndicator-NB-r13'},
                                                                                    {'name': 'freqBandInfo-r13',
                                                                                     'optional': True,
                                                                                     'type': 'NS-PmaxList-NB-r13'},
                                                                                    {'name': 'multiBandInfoList-r13',
                                                                                     'optional': True,
                                                                                     'type': 'MultiBandInfoList-NB-r13'},
                                                                                    {'name': 'downlinkBitmap-r13',
                                                                                     'optional': True,
                                                                                     'type': 'DL-Bitmap-NB-r13'},
                                                                                    {'name': 'eutraControlRegionSize-r13',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('n1',
                                                                                                 0),
                                                                                                ('n2',
                                                                                                 1),
                                                                                                ('n3',
                                                                                                 2)]},
                                                                                    {'name': 'nrs-CRS-PowerOffset-r13',
                                                                                     'optional': True,
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('dB-6',
                                                                                                 0),
                                                                                                ('dB-4dot77',
                                                                                                 1),
                                                                                                ('dB-3',
                                                                                                 2),
                                                                                                ('dB-1dot77',
                                                                                                 3),
                                                                                                ('dB0',
                                                                                                 4),
                                                                                                ('dB1',
                                                                                                 5),
                                                                                                ('dB1dot23',
                                                                                                 6),
                                                                                                ('dB2',
                                                                                                 7),
                                                                                                ('dB3',
                                                                                                 8),
                                                                                                ('dB4',
                                                                                                 9),
                                                                                                ('dB4dot23',
                                                                                                 10),
                                                                                                ('dB5',
                                                                                                 11),
                                                                                                ('dB6',
                                                                                                 12),
                                                                                                ('dB7',
                                                                                                 13),
                                                                                                ('dB8',
                                                                                                 14),
                                                                                                ('dB9',
                                                                                                 15)]},
                                                                                    {'name': 'schedulingInfoList-r13',
                                                                                     'type': 'SchedulingInfoList-NB-r13'},
                                                                                    {'name': 'si-WindowLength-r13',
                                                                                     'type': 'ENUMERATED',
                                                                                     'values': [('ms160',
                                                                                                 0),
                                                                                                ('ms320',
                                                                                                 1),
                                                                                                ('ms480',
                                                                                                 2),
                                                                                                ('ms640',
                                                                                                 3),
                                                                                                ('ms960',
                                                                                                 4),
                                                                                                ('ms1280',
                                                                                                 5),
                                                                                                ('ms1600',
                                                                                                 6),
                                                                                                ('spare1',
                                                                                                 7)]},
                                                                                    {'name': 'si-RadioFrameOffset-r13',
                                                                                     'optional': True,
                                                                                     'restricted-to': [(1,
                                                                                                        15)],
                                                                                     'type': 'INTEGER'},
                                                                                    {'name': 'systemInfoValueTagList-r13',
                                                                                     'optional': True,
                                                                                     'type': 'SystemInfoValueTagList-NB-r13'},
                                                                                    {'name': 'lateNonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'OCTET '
                                                                                             'STRING'},
                                                                                    {'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'SystemInformationBlockType1-NB-v1350'}],
                                                                        'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-NB-v1350': {'members': [{'name': 'cellSelectionInfo-v1350',
                                                                                           'optional': True,
                                                                                           'type': 'CellSelectionInfo-NB-v1350'},
                                                                                          {'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SystemInformationBlockType1-NB-v1430'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType1-NB-v1430': {'members': [{'name': 'cellSelectionInfo-v1430',
                                                                                           'optional': True,
                                                                                           'type': 'CellSelectionInfo-NB-v1430'},
                                                                                          {'members': [],
                                                                                           'name': 'nonCriticalExtension',
                                                                                           'optional': True,
                                                                                           'type': 'SEQUENCE'}],
                                                                              'type': 'SEQUENCE'},
                                     'SystemInformationBlockType14-NB-r13': {'members': [{'members': [{'name': 'ab-Common-r13',
                                                                                                       'type': 'AB-Config-NB-r13'},
                                                                                                      {'element': {'type': 'AB-ConfigPLMN-NB-r13'},
                                                                                                       'name': 'ab-PerPLMN-List-r13',
                                                                                                       'size': [(1,
                                                                                                                 'maxPLMN-r11')],
                                                                                                       'type': 'SEQUENCE '
                                                                                                               'OF'}],
                                                                                          'name': 'ab-Param-r13',
                                                                                          'optional': True,
                                                                                          'type': 'CHOICE'},
                                                                                         {'name': 'lateNonCriticalExtension',
                                                                                          'optional': True,
                                                                                          'type': 'OCTET '
                                                                                                  'STRING'},
                                                                                         None],
                                                                             'type': 'SEQUENCE'},
                                     'SystemInformationBlockType15-NB-r14': {'members': [{'name': 'mbms-SAI-IntraFreq-r14',
                                                                                          'optional': True,
                                                                                          'type': 'MBMS-SAI-List-r11'},
                                                                                         {'name': 'mbms-SAI-InterFreqList-r14',
                                                                                          'optional': True,
                                                                                          'type': 'MBMS-SAI-InterFreqList-NB-r14'},
                                                                                         {'name': 'lateNonCriticalExtension',
                                                                                          'optional': True,
                                                                                          'type': 'OCTET '
                                                                                                  'STRING'},
                                                                                         None],
                                                                             'type': 'SEQUENCE'},
                                     'SystemInformationBlockType16-NB-r13': {'type': 'SystemInformationBlockType16-r11'},
                                     'SystemInformationBlockType2-NB-r13': {'members': [{'name': 'radioResourceConfigCommon-r13',
                                                                                         'type': 'RadioResourceConfigCommonSIB-NB-r13'},
                                                                                        {'name': 'ue-TimersAndConstants-r13',
                                                                                         'type': 'UE-TimersAndConstants-NB-r13'},
                                                                                        {'members': [{'name': 'ul-CarrierFreq-r13',
                                                                                                      'optional': True,
                                                                                                      'type': 'CarrierFreq-NB-r13'},
                                                                                                     {'name': 'additionalSpectrumEmission-r13',
                                                                                                      'type': 'AdditionalSpectrumEmission'}],
                                                                                         'name': 'freqInfo-r13',
                                                                                         'type': 'SEQUENCE'},
                                                                                        {'name': 'timeAlignmentTimerCommon-r13',
                                                                                         'type': 'TimeAlignmentTimer'},
                                                                                        {'element': {'type': 'AdditionalSpectrumEmission'},
                                                                                         'name': 'multiBandInfoList-r13',
                                                                                         'optional': True,
                                                                                         'size': [(1,
                                                                                                   'maxMultiBands')],
                                                                                         'type': 'SEQUENCE '
                                                                                                 'OF'},
                                                                                        {'name': 'lateNonCriticalExtension',
                                                                                         'optional': True,
                                                                                         'type': 'OCTET '
                                                                                                 'STRING'},
                                                                                        None,
                                                                                        [{'name': 'cp-Reestablishment-r14',
                                                                                          'optional': True,
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('true',
                                                                                                      0)]}]],
                                                                            'type': 'SEQUENCE'},
                                     'SystemInformationBlockType20-NB-r14': {'members': [{'name': 'npdcch-SC-MCCH-Config-r14',
                                                                                          'type': 'NPDCCH-SC-MCCH-Config-NB-r14'},
                                                                                         {'members': [{'name': 'dl-CarrierConfig-r14',
                                                                                                       'type': 'DL-CarrierConfigCommon-NB-r14'},
                                                                                                      {'name': 'dl-CarrierIndex-r14',
                                                                                                       'restricted-to': [(0,
                                                                                                                          'maxNonAnchorCarriers-NB-r14')],
                                                                                                       'type': 'INTEGER'}],
                                                                                          'name': 'sc-mcch-CarrierConfig-r14',
                                                                                          'type': 'CHOICE'},
                                                                                         {'name': 'sc-mcch-RepetitionPeriod-r14',
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('rf32',
                                                                                                      0),
                                                                                                     ('rf128',
                                                                                                      1),
                                                                                                     ('rf512',
                                                                                                      2),
                                                                                                     ('rf1024',
                                                                                                      3),
                                                                                                     ('rf2048',
                                                                                                      4),
                                                                                                     ('rf4096',
                                                                                                      5),
                                                                                                     ('rf8192',
                                                                                                      6),
                                                                                                     ('rf16384',
                                                                                                      7)]},
                                                                                         {'name': 'sc-mcch-Offset-r14',
                                                                                          'restricted-to': [(0,
                                                                                                             10)],
                                                                                          'type': 'INTEGER'},
                                                                                         {'name': 'sc-mcch-ModificationPeriod-r14',
                                                                                          'type': 'ENUMERATED',
                                                                                          'values': [('rf32',
                                                                                                      0),
                                                                                                     ('rf128',
                                                                                                      1),
                                                                                                     ('rf256',
                                                                                                      2),
                                                                                                     ('rf512',
                                                                                                      3),
                                                                                                     ('rf1024',
                                                                                                      4),
                                                                                                     ('rf2048',
                                                                                                      5),
                                                                                                     ('rf4096',
                                                                                                      6),
                                                                                                     ('rf8192',
                                                                                                      7),
                                                                                                     ('rf16384',
                                                                                                      8),
                                                                                                     ('rf32768',
                                                                                                      9),
                                                                                                     ('rf65536',
                                                                                                      10),
                                                                                                     ('rf131072',
                                                                                                      11),
                                                                                                     ('rf262144',
                                                                                                      12),
                                                                                                     ('rf524288',
                                                                                                      13),
                                                                                                     ('rf1048576',
                                                                                                      14),
                                                                                                     ('spare1',
                                                                                                      15)]},
                                                                                         {'name': 'sc-mcch-SchedulingInfo-r14',
                                                                                          'optional': True,
                                                                                          'type': 'SC-MCCH-SchedulingInfo-NB-r14'},
                                                                                         {'name': 'lateNonCriticalExtension',
                                                                                          'optional': True,
                                                                                          'type': 'OCTET '
                                                                                                  'STRING'},
                                                                                         None],
                                                                             'type': 'SEQUENCE'},
                                     'SystemInformationBlockType22-NB-r14': {'members': [{'name': 'dl-ConfigList-r14',
                                                                                          'optional': True,
                                                                                          'type': 'DL-ConfigCommonList-NB-r14'},
                                                                                         {'name': 'ul-ConfigList-r14',
                                                                                          'optional': True,
                                                                                          'type': 'UL-ConfigCommonList-NB-r14'},
                                                                                         {'name': 'pagingWeightAnchor-r14',
                                                                                          'optional': True,
                                                                                          'type': 'PagingWeight-NB-r14'},
                                                                                         {'name': 'nprach-ProbabilityAnchorList-r14',
                                                                                          'optional': True,
                                                                                          'type': 'NPRACH-ProbabilityAnchorList-NB-r14'},
                                                                                         {'name': 'lateNonCriticalExtension',
                                                                                          'optional': True,
                                                                                          'type': 'OCTET '
                                                                                                  'STRING'},
                                                                                         None],
                                                                             'type': 'SEQUENCE'},
                                     'SystemInformationBlockType3-NB-r13': {'members': [{'members': [{'name': 'q-Hyst-r13',
                                                                                                      'type': 'ENUMERATED',
                                                                                                      'values': [('dB0',
                                                                                                                  0),
                                                                                                                 ('dB1',
                                                                                                                  1),
                                                                                                                 ('dB2',
                                                                                                                  2),
                                                                                                                 ('dB3',
                                                                                                                  3),
                                                                                                                 ('dB4',
                                                                                                                  4),
                                                                                                                 ('dB5',
                                                                                                                  5),
                                                                                                                 ('dB6',
                                                                                                                  6),
                                                                                                                 ('dB8',
                                                                                                                  7),
                                                                                                                 ('dB10',
                                                                                                                  8),
                                                                                                                 ('dB12',
                                                                                                                  9),
                                                                                                                 ('dB14',
                                                                                                                  10),
                                                                                                                 ('dB16',
                                                                                                                  11),
                                                                                                                 ('dB18',
                                                                                                                  12),
                                                                                                                 ('dB20',
                                                                                                                  13),
                                                                                                                 ('dB22',
                                                                                                                  14),
                                                                                                                 ('dB24',
                                                                                                                  15)]}],
                                                                                         'name': 'cellReselectionInfoCommon-r13',
                                                                                         'type': 'SEQUENCE'},
                                                                                        {'members': [{'name': 's-NonIntraSearch-r13',
                                                                                                      'type': 'ReselectionThreshold'}],
                                                                                         'name': 'cellReselectionServingFreqInfo-r13',
                                                                                         'type': 'SEQUENCE'},
                                                                                        {'members': [{'name': 'q-RxLevMin-r13',
                                                                                                      'type': 'Q-RxLevMin'},
                                                                                                     {'name': 'q-QualMin-r13',
                                                                                                      'optional': True,
                                                                                                      'type': 'Q-QualMin-r9'},
                                                                                                     {'name': 'p-Max-r13',
                                                                                                      'optional': True,
                                                                                                      'type': 'P-Max'},
                                                                                                     {'name': 's-IntraSearchP-r13',
                                                                                                      'type': 'ReselectionThreshold'},
                                                                                                     {'name': 't-Reselection-r13',
                                                                                                      'type': 'T-Reselection-NB-r13'}],
                                                                                         'name': 'intraFreqCellReselectionInfo-r13',
                                                                                         'type': 'SEQUENCE'},
                                                                                        {'name': 'freqBandInfo-r13',
                                                                                         'optional': True,
                                                                                         'type': 'NS-PmaxList-NB-r13'},
                                                                                        {'element': {'type': 'NS-PmaxList-NB-r13'},
                                                                                         'name': 'multiBandInfoList-r13',
                                                                                         'optional': True,
                                                                                         'size': [(1,
                                                                                                   'maxMultiBands')],
                                                                                         'type': 'SEQUENCE '
                                                                                                 'OF'},
                                                                                        {'name': 'lateNonCriticalExtension',
                                                                                         'optional': True,
                                                                                         'type': 'OCTET '
                                                                                                 'STRING'},
                                                                                        None,
                                                                                        [{'name': 'intraFreqCellReselectionInfo-v1350',
                                                                                          'optional': True,
                                                                                          'type': 'IntraFreqCellReselectionInfo-NB-v1350'}],
                                                                                        [{'name': 'intraFreqCellReselectionInfo-v1360',
                                                                                          'optional': True,
                                                                                          'type': 'IntraFreqCellReselectionInfo-NB-v1360'}],
                                                                                        [{'name': 'intraFreqCellReselectionInfo-v1430',
                                                                                          'optional': True,
                                                                                          'type': 'IntraFreqCellReselectionInfo-NB-v1430'}]],
                                                                            'type': 'SEQUENCE'},
                                     'SystemInformationBlockType4-NB-r13': {'members': [{'name': 'intraFreqNeighCellList-r13',
                                                                                         'optional': True,
                                                                                         'type': 'IntraFreqNeighCellList'},
                                                                                        {'name': 'intraFreqBlackCellList-r13',
                                                                                         'optional': True,
                                                                                         'type': 'IntraFreqBlackCellList'},
                                                                                        {'name': 'lateNonCriticalExtension',
                                                                                         'optional': True,
                                                                                         'type': 'OCTET '
                                                                                                 'STRING'},
                                                                                        None],
                                                                            'type': 'SEQUENCE'},
                                     'SystemInformationBlockType5-NB-r13': {'members': [{'name': 'interFreqCarrierFreqList-r13',
                                                                                         'type': 'InterFreqCarrierFreqList-NB-r13'},
                                                                                        {'name': 't-Reselection-r13',
                                                                                         'type': 'T-Reselection-NB-r13'},
                                                                                        {'name': 'lateNonCriticalExtension',
                                                                                         'optional': True,
                                                                                         'type': 'OCTET '
                                                                                                 'STRING'},
                                                                                        None,
                                                                                        [{'name': 'scptm-FreqOffset-r14',
                                                                                          'optional': True,
                                                                                          'restricted-to': [(1,
                                                                                                             8)],
                                                                                          'type': 'INTEGER'}]],
                                                                            'type': 'SEQUENCE'},
                                     'T-PollRetransmit-NB-r13': {'type': 'ENUMERATED',
                                                                 'values': [('ms250',
                                                                             0),
                                                                            ('ms500',
                                                                             1),
                                                                            ('ms1000',
                                                                             2),
                                                                            ('ms2000',
                                                                             3),
                                                                            ('ms3000',
                                                                             4),
                                                                            ('ms4000',
                                                                             5),
                                                                            ('ms6000',
                                                                             6),
                                                                            ('ms10000',
                                                                             7),
                                                                            ('ms15000',
                                                                             8),
                                                                            ('ms25000',
                                                                             9),
                                                                            ('ms40000',
                                                                             10),
                                                                            ('ms60000',
                                                                             11),
                                                                            ('ms90000',
                                                                             12),
                                                                            ('ms120000',
                                                                             13),
                                                                            ('ms180000',
                                                                             14),
                                                                            ('spare1',
                                                                             15)]},
                                     'T-Reselection-NB-r13': {'type': 'ENUMERATED',
                                                              'values': [('s0',
                                                                          0),
                                                                         ('s3',
                                                                          1),
                                                                         ('s6',
                                                                          2),
                                                                         ('s9',
                                                                          3),
                                                                         ('s12',
                                                                          4),
                                                                         ('s15',
                                                                          5),
                                                                         ('s18',
                                                                          6),
                                                                         ('s21',
                                                                          7)]},
                                     'UE-Capability-NB-r13': {'members': [{'name': 'accessStratumRelease-r13',
                                                                           'type': 'AccessStratumRelease-NB-r13'},
                                                                          {'name': 'ue-Category-NB-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('nb1',
                                                                                       0)]},
                                                                          {'name': 'multipleDRB-r13',
                                                                           'optional': True,
                                                                           'type': 'ENUMERATED',
                                                                           'values': [('supported',
                                                                                       0)]},
                                                                          {'name': 'pdcp-Parameters-r13',
                                                                           'optional': True,
                                                                           'type': 'PDCP-Parameters-NB-r13'},
                                                                          {'name': 'phyLayerParameters-r13',
                                                                           'type': 'PhyLayerParameters-NB-r13'},
                                                                          {'name': 'rf-Parameters-r13',
                                                                           'type': 'RF-Parameters-NB-r13'},
                                                                          {'name': 'nonCriticalExtension',
                                                                           'optional': True,
                                                                           'type': 'UE-Capability-NB-v1430-IEs'}],
                                                              'type': 'SEQUENCE'},
                                     'UE-Capability-NB-v1430-IEs': {'members': [{'name': 'ue-Category-NB-r14',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('nb2',
                                                                                             0)]},
                                                                                {'name': 'mac-Parameters-r14',
                                                                                 'optional': True,
                                                                                 'type': 'MAC-Parameters-NB-r14'},
                                                                                {'name': 'phyLayerParameters-v1430',
                                                                                 'optional': True,
                                                                                 'type': 'PhyLayerParameters-NB-v1430'},
                                                                                {'name': 'rf-Parameters-v1430',
                                                                                 'type': 'RF-Parameters-NB-v1430'},
                                                                                {'name': 'nonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'UE-Capability-NB-v1440-IEs'}],
                                                                    'type': 'SEQUENCE'},
                                     'UE-Capability-NB-v1440-IEs': {'members': [{'name': 'phyLayerParameters-v1440',
                                                                                 'optional': True,
                                                                                 'type': 'PhyLayerParameters-NB-v1440'},
                                                                                {'members': [],
                                                                                 'name': 'nonCriticalExtension',
                                                                                 'optional': True,
                                                                                 'type': 'SEQUENCE'}],
                                                                    'type': 'SEQUENCE'},
                                     'UE-RadioPagingInfo-NB-r13': {'members': [{'name': 'ue-Category-NB-r13',
                                                                                'optional': True,
                                                                                'type': 'ENUMERATED',
                                                                                'values': [('nb1',
                                                                                            0)]},
                                                                               None,
                                                                               [{'name': 'multiCarrierPaging-r14',
                                                                                 'optional': True,
                                                                                 'type': 'ENUMERATED',
                                                                                 'values': [('true',
                                                                                             0)]}]],
                                                                   'type': 'SEQUENCE'},
                                     'UE-TimersAndConstants-NB-r13': {'members': [{'name': 't300-r13',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('ms2500',
                                                                                               0),
                                                                                              ('ms4000',
                                                                                               1),
                                                                                              ('ms6000',
                                                                                               2),
                                                                                              ('ms10000',
                                                                                               3),
                                                                                              ('ms15000',
                                                                                               4),
                                                                                              ('ms25000',
                                                                                               5),
                                                                                              ('ms40000',
                                                                                               6),
                                                                                              ('ms60000',
                                                                                               7)]},
                                                                                  {'name': 't301-r13',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('ms2500',
                                                                                               0),
                                                                                              ('ms4000',
                                                                                               1),
                                                                                              ('ms6000',
                                                                                               2),
                                                                                              ('ms10000',
                                                                                               3),
                                                                                              ('ms15000',
                                                                                               4),
                                                                                              ('ms25000',
                                                                                               5),
                                                                                              ('ms40000',
                                                                                               6),
                                                                                              ('ms60000',
                                                                                               7)]},
                                                                                  {'name': 't310-r13',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('ms0',
                                                                                               0),
                                                                                              ('ms200',
                                                                                               1),
                                                                                              ('ms500',
                                                                                               2),
                                                                                              ('ms1000',
                                                                                               3),
                                                                                              ('ms2000',
                                                                                               4),
                                                                                              ('ms4000',
                                                                                               5),
                                                                                              ('ms8000',
                                                                                               6)]},
                                                                                  {'name': 'n310-r13',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('n1',
                                                                                               0),
                                                                                              ('n2',
                                                                                               1),
                                                                                              ('n3',
                                                                                               2),
                                                                                              ('n4',
                                                                                               3),
                                                                                              ('n6',
                                                                                               4),
                                                                                              ('n8',
                                                                                               5),
                                                                                              ('n10',
                                                                                               6),
                                                                                              ('n20',
                                                                                               7)]},
                                                                                  {'name': 't311-r13',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('ms1000',
                                                                                               0),
                                                                                              ('ms3000',
                                                                                               1),
                                                                                              ('ms5000',
                                                                                               2),
                                                                                              ('ms10000',
                                                                                               3),
                                                                                              ('ms15000',
                                                                                               4),
                                                                                              ('ms20000',
                                                                                               5),
                                                                                              ('ms30000',
                                                                                               6)]},
                                                                                  {'name': 'n311-r13',
                                                                                   'type': 'ENUMERATED',
                                                                                   'values': [('n1',
                                                                                               0),
                                                                                              ('n2',
                                                                                               1),
                                                                                              ('n3',
                                                                                               2),
                                                                                              ('n4',
                                                                                               3),
                                                                                              ('n5',
                                                                                               4),
                                                                                              ('n6',
                                                                                               5),
                                                                                              ('n8',
                                                                                               6),
                                                                                              ('n10',
                                                                                               7)]},
                                                                                  None,
                                                                                  [{'name': 't311-v1350',
                                                                                    'optional': True,
                                                                                    'type': 'ENUMERATED',
                                                                                    'values': [('ms40000',
                                                                                                0),
                                                                                               ('ms60000',
                                                                                                1),
                                                                                               ('ms90000',
                                                                                                2),
                                                                                               ('ms120000',
                                                                                                3)]}]],
                                                                      'type': 'SEQUENCE'},
                                     'UECapabilityEnquiry-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                             'type': 'RRC-TransactionIdentifier'},
                                                                            {'members': [{'members': [{'name': 'ueCapabilityEnquiry-r13',
                                                                                                       'type': 'UECapabilityEnquiry-NB-r13-IEs'},
                                                                                                      {'name': 'spare1',
                                                                                                       'type': 'NULL'}],
                                                                                          'name': 'c1',
                                                                                          'type': 'CHOICE'},
                                                                                         {'members': [],
                                                                                          'name': 'criticalExtensionsFuture',
                                                                                          'type': 'SEQUENCE'}],
                                                                             'name': 'criticalExtensions',
                                                                             'type': 'CHOICE'}],
                                                                'type': 'SEQUENCE'},
                                     'UECapabilityEnquiry-NB-r13-IEs': {'members': [{'name': 'lateNonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'OCTET '
                                                                                             'STRING'},
                                                                                    {'members': [],
                                                                                     'name': 'nonCriticalExtension',
                                                                                     'optional': True,
                                                                                     'type': 'SEQUENCE'}],
                                                                        'type': 'SEQUENCE'},
                                     'UECapabilityInformation-NB': {'members': [{'name': 'rrc-TransactionIdentifier',
                                                                                 'type': 'RRC-TransactionIdentifier'},
                                                                                {'members': [{'name': 'ueCapabilityInformation-r13',
                                                                                              'type': 'UECapabilityInformation-NB-r13-IEs'},
                                                                                             {'members': [],
                                                                                              'name': 'criticalExtensionsFuture',
                                                                                              'type': 'SEQUENCE'}],
                                                                                 'name': 'criticalExtensions',
                                                                                 'type': 'CHOICE'}],
                                                                    'type': 'SEQUENCE'},
                                     'UECapabilityInformation-NB-r13-IEs': {'members': [{'name': 'ue-Capability-Container-r13',
                                                                                         'type': 'UE-Capability-NB-r13'},
                                                                                        {'name': 'ue-RadioPagingInfo-r13',
                                                                                         'type': 'UE-RadioPagingInfo-NB-r13'},
                                                                                        {'name': 'lateNonCriticalExtension',
                                                                                         'optional': True,
                                                                                         'type': 'OCTET '
                                                                                                 'STRING'},
                                                                                        {'members': [],
                                                                                         'name': 'nonCriticalExtension',
                                                                                         'optional': True,
                                                                                         'type': 'SEQUENCE'}],
                                                                            'type': 'SEQUENCE'},
                                     'UL-AM-RLC-NB-r13': {'members': [{'name': 't-PollRetransmit-r13',
                                                                       'type': 'T-PollRetransmit-NB-r13'},
                                                                      {'name': 'maxRetxThreshold-r13',
                                                                       'type': 'ENUMERATED',
                                                                       'values': [('t1',
                                                                                   0),
                                                                                  ('t2',
                                                                                   1),
                                                                                  ('t3',
                                                                                   2),
                                                                                  ('t4',
                                                                                   3),
                                                                                  ('t6',
                                                                                   4),
                                                                                  ('t8',
                                                                                   5),
                                                                                  ('t16',
                                                                                   6),
                                                                                  ('t32',
                                                                                   7)]}],
                                                          'type': 'SEQUENCE'},
                                     'UL-CCCH-Message-NB': {'members': [{'name': 'message',
                                                                         'type': 'UL-CCCH-MessageType-NB'}],
                                                            'type': 'SEQUENCE'},
                                     'UL-CCCH-MessageType-NB': {'members': [{'members': [{'name': 'rrcConnectionReestablishmentRequest-r13',
                                                                                          'type': 'RRCConnectionReestablishmentRequest-NB'},
                                                                                         {'name': 'rrcConnectionRequest-r13',
                                                                                          'type': 'RRCConnectionRequest-NB'},
                                                                                         {'name': 'rrcConnectionResumeRequest-r13',
                                                                                          'type': 'RRCConnectionResumeRequest-NB'},
                                                                                         {'name': 'spare1',
                                                                                          'type': 'NULL'}],
                                                                             'name': 'c1',
                                                                             'type': 'CHOICE'},
                                                                            {'members': [],
                                                                             'name': 'messageClassExtension',
                                                                             'type': 'SEQUENCE'}],
                                                                'type': 'CHOICE'},
                                     'UL-CarrierConfigDedicated-NB-r13': {'members': [{'name': 'ul-CarrierFreq-r13',
                                                                                       'optional': True,
                                                                                       'type': 'CarrierFreq-NB-r13'},
                                                                                      None],
                                                                          'type': 'SEQUENCE'},
                                     'UL-ConfigCommon-NB-r14': {'members': [{'name': 'ul-CarrierFreq-r14',
                                                                             'type': 'CarrierFreq-NB-r13'},
                                                                            {'name': 'nprach-ParametersList-r14',
                                                                             'optional': True,
                                                                             'type': 'NPRACH-ParametersList-NB-r14'},
                                                                            None],
                                                                'type': 'SEQUENCE'},
                                     'UL-ConfigCommonList-NB-r14': {'element': {'type': 'UL-ConfigCommon-NB-r14'},
                                                                    'size': [(1,
                                                                              'maxNonAnchorCarriers-NB-r14')],
                                                                    'type': 'SEQUENCE '
                                                                            'OF'},
                                     'UL-DCCH-Message-NB': {'members': [{'name': 'message',
                                                                         'type': 'UL-DCCH-MessageType-NB'}],
                                                            'type': 'SEQUENCE'},
                                     'UL-DCCH-MessageType-NB': {'members': [{'members': [{'name': 'rrcConnectionReconfigurationComplete-r13',
                                                                                          'type': 'RRCConnectionReconfigurationComplete-NB'},
                                                                                         {'name': 'rrcConnectionReestablishmentComplete-r13',
                                                                                          'type': 'RRCConnectionReestablishmentComplete-NB'},
                                                                                         {'name': 'rrcConnectionSetupComplete-r13',
                                                                                          'type': 'RRCConnectionSetupComplete-NB'},
                                                                                         {'name': 'securityModeComplete-r13',
                                                                                          'type': 'SecurityModeComplete'},
                                                                                         {'name': 'securityModeFailure-r13',
                                                                                          'type': 'SecurityModeFailure'},
                                                                                         {'name': 'ueCapabilityInformation-r13',
                                                                                          'type': 'UECapabilityInformation-NB'},
                                                                                         {'name': 'ulInformationTransfer-r13',
                                                                                          'type': 'ULInformationTransfer-NB'},
                                                                                         {'name': 'rrcConnectionResumeComplete-r13',
                                                                                          'type': 'RRCConnectionResumeComplete-NB'},
                                                                                         {'name': 'spare8',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare7',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare6',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare5',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare4',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare3',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare2',
                                                                                          'type': 'NULL'},
                                                                                         {'name': 'spare1',
                                                                                          'type': 'NULL'}],
                                                                             'name': 'c1',
                                                                             'type': 'CHOICE'},
                                                                            {'members': [],
                                                                             'name': 'messageClassExtension',
                                                                             'type': 'SEQUENCE'}],
                                                                'type': 'CHOICE'},
                                     'UL-ReferenceSignalsNPUSCH-NB-r13': {'members': [{'name': 'groupHoppingEnabled-r13',
                                                                                       'type': 'BOOLEAN'},
                                                                                      {'name': 'groupAssignmentNPUSCH-r13',
                                                                                       'restricted-to': [(0,
                                                                                                          29)],
                                                                                       'type': 'INTEGER'}],
                                                                          'type': 'SEQUENCE'},
                                     'ULInformationTransfer-NB': {'members': [{'members': [{'name': 'ulInformationTransfer-r13',
                                                                                            'type': 'ULInformationTransfer-NB-r13-IEs'},
                                                                                           {'members': [],
                                                                                            'name': 'criticalExtensionsFuture',
                                                                                            'type': 'SEQUENCE'}],
                                                                               'name': 'criticalExtensions',
                                                                               'type': 'CHOICE'}],
                                                                  'type': 'SEQUENCE'},
                                     'ULInformationTransfer-NB-r13-IEs': {'members': [{'name': 'dedicatedInfoNAS-r13',
                                                                                       'type': 'DedicatedInfoNAS'},
                                                                                      {'name': 'lateNonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'OCTET '
                                                                                               'STRING'},
                                                                                      {'members': [],
                                                                                       'name': 'nonCriticalExtension',
                                                                                       'optional': True,
                                                                                       'type': 'SEQUENCE'}],
                                                                          'type': 'SEQUENCE'},
                                     'UplinkPowerControlCommon-NB-r13': {'members': [{'name': 'p0-NominalNPUSCH-r13',
                                                                                      'restricted-to': [(-126,
                                                                                                         24)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'alpha-r13',
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('al0',
                                                                                                  0),
                                                                                                 ('al04',
                                                                                                  1),
                                                                                                 ('al05',
                                                                                                  2),
                                                                                                 ('al06',
                                                                                                  3),
                                                                                                 ('al07',
                                                                                                  4),
                                                                                                 ('al08',
                                                                                                  5),
                                                                                                 ('al09',
                                                                                                  6),
                                                                                                 ('al1',
                                                                                                  7)]},
                                                                                     {'name': 'deltaPreambleMsg3-r13',
                                                                                      'restricted-to': [(-1,
                                                                                                         6)],
                                                                                      'type': 'INTEGER'}],
                                                                         'type': 'SEQUENCE'},
                                     'UplinkPowerControlDedicated-NB-r13': {'members': [{'name': 'p0-UE-NPUSCH-r13',
                                                                                         'restricted-to': [(-8,
                                                                                                            7)],
                                                                                         'type': 'INTEGER'}],
                                                                            'type': 'SEQUENCE'}},
                           'values': {'maxDRB-NB-r13': {'type': 'INTEGER',
                                                        'value': 2},
                                      'maxNPRACH-Resources-NB-r13': {'type': 'INTEGER',
                                                                     'value': 3},
                                      'maxNS-Pmax-NB-r13': {'type': 'INTEGER',
                                                            'value': 4},
                                      'maxNeighCell-SCPTM-NB-r14': {'type': 'INTEGER',
                                                                    'value': 8},
                                      'maxNonAnchorCarriers-NB-r14': {'type': 'INTEGER',
                                                                      'value': 15},
                                      'maxSC-MTCH-NB-r14': {'type': 'INTEGER',
                                                            'value': 64},
                                      'maxSI-Message-NB-r13': {'type': 'INTEGER',
                                                               'value': 8}}},
 'NBIOT-UE-Variables': {'extensibility-implied': False,
                        'imports': {'EUTRA-UE-Variables': ['VarShortMAC-Input',
                                                           'VarShortResumeMAC-Input-r13']},
                        'object-classes': {},
                        'object-sets': {},
                        'tags': 'AUTOMATIC',
                        'types': {'VarShortMAC-Input-NB-r13': {'type': 'VarShortMAC-Input'},
                                  'VarShortResumeMAC-Input-NB-r13': {'type': 'VarShortResumeMAC-Input-r13'}},
                        'values': {}},
 'PC5-RRC-Definitions': {'extensibility-implied': False,
                         'imports': {'EUTRA-RRC-Definitions': ['TDD-ConfigSL-r12']},
                         'object-classes': {},
                         'object-sets': {},
                         'tags': 'AUTOMATIC',
                         'types': {'MasterInformationBlock-SL': {'members': [{'name': 'sl-Bandwidth-r12',
                                                                              'type': 'ENUMERATED',
                                                                              'values': [('n6',
                                                                                          0),
                                                                                         ('n15',
                                                                                          1),
                                                                                         ('n25',
                                                                                          2),
                                                                                         ('n50',
                                                                                          3),
                                                                                         ('n75',
                                                                                          4),
                                                                                         ('n100',
                                                                                          5)]},
                                                                             {'name': 'tdd-ConfigSL-r12',
                                                                              'type': 'TDD-ConfigSL-r12'},
                                                                             {'name': 'directFrameNumber-r12',
                                                                              'size': [10],
                                                                              'type': 'BIT '
                                                                                      'STRING'},
                                                                             {'name': 'directSubframeNumber-r12',
                                                                              'restricted-to': [(0,
                                                                                                 9)],
                                                                              'type': 'INTEGER'},
                                                                             {'name': 'inCoverage-r12',
                                                                              'type': 'BOOLEAN'},
                                                                             {'name': 'reserved-r12',
                                                                              'size': [19],
                                                                              'type': 'BIT '
                                                                                      'STRING'}],
                                                                 'type': 'SEQUENCE'},
                                   'MasterInformationBlock-SL-V2X-r14': {'members': [{'name': 'sl-Bandwidth-r14',
                                                                                      'type': 'ENUMERATED',
                                                                                      'values': [('n6',
                                                                                                  0),
                                                                                                 ('n15',
                                                                                                  1),
                                                                                                 ('n25',
                                                                                                  2),
                                                                                                 ('n50',
                                                                                                  3),
                                                                                                 ('n75',
                                                                                                  4),
                                                                                                 ('n100',
                                                                                                  5)]},
                                                                                     {'name': 'tdd-ConfigSL-r14',
                                                                                      'type': 'TDD-ConfigSL-r12'},
                                                                                     {'name': 'directFrameNumber-r14',
                                                                                      'size': [10],
                                                                                      'type': 'BIT '
                                                                                              'STRING'},
                                                                                     {'name': 'directSubframeNumber-r14',
                                                                                      'restricted-to': [(0,
                                                                                                         9)],
                                                                                      'type': 'INTEGER'},
                                                                                     {'name': 'inCoverage-r14',
                                                                                      'type': 'BOOLEAN'},
                                                                                     {'name': 'reserved-r14',
                                                                                      'size': [27],
                                                                                      'type': 'BIT '
                                                                                              'STRING'}],
                                                                         'type': 'SEQUENCE'},
                                   'SBCCH-SL-BCH-Message': {'members': [{'name': 'message',
                                                                         'type': 'SBCCH-SL-BCH-MessageType'}],
                                                            'type': 'SEQUENCE'},
                                   'SBCCH-SL-BCH-Message-V2X-r14': {'members': [{'name': 'message',
                                                                                 'type': 'SBCCH-SL-BCH-MessageType-V2X-r14'}],
                                                                    'type': 'SEQUENCE'},
                                   'SBCCH-SL-BCH-MessageType': {'type': 'MasterInformationBlock-SL'},
                                   'SBCCH-SL-BCH-MessageType-V2X-r14': {'type': 'MasterInformationBlock-SL-V2X-r14'}},
                         'values': {}}}